@@ -5,7 +5,7 @@
 #   OS:         Linux
 #   Processor:  Intel X86(-64)
 #
-# $Revision: 655607 $  // Dmitriy Beloslyudtsev, NCBI (beloslyu@ncbi.nlm.nih.gov)
+# $Revision: 687688 $  // Dmitriy Beloslyudtsev, NCBI (beloslyu@ncbi.nlm.nih.gov)
 #############################################################################
 
 
@@ -48,6 +48,11 @@ case "$1" in
       CC=icx
       CXX=icpx
       ;;
+  24* | 2024  )
+      search=$intel_root/Compiler/24.0/compiler/latest/bin
+      CC=icx
+      CXX=icpx
+      ;;
   *           ) search=                                          ;;
 esac
 
