@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: ptb.sh 604636 2020-03-31 14:21:31Z ucko $
+# $Id: ptb.sh 663672 2023-02-27 14:12:38Z ucko $
 # ===========================================================================
 # 
 #                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,9 @@
 # ===========================================================================
 
 DEFPTB_LOCATION="/net/snowman/vol/export2/win-coremake/App/Ncbi/cppcore/ptb"
+if [ ! -d $DEFPTB_LOCATION -a -d /Volumes/win-coremake ]; then
+    DEFPTB_LOCATION="/Volumes/win-coremake/App/Ncbi/cppcore/ptb"
+fi
 IDE="30"
 PTB_EXTRA=""
 ptbname="project_tree_builder"
