@@ -1,4 +1,4 @@
-/*  $Id: blast_args.hpp 683928 2024-06-10 12:07:11Z ivanov $
+/*  $Id: blast_args.hpp 695016 2025-03-13 18:43:28Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -383,13 +383,6 @@ public:
     /** Interface method, \sa IBlastCmdLineArgs::SetArgumentDescriptions */
     virtual void ExtractAlgorithmOptions(const CArgs& cmd_line_args,
                                          CBlastOptions& options);
-
-    /// Value to specify coding template type
-    static const string kTemplType_Coding;
-    /// Value to specify optimal template type
-    static const string kTemplType_Optimal;
-    /// Value to specify coding+optimal template type
-    static const string kTemplType_CodingAndOptimal;
 };
 
 /** Argument class for collecting composition based statistics options */
@@ -1070,6 +1063,8 @@ public:
         
         ///igblast AIRR rearrangement, 19
         eAirrRearrangement,
+        /// Comma-separated values with a header
+        eCommaSeparatedValuesWithHeader,
 
         /// unaligned reads in magicblast
         eFasta,
