@@ -1,4 +1,4 @@
-/*  $Id: igblast.hpp 625641 2021-02-17 18:15:12Z ivanov $
+/*  $Id: igblast.hpp 685571 2024-07-25 12:34:40Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -54,7 +54,7 @@ BEGIN_SCOPE(blast)
 class CIgBlastVersion : public CVersionInfo {
 public:
     CIgBlastVersion()
-        : CVersionInfo(1, 17, 1) {}
+        : CVersionInfo(1, 22, 0) {}
 };
 
 class IQueryFactory;
@@ -72,11 +72,14 @@ public:
     int m_D_penalty;                 //the mismatch penalty for D gene search
     int m_J_penalty;                 //the mismatch penalty for J gene search
     string m_AuxFilename;            // auxulary file name
+    string m_DFrameFileName;         // D gene frame definition file  
+    string m_CustomInternalData;     // custom internal data
     string m_IgDataPath;             // internal data path
-    CRef<CLocalDbAdapter> m_Db[4];   // user specified germline database
+    CRef<CLocalDbAdapter> m_Db[5];   // user specified germline database
                                      // 0-2: - user specified V, D, J
                                      // 3:   - the default V gl db
-    int  m_NumAlign[3];              // number of VDJ alignments to show
+                                     // 4:C gene
+    int  m_NumAlign[4];              // number of VDJC alignments to show
     bool m_FocusV;                   // should alignment restrict to V
     bool m_Translate;                // should translation be displayed
     bool m_ExtendAlign5end;
@@ -96,9 +99,9 @@ public:
     string m_ChainTypeToShow;        // chain type to show to user.  Normally this is 
                                      //the same as m_ChainType[0] but could be different
                                      // in case o TCRA/D chains which can use both JA and JD
-    int m_GeneInfo[6];               // The (start) and (end offset + 1) for VDJ
+    int m_GeneInfo[8];               // The (start) and (end offset + 1) for VDJC
     int m_FrameInfo[3];              // Coding frame start offset for V start, V end,
-                                     // and V start.
+                                     // J start
     int m_DomainInfo[12];            // The (start) and (end offset) for FWR1, 
                                      // CDR1, FWR2, CDR2, FWR3, CDR3 domains
                                      // note: the first and last domains are be extended
@@ -106,17 +109,22 @@ public:
                                      // CDR1, FWR2, CDR2, FWR3, CDR3 domains on topV sequence
 
     int m_JDomain[5];                // CDr3 start, stop, FWR4 start, stop, extra number of bases past last J codon (i.e., m_Fwr4EndOffset in CIgAnnotationInfo
+    
+    int m_CDomain[2];                //start and end
+    int m_DframeStart;
 
     /// Constructor
     CIgAnnotation() 
         : m_MinusStrand (false) 
     {
-        for (int i=0; i<3; i++) m_TopGeneIds.push_back("N/A");
-        for (int i=0; i<6; i++) m_GeneInfo[i] = -1;
+        for (int i=0; i<4; i++) m_TopGeneIds.push_back("N/A");
+        for (int i=0; i<8; i++) m_GeneInfo[i] = -1;
         for (int i=0; i<3; i++) m_FrameInfo[i] = -1;
         for (int i=0; i<12; i++) m_DomainInfo[i] = -1;
         for (int i=0; i<10; i++) m_DomainInfo_S[i] = -1;
         for (int i=0; i<5; i++) m_JDomain[i] = -1;
+        for (int i=0; i<2; i++) m_CDomain[i] = -1;
+        m_DframeStart = -1;
     }
 
 };
@@ -150,6 +158,8 @@ public:
         }
         return -1;
     }
+
+
     int GetJDomain(const string& sid) {
         if (m_JDomainInfo.find(sid) != m_JDomainInfo.end()) {
             return m_JDomainInfo[sid];
@@ -181,6 +191,7 @@ private:
     map<string, string> m_DJChainType;    
     map<string, int>  m_JDomainInfo;   
     map<string, int>  m_Fwr4EndOffset;  //extra number of bases past J end
+ 
 };
 
 class CIgBlastResults : public CSearchResults 
@@ -190,6 +201,7 @@ public:
     int m_NumActualV;
     int m_NumActualD;
     int m_NumActualJ;
+    int m_NumActualC;
 
     const CRef<CIgAnnotation> & GetIgAnnotation() const {
         return m_Annotation;
@@ -215,7 +227,7 @@ public:
                     const TQueryMessages         &errs,
                     CRef<CBlastAncillaryData>     ancillary_data)
            : CSearchResults(query, align, errs, ancillary_data),
-             m_NumActualV(0), m_NumActualD(0), m_NumActualJ(0) { }
+             m_NumActualV(0), m_NumActualD(0), m_NumActualJ(0),  m_NumActualC(0) { }
 
 private:
     CRef<CIgAnnotation> m_Annotation;
@@ -311,6 +323,11 @@ private:
                          CRef<IQueryFactory>           &qf,
                          CRef<CBlastOptionsHandle>     &opts_hndl,
                          int db_type);
+
+    void x_SetupCRegionSearch(const vector<CRef <CIgAnnotation> > &annots,
+                         CRef<IQueryFactory>           &qf,
+                         CRef<CBlastOptionsHandle>     &opts_hndl);
+
     void x_SetupNoOverlapDSearch(const vector<CRef <CIgAnnotation> > &annots,
                                  CRef<CSearchResultSet>        &results,
                                  CRef<IQueryFactory>           &qf,
@@ -335,6 +352,10 @@ private:
     void x_AnnotateJ(CRef<CSearchResultSet>        &results_J,
                      vector<CRef <CIgAnnotation> > &annot);
 
+    void x_AnnotateC(CRef<CSearchResultSet>        &results_c,
+                     CRef<CSearchResultSet>        &results_j,
+                     vector<CRef <CIgAnnotation> > &annot);
+
     /// Annotate the query chaintype and domains based on blast results
     void x_AnnotateDomain(CRef<CSearchResultSet>        &gl_results, 
                           CRef<CSearchResultSet>        &dm_results, 
@@ -393,9 +414,61 @@ private:
                               CRef<CSearchResultSet>& results_D,
                               CRef<CSearchResultSet>& results_J,
                               vector<CRef <CIgAnnotation> > &annots); 
+    
+    void x_ProcessCResult(CRef<CSearchResultSet>& results_C,
+                          vector<CRef <CIgAnnotation> > &annots); 
         
 };
 
+class ScorePositionSort{
+    CRef<CScope> m_scope;
+    int min_J_position_diff = 100;
+    double reliable_J_match_factor = 0.5;
+public:
+    ScorePositionSort(CRef<CScope> scope){ m_scope = scope; }
+    bool operator()(CRef<CSeq_align> &x, CRef<CSeq_align> &y) {
+        return s_CompareSeqAlignByScoreAndPosition(x, y, m_scope);
+    }
+    bool s_CompareSeqAlignByScoreAndPosition(const CRef<CSeq_align> &x, const CRef<CSeq_align> &y, 
+                                             CRef<CScope> scope){
+        int lx = x->GetAlignLength(), ly = y->GetAlignLength();
+        int jx_start = x->GetSeqStart(0);
+        int jy_start = y->GetSeqStart(0);
+        int sx = 0, sy = 0;
+        x->GetNamedScore(CSeq_align::eScore_Score, sx);
+        y->GetNamedScore(CSeq_align::eScore_Score, sy);
+        
+        int reliable_J_match = static_cast<int> (
+            reliable_J_match_factor *
+            (scope->GetBioseqHandle(x->GetSeq_id(1)).GetBioseqLength() +
+             scope->GetBioseqHandle(y->GetSeq_id(1)).GetBioseqLength())/2
+        );
+        
+        //if there are additional J genes, consider using the 5' one if conditions met.
+        //this reflect the case where rearrangement is followed by downstream J's
+        if (lx > reliable_J_match && ly > reliable_J_match && abs(jx_start - jy_start) > min_J_position_diff) {
+            if (x->GetSeqStrand(0) == eNa_strand_minus){
+                return (jx_start > jy_start + min_J_position_diff && sx>sy*reliable_J_match_factor);
+            } else {
+                return (jx_start < jy_start - min_J_position_diff && sx>sy*reliable_J_match_factor);
+                
+            }
+        } else if (sx != sy) {
+            return (sx > sy);
+        } else if (lx != ly) {
+            return (lx >= ly);
+        }
+        
+        string x_id = NcbiEmptyString;
+        string y_id = NcbiEmptyString;
+        x->GetSeq_id(1).GetLabel(&x_id, CSeq_id::eContent);
+        y->GetSeq_id(1).GetLabel(&y_id, CSeq_id::eContent);
+        
+        return (x_id < y_id);
+        
+    }
+};
+
 END_SCOPE(blast)
 END_NCBI_SCOPE
 
