@@ -1,7 +1,7 @@
 #ifndef CGI___CGIAPP__HPP
 #define CGI___CGIAPP__HPP
 
-/*  $Id: cgiapp.hpp 658185 2022-10-26 16:12:23Z grichenk $
+/*  $Id: cgiapp.hpp 696030 2025-04-07 12:59:48Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -63,6 +63,7 @@ class NCBI_XCGI_EXPORT CCgiApplication : public CNcbiApplication
 {
     friend class CCgiStatistics;
     friend class CCgiRequestProcessor;
+    friend class CFastCgiApplication;
     friend void s_ScheduleFastCGIExit(void);
 
 public:
@@ -327,6 +328,9 @@ protected:
     enum EAdminCommand {
         eAdmin_Health,       ///< Report health for this CGI only
         eAdmin_HealthDeep,   ///< Report health for this CGI and any services used by it.
+        eAdmin_HealthZ,      ///< Report health for this CGI as kubernetes expects
+        eAdmin_ReadyZ,       ///< Report readyness for this CGI as kubernetes expects
+        eAdmin_LiveZ,        ///< Report if this CGI is alive as kubernetes expects
         eAdmin_Unknown       ///< Unrecognized command. Overriden ProcessAdminRequest()
                              ///< can use GetEntry() to fetch command name if necessary.
     };
@@ -476,6 +480,7 @@ private:
 class NCBI_XCGI_EXPORT CCgiStatistics
 {
     friend class CCgiApplication;
+    friend class CFastCgiApplication;
     friend class CFastCgiApplicationMT;
 public:
     virtual ~CCgiStatistics();
