@@ -1,7 +1,7 @@
 #ifndef CGI___NCBICGI__HPP
 #define CGI___NCBICGI__HPP
 
-/*  $Id: ncbicgi.hpp 595175 2019-10-16 17:41:10Z grichenk $
+/*  $Id: ncbicgi.hpp 651832 2022-06-26 03:46:48Z lavr $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -39,7 +39,6 @@
 #include <corelib/stream_utils.hpp>
 #include <cgi/cgi_util.hpp>
 #include <cgi/user_agent.hpp>
-#include <set>
 
 
 /** @addtogroup CGIReqRes
@@ -48,15 +47,14 @@
  */
 
 
-#define HTTP_EOL "\r\n"
-
-
 BEGIN_NCBI_SCOPE
 
+
 class CTime;
 class CCgiSession;
 class CCgiEntryReaderContext;
 
+
 ///////////////////////////////////////////////////////
 ///
 ///  CCgiCookie::
@@ -321,6 +319,14 @@ public:
     /// secure cookies can be sent only trough secure connections.
     void SetSecure(bool secure) { m_Secure = secure; }
 
+    /// Mark all cookies as secure.
+    void SetAllCookiesSecure(bool value);
+    bool GetAllCookiesSecure(void) const { return m_AllSecure; }
+
+    /// Mark all cookies as HTTP_ONLY.
+    void SetAllCookiesHttpOnly(bool value);
+    bool GetAllCookiesHttpOnly(void) const { return m_AllHttpOnly; }
+
 private:
     enum ECheckResult {
         eCheck_Valid,         // Cookie is valid
@@ -336,6 +342,8 @@ private:
     NStr::EUrlEncode m_EncodeFlag;
     TSet             m_Cookies;
     bool             m_Secure;
+    bool             m_AllSecure;
+    bool             m_AllHttpOnly;
 
     /// prohibit default initialization and assignment
     CCgiCookies(const CCgiCookies&);
@@ -727,7 +735,9 @@ public:
         /// When parsing input on demand iterate all existing entries (e.g. those
         /// read from QUERY_STRING) before parsing POST data.
         /// @sa fParseInputOnDemand
-        fIncludePreparsedEntries = (1 << 13)
+        fIncludePreparsedEntries    = (1 << 13),
+        /// Disable parsing input as 'indexed' query (RFC3875) even if no '=' is present
+        fDisableParsingAsIndex      = (1 << 14),
     };
     CCgiRequest(const         CNcbiArguments*   args = 0,
                 const         CNcbiEnvironment* env  = 0,
@@ -1058,7 +1068,9 @@ inline void CCgiCookie::ResetInvalid(TInvalidFlag flag)
 inline CCgiCookies::CCgiCookies(void)
     : m_EncodeFlag(NStr::eUrlEnc_SkipMarkChars),
       m_Cookies(),
-      m_Secure(false)
+      m_Secure(false),
+      m_AllSecure(false),
+      m_AllHttpOnly(false)
 {
     return;
 }
@@ -1066,7 +1078,9 @@ inline CCgiCookies::CCgiCookies(void)
 inline CCgiCookies::CCgiCookies(EUrlEncode encode_flag)
     : m_EncodeFlag(NStr::EUrlEncode(encode_flag)),
       m_Cookies(),
-      m_Secure(false)
+      m_Secure(false),
+      m_AllSecure(false),
+      m_AllHttpOnly(false)
 {
     return;
 }
@@ -1076,7 +1090,9 @@ inline CCgiCookies::CCgiCookies(const string& str,
                                 EUrlEncode   encode_flag)
     : m_EncodeFlag(NStr::EUrlEncode(encode_flag)),
       m_Cookies(),
-      m_Secure(false)
+      m_Secure(false),
+      m_AllSecure(false),
+      m_AllHttpOnly(false)
 {
     Add(str, on_bad_cookie);
 }
@@ -1134,7 +1150,7 @@ CNcbiIstream* CCgiEntry::GetValueStream()
         return new CRStream(m_Data->m_Reader.release(), 0, 0,
                             CRWStreambuf::fOwnReader);
     } else {
-        return new CNcbiIstrstream(GetValue().data(), GetValue().size());
+        return new CNcbiIstrstream(GetValue());
     }
 }
 
