@@ -1,4 +1,4 @@
-/* $Id: ncbiconf_xcode.h 667394 2023-05-15 17:19:52Z ucko $
+/* $Id: ncbiconf_xcode.h 687051 2024-09-03 18:53:23Z ucko $
  * By Vlad Lebedev, NCBI (lebedev@ncbi.nlm.nih.gov)
  *
  * Mac OS X - xCode Build
@@ -147,6 +147,9 @@
 /* Define to 1 if you have the `basename' function. */
 #define HAVE_BASENAME 1
 
+/* Define to 1 if you have the `bcopy' function. */
+#define HAVE_BCOPY 1
+
 /* Define to 1 if the Berkeley `db_cxx' library is available. */
 /* #undef HAVE_BERKELEY_DB_CXX */
 
@@ -352,6 +355,9 @@
 /* Define to 1 if libbz2 is available. */
 #define HAVE_LIBBZ2 1
 
+/* Define to 1 if Cloudflare zlib is available (embedded). */
+#define HAVE_LIBZCF 1
+
 /* Define to 1 if CRYPT is available, either in its own library or as part of
    the standard libraries. */
 #define HAVE_LIBCRYPT 1
@@ -461,9 +467,18 @@
 /* Define to 1 if you have the `malloc_options' function. */
 /* #undef HAVE_MALLOC_OPTIONS */
 
+/* Define to 1 if you have the `memfd_create' function. */
+/* #undef HAVE_MEMFD_CREATE */
+
+/* Define to 1 if you have the `memmove' function. */
+#define HAVE_MEMMOVE 1
+
 /* Define to 1 if you have the <memory.h> header file. */
 #define HAVE_MEMORY_H 1
 
+/* Define to 1 if you have the `mkostemp' function. */
+#define HAVE_MKOSTEMP 1
+
 /* Define to 1 if you have the `nanosleep' function. */
 #define HAVE_NANOSLEEP 1
 
@@ -534,6 +549,9 @@
 /* Define to 1 if you have the `readv' function. */
 #define HAVE_READV 1
 
+/* Define to 1 if you have the `realpath' function. */
+#define HAVE_REALPATH 1
+
 /* Define to 1 if your C compiler supports some variant of the C99 `restrict'
    keyword. */
 #define HAVE_RESTRICT_C 1
@@ -548,6 +566,9 @@
 /* Define to 1 if you have the `sched_yield' function. */
 #define HAVE_SCHED_YIELD 1
 
+/* Define to 1 if you have the `secure_getenv' function. */
+/* #undef HAVE_SECURE_GETENV */
+
 /* Define to 1 if you have the `select' function. */
 #define HAVE_SELECT 1
 
@@ -611,6 +632,9 @@
 /* Define to 1 if you have the <stdint.h> header file. */
 #define HAVE_STDINT_H 1
 
+/* Define to 1 if you have the <stdio.h> header file. */
+#define HAVE_STDIO_H 1
+
 /* Define to 1 if you have the <stdlib.h> header file. */
 #define HAVE_STDLIB_H 1
 
@@ -623,6 +647,9 @@
 /* Define to 1 if you have the `strdup' function. */
 #define HAVE_STRDUP 1
 
+/* Define to 1 if you have the `strerror' function. */
+#define HAVE_STRERROR 1
+
 /* Define to 1 if you have the <strings.h> header file. */
 #define HAVE_STRINGS_H 1
 
