@@ -1,7 +1,7 @@
 #ifndef CONNECT__IMPL__SOCKET_ADDRESS__HPP
 #define CONNECT__IMPL__SOCKET_ADDRESS__HPP
 
-/*  $Id: connect_misc.hpp 675422 2023-11-21 19:49:15Z ivanov $
+/*  $Id: connect_misc.hpp 676916 2023-12-28 14:55:16Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -134,7 +134,7 @@ struct SThreadSafe
     };
 
     template <class... TArgs>
-    SThreadSafe(TArgs&&... args) : m_Object(forward<TArgs>(args)...) {}
+    SThreadSafe(TArgs&&... args) : m_Object(std::forward<TArgs>(args)...) {}
 
     SLock<      TType> GetLock()       { return { &m_Object, m_Mutex }; }
     SLock<const TType> GetLock() const { return { &m_Object, m_Mutex }; }
@@ -164,9 +164,9 @@ public:
     };
 
     CLogLatencies(SRegex start, SRegex stop, std::optional<SRegex> server_side = nullopt) :
-        m_Start(move(start)),
-        m_Stop(move(stop)),
-        m_ServerSide(move(server_side))
+        m_Start(std::move(start)),
+        m_Stop(std::move(stop)),
+        m_ServerSide(std::move(server_side))
     {}
 
     void SetWhich(EWhich which) { m_Which = which; }
@@ -191,7 +191,7 @@ class NCBI_XCONNECT_EXPORT CLogLatencyReport : public CLogLatencies
 {
 public:
     template <class... TArgs>
-    CLogLatencyReport(string filter, TArgs&&... args) : CLogLatencies(forward<TArgs>(args)...), m_Filter(move(filter)) {}
+    CLogLatencyReport(string filter, TArgs&&... args) : CLogLatencies(std::forward<TArgs>(args)...), m_Filter(std::move(filter)) {}
 
     ~CLogLatencyReport();
 
