@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_TEST__HPP
 #define CONNECT___NCBI_CONN_TEST__HPP
 
-/* $Id: ncbi_conn_test.hpp 654831 2022-08-22 21:20:43Z lavr $
+/* $Id: ncbi_conn_test.hpp 676498 2023-12-15 16:56:06Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -247,7 +247,8 @@ private:
     /// Return verbal timeout suggestion
     string     x_TimeoutMsg(void);
     /// Obtain and populate FWD connection points from NCBI
-    EIO_Status x_GetFirewallConfiguration(const SConnNetInfo* net_info);
+    EIO_Status x_GetFirewallConfiguration(const SConnNetInfo* net_info,
+                                          bool fwbports);
 
     /// Cancellation support
     CConstIRef<ICanceled> m_Canceled;
@@ -257,7 +258,7 @@ public:
     /// @note  Do not use this method anywhere else other than when deciding
     ///        whether to proceed with Execute() in this class with more
     ///        thorough connection checks!
-    static bool IsNcbiInhouseClient(void);
+    static bool IsNcbiInhouseClient(void) THROWS_NONE;
 };
 
 
