@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/* $Id: ncbi_connutil.h 660675 2022-12-21 21:59:38Z lavr $
+/* $Id: ncbi_connutil.h 676498 2023-12-15 16:56:06Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,9 @@
  *       ConnNetInfo_SetupStandardArgs()
  *       ConnNetInfo_SetUserHeader()
  *       ConnNetInfo_AppendUserHeader()
+ *       ConnNetInfo_PrependUserHeader()
  *       ConnNetInfo_OverrideUserHeader()
+ *       ConnNetInfo_PreOverrideUserHeader()
  *       ConnNetInfo_ExtendUserHeader()
  *       ConnNetInfo_DeleteUserHeader()
  *       ConnNetInfo_SetTimeout()
@@ -165,6 +167,15 @@ typedef enum {
 typedef unsigned EBDebugPrintout;
 
 
+typedef enum {
+    fProxy_None = 0,  /**< No $http_proxy / $https_proxy used */
+    fProxy_Http,      /**< $http_proxy  used                  */
+    fProxy_Https      /**< $https_proxy used                  */
+} EProxyType;
+
+typedef unsigned EBProxyType;
+
+
 /* Network connection-related configurable informational structure.
  * ATTENTION:  Do NOT fill out this structure (SConnNetInfo) "from scratch"!
  *             Instead, use ConnNetInfo_Create() described below to create it,
@@ -193,8 +204,8 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
     unsigned        http_push_auth:1; /* push authorize tags even w/o 401/407*/
     unsigned        http_proxy_leak:1;/* TRUE when may fallback to direct    */
     unsigned        http_proxy_skip:1;/* TRUE when *NOT* to read $http_proxy */
-    unsigned        http_proxy_only:1;/* TRUE when proxy is from $http_proxy */
-    unsigned        reserved:12;      /* MBZ                                 */
+    EBProxyType     http_proxy_mask:2;/* $http_proxy / $https_proxy          */
+    unsigned        reserved:11;      /* MBZ                                 */
     char            user[CONN_USER_LEN+1];  /* username (if spec'd or req'd) */
     char            pass[CONN_PASS_LEN+1];  /* password (for non-empty user) */
     char            host[CONN_HOST_LEN+1];  /* host name to connect to       */
@@ -598,13 +609,25 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_AppendUserHeader
  );
 
 
+/* Prepend to user header (same as ConnNetInfo_SetUserHeader() if no previous
+ * header was set);  do nothing if the provided "header" is NULL or empty.
+ * @warning
+ *   New "header" may not be any part of the header kept in the structure.
+ * Return non-zero if successful;  otherwise, return 0 to indicate an error.
+ */
+extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PrependUserHeader
+(SConnNetInfo*       net_info,
+ const char*         header
+ );
+
+
 /* Override user header.
  * Tags replaced (case-insensitively), and tags with empty values effectively
  * delete all existing occurrences of the matching tags from the old user
  * header, e.g. "My-Tag:\r\n" deletes all appearences of "My-Tag: [<value>]"
  * from the user header, regardless of the "<value>" (if any).
  * Unmatched tags with non-empty values are simply added to the existing user
- * header (as with "Append" above).  No-op if "header" is an empty string ("").
+ * header (as with "Append" above).  No-op if "header" is NULL or empty.
  * @note
  *   New "header" may be part of the current header from the structure.
  * Return non-zero if successful, otherwise return 0 to indicate an error.
@@ -614,6 +637,25 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_OverrideUserHeader
  const char*         header
  );
 
+#define ConnNetInfo_PostOverrideUserHeader  ConnNetInfo_OverrideUserHeader
+
+
+/* Pre-override user header.
+ * Tags replaced (case-insensitively), and tags with empty values effectively
+ * delete all existing occurrences of the matching tags from the old user
+ * header, e.g. "My-Tag:\r\n" deletes all appearences of "My-Tag: [<value>]"
+ * from the user header, regardless of the "<value>" (if any).
+ * Unmatched tags with non-empty values get prepended to the existing user
+ * header (as with "Prepend" above).  No-op if "header" is NULL or empty.
+ * @note
+ *   New "header" may be part of the current header from the structure.
+ * Return non-zero if successful, otherwise return 0 to indicate an error.
+ */
+extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PreOverrideUserHeader
+(SConnNetInfo*       net_info,
+ const char*         header
+ );
+
 
 /* Extend user header.
  * Existing tags matching (case-insensitively) first appearances of those
@@ -691,6 +733,11 @@ extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_Destroy
  );
 
 
+/** Extra URL_ConnectEx() parameters. */
+typedef struct {
+    NCBI_CRED   cred;  /**< SSL credentials (if any)                       */
+    const char* host;  /**< SSL host id (aka SNI) (if differs from "host") */
+} SURLExtra;
 
 /* Very low-level HTTP initiation routine.  Regular use is highly discouraged.
  * Instead, please consider using higher level APIs such as HTTP connections
@@ -739,8 +786,8 @@ extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_Destroy
  * header, followed by the header termination sequence of "\r\n\r\n".
  * @note that any interior whitespace in "user_header" is not analyzed/guarded.
  *
- * The "cred" parameter is only used to pass additional connection credentials
- * for secure connections, and is ignored otherwise.
+ * The "extra" parameter is only used to pass additional connection credentials
+ * and server name info for secure connections, and is ignored otherwise.
  *
  * If the request method contains "eReqMethod_Connect", then the connection is
  * assumed to be tunneled via a proxy, so "path" must specify a "host:port"
@@ -780,7 +827,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status URL_ConnectEx
  const STimeout* o_timeout,       /* timeout for an OPEN stage               */
  const STimeout* rw_timeout,      /* timeout for READ and WRITE              */
  const char*     user_header,     /* should include "Host:" in most cases    */
- NCBI_CRED       cred,            /* connection credentials, if any          */
+ SURLExtra*      extra,           /* additional connection params, if any    */
  TSOCK_Flags     flags,           /* additional socket requirements          */
  SOCK*           sock             /* returned socket (on eIO_Success only)   */
  );
