@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CORE__H
 #define CONNECT___NCBI_CORE__H
 
-/* $Id: ncbi_core.h 604951 2020-04-05 03:12:01Z lavr $
+/* $Id: ncbi_core.h 677341 2024-01-11 16:56:03Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,8 +35,8 @@
  * I/O status and direction:
  *    enum:       EIO_ReadMethod
  *    enum:       EIO_WriteMethod
- *    enum:       EIO_Status,  verbal: IO_StatusStr()
  *    enum:       EIO_Event
+ *    enum:       EIO_Status,  verbal: IO_StatusStr()
  *
  * Critical section (basic multi-thread synchronization):
  *    handle:     MT_LOCK
@@ -110,8 +110,8 @@ typedef enum {
  *  be changed in this header.  On the other hand, user code should not rely
  *  on the values of these constants, either.
  * @warning
- *  Careful with an unfortunate naming similarity of eIO_Close from this set
- *  and eIO_Closed from EIO_Status -- do not mix the two!
+ *  Careful with an unfortunate naming similarity of eIO_Close from this enum
+ *  and EIO_Status::eIO_Closed -- do not mix the two!
  * @sa
  *  SOCK_Wait, SOCK_Poll, CONN_Wait, SOCK_SetTimeout, CONN_SetTimeout
  */
@@ -126,13 +126,13 @@ typedef enum {
 
 /** I/O status.
  * @warning
- *  Careful with an unfortunate naming similarity of eIO_Closed from this set
- *  and eIO_Close from EIO_Event -- do not mix the two!
+ *  Careful with an unfortunate naming similarity of eIO_Closed from this enum
+ *  and EIO_Event::eIO_Close -- do not mix the two!
  */
 typedef enum {
     eIO_Success = 0,    /**< everything is fine, no error occurred           */
     eIO_Timeout,        /**< timeout expired before any I/O succeeded        */
-    eIO_Reserved,       /**< reserved status code -- do not use!             */
+    eIO_Reserved,       /**< reserved status code -- DO NOT USE!             */
     eIO_Interrupt,      /**< signal arrival prevented any I/O to succeed     */
     eIO_InvalidArg,     /**< bad argument / parameter value(s) supplied      */
     eIO_NotSupported,   /**< operation is not supported or is not available  */
@@ -261,9 +261,9 @@ extern NCBI_XCONNECT_EXPORT MT_LOCK MT_LOCK_Delete(MT_LOCK lk);
  *  Whether to lock (and how: read, write) or to unlock
  * @return
  *  Value returned by the lock handler ("handler" in MT_LOCK_Create()).
- * If lock handler is not specified then always return "-1".
+ *  If the lock handler is not specified then always return "-1" (noop).
  * @note
- *  Use MT_LOCK_Do to avoid overhead!
+ *  Use MT_LOCK_Do() to avoid overhead!
  * @sa
  *  MT_LOCK_Create, FMT_LOCK_Handler, EMT_Lock
  */
@@ -313,7 +313,7 @@ extern NCBI_XCONNECT_EXPORT const char* LOG_LevelStr(ELOG_Level level);
 
 /** Message and miscellaneous data to pass to log post callback FLOG_Handler.
  * @param dynamic
- *  if non-zero then LOG_WriteInternal() will call free(message) before return
+ *  if non-zero then LOG_WriteInternal() calls free(message) before returning
  * @param message
  *  A message to post, can be NULL
  * @param level
@@ -469,7 +469,7 @@ extern NCBI_XCONNECT_EXPORT LOG LOG_Delete(LOG lg);
  * @param raw_size
  *  Size of the raw data (can be zero)
  * @sa
- *  LOG_Create, ELOG_Level, FLOG_Handler, LOG_WriteInternal
+ *  LOG_Create, ELOG_Level, LOG_WriteInternal, LOG_WRITE, LOG_DATA
  */
 extern NCBI_XCONNECT_EXPORT void LOG_Write
 (LOG         lg,
@@ -495,6 +495,10 @@ extern NCBI_XCONNECT_EXPORT void LOG_Write
  *  This call free()s "mess->message" when "mess->dynamic" is set non-zero!
  * @param lg
  *  A log handle previously obtained from LOG_Create
+ * @note
+ *  In case of a fatal error (eLOG_Fatal found in mess->code), it calls abort()
+ *  (in Debug builds) or _exit(-1) (in Release builds).  Note that no exit
+ *  handlers are run or any other normal process run-down procedure performed.
  * @sa
  *  LOG_Create, ELOG_Level, FLOG_Handler, LOG_Write
  */
@@ -528,14 +532,16 @@ typedef enum {
 
 
 /** Registry getter callback.
- * Copy registry value stored in "section" under name "name" to buffer "value".
- * Look for the matching entry first in the transient storage, and then in
- * the persistent storage.  Do not modify the "value" (leave it "as is",
- * i.e. default) if the requested entry is not found in the registry.
+ * Copy registry value stored in "section" under "name" to the "value" buffer.
+ * Look for a matching entry first in the transient storage, and then in the
+ * persistent storage.
  * @note
- *  Always terminate value with '\0'.
+ *  Do not modify "value" (leave it "as is", i.e. empty) if the requested entry
+ *  was not found in the registry;  then, return -1.
  * @note
- *  Do not put more than "value_size" bytes to "value".
+ *  Always terminate the "value" with '\0'.
+ * @note
+ *  Do not put more than "value_size" bytes into "value" (incl. trailing '\0').
  * @param data
  *  Unspecified data as passed to REG_Create or REG_Reset
  * @param section
@@ -543,14 +549,14 @@ typedef enum {
  * @param name
  *  Key name to search within the section
  * @param value
- *  Default value passed in (cut to "value_size") symbols, found value out
+ *  Empty value passed in, found (if any) value out
  * @param value_size
  *  Size of "value" storage, must be greater than 0
  * @return
  *  1 if successfully found and stored;  -1 if not found (the default is to be
  *  used);  0 if an error (including truncation) occurred
  * @sa
- *  REG_Create, REG_Reset
+ *  REG_Create, REG_Reset, REG_Get, FREG_Set
  */
 typedef int (*FREG_Get)
 (void*       data,
@@ -562,8 +568,8 @@ typedef int (*FREG_Get)
 
 
 /** Registry setter callback.
- * Store the "value" to  the registry section "section" under name "name",
- * and according to "storage".
+ * Store the "value" in the registry "section" under the "name" key, and
+ * according to "storage".
  * @param data
  *  Unspecified data as passed to REG_Create or REG_Reset
  * @param section
@@ -577,7 +583,7 @@ typedef int (*FREG_Get)
  * @return
  *  Non-zero if successful (including replacing a value with itself)
  * @sa
- *  REG_Create, REG_Reset, EREG_Storage
+ *  REG_Create, REG_Reset, EREG_Storage, REG_Set, FREG_Get
  */
 typedef int/*bool*/ (*FREG_Set)
 (void*        data,
