@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SOCKET__H
 #define CONNECT___NCBI_SOCKET__H
 
-/* $Id: ncbi_socket.h 608631 2020-05-19 15:57:04Z lavr $
+/* $Id: ncbi_socket.h 676496 2023-12-15 16:43:56Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,8 +40,9 @@
  *
  *  SOCK_InitializeAPI
  *  SOCK_ShutdownAPI
- *  SOCK_AllowSigPipeAPI
  *  SOCK_OSHandleSize
+ *  SOCK_AllowSigPipeAPI
+ *  SOCK_SetApproveHookAPI
  *
  * Event trigger (handle TRIGGER):
  *
@@ -124,6 +125,17 @@
  *  SOCK_SetDataLogging
  *  SOCK_SetErrHookAPI
  *
+ * Generic POLLABLE API:
+ *
+ *  POLLABLE_Poll
+ *  POLLABLE_What
+ *  POLLABLE_FromSOCK
+ *  POLLABLE_FromLSOCK
+ *  POLLABLE_FromTrigger
+ *  POLLABLE_ToSOCK
+ *  POLLABLE_ToLSOCK
+ *  POLLABLE_ToTrigger
+ *
  * Auxiliary:
  *
  *  SOCK_ntoa
@@ -171,6 +183,9 @@ extern "C" {
  *  TYPEDEFS & MACROS
  */
 
+typedef unsigned EBIO_Event;
+typedef unsigned EBIO_Status;
+
 
 /** Network and host byte order enumeration type
  */
@@ -180,6 +195,24 @@ typedef enum {
 } ENH_ByteOrder;
 
 
+/** Socket type (internal)
+ */
+typedef enum {
+    eSOCK_Listening = 0,
+    eSOCK_Trigger   = 1,
+    eSOCK_Socket    = 2,
+    eSOCK_Datagram  = 3/*2|1*/
+} ESOCK_Type;
+
+
+/** Sides of socket
+ */
+typedef enum {
+    eSOCK_Server = 0,
+    eSOCK_Client = 1
+} ESOCK_Side;
+
+
 /** Forward declarations of the hidden socket internal structures, and
  *  their upper-level handles to use by the SOCK API.
  */
@@ -264,6 +297,15 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_InitializeAPI(void);
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_ShutdownAPI(void);
 
 
+/** Get size of OS-dependent native socket handle.
+ * @return
+ *  OS-dependent handle size or 0 in case of an error
+ * @sa
+ *  SOCK_GetOSHandle
+ */
+extern NCBI_XCONNECT_EXPORT size_t SOCK_OSHandleSize(void);
+
+
 /** By default (on UNIX platforms) the SOCK API functions automagically call
  * "signal(SIGPIPE, SIG_IGN)" on initialization.  To prohibit this feature you
  * must call SOCK_AllowSigPipeAPI() before you call any other function from the
@@ -272,13 +314,52 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_ShutdownAPI(void);
 extern NCBI_XCONNECT_EXPORT void SOCK_AllowSigPipeAPI(void);
 
 
-/** Get size of OS-dependent native socket handle.
- * @return
- *  OS-dependent handle size or 0 in case of an error
+/** User-level connection approval hook.
+ * The hook (when installed) gets called for any attempt to establish an
+ * outgoing(eSOCK_Client) or incoming(eSOCK_Server) connection(eSOCK_Socket),
+ * or to send(eSOCK_Client) or receive(eSOCK_Server) a message packet in a
+ * datagram socket(eSOCK_Datagram).  The peer end is always identified by the
+ * "addr:port" pair (which is non-0 in both parts).  Optionally for outgoing
+ * transaction, a textual hostname("host") can be provided (to which "addr"
+ * corresponds).  If the action was requested by a plain IP address, the "host"
+ * field is set to NULL.
+ *
+ * The hook is expected to return eIO_Success to approve the action, or any
+ * other status code to deny.  Note that either eIO_Timeout or eIO_Closed
+ * returned from the hook, will get converted to eIO_Unknown;  and other codes
+ * will get passed through to the caller "as is".
+ * @warning
+ *   Although returning eIO_Interrupt may be helpful to prevent the upper-lever
+ *   connection (CONN) layer from processing further I/O on this communication
+ *   channel, it may also be just an undesireable side-effect -- so it should
+ *   be used with caution.
+ * @note
+ *   When the hook is not installed, all the actions described above are
+ *   treated as allowed (as if eIO_Success was returned).
  * @sa
- *  SOCK_GetOSHandle
+ *   SOCK_Create, SOCK_Reconnect, LSOCK_Accept, DSOCK_SendMsg, DSOCK_RecvMsg
  */
-extern NCBI_XCONNECT_EXPORT size_t SOCK_OSHandleSize(void);
+
+typedef struct {
+    const char*    host;  /**< Textual hostname if provided for outgoing     */
+    unsigned int   addr;  /**< IPv4 (or -1 if unknown / broadcast), never 0  */
+    unsigned short port;  /**< Port number, host byte order, never 0         */
+    ESOCK_Side     side;  /**< eSOCK_Client (out)  / eSOCK_Server (in)       */
+    ESOCK_Type     type;  /**< eSOCK_Socket (conn) / eSOCK_Datagram (packet) */
+} SSOCK_ApproveInfo;
+
+
+/** The approve hook is global per API and gets called with an arbitrary
+ * user "data", if so desired as a second argument, which was specified when
+ * the hook was installed with the SOCK_SetApproveHookAPI() call.
+ */
+typedef EIO_Status (*FSOCK_ApproveHook)(const SSOCK_ApproveInfo* info,
+                                        void*                    data);
+
+/** The hook is installed when non-NULL, and de-installed otherwise. */
+extern NCBI_XCONNECT_EXPORT
+void SOCK_SetApproveHookAPI(FSOCK_ApproveHook hook,  /**< the hook function  */
+                            void*             data); /**< optional user data */
 
 
 /** This is a helper call that can improve I/O behavior.
@@ -306,13 +387,14 @@ typedef enum {
     eSOCK_IOWaitSysAPISelect  /**< always use select()                       */
 } ESOCK_IOWaitSysAPI;
 
-/** This is a helper call that can improve I/O behavior (ignored for Windows).
+/** This is a helper call that can improve I/O performance (ignored for MSVC).
  * @param api
  *  [in]  Default behavior is to wait for I/O such a way that accomodates the
  *  requested sockets accordingly.  There is a known limitation of the select()
- *  API that requires all sockets to have low-level IO descriptors less than
- *  1024, but works faster than the poll() API that does not have limits on the
- *  numeric values of the descriptors.  Either API can be enforced here.
+ *  API that requires all sockets to have numeric values of their low-level I/O
+ *  handles less than (as little as) 1024, but works faster than the poll() API
+ *  that does not have limits on the number or numeric values of the handles.
+ *  Either API can be enforced here.
  * @return
  *  Previous value of the API selector
  * @sa
@@ -429,7 +511,7 @@ typedef unsigned int TSOCK_Flags;  /**< bitwise "OR" of ESOCK_Flags */
 /** [SERVER-side]  Create and initialize the server-side(listening) socket
  * (socket() + bind() + listen())
  * @param port
- *  [in]  the port to listen at (0 to select first available)
+ *  [in]  the port to listen at (0 to choose first available)
  * @param backlog
  *  [in]  maximal # of pending connections
  *  @note  On some systems, "backlog" may be silently limited down to 128
@@ -633,15 +715,18 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Create
 
 /** [SERVER-side]  Create a SOCKet on top of either an OS-dependent "handle"
  * (file descriptor on Unix, SOCKET on MS-Windows) or an existing SOCK object.
- * In the later case, the returned socket is not reopenable to its default peer
- * (SOCK_Reconnect() may not specify zeros for the connection point).
+ * In the former case, the returned socket is not reopenable to its default
+ * peer (SOCK_Reconnect() may not specify zeros for the connection point).
+ * In the latter case, the returned socket is reopenable to its default peer
+ * only if the original existing socket is a client-side one (connected to a
+ * remote server end), not a server-side socket (accepted at the server end).
  * All timeouts are set to default [infinite] values.
  * The call does *not* destroy either the OS handle or the SOCK passed in the
  * arguments, regardless of the return status code.
  * When a socket is being created on top of a SOCK handle, the original SOCK
  * gets emptied (and the underlying OS handle removed from it) upon the call
- * returns (whether successfully or not), yet will still need SOCK_Destroy() in
- * the caller's code to free up the memory it occupies.
+ * return (whether successfully or not), yet it will still need SOCK_Destroy()
+ * in the caller's code to free up the memory it occupies.
  * Any secure session that may have existed in the original SOCK will have
  * been migrated to the new socket iff "flags" indicate fSOCK_Secure (if no
  * session existed, a new session may have been initiated in the new SOCK).
@@ -663,7 +748,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Create
  * @note
  *  (SOCK_IsClientSide() | SOCK_IsServerSide()) can be used to determine if the
  *  original SOCK was stripped off the underlying handle:  the expression would
- *  evaluate to 0(false) iff the SOCK object does not have the OS handle.
+ *  evaluate to 0(false) iff the SOCK object does no longer have the OS handle.
  * @warning
  *  If a new SOCK is being built from an existing SOCK, then it is assumed that
  *  the underlying OS handle's properies have not been modified outside that
@@ -671,7 +756,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Create
  * @param handle
  *  [in]  OS-dependent "handle" or SOCK to be converted
  * @param handle_size
- *  [in]  "handle" size (0 if a SOCK passed in "handle")
+ *  [in]  "handle" size (or 0 if a SOCK passed in "handle")
  * @param sock
  *  [out] SOCK built on top of the "handle"
  * @param data
@@ -800,7 +885,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Close(SOCK sock);
  *  On MS-Windows closing a SOCK whose OS handle has been used elsewhere
  *  (e.g. in SOCK_CreateOnTop()) may render the OS handle unresponsive, so
  *  always make sure to close the current SOCK first (assuming
- *  fSOCK_KeepOnClose), and only then use the extracted handle to buil
+ *  fSOCK_KeepOnClose), and only then use the extracted handle to build
  *  another socket.
  * @param sock
  *  [in]  handle of the socket to close
@@ -842,8 +927,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_CloseOSHandle
  * eIO_Open (as "event") can be used to check whether the socket has been
  * connected.  When eIO_Read is requested as an "event" for a datagram socket,
  * then eIO_Closed results if the internally latched message has been entirely
- * read out.  Either of eIO_Open, eIO_Write and eIO_ReadWrite always succeed
- * immediately for the datagram socket.
+ * read out.  Either of eIO_Open, eIO_Write and eIO_ReadWrite always succeeds
+ * immediately for a datagram socket.
  * @param sock
  *  [in]  socket handle
  * @param event
@@ -1005,8 +1090,8 @@ extern NCBI_XCONNECT_EXPORT const STimeout* SOCK_GetTimeout
  * whether it is okay to read again.
  *
  * As a special case, "buf" may passed as NULL:
- *   eIO_ReadPeek      -- read up to "size" bytes and store them
- *                        in internal buffer;
+ *   eIO_ReadPeek      -- read up to "size" bytes and store them in internal
+ *                        buffer;
  *   eIO_Read[Persist] -- discard up to "size" bytes from internal buffer
  *                        and socket (check "*n_read" to know how many).
  *
@@ -1032,7 +1117,7 @@ extern NCBI_XCONNECT_EXPORT const STimeout* SOCK_GetTimeout
  * @param how
  *  [in]  how to read the data
  * @sa
- *  SOCK_SetTimeout
+ *  SOCK_SetTimeout, SOCK_PushBack
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Read
 (SOCK           sock,
@@ -1043,52 +1128,61 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Read
  );
 
 
-/** Read a line from SOCK.  A line is terminated by either '\n' (with
- * an optional preceding '\r') or '\0'.  Returned result is always '\0'-
- * terminated and having '\r'(if any)'\n' stripped.  *n_read (if 'n_read'
- * passed non-NULL) contains the numbed of characters written into
- * 'buf' (not counting the terminating '\0').  If 'buf', whose size is
- * specified via 'size' parameter, is not big enough to contain the
- * line, all 'size' bytes will be filled, with *n_read == size upon
- * return.  Note that there will be no terminating '\0' in this
- * (and the only) case, which the caller can easily distinguish.
+/** Read a line from SOCK.  A line is terminated by either '\n' (with an
+ * optional preceding '\r') or '\0', and is stored in the buffer "line" of
+ * "size" characters long.  "*n_read" (if "n_read" passed non-NULL) receives
+ * the number of characters written into "line", not counting the terminating
+ * '\0'.  Returned result is '\0'-terminated (and has '\r'(if any)'\n'
+ * stripped) but only if "size" is big enough to contain the entire line.
+ * Otherwise, all "size" bytes are filled, and "*n_read" == "size" upon return,
+ * and there is no terminating '\0' in this (and the only!) case, which the
+ * caller can easily distinguish.  The remainder of the line can be read with
+ * successive call(s) to SOCK_ReadLine() until the '\0' terminator is received
+ * in the buffer.
  * @param sock
  *  [in]  socket handle
- * @param buf
- *  [out] data buffer to read to
+ * @param line
+ *  [out] line buffer to read to, non-NULL
  * @param size
- *  [in]  max # of bytes to read to "buf"
+ *  [in]  max # of bytes to read to "line", non-0
  * @param n_read
- *  [out] # of bytes read  (can be NULL)
+ *  [out] # of bytes read  (optional, can be NULL)
  * @return
- *  Return code eIO_Success upon successful completion, other - upon
- *  an error.  Note that *n_read must be analyzed prior to return code,
- *  because the buffer could have received some contents before
- *  the indicated error occurred (especially when connection closed).
+ *  Return eIO_Success upon successful completion:  either a line separator is
+ *  encountered or the buffer is filled up completely, and any excess read has
+ *  been successfully saved for further I/O.  Other code, otherwise.
+ * @note that "*n_read" should be analyzed prior to the return status, because
+ *  the buffer could have received some contents before the indicated error
+ *  occurred (e.g. when the connection closes before a line separator is seen).
  * @sa
- *  SOCK_SetTimeout
+ *  SOCK_SetTimeout, SOCK_PushBack
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_ReadLine
 (SOCK    sock,
- char*   buf,
+ char*   line,
  size_t  size,
  size_t* n_read
  );
 
 
-/** Push the specified data back to the socket input queue (in the socket's
- * internal read buffer). These can be any data, not necessarily the data
- * previously read from the socket.
+/** Push the specified data back to the socket's input queue (in the socket's
+ * internal read buffer).  These can be any data, not necessarily the data
+ * previously read from the socket.  The most recently pushed back data are the
+ * data that will be read from the socket first (by either SOCK_ReadLine() or
+ * SOCK_Read()).  The presence of pushed back data makes the socket ready
+ * for read immediately (SOCK_Wait(sock, eIO_Read) succeeds without waiting).
  * @param sock
  *  [in]  socket handle
- * @param buf
+ * @param data
  *  [in]  data to push back to the socket's local buffer
  * @param size
- *  [in]  # of bytes (starting at "buf") to push back
+ *  [in]  # of bytes (starting at "data") to push back
+ * @sa
+ *   SOCK_Read, SOCK_ReadLine, SOCK_Wait
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Pushback
 (SOCK        sock,
- const void* buf,
+ const void* data,
  size_t      size
  );
 
@@ -1128,29 +1222,29 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Status
  );
 
 
-/** Write "size" bytes from buffer "buf" to "sock".
+/** Write "size" bytes of "data" to "sock".
  * @param sock
  *  [in]  socket handle
- * @param buf
+ * @param data
  *  [in]  data to write to the socket
  * @param size
- *  [in]  # of bytes (starting at "buf") to write
+ *  [in]  # of bytes (starting at "data") to write
  * @param n_written
  *  [out] # of written bytes (can be NULL)
  * @param how
- *  [in]  eIO_WritePlain | eIO_WritePersist
+ *  [in]  either eIO_WritePlain or eIO_WritePersist
  * @return
  *  In "*n_written", return the number of bytes actually written.
  *  eIO_WritePlain   -- write as many bytes as possible at once and return
- *                      immediately; if no bytes can be written then wait
+ *                      immediately;  if no bytes can be written then wait
  *                      at most WRITE timeout, try again and return.
  *  eIO_WritePersist -- write all data (doing an internal retry loop
- *                      if necessary); if any single write attempt times out
+ *                      if necessary);  if any single write attempt times out
  *                      or fails then stop writing and return (error code).
  *  Return status: eIO_Success -- some bytes were written successfully  [Plain]
  *                             -- all bytes were written successfully [Persist]
  *                 other code denotes an error, but some bytes might have
- *                 been sent nevertheless (always check *n_written to know).
+ *                 been sent nevertheless (always check "*n_written" to know).
  *
  * @note  With eIO_WritePlain the call returns eIO_Success if and only if
  *        some data were actually written to the socket.  If no data could
@@ -1168,7 +1262,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Status
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status SOCK_Write
 (SOCK            sock,
- const void*     buf,
+ const void*     data,
  size_t          size,
  size_t*         n_written,
  EIO_WriteMethod how
@@ -1370,7 +1464,7 @@ extern NCBI_XCONNECT_EXPORT ESwitch SOCK_SetReadOnWriteAPI
  * @param sock
  *  [in]  socket handle
  * @param on_off
- *
+ *  [in]  R-on-W setting for this socket
  * @return
  *  Prior setting
  * @sa
@@ -1411,7 +1505,7 @@ extern NCBI_XCONNECT_EXPORT void SOCK_SetCork
  * Disabling the Nagle algorithm causes all internally pending yet
  * untransmitted data to flush down to the hardware.
  * @note The setting is overridden by SOCK_SetCork() but it still performs the
- * flush, if set to disable.
+ *       flush, if set to disable.
  * @param sock
  *  [in]  socket handle [stream socket only]
  * @param on_off
@@ -1432,16 +1526,16 @@ extern NCBI_XCONNECT_EXPORT void SOCK_DisableOSSendDelay
  *  How the datagram exchange API works:
  *
  *  Datagram socket is created with special DSOCK_Create[Ex]() calls but the
- *  resulting object is a SOCK handle.  That is, almost all SOCK routines
+ *  resulting object is still a SOCK handle.  That is, almost all SOCK routines
  *  may be applied to the handle.  There are few exceptions, though.
- *  In datagram sockets I/O differs from how it is done in stream sockets:
+ *  In datagram sockets the I/O differs from how it is done in stream sockets:
  *
  *  SOCK_Write() writes data into an internal message buffer, appending new
  *  data as they come with each SOCK_Write().  When the message is complete,
- *  SOCK_SendMsg() should be called (optionally with additional last,
+ *  SOCK_SendMsg() should be called (optionally with an additional last,
  *  or the only [if no SOCK_Write() preceded the call] message fragment)
  *  to actually send the message down the wire.  If successful, SOCK_SendMsg()
- *  cleans the internal buffer, and the process may repeat.  If unsuccessful,
+ *  clears the internal buffer, and the process may repeat.  If unsuccessful,
  *  SOCK_SendMsg() can be repeated with restiction that no additional data are
  *  provided in the call.  This way, the entire message will be attempted to
  *  be sent again.  On the other hand, if after any SOCK_SendMsg() new data
@@ -1458,7 +1552,7 @@ extern NCBI_XCONNECT_EXPORT void SOCK_DisableOSSendDelay
  *  in the internal buffer].  Optimized version can supply a maximal message
  *  size (if known in advance), or 0 to get a message of any allowed size.
  *  The actual size of the received message can be obtained via a
- *  pointer-type argument 'msgsize'.  The message kept in the internal buffer
+ *  pointer-type argument "msgsize".  The message kept in the internal buffer
  *  can be read out in several SOCK_Read() calls, last returning eIO_Closed,
  *  when all data have been taken out.  SOCK_Wait() returns eIO_Success while
  *  there are data in the internal message buffer that SOCK_Read() can read.
@@ -1495,12 +1589,12 @@ extern NCBI_XCONNECT_EXPORT EIO_Status DSOCK_Create
  * busy") unless SOCK_SetReuseAddress() is called, which then allows multiple
  * sockets to bind to the same port, and receive messages, in undefined order,
  * arriving at that port.
- * Passing 0 will ask the OS to automatically select an unused port, which then
+ * Passing 0 will ask the OS to automatically choose an unused port, which then
  * can be obtained via SOCK_GetLocalPort().
  * @param sock
  *  [in]  SOCK from DSOCK_Create[Ex]()
  * @param port
- *  [in]  port to bind to (0 to auto-select)
+ *  [in]  port to bind to (0 to auto-choose)
  * @sa
  *  SOCK_SetReuseAddress, SOCK_GetLocalPort
  */
@@ -1844,7 +1938,7 @@ extern NCBI_XCONNECT_EXPORT ESwitch SOCK_SetDataLogging
  );
 
 
-/*  User-level error hook.
+/** User-level error hook.
  */
 
 typedef enum {
@@ -1858,16 +1952,17 @@ typedef struct {
     SOCK           sock;    /**< Non-NULL when SOCK-related                  */
     const char*    host;    /**< Host name/IP (or path for non-IP SOCK)      */
     unsigned short port;    /**< Port (host byte order), 0 for non-IP SOCK   */
-    EIO_Event      event;   /**< Meaningful only for eSOCK_ErrIO             */
+    EIO_Event      event;   /**< Meaningful only for the eSOCK_ErrIO type    */
     EIO_Status     status;  /**< Status code about to be returned (if known) */
 } SSOCK_ErrInfo;
 
 
-typedef void (*FSOCK_ErrHook)(const SSOCK_ErrInfo* info, void* data);
+typedef void (*FSOCK_ErrHook)(const SSOCK_ErrInfo* info,
+                              void*                data);
 
-
-extern NCBI_XCONNECT_EXPORT void SOCK_SetErrHookAPI(FSOCK_ErrHook hook,
-                                                    void*         data);
+extern NCBI_XCONNECT_EXPORT
+void SOCK_SetErrHookAPI(FSOCK_ErrHook hook,
+                        void*         data);
 
 
 
@@ -1897,10 +1992,10 @@ typedef struct {
  *  [out]     how many elements of the "polls" array returned with their
  *            I/O marked ready
  * @return
- *  eIO_Success if at least one element was found ready, eIO_Timeout if
- *  none were and the specified time interval had elapsed, other error code
- *  for some other error condition (in which case the "revent" fields in the
- *  array may not have been updated with valid values).
+ *  eIO_Success if at least one element was found ready, eIO_Timeout if none
+ *  and the specified time interval had elapsed, other error code for some
+ *  other error condition (in which case the "revent" fields in the array may
+ *  not have been updated with valid values).
  * @sa
  *   SOCK_Wait, SOCK_Poll
  */
@@ -1912,9 +2007,13 @@ extern NCBI_XCONNECT_EXPORT EIO_Status POLLABLE_Poll
  );
 
 
+/** Identifies a non-NULL POLLABLE. */
+extern NCBI_XCONNECT_EXPORT ESOCK_Type POLLABLE_What(POLLABLE);
+
+
 /** Conversion utilities from handles to POLLABLEs, and back.
  * @return
- *  Return 0 if conversion cannot be made; otherwise the converted handle
+ *  Return 0 if conversion cannot be made; otherwise the converted handle.
  */
 extern NCBI_XCONNECT_EXPORT POLLABLE POLLABLE_FromSOCK   (SOCK);
 extern NCBI_XCONNECT_EXPORT POLLABLE POLLABLE_FromLSOCK  (LSOCK);
@@ -2032,7 +2131,7 @@ unsigned short SOCK_htons(unsigned short);
  *  Zero on success, non-zero on error.  See BSD gethostname().
  *  On error (incl. insufficient room) "name" returned empty (name[0] == '\0').
  * @sa
- *  SOCK_gethostname
+ *  SOCK_gethostname, SOCK_gethostbyaddr
  */
 extern NCBI_XCONNECT_EXPORT int SOCK_gethostnameEx
 (char*   name,
@@ -2041,9 +2140,9 @@ extern NCBI_XCONNECT_EXPORT int SOCK_gethostnameEx
  );
 
 
-/** Same as SOCK_gethostnameEx(,,eOff)
+/** Same as SOCK_gethostnameEx(,,<current API data logging>)
  * @sa
- *  SOCK_gethostnameEx
+ *  SOCK_gethostnameEx, SOCK_gethostbyaddrEx, SOCK_SetDataLoggingAPI
  */
 extern NCBI_XCONNECT_EXPORT int SOCK_gethostname
 (char*  name,
@@ -2053,19 +2152,19 @@ extern NCBI_XCONNECT_EXPORT int SOCK_gethostname
 
 /** Find and return IPv4 address of a named host.  The call also accepts dotted
  * IPv4 notation, in which case the conversion is done without consulting the
- * name resolver).
+ * name resolver (DNS).
  * @param hostname
- *  [in]  specified host, or the current host if hostname is 0
+ *  [in]  specified host, or the current host if hostname is either 0 or ""
  * @param log
  *  [in]  whether to log failures
  * @return
- *  IPv4 host address (in network byte order) of the specified host (or of
- *  local host, if hostname is passed as NULL), which could have been specified
+ *  IPv4 host address (in network byte order) of the specified host (or of the
+ *  local host, if hostname is passed as NULL / empty), which can be specified
  *  as either a domain name or an IPv4 address in the dotted notation (e.g.
  *  "123.45.67.89\0").  Return 0 on error.
- * @note "0.0.0.0" and "255.255.255.255" are both considered invalid.
+ * @note "0.0.0.0" and "255.255.255.255" are both considered invalid
  * @sa
- *  SOCK_gethostbyname, SOCK_gethostname
+ *  SOCK_gethostbyname, SOCK_gethostname, NcbiStringToIPv4
  */
 extern NCBI_XCONNECT_EXPORT unsigned int SOCK_gethostbynameEx
 (const char* hostname,
@@ -2073,9 +2172,9 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_gethostbynameEx
  );
 
 
-/** Same as SOCK_gethostbynameEx(,eOff)
+/** Same as SOCK_gethostbynameEx(,<current API data logging>)
  * @sa
- *  SOCK_gethostbynameEx
+ *  SOCK_gethostbynameEx, SOCK_SetDataLoggingAPI
  */
 extern NCBI_XCONNECT_EXPORT unsigned int SOCK_gethostbyname
 (const char* hostname
@@ -2097,7 +2196,7 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_gethostbyname
  *  Value 0 means error;  success is denoted by the 'name' argument returned.
  *  Note that on error the name gets emptied (name[0] == '\0').
  * @sa
- *  SOCK_gethostbyaddr
+ *  SOCK_gethostbyaddr, SOCK_gethostname
  */
 extern NCBI_XCONNECT_EXPORT const char* SOCK_gethostbyaddrEx
 (unsigned int addr,
@@ -2107,9 +2206,9 @@ extern NCBI_XCONNECT_EXPORT const char* SOCK_gethostbyaddrEx
  );
 
 
-/** Same as SOCK_gethostbyaddrEx(,,eOff)
+/** Same as SOCK_gethostbyaddrEx(,,<current API data logging>)
  * @sa
- *  SOCK_gethostbyaddrEx
+ *  SOCK_gethostbyaddrEx, SOCK_gethostnameEx, SOCK_SetDataLoggingAPI
  */
 extern NCBI_XCONNECT_EXPORT const char* SOCK_gethostbyaddr
 (unsigned int addr,
@@ -2127,9 +2226,9 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_GetLoopbackAddress(void);
 
 /** Get (and cache for faster follow-up retrievals) IPv4 address of local host
  * @param reget
- *  eOn      to forcibly recache and return the address;
- *  eDefault to recache only if unknown, return the cached value;
- *  eOff     not to recache even if unknown, return whatever is available.
+ *  eOn      to forcibly re-cache and return the address;
+ *  eDefault to re-cache only if unknown, return the [new] cached value;
+ *  eOff     not to re-cache even if unknown, return whatever is available.
  * @return
  *  Local address (in network byte order).
  * @sa
@@ -2153,6 +2252,7 @@ extern NCBI_XCONNECT_EXPORT unsigned int SOCK_GetLocalHostAddress
  *  If no host/port detected, return 'str'.  On format error, return 0.
  *  If either host or port fragment is missing, then the corresponding 'host'/
  *  'port' parameters get a value of 0.
+ * @note  "0.0.0.0" for the host part gets the host returned as 0 as well.
  * @note  'host' gets returned in network byte order, unlike 'port', which
  *        always comes out in host (native) byte order.
  * @note  ":0" is accepted to denote no-host:zero-port.  
@@ -2169,7 +2269,7 @@ extern NCBI_XCONNECT_EXPORT const char* SOCK_StringToHostPort
 /** Print numeric string "host:port" into a buffer provided, not to exceed
  * 'bufsize' bytes (including the teminating '\0' character).  Suppress
  * printing the "host" if the 'host' parameter is zero.  Suppress printing the
- * ":port" part if 'port' passed as zero.  If both the 'host' and the 'port'
+ * ":port" part if 'port' passed as zero, but if both the 'host' and the 'port'
  * parameters are zero, output is the literal ":0".
  * @param host
  *  IPv4 in network byte order
@@ -2204,7 +2304,7 @@ typedef struct SNcbiCred* NCBI_CRED;
 
 /*fwdecl*/
 struct SOCKSSL_struct;
-/** Opaque type for SSL impelentation */
+/** Opaque type for SSL implementation */
 typedef const struct SOCKSSL_struct* SOCKSSL;
 
 
@@ -2216,7 +2316,7 @@ typedef SOCKSSL (*FSSLSetup)(void);
  * @param setup
  *  non-NULL SSL setup routine, or NULL to shut the SSL down
  * @warning
- *  Do not use this function unless you know what you're doing.
+ *  Do _not_ use this function unless you know what you're doing.
  *  Use other means of initialization such as CONNECT_Init() or CConnIniter.
  * @sa
  *  SOCK_SetupSSLEx, CONNECT_Init, CConnIniter
@@ -2230,7 +2330,7 @@ extern NCBI_XCONNECT_EXPORT void SOCK_SetupSSL(FSSLSetup setup);
  * @return
  *  eIO_Success if successful, other code on error.
  * @warning
- *  Do not use this function unless you know what you're doing.
+ *  Do _not_ use this function unless you know what you're doing.
  *  Use other means of initialization such as CONNECT_Init() or CConnIniter.
  * @sa
  *  SOCK_SetupSSL, CONNECT_Init, CConnIniter
