@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SOCKET__HPP
 #define CONNECT___NCBI_SOCKET__HPP
 
-/* $Id: ncbi_socket.hpp 604953 2020-04-05 03:18:20Z lavr $
+/* $Id: ncbi_socket.hpp 693400 2025-02-11 14:12:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -58,7 +58,8 @@ class NCBI_XCONNECT_EXPORT CPollable
 {
 public:
     virtual
-    EIO_Status GetOSHandle(void* handle_buf, size_t handle_size) const = 0;
+    EIO_Status GetOSHandle(void* handle_buf, size_t handle_size,
+                           EOwnership ownership = eNoOwnership) const = 0;
 
     virtual
     POLLABLE   GetPOLLABLE(void) const = 0;
@@ -95,8 +96,9 @@ public:
     EIO_Status IsSet(void);
     EIO_Status Reset(void);
 
-    /// Access to the the system-specific handle.
-    virtual EIO_Status GetOSHandle(void* handle_buf, size_t handle_size) const;
+    /// Access to the system-specific handle.
+    virtual EIO_Status GetOSHandle(void* handle_buf, size_t handle_size,
+                                   EOwnership ownership = eNoOwnership) const;
 
     /// Access to the underlying "TRIGGER".
     TRIGGER GetTRIGGER(void) const;
@@ -143,6 +145,22 @@ public:
             const STimeout* timeout = kInfiniteTimeout,
             TSOCK_Flags     flags   = fSOCK_LogDefault);
 
+    /// Create a client-side socket connected to "host:port".
+    /// @note  The created underlying "SOCK" will be owned by the "CSocket";
+    /// @note  Timeout from the argument becomes new eIO_Open timeout.
+    ///
+    /// @param hostport
+    ///  "host:port" as a string
+    /// @param timeout
+    ///  maximal time to wait for connection to be established
+    /// @param flags
+    ///  additional socket properties (including logging)
+    /// @sa
+    ///  CSocket::Connect, SOCK_Create
+    CSocket(const string&   hostport,
+            const STimeout* timeout = kInfiniteTimeout,
+            TSOCK_Flags     flags   = fSOCK_LogDefault);
+
     /// Variant of the above, which takes host as a binary value in 
     /// network byte order.
     /// @param host
@@ -195,6 +213,22 @@ public:
                        const STimeout* timeout = kDefaultTimeout,
                        TSOCK_Flags     flags   = fSOCK_LogDefault);
 
+    /// Connect to "host:port".
+    /// @note  Should not be called if already connected;
+    /// @note  Timeout from the argument becomes new eIO_Open timeout.
+    ///
+    /// @param hostport
+    ///  "host:port" as a string
+    /// @param timeout
+    ///  maximal time to wait for connection to be established
+    /// @param flags
+    ///  additional socket properties (including logging)
+    /// @sa
+    ///  SOCK_Create
+    EIO_Status Connect(const string&   hostport,
+                       const STimeout* timeout = kDefaultTimeout,
+                       TSOCK_Flags     flags   = fSOCK_LogDefault);
+
     /// Reconnect to the same address.
     /// @note  The socket must not be closed by the time this call is made;
     /// @note  Not for the sockets created by CListeningSocket::Accept();
@@ -233,12 +267,10 @@ public:
 
     /// Set timeout for I/O in the specified direction.
     /// @note  Initially all timeouts are infinite by default;
-    /// @note  SetTimeout(..., kDefaultTimeout) has no effect;
-    /// @note  GetTimeout(eIO_ReadWrite) returns the least
-    ///        of eIO_Read and eIO_Write ones.
-    ///
-    /// @param  event
-    ///  either of eIO_Open, eIO_Read, eUI_Write, eIO_ReadWrite, or eIO_Close
+    /// @note  SetTimeout(..., kDefaultTimeout) has no effect and results
+    ///        eIO_Success unconditionally.
+    /// @param event
+    ///  either of eIO_Open, eIO_Read, eIO_Write, eIO_ReadWrite, or eIO_Close
     /// @param timeout
     ///  new value to set (including special kInfiniteTimeout)
     /// @sa
@@ -247,6 +279,13 @@ public:
                                const STimeout* timeout);
 
     /// Get timeout for I/O in the specified direction.
+    /// @param event
+    ///  either of eIO_Open, eIO_Read, eIO_Write, eIO_ReadWrite, or eIO_Close
+    /// @return
+    ///  A timeout value set (or defaulted) in the "event" direction; or
+    ///  kDefaultTimeout for an unrecognized "event" value
+    /// @note  GetTimeout(eIO_ReadWrite) returns the least
+    ///        of eIO_Read and eIO_Write ones.
     /// @sa
     ///  CSocket::SetTimeout, SOCK_GetTimeout
     const STimeout* GetTimeout(EIO_Event event) const;
@@ -366,9 +405,9 @@ public:
     /// @param handle_size
     ///
     /// @sa
-    ///  SOCK_GetOSHandle, SOCK_GetOSHandleSize
-    virtual EIO_Status GetOSHandle(void*  handle_buf,
-                                   size_t handle_size) const;
+    ///  SOCK_GetOSHandleEx, CSocketAPI::OSHandleSize, SOCK_GetOSHandleSize
+    virtual EIO_Status GetOSHandle(void*  handle_buf, size_t handle_size,
+                                   EOwnership ownership = eNoOwnership) const;
 
     /// @note  Use CSocketAPI::SetReadOnWrite() to set the default value.
     ///
@@ -501,16 +540,18 @@ public:
     CDatagramSocket(TSOCK_Flags flags = fSOCK_LogDefault);
 
     /// @param port
-    ///
+    ///  port is in host byte order
     EIO_Status Bind(unsigned short port);
-
   
     /// @param host
-    ///
+    ///  host name or IP address as a string
     /// @param port
     ///  "port" is in host byte order
     EIO_Status Connect(const string& host, unsigned short port);
 
+    /// @param hostport
+    ///   "host:port" as a string (no-port and no-host accepted, such as ":0")
+    EIO_Status Connect(const string& hostport);
     
     /// @param host
     ///  "host" is accepted in network byte order
@@ -534,8 +575,8 @@ public:
     ///
     /// @param maxmsglen
     ///
-    EIO_Status Recv(void*           buf         = 0,
-                    size_t          buflen      = 0,
+    EIO_Status Recv(void*           buf,
+                    size_t          buflen,
                     size_t*         msglen      = 0,
                     string*         sender_host = 0,
                     unsigned short* sender_port = 0,
@@ -549,8 +590,8 @@ public:
     ///
     /// @param port
     ///  host byte order
-    EIO_Status Send(const void*     data    = 0,
-                    size_t          datalen = 0,
+    EIO_Status Send(const void*     data,
+                    size_t          datalen,
                     const string&   host    = string(),
                     unsigned short  port    = 0);
 
@@ -649,7 +690,8 @@ public:
     ///
     /// @param handle_size
     ///
-    virtual EIO_Status GetOSHandle(void* handle_buf, size_t handle_size) const;
+    virtual EIO_Status GetOSHandle(void* handle_buf, size_t handle_size,
+                                   EOwnership ownership = eNoOwnership) const;
 
     /// Return port which the server listens on
     unsigned short GetPort(ENH_ByteOrder byte_order) const;
@@ -721,10 +763,11 @@ public:
     static ESwitch SetDataLogging(ESwitch log);
 
     /// Polling structure
+    /// m_Event can be either of eIO_Open, eIO_Read, eIO_Write, eIO_ReadWrite
     struct SPoll {
-        CPollable* m_Pollable;  ///< object pointer (or NULL not to poll)
-        EIO_Event  m_Event;     ///< event inqury (or eIO_Open not to poll)
-        EIO_Event  m_REvent;    ///< event ready (eIO_Open if not ready)
+        CPollable* m_Pollable;  ///< [in]  object pointer (or NULL not to poll)
+        EIO_Event  m_Event;     ///< [in]  event inqury (or eIO_Open for none)
+        EIO_Event  m_REvent;    ///< [out] event ready (eIO_Open if not ready)
 
         SPoll(CPollable* pollable = 0, EIO_Event event = eIO_Open)
             : m_Pollable(pollable), m_Event(event), m_REvent(eIO_Open)
@@ -733,7 +776,7 @@ public:
 
     /// Poll a vector of CPollable objects for I/O readiness.
     ///
-    /// @note  If possible, consider using lower-level SOCK_Poll() for
+    /// @note  If possible, please consider using lower-level SOCK_Poll() for
     ///        performance reasons (the code may still use a vector to prepare
     ///        the array of elements, then pass vector::data() and
     ///        vector::size() when making the SOCK_Poll() call).
@@ -772,7 +815,7 @@ public:
     static string    HostPortToString(unsigned int   host,
                                       unsigned short port);
 
-    /// Return position past the end of parsed portion, NPOS on error
+    /// Return position past the end of the parsed portion, NPOS on error
     static SIZE_TYPE StringToHostPort(const string&   str,
                                       unsigned int*   host,
                                       unsigned short* port);
@@ -824,7 +867,7 @@ inline EIO_Status CTrigger::Reset(void)
 }
 
 
-inline EIO_Status CTrigger::GetOSHandle(void*, size_t) const
+inline EIO_Status CTrigger::GetOSHandle(void*, size_t, EOwnership) const
 {
     return m_Trigger ? eIO_NotSupported : eIO_Closed;
 }
@@ -905,9 +948,12 @@ inline unsigned short CSocket::GetRemotePort(ENH_ByteOrder byte_order) const
 }
 
 
-inline EIO_Status CSocket::GetOSHandle(void* hnd_buf, size_t hnd_siz) const
+inline EIO_Status CSocket::GetOSHandle(void* handle_buf, size_t handle_size,
+                                       EOwnership ownership) const
 {
-    return m_Socket? SOCK_GetOSHandle(m_Socket, hnd_buf, hnd_siz) : eIO_Closed;
+    return m_Socket
+        ? SOCK_GetOSHandleEx(m_Socket, handle_buf, handle_size, ownership)
+        : eIO_Closed;
 }
 
  
@@ -1109,10 +1155,12 @@ inline EIO_Status CListeningSocket::GetStatus(void) const
 
 
 inline EIO_Status CListeningSocket::GetOSHandle(void*  handle_buf,
-                                                size_t handle_size) const
+                                                size_t handle_size,
+                                                EOwnership ownership) const
 {
     return m_Socket
-        ? LSOCK_GetOSHandle(m_Socket, handle_buf, handle_size) : eIO_Closed;
+        ? LSOCK_GetOSHandleEx(m_Socket, handle_buf, handle_size, ownership)
+        : eIO_Closed;
 }
 
 
