@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_UTIL__H
 #define CONNECT___NCBI_UTIL__H
 
-/* $Id: ncbi_util.h 668369 2023-06-03 01:48:35Z lavr $
+/* $Id: ncbi_util.h 689726 2024-11-05 14:10:58Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,11 +44,13 @@
  *                CORE_SetLOGFILE[_Ex](), CORE_SetLOGFILE_NAME[_Ex]()
  *                LOG_ToFILE[_Ex]()
  *
- * 2. Auxiliary API:
+ * 2. Auxiliary APIs:
  *       CORE_GetAppName()
- *       CORE_GetNcbiRequestID()
  *       CORE_GetPlatform()
+ *       CORE_GetNcbiRequestID()
+ *
  *       CORE_GetUsername[Ex]()
+ * 
  *       CORE_GetVMPageSize()
  *       CORE_Msdelay()
  *
@@ -57,6 +59,8 @@
  *       UTIL_Adler32_Update()
  *
  * 4. Miscellaneous:
+ *       UTIL_Timezone()
+ *       UTIL_HelpRequested()
  *       UTIL_MatchesMask[Ex]()
  *       UTIL_NcbiLocalHostName()
  *       UTIL_PrintableString[Ex|Size]()
@@ -67,6 +71,7 @@
 
 #include <connect/ncbi_core.h>
 #include <stdio.h>
+#include <time.h>
 
 
 /** @addtogroup UtilityFunc
@@ -414,7 +419,7 @@ extern NCBI_XCONNECT_EXPORT REG  CORE_GetREG(void);
 
 
 /******************************************************************************
- *  Auxiliary API
+ *  Auxiliary APIs
  */
 
 /** Obtain current application name (toolkit dependent).
@@ -428,18 +433,22 @@ extern NCBI_XCONNECT_EXPORT REG  CORE_GetREG(void);
 extern NCBI_XCONNECT_EXPORT const char* CORE_GetAppName(void);
 
 
+/** Return NCBI platrofm ID (if known).
+ * @return
+ *  Return read-only textual but machine-readable platform description.
+ */
+extern NCBI_XCONNECT_EXPORT const char* CORE_GetPlatform(void);
+
+
 /** NCBI request ID enumerator */
 typedef enum {
     eNcbiRequestID_None = 0,
     eNcbiRequestID_HitID,     /**< NCBI Hit     ID */
-    eNcbiRequestID_SID        /**< NCBI Session ID */
+    eNcbiRequestID_SID,       /**< NCBI Session ID */
+    eNcbiTraceState,          /**< OpenTelemetry tracestate header */
+    eNcbiTraceParent          /**< OpenTelemetry traceparent header */
 } ENcbiRequestID;
 
-
-/** NCBI request "DTab-Local" header  */
-extern NCBI_XCONNECT_EXPORT char* CORE_GetNcbiRequestDtab;
-
-
 /** Obtain current NCBI request ID (if known, per thread).
  * @return
  *  Return NULL when the ID cannot be determined or an error has occurred;
@@ -450,13 +459,6 @@ extern NCBI_XCONNECT_EXPORT char* CORE_GetNcbiRequestID
 (ENcbiRequestID reqid);
 
 
-/** Return NCBI platrofm ID (if known).
- * @return
- *  Return read-only textual but machine-readable platform description.
- */
-extern NCBI_XCONNECT_EXPORT const char* CORE_GetPlatform(void);
-
-
 /** Select which username is the most preferable to obtain from the system. */
 typedef enum {
     eCORE_UsernameCurrent,  /**< process UID */
@@ -464,7 +466,6 @@ typedef enum {
     eCORE_UsernameReal      /**< real UID    */
 } ECORE_Username;
 
-
 /** Obtain and store in the buffer provided, the best (as possible) user name
  * that matches the requested username selector.
  * Both "buf" and "bufsize" must not be zeros.
@@ -718,7 +719,8 @@ extern NCBI_XCONNECT_EXPORT size_t UTIL_PrintableStringSize
  );
 
 
-#define UTIL_PRINTABLE_WIDTH  80   /** Default minimum printable width */
+#define UTIL_PRINTABLE_WIDTH_MIN  80  /** Default minimum printable width  */
+#define UTIL_PRINTABLE_WIDTH     130  /** Default printable width for data */
 
 
 typedef enum {
@@ -787,6 +789,14 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ UTIL_HelpRequested
 (int argc, char** argv);
 
 
+/** Return timezone offset (in seconds West of UTC) for the current time zone.
+ * Must be called after either mktime() or localtime() or tzset() to return a
+ * valid result.  On platforms that do not provide TZ information, the result
+ * is cached internally for the life of the application.
+ */
+extern NCBI_XCONNECT_EXPORT time_t UTIL_Timezone(void);
+
+
 /** Conversion from Unicode to UTF8, and back.  MSWIN-specific and internal.
  *
  * @note  UTIL_ReleaseBufferOnHeap() must be used to free the buffers returned
