@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_PARAM_IMPL__HPP
 #define CORELIB___NCBI_PARAM_IMPL__HPP
 
-/*  $Id: ncbi_param_impl.hpp 606787 2020-04-27 16:15:35Z lavr $
+/*  $Id: ncbi_param_impl.hpp 621615 2020-12-15 12:55:45Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -230,7 +230,7 @@ typename CParamParser<TDescription, TParam>::TValueType
 CParamParser<TDescription, TParam>::StringToValue(const string& str,
                                                   const TParamDesc&)
 {
-    CNcbiIstrstream in(str.c_str());
+    CNcbiIstrstream in(str);
     TValueType val;
     in >> val;
 
