@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMISC__HPP
 #define CORELIB___NCBIMISC__HPP
 
-/*  $Id: ncbimisc.hpp 668988 2023-06-16 13:42:24Z ivanov $
+/*  $Id: ncbimisc.hpp 695938 2025-04-03 19:38:10Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,10 @@
 #  include <sys/types.h>
 #endif
 
+#if __has_include(<format>)
+#  include <format>
+#endif
+
 #if !defined(HAVE_NULLPTR)  &&  !defined(nullptr)
 #  define nullptr NULL
 #endif
@@ -1214,6 +1218,15 @@ private:
 /// @sa NCBI_DEPRECATED_CTOR
 #define NCBI_DEPRECATED_CLASS NCBI_DEPRECATED_CTOR(class)
 
+#ifdef __clang__ // ... including modern ICC
+// Preexpand to reduce warning noise -- Clang otherwise recapitulates these
+// macros' expansions step by step.
+#  undef NCBI_DEPRECATED_CTOR
+#  undef NCBI_DEPRECATED_CLASS
+#  define NCBI_DEPRECATED_CTOR(decl) decl  __attribute__((deprecated))
+#  define NCBI_DEPRECATED_CLASS      class __attribute__((deprecated))
+#endif
+
 //#define NCBI_ENABLE_SAFE_FLAGS
 #ifdef NCBI_ENABLE_SAFE_FLAGS
 /////////////////////////////////////////////////////////////////////////////
@@ -1252,16 +1265,16 @@ public:
     typedef Enum enum_type;
     typedef typename underlying_type<enum_type>::type storage_type;
 
-    CSafeFlags()
+    constexpr CSafeFlags()
         : m_Flags(0)
         {
         }
     explicit
-    CSafeFlags(storage_type flags)
+    constexpr CSafeFlags(storage_type flags)
         : m_Flags(flags)
         {
         }
-    CSafeFlags(enum_type flags)
+    constexpr CSafeFlags(enum_type flags)
         : m_Flags(static_cast<storage_type>(flags))
         {
         }
@@ -1542,6 +1555,20 @@ template <class TKey, class TStorage> struct hash<ncbi::CStrictId<TKey, TStorage
         return hash<TStorage>()(x.Get());
     }
 };
+
+#  if __has_include(<format>)
+template <class TKey, class TStorage, class TChar>
+struct formatter<ncbi::CStrictId<TKey, TStorage>, TChar>
+    : public formatter<TStorage, TChar>
+{
+    typedef ncbi::CStrictId<TKey, TStorage> TId;
+    
+    template<class TFmtContext>
+    TFmtContext::iterator format(const TId& id, TFmtContext& ctx) const {
+        return formatter<TStorage, TChar>::format(id.Get(), ctx);
+    }
+};
+#  endif // __has_include(<format>)
 #endif /* NCBI_STRICT_GI */
 
 #endif
