@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMTX__HPP
 #define CORELIB___NCBIMTX__HPP
 
-/* $Id: ncbimtx.hpp 604463 2020-03-27 23:02:26Z lavr $
+/* $Id: ncbimtx.hpp 666594 2023-04-27 17:38:45Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -410,9 +410,9 @@ struct SSystemMutex
 {
     SSystemFastMutex m_Mutex; ///< Mutex value
 
-    volatile TThreadSystemID m_Owner; ///< Platform-dependent owner thread ID
+    atomic<TThreadSystemID> m_Owner; ///< Platform-dependent owner thread ID
 
-    volatile int m_Count; ///< # of recursive (in the same thread) locks
+    atomic<int> m_Count; ///< # of recursive (in the same thread) locks
 
     /// Acquire mutex for the current thread.
     NCBI_XNCBI_EXPORT
@@ -1412,7 +1412,7 @@ private:
     mutex                 m_Mtx;
     condition_variable    m_Cv;
     const unsigned int    m_Max;
-    atomic<unsigned int>  m_Count;
+    unsigned int          m_Count;
     bool x_TryAcquire(void);
 #else
     struct SSemaphore* m_Sem;  ///< System-specific semaphore data.
