@@ -1,7 +1,7 @@
 #ifndef CORELIB__NCBITIME__HPP
 #define CORELIB__NCBITIME__HPP
 
-/*  $Id: ncbitime.hpp 606821 2020-04-27 18:50:52Z sadyrovr $
+/*  $Id: ncbitime.hpp 682502 2024-05-01 14:04:19Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1829,6 +1829,14 @@ public:
 class NCBI_XNCBI_EXPORT CDeadline
 {
 public:
+    /// Type of special deadlines.
+    enum EType : unsigned {
+        eInfinite = numeric_limits<unsigned>::max(), ///< Infinite deadline.
+        eNoWait = 0u,                                ///< No-wait, expires immediately.
+    };
+    /// Initialize deadline of specified type.
+    CDeadline(EType type = eNoWait);
+
     /// Initialize deadline using seconds and nanoseconds
     /// (adding to the current time)
     /// @param seconds
@@ -1840,14 +1848,6 @@ public:
     /// Initialize deadline by adding relative timeout to the current time.
     CDeadline(const CTimeout& timeout);
 
-    /// Type of special deadlines.
-    enum EType : unsigned {
-        eInfinite = numeric_limits<unsigned>::max(), ///< Infinite deadline.
-    };
-
-    /// Initialize deadline of specified type.
-    CDeadline(EType);
-
     /// Check if the deadline is infinite.
     bool IsInfinite(void) const { return m_Infinite; }
 
@@ -1868,8 +1868,7 @@ public:
 private:
     CDeadline(CTimeout::EType);
 
-    void x_Now(void);
-    void x_Add(unsigned int seconds, unsigned int nanoseconds);
+    void x_SetNowPlus(unsigned int seconds, unsigned int nanoseconds);
 
     time_t       m_Seconds;
     unsigned int m_Nanoseconds;
