@@ -1,7 +1,7 @@
 #ifndef CORELIB___REQUEST_CTX__HPP
 #define CORELIB___REQUEST_CTX__HPP
 
-/*  $Id: request_ctx.hpp 664060 2023-03-07 16:40:29Z ivanov $
+/*  $Id: request_ctx.hpp 689783 2024-11-06 16:29:40Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -151,6 +151,50 @@ class ITracerSpan
 {
 public:
     virtual ~ITracerSpan(void) {}
+
+    enum ESpanKind {
+        eKind_NotSet,
+        eKind_Internal,
+        eKind_Server,
+        eKind_Client
+    };
+
+    enum ESpanAttribute {
+        eSessionId,
+        eClientAddress,
+        eClientPort,
+        eServerAddress,
+        eServerPort,
+        eUrl,
+        eRequestMethod,
+        eUrlScheme,
+        eHttpScheme,
+        eStatusCode,
+        eStatusString
+    };
+
+    enum EHttpHeaderType {
+        eRequest,
+        eResponse
+    };
+
+    enum ESpanStatus {
+        eSuccess,
+        eError
+    };
+
+    virtual void SetName(const string& name) = 0;
+    virtual void SetAttribute(ESpanAttribute attr, const string& value) = 0;
+    virtual void SetCustomAttribute(const string& attr, const string& value) = 0;
+    virtual void SetHttpHeader(EHttpHeaderType header_type, const string& name, const string& value) = 0;
+    virtual void SetSpanStatus(ESpanStatus status) = 0;
+
+    virtual const string& GetTraceState(void) const = 0;
+    virtual const string& GetTraceParent(void) const = 0;
+
+    virtual void PostEvent(const SDiagMessage& message) = 0;
+
+    virtual void EndSpan(void) = 0;
 };
 
 
@@ -373,6 +417,11 @@ public:
     /// Get current read-only flag.
     bool GetReadOnly(void) const { return m_IsReadOnly; }
 
+    void SetDisabledAppLog(bool disable) { m_DisabledAppLog = disable; }
+    bool GetDisabledAppLog(void) const { return m_DisabledAppLog; }
+    void SetDisabledAppLogEvents(CDiagContext::EDisabledAppLogEvents events) { m_DisabledAppLogEvents = events; }
+    CDiagContext::EDisabledAppLogEvents GetDisabledAppLogEvents(void) const { return m_DisabledAppLogEvents; }
+
     typedef CAtomicCounter::TValue TVersion;
 
     /// Return version increased on every context change (hit/subhit id, client ip,
@@ -381,10 +430,32 @@ public:
 
     /// Set request tracer to be called on context events (start/stop etc.).
     /// @sa IRequestTracer
-    void SetRequestTracer(const shared_ptr<IRequestTracer>& tracer) { m_Tracer = tracer; }
+    static void SetRequestTracer(const shared_ptr<IRequestTracer>& tracer);
+
+    /// At least Opentememetry tracer allows to set span kind only at span creation time.
+    /// These methods allow to pass span kind to the tracer.
+    void SetTracerSpanKind(ITracerSpan::ESpanKind kind) { m_SpanKind = kind; }
+    ITracerSpan::ESpanKind GetTracerSpanKind(void) const { return m_SpanKind; }
 
     void SetTracerSpan(const shared_ptr<ITracerSpan>& span) { m_TracerSpan = span; }
-    shared_ptr<ITracerSpan> GetTracerSpan(void) { return m_TracerSpan; }
+    shared_ptr<ITracerSpan> GetTracerSpan(void) const { return m_TracerSpan; }
+
+    /// OpenTelemetry: get 'tracestate' and 'traceparent' values to be passed in
+    /// HTTP headers. NCBI HTTP streams do this automatically, third party HTTP
+    /// libraries may require manual propagation.
+    const string GetTraceState(void) const;
+    const string GetTraceParent(void) const;
+
+    /// OpenTelemetry: set the incoming 'tracestate' and 'traceparent' values.
+    /// This must be done before the call to StartRequest() for the values
+    /// to be available to the tracer for generating correct trace-id and parent-id.
+    void SetSrcTraceState(const string& state) { m_TraceState = state; }
+    void SetSrcTraceParent(const string& parent) { m_TraceParent = parent; }
+    /// The getters provide access to the incoming values (unlike GetTraceState and
+    /// GetTraceParent which return the new values to be used for outgoing connections).
+    /// The getters are expected to be used mostly by tracer and span classes.
+    const string& GetSrcTraceState(void) const { return m_TraceState; }
+    const string& GetSrcTraceParent(void) const { return m_TraceParent; }
 
 private:
     // Prohibit copying
@@ -480,6 +551,10 @@ private:
     TContextFlags  m_Flags;
     string         m_SubHitIDCache;
 
+    // Disabling diag messages; By default the global values from CDiagContext are used.
+    bool                                m_DisabledAppLog;
+    CDiagContext::EDisabledAppLogEvents m_DisabledAppLogEvents;
+
     // For saving/checking owner TID.
     friend class CDiagContextThreadData;
     // TID of the thread currently using this context or -1.
@@ -495,8 +570,14 @@ private:
     friend class CRequestContext_PassThrough;
     mutable TPassThroughProperties m_PassThroughProperties;
 
+    static shared_ptr<IRequestTracer> sm_Tracer;
+
+    // Make sure the same tracer is used through the whole request.
     shared_ptr<IRequestTracer> m_Tracer;
+    ITracerSpan::ESpanKind m_SpanKind;
     shared_ptr<ITracerSpan> m_TracerSpan;
+    string m_TraceState;
+    string m_TraceParent;
 
     // Patterns from NCBI_CONTEXT_FIELDS variable.
     static unique_ptr<CMaskFileName> sm_ContextFields;
@@ -912,6 +993,20 @@ void CRequestContext::UnsetBytesWr(void)
 }
 
 
+inline
+const string CRequestContext::GetTraceState(void) const
+{
+    return m_TracerSpan ? m_TracerSpan->GetTraceState() : m_TraceState;
+}
+
+
+inline
+const string CRequestContext::GetTraceParent(void) const
+{
+    return m_TracerSpan ? m_TracerSpan->GetTraceParent() : m_TraceParent;
+}
+
+
 inline
 bool CRequestContext::x_IsSetProp(EProperty prop) const
 {
