@@ -1,7 +1,7 @@
 #ifndef CORELIB___TEST_BOOST__HPP
 #define CORELIB___TEST_BOOST__HPP
 
-/*  $Id: test_boost.hpp 631032 2021-05-12 14:09:52Z ivanov $
+/*  $Id: test_boost.hpp 636160 2021-08-17 15:13:26Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,7 +42,6 @@
 
 
 #include <corelib/expr.hpp>
-#include <corelib/ncbiapp.hpp>
 #include <corelib/request_ctx.hpp>
 #include <corelib/ncbisys.hpp>
 #include <corelib/ncbimtx.hpp>
@@ -791,6 +790,9 @@ static struct BOOST_JOIN( test_name, _timeout_spec )                    \
 
 BEGIN_NCBI_SCOPE
 
+class CArgDescriptions;
+class CNcbiApplication;
+
 
 /// Macro for introducing function initializing argument descriptions for
 /// tests. This function will be called before CNcbiApplication will parse
