@@ -1,7 +1,7 @@
 #ifndef DBAPI___DBAPI__HPP
 #define DBAPI___DBAPI__HPP
 
-/* $Id: dbapi.hpp 627083 2021-03-09 14:01:57Z ivanov $
+/* $Id: dbapi.hpp 684655 2024-06-28 14:39:22Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,12 @@
 #include <dbapi/variant.hpp>
 
 
+// Forward declaration.
+BEGIN_NCBI_SCOPE
+class CVersionInfo;
+END_NCBI_SCOPE
+
+
 /** @addtogroup DbAPI
  *
  * @{
@@ -948,6 +954,8 @@ public:
     /// Returns all error messages as a single string
     virtual string GetErrorInfo() = 0;
 
+    virtual const CVersionInfo& GetVersionInfo() const = 0;
+
     /// Returns the internal driver connection object
     virtual CDB_Connection* GetCDB_Connection() = 0;
 };
