@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER_IMPL___DBAPI_IMPL_CONTEXT__HPP
 #define DBAPI_DRIVER_IMPL___DBAPI_IMPL_CONTEXT__HPP
 
-/* $Id: dbapi_impl_context.hpp 653775 2022-08-03 14:42:32Z ucko $
+/* $Id: dbapi_impl_context.hpp 693136 2025-02-06 19:39:57Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -340,6 +340,9 @@ private:
     size_t          m_MaxBlobSize;
     EEncoding       m_ClientEncoding;
 
+
+    void x_ReadDBConfParams(const IRegistry& reg, const string& section_name,
+                            SDBConfParams* params);
     /// Return unused connection "conn" to the driver context for future
     /// reuse (if "conn_reusable" is TRUE) or utilization
     void x_Recycle(CConnection* conn, bool conn_reusable);
