@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___INTERFACES__HPP
 #define DBAPI_DRIVER___INTERFACES__HPP
 
-/* $Id: interfaces.hpp 626337 2021-02-25 18:42:32Z ivanov $
+/* $Id: interfaces.hpp 684655 2024-06-28 14:39:22Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1117,7 +1117,7 @@ public:
     ///   connecting to a server.
     ///
     /// @sa
-    ///   GetApplicationName()
+    ///   GetApplicationName(), InitApplicationName()
     virtual void SetApplicationName(const string& app_name) = 0;
     
     /// @brief 
@@ -1127,9 +1127,17 @@ public:
     ///   Application name.
     ///
     /// @sa
-    ///   SetApplicationName()
+    ///   InitApplicationName(), SetApplicationName()
     virtual string GetApplicationName(void) const = 0;
 
+    /// @brief
+    ///   Explicitly auto-initialize the application name, if necessary.
+    ///
+    /// @sa
+    ///   GetApplicationName(), SetApplicationName()
+    virtual void InitApplicationName(void)
+    { }
+
     /// @brief 
     ///   Set host name.
     /// 
@@ -1397,6 +1405,10 @@ protected:
     /// Get interface for extra features that could be implemented in the driver.
     virtual I_ConnectionExtra& GetExtraFeatures(void) = 0;
 
+    /// Driver version, supplied here rather than by the context for
+    /// ODBC's sake.
+    virtual string GetVersionString(void) const;
+
 public:
     // Deprecated legacy methods.
 
