@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER_ODBC___INTERFACES__HPP
 #define DBAPI_DRIVER_ODBC___INTERFACES__HPP
 
-/* $Id: interfaces.hpp 535289 2017-05-08 13:38:35Z ivanov $
+/* $Id: interfaces.hpp 684655 2024-06-28 14:39:22Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -238,6 +238,7 @@ private:
 
 
     friend class CODBCContextRegistry;
+    friend class CODBC_Connection;
 };
 
 
@@ -312,6 +313,7 @@ protected:
     }
 
     virtual string GetDriverName(void) const;
+    virtual string GetVersionString(void) const;
 
 protected:
     // for NCBI_DATABASE_THROW_ANNOTATED
@@ -667,10 +669,10 @@ private:
         return m_Cmd;
     }
     bool x_AssignParams(void* p);
-    static int x_GetBCPDataType(EDB_Type type);
+    static int x_GetBCPDataType(const CDB_Object& param);
     static size_t x_GetBCPDataSize(EDB_Type type);
-    static void* x_GetDataTerminator(EDB_Type type);
-    static size_t x_GetDataTermSize(EDB_Type type);
+    static const void* x_GetDataTerminator(const CDB_Object& param);
+    static size_t x_GetDataTermSize(const CDB_Object& param);
     static const void* x_GetDataPtr(EDB_Type type, void* pb);
 
     SQLHDBC m_Cmd;
@@ -754,6 +756,8 @@ protected:
     virtual ~CODBC_RowResult(void);
 
 protected:
+    typedef AutoPtr<char, ArrayDeleter<char>> TItemBuffer;
+
     virtual EDB_ResType     ResultType(void) const;
     virtual bool            Fetch(void);
     virtual int             CurrentItemNo(void) const;
@@ -767,7 +771,9 @@ protected:
     virtual bool            SkipItem(void);
 
     int xGetData(SQLSMALLINT target_type, SQLPOINTER buffer,
-        SQLINTEGER buffer_size);
+                 SQLINTEGER buffer_size, bool* more = NULL);
+    ssize_t x_GetVarLenData(SQLSMALLINT target_type, TItemBuffer& buffer,
+                            SQLINTEGER buffer_size);
     CDB_Object* x_LoadItem(I_Result::EGetItem policy, CDB_Object* item_buf);
     CDB_Object* x_MakeItem(void);
 
