@@ -1,7 +1,7 @@
 #ifndef MISC_JSONWRAPP___JSONWRAPP11__HPP
 #define MISC_JSONWRAPP___JSONWRAPP11__HPP
 
-/*  $Id: jsonwrapp11.hpp 675766 2023-12-01 14:57:17Z gouriano $
+/*  $Id: jsonwrapp11.hpp 692060 2025-01-10 16:42:42Z sadyrovr $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -2683,9 +2683,12 @@ inline std::string CJson_Schema::GetInvalidValueSchemaUri(void) const {
 inline std::string CJson_Schema::GetValidationError() const {
     std::string res;
     if (!IsValid()) {
+        rapidjson::StringBuffer sb1, sb2;
+        m_SchemaValidator.GetInvalidDocumentPointer().Stringify(sb1);
+        m_SchemaValidator.GetInvalidSchemaPointer().Stringify(sb2);
         res = "Invalid property \'"     + GetInvalidValueProperty()    +
-                "\' of value \'"        + GetInvalidValueDocumentUri() +
-                "\',  see schema at \'" + GetInvalidValueSchemaUri()   +
+                "\' of value \'"        + sb1.GetString()              +
+                "\',  see schema at \'" + sb2.GetString()              +
                 "\'";
     }
     return res;
