@@ -1,7 +1,7 @@
 #ifndef OBJECTS_MISC_SERIAL___GRID_RPC_CLIENT__HPP
 #define OBJECTS_MISC_SERIAL___GRID_RPC_CLIENT__HPP
 
-/* $Id: gridrpcclient.hpp 666819 2023-05-02 14:14:50Z evgeniev $
+/* $Id: gridrpcclient.hpp 687050 2024-09-03 18:34:15Z evgeniev $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -245,6 +245,10 @@ protected:
             return "GZipFile";
         case CCompressStream::eConcatenatedGZipFile:
             return "GZipFile";
+        case CCompressStream::eZipCloudflare:
+            return "Zip";
+        case CCompressStream::eGZipCloudflareFile:
+            return "GZipFile";
         case CCompressStream::eZstd:
             return "ZStandard";
         };
