@@ -1,7 +1,7 @@
 #ifndef OBJMGR_IMPL_SCOPE_IMPL__HPP
 #define OBJMGR_IMPL_SCOPE_IMPL__HPP
 
-/*  $Id: scope_impl.hpp 667314 2023-05-12 14:57:38Z vasilche $
+/*  $Id: scope_impl.hpp 679239 2024-02-20 19:05:51Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -141,6 +141,7 @@ public:
     typedef map<CConstRef<CObject>, CRef<CObject> >  TEditInfoMap;
     typedef map<CRef<CDataSource>, CRef<CDataSource_ScopeInfo> > TDSMap;
     typedef vector<CSeq_id_Handle>                   TIds;
+    typedef vector<TIds>                             TBulkIds;
     typedef vector<CSeq_entry_Handle>                TTSE_Handles;
 
     CObjectManager& GetObjectManager(void);
@@ -350,6 +351,9 @@ public:
     TCDD_Entries GetCDDAnnots(const TIds& idhs);
     TCDD_Entries GetCDDAnnots(const TBioseqHandles& bhs);
 
+    // Get a set of all ids of requested sequences
+    void GetBulkIds(TBulkIds& ret, const TIds& idhs, TGetFlags flags);
+
     // Get a set of accession/version pairs
     void GetAccVers(TIds& ret, const TIds& idhs, TGetFlags flags);
 
