@@ -1,4 +1,4 @@
-/* $Id: tabular.hpp 673788 2023-10-10 17:45:20Z ivanov $ 
+/* $Id: tabular.hpp 698230 2025-06-05 13:56:14Z ivanov $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -146,6 +146,7 @@ public:
     /// @param iteration Iteration number (for PSI-BLAST), use default
     /// parameter value when not applicable [in]
     /// @param align_set All alignments for this query [in]
+    /// @param is_csv Is a CSV format requested [in]
     void PrintHeader(const string& program, 
                              const objects::CBioseq& bioseq, 
                              const string& dbname, 
@@ -154,7 +155,8 @@ public:
                                 numeric_limits<unsigned int>::max(),
                              const objects::CSeq_align_set* align_set=0,
                              CConstRef<objects::CBioseq> subj_bioseq
-                                = CConstRef<objects::CBioseq>()); 
+                                = CConstRef<objects::CBioseq>(),
+                             bool is_csv = false); 
 
      /// Prints number of queries processed.
      /// @param num_queries number of queries processed [in]
@@ -188,6 +190,13 @@ public:
     /// @param query range [in]
     void SetQueryRange(TSeqRange & q_range) { m_QueryRange = q_range;}
 
+    /// Print the names of all supported fields
+    /// @param is_csv Is a csv format requested [in]
+    void PrintFieldNames(bool is_csv = false);
+
+    /// Print field specifiers of all requested fields
+    void PrintFieldSpecs(void);
+    
 protected:
     bool x_IsFieldRequested(ETabularField field);
     /// Add a field to the list of fields to show, if it is not yet present in
@@ -207,8 +216,6 @@ protected:
     /// Set the tabular fields delimiter.
     /// @param delim Which delimiter to use
     void x_SetFieldDelimiter(EFieldDelimiter delim, string customDelim = "");
-    /// Print the names of all supported fields
-    void x_PrintFieldNames(void);
     /// Print the value of a given field
     /// @param field Which field to show? [in]
     void x_PrintField(ETabularField field);
@@ -369,6 +376,11 @@ private:
 
     TSeqRange m_QueryRange;
     string m_CustomDelim;
+
+    /// Convenience method to determine if this object is dealing with CSV output formats
+    inline bool x_IsCsv() const {
+        return (m_CustomDelim == "," || m_FieldDelimiter == ",");
+    }
 };
 
 
