@@ -1,4 +1,4 @@
-/*  $Id: vectorscreen.hpp 657548 2022-10-12 15:36:06Z boukn $
+/*  $Id: vectorscreen.hpp 686060 2024-08-06 15:09:32Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -182,7 +182,7 @@ public:
     void VecscreenPrint(CNcbiOstream& out);
 
     ///Returns a string concerning the strength of the match for a given enum value
-    static const string& GetStrengthString(MatchType match_type);
+    static string GetStrengthString(MatchType match_type);
  
 protected:
     
