@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBATLAS_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBATLAS_HPP
 
-/*  $Id: seqdbatlas.hpp 631567 2021-05-19 13:53:46Z ivanov $
+/*  $Id: seqdbatlas.hpp 696150 2025-04-08 18:25:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,6 @@
 #include <corelib/ncbiapp_api.hpp>
 
 #include <objtools/blast/seqdb_reader/impl/seqdbgeneral.hpp>
-
 #include <vector>
 #include <map>
 #include <set>
@@ -672,11 +671,10 @@ public:
     ///
     /// @param filename
     ///   file to memory map    
-    CSeqDBFileMemMap(class CSeqDBAtlas & atlas, const string filename)
+    CSeqDBFileMemMap(class CSeqDBAtlas & atlas, const string & filename)
         : m_Atlas(atlas),
           m_DataPtr (NULL),
-          m_MappedFile( NULL),
-          m_Mapped(false)
+          m_MappedFile( NULL)
     {
         Init(filename);
     }
@@ -684,8 +682,7 @@ public:
     CSeqDBFileMemMap(class CSeqDBAtlas & atlas)
         : m_Atlas(atlas),
           m_DataPtr (NULL),
-          m_MappedFile( NULL),
-          m_Mapped(false)
+          m_MappedFile( NULL)
     {
         
     }
@@ -700,47 +697,44 @@ public:
     ///
     /// @param filename
     ///   file to memory map    
-    void Init(const string filename) {                
-    	 CSeqDBLockHold locked(m_Atlas);
-    	 m_Atlas.Lock(locked);
+    void Init(const string & filename) {
+    	CFastMutexGuard mtx_gurad(m_Mtx);
         if(!m_MappedFile || m_Filename != filename)
         {
-        	Clear();
+            if(m_MappedFile) {
+            	m_MappedFile = m_Atlas.ReturnMemoryFile(m_Filename);
+            	m_DataPtr = NULL;
+            }
             m_Filename = filename;
-            Init();
+            try {
+                m_MappedFile = m_Atlas.GetMemoryFile(m_Filename);
+            }
+            catch (const CFileException& e) {
+                if (e.GetErrCode() == CFileException::eMemoryMap &&
+                    NStr::Find(e.GetMsg(), "Too many open files") != NPOS) {
+                    NCBI_THROW(CSeqDBException, eTooManyOpenFiles, e.GetMsg());
+                }
+                else {
+                    NCBI_THROW(CSeqDBException, eMemoryMappingFailure, e.GetMsg());
+                }
+            }
+
+            m_DataPtr = (char *)(m_MappedFile->GetPtr());
         }
-        m_Atlas.Unlock(locked);
     }
 
-    //m_Filename is set
-    void Init(void)
-    {
-        try {
-            m_MappedFile = m_Atlas.GetMemoryFile(m_Filename);
-            m_Mapped = true;
-        }
-        catch (const std::exception&) {
-            NCBI_THROW(CSeqDBException,
-                eFileErr,
-                "Cannot memory map " + m_Filename + ". Number of files opened: " + NStr::IntToString(m_Atlas.GetOpenedFilseCount()));
-        }
-
-        m_DataPtr = (char *)(m_MappedFile->GetPtr());
-    }
-
-    
     /// Clears the memory mapobject.
     ///    
     void Clear()
     {
-        
-        if(m_MappedFile && m_Mapped ) {
+    	CFastMutexGuard mtx_gurad(m_Mtx);
+        if(m_MappedFile ) {
         	m_MappedFile = m_Atlas.ReturnMemoryFile(m_Filename);
-            m_Mapped = false;
+        	m_DataPtr = NULL;
         }        
     }
 
-    bool IsMapped(){return m_Mapped;}
+    bool IsMapped(){return (m_MappedFile != NULL);}
 
     /// Get a pointer to the specified offset.
     ///
@@ -753,9 +747,7 @@ public:
     ///   A pointer to the data at the requested location.       
     const char *GetFileDataPtr(const string   & fname,TIndx offset)                    
     {
-        if(!m_MappedFile || m_Filename != fname) {
-            Init(fname);                        
-        }
+        Init(fname);
 
         return(const char *)(m_DataPtr + offset);    
     }
@@ -767,6 +759,8 @@ public:
 
 private:
     CSeqDBAtlas & m_Atlas;    
+
+    mutable CFastMutex m_Mtx;
      /// Points to the beginning of the data area.
     const char * m_DataPtr;
 
@@ -774,7 +768,6 @@ private:
 
     CMemoryFile *m_MappedFile;
 
-    bool m_Mapped;
 };
 
 
