@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 
-/*  $Id: seqdbvol.hpp 679614 2024-02-28 18:46:23Z ivanov $
+/*  $Id: seqdbvol.hpp 696151 2025-04-08 18:26:06Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1363,6 +1363,9 @@ private:
     /// The name of this volume.
     string m_VolName;
 
+    //Assume Gis exists, set to false when file cannot be found during lookup
+    mutable bool m_HasGis;
+
     /// Metadata plus offsets into the sequence, header, and ambiguity data.
     CRef<CSeqDBIdxFile> m_Idx;
 
