@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 
-/*  $Id: seqdb.hpp 654652 2022-08-18 16:44:30Z fongah2 $
+/*  $Id: seqdb.hpp 689255 2024-10-23 13:43:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -250,7 +250,7 @@ public:
         }
 
         void x_reallocate_if_necessary() {
-            static size_t kResizeFactor = 2;
+            const size_t kResizeFactor = 2;
             if (_size + 1 > _capacity) {
                 reserve((_capacity + 1) * kResizeFactor -1);
             }
@@ -313,7 +313,7 @@ public:
     };
     /// String containing the error message in exceptions thrown when a given
     /// OID cannot be found
-    static const string kOidNotFound;
+    static const char* kOidNotFound;
 
     /// Short Constructor
     ///
