@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 
-/*  $Id: seqdbcommon.hpp 683533 2024-05-30 12:54:51Z ivanov $
+/*  $Id: seqdbcommon.hpp 696144 2025-04-08 18:24:55Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -90,22 +90,10 @@ public:
         eTaxidErr,
 
 		/// Too many open files
-		eOpenFileErr
-    };
+		eTooManyOpenFiles,
 
-    /// Get a message describing the situation leading to the throw.
-    virtual const char* GetErrCodeString() const override
-    {
-        switch ( GetErrCode() ) {
-        case eArgErr:  return "eArgErr";
-        case eFileErr: return "eFileErr";
-        case eMemErr: return "eMemErr";
-        case eVersionErr: return "eVersionErr";
-        case eTaxidErr: return "eTaxidErr";
-        case eOpenFileErr: return "eOpenFileErr";
-        default:       return CException::GetErrCodeString();
-        }
-    }
+        eMemoryMappingFailure
+    };
 
     /// Include standard NCBI exception behavior.
     NCBI_EXCEPTION_DEFAULT(CSeqDBException,CException);
