@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_WRITERS_WRITEDB__WRITEDB_FILES_HPP
 #define OBJTOOLS_WRITERS_WRITEDB__WRITEDB_FILES_HPP
 
-/*  $Id: writedb_files.hpp 553715 2017-12-20 18:37:47Z vakatov $
+/*  $Id: writedb_files.hpp 649164 2022-04-28 11:08:55Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -95,6 +95,8 @@ public:
     /// @return File offset after write.
     unsigned int Write(const CTempString & data);
 
+    unsigned int Write(const char *  data, int length);
+
     /// Write an Int4 (in bigendian order) to the file.
     /// @param data String to write.
     /// @return File offset after write.
@@ -130,6 +132,8 @@ public:
     /// Rename this file, disincluding the volume index.
     virtual void RenameSingle();
 
+    virtual void RenameFileIndex(unsigned int num_digits);
+
     /// Construct the short name for a volume.
     ///
     /// Volume names consist of the database base name, ".", and the
