@@ -2,7 +2,7 @@
 #define BLOB_ID__HPP_INCLUDED
 /* */
 
-/*  $Id: blob_id.hpp 631263 2021-05-14 15:54:29Z vasilche $
+/*  $Id: blob_id.hpp 681498 2024-04-09 17:27:24Z vasilche $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -115,10 +115,24 @@ public:
             m_SatKey = v;
         }
 
+    const string& GetTSEName() const
+    {
+        return m_TSEName;
+    }
+    bool HasTSEName() const
+    {
+        return !m_TSEName.empty();
+    }
+    void SetTSEName(const string& name)
+    {
+        m_TSEName = name;
+    }
+
 protected:
     TSat m_Sat;
     TSubSat m_SubSat;
     TSatKey m_SatKey;
+    string m_TSEName;
 };
 
 
