@@ -1,7 +1,7 @@
 #ifndef GBLOADER_REQUEST_RESULT__HPP_INCLUDED
 #define GBLOADER_REQUEST_RESULT__HPP_INCLUDED
 
-/*  $Id: request_result.hpp 578138 2019-01-14 15:36:05Z vasilche $
+/*  $Id: request_result.hpp 665099 2023-03-28 13:31:12Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -277,6 +277,8 @@ public:
             return *begin();
         }
 
+    CFixedBlob_ids Filter(const SAnnotSelector* sel) const;
+
 private:
     typedef CObjectFor<TList> TObject;
 
@@ -305,7 +307,7 @@ public:
     typedef CDataLoader::SGiFound TSequenceGi;
     typedef GBL::CInfoCache<CSeq_id_Handle, TSequenceGi> TCacheGi;
     typedef GBL::CInfoCache<CSeq_id_Handle, string> TCacheLabel;
-    typedef int TTaxId;
+    typedef ncbi::TTaxId TTaxId;
     typedef GBL::CInfoCache<CSeq_id_Handle, TTaxId> TCacheTaxId;
     typedef pair<CSeq_id_Handle, string> TKeyBlobIds;
     typedef GBL::CInfoCache<TKeyBlobIds, CFixedBlob_ids> TCacheBlobIds;
@@ -556,7 +558,7 @@ public:
 
     static bool IsFound(const TData& data)
         {
-            return data != -1;
+            return data != INVALID_TAX_ID;
         }
     bool IsFound(void) const
         {
@@ -785,6 +787,11 @@ public:
         {
             return SetLoaded(data, GetExpType(data));
         }
+    bool SetLoadedBlob_ids(const SAnnotSelector* sel,
+                           const TData& blob_ids,
+                           TExpirationTime expiration_time);
+    bool SetLoadedBlob_ids(const SAnnotSelector* sel,
+                           const TData& blob_ids);
     bool SetNoBlob_ids(const CFixedBlob_ids::TState& state,
                        TExpirationTime expiration_time)
         {
@@ -973,6 +980,8 @@ public:
 
     void SetSeq_entry(CSeq_entry& entry, CTSE_SetObjectInfo* set_info = 0);
 
+    void AllowIncompleteLoading();
+
 protected:
 
     void x_SelectChunk(TChunkId chunk_id);
@@ -983,6 +992,8 @@ protected:
 private:
     CTSE_LoadLock   m_TSE_LoadLock;
     CRef<CTSE_Chunk_Info> m_Chunk;
+    AutoPtr<CInitGuard>   m_ChunkLoadLock;
+    bool m_AllowIncompleteLoading;
 };
 
 
