@@ -1,6 +1,6 @@
 #ifndef READER__HPP_INCLUDED
 #define READER__HPP_INCLUDED
-/*  $Id: reader.hpp 675198 2023-11-16 13:48:10Z ivanov $
+/*  $Id: reader.hpp 683019 2024-05-15 17:09:16Z grichenk $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -34,7 +34,7 @@
 #include <corelib/ncbitime.hpp>
 #include <util/cache/icache.hpp>
 #include <objtools/data_loaders/genbank/impl/request_result.hpp>
-#include <objtools/data_loaders/genbank/impl/incr_time.hpp>
+#include <util/incr_time.hpp>
 #include <list>
 
 BEGIN_NCBI_SCOPE
@@ -122,6 +122,7 @@ public:
     // bulk requests
     typedef vector<CSeq_id_Handle> TIds;
     typedef vector<bool> TLoaded;
+    typedef vector<TIds> TBulkIds;
     typedef vector<TGi> TGis;
     typedef vector<string> TLabels;
     typedef vector<TTaxId> TTaxIds;
@@ -130,6 +131,10 @@ public:
     typedef vector<bool> TKnown;
     typedef vector<TSeqPos> TLengths;
     typedef vector<CSeq_inst::EMol> TTypes;
+    typedef vector<CBlob_id> TPlainBlobIds;
+    typedef vector<pair<TBlobId, TChunkIds>> TBlobChunkIds;
+    virtual bool LoadBulkIds(CReaderRequestResult& result,
+                             const TIds& ids, TLoaded& loaded, TBulkIds& ret);
     virtual bool LoadAccVers(CReaderRequestResult& result,
                              const TIds& ids, TLoaded& loaded, TIds& ret);
     virtual bool LoadGis(CReaderRequestResult& result,
@@ -170,8 +175,12 @@ public:
     virtual bool LoadChunks(CReaderRequestResult& result,
                             const TBlobId& blob_id,
                             const TChunkIds& chunk_ids);
+    virtual bool LoadChunks(CReaderRequestResult& result,
+                            const TBlobChunkIds& chunk_ids);
     virtual bool LoadBlobSet(CReaderRequestResult& result,
                              const TSeqIds& seq_ids);
+    virtual bool LoadBlobs(CReaderRequestResult& result,
+                           const TBlobIds& blob_infos);
 
     void SetAndSaveSeq_idSeq_ids(CReaderRequestResult& result,
                                  const CSeq_id_Handle& seq_id,
