@@ -1,4 +1,4 @@
-/*  $Id: feature_propagate.hpp 632623 2021-06-03 17:38:11Z ivanov $
+/*  $Id: feature_propagate.hpp 638575 2021-10-01 16:40:21Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,11 +53,11 @@ class NCBI_XOBJEDIT_EXPORT CFeaturePropagator
 public:
     NCBI_DEPRECATED CFeaturePropagator(CBioseq_Handle src, CBioseq_Handle target, const CSeq_align& align,
         bool stop_at_stop = true, bool cleanup_partials = true, bool merge_abutting = true,
-        CMessageListener_Basic* pMessageListener = 0,
+        CMessageListener_Basic* pMessageListener = nullptr,
         CObject_id::TId* feat_id = nullptr);
     CFeaturePropagator(CBioseq_Handle src, CBioseq_Handle target, const CSeq_align& align,
                        bool stop_at_stop = true, bool cleanup_partials = true, bool merge_abutting = true, bool expand_over_gaps = true,
-                       CMessageListener_Basic* pMessageListener = 0,
+                       CMessageListener_Basic* pMessageListener = nullptr,
                        CObject_id::TId* feat_id = nullptr);
     ~CFeaturePropagator() {}
 
