@@ -1,4 +1,4 @@
-/*  $Id: field_handler.hpp 638615 2021-10-02 09:47:34Z stakhovv $
+/*  $Id: field_handler.hpp 689413 2024-10-28 14:18:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -87,10 +87,10 @@ NCBI_XOBJEDIT_EXPORT bool DoesObjectMatchFieldConstraint (const CObject& object,
 NCBI_XOBJEDIT_EXPORT bool DoesApplyObjectMatchFieldConstraint (const CApplyObject& object, const string& field_name, CRef<CStringConstraint> string_constraint);
 NCBI_XOBJEDIT_EXPORT vector<CConstRef<CSeq_feat> > GetRelatedFeatures(const CSeq_feat& obj_feat, CSeqFeatData::ESubtype constraint_type, CRef<CScope> scope);
 
-static const string kFieldTypeSeqId = "SeqId";
-const string kDefinitionLineLabel = "Definition Line";
-const string kCommentDescriptorLabel = "Comment Descriptor";
-const string kGenbankBlockKeyword = "Keyword";
+static const char kFieldTypeSeqId[]         = "SeqId";
+static const char kDefinitionLineLabel[]    = "Definition Line";
+static const char kCommentDescriptorLabel[] = "Comment Descriptor";
+static const char kGenbankBlockKeyword[]    = "Keyword";
 
 END_SCOPE(edit)
 END_SCOPE(objects)
