@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___CONTEXT__HPP
 #define OBJTOOLS_FORMAT___CONTEXT__HPP
 
-/*  $Id: context.hpp 602293 2020-02-20 18:24:39Z kans $
+/*  $Id: context.hpp 665420 2023-04-04 13:16:28Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -79,18 +79,18 @@ public:
 
     // constructor
     CBioseqContext(
-        const CBioseq_Handle& seq, 
+        const CBioseq_Handle& seq,
         CFlatFileContext& ffctx,
-        CMasterContext* mctx = 0,
-        CTopLevelSeqEntryContext *tlsec = 0);
+        CMasterContext* mctx = nullptr,
+        CTopLevelSeqEntryContext* tlsec = nullptr);
 
     CBioseqContext(
-        const CBioseq_Handle& prev_seq, 
-        const CBioseq_Handle& seq, 
+        const CBioseq_Handle& prev_seq,
+        const CBioseq_Handle& seq,
         const CBioseq_Handle& next_seq,
         CFlatFileContext& ffctx,
-        CMasterContext* mctx = 0,
-        CTopLevelSeqEntryContext *tlsec = 0);
+        CMasterContext* mctx = nullptr,
+        CTopLevelSeqEntryContext* tlsec = nullptr);
     // destructor
     ~CBioseqContext(void);
 
@@ -109,7 +109,7 @@ public:
     CSeq_id_Handle GetPreferredSynonym(const CSeq_id& id) const;
     const string& GetAccession(void) const { return m_Accession; }
     TGi  GetGI(void) const { return m_Gi; }
-    
+
     // molecular type (nucleotide / protein)
     bool IsProt(void) const { return m_IsProt;  }
     bool IsNuc (void) const { return !m_IsProt; }
@@ -124,7 +124,7 @@ public:
     // segmented bioseq
     bool IsSegmented(void) const { return m_Repr == CSeq_inst::eRepr_seg; }
     bool HasParts(void) const { return m_HasParts; }
-    
+
     // part of a segmented bioseq
     bool IsPart(void) const { return m_IsPart; }
     SIZE_TYPE GetPartNumber   (void) const { return m_PartNumber; }
@@ -149,7 +149,7 @@ public:
     const string& GetWGSMasterAccn(void) const { return m_WGSMasterAccn; }
     const string& GetWGSMasterName(void) const { return m_WGSMasterName; }
 
-    // Transcriptome Shotgun Assembly 
+    // Transcriptome Shotgun Assembly
     bool IsTSA      (void) const { return m_IsTSA;       }
     bool IsTSAMaster(void) const { return m_IsTSAMaster; }
     const string& GetTSAMasterAccn(void) const { return m_TSAMasterAccn; }
@@ -189,7 +189,7 @@ public:
     bool IsGbGenomeProject(void) const { return m_IsGbGenomeProject; } // AE
     bool IsNcbiCONDiv     (void) const { return m_IsNcbiCONDiv; }      // CH
     bool IsNcbiGenomes    (void) const { return m_IsNcbiGenomes; } // db NCBI_GENOMES
-    
+
     // RefSeq ID queries
     bool IsRefSeq(void) const { return m_IsRefSeq; }
     bool IsRSCompleteGenomic  (void) const;  // NC_
@@ -205,7 +205,7 @@ public:
     bool IsRSWGSNuc           (void) const;  // NZ_
     bool IsRSWGSProt          (void) const;  // ZP_
     bool IsRSUniqueProt       (void) const;  // WP_
-    
+
     bool IsEncode             (void) const;  // provided by the ENCODE project
     const CUser_object& GetEncode(void) const;
 
@@ -224,11 +224,20 @@ public:
         fUnverified_SequenceOrAnnotation  = 1 << 1,
         fUnverified_Misassembled          = 1 << 2,
         fUnverified_Contaminant           = 1 << 3
-        // if you add more here, make sure 
+        // if you add more here, make sure
         // to update CFlatGatherer::x_UnverifiedComment
     };
     typedef Int8 TUnverified;
     TUnverified GetUnverifiedType(void) const;
+    enum FUnreviewed {
+        fUnreviewed_None = 0, // for convenience
+
+        fUnreviewed_Unannotated = 1 << 0
+        // if you add more here, make sure
+        // to update CFlatGatherer::x_UnreviewedComment
+    };
+    typedef Int8 TUnreviewed;
+    TUnreviewed GetUnreviewedType(void) const;
     bool ShowAnnotCommentAsCOMMENT() const;
 
     bool IsHup(void) const { return m_IsHup; }  // !!! should move to global?
@@ -266,7 +275,7 @@ public:
     const string & GetAuthorizedAccess(void) const { return m_AuthorizedAccess; }
 
     /// Empty or NULL if no points or if this bioseq isn't an optical map.
-    const CPacked_seqpnt * GetOpticalMapPoints(void) const { 
+    const CPacked_seqpnt * GetOpticalMapPoints(void) const {
         return m_pOpticalMapPoints; }
 
    vector<string>* GetRefCache (void) const { return m_RefCache;   }
@@ -283,7 +292,7 @@ private:
     bool x_IsInSGS(void) const;
     bool x_IsInGPS(void) const;
     bool x_IsInNucProt(void) const;
-    void x_SetLocation(const CSeq_loc* user_loc = 0);
+    void x_SetLocation(const CSeq_loc* user_loc = nullptr);
     void x_SetMapper(const CSeq_loc& loc);
     void x_SetHasMultiIntervalGenes(void) const;
     void x_SetDataFromUserObjects(void);
@@ -319,12 +328,11 @@ private:
     const CPacked_seqpnt* m_pOpticalMapPoints;
     // used to destroy m_pOpticalMapPoints if it was manually
     // created.
-    auto_ptr<CPacked_seqpnt> m_pOpticalMapPointsDestroyer;
-    
+    unique_ptr<CPacked_seqpnt> m_pOpticalMapPointsDestroyer;
 
     CSeq_inst::TRepr      m_Repr;
     CSeq_inst::TMol       m_Mol;
-    CConstRef<CMolInfo>   m_Molinfo; 
+    CConstRef<CMolInfo>   m_Molinfo;
 
     // segmented bioseq
     bool        m_HasParts;
@@ -369,11 +377,12 @@ private:
     mutable bool m_IsCrossKingdom;
     mutable bool m_UsePDBCompoundForComment;
     TUnverified m_fUnverified;
+    TUnreviewed m_fUnreviewed;
     mutable bool m_ShowAnnotCommentAsCOMMENT;
     mutable bool m_ShowAnnotCommentAsCOMMENT_checked;
 
     CConstRef<CUser_object> m_Encode;
-    
+
     TReferences             m_References;
     CConstRef<CSeq_loc>     m_Location;
     CRef<CSeq_loc_Mapper>   m_Mapper;
@@ -457,11 +466,11 @@ public:
     const feature::CFeatTree* GetFeatTree(void) const { return m_FeatTree; }
     feature::CFeatTree* GetFeatTree(void) { return m_FeatTree; }
     void SetFeatTree(feature::CFeatTree* tree) { m_FeatTree.Reset(tree); }
-    
+
     bool UsingSeqEntryIndex(void) const { return (m_Idx != 0); }
     const CRef<CSeqEntryIndex> GetSeqEntryIndex(void) const { return m_Idx; }
     void SetSeqEntryIndex(CRef<CSeqEntryIndex> idx) { m_Idx = idx; }
-    void ResetSeqEntryIndex(void) { m_Idx.Reset(NULL); }
+    void ResetSeqEntryIndex(void) { m_Idx.Reset(); }
 
     bool GetSGS(void) const { return m_SmallGenomeSet; }
     void SetSGS(const bool sgs) { m_SmallGenomeSet = sgs; }
@@ -476,7 +485,7 @@ private:
     CSeq_entry_Handle           m_Entry;
     TSections                   m_Sections;
     CConstRef<CSubmit_block>    m_Submit;
-    auto_ptr<SAnnotSelector>    m_Selector;
+    unique_ptr<SAnnotSelector>  m_Selector;
     CConstRef<CSeq_loc>         m_Loc;
     CRef<feature::CFeatTree>    m_FeatTree;
     CRef<CSeqEntryIndex>        m_Idx;
@@ -495,7 +504,8 @@ private:
 class CTopLevelSeqEntryContext  : public CObject
 {
 public:
-    CTopLevelSeqEntryContext( const CSeq_entry_Handle &entry_handle );
+    CTopLevelSeqEntryContext( const CSeq_entry_Handle &entry_handle, bool useIndexedFasterSets  = false );
+    void x_InitSeqs (const CSeq_entry& sep);
 
     bool GetCanSourcePubsBeFused(void) const { return m_CanSourcePubsBeFused; }
     bool GetHasSmallGenomeSet(void) const { return m_HasSmallGenomeSet; }
@@ -656,6 +666,12 @@ CBioseqContext::TUnverified CBioseqContext::GetUnverifiedType(void) const
     return m_fUnverified;
 }
 
+inline
+CBioseqContext::TUnreviewed CBioseqContext::GetUnreviewedType(void) const
+{
+    return m_fUnreviewed;
+}
+
 inline
 bool CBioseqContext::UsingSeqEntryIndex(void) const
 {
@@ -752,7 +768,7 @@ const SAnnotSelector* CFlatFileContext::GetAnnotSelector(void) const
 inline
 SAnnotSelector& CFlatFileContext::SetAnnotSelector(void)
 {
-    if ( m_Selector.get() == 0 ) {
+    if (! m_Selector) {
         m_Selector.reset(new SAnnotSelector(CSeq_annot::TData::e_Ftable));
     }
 
