@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 #define OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 
-/*  $Id: flat_file_config.hpp 676886 2023-12-27 15:10:53Z sadyrovr $
+/*  $Id: flat_file_config.hpp 693505 2025-02-13 14:43:47Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -84,14 +84,14 @@ public:
 
     virtual void FormatProteinId(string& str, const CSeq_id& seq_id, const string& prot_id) const = 0;
     virtual void FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const = 0;
-    virtual void FormatNucSearch(CNcbiOstream& os, const string& id) const = 0;
+    virtual void FormatNucSearch(ostream& os, const string& id) const = 0;
     virtual void FormatNucId(string& str, const CSeq_id& seq_id, TIntId gi, const string& acc_id) const = 0;
     virtual void FormatTaxid(string& str, const TTaxId taxid, const string& taxname) const = 0;
     virtual void FormatLocation(string& str, const CSeq_loc& loc, TIntId gi, const string& visible_text) const = 0;
     virtual void FormatModelEvidence(string& str, const SModelEvidance& me) const = 0;
     virtual void FormatTranscript(string& str, const string& name) const = 0;
-    virtual void FormatGeneralId(CNcbiOstream& os, const string& id) const = 0;
-    virtual void FormatGapLink(CNcbiOstream& os, TSeqPos gap_size, const string& id, bool is_prot) const = 0;
+    virtual void FormatGeneralId(ostream& os, const string& id) const = 0;
+    virtual void FormatGapLink(ostream& os, TSeqPos gap_size, const string& id, bool is_prot) const = 0;
     virtual void FormatUniProtId(string& str, const string& prot_id) const = 0;
 };
 
@@ -102,14 +102,14 @@ public:
 
     void FormatProteinId(string& str, const CSeq_id& seq_id, const string& prot_id) const override;
     void FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const override;
-    void FormatNucSearch(CNcbiOstream& os, const string& id) const override;
+    void FormatNucSearch(ostream& os, const string& id) const override;
     void FormatNucId(string& str, const CSeq_id& seq_id, TIntId gi, const string& acc_id) const override;
     void FormatTaxid(string& str, const TTaxId taxid, const string& taxname) const override;
     void FormatLocation(string& str, const CSeq_loc& loc, TIntId gi, const string& visible_text) const override;
     void FormatModelEvidence(string& str, const SModelEvidance& me) const override;
     void FormatTranscript(string& str, const string& name) const override;
-    void FormatGeneralId(CNcbiOstream& os, const string& id) const override;
-    void FormatGapLink(CNcbiOstream& os, TSeqPos gap_size, const string& id, bool is_prot) const override;
+    void FormatGeneralId(ostream& os, const string& id) const override;
+    void FormatGapLink(ostream& os, TSeqPos gap_size, const string& id, bool is_prot) const override;
     void FormatUniProtId(string& str, const string& prot_id) const override;
 };
 
@@ -204,7 +204,8 @@ public:
         fDoNotUseAutoDef       = 1 << 18,
         fOldTpaDisplay         = 1 << 19,
         fDisableDefaultIndex   = 1 << 20,
-        fGeoLocNameCountry     = 1 << 21
+        fGeoLocNameCountry     = 1 << 21,
+        fLeavePrefixSuffix     = 1 << 22
     };
 
     enum EView {
@@ -635,7 +636,8 @@ public:
     bool DoNotUseAutoDef       (void) const;
     bool OldTpaDisplay         (void) const;
     bool DisableDefaultIndex   (void) const;
-    bool GeoLocNameCountry  (void) const;
+    bool GeoLocNameCountry     (void) const;
+    bool LeavePrefixSuffix     (void) const;
 
     // setters
     void SetCustom(const TCustom& custom) { m_Custom = custom; }
@@ -658,7 +660,8 @@ public:
     CFlatFileConfig& SetDoNotUseAutoDef       (bool val = true);
     CFlatFileConfig& SetOldTpaDisplay         (bool val = true);
     CFlatFileConfig& SetDisableDefaultIndex   (bool val = true);
-    CFlatFileConfig& SetGeoLocNameCountry  (bool val = true);
+    CFlatFileConfig& SetGeoLocNameCountry     (bool val = true);
+    CFlatFileConfig& SetLeavePrefixSuffix     (bool val = true);
 
     // adjust mode dependant flags for RefSeq
     void SetRefSeqConventions(void);
@@ -867,6 +870,7 @@ CUSTOM_ARG_IMP(DoNotUseAutoDef)
 CUSTOM_ARG_IMP(OldTpaDisplay)
 CUSTOM_ARG_IMP(DisableDefaultIndex)
 CUSTOM_ARG_IMP(GeoLocNameCountry)
+CUSTOM_ARG_IMP(LeavePrefixSuffix)
 
 #undef FLAG_ARG_IMP
 #undef FLAG_ARG_GET
