@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FLAT_FILE_GENERATOR__HPP
 #define OBJTOOLS_FORMAT___FLAT_FILE_GENERATOR__HPP
 
-/*  $Id: flat_file_generator.hpp 671848 2023-08-24 20:01:29Z ivanov $
+/*  $Id: flat_file_generator.hpp 696066 2025-04-07 18:34:22Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -39,7 +39,6 @@
 #include <objtools/format/item_ostream.hpp>
 #include <objtools/format/context.hpp>
 
-
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
@@ -53,6 +52,7 @@ class CSeq_loc;
 class CSeq_entry_Handle;
 class CBioseq_Handle;
 class CSeq_id;
+class CTrna_ext;
 
 
 struct NCBI_FORMAT_EXPORT multiout
@@ -115,6 +115,9 @@ public:
     static string GetSeqFeatText(const CMappedFeat& feat, CScope& scope,
         const CFlatFileConfig& cfg, CRef<feature::CFeatTree> ftree = null);
 
+    static string GetFTableAnticodonText(const CTrna_ext& trna_ext, CBioseqContext& ctx);
+    static string GetFTableAnticodonText(const CSeq_feat& feat, CScope& scope);
+
     void x_GetLocation(const CSeq_entry_Handle& entry,
          TSeqPos from, TSeqPos to, ENa_strand strand, CSeq_loc& loc);
     CBioseq_Handle x_DeduceTarget(const CSeq_entry_Handle& entry);
@@ -125,6 +128,8 @@ public:
 
     void SetConfig(const CFlatFileConfig& cfg);
 
+    static bool HasInference(const CSeq_entry_Handle& topseh);
+
 protected:
     CRef<CFlatFileContext>    m_Ctx;
     bool                      m_Failed;
@@ -141,8 +146,8 @@ protected:
               m_pCanceledCallback(pCanceledCallback)
         { }
 
-        virtual void SetFormatter(IFormatter* formatter);
-        virtual void AddItem (CConstRef<IFlatItem> item);
+        void SetFormatter(IFormatter* formatter) override;
+        void AddItem(CConstRef<IFlatItem> item) override;
 
     private:
         CRef<CFlatItemOStream> m_pUnderlying;
