@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 #define OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 
-/*  $Id: feature_item.hpp 613862 2020-08-13 17:00:53Z kans $
+/*  $Id: feature_item.hpp 665891 2023-04-17 14:20:00Z gotvyans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -54,22 +54,22 @@ BEGIN_SCOPE(objects)
 class CTrna_ext;
 
 //  ============================================================================
-class NCBI_FORMAT_EXPORT CFeatHeaderItem : 
+class NCBI_FORMAT_EXPORT CFeatHeaderItem :
     public CFlatItem
 //  ============================================================================
 {
 public:
     CFeatHeaderItem(CBioseqContext& ctx);
     void Format(IFormatter& formatter,
-        IFlatTextOStream& text_os) const {
+        IFlatTextOStream& text_os) const override {
         formatter.FormatFeatHeader(*this, text_os);
     }
 
     const CSeq_id& GetId(void) const { return *m_Id; }  // for FTable format
 
-    virtual EItem GetItemType(void) const;
+    EItem GetItemType() const override;
 private:
-    void x_GatherInfo(CBioseqContext& ctx);
+    void x_GatherInfo(CBioseqContext& ctx) override;
 
     // data
     CConstRef<CSeq_id>  m_Id;  // for FTable format
@@ -77,7 +77,7 @@ private:
 
 
 //  ============================================================================
-class NCBI_FORMAT_EXPORT CFlatFeature : 
+class NCBI_FORMAT_EXPORT CFlatFeature :
     public CObject
 //  ============================================================================
 {
@@ -106,7 +106,7 @@ private:
 
 
 //  ============================================================================
-class NCBI_FORMAT_EXPORT CFeatureItemBase: 
+class NCBI_FORMAT_EXPORT CFeatureItemBase:
     public CFlatItem
 //  ============================================================================
 {
@@ -116,13 +116,13 @@ public:
         formatter.FormatFeature(*this, text_os);
     }
     bool operator<(const CFeatureItemBase& f2) const {
-        //return m_Feat.Compare(*f2.m_Feat, GetLoc(), f2.GetLoc()) < 0; 
+        //return m_Feat.Compare(*f2.m_Feat, GetLoc(), f2.GetLoc()) < 0;
         return m_Feat < f2.m_Feat;
     }
     const CMappedFeat& GetFeat(void)  const { return m_Feat; }
     const CSeq_loc&    GetLoc(void)   const { return *m_Loc; }
 
-    virtual string GetKey(void) const { 
+    virtual string GetKey(void) const {
         return m_Feat.GetData().GetKey(CSeqFeatData::eVocabulary_genbank);
     }
 
@@ -146,7 +146,7 @@ protected:
 
 
 //  ============================================================================
-class NCBI_FORMAT_EXPORT CFeatureItem: 
+class NCBI_FORMAT_EXPORT CFeatureItem:
     public CFeatureItemBase
 //  ============================================================================
 {
@@ -171,9 +171,9 @@ public:
     virtual ~CFeatureItem() {};
 
     // fetaure key (name)
-    string GetKey(void) const;
+    string GetKey(void) const override;
 
-    virtual EItem GetItemType(void) const;
+    EItem GetItemType() const override;
 
     // mapping
     bool IsMapped           (void) const { return m_Mapped != eMapped_not_mapped;   }
@@ -201,7 +201,7 @@ protected:
     void x_AddQualsRegulatoryClass( CBioseqContext& ctx, CSeqFeatData::ESubtype subtype );
     void x_AddQualsRecombinationClass( CBioseqContext& ctx, CSeqFeatData::ESubtype subtype );
 
-    void x_AddQualPseudo( CBioseqContext&, CSeqFeatData::E_Choice, 
+    void x_AddQualPseudo( CBioseqContext&, CSeqFeatData::E_Choice,
         CSeqFeatData::ESubtype, bool );
     void x_AddQualExceptions( CBioseqContext& );
     void x_AddQualNote( CConstRef<CSeq_feat> );
@@ -216,7 +216,7 @@ protected:
     void x_AddQualTranslationExceptionIdx( const CCdregion&, CBioseqContext&, string& tr_ex );
     void x_AddQualProteinConflict( const CCdregion&, CBioseqContext& );
     void x_AddQualCodedBy( CBioseqContext& );
-    void x_AddQualProteinId( CBioseqContext&, const CBioseq_Handle&, CConstRef<CSeq_id> );  
+    void x_AddQualProteinId( CBioseqContext&, const CBioseq_Handle&, CConstRef<CSeq_id> );
     void x_AddQualProtComment( const CBioseq_Handle& );
     void x_AddQualProtMethod( const CBioseq_Handle& );
     void x_AddQualProtNote( const CProt_ref*, const CMappedFeat& );
@@ -234,6 +234,7 @@ protected:
         bool pseudo);
     virtual void x_AddQualsRna(const CMappedFeat& feat, CBioseqContext& ctx,
          bool pseudo);
+    void x_AddQualsExt( const CUser_field& field, const CSeq_feat::TExt& ext );
     void x_AddQualsExt( const CSeq_feat::TExt& );
     void x_AddQualsBond( CBioseqContext& );
     void x_AddQualsSite( CBioseqContext& );
@@ -246,7 +247,7 @@ protected:
     void x_AddQualsVariation( CBioseqContext& ctx );
 
     void x_AddQuals( CBioseqContext& ctx, CConstRef<CFeatureItem> parentFeatureItem );
-    void x_AddQuals( CBioseqContext& ctx ) { x_AddQuals( ctx, CConstRef<CFeatureItem>() ); }
+    void x_AddQuals( CBioseqContext& ctx ) override { x_AddQuals( ctx, CConstRef<CFeatureItem>() ); }
     void x_AddQualsIdx( CBioseqContext& ctx, CConstRef<CFeatureItem> parentFeatureItem );
     void x_AddQualsIdx( CBioseqContext& ctx ) { x_AddQualsIdx( ctx, CConstRef<CFeatureItem>() ); }
 
@@ -254,6 +255,7 @@ protected:
     void x_AddProductIdQuals(CBioseq_Handle& prod, EFeatureQualifier slot);
     void x_AddQualsGene(const CBioseqContext& ctx, const CGene_ref*, CConstRef<CSeq_feat>&,
         bool from_overlap);
+    void x_AddGoQuals(const CUser_field& field);
     void x_AddGoQuals(const CUser_object& uo);
     void x_ImportQuals(CBioseqContext& ctx);
     void x_AddRptUnitQual(const string& rpt_unit);
@@ -306,7 +308,7 @@ protected:
     typedef TQuals::iterator                  TQI;
     typedef TQuals::const_iterator            TQCI;
     typedef IFlatQVal::TFlags                 TQualFlags;
-     
+
     // qualifiers container
     void x_AddQual(EFeatureQualifier slot, const IFlatQVal* value) {
         m_Quals.AddQual(slot, value);
@@ -314,7 +316,7 @@ protected:
     void x_RemoveQuals(EFeatureQualifier slot) const {
         m_Quals.RemoveQuals(slot);
     }
-    bool x_HasQual(EFeatureQualifier slot) const { 
+    bool x_HasQual(EFeatureQualifier slot) const {
         return m_Quals.HasQual(slot);
     }
     /*pair<TQCI, TQCI> x_GetQual(EFeatureQualifier slot) const {
@@ -331,11 +333,11 @@ protected:
     bool x_HasMethodtRNAscanSE(void) const;
 
     // format
-    void x_FormatQuals(CFlatFeature& ff) const;
+    void x_FormatQuals(CFlatFeature& ff) const override;
     void x_FormatNoteQuals(CFlatFeature& ff) const;
     void x_FormatQual(EFeatureQualifier slot, const char* name,
         CFlatFeature::TQuals& qvec, TQualFlags flags = 0) const;
-    void x_FormatNoteQual(EFeatureQualifier slot, const CTempString & name, 
+    void x_FormatNoteQual(EFeatureQualifier slot, const CTempString & name,
             CFlatFeature::TQuals& qvec, TQualFlags flags = 0) const;
     void x_FormatGOQualCombined( EFeatureQualifier slot, const CTempString & name,
         CFlatFeature::TQuals& qvec, TQualFlags flags = 0) const;
@@ -362,7 +364,7 @@ inline void CFeatureItem::x_AddQualDb(
     }
     x_AddQual(eFQ_gene_xref, new CFlatXrefQVal( gene_ref->GetDb() ) );
 }
-    
+
 //  ----------------------------------------------------------------------------
 inline void CFeatureItem::x_AddQualCitation()
 //  ----------------------------------------------------------------------------
@@ -374,7 +376,7 @@ inline void CFeatureItem::x_AddQualCitation()
 }
 
 //  ----------------------------------------------------------------------------
-inline void CFeatureItem::x_AddQualsGb( 
+inline void CFeatureItem::x_AddQualsGb(
     CBioseqContext& ctx )
 //  ----------------------------------------------------------------------------
 {
@@ -393,7 +395,7 @@ inline void CFeatureItem::x_AddQualExt()
 }
 
 //  ============================================================================
-class NCBI_FORMAT_EXPORT CSourceFeatureItem: 
+class NCBI_FORMAT_EXPORT CSourceFeatureItem:
     public CFeatureItemBase
 //  ============================================================================
 {
@@ -403,7 +405,7 @@ public:
     CSourceFeatureItem(const CBioSource& src, TRange range,
         CBioseqContext& ctx, CRef<feature::CFeatTree> ftree);
     CSourceFeatureItem(const CMappedFeat& feat, CBioseqContext& ctx,
-        CRef<feature::CFeatTree> ftree, const CSeq_loc* loc = NULL);
+        CRef<feature::CFeatTree> ftree, const CSeq_loc* loc = nullptr);
 
     void SetObject(const CSerialObject& obj) { x_SetObject(obj); }
 
@@ -411,7 +413,7 @@ public:
     const CBioSource& GetSource(void) const {
         return m_Feat.GetData().GetBiosrc();
     }
-    string GetKey(void) const { return "source"; }
+    string GetKey(void) const override { return "source"; }
 
     bool IsFocus    (void) const { return m_IsFocus;     }
     bool IsSynthetic(void) const { return m_IsSynthetic; }
@@ -419,26 +421,26 @@ public:
 
     void SetLoc(const CSeq_loc& loc);
 
-    virtual EItem GetItemType(void) const;
+    EItem GetItemType() const override;
 
 private:
     typedef CQualContainer<ESourceQualifier> TQuals;
     typedef TQuals::const_iterator           TQCI;
     typedef IFlatQVal::TFlags                TQualFlags;
 
-    void x_GatherInfo(CBioseqContext& ctx);
+    void x_GatherInfo(CBioseqContext& ctx) override;
 
-    void x_AddQuals(CBioseqContext& ctx);
+    void x_AddQuals(CBioseqContext& ctx) override;
     void x_AddQuals(const CBioSource& src, CBioseqContext& ctx) const;
     void x_AddQuals(const COrg_ref& org, CBioseqContext& ctx) const;
     void x_AddPcrPrimersQuals(const CBioSource& src, CBioseqContext& ctx) const;
 
     // XXX - massage slot as necessary and perhaps sanity-check value's type
     void x_AddQual (ESourceQualifier slot, const IFlatQVal* value) const {
-        m_Quals.AddQual(slot, value); 
+        m_Quals.AddQual(slot, value);
     }
 
-    void x_FormatQuals(CFlatFeature& ff) const;
+    void x_FormatQuals(CFlatFeature& ff) const override;
     void x_FormatGBNoteQuals(CFlatFeature& ff) const;
     void x_FormatNoteQuals(CFlatFeature& ff) const;
     void x_FormatQual(ESourceQualifier slot, const CTempString& name,
