@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___ALN_READER__HPP
 #define OBJTOOLS_READERS___ALN_READER__HPP
 
-/*  $Id: aln_reader.hpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: aln_reader.hpp 671316 2023-08-11 17:49:41Z gotvyans $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,7 @@ public:
     private:
         list<CAlnError> errors;
     public:
-        size_t GetErrorCount(CAlnError::EAlnErr category) const
+        size_t GetErrorCount(CAlnError::EAlnErr /*category*/) const
         {
             return 0;
         }
@@ -124,7 +124,7 @@ public:
         void clear(void) {
         }
 
-        void push_back(const CAlnError& error) {
+        void push_back(const CAlnError& /*error*/) {
         }
 
         size_t size(void) const {
