@@ -1,4 +1,4 @@
-/*  $Id: gff2_data.hpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: gff2_data.hpp 663694 2023-02-27 16:15:10Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,11 +44,9 @@ class CGff2Record:
     public CGffBaseColumns
 {
 public:
-    //typedef CCdregion::EFrame TFrame;
     typedef map<string, string> TAttributes;
     typedef TAttributes::iterator TAttrIt;
     typedef TAttributes::const_iterator TAttrCit;
-    using SeqIdResolver = CRef<CSeq_id> (*)(const string&, unsigned int, bool);
 
 
 public:
@@ -92,12 +90,12 @@ public:
         list<string>& ) const;
 
     virtual bool InitializeFeature(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         SeqIdResolver = nullptr ) const;
 
     virtual bool UpdateFeature(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         SeqIdResolver = nullptr ) const;
 
@@ -117,30 +115,30 @@ protected:
         vector< string >& ) const;
 
     virtual bool xMigrateAttributes(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat> ) const;
 
     virtual bool xInitFeatureData(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>) const;
 
     virtual bool xUpdateFeatureData(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         SeqIdResolver = nullptr ) const;
 
     virtual bool xMigrateAttributesSubSource(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         TAttributes& ) const;
 
     virtual bool xMigrateAttributesOrgName(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         TAttributes& ) const;
 
     virtual bool xMigrateAttributesGo(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         TAttributes& ) const;
 
@@ -157,14 +155,14 @@ protected:
         const string&,
         CRef<CSeq_feat>,
         const string&,
-        int);
+        TReaderFlags);
 
     static bool xMigrateAttributeSingle(
         TAttributes&,
         const string&,
         CRef<CSeq_feat>,
         const string&,
-        int);
+        TReaderFlags);
 
     //
     // Data:
