@@ -1,4 +1,4 @@
-/*  $Id: ucscregion_reader.hpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: ucscregion_reader.hpp 631224 2021-05-13 21:23:28Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,11 +53,11 @@ public:
     CUCSCRegionReader(unsigned int = fNormal);
     virtual ~CUCSCRegionReader();
 
-    virtual CRef<CSeq_annot> ReadSeqAnnot(ILineReader& lr, ILineErrorListener* pEC);
-    virtual CRef< CSerialObject >
+    CRef<CSeq_annot> ReadSeqAnnot(ILineReader& lr, ILineErrorListener* pEC) override;
+    CRef< CSerialObject >
     ReadObject(
         ILineReader& lr,
-        ILineErrorListener* pErrors=0);
+        ILineErrorListener* pErrors=nullptr) override;
 
 protected:
 
