@@ -1,4 +1,4 @@
-/*  $Id: wiggle_reader.hpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: wiggle_reader.hpp 697928 2025-05-27 18:27:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -265,16 +265,16 @@ public:
     };
     typedef int TFlags;
 
-    virtual CRef< CSeq_annot >
+    CRef< CSeq_annot >
     ReadSeqAnnot(
         ILineReader&,
-        ILineErrorListener* =0 );
+        ILineErrorListener* =nullptr ) override;
 
     virtual bool
     ReadTrackData(
         ILineReader&,
         CRawWiggleTrack&,
-        ILineErrorListener* =0 );
+        ILineErrorListener* =nullptr );
 
     //
     //  helpers:
@@ -282,22 +282,23 @@ public:
 protected:
     void xGetData(
         ILineReader&,
-        TReaderData&);
+        TReaderData&) override;
 
     void xProcessData(
         const TReaderData&,
-        CSeq_annot&);
+        CSeq_annot&) override;
 
     void xPostProcessAnnot(
-        CSeq_annot&);
+        CSeq_annot&) override;
 
     bool
     xParseBrowserLine(
-        const string&);
+        const CTempString&,
+        CSeq_annot&) override;
 
     bool
     xParseTrackLine(
-        const string&);
+        const CTempString&) override;
 
     bool
     xProcessFixedStepData(
