@@ -1,7 +1,7 @@
 #ifndef SERIAL___OBJISTR__HPP
 #define SERIAL___OBJISTR__HPP
 
-/*  $Id: objistr.hpp 607093 2020-04-30 12:29:36Z grichenk $
+/*  $Id: objistr.hpp 681710 2024-04-12 15:49:00Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -494,6 +494,13 @@ public:
     ///   string
     virtual string GetPosition(void) const override;
 
+    /// Get current stream location as tuple (positiontype:string, size_t).
+    /// Useful for customized diagnostic and information messages.
+    /// The meaning of positiontype depends on the implementation and coincides
+    /// with the prefix of the string returned by GetPosition().
+    ///
+    virtual void Location(string&, size_t&) const;
+
 //---------------------------------------------------------------------------
 // Local read hooks
     void SetPathReadObjectHook( const string& path, CReadObjectHook*        hook);
@@ -538,7 +545,7 @@ public:
     ///   Maximum number of input bytes to use for the lookup.
     /// @return
     ///   Set of matching types.
-    virtual set<TTypeInfo> GuessDataType(set<TTypeInfo>& known_types,
+    virtual set<TTypeInfo> GuessDataType(const set<TTypeInfo>& known_types,
                                          size_t max_length = 16,
                                          size_t max_bytes  = 1024*1024);
     // root reader
@@ -689,7 +696,7 @@ public:
     void SkipStd(const char* const& );
 
     void ReadStd(CBitString& data);
-    void SkipStd(CBitString& data);
+    void SkipStd(const CBitString& data);
 
     // primitive readers
     // bool
@@ -781,9 +788,10 @@ public:
         ReadStd(data.Set());
     }
 
-    template<class TKey, class TStorage> void SkipStd(CStrictId<TKey, TStorage>& data)
+    template<class TKey, class TStorage> void SkipStd
+        (const CStrictId<TKey, TStorage>& data)
     {
-        SkipStd(data.Set());
+        SkipStd(data.Get());
     }
 
 #ifdef NCBI_STRICT_GI
@@ -905,7 +913,7 @@ public:
 
 #if HAVE_NCBI_C
     // ASN.1 interface
-    class NCBI_XSERIAL_EXPORT AsnIo
+    class AsnIo
     {
     public:
         AsnIo(CObjectIStream& in, const string& rootTypeName);
@@ -1032,9 +1040,7 @@ protected:
     virtual void x_SetPathHooks(bool set) override;
     bool x_HavePathHooks() const;
     EFixNonPrint x_GetFixCharsMethodDefault(void) const;
-    EFixNonPrint x_FixCharsMethod(void) const {
-        return m_FixMethod;
-    }
+    EFixNonPrint x_FixCharsMethod(void) const;
     char x_FixCharsSubst(void) const {
         return m_NonPrintSubst;
     }
