@@ -1,7 +1,7 @@
 #ifndef OBJOSTR__HPP
 #define OBJOSTR__HPP
 
-/*  $Id: objostr.hpp 623252 2021-01-11 16:26:27Z gouriano $
+/*  $Id: objostr.hpp 674289 2023-10-23 14:30:08Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -786,9 +786,7 @@ protected:
 
     virtual void x_SetPathHooks(bool set) override;
     EFixNonPrint x_GetFixCharsMethodDefault(void) const;
-    EFixNonPrint x_FixCharsMethod(void) const {
-        return m_FixMethod;
-    }
+    EFixNonPrint x_FixCharsMethod(void) const;
     char x_FixCharsSubst(void) const {
         return m_NonPrintSubst;
     }
