@@ -1,7 +1,7 @@
 #ifndef STREAMITER__HPP
 #define STREAMITER__HPP
 
-/*  $Id: streamiter.hpp 609562 2020-06-03 19:09:59Z whlavina $
+/*  $Id: streamiter.hpp 689838 2024-11-07 15:53:03Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -474,9 +474,13 @@ public:
 
 template<typename TRoot>
 class CObjectIStreamIterator<TRoot>
-    : public iterator< input_iterator_tag, TRoot, ptrdiff_t, TRoot*, TRoot& >
 {
 public:
+    using iterator_category = input_iterator_tag;
+    using value_type        = TRoot;
+    using difference_type   = ptrdiff_t;
+    using pointer           = TRoot*;
+    using reference         = TRoot&;
     using CParams = CObjectIStreamIterator<>::CParams<TRoot>;
 
     CObjectIStreamIterator( CObjectIStream& istr,
@@ -619,7 +623,7 @@ xxx_GetTypeInfo(void)
 
 template<typename TRoot>
 //typename enable_if< !is_base_of< CSerialObject, TRoot>::value, TTypeInfo>::type
-typename enable_if< is_pod<TRoot>::value || is_convertible<TRoot, std::string>::value, TTypeInfo>::type
+typename enable_if< (is_trivial<TRoot>::value && is_standard_layout <TRoot>::value) || is_convertible<TRoot, std::string>::value, TTypeInfo>::type
 xxx_GetTypeInfo(void)
 {
     return CStdTypeInfo<TRoot>::GetTypeInfo();
@@ -677,8 +681,7 @@ CObjectIStreamIterator<TRoot>::CObjectIStreamIterator(
 {
     if (m_Data->m_FilterType != CData::eNone && !m_Data->m_EndOfData) {
         m_Data->m_HasReader = true;
-        m_Data->m_Reader = thread( mem_fun<void, CObjectIStreamIterator<TRoot> >(
-            &CObjectIStreamIterator<TRoot>::x_ReaderThread), this);
+        m_Data->m_Reader = thread([this](){x_ReaderThread();});
     }
     ++(*this);
 }
@@ -1104,8 +1107,7 @@ CObjectIStreamIterator<TRoot, TChild>::CObjectIStreamIterator(
 {
     if (!this->m_Data->m_EndOfData) {
         this->m_Data->m_HasReader = true;
-        this->m_Data->m_Reader = thread( mem_fun<void, CObjectIStreamIterator<TRoot,TChild> >(
-            &CObjectIStreamIterator<TRoot,TChild>::x_ReaderThread), this);
+        this->m_Data->m_Reader = thread([this](){x_ReaderThread();});
     }
     ++(*this);
 }
@@ -1152,9 +1154,13 @@ CObjectIStreamIterator<TRoot, TChild>::end(void) {
 
 template<typename TRoot>
 class CObjectIStreamAsyncIterator<TRoot>
-    : public iterator< input_iterator_tag, TRoot, ptrdiff_t, TRoot*, TRoot& >
 {
 public:
+    using iterator_category = input_iterator_tag;
+    using value_type        = TRoot;
+    using difference_type   = ptrdiff_t;
+    using pointer           = TRoot*;
+    using reference         = TRoot&;
     using CParams = CObjectIStreamAsyncIterator<>::CParams<TRoot>;
 
     CObjectIStreamAsyncIterator( CObjectIStream& istr,
@@ -1433,9 +1439,7 @@ CObjectIStreamAsyncIterator<TRoot>::CData::CData(
     , m_Params(params)
 {
     if (m_MaxRawSize != 0 && !m_EndOfData) {
-        m_Reader = thread(
-            mem_fun<void, CObjectIStreamAsyncIterator<TRoot>::CData >(
-                &CObjectIStreamAsyncIterator<TRoot>::CData::x_ReaderThread), this);
+        m_Reader = thread([this](){x_ReaderThread();});
     }
 }
 
@@ -1527,7 +1531,7 @@ CObjectIStreamAsyncIterator<TRoot>::CData::x_UpdateFuturesQueue()
     swap(m_GarbageQueue, tmp_garbage_queue);
 
     m_FuturesQueue.push( async( m_Policy, m_Parser,
-        data,  m_Istr->GetDataFormat(), m_Params, move(tmp_garbage_queue)));
+        data,  m_Istr->GetDataFormat(), m_Params, std::move(tmp_garbage_queue)));
 }
 
 template<typename TRoot>
