@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_DATA_LOADERS_CSRA___CSRALOADER_IMPL__HPP
 #define OBJTOOLS_DATA_LOADERS_CSRA___CSRALOADER_IMPL__HPP
 
-/*  $Id: csraloader_impl.hpp 671487 2023-08-16 12:51:25Z ivanov $
+/*  $Id: csraloader_impl.hpp 671278 2023-08-10 21:32:34Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -128,8 +128,8 @@ public:
                 return *this;
             }
         CLock(CLock&& lock)
-            : m_Cache(move(lock.m_Cache)),
-              m_Slot(move(lock.m_Slot))
+            : m_Cache(std::move(lock.m_Cache)),
+              m_Slot(std::move(lock.m_Slot))
             {
             }
         CLock& operator=(CLock&& lock)
