@@ -1,6 +1,6 @@
 #ifndef SRA__READER__SRA__CSRAREAD__HPP
 #define SRA__READER__SRA__CSRAREAD__HPP
-/*  $Id: csraread.hpp 668514 2023-06-06 20:12:19Z vasilche $
+/*  $Id: csraread.hpp 677257 2024-01-09 18:27:59Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -147,6 +147,10 @@ public:
     CRef<CSeq_id> MakeShortReadId(TVDBRowId id1, INSDC_coord_one id2) const;
     void SetShortReadId(string& str, TVDBRowId id1, INSDC_coord_one id2) const;
 
+    bool IsCSRA() const {
+        return m_IsCSRA;
+    }
+
 protected:
     friend class CCSraRefSeqIterator;
     friend class CCSraAlignIterator;
@@ -218,6 +222,7 @@ private:
     CVDBObjectCache<SAlnTableCursor> m_Aln[2];
     CVDBObjectCache<SSeqTableCursor> m_Seq;
 
+    bool m_IsCSRA;
     TSeqPos m_RowSize; // cached size of refseq row in bases
     TRefInfoList m_RefList; // list of cached refseqs' information
     TRefInfoMapByName m_RefMapByName; // index for refseq info lookup
