@@ -1,6 +1,6 @@
 #ifndef SRA__READER__SRA__WGSREAD__HPP
 #define SRA__READER__SRA__WGSREAD__HPP
-/*  $Id: wgsread.hpp 669671 2023-07-07 12:52:48Z ivanov $
+/*  $Id: wgsread.hpp 686378 2024-08-16 17:07:33Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -183,6 +183,13 @@ struct SWGSDb_Defs
         eFeatLocIdAccNoVer
     };
     
+    enum EGBStateType {
+        eGBStateZero    = 0,
+        eGBStateRaw     = 1,
+        eGBStateProject = 2,
+        eGBStateAll     = 3
+    };
+    
     typedef int TSplitVersion;
     static const TSplitVersion kDefaultSplitVersion = 0;
 
@@ -257,7 +264,7 @@ public:
     TVDBRowId ParseProteinRow(CTempString acc) {
         return ParseRowType(acc, fAllowRowType_protein).first;
     }
-    SIZE_TYPE GetIdRowDigits(void) const {
+    Uint4 GetIdRowDigits(void) const {
         return m_IdRowDigits;
     }
 
@@ -588,7 +595,7 @@ private:
     string m_IdPrefixDbWithVersion;
     string m_IdPrefixDb;
     int m_IdVersion;
-    SIZE_TYPE m_IdRowDigits;
+    Uint4 m_IdRowDigits;
 
     CFastMutex m_TableMutex;
     atomic<bool> m_ScfTableIsOpened;
@@ -808,6 +815,7 @@ public:
         fIncludeWithdrawn  = 1 << NCBI_gb_state_eWGSGenBankWithdrawn,
         fIncludeUnverified = 1 << NCBI_gb_state_eWGSGenBankUnverified,
         fIncludeAll        = 0xff,
+        fExcludeProjectGBState = 1 << 8, // do not take into account project GB state
         fIncludeDefault    = fIncludeLive | fIncludeUnverified
     };
     DECLARE_SAFE_FLAGS_TYPE(EIncludeFlags, TIncludeFlags);
@@ -1061,6 +1069,8 @@ public:
     string GetQualityAnnotName(void) const;
 
     NCBI_gb_state GetGBState(void) const;
+    NCBI_gb_state GetRawGBState(void) const;
+    NCBI_gb_state GetGBState(EGBStateType type) const;
 
     bool HasPublicComment(void) const;
     CTempString GetPublicComment(void) const;
@@ -1291,6 +1301,8 @@ public:
     TVDBRowIdRange GetLocFeatRowIdRange(void) const;
 
     NCBI_gb_state GetGBState(void) const;
+    NCBI_gb_state GetRawGBState(void) const;
+    NCBI_gb_state GetGBState(EGBStateType type) const;
     
     CRef<CSeq_id> GetId(TFlags flags = fDefaultFlags) const;
     void GetIds(CBioseq::TId& ids, TFlags flags = fDefaultFlags) const;
@@ -1472,6 +1484,8 @@ public:
     CTempString GetRefAcc(void) const;
 
     NCBI_gb_state GetGBState(void) const;
+    NCBI_gb_state GetRawGBState(void) const;
+    NCBI_gb_state GetGBState(EGBStateType type) const;
 
     bool HasPublicComment(void) const;
     CTempString GetPublicComment(void) const;
