@@ -1,7 +1,7 @@
 #ifndef BMCONST__H__INCLUDED__
 #define BMCONST__H__INCLUDED__
 /*
-Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
+Copyright(c) 2002-2025 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -19,11 +19,12 @@ For more information please visit:  http://bitmagic.io
 */
 
 /*! \file bmconst.h
-    \brief Constants, tables and typedefs
+    \brief Constants, lookup tables and typedefs
     @internal
 */
 
 #include <cstdint>
+#include <inttypes.h>
 
 namespace bm
 {
@@ -77,6 +78,7 @@ const unsigned set_word_mask  = 0x1Fu;
 typedef unsigned short gap_word_t;
 
 const unsigned gap_max_buff_len = 1280;
+const unsigned gap_max_safe_len = 8189; 
 const unsigned gap_max_bits = 65536;
 const unsigned gap_equiv_len = (unsigned)
    ((sizeof(bm::word_t) * bm::set_block_size) / sizeof(bm::gap_word_t));
@@ -114,14 +116,17 @@ const unsigned bits_in_block = bm::set_block_size * unsigned((sizeof(bm::word_t)
 const unsigned bits_in_array = bm::bits_in_block * bm::set_array_size32;
 
 
-// Rank-Select parameters
+// Rank-Select parameters (linear address to split the searches
 const unsigned rs3_border0 = 21824; // 682 words by 32-bits
 const unsigned rs3_border1 = (rs3_border0 * 2); // 43648
 const unsigned rs3_half_span = rs3_border0 / 2;
+const unsigned rs3_border0_1 = rs3_border0 + rs3_half_span; // intermed pnt 1
+const unsigned rs3_border1_1 = rs3_border1 + rs3_half_span; // intermed pnt 2
 
 // misc parameters for sparse vec algorithms
-const unsigned sub_block3_size = bm::gap_max_bits / 4;
-
+//const unsigned sub_bfind_block_cnt = 32; // bfind discretization factor
+//const unsigned sub_block_l1_size =
+//        bm::gap_max_bits / bm::sub_bfind_block_cnt; // size in bits/elements
 
 #if defined(BM64OPT) || defined(BM64_SSE4)
 typedef id64_t  wordop_t;
@@ -144,6 +149,17 @@ enum strategy
     BM_GAP = 1  //!< GAP compression is ON.
 };
 
+/*!
+   @brief copy strategy
+   @ingroup bvector
+*/
+enum class finalization
+{
+    UNDEFINED = 0,
+    READONLY = 1,   //!<  immutable (read-only object)
+    READWRITE = 2,  //!<  mutable (read-write object)
+};
+
 
 /**
     Codes of set operations
@@ -174,10 +190,10 @@ enum set_operation
 */
 enum operation
 {
-    BM_AND = set_AND,
-    BM_OR  = set_OR,
-    BM_SUB = set_SUB,
-    BM_XOR = set_XOR
+    BM_AND = bm::set_AND,
+    BM_OR  = bm::set_OR,
+    BM_SUB = bm::set_SUB,
+    BM_XOR = bm::set_XOR
 };
 
 
@@ -198,7 +214,7 @@ enum sort_order
     @brief set representation variants
     @internal
 */
-enum set_representation
+enum class set_representation
 {
     set_bitset  = 0,  //!< Simple bitset
     set_gap     = 1,  //!< GAP-RLE compression
@@ -223,14 +239,20 @@ enum null_support
 */
 template<bool T> struct _copyright
 {
-    static const char _p[];
-    static const unsigned _v[3];
+    static const char _p[]; ///< Version as a text
+    static const unsigned _v[3]; ///< MAJOR.MINOR.PATCH version components
 };
 
-template<bool T> const char _copyright<T>::_p[] = 
-    "BitMagic C++ Library. v.7.3.3 (c) 2002-2021 Anatoliy Kuznetsov.";
-template<bool T> const unsigned _copyright<T>::_v[3] = {7, 3, 3};
+#define BM_VERSION_MAJOR 8
+#define BM_VERSION_MINOR 0
+#define BM_VERSION_PATCH 1
 
+template<bool T> const char _copyright<T>::_p[] = 
+    "BitMagic Library. v.8.0.1 (c) 2002-2025 Anatoliy Kuznetsov.";
+template<bool T> const unsigned _copyright<T>::_v[3] =
+                    { BM_VERSION_MAJOR, BM_VERSION_MINOR, BM_VERSION_PATCH };
+
+#define BM_SCALAR_VERSION (((BM_VERSION_MAJOR) << 16) + ((BM_VERSION_MINOR) << 8) + (BM_VERSION_PATCH))
 
 
 /**
@@ -417,7 +439,9 @@ enum simd_codes
     simd_sse2  = 1,   ///!< Intel SSE2
     simd_sse42 = 2,   ///!< Intel SSE4.2
     simd_avx2  = 5,   ///!< Intel AVX2
-    simd_avx512  = 6  ///!< Intel AVX512
+    simd_avx512  = 6,  ///!< Intel AVX512
+    simd_wasm128 = 7,  ///! WASM SIMD 128
+    simd_neon = 8      ///!< ARM Neon
 };
 
 
@@ -466,6 +490,7 @@ template<bool T> struct globals
 template<bool T> typename globals<T>::bo globals<T>::_bo;
 
 
+
 } // namespace
 
 #endif
