@@ -19,7 +19,7 @@ For more information please visit:  http://bitmagic.io
 */
 
 /*! \file bmdbg.h
-    \brief Debugging functions (internal)
+    \brief Debugging functions (internal). Poorly documented, not well written.
 */
 
 
@@ -49,31 +49,31 @@ For more information please visit:  http://bitmagic.io
 namespace bm
 {
 
-inline
-void PrintGap(const bm::gap_word_t* gap_buf)
+template<class TOut>
+void PrintGap(TOut& tout, const bm::gap_word_t* gap_buf)
 {
     unsigned len = (*gap_buf >> 3);
-    std::cout << "[" << *gap_buf << " len=" << len << "] ";
+    tout << "[" << *gap_buf << " len=" << len << "] ";
     for (unsigned i = 0; i < len; ++i)
     {
         ++gap_buf;
-        std::cout << *gap_buf << "; ";
+        tout << *gap_buf << "; ";
     } 
-    std::cout << std::endl;
+    tout << std::endl;
 }
 
-inline
-void PrintDGap(const bm::gap_word_t* gap_buf, unsigned gap_len=0)
+template<class TOut>
+void PrintDGap(TOut& tout, const bm::gap_word_t* gap_buf, unsigned gap_len=0)
 {
 
     unsigned len = gap_len ? gap_len : (*gap_buf >> 3);
-    std::cout << "[" " len=" << len << "] ";
+    tout << "[" " len=" << len << "] ";
     unsigned i = gap_len ? 0 : 1;
     for (; i < len; ++i)
     {
-        std::cout << gap_buf[i] << "; ";
+        tout << gap_buf[i] << "; ";
     } 
-    std::cout << std::endl;
+    tout << std::endl;
 }
 
 inline unsigned int iLog2(unsigned int value)
@@ -83,49 +83,49 @@ inline unsigned int iLog2(unsigned int value)
     return l;
 }
 
-inline
-unsigned PrintGammaCode(unsigned value)
+template<class TOut>
+unsigned PrintGammaCode(TOut& tout, unsigned value)
 {
     unsigned bits = 0;
     // Elias gamma encode
     {
         unsigned l = iLog2(value);
-        //cout << "log2=" << l << endl;
+        //tout << "log2=" << l << endl;
         for (unsigned i = 0; i < l; ++i)
         {
-            std::cout << 0;
+            tout << 0;
             ++bits;
         }
-        std::cout << 1; ++bits;
+        tout << 1; ++bits;
         for (unsigned i = 0; i < l; ++i)
         {
             if (value & 1 << i) 
-                std::cout << 1;
+                tout << 1;
             else
-                std::cout << 0;
+                tout << 0;
             ++bits;
         }
     }
     return bits;
 }
 
-inline 
-void PrintDGapGamma(const bm::gap_word_t* gap_buf, unsigned gap_len=0)
+template<typename TOut>
+void PrintDGapGamma(TOut& tout, const bm::gap_word_t* gap_buf, unsigned gap_len=0)
 {
     unsigned total = 0;
     unsigned len = gap_len ? gap_len : (*gap_buf >> 3);
-    std::cout << "[" " len=" << len << "] ";
+    tout << "[" " len=" << len << "] ";
     unsigned i = gap_len ? 0 : 1;
     for (; i < len; ++i)
     {
         unsigned v = gap_buf[i];
 
-        unsigned bits = PrintGammaCode(v+1);
-        std::cout << "; ";
+        unsigned bits = PrintGammaCode(tout, v+1);
+        tout << "; ";
         total += bits;
     } 
-    std::cout << "  gamma_bits=" << total << " src_bits =" << len * 16;
-    std::cout << std::endl;
+    tout << "  gamma_bits=" << total << " src_bits =" << len * 16;
+    tout << std::endl;
 
 }
 
@@ -169,14 +169,14 @@ void LoadBVector(const char* fname, TBV& bvector, unsigned* file_size=0)
     std::ifstream bv_file (fname, std::ios::in | std::ios::binary);
     if (!bv_file.good())
     {
-        std::cout << "Cannot open file: " << fname << std::endl;
+        std::cerr << "Cannot open file: " << fname << std::endl;
         exit(1);
     }
     bv_file.seekg(0, std::ios_base::end);
     unsigned length = (unsigned)bv_file.tellg();
     if (length == 0)
     {
-        std::cout << "Empty file:" << fname << std::endl;
+        std::cerr << "Empty file:" << fname << std::endl;
         exit(1);
     }
     if (file_size)
@@ -194,25 +194,31 @@ void LoadBVector(const char* fname, TBV& bvector, unsigned* file_size=0)
 }
 
 template<class TBV>
-void SaveBVector(const char* fname, const TBV& bvector)
+void SaveBVector(const char* fname, const TBV& bvector, bool safe_mode=false)
 {
     std::ofstream bfile (fname, std::ios::out | std::ios::binary);
     if (!bfile.good())
     {
-        std::cout << "Cannot open file: " << fname << std::endl;
+        std::cerr << "Cannot open file: " << fname << std::endl;
         exit(1);
     }
     typename TBV::statistics st1;
     bvector.calc_stat(&st1);
 
-    unsigned char* blob = new unsigned char[st1.max_serialize_mem];
-    size_t blob_size = bm::serialize(bvector, blob);
+    BM_DECLARE_TEMP_BLOCK(tb)
+    bm::serializer<TBV> bv_ser(tb);
+    if (safe_mode)
+    {
+        bv_ser.set_compression_level(3);
+        bv_ser.set_bic_dynamic_range_reduce(false);
+    }
 
+    typename bm::serializer<TBV>::buffer sermem_buf;
+    bv_ser.serialize(bvector, sermem_buf, 0);
+    auto blob_size = sermem_buf.size();
 
-    bfile.write((char*)blob, std::streamsize(blob_size));
+    bfile.write((char*)sermem_buf.buf(), std::streamsize(blob_size));
     bfile.close();
-
-    delete [] blob;
 }
 
 inline
@@ -227,7 +233,7 @@ void SaveBlob(const char* name_prefix, unsigned num, const char* ext,
     std::ofstream bfile (fname, std::ios::out | std::ios::binary);
     if (!bfile.good())
     {
-        std::cout << "Cannot open file: " << fname << std::endl;
+        std::cerr << "Cannot open file: " << fname << std::endl;
         exit(1);
     }
     bfile.write((char*)blob, std::streamsize(blob_size));
@@ -235,60 +241,59 @@ void SaveBlob(const char* name_prefix, unsigned num, const char* ext,
 }
 
 
-template<typename V> 
-void PrintBinary(V val)
+template<typename V, typename TOut>
+void PrintBinary(TOut& tout, V val)
 {
     for (unsigned i = 0; i < sizeof(V)*8; i++)
     {
-        std::cout << (unsigned)((val >> i) & 1);
-        if (i == 15 && (sizeof(V)*8 > 16)) std::cout << "-";
+        tout << (unsigned)((val >> i) & 1);
+        if (i == 15 && (sizeof(V)*8 > 16)) tout << "-";
     }
-//    cout << " :" << val;
 }
 
-inline 
-void PrintBits32(unsigned val)
+template<typename TOut>
+void PrintBits32(TOut& tout, unsigned val)
 {
-    PrintBinary(val);
+    PrintBinary(tout, val);
 }
 
-inline
-void PrintDistanceMatrix(
+template<typename TOut>
+void PrintDistanceMatrix(TOut& tout,
    const unsigned distance[bm::set_block_plane_cnt][bm::set_block_plane_cnt])
 {
     for (unsigned i = 0; i < bm::set_block_plane_cnt; ++i)
     {
         const unsigned* row = distance[i];
-        std::cout << i << ": ";
+        tout << i << ": ";
         for (unsigned j = i; j < bm::set_block_plane_cnt; ++j)
         {
-            std::cout << std::setw(4) << std::setfill('0') << row[j] << " ";
+            tout << std::setw(4) << std::setfill('0') << row[j] << " ";
         }
-        std::cout << std::endl;
+        tout << std::endl;
     }
 }
 
-template<typename TM>
-void PrintTMatrix(const TM& tmatrix, unsigned cols=0, bool binary = false)
+template<typename TM, typename TOut>
+void PrintTMatrix(TOut& tout, const TM& tmatrix, unsigned cols=0, bool binary = false)
 {
     unsigned columns = cols ? cols : tmatrix.cols();
     for (unsigned i = 0; i < tmatrix.rows(); ++i)
     {
         const typename TM::value_type* row = tmatrix.row(i);
-        std::cout << i << ": ";
-        if (i < 10) std::cout << " ";
+        tout << i << ": ";
+        if (i < 10) tout << " ";
         for (unsigned j = 0; j < columns; ++j)
         {
             if (!binary)
             {
-                std::cout << std::setw(4) << std::setfill('0') << row[j] << " ";
+                tout << std::setw(4) << std::setfill('0') << row[j] << " ";
             }
             else
             {
-                PrintBinary(row[j]);
+                PrintBinary(tout, row[j]);
             }
         }
-        std::cout << std::endl;
+        tout << std::endl;
     }
 }
 
@@ -320,8 +325,8 @@ unsigned BinStrLR(const char* str)
     return value;
 }
 
-template<class BV>
-void print_blocks_count(const BV& bv)
+template<class BV, typename TOut>
+void print_blocks_count(TOut& tout, const BV& bv)
 {
     const unsigned sz = 128000;
     unsigned* bc_arr = new unsigned[sz];
@@ -333,23 +338,23 @@ void print_blocks_count(const BV& bv)
 
     for (unsigned i = 0; i <= last_block; ++i)
     {
-        std::cout << i << ":";
+        tout << i << ":";
 
         unsigned j = 0;
         for (; i <= last_block; ++i)
         {
-            std::cout << std::setw(5) << std::setfill('0') << bc_arr[i] << " ";
+            tout << std::setw(5) << std::setfill('0') << bc_arr[i] << " ";
             sum += bc_arr[i];
             if (++j == 10) break;
         }
-        std::cout << " | " << sum << std::endl;
+        tout << " | " << sum << std::endl;
     }
-    std::cout << "Total=" << sum << std::endl;
-
+    tout << "Total=" << sum << std::endl;
     delete [] bc_arr;
 }
-inline 
-void print_bc(unsigned i, unsigned count)
+
+template<typename TOut>
+void print_bc(TOut& tout, unsigned i, unsigned count)
 {
     static unsigned sum = 0;
     static unsigned row_idx = 0;
@@ -362,28 +367,28 @@ void print_bc(unsigned i, unsigned count)
     else
     {
         if (prev +1 < i)
-            print_bc(prev+1, 0);
+            print_bc(tout, prev+1, 0);
         prev = i;
     }
 
     if (row_idx == 0)
     {
-        std::cout << i << ":";
+        tout << i << ":";
     }
 
-    std::cout << std::setw(5) << std::setfill('0') << count << " ";
+    tout << std::setw(5) << std::setfill('0') << count << " ";
     sum += count;
 
     ++row_idx;
     if (row_idx == 10)
     {
         row_idx = 0;
-        std::cout << " | " << sum << std::endl;
+        tout << " | " << sum << std::endl;
     }
 }
 
-template<class BV>
-size_t print_bvector_stat(const BV& bvect)
+template<class BV, typename TOut>
+size_t print_bvector_stat(TOut& tout, const BV& bvect)
 {
     typename BV::statistics st;
     bvect.calc_stat(&st);
@@ -393,7 +398,7 @@ size_t print_bvector_stat(const BV& bvect)
     ser.serialize(bvect, buf, &st);
     auto ssize = buf.size();
     
-    std::cout << " - Blocks: [ "
+    tout << " - Blocks: [ "
               << "B:"     << st.bit_blocks
               << ", G:"   << st.gap_blocks << "] "
               << " count() = " << bvect.count() 
@@ -405,12 +410,12 @@ size_t print_bvector_stat(const BV& bvect)
 }
 
 
-template<class BV>
-void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
+template<class BV, typename TOut>
+void print_stat(TOut& tout, const BV& bv, typename BV::block_idx_type blocks = 0)
 {
     const typename BV::blocks_manager_type& bman = bv.get_blocks_manager();
 
-    bm::id_t count = 0;
+    bm::id_t count = 0; (void)count;
     int printed = 0;
 
     int total_gap_eff = 0;
@@ -435,7 +440,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
         {
            if (BM_IS_GAP(blk)) // gap block
            {
-               std::cout << "[Alert!" << nb << "]";
+               tout << "[Alert!" << nb << "]";
                assert(0);
            }
            
@@ -448,7 +453,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
                {
                  if (BM_IS_GAP(blk)) // gap block
                  {
-                     std::cout << "[Alert!" << nb << "]";
+                     tout << "[Alert!" << nb << "]";
                      assert(0);
                      --nb;
                      break;
@@ -462,14 +467,14 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
                }
            }
 
-           std::cout << "{F." << start << ":" << nb << "}";
+           tout << "{F." << start << ":" << nb << "}";
            ++printed;
         }
         else
         {
             if ((nb-1) != nb_prev)
             {
-                std::cout << ".." << (size_t)nb-nb_prev << "..";
+                tout << ".." << (size_t)nb-nb_prev << "..";
             }
 
             if (BM_IS_GAP(blk))
@@ -486,7 +491,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
                
                unsigned i,j;
                bm::get_block_coord(nb, i, j);
-               std::cout << " [GAP " << nb << "(" << i << "," << j << ")"
+               tout << " [GAP " << nb << "(" << i << "," << j << ")"
                          << "=" << bc << ":" << level << "-L" << len << "(" << mem_eff << ")]";
                 ++printed;
             }
@@ -501,7 +506,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
                 }
 
                 count += bc;
-                std::cout << " (BIT " << nb << "=" << bc << "[" << zw << "])";
+                tout << " (BIT " << nb << "=" << bc << "[" << zw << "])";
                 ++printed;
             }
         }
@@ -512,7 +517,7 @@ void print_stat(const BV& bv, typename BV::block_idx_type blocks = 0)
         }
         nb_prev = nb;
     } // for nb
-    std::cout << std::endl << "gap_efficiency=" << total_gap_eff << std::endl;
+    tout << std::endl << "gap_efficiency=" << total_gap_eff << std::endl;
 
 }
 
@@ -544,8 +549,8 @@ size_t compute_serialization_size(const BV& bv)
 }
 
 #if 0
-template<class SV>
-void print_svector_xor_stat(const SV& sv)
+template<class SV, typename TOut>
+void print_svector_xor_stat(TOut& toutconst SV& sv)
 {
     BM_DECLARE_TEMP_BLOCK(tb)
     typename SV::size_type sz = sv.size();
@@ -555,7 +560,7 @@ void print_svector_xor_stat(const SV& sv)
 
     for (typename SV::size_type nb = 0; nb < nb_max; ++nb)
     {
-        std::cout << "nb = " << nb << std::endl;
+        tout << "nb = " << nb << std::endl;
 
         unsigned i0 = unsigned(nb >> bm::set_array_shift);
         unsigned j0 = unsigned(nb &  bm::set_array_mask);
@@ -613,27 +618,22 @@ void print_svector_xor_stat(const SV& sv)
                         kb_found = true;
                         //*kb_j = j0;
                     }
-
                 }
-
             } // for k
 
             if (kb_found)
             {
-                std::cout << "XOR match " << "metric gain = " << std::endl;
+                tout << "XOR match " << "metric gain = " << std::endl;
             }
-
-
-            std::cout << std::endl;
-
+            tout << std::endl;
         } // for i
 
     } // for nb
 }
 #endif
 
-template<class SV>
-void print_svector_stat(const SV& svect, bool print_sim = false)
+template<class SV, typename TOut>
+void print_svector_stat(TOut& tout, const SV& svect, bool print_sim = false)
 {
     typedef typename SV::bvector_type bvector_type;
     /// Functor to compute jaccard similarity
@@ -670,9 +670,12 @@ void print_svector_stat(const SV& svect, bool print_sim = false)
     
     bm::build_jaccard_similarity_batch(sbatch, svect);
     
-    sbatch.calculate();
-    sbatch.sort();
-    
+    if (print_sim)
+    {
+        sbatch.calculate();
+        sbatch.sort();
+    }
+
     typename similarity_batch_type::vector_type& sim_vec = sbatch.descr_vect_;
     if (print_sim)
     {
@@ -698,14 +701,14 @@ void print_svector_stat(const SV& svect, bool print_sim = false)
                     size_t sz10p = bv_size2 / 10;
                     if (diff > sz10p)
                     {
-                        std:: cout << "["  << sim_vec[k].get_first_idx()
-                                   << ", " << sim_vec[k].get_second_idx()
-                                   << "] = "  << sim
-                                   << " size(" << sim_vec[k].get_second_idx() << ")="
-                                   << bv_size2
-                                   << " size(x)=" << bv_size_x
-                                   << " diff=" << diff
-                                   << std:: endl;
+                        tout << "["  << sim_vec[k].get_first_idx()
+                             << ", " << sim_vec[k].get_second_idx()
+                             << "] = "  << sim
+                             << " size(" << sim_vec[k].get_second_idx() << ")="
+                             << bv_size2
+                             << " size(x)=" << bv_size_x
+                             << " diff=" << diff
+                             << std:: endl;
                     }
                 }
             }
@@ -716,93 +719,112 @@ void print_svector_stat(const SV& svect, bool print_sim = false)
     typename SV::statistics st;
     svect.calc_stat(&st);
     
-    std::cout << "size = " << svect.size() << std::endl;
-    std::cout << "Bit blocks:       " << st.bit_blocks << std::endl;
-    std::cout << "Gap blocks:       " << st.gap_blocks << std::endl;
-    std::cout << "Max serialize mem:" << st.max_serialize_mem << " "
+    tout << "size = " << svect.size() << std::endl;
+
+    tout << "Bit blocks:       " << st.bit_blocks << std::endl;
+    tout << "GAP blocks:       " << st.gap_blocks << std::endl;
+    tout << "GAP levels counts:";
+    for (unsigned g = 0; g < bm::gap_levels; ++g)
+    {
+        switch (g)
+        {
+        case 0: tout << "[ I:  " << st.gap_levels[g] << "] "; break;
+        case 1: tout << "[ II: " << st.gap_levels[g] << "] "; break;
+        case 2: tout << "[ III:" << st.gap_levels[g] << "] "; break;
+        case 3: tout << "[ IV: " << st.gap_levels[g] << "] "; break;
+        default:
+                tout << "[ " << g << ": " << st.gap_levels[g] << "] "; break;
+        }
+    } // for
+    tout << std::endl;
+
+    tout << "Max serialize mem:" << st.max_serialize_mem << " "
               << (st.max_serialize_mem / (1024 * 1024)) << "MB" << std::endl;
-    std::cout << "Memory used:      " << st.memory_used << " "
+    tout << "Memory used:      " << st.memory_used << " "
               << (st.memory_used / (1024 * 1024))       << "MB" << std::endl;
     
     auto eff_max_element = svect.effective_vector_max();
     size_t std_vect_size = sizeof(typename SV::value_type) * svect.size() * eff_max_element;
-    std::cout << "Projected mem usage for vector<value_type>:"
+    tout << "Projected mem usage for vector<value_type>:"
               << std_vect_size << " "
               << std_vect_size / (1024 * 1024) << "MB"
               << std::endl;
     if (sizeof(typename SV::value_type) > 4 && (eff_max_element == 1))
     {
-        std::cout << "Projected mem usage for vector<long long>:"
+        tout << "Projected mem usage for vector<long long>:"
                   << sizeof(long long) * svect.size() << std::endl;
     }
     
-    std::cout << "\nplanes:" << std::endl;
+    tout << "\nplanes:" << std::endl;
 
     size_t ssize(0), octet_ssize(0);
 
     typename SV::bvector_type bv_join; // global OR of all planes
-    auto planes = svect.planes();
+    auto planes = svect.get_bmatrix().rows();
 
     unsigned octet_cnt(0), octet(0);
     for (unsigned i = 0; i < planes; ++i)
     {
-        const typename SV::bvector_type* bv_plane = svect.get_plane(i);
-        std::cout << i << "-" << octet_cnt << ":";
+        const typename SV::bvector_type* bv_plane = svect.get_slice(i);
+        tout << i << "-" << octet_cnt << ":";
         if (bv_plane == 0)
         {
-            std::cout << "NULL\n";
+            tout << "NULL\n";
             bool any_else = false;
             for (unsigned j = i+1; j < planes; ++j) // look ahead
             {
-                if (svect.get_plane(j))
+                if (svect.get_slice(j))
                 {
                     any_else = true;
                     break;
                 }
             }
             if (!any_else)
-            {
                 break;
-            }
         }
         else
         {
             bv_join |= *bv_plane;
-            auto pssize = bm::print_bvector_stat(*bv_plane);
+            auto pssize = bm::print_bvector_stat(tout,*bv_plane);
             ssize += pssize;
             octet_ssize += pssize;
         }
         if (octet_cnt == 7)
         {
-            std::cout << "--------------------" << std::endl;
-            std::cout << "octet N = " << octet <<
+            tout << "--------------------" << std::endl;
+            tout << "octet N = " << octet <<
                     "  compressed = " << octet_ssize <<
                     " " << octet_ssize/(1024*1024) << "MB" << std::endl;
             octet_cnt = 0; octet_ssize = 0;
             octet++;
-            std::cout << std::endl;
+            tout << std::endl;
         }
         else
         {
             octet_cnt++;
         }
     } // for i
+    tout << "-------------------- END of OCTETS\n";
 
     const typename SV::bvector_type* bv_null = svect.get_null_bvector();
     if (bv_null)
     {
-        std::cout << "(not) NULL plane:\n";
-        ssize += print_bvector_stat(*bv_null);
+        tout << "NULL plane:\n";
+        ssize += print_bvector_stat(tout,*bv_null);
         typename SV::size_type not_null_cnt = bv_null->count();
-        std::cout << " - Bitcount: " << not_null_cnt << std::endl;
+        tout << " - Bitcount: " << not_null_cnt << std::endl;
 
-        std::cout << "Projected mem usage for std::vector<pair<unsigned, value_type> >:"
+        tout << "Projected mem usage for std::vector<pair<unsigned, value_type> >:"
             << ((sizeof(typename SV::value_type) + sizeof(unsigned)) * not_null_cnt) << " "
             << ((sizeof(typename SV::value_type) + sizeof(unsigned)) * not_null_cnt) / (1024 * 1024) << "MB"
             << std::endl;
     }
+    else
+    {
+        tout << "NO NULL plane:\n";
+    }
 
-    std::cout << " Total serialized size (planes): " << ssize
+    tout << " Total serialized size (planes): " << ssize
               << std::endl
               << " " << ssize / (1024 * 1024) << " MB" << std::endl;
 
@@ -810,27 +832,27 @@ void print_svector_stat(const SV& svect, bool print_sim = false)
     {
         bm::id64_t bv_join_cnt = bv_join.count();
         double fr = double(bv_join_cnt) / double (svect.size());
-        std::cout << "Non-zero elements: " << bv_join_cnt << " "
+        tout << "Non-zero elements: " << bv_join_cnt << " "
                   << "ratio=" << fr
                   << std::endl;
         size_t non_zero_mem = size_t(bv_join_cnt) * sizeof(typename SV::value_type);
-        std::cout << "Projected mem usage for non-zero elements: " << non_zero_mem << " "
+        tout << "Projected mem usage for non-zero elements: " << non_zero_mem << " "
                   << non_zero_mem / (1024*1024) << " MB"
                   << std::endl;        
     }
 }
 
 
-template<class SV>
-void print_str_svector_stat(const SV& str_svect)
+template<class SV, typename TOut>
+void print_str_svector_stat(TOut& tout, const SV& str_svect)
 {
-    typename SV::plane_octet_matrix_type octet_stat_matr;
+    typename SV::octet_freq_matrix_type octet_stat_matr;
     
     str_svect.calc_octet_stat(octet_stat_matr);
     
     for (unsigned i = 0; i < octet_stat_matr.rows(); ++i)
     {
-        const typename SV::plane_octet_matrix_type::value_type* row
+        const typename SV::octet_freq_matrix_type::value_type* row
                                                 = octet_stat_matr.row(i);
         bool any = false;
         for (unsigned j = 0; j < octet_stat_matr.cols(); ++j)
@@ -844,28 +866,74 @@ void print_str_svector_stat(const SV& str_svect)
         if (!any)
             continue;
     
-        std::cout << i << " : ";
+        tout << i << " : ";
         unsigned cnt = 0;
         for (unsigned j = 0; j < octet_stat_matr.cols(); ++j)
         {
             if (row[j]) // letter is present
             {
-                std::cout << char(j);
+                tout << char(j);
                 ++cnt;
             }
         } // for j
         if (cnt)
         {
-            std::cout << "\t total= " << cnt;
+            tout << "\t total= " << cnt;
         }
         else
         {
-            std::cout << " (empty) ";
+            tout << " (empty) ";
         }
-        std::cout << std::endl;
+        tout << std::endl;
     } // for i
 }
 
+// Save std::vector
+//
+template<class VECT>
+int save_vector(const VECT& vect, const std::string& fname)
+{
+    std::ofstream fout(fname.c_str(), std::ios::binary);
+    if (!fout.good())
+        return -1;
+    size_t sz = vect.size();
+    fout.write((char*)&sz, sizeof(sz));
+    if (!fout.good())
+        return -1;
+    if (sz)
+    {
+        fout.write((char*)vect.data(),
+            (std::streamsize) (sz*sizeof(typename VECT::value_type)));
+        if (!fout.good())
+            return -1;
+    }
+    fout.close();
+    return 0;
+}
+
+// Save std::vector
+//
+template<class VECT>
+int load_vector(VECT& vect, const std::string& fname)
+{
+    std::ifstream fin(fname.c_str(), std::ios::in | std::ios::binary);
+    if (!fin.good())
+        return -1;
+    size_t sz;
+    fin.read((char*) &sz, sizeof(sz));
+    if (!fin.good())
+        return -2;
+    vect.resize(sz);
+    if (sz)
+    {
+        fin.read((char*)vect.data(), sz*sizeof(typename VECT::value_type));
+        if (!fin.good())
+            return -1;
+    }
+    fin.close();
+    return 0;
+}
+
 
 
 // save compressed collection to disk
@@ -947,7 +1015,8 @@ int file_save_svector(const SV& sv, const std::string& fname,
         return -1;
     }
     const char* buf = (char*)sv_lay.buf();
-    fout.write(buf, std::streamsize(sv_lay.size()));
+    auto sz = sv_lay.size();
+    fout.write(buf, std::streamsize(sz));
     if (!fout.good())
     {
         return -1;
@@ -1025,6 +1094,38 @@ void convert_bv2sv(SV& sv, const BV& bv)
     bit.flush();
 }
 
+#if 0
+/**
+    Get RSS on
+    @internal
+ */
+size_t getCurrentRSS( )
+{
+    long rss = 0L;
+    FILE* fp = NULL;
+    if ( (fp = fopen( "/proc/self/statm", "r" )) == NULL )
+        return (size_t)0L;      /* Can't open? */
+    if ( fscanf( fp, "%*s%ld", &rss ) != 1 )
+    {
+        fclose( fp );
+        return (size_t)0L;      /* Can't read? */
+    }
+    fclose( fp );
+    return (size_t)rss * (size_t)sysconf( _SC_PAGESIZE);
+}
+#endif
+
+
+template<class BV, typename TOut>
+void print_bv(TOut& tout, const BV& bv)
+{
+    tout << bv.count() << ": ";
+    typename BV::enumerator en = bv.first();
+    for (; en.valid(); ++en)
+        tout << *en << ", ";
+    tout << std::endl;
+}
+
 
 } // namespace
 
