@@ -727,11 +727,8 @@ public:
     {
         size_type rows = bmatr.rows();
         for (size_type r = 0; r < rows; ++r)
-        {
-            bvector_type_const_ptr bv = bmatr.get_row(r);
-            if (bv)
+            if (bvector_type_const_ptr bv = bmatr.get_row(r))
                 add(bv, rows_acc_ + r);
-        } // for r
         rows_acc_ += unsigned(rows);
     }
 
@@ -750,20 +747,24 @@ public:
                            size_type total_blocks) const
     {
         if (total_blocks)
-            matr.resize(ref_bvects_.size(), total_blocks);
+            matr.resize(ref_bvects_.size(), total_blocks, false /*no-copy*/);
         else
             matr.resize(0, 0);
     }
 
     /** Calculate blocks digest and resize XOR distance matrix
         based on total number of available blocks
+        @return true if created ok (false if no blocks found)
      */
-    void build_nb_digest_and_xor_matrix(matrix_chain_type& matr,
+    bool build_nb_digest_and_xor_matrix(matrix_chain_type& matr,
                                         bvector_type& bv_blocks) const
     {
         fill_alloc_digest(bv_blocks);
         size_type cnt = bv_blocks.count();
+        if (!cnt)
+            return false;
         resize_xor_matrix(matr, cnt);
+        return true;
     }
 
 protected:
@@ -892,22 +893,25 @@ public:
     /**
         Calculate matrix of best XOR match metrics per block
         for the attached collection of bit-vectors
+        @return true if computed successfully
      */
-    void compute_sim_model(xor_sim_model<BV>&        sim_model,
+    bool compute_sim_model(xor_sim_model<BV>&        sim_model,
                            const bv_ref_vector_type& ref_vect,
                            const bm::xor_sim_params& params);
 
     /**
         Calculate matrix of best XOR match metrics per block
         for the attached collection of bit-vectors
+        @return true if computed successfully
      */
-    void compute_sim_model(xor_sim_model<BV> &sim_model,
+    bool compute_sim_model(xor_sim_model<BV> &sim_model,
                            const bm::xor_sim_params& params);
 
     /**
         Compute similarity model for block
      */
-    void compute_sim_model(xor_sim_model<BV>        &sim_model,
+    void compute_sim_model(
+        typename xor_sim_model<BV>::matrix_chain_type &sim_model_matr,
                            size_type                 nb,
                            size_type                 nb_rank,
                            const bm::xor_sim_params& params);
@@ -1433,27 +1437,30 @@ typename xor_scanner<BV>::size_type xor_scanner<BV>::refine_match_chain()
 // --------------------------------------------------------------------------
 
 template<typename BV>
-void xor_scanner<BV>::compute_sim_model(xor_sim_model<BV> &sim_model,
+bool xor_scanner<BV>::compute_sim_model(xor_sim_model<BV> &sim_model,
                                         const bv_ref_vector_type& ref_vect,
                                         const bm::xor_sim_params& params)
 {
     const bv_ref_vector_type* ref_vect_curr = this->ref_vect_; // save ref-vect
 
     ref_vect_ = &ref_vect;
-    compute_sim_model(sim_model, params);
+    bool sim_ok = compute_sim_model(sim_model, params);
 
     ref_vect_ = ref_vect_curr; // restore state
+    return sim_ok;
 }
 
 template<typename BV>
-void xor_scanner<BV>::compute_sim_model(bm::xor_sim_model<BV>& sim_model,
+bool xor_scanner<BV>::compute_sim_model(bm::xor_sim_model<BV>& sim_model,
                                         const bm::xor_sim_params& params)
 {
     BM_ASSERT(ref_vect_);
 
     sim_model.bv_blocks.clear(false);
-    ref_vect_->build_nb_digest_and_xor_matrix(sim_model.matr,
-                                              sim_model.bv_blocks);
+    bool ret = ref_vect_->build_nb_digest_and_xor_matrix(sim_model.matr,
+                                                         sim_model.bv_blocks);
+    if (!ret)
+        return ret;
 
     sync_nb_vect();
 
@@ -1461,21 +1468,21 @@ void xor_scanner<BV>::compute_sim_model(bm::xor_sim_model<BV>& sim_model,
     for (size_type col = 0; en.valid(); ++en, ++col)
     {
         size_type nb = *en;
-        compute_sim_model(sim_model, nb, col, params);
+        compute_sim_model(sim_model.matr, nb, col, params);
     } // for en
+    return true;
 }
 
 // --------------------------------------------------------------------------
 
 template<typename BV>
-void xor_scanner<BV>::compute_sim_model(xor_sim_model<BV>        &sim_model,
-                                        size_type                 nb,
-                                        size_type                 nb_rank,
-                                        const bm::xor_sim_params& params)
+void xor_scanner<BV>::compute_sim_model(
+                typename xor_sim_model<BV>::matrix_chain_type &sim_model_matr,
+                        size_type                 nb,
+                        size_type                 nb_rank,
+                        const bm::xor_sim_params& params)
 {
-    BM_ASSERT(nb_rank < sim_model.matr.cols());
-    BM_ASSERT(sim_model.bv_blocks.test(nb));
-    BM_ASSERT(nb_rank == (sim_model.bv_blocks.count_range(0, nb)-1));
+    BM_ASSERT(nb_rank < sim_model_matr.cols());
 
     const float bie_bits_per_int = 3.0f; // c_level_ < 6 ? 3.75f : 3.0f;
     const unsigned bie_limit =
@@ -1490,7 +1497,7 @@ void xor_scanner<BV>::compute_sim_model(xor_sim_model<BV>        &sim_model,
 
     for (size_type ri=0; true; ++ri)
     {
-        bm::block_match_chain<size_type>* m_row = sim_model.matr.row(ri);
+        bm::block_match_chain<size_type>* m_row = sim_model_matr.row(ri);
         bm::block_match_chain<size_type>& bmc = m_row[nb_rank];
         bmc.nb = nb;
         bmc.chain_size = 0;
