@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__BZIP2__HPP
 #define UTIL_COMPRESS__BZIP2__HPP
 
-/*  $Id: bzip2.hpp 538813 2017-06-14 12:15:14Z ivanov $
+/*  $Id: bzip2.hpp 676281 2023-12-13 16:37:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -45,41 +45,6 @@
 BEGIN_NCBI_SCOPE
 
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Special compression parameters (description from bzip2 docs)
-//        
-// <verbosity>
-//    This parameter should be set to a number between 0 and 4 inclusive.
-//    0 is silent, and greater numbers give increasingly verbose
-//    monitoring/debugging output. If the library has been compiled with
-//    -DBZ_NO_STDIO, no such output will appear for any verbosity setting. 
-//
-// <work_factor> 
-//    Parameter work_factor controls how the compression phase behaves when
-//    presented with worst case, highly repetitive, input data.
-//    If compression runs into difficulties caused by repetitive data, the
-//    library switches from the standard sorting algorithm to a fallback
-//    algorithm. The fallback is slower than the standard algorithm by
-//    perhaps a factor of three, but always behaves reasonably, no matter
-//    how bad the input. Lower values of work_factor reduce the amount of
-//    effort the standard algorithm will expend before resorting to the
-//    fallback. You should set this parameter carefully; too low, and many
-//    inputs will be handled by the fallback algorithm and so compress
-//    rather slowly, too high, and your average-to-worst case compression
-//    times can become very large. The default value of 30 gives reasonable
-//    behaviour over a wide range of circumstances. Allowable values range
-//    from 0 to 250 inclusive. 0 is a special case, equivalent to using
-//    the default value of 30.
-//
-// <small_decompress> 
-//    If it is nonzero, the library will use an alternative decompression
-//    algorithm which uses less memory but at the cost of decompressing more
-//    slowly (roughly speaking, half the speed, but the maximum memory
-//    requirement drops to around 2300k).
-//
-
-
 //////////////////////////////////////////////////////////////////////////////
 ///
 /// CBZip2Compression --
@@ -90,27 +55,25 @@ BEGIN_NCBI_SCOPE
 class NCBI_XUTIL_EXPORT CBZip2Compression : public CCompression 
 {
 public:
+    /// Initialize compression  library (for API compatibility, bz2 don't need it).
+    static bool Initialize(void) { return true; };
+
     /// Compression/decompression flags.
     enum EFlags {
-        ///< Allow transparent reading data from buffer/file/stream
-        ///< regardless is it compressed or not. But be aware,
-        ///< if data source contains broken data and API cannot detect that
-        ///< it is compressed data, that you can get binary instead of
-        ///< decompressed data. By default this flag is OFF.
+        /// Allow transparent reading data from buffer/file/stream
+        /// regardless is it compressed or not. But be aware,
+        /// if data source contains broken data and API cannot detect that
+        /// it is compressed data, that you can get binary instead of
+        /// decompressed data. By default this flag is OFF.
         fAllowTransparentRead = (1<<0),
-        ///< Allow to "compress/decompress" empty data. 
-        ///< The output compressed data will have header and footer only.
+        /// Allow to "compress/decompress" empty data. 
+        /// The output compressed data will have header and footer only.
         fAllowEmptyData       = (1<<1)
     };
     typedef CBZip2Compression::TFlags TBZip2Flags; ///< Bitwise OR of EFlags
 
     /// Constructor.
-    CBZip2Compression(
-        ELevel level            = eLevel_Default,
-        int    verbosity        = 0,              // [0..4]
-        int    work_factor      = 0,              // [0..250] 
-        int    small_decompress = 0               // [0,1]
-    );
+    CBZip2Compression(ELevel level = eLevel_Default);
 
     /// Destructor.
     virtual ~CBZip2Compression(void);
@@ -120,18 +83,22 @@ public:
 
     /// Get compression level.
     ///
-    /// NOTE: BZip2 algorithm do not support zero level compression.
-    ///       So the "eLevel_NoCompression" will be translated to
-    ///       "eLevel_Lowest".
+    /// @note
+    ///   bzip2 doesn't support zero level compression, so eLevel_NoCompression
+    ///   will be translated to eLevel_Lowest.
     virtual ELevel GetLevel(void) const;
 
     /// Return default compression level for a compression algorithm
     virtual ELevel GetDefaultLevel(void) const
         { return eLevel_VeryHigh; };
 
-    //
+    /// Check if compression have support for a specified feature
+    virtual bool HaveSupport(ESupportFeature feature);
+
+
+    //=======================================================================
     // Utility functions 
-    //
+    //=======================================================================
 
     /// Compress data in the buffer.
     ///
@@ -182,22 +149,59 @@ public:
         /* out */            size_t* dst_len
     );
 
+    /// @warning No support for BZip2. Always return 0.
+    /// @sa HaveSupport
+    virtual size_t EstimateCompressionBufferSize(size_t) { return 0; };
+
+    /// Get recommended buffer sizes for stream/file I/O.
+    ///
+    /// These buffer sizes are softly recommended. They are not required, (de)compression
+    /// streams accepts any reasonable buffer size, for both input and output.
+    /// Respecting the recommended size just makes it a bit easier for (de)compressor,
+    /// reducing the amount of memory shuffling and buffering, resulting in minor 
+    /// performance savings. If compression library doesn't have preferences about 
+    /// I/O buffer sizes, kCompressionDefaultBufSize will be used.
+    /// @param round_up
+    ///   If specified, round up a returned value by specified amount. 
+    ///   Useful for better memory management. For example you can round up to virtual
+    ///   memory page size.
+    /// @return
+    ///   Structure with recommended buffer sizes.
+    /// @note
+    ///   Applicable for streaming/file operations.
+    /// @sa
+    ///   kCompressionDefaultBufSize, CSystemInfo::GetVirtualMemoryPageSize()
+    /// 
+    static SRecommendedBufferSizes GetRecommendedBufferSizes(size_t round_up = 0);
+
     /// Compress file.
     ///
     /// @param src_file
     ///   File name of source file.
     /// @param dst_file
     ///   File name of result file.
-    /// @param buf_size
-    ///   Buffer size used to read/write files.
+    /// @param file_io_bufsize
+    ///   Size of the buffer used to read from a source file. 
+    ///   Writing happens immediately on receiving some data from a compressor.
+    /// @param compression_in_bufsize
+    ///   Size of the internal buffer holding input data to be compressed.
+    ///   It can be different from 'file_io_bufsize' depending on a using 
+    ///   compression method, OS and file system.
+    /// @param compression_out_bufsize
+    ///   Size of the internal buffer to receive data from a compressor.
     /// @return
     ///   Return TRUE on success, FALSE on error.
+    /// @note
+    ///   This method don't store any file meta information like name, date/time, owner or attributes.
     /// @sa
-    ///   DecompressFile
+    ///   DecompressFile, GetRecommendedBufferSizes, CBZip2CompressionFile
+    /// 
     virtual bool CompressFile(
         const string& src_file,
         const string& dst_file,
-        size_t        buf_size = kCompressionDefaultBufSize
+        size_t        file_io_bufsize         = kCompressionDefaultBufSize,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     );
 
     /// Decompress file.
@@ -206,18 +210,83 @@ public:
     ///   File name of source file.
     /// @param dst_file
     ///   File name of result file.
-    /// @param buf_size
-    ///   Buffer size used to read/write files.
+    /// @param file_io_bufsize
+    ///   Size of the buffer used to read from a source file. 
+    ///   Writing happens immediately on receiving some data from a decompressor.
+    /// @param decompression_in_bufsize
+    ///   Size of the internal buffer holding input data to be decompressed.
+    ///   It can be different from 'file_io_bufsize' depending on a using 
+    ///   compression method, OS and file system.
+    /// @param decompression_out_bufsize
+    ///   Size of the internal buffer to receive data from a decompressor.
     /// @return
     ///   Return TRUE on success, FALSE on error.
     /// @sa
-    ///   CompressFile
+    ///   CompressFile, GetRecommendedBufferSizes, CBZip2CompressionFile
+    /// 
     virtual bool DecompressFile(
         const string& src_file,
         const string& dst_file, 
-        size_t        buf_size = kCompressionDefaultBufSize
+        size_t        file_io_bufsize           = kCompressionDefaultBufSize,
+        size_t        decompression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        decompression_out_bufsize = kCompressionDefaultBufSize
     );
 
+    /// @warning No dictionary support for bzip2. Always return FALSE.
+    /// @sa HaveSupport
+    virtual bool SetDictionary(
+        CCompressionDictionary& dict, 
+        ENcbiOwnership          own = eNoOwnership
+    );
+
+    //=======================================================================
+    // Advanced compression-specific parameters
+    //=======================================================================
+    // Allow to tune up (de)compression for a specific needs.
+    //
+    // - Pin down compression parameters to some specific values, so these
+    //   values are no longer dynamically selected by the compressor.
+    // - All setting parameters should be in the range [min,max], 
+    //   or equal to default.
+    // - All parameters should be set before starting (de)compression, 
+    //   or it will be ignored for current operation.
+    //=======================================================================
+
+    /// Work factor.
+    ///
+    /// This parameter controls how the compression phase behaves when
+    /// presented with worst case, highly repetitive, input data.
+    /// If compression runs into difficulties caused by repetitive data, 
+    /// the library switches from the standard sorting algorithm to a fallback
+    /// algorithm. The fallback is slower than the standard algorithm by
+    /// perhaps a factor of three, but always behaves reasonably, no matter
+    /// how bad the input. Lower values of work_factor reduce the amount of
+    /// effort the standard algorithm will expend before resorting to the
+    /// fallback. You should set this parameter carefully; too low, and many
+    /// inputs will be handled by the fallback algorithm and so compress
+    /// rather slowly, too high, and your average-to-worst case compression
+    /// times can become very large. The default value 30 gives reasonable
+    /// behaviour over a wide range of circumstances. Allowable values range
+    /// from 0 to 250 inclusive. 0 is a special case, equivalent to using
+    /// the default value of 30.
+    /// 
+    void SetWorkFactor(int work_factor) { m_c_WorkFactor = work_factor; }
+    int  GetWorkFactor(void) const { return m_c_WorkFactor; }
+    static int GetWorkFactorDefault(void);
+    static int GetWorkFactorMin(void);
+    static int GetWorkFactorMax(void);
+
+    /// Small decompress.
+    ///
+    /// If small decompress is set (TRUE), the library will use an alternative
+    /// decompression algorithm which uses less memory but at the cost of 
+    /// decompressing more slowly (roughly speaking, half the speed, but 
+    /// the maximum memory requirement drops to around 2300k).
+    ///
+    void SetSmallDecompress(bool small_decompres) { m_d_SmallDecompress =  small_decompres; }
+    bool GetSmallDecompress(void) const { return (bool)m_d_SmallDecompress; }
+    static bool GetSmallDecompressDefault(void) ;
+
 protected:
     /// Get error description for specified error code.
     const char* GetBZip2ErrorDescription(int errcode);
@@ -226,10 +295,9 @@ protected:
     string FormatErrorMessage(string where, bool use_stream_data = true) const;
 
 protected:
-    void*  m_Stream;          ///< Compressor stream
-    int    m_Verbosity;       ///< Verbose monitoring/debugging output level
-    int    m_WorkFactor;      ///< See description above
-    int    m_SmallDecompress; ///< Use memory-frugal decompression algorithm
+    void*  m_Stream;            ///< Compressor stream
+    int    m_c_WorkFactor;      ///< See description above
+    int    m_d_SmallDecompress; ///< Use memory-frugal decompression algorithm
 
 private:
     /// Private copy constructor to prohibit copy.
@@ -251,23 +319,27 @@ class NCBI_XUTIL_EXPORT CBZip2CompressionFile : public CBZip2Compression,
 {
 public:
     /// Constructor.
-    /// For a special parameters description see CBZip2Compression.
+    ///
+    /// Automatically calls Open() with given file name, mode and compression level.
+    /// @note
+    ///   This constructor don't allow to use any advanced compression parameters
+    ///   or a dictionary. If you need to set any of them, please use simplified
+    ///   conventional constructor, set advanced parameters and use Open().
+    /// @note
+    ///   Current implementation uses default bzip2 methods to work with compression
+    ///   files, so in/out buffer sizes are not used, and added for API uniformity only.
+    /// 
     CBZip2CompressionFile(
         const string& file_name,
         EMode         mode,
-        ELevel        level            = eLevel_Default,
-        int           verbosity        = 0,
-        int           work_factor      = 0,
-        int           small_decompress = 0 
+        ELevel        level = eLevel_Default,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     );
 
     /// Conventional constructor.
-    /// For a special parameters description see CBZip2Compression.
     CBZip2CompressionFile(
-        ELevel        level            = eLevel_Default,
-        int           verbosity        = 0,
-        int           work_factor      = 0,
-        int           small_decompress = 0 
+        ELevel        level = eLevel_Default
     );
 
     /// Destructor.
@@ -280,10 +352,22 @@ public:
     /// @param mode
     ///   File open mode.
     /// @return
-    ///   TRUE if file was opened succesfully or FALSE otherwise.
+    ///   TRUE if file was opened successfully or FALSE otherwise.
     /// @sa
     ///   CBZip2Compression, Read, Write, Close
-    virtual bool Open(const string& file_name, EMode mode);
+    /// @note
+    ///   All advanced compression parameters or a dictionary should be set before
+    ///   Open() method, otherwise they will not have any effect.
+    /// @note
+    ///   Current implementation uses default bzip2 methods to work with compression
+    ///   files, so in/out buffer sizes are not used, and added for API uniformity only.
+    /// 
+    virtual bool Open(
+        const string& file_name, 
+        EMode         mode,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
+    );
 
     /// Read data from compressed file.
     /// 
@@ -298,6 +382,7 @@ public:
     ///   The number of really read bytes can be less than requested.
     /// @sa
     ///   Open, Write, Close
+    ///
     virtual long Read(void* buf, size_t len);
 
     /// Write data to compressed file.
@@ -312,6 +397,7 @@ public:
     ///   Number of bytes actually written or -1 for error.
     /// @sa
     ///   Open, Read, Close
+    ///
     virtual long Write(const void* buf, size_t len);
 
     /// Close compressed file.
@@ -321,12 +407,13 @@ public:
     ///   TRUE on success, FALSE on error.
     /// @sa
     ///   Open, Read, Write
+    ///
     virtual bool Close(void);
 
 protected:
-    FILE*      m_FileStream;   ///< Underlying file stream
-    bool       m_EOF;          ///< EOF flag for read mode
-    bool       m_HaveData;     ///< Flag that we read/write some data
+    FILE*  m_FileStream;   ///< Underlying file stream
+    bool   m_EOF;          ///< EOF flag for read mode
+    bool   m_HaveData;     ///< Flag that we read/write some data
 
 private:
     /// Private copy constructor to prohibit copy.
@@ -350,15 +437,20 @@ class NCBI_XUTIL_EXPORT CBZip2Compressor : public CBZip2Compression,
 public:
     /// Constructor.
     CBZip2Compressor(
-        ELevel      level       = eLevel_Default,
-        int         verbosity   = 0,           // [0..4]
-        int         work_factor = 0,           // [0..250] 
-        TBZip2Flags flags       = 0
+        ELevel      level = eLevel_Default,
+        TBZip2Flags flags = 0
     );
 
     /// Destructor.
     virtual ~CBZip2Compressor(void);
 
+    /// Return TRUE if fAllowEmptyData flag is set. 
+    /// @note
+    ///   Used by stream buffer, that don't have access to specific
+    ///   compression implementation flags.
+    virtual bool AllowEmptyData() const
+        { return (GetFlags() & fAllowEmptyData) == fAllowEmptyData; }
+
 protected:
     virtual EStatus Init   (void);
     virtual EStatus Process(const char* in_buf,  size_t  in_len,
@@ -385,15 +477,18 @@ class NCBI_XUTIL_EXPORT CBZip2Decompressor : public CBZip2Compression,
 {
 public:
     /// Constructor.
-    CBZip2Decompressor(
-        int         verbosity        = 0,  // [0..4]
-        int         small_decompress = 0,  // [0,1]
-        TBZip2Flags flags            = 0
-    );
+    CBZip2Decompressor(TBZip2Flags flags = 0 );
 
     /// Destructor.
     virtual ~CBZip2Decompressor(void);
 
+    /// Return TRUE if fAllowEmptyData flag is set. 
+    /// @note
+    ///   Used by stream buffer, that don't have access to specific
+    ///   compression implementation flags.
+    virtual bool AllowEmptyData() const
+        { return (GetFlags() & fAllowEmptyData) == fAllowEmptyData; }
+
 protected:
     virtual EStatus Init   (void); 
     virtual EStatus Process(const char* in_buf,  size_t  in_len,
@@ -425,31 +520,36 @@ public:
         CBZip2Compression::ELevel level,
         streamsize                in_bufsize,
         streamsize                out_bufsize,
-        int                       verbosity   = 0,
-        int                       work_factor = 0,
         CBZip2Compression::TBZip2Flags flags  = 0
         )
         : CCompressionStreamProcessor(
-              new CBZip2Compressor(level, verbosity, work_factor, flags),
-              eDelete, in_bufsize, out_bufsize)
+              new CBZip2Compressor(level, flags), eDelete, in_bufsize, out_bufsize)
     {}
 
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CBZip2StreamCompressor(
         CBZip2Compression::ELevel level,
         CBZip2Compression::TBZip2Flags flags = 0
         )
         : CCompressionStreamProcessor(
-              new CBZip2Compressor(level, 0, 0, flags),
+              new CBZip2Compressor(level, flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
 
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CBZip2StreamCompressor(CBZip2Compression::TBZip2Flags flags = 0)
         : CCompressionStreamProcessor(
-              new CBZip2Compressor(CBZip2Compression::eLevel_Default, 0, 0, flags),
+              new CBZip2Compressor(CBZip2Compression::eLevel_Default, flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
+
+    /// Return a pointer to compressor.
+    /// Can be used mostly for setting an advanced compression-specific parameters.
+    CBZip2Compressor* GetCompressor(void) const {
+        return dynamic_cast<CBZip2Compressor*>(GetProcessor());
+    }
 };
 
 
@@ -468,21 +568,25 @@ public:
     CBZip2StreamDecompressor(
         streamsize                     in_bufsize,
         streamsize                     out_bufsize,
-        int                            verbosity        = 0,
-        int                            small_decompress = 0,
-        CBZip2Compression::TBZip2Flags flags            = 0
+        CBZip2Compression::TBZip2Flags flags = 0
         )
         : CCompressionStreamProcessor(
-             new CBZip2Decompressor(verbosity, small_decompress, flags),
-             eDelete, in_bufsize, out_bufsize)
+             new CBZip2Decompressor(flags), eDelete, in_bufsize, out_bufsize)
     {}
 
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CBZip2StreamDecompressor(CBZip2Compression::TBZip2Flags flags = 0)
         : CCompressionStreamProcessor( 
-              new CBZip2Decompressor(0, 0, flags),
+              new CBZip2Decompressor(flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
+
+    /// Return a pointer to compressor.
+    /// Can be used mostly for setting an advanced compression-specific parameters.
+    CBZip2Decompressor* GetDecompressor(void) const {
+        return dynamic_cast<CBZip2Decompressor*>(GetProcessor());
+    }
 };
 
 
