@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__COMPRESS__HPP
 #define UTIL_COMPRESS__COMPRESS__HPP
 
-/*  $Id: compress.hpp 592069 2019-08-26 15:27:03Z vasilche $
+/*  $Id: compress.hpp 658835 2022-11-09 13:48:43Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,6 +32,36 @@
  *
  */
 
+/// @file compress.hpp
+///
+/// @warning
+///   Compression API have support for some compression libraies. Toolkit checks
+///   existing compressions libraries on a configuration stage. If system version 
+///   of some compression library is not found, for some libraries like BZIP22, ZLIB, 
+///   GZIP we have an embedded alternatives, that will be used automatically. 
+///   But for some others, like LZO and ZSTD, we dont have such options.
+///   The configure define next macros, that can be used to put specific library
+///   related code to corresponding #if/#endif block to avoid compilation errors
+///   on some platforms that miss a specific compression library:
+/// 
+///   - HAVE_LIBZ       // for ZLIB and GZIP
+///   - HAVE_LIBBZ2     // for BZIP2
+///   - HAVE_LIBLZO     // for LZO
+///   - HAVE_LIBZSTD    // for ZSTD
+/// 
+///   #if defined(HAVE_LIB***)
+///       // use selected library related code here
+///   #else
+///      // some backup code, or error reporting
+///   #endif
+/// 
+///   Theoretically, you need a check for LZO and ZSTD only.
+/// 
+///   This is related to a library-specific classes only. Many other common compession 
+///   classes/interfaces can be used without such guards, like ICompression or streams,
+///   for example.
+
+
 #include <corelib/ncbistd.hpp>
 #include <corelib/version_api.hpp>
 
@@ -44,11 +74,13 @@
 
 BEGIN_NCBI_SCOPE
 
-
-/// Default compression I/O stream buffer size.
-const streamsize kCompressionDefaultBufSize = 16*1024;
-
 /// Macro to report errors in compression API.
+///
+/// Error codes for ERR_COMPRESS and OMPRESS_HANDLE_EXCEPTIONS are really
+/// a subcodes and current maximum  value is defined in 'include/util/error_codes.hpp':
+///     NCBI_DEFINE_ERRCODE_X(Util_Compress, 210, max);
+/// For new values use 'max'+1 and update it there.
+///
 #define ERR_COMPRESS(subcode, message) ERR_POST_X(subcode, Warning << message)
 
 /// Macro to catch and handle exceptions (from streams in the destructor)
@@ -75,10 +107,14 @@ const streamsize kCompressionDefaultBufSize = 16*1024;
         }                                                             \
     }                                                                 \
 
+/// Default compression I/O stream buffer size.
+const streamsize kCompressionDefaultBufSize = 16*1024;
+
 
 // Forward declaration
 class CCompressionFile;
 class CCompressionStreambuf;
+class CCompressionDictionary;
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -93,12 +129,19 @@ public:
     ///
     /// It is in range [0..9]. Increase of level might mean better compression
     /// and usualy greater time of compression. Usualy 1 gives best speed,
-    /// 9 gives best compression, 0 gives no compression at all.
+    /// 9 gives best compression, 0 gives no compression at all (if supported).
     /// eDefault value requests a compromise between speed and compression
     /// (according to developers of the corresponding compression algorithm).
+    /// @note
+    ///   Each compression library/algorithm have its own set of compression levels,
+    ///   than not usually fit into [0..9] range. But for convenience of use and 
+    ///   easier switching we prowide conversion between levels, whehe "lowest"
+    ///   is a always a minimem allowed compression level, and "best" is a maximum.
+    ///   All other approximately fits in between.
+    /// 
     enum ELevel {
         eLevel_Default       = -1,  // default
-        eLevel_NoCompression =  0,  // just store data
+        eLevel_NoCompression =  0,  // just store data (if supported, or eLevel_Lowest)
         eLevel_Lowest        =  1,
         eLevel_VeryLow       =  2,
         eLevel_Low           =  3,
@@ -139,15 +182,45 @@ public:
     virtual TFlags GetFlags(void) const = 0;
     virtual void   SetFlags(TFlags flags) = 0;
 
-    //
-    // Utility functions 
-    //
 
-    // (De)compress the source buffer into the destination buffer.
-    // Return TRUE on success, FALSE on error.
-    // The compressor error code can be acquired via GetErrorCode() call.
-    // Notice that altogether the total size of the destination buffer must
-    // be little more then size of the source buffer. 
+    //=======================================================================
+    // Supported features
+    //=======================================================================
+
+    /// Supported features
+    enum ESupportFeature {
+        /// Check if current compression method have a real support for 
+        /// eLevel_NoCompression, so it can store data without compression.
+        /// If this method of compression is not supported, the lowest
+        /// supported level will be used instead, automatically.
+        /// @sa GetLevel, ELevel
+        eFeature_NoCompression,
+
+        /// Check if current compression have support for dictionaries.
+        /// Without support SetDictionary() always return FALSE.
+        /// @sa SetDictionary
+        eFeature_Dictionary,
+
+        /// Check if current compression have implementation for
+        /// the EstimateCompressionBufferSize() method. 
+        /// Without support it always return 0.
+        /// @sa EstimateCompressionBufferSize, CompressBuffer
+        eFeature_EstimateCompressionBufferSize,
+    };
+
+    /// Check if compression have support for a specified feature
+    virtual bool HaveSupport(ESupportFeature feature) = 0;
+
+
+    //=======================================================================
+    // Utility functions 
+    //=======================================================================
+
+    /// (De)compress the source buffer into the destination buffer.
+    /// Return TRUE on success, FALSE on error.
+    /// The compressor error code can be acquired via GetErrorCode() call.
+    /// Notice that altogether the total size of the destination buffer must
+    /// be little more then size of the source buffer. 
     virtual bool CompressBuffer(
         const void* src_buf, size_t  src_len,
         void*       dst_buf, size_t  dst_size,
@@ -160,17 +233,30 @@ public:
         /* out */            size_t* dst_len
     ) = 0;
 
-    // (De)compress file "src_file" and put result to file "dst_file".
-    // Return TRUE on success, FALSE on error.
+    /// Estimate buffer size for data compression (if supported).
+    virtual size_t EstimateCompressionBufferSize(size_t src_len) = 0;
+
+    /// (De)compress file "src_file" and put result to file "dst_file".
+    /// Return TRUE on success, FALSE on error.
     virtual bool CompressFile(
-        const string&     src_file,
-        const string&     dst_file,
-        size_t            buf_size = kCompressionDefaultBufSize
+        const string& src_file,
+        const string& dst_file,
+        size_t        file_io_bufsize         = kCompressionDefaultBufSize,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     ) = 0;
     virtual bool DecompressFile(
-        const string&     src_file,
-        const string&     dst_file, 
-        size_t            buf_size = kCompressionDefaultBufSize
+        const string& src_file,
+        const string& dst_file, 
+        size_t        file_io_bufsize         = kCompressionDefaultBufSize,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
+    ) = 0;
+
+    /// Set a dictionary for all compression/decompression operations (if supported).
+    virtual bool SetDictionary(
+        CCompressionDictionary& dict, 
+        ENcbiOwnership          own = eNoOwnership
     ) = 0;
 };
 
@@ -190,42 +276,56 @@ public:
     /// Return name and version of the compression library.
     virtual CVersionInfo GetVersion(void) const = 0;
 
-    // Get/set compression level.
-    // NOTE 1:  Changing compression level after compression has begun will
-    //          be ignored.
-    // NOTE 2:  If the level is not supported by the underlying algorithm,
-    //          then it will be translated to the nearest supported value.
+    /// Get/set compression level.
+    /// NOTE 1:  Changing compression level after compression has begun will
+    ///          be ignored.
+    /// NOTE 2:  If the level is not supported by the underlying algorithm,
+    ///          then it will be translated to the nearest supported value.
     virtual void   SetLevel(ELevel level);
     virtual ELevel GetLevel(void) const;
 
-    // Get compressor's internal status/error code and description
-    // for the last operation.
+    /// Get compressor's internal status/error code and description
+    /// for the last operation.
     virtual int    GetErrorCode(void) const;
     virtual string GetErrorDescription(void) const;
 
-    /// Get flags.
+    /// Get/set flags.
     virtual TFlags GetFlags(void) const;
-    /// Set flags.
     virtual void   SetFlags(TFlags flags);
 
+    /// Structure to get information about recommended buffer sizes for 
+    /// file/stream I/O to tune up a (de)compression performance.
+    /// @sa GetRecommendedBufferSizes
+    struct SRecommendedBufferSizes {
+        size_t compression_in;     ///< compression: recommended size for input buffer
+        size_t compression_out;    ///< compression: recommended size for output buffer
+        size_t decompression_in;   ///< decompression: recommended size for input buffer
+        size_t decompression_out;  ///< decompression: recommended size for output buffer
+
+        // Round up buffer size to a value divisible by 'precision'
+        size_t RoundUp(size_t value, size_t precision);
+    };
+
 protected:
-    // Universal file compression/decompression functions.
-    // Return TRUE on success, FALSE on error.
+    /// Set last action error/status code and description
+    void SetError(int status, const char* description = 0);
+    void SetError(int status, const string& description);
+
+    /// Universal file compression function.
+    /// Return TRUE on success, FALSE on error.
     virtual bool x_CompressFile(
         const string&     src_file,
         CCompressionFile& dst_file,
-        size_t            buf_size = kCompressionDefaultBufSize
+        size_t            file_io_bufsize = kCompressionDefaultBufSize
     );
+    /// Universal file decompression function.
+    /// Return TRUE on success, FALSE on error.
     virtual bool x_DecompressFile(
         CCompressionFile& src_file,
         const string&     dst_file,
-        size_t            buf_size = kCompressionDefaultBufSize
+        size_t            file_io_bufsize = kCompressionDefaultBufSize
     );
 
-    // Set last action error/status code and description
-    void SetError(int status, const char* description = 0);
-    void SetError(int status, const string& description);
-
 protected:
     /// Decompression mode (see fAllowTransparentRead flag).
     enum EDecompressMode {
@@ -233,14 +333,17 @@ protected:
         eMode_Decompress,      ///< Generic decompression
         eMode_TransparentRead  ///< Transparent read, the data is uncompressed
     };
-    ///< Decompress mode (Decompress/TransparentRead/Unknown).
-    EDecompressMode m_DecompressMode;
+    EDecompressMode m_DecompressMode;  ///< Decompress mode (Decompress/TransparentRead/Unknown)
+
+    // Dictionary
+    CCompressionDictionary* m_Dict;    ///< Dictionary for compression/decompression
+    ENcbiOwnership          m_DictOwn; ///< Dictionary ownership
 
 private:
-    ELevel  m_Level;      // Compression level
-    int     m_ErrorCode;  // Last compressor action error/status
-    string  m_ErrorMsg;   // Last compressor action error message
-    TFlags  m_Flags;      // Bitwise OR of flags
+    ELevel  m_Level;      ///< Compression level
+    int     m_ErrorCode;  ///< Last compressor action error/status
+    string  m_ErrorMsg;   ///< Last compressor action error message
+    TFlags  m_Flags;      ///< Bitwise OR of flags
 
     // Friend classes
     friend class CCompressionStreambuf;
@@ -272,12 +375,16 @@ public:
 
     // 'ctors
     CCompressionFile(void);
-    CCompressionFile(const string& path, EMode mode); 
+    CCompressionFile(const string& path, EMode mode, 
+                     size_t compression_in_bufsize  = 0,
+                     size_t compression_out_bufsize = 0); 
     virtual ~CCompressionFile(void);
 
     /// Opens a compressed file for reading or writing.
     /// Return NULL if error has been occurred.
-    virtual bool Open(const string& path, EMode mode) = 0; 
+    virtual bool Open(const string& path, EMode mode, 
+                      size_t compression_in_bufsize  = 0,
+                      size_t compression_out_bufsize = 0) = 0; 
 
     /// Read up to "len" uncompressed bytes from the compressed file "file"
     /// into the buffer "buf". Return the number of bytes actually read
@@ -301,6 +408,51 @@ protected:
 
 
 
+//////////////////////////////////////////////////////////////////////////////
+//
+// CCompressionDictionary -- class to load/keep/manage compression dictionary data
+//
+
+class NCBI_XUTIL_EXPORT CCompressionDictionary
+{
+public:
+    /// Use dictionary data from a memory buffer.
+    /// CCompressionDictionary can take an ownership of allocated memory,
+    /// so it can be automatically deallocated when not needed anymore.
+    CCompressionDictionary(const void* buf, size_t size, ENcbiOwnership own = eNoOwnership);
+
+    /// Load a dictionary from file. 
+    /// Throw an CCompressionException on file opening error.
+    CCompressionDictionary(const string& filename);
+
+    /// Load a dictionary up to 'size' bytes from a stream 'is'.
+    /// Ideally, 'size' should be a real dictionary data size. 
+    /// You can use bigger sizes, but it will be less memory efficient.
+    CCompressionDictionary(istream& is, size_t size);
+
+    /// Destructor.
+    virtual ~CCompressionDictionary(void);
+
+    /// Return pointer to the dictionary data.
+    const void* GetData(void) { return m_Data; };
+
+    /// Return dictionary data size.
+    size_t GetSize(void) { return m_Size; };
+
+    /// Free used memory
+    void Free(void);
+
+protected:
+    size_t LoadFromStream(istream& is, size_t size);
+
+private:
+    const void*    m_Data;   ///< Pointer to the dictionary data
+    size_t         m_Size;   ///< Size of the data
+    ENcbiOwnership m_Own;    ///< Data ownership
+};
+
+
+
 //////////////////////////////////////////////////////////////////////////////
 //
 // CCompressionProcessor -- abstract base class
@@ -339,6 +491,13 @@ public:
     /// compression session.
     bool IsBusy(void) const;
 
+    /// Return TRUE if fAllowEmptyData flag is set for this compression. 
+    /// @note
+    ///   Used by stream buffer, that don't have access to specific
+    ///   compression implementation flags. So this method should be 
+    ///   implemented in each processor.
+    virtual bool AllowEmptyData() const = 0;
+
     // Return number of processed/output bytes.
     size_t GetProcessedSize(void);
     size_t GetOutputSize(void);
@@ -403,7 +562,7 @@ protected:
 private:
     size_t  m_ProcessedSize;  //< The number of processed bytes
     size_t  m_OutputSize;     //< The number of output bytes
-    bool    m_Busy;           //< Is true if compressor is ready to begin next session
+    bool    m_Busy;           //< TRUE if compressor is busy and not ready to begin next session
     // Friend classes
     friend class CCompressionStream;
     friend class CCompressionStreambuf;
@@ -502,6 +661,15 @@ public:
 //
 //===========================================================================
 
+inline
+size_t CCompression::SRecommendedBufferSizes::RoundUp(size_t value, size_t precision)
+{
+    if ( precision <= 1 ) {
+        return value;
+    }
+    return size_t(value / precision) * precision + ((value % precision > 0) ? precision : 0);
+}
+
 inline
 void CCompressionProcessor::Reset(void)
 {
@@ -551,6 +719,7 @@ size_t CCompressionProcessor::GetOutputSize(void)
 }
 
 
+
 END_NCBI_SCOPE
 
 
