@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__LZO__HPP
 #define UTIL_COMPRESS__LZO__HPP
 
-/*  $Id: lzo.hpp 667989 2023-05-24 13:59:46Z ivanov $
+/*  $Id: lzo.hpp 676281 2023-12-13 16:37:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -167,13 +167,6 @@ public:
     /// Constructor.
     CLZOCompression(ELevel level = eLevel_Default);
 
-    /// @deprecated 
-    ///   Use CLZOCompression(ELevel) constructor without block size, that can be set separately
-    NCBI_DEPRECATED_CTOR(CLZOCompression(
-        ELevel level,
-        size_t blocksize
-    ));
-
     /// Destructor.
     virtual ~CLZOCompression(void);
 
@@ -480,10 +473,6 @@ class NCBI_XUTIL_EXPORT CLZOCompressionFile : public CLZOCompression,
                                               public CCompressionFile
 {
 public:
-    // TODO: add compression_in_bufsize / compression_out_bufsize parameters after 
-    // removing deprecated constructors only, to avoid conflicts. See zst as example.
-    // JIRA: CXX-12640
-
     /// Constructor.
     ///
     /// Automatically calls Open() with given file name, mode and compression level.
@@ -495,24 +484,14 @@ public:
     CLZOCompressionFile(
         const string& file_name,
         EMode         mode,
-        ELevel        level = eLevel_Default
+        ELevel        level = eLevel_Default,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     );
     /// Conventional constructor.
     CLZOCompressionFile(
         ELevel        level = eLevel_Default
     );
-    /// @deprecated Use CLZOCompressionFile(const string&, EMode, ELevel) construtor instead.
-    NCBI_DEPRECATED_CTOR(CLZOCompressionFile(
-        const string& file_name,
-        EMode         mode,
-        ELevel        level,
-        size_t        blocksize
-    ));
-    /// @deprecated Use CLZOCompressionFile(ELevel) construtor instead.
-    NCBI_DEPRECATED_CTOR(CLZOCompressionFile(
-        ELevel        level,
-        size_t        blocksize
-    ));
 
     /// Destructor
     ~CLZOCompressionFile(void);
@@ -689,14 +668,6 @@ public:
         ELevel    level = eLevel_Default,
         TLZOFlags flags = 0
     );
-    /// @deprecated 
-    ///   Use CLZOCompressor(ELevel = eLevel_Default, TLZOFlags = 0) constructor
-    ///   without block size parameter, that can be set separately if necessary.
-    NCBI_DEPRECATED_CTOR(CLZOCompressor(
-        ELevel    level,
-        size_t    blocksize,
-        TLZOFlags flags
-    ));
 
     /// Destructor.
     virtual ~CLZOCompressor(void);
@@ -748,14 +719,6 @@ public:
     /// Constructor.
     CLZODecompressor(TLZOFlags flags);
 
-    /// @deprecated 
-    ///   Use CLZODecompressor(TLZOFlags = 0) constructor
-    ///   without block size parameter, that can be set separately if necessary.
-    NCBI_DEPRECATED_CTOR(CLZODecompressor(
-        size_t    blocksize,
-        TLZOFlags flags
-    ));
-
     /// Destructor.
     virtual ~CLZODecompressor(void);
 
@@ -819,18 +782,8 @@ public:
               new CLZOCompressor(level, flags), eDelete, in_bufsize, out_bufsize)
     {}
 
-    /// @deprecated 
-    ///   Use CLZOStreamCompressor() constructor
-    ///   without block size parameter, that can be set separately if necessary.
-    NCBI_DEPRECATED_CTOR(CLZOStreamCompressor(
-        CLZOCompression::ELevel    level,
-        streamsize                 in_bufsize,
-        streamsize                 out_bufsize,
-        size_t                     blocksize,
-        CLZOCompression::TLZOFlags flags
-    ));
-
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CLZOStreamCompressor(
         CLZOCompression::ELevel    level,
         CLZOCompression::TLZOFlags flags = 0
@@ -840,7 +793,8 @@ public:
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
 
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CLZOStreamCompressor(CLZOCompression::TLZOFlags flags = 0)
         : CCompressionStreamProcessor(
               new CLZOCompressor(CLZOCompression::eLevel_Default, flags),
@@ -879,17 +833,8 @@ public:
              new CLZODecompressor(flags), eDelete, in_bufsize, out_bufsize)
     {}
 
-    /// @deprecated 
-    ///   Use CLZOStreamDecompressor() constructor
-    ///   without block size parameter, that can be set separately if necessary.
-    NCBI_DEPRECATED_CTOR(CLZOStreamDecompressor(
-        streamsize                 in_bufsize,
-        streamsize                 out_bufsize,
-        size_t                     blocksize,
-        CLZOCompression::TLZOFlags flags
-    ));
-
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CLZOStreamDecompressor(CLZOCompression::TLZOFlags flags = 0)
         : CCompressionStreamProcessor( 
               new CLZODecompressor(flags),
