@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__TAR__HPP
 #define UTIL_COMPRESS__TAR__HPP
 
-/* $Id: tar.hpp 587245 2019-05-31 23:39:51Z lavr $
+/* $Id: tar.hpp 684843 2024-07-03 15:40:31Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -224,6 +224,9 @@ public:
     Uint8         GetPosition(EPos which)    const
     { return which == ePos_Header ? m_Pos : m_Pos + m_HeaderSize; }
 
+    // Non-empty only if filesystem was involed (append/extract/update)
+    const string& GetPath(void)              const { return m_Path; }
+
     // Comparison operator.
     bool operator == (const CTarEntryInfo& info) const
     { return (m_Type       == info.m_Type                        &&
@@ -250,6 +253,8 @@ protected:
     CDirEntry::SStat m_Stat;       ///< Direntry-compatible info
     Uint8            m_Pos;        ///< Entry (not data!) position in archive
 
+    string           m_Path;       ///< Iff filesystem involved in processing
+
     friend class CTar;             // Setter
 };
 
@@ -344,12 +349,18 @@ public:
         fSlowSkipWithRead   = (1<<21),
 
         // --- Miscellaneous ---
+        /// Ignore case difference in names
+        fIgnoreNameCase     = (1<<24),
+        /// Conflict overwrite allowed for entries
+        fConflictOverwrite  = (1<<25),
+
         /// Stream tar data through
-        fStreamPipeThrough  = (1<<24),
+        fStreamPipeThrough  = (1<<28),
         /// Do not trim tar file size after append/update
-        fTarfileNoTruncate  = (1<<26),
+        fTarfileNoTruncate  = (1<<29),
+
         /// Suppress NCBI signatures in entry headers
-        fStandardHeaderOnly = (1<<28),
+        fStandardHeaderOnly = (1<<30),
 
         /// Default flags
         fDefault            = fOverwrite | fPreserveAll
@@ -432,8 +443,8 @@ public:
     /// MS-Windows, stripped).  All entries will be added at the logical end
     /// (not always EOF) of the archive, when appending to a non-empty one.
     ///
-    /// @note Adding to a stream archive does not seek to the logical end of
-    /// the archive but begins at the current position right away.
+    /// @warning Adding to a stream archive does not seek to the logical end
+    /// of the archive but begins at the current position right away.
     ///
     /// @return
     ///   A list of entries appended.
@@ -443,7 +454,8 @@ public:
 
     /// Append an entry from a stream (exactly entry.GetSize() bytes).
     /// @note
-    ///   Name masks (if any set with SetMask()) are all ignored.
+    ///   The base directory and name masks (if any set with SetBaseDir() or
+    ///   SetMask(), respectively) are all ignored.
     /// @return
     ///   A list (containing this one entry) with full archive info filled in
     /// @sa
@@ -480,8 +492,8 @@ public:
     /// If the same-named files exist, they will be replaced (subject to
     /// fOverwrite) or backed up (fBackup), unless fUpdate is set, which would
     /// cause the replacement / backup only if the files are older than the
-    /// archive entries.  Note that if fOverwrite is stripped, no matching
-    /// files will be updated / backed up / overwritten, but skipped.
+    /// archive entries.  Note that if the fOverwrite bit is stripped, then no
+    /// matching files will be updated / backed up / overwritten, but skipped.
     ///
     /// Extract all archive entries, whose names match the pre-set mask.
     /// @note
@@ -747,7 +759,7 @@ private:
     const char* x_ReadArchive (size_t& n);
     void        x_WriteArchive(size_t  n, const char* buffer = 0);
 
-    // Append an entry from the file system to the archive.
+    // Append an entry from the file system to the archive ("toc" when update).
     unique_ptr<TEntries> x_Append(const string& name, const TEntries* toc = 0);
 
     // Append an entry from an istream to the archive.
