@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__ZLIB__HPP
 #define UTIL_COMPRESS__ZLIB__HPP
 
-/*  $Id: zlib.hpp 591883 2019-08-21 19:15:07Z lavr $
+/*  $Id: zlib.hpp 676281 2023-12-13 16:37:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,8 @@
  */
 
 /// @file zlib.hpp
-/// ZLib Compression API.
+///
+/// ZLib Compression API
 ///
 /// CZipCompression        - base methods for compression/decompression
 ///                          memory buffers and files.
@@ -61,36 +62,8 @@
 BEGIN_NCBI_SCOPE
 
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Special compressor's parameters (description from zlib docs)
-//        
-// <window_bits>
-//    This parameter is the base two logarithm of the window size
-//    (the size of the history buffer). It should be in the range 8..15 for
-//    this version of the library. Larger values of this parameter result
-//    in better compression at the expense of memory usage. 
-//
-// <mem_level> 
-//    The "mem_level" parameter specifies how much memory should be
-//    allocated for the internal compression state. mem_level=1 uses minimum
-//    memory but is slow and reduces compression ratio; mem_level=9 uses
-//    maximum memory for optimal speed. The default value is 8. See zconf.h
-//    for total memory usage as a function of windowBits and memLevel.
-//
-// <strategy> 
-//    The strategy parameter is used to tune the compression algorithm.
-//    Use the value Z_DEFAULT_STRATEGY for normal data, Z_FILTERED for data
-//    produced by a filter (or predictor), or Z_HUFFMAN_ONLY to force
-//    Huffman encoding only (no string match). Filtered data consists mostly
-//    of small values with a somewhat random distribution. In this case,
-//    the compression algorithm is tuned to compress them better. The effect
-//    of Z_FILTERED is to force more Huffman coding and less string matching;
-//    it is somewhat intermediate between Z_DEFAULT and Z_HUFFMAN_ONLY.
-//    The strategy parameter only affects the compression ratio but not the
-//    correctness of the compressed output even if it is not set appropriately.
-
 // Use default values, defined in zlib library
+// @deprecated Please don't use, will be deleted later
 const int kZlibDefaultWbits       = -1;
 const int kZlibDefaultMemLevel    = -1;
 const int kZlibDefaultStrategy    = -1;
@@ -107,6 +80,9 @@ const int kZlibDefaultCompression = -1;
 class NCBI_XUTIL_EXPORT CZipCompression : public CCompression
 {
 public:
+    /// Initialize compression  library (for API compatibility, zlib don't need it).
+    static bool Initialize(void) { return true; };
+
     /// Compression/decompression flags.
     enum EFlags {
         /// Allow transparent reading data from buffer/file/stream
@@ -151,12 +127,9 @@ public:
     typedef CZipCompression::TFlags TZipFlags; ///< Bitwise OR of EFlags
 
     /// Constructor.
-    CZipCompression(
-        ELevel level       = eLevel_Default,
-        int    window_bits = kZlibDefaultWbits,     // [8..15]
-        int    mem_level   = kZlibDefaultMemLevel,  // [1..9] 
-        int    strategy    = kZlibDefaultStrategy   // [0..2]
-    );
+    /// @note 
+    ///   For setting up advanced compression parameters see Set*() methods.
+    CZipCompression(ELevel level = eLevel_Default);
 
     /// Destructor.
     virtual ~CZipCompression(void);
@@ -166,11 +139,15 @@ public:
 
     /// Returns default compression level for a compression algorithm.
     virtual ELevel GetDefaultLevel(void) const
-        { return ELevel(kZlibDefaultCompression); };
+        { return ELevel(eLevel_Default); };
 
-    //
+    /// Check if compression have support for a specified feature
+    virtual bool HaveSupport(ESupportFeature feature);
+
+
+    //=======================================================================
     // Utility functions 
-    //
+    //=======================================================================
 
     /// Compress data in the buffer.
     ///
@@ -234,15 +211,37 @@ public:
     /// specified number of bytes of data using the CompressBuffer() function.
     /// This function may return a conservative value that may be larger
     /// than 'src_len'. 
+    ///
     /// @param src_len
-    ///   Size of compressed data.
+    ///   Size of data in source buffer.
     /// @return
-    ///   Estimated buffer size.
-    ///   Return -1 on error, or if this method is not supported by current
-    ///   version of the zlib library. 
+    ///   Estimated buffer size. 0 if unable to determine.
+    /// @note
+    ///   This method ignores used dictionary.
     /// @sa
     ///   CompressBuffer
-    long EstimateCompressionBufferSize(size_t src_len);
+    virtual size_t EstimateCompressionBufferSize(size_t src_len);
+
+    /// Get recommended buffer sizes for stream/file I/O.
+    ///
+    /// These buffer sizes are softly recommended. They are not required, (de)compression
+    /// streams accepts any reasonable buffer size, for both input and output.
+    /// Respecting the recommended size just makes it a bit easier for (de)compressor,
+    /// reducing the amount of memory shuffling and buffering, resulting in minor 
+    /// performance savings. If compression library doesn't have preferences about 
+    /// I/O buffer sizes, kCompressionDefaultBufSize will be used.
+    /// @param round_up_by
+    ///   If specified, round up a returned value by specified amount. 
+    ///   Useful for better memory management. For example you can round up to virtual
+    ///   memory page size.
+    /// @return
+    ///   Structure with recommended buffer sizes.
+    /// @note
+    ///   Applicable for streaming/file operations.
+    /// @sa
+    ///   kCompressionDefaultBufSize, CSystemInfo::GetVirtualMemoryPageSize()
+    /// 
+    static SRecommendedBufferSizes GetRecommendedBufferSizes(size_t round_up = 0);
 
     /// Compress file.
     ///
@@ -250,29 +249,38 @@ public:
     ///   File name of source file.
     /// @param dst_file
     ///   File name of result file.
-    /// @param buf_size
-    ///   Buffer size used to read/write files.
+    /// @param file_io_bufsize
+    ///   Size of the buffer used to read from a source file. 
+    ///   Writing happens immediately on receiving some data from a compressor.
+    /// @param compression_in_bufsize
+    ///   Size of the internal buffer holding input data to be compressed.
+    ///   It can be different from 'file_io_bufsize' depending on a using 
+    ///   compression method, OS and file system.
+    /// @param compression_out_bufsize
+    ///   Size of the internal buffer to receive data from a compressor.
     /// @return
     ///   Return TRUE on success, FALSE on error.
-    /// @sa
-    ///   DecompressFile, DecompressFileIntoDir
     /// @note
-    ///   This method, as well as some gzip utilities, always
-    ///   keeps the original file name and timestamp in
-    ///   the compressed file. On this moment DecompressFile()
-    ///   do not use original file name at all, but be aware...
-    ///   If you assign different base name to destination
+    ///   This method, as well as some gzip utilities, always keeps the original
+    ///   file name and timestamp in the compressed file. On this moment 
+    ///   DecompressFile() method do not use original file name at all, 
+    ///   but be aware... If you assign different base name to destination
     ///   compressed file, that behavior of decompression utilities
-    ///   on different platforms may differ.
-    ///   For example, WinZip on MS Windows always restore
-    ///   original file name and timestamp stored in the file.
-    ///   UNIX gunzip have -N option for this, but by default
-    ///   do not use it, and just creates a decompressed file with
-    ///   the name of the compressed file without .gz extension.
+    ///   on different platforms may differ. For example, WinZip on MS Windows
+    ///   always restore original file name and timestamp stored in the file.
+    ///   UNIX gunzip have -N option for this, but by default do not use it,
+    ///   and just creates a decompressed file with the name of the compressed
+    ///   file without .gz extension.
+    /// @sa
+    ///   DecompressFile, DecompressFileIntoDir, GetRecommendedBufferSizes,
+    ///   CZipCompressionFile
+    /// 
     virtual bool CompressFile(
         const string& src_file,
         const string& dst_file,
-        size_t        buf_size = kCompressionDefaultBufSize
+        size_t        file_io_bufsize         = kCompressionDefaultBufSize,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     );
 
     /// Decompress file.
@@ -281,23 +289,32 @@ public:
     ///   File name of source file.
     /// @param dst_file
     ///   File name of result file.
-    /// @param buf_size
-    ///   Buffer size used to read/write files.
+    /// @param file_io_bufsize
+    ///   Size of the buffer used to read from a source file. 
+    ///   Writing happens immediately on receiving some data from a decompressor.
+    /// @param decompression_in_bufsize
+    ///   Size of the internal buffer holding input data to be decompressed.
+    ///   It can be different from 'file_io_bufsize' depending on a using 
+    ///   compression method, OS and file system.
+    /// @param decompression_out_bufsize
+    ///   Size of the internal buffer to receive data from a decompressor.
     /// @return
     ///   Return TRUE on success, FALSE on error.
     /// @sa
-    ///   CompressFile, DecompressFileIntoDir
+    ///   CompressFile, DecompressFileIntoDir, GetRecommendedBufferSizes, CZipCompressionFile
     /// @note
-    ///   CompressFile() method, as well as some gzip utilities,
-    ///   always keeps the original file name and timestamp in
-    ///   the compressed file. If fRestoreFileAttr flag is set,
-    ///   that time stamp, stored in the file header will be restored.
-    ///   The original file name cannot be restored here,
+    ///   CompressFile() method, as well as some gzip utilities, always keeps
+    ///   the original file name and timestamp in the compressed file. 
+    ///   If fRestoreFileAttr flag is set, that timestamp, stored in the file
+    ///   header will be restored. The original file name cannot be restored here,
     ///   see DecompressFileIntoDir().
+    /// 
     virtual bool DecompressFile(
         const string& src_file,
         const string& dst_file, 
-        size_t        buf_size = kCompressionDefaultBufSize
+        size_t        file_io_bufsize           = kCompressionDefaultBufSize,
+        size_t        decompression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        decompression_out_bufsize = kCompressionDefaultBufSize
     );
 
     /// Decompress file into specified directory.
@@ -306,23 +323,32 @@ public:
     ///   File name of source file.
     /// @param dst_dir
     ///   Destination directory.
-    /// @param buf_size
-    ///   Buffer size used to read/write files.
+    /// @param file_io_bufsize
+    ///   Size of the buffer used to read from a source file. 
+    ///   Writing happens immediately on receiving some data from a decompressor.
+    /// @param decompression_in_bufsize
+    ///   Size of the internal buffer holding input data to be decompressed.
+    ///   It can be different from 'file_io_bufsize' depending on a using 
+    ///   compression method, OS and file system.
+    /// @param decompression_out_bufsize
+    ///   Size of the internal buffer to receive data from a decompressor.
     /// @return
     ///   Return TRUE on success, FALSE on error.
     /// @sa
-    ///   CompressFile, DecompressFile
+    ///   CompressFile, DecompressFile, GetRecommendedBufferSizes, CZipCompressionFile
     /// @note
-    ///   CompressFile() method, as well as some gzip utilities,
-    ///   always keeps the original file name and timestamp in
-    ///   the compressed file. If fRestoreFileAttr flag is set,
-    ///   that original file name and time stamp, stored in
-    ///   the file header will be restored. If not, that destination
+    ///   CompressFile() method, as well as some gzip utilities, always keeps
+    ///   the original file name and timestamp in the compressed file.
+    ///   If fRestoreFileAttr flag is set, that original file name and timestamp,
+    ///   stored in the file header will be restored. If not, that destination
     ///   file will be named as archive name without extension.
+    ///
     virtual bool DecompressFileIntoDir(
         const string& src_file,
         const string& dst_dir, 
-        size_t        buf_size = kCompressionDefaultBufSize
+        size_t        file_io_bufsize           = kCompressionDefaultBufSize,
+        size_t        decompression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        decompression_out_bufsize = kCompressionDefaultBufSize
     );
 
     /// Structure to keep compressed file information.
@@ -333,18 +359,137 @@ public:
         SFileInfo(void) : mtime(0) {};
     };
 
+    /// Set a dictionary for all compression/decompression operations.
+    ///
+    /// Using dictionary can significantly reduce the size of the compressed data. 
+    /// Refer to the C++ documentation how to choose/prepare a dictionary.
+    /// 
+    /// @param dict
+    ///   Dictionary to use. New dictionary will be used for all subsequent 
+    ///   compression/decompression buffer and file operations. NULL value 
+    ///   invalidates previous dictionary, meaning "return to no-dictionary mode".
+    /// @param own
+    ///   If set to eTakeOwnership the dictionary will be owned by CCompression and 
+    ///   automatically deleted when necessary.
+    /// @return
+    ///   Return TRUE on success, FALSE on error. 
+    ///   FALSE usually mean that dictionaries are not supported for a current compression.
+    /// @note
+    ///   Each compression algorithm have its own dictionary format and cannot
+    ///   be reused by some other compression algorithm.
+    /// @note
+    ///   Same dictionary should be used to compress and decompress data.
+    /// @note
+    ///   .gz files don't store a dictionary inside, so you will be unable to decompress
+    ///   files created using CompressFile() or streams with an active dictionary using
+    ///   any external utilities like 'gunzip'. But they will be still decompressible 
+    ///   with DecompressFile() using the same dictionary.
+    /// @note
+    ///   If decompressed data have concatenated gzip files, and it is allowed to process
+    ///   them all, each gzip file should be compressed with the same dictionary.
+    ///   It is allowed to mix dictionary and non-dictionary compressed gzip files,
+    ///   the dictionary will be applied only when necessary.
+    /// @sa
+    ///   CompressBuffer, DecompressBuffer, CompressFile, DecompressFile
+    /// 
+    virtual bool SetDictionary(
+        CCompressionDictionary& dict, 
+        ENcbiOwnership          own = eNoOwnership
+    );
+
+    //=======================================================================
+    // Advanced compression-specific parameters
+    //=======================================================================
+    // Allow to tune up (de)compression for a specific needs.
+    //
+    // - Pin down compression parameters to some specific values, so these
+    //   values are no longer dynamically selected by the compressor.
+    // - All setting parameters should be in the range [min,max], 
+    //   or equal to default.
+    // - All parameters should be set before starting (de)compression, 
+    //   or it will be ignored for current operation.
+    //=======================================================================
+    // You can use listed Z_* parameters after #include <zlib.h>.
+
+    /// Compression strategy.
+    /// 
+    /// The strategy parameter is used to tune the compression algorithm.
+    /// - Z_DEFAULT_STRATEGY 
+    ///     for normal data;
+    /// - Z_HUFFMAN_ONLY 
+    ///     to force Huffman encoding only (no string match);
+    /// - Z_RLE 
+    ///     run-length encoding (RLE) compression;
+    /// - Z_FILTERED 
+    ///     for data produced by a filter (or predictor);
+    ///     Filtered data consists mostly of small values with a somewhat
+    ///     random distribution. In this case, the compression algorithm 
+    ///     is tuned to compress them better. The effect of Z_FILTERED is 
+    ///     to force more Huffman coding and less string matching; 
+    ///     it is somewhat intermediate between Z_DEFAULT and Z_HUFFMAN_ONLY.
+    /// - Z_FIXED
+    ///     prevents the use of dynamic Huffman codes, allowing for a simpler
+    ///     decoder for special applications;
+    /// 
+    /// The strategy parameter only affects the compression ratio but not the
+    /// correctness of the compressed output even if it is not set appropriately.
+    /// Used for compression only.
+    /// 
+    void SetStrategy(int strategy) { 
+        m_c_Strategy = (strategy == kZlibDefaultStrategy ) ? GetStrategyDefault() : strategy; 
+    }
+    int  GetStrategy(void) const { return m_c_Strategy; }
+    static int GetStrategyDefault(void);
+    static int GetStrategyMin(void);
+    static int GetStrategyMax(void);
+
+    /// Memory level.
+    /// 
+    /// The "mem_level" parameter specifies how much memory should be
+    /// allocated for the internal compression state. Low levels uses
+    /// less memory but are slow and reduces compression ratio; maximum level
+    /// uses maximum memory for optimal speed. See zconf.h for total memory usage
+    /// as a function of windowBits and memLevel.
+    /// 
+    void SetMemoryLevel(int mem_level) { 
+        m_c_MemLevel = (mem_level == kZlibDefaultMemLevel) ? GetMemoryLevelDefault() : mem_level;
+    }
+    int  GetMemoryLevel(void) const { return m_c_MemLevel; }
+    static int GetMemoryLevelDefault(void);
+    static int GetMemoryLevelMin(void);
+    static int GetMemoryLevelMax(void);
+
+    /// Window bits.
+    ///
+    /// This parameter is the base two logarithm of the window size
+    /// (the size of the history buffer). Larger values of this parameter result
+    /// in better compression at the expense of memory usage.
+    /// Used for compression and decompression. By default it is set to a maximum
+    /// allowed values. Reducing windows bits from default can make to it unable 
+    /// to extract .gz files created by gzip.
+    /// @note
+    ///   API support positive values for this parameters only. RAW deflate
+    ///   data is processed by default, for gzip format we have a apecial 
+    ///   flags, see description for: fGZip, fCheckFileHeader, fWriteGZipFormat.
+    /// 
+    void SetWindowBits(int window_bits) { 
+        m_cd_WindowBits = (window_bits == kZlibDefaultWbits) ? GetWindowBitsDefault() : window_bits;
+    }
+    int  GetWindowBits(void) const { return m_cd_WindowBits; }
+    static int GetWindowBitsDefault(void);
+    static int GetWindowBitsMin(void);
+    static int GetWindowBitsMax(void);
+
 protected:
     /// Format string with last error description.
     /// If pos == 0, that use internal m_Stream's position to report.
     string FormatErrorMessage(string where, size_t pos = 0) const;
 
 protected:
-    void*  m_Stream;     ///< Compressor stream.
-    int    m_WindowBits; ///< The base two logarithm of the window size
-                         ///< (the size of the history buffer). 
-    int    m_MemLevel;   ///< The allocation memory level for the
-                         ///< internal compression state.
-    int    m_Strategy;   ///< The parameter to tune compression algorithm.
+    void*  m_Stream;        ///< Compressor stream.
+    int    m_cd_WindowBits; ///< The base two logarithm of the window size.
+    int    m_c_MemLevel;    ///< The allocation memory level for the compression.
+    int    m_c_Strategy;    ///< The parameter to tune up a compression algorithm.
 
 private:
     /// Private copy constructor to prohibit copy.
@@ -366,22 +511,23 @@ class NCBI_XUTIL_EXPORT CZipCompressionFile : public CZipCompression,
 {
 public:
     /// Constructor.
-    /// For a special parameters description see CZipCompression.
+    ///
+    /// Automatically calls Open() with given file name, mode and compression level.
+    /// @note
+    ///   This constructor don't allow to use any advanced compression parameters
+    ///   or a dictionary. If you need to set any of them, please use simplified
+    ///   conventional constructor, set advanced parameters and use Open().
+    /// 
     CZipCompressionFile(
         const string& file_name,
         EMode         mode,
-        ELevel        level       = eLevel_Default,
-        int           window_bits = kZlibDefaultWbits,
-        int           mem_level   = kZlibDefaultMemLevel,
-        int           strategy    = kZlibDefaultStrategy
+        ELevel        level = eLevel_Default,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     );
     /// Conventional constructor.
-    /// For a special parameters description see CZipCompression.
     CZipCompressionFile(
-        ELevel        level       = eLevel_Default,
-        int           window_bits = kZlibDefaultWbits,
-        int           mem_level   = kZlibDefaultMemLevel,
-        int           strategy    = kZlibDefaultStrategy
+        ELevel        level = eLevel_Default
     );
 
     /// Destructor
@@ -393,11 +539,24 @@ public:
     ///   File name of the file to open.
     /// @param mode
     ///   File open mode.
+    /// @param compression_in_bufsize
+    ///   Size of the internal buffer holding input data to be (de)compressed.
+    /// @param compression_out_bufsize
+    ///   Size of the internal buffer to receive data from a (de)compressor.
     /// @return
     ///   TRUE if file was opened successfully or FALSE otherwise.
     /// @sa
     ///   CZipCompression, Read, Write, Close
-    virtual bool Open(const string& file_name, EMode mode);
+    /// @note
+    ///   All advanced compression parameters or a dictionary should be set before
+    ///   Open() method, otherwise they will not have any effect.
+    /// 
+    virtual bool Open(
+        const string& file_name, 
+        EMode         mode,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
+    );
 
     /// Opens a compressed file for reading or writing.
     ///
@@ -410,11 +569,22 @@ public:
     ///   Pointer to file information structure. If it is not NULL,
     ///   that it will be used to get information about compressed file
     ///   in the read mode, and set it in the write mode for gzip files.
+    /// @param compression_in_bufsize
+    ///   Size of the internal buffer holding input data to be (de)compressed.
+    /// @param compression_out_bufsize
+    ///   Size of the internal buffer to receive data from a (de)compressor.
     /// @return
     ///   TRUE if file was opened successfully or FALSE otherwise.
     /// @sa
     ///   CZipCompression, Read, Write, Close
-    virtual bool Open(const string& file_name, EMode mode, SFileInfo* info);
+    ///
+    virtual bool Open(
+        const string& file_name, 
+        EMode         mode,
+        SFileInfo*    info,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
+    );
 
     /// Read data from compressed file.
     /// 
@@ -429,6 +599,7 @@ public:
     ///   The number of really read bytes can be less than requested.
     /// @sa
     ///   Open, Write, Close
+    ///
     virtual long Read(void* buf, size_t len);
 
     /// Write data to compressed file.
@@ -444,6 +615,7 @@ public:
     ///   Returned value can be less than "len".
     /// @sa
     ///   Open, Read, Close
+    ///
     virtual long Write(const void* buf, size_t len);
 
     /// Close compressed file.
@@ -453,6 +625,7 @@ public:
     ///   TRUE on success, FALSE on error.
     /// @sa
     ///   Open, Read, Write
+    ///
     virtual bool Close(void);
 
 protected:
@@ -486,12 +659,10 @@ class NCBI_XUTIL_EXPORT CZipCompressor : public CZipCompression,
 public:
     /// Constructor.
     CZipCompressor(
-        ELevel    level       = eLevel_Default,
-        int       window_bits = kZlibDefaultWbits,
-        int       mem_level   = kZlibDefaultMemLevel,
-        int       strategy    = kZlibDefaultStrategy,
-        TZipFlags flags       = 0
+        ELevel    level = eLevel_Default,
+        TZipFlags flags = 0
     );
+
     /// Destructor.
     virtual ~CZipCompressor(void);
 
@@ -500,6 +671,13 @@ public:
     /// Used for compression of gzip files.
     void SetFileInfo(const SFileInfo& info);
 
+    /// Return TRUE if fAllowEmptyData flag is set. 
+    /// @note
+    ///   Used by stream buffer, that don't have access to specific
+    ///   compression implementation flags.
+    virtual bool AllowEmptyData() const
+        { return (GetFlags() & fAllowEmptyData) == fAllowEmptyData; }
+
 protected:
     virtual EStatus Init   (void);
     virtual EStatus Process(const char* in_buf,  size_t  in_len,
@@ -534,13 +712,18 @@ class NCBI_XUTIL_EXPORT CZipDecompressor : public CZipCompression,
 {
 public:
     /// Constructor.
-    CZipDecompressor(
-        int       window_bits = kZlibDefaultWbits,
-        TZipFlags flags       = 0
-    );
+    CZipDecompressor(TZipFlags flags = 0);
+
     /// Destructor.
     virtual ~CZipDecompressor(void);
 
+    /// Return TRUE if fAllowEmptyData flag is set. 
+    /// @note
+    ///   Used by stream buffer, that don't have access to specific
+    ///   compression implementation flags.
+    virtual bool AllowEmptyData() const
+        { return (GetFlags() & fAllowEmptyData) == fAllowEmptyData; }
+
 protected:
     virtual EStatus Init   (void); 
     virtual EStatus Process(const char* in_buf,  size_t  in_len,
@@ -584,36 +767,36 @@ public:
         CZipCompression::ELevel    level,
         streamsize                 in_bufsize,
         streamsize                 out_bufsize,
-        int                        window_bits = kZlibDefaultWbits,
-        int                        mem_level   = kZlibDefaultMemLevel,
-        int                        strategy    = kZlibDefaultStrategy,
-        CZipCompression::TZipFlags flags       = 0
+        CZipCompression::TZipFlags flags = 0
         ) 
         : CCompressionStreamProcessor(
-              new CZipCompressor(level,window_bits,mem_level,strategy,flags),
-              eDelete, in_bufsize, out_bufsize)
+              new CZipCompressor(level, flags), eDelete, in_bufsize, out_bufsize)
     {}
 
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CZipStreamCompressor(
         CZipCompression::ELevel    level,
         CZipCompression::TZipFlags flags = 0
         )
         : CCompressionStreamProcessor(
-              new CZipCompressor(level, kZlibDefaultWbits,
-                                 kZlibDefaultMemLevel, kZlibDefaultStrategy,
-                                 flags),
+              new CZipCompressor(level, flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
 
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CZipStreamCompressor(CZipCompression::TZipFlags flags = 0)
         : CCompressionStreamProcessor(
-              new CZipCompressor(CZipCompression::eLevel_Default,
-                                 kZlibDefaultWbits, kZlibDefaultMemLevel,
-                                 kZlibDefaultStrategy, flags),
+              new CZipCompressor(CZipCompression::eLevel_Default, flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
+
+    /// Return a pointer to compressor.
+    /// Can be used mostly for setting an advanced compression-specific parameters.
+    CZipCompressor* GetCompressor(void) const {
+        return dynamic_cast<CZipCompressor*>(GetProcessor());
+    }
 };
 
 
@@ -637,20 +820,25 @@ public:
     CZipStreamDecompressor(
         streamsize                 in_bufsize,
         streamsize                 out_bufsize,
-        int                        window_bits = kZlibDefaultWbits,
-        CZipCompression::TZipFlags flags       = 0
+        CZipCompression::TZipFlags flags = 0
         )
         : CCompressionStreamProcessor( 
-              new CZipDecompressor(window_bits, flags),
-              eDelete, in_bufsize, out_bufsize)
+              new CZipDecompressor(flags), eDelete, in_bufsize, out_bufsize)
     {}
 
-    /// Conventional constructor
+    /// Conventional constructor.
+    /// Uses default buffer sizes for I/O, that can be not ideal for some scenarios.
     CZipStreamDecompressor(CZipCompression::TZipFlags flags = 0)
         : CCompressionStreamProcessor( 
-              new CZipDecompressor(kZlibDefaultWbits, flags),
+              new CZipDecompressor(flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
+
+    /// Return a pointer to decompressor.
+    /// Can be used mostly for setting an advanced compression-specific parameters.
+    CZipDecompressor* GetDecompressor(void) const {
+        return dynamic_cast<CZipDecompressor*>(GetProcessor());
+    }
 };
 
 
@@ -666,9 +854,12 @@ public:
 /// @param is
 ///   Opened input stream to scan (should be opened in binary mode).
 /// @param handler
-///   Call handler's IChunkHandler::OnChunk() method with positions 
+///   Call handler's IChunkHandler::OnChunk() method and pass position 
 ///   of each new gzip file inside a stream and size of uncompressed data
 ///   on that moment.
+/// @note
+///   This method don't support concatenated .gz files compressed with a dictionary.
+///
 NCBI_XUTIL_EXPORT
 void g_GZip_ScanForChunks(CNcbiIstream& is, IChunkHandler& handler);
 
