@@ -1203,7 +1203,8 @@ public:
 };
 
 #if !(defined(__COVERITY__) || defined(_MSC_VER) || defined(__INTEL_COMPILER))
-static_assert(std::is_pod<lmdb::val>::value, "lmdb::val must be a POD type");
+static_assert(std::is_standard_layout<lmdb::val>::value, "lmdb::val must be a standard-layout type");
+static_assert(std::is_trivial<lmdb::val>::value, "lmdb::val must be a trivial type");
 static_assert(sizeof(lmdb::val) == sizeof(MDB_val), "sizeof(lmdb::val) != sizeof(MDB_val)");
 #endif
 
