@@ -1,7 +1,7 @@
 #ifndef UTIL___SYNC_QUEUE__HPP
 #define UTIL___SYNC_QUEUE__HPP
 
-/*  $Id: sync_queue.hpp 622192 2020-12-21 19:08:05Z ivanov $
+/*  $Id: sync_queue.hpp 667214 2023-05-10 17:48:43Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -513,7 +513,7 @@ private:
     /// Current number of elements in the queue.
     /// Stored separately because some containers do not provide a size()
     /// method working in constant time
-    volatile TSize m_Size;
+    atomic<TSize> m_Size;
 
     /// Maximum size of the queue
     const TSize m_MaxSize;
@@ -534,7 +534,7 @@ private:
     mutable CAtomicCounter_WithAutoInit m_CntWaitNotFull;
 
     /// ID of the thread in which the queue has been locked by a guardian
-    mutable TThreadSystemID m_CurGuardTID;
+    mutable atomic<TThreadSystemID> m_CurGuardTID;
 
     /// Number of lockings of this queue with access guardians in one thread
     mutable int m_LockCount;
@@ -1246,6 +1246,27 @@ CSyncQueue<Type, Container, Traits>::CSyncQueue(TSize max_size)
 }
 
 
+template <class Type, class Container, class Traits>
+inline
+typename CSyncQueue<Type, Container, Traits>::TSize
+    CSyncQueue<Type, Container, Traits>::GetSize(void) const
+{
+    // GCC thread sanitizer says this is a data race. It was decided to
+    // consider it rather as timing for this particular case so the method is
+    // included into sanitizer suppressions
+    return m_Size;
+}
+
+
+template <class Type, class Container, class Traits>
+inline
+typename CSyncQueue<Type, Container, Traits>::TSize
+    CSyncQueue<Type, Container, Traits>::GetMaxSize(void) const
+{
+    return m_MaxSize;
+}
+
+
 template <class Type, class Container, class Traits>
 inline
 bool CSyncQueue<Type, Container, Traits>::IsEmpty(void) const
@@ -1448,7 +1469,7 @@ void CSyncQueue<Type, Container, Traits>::x_GuardedLock(
         // it's not because this happens only after wait on m_TrigLock's value
         // (to be reset to 0) and in x_GuardedUnlock it happens before
         // m_TrigLock's value raised to 1.
-        CThread::GetSystemID(&m_CurGuardTID);
+        m_CurGuardTID = GetCurrentThreadSystemID();
         m_LockCount = 1;
     }
 }
@@ -1477,30 +1498,7 @@ bool CSyncQueue<Type, Container, Traits>::x_IsGuarded(void) const
     if (m_CurGuardTID == TThreadSystemID(kThreadSystemID_None))
         return false;
 
-    TThreadSystemID thread_id;
-    CThread::GetSystemID(&thread_id);
-    return m_CurGuardTID == thread_id;
-}
-
-
-template <class Type, class Container, class Traits>
-inline
-typename CSyncQueue<Type, Container, Traits>::TSize
-    CSyncQueue<Type, Container, Traits>::GetSize(void) const
-{
-    // GCC thread sanitizer says this is a data race. It was decided to
-    // consider it rather as timing for this particular case so the method is
-    // included into sanitizer suppressions
-    return m_Size;
-}
-
-
-template <class Type, class Container, class Traits>
-inline
-typename CSyncQueue<Type, Container, Traits>::TSize
-    CSyncQueue<Type, Container, Traits>::GetMaxSize(void) const
-{
-    return m_MaxSize;
+    return m_CurGuardTID == GetCurrentThreadSystemID();
 }
 
 
