@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_exec_test.sh 676064 2023-12-07 19:04:25Z ivanov $
+# $Id: check_exec_test.sh 685622 2024-07-26 16:35:54Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -43,10 +43,15 @@ timestamp_file=`mktemp /tmp/check_exec_timestamp.XXXXXXXXXX`
 touch $timestamp_file
 trap "rm -f $tmp $timestamp_file" 0 1 2 15
 
-# Reinforce timeout
-# Note, we cannot set it to $timeout for MT-test, because
-# CPU time count for each thread and sum.
-ulimit -t `expr $timeout \* 25` > /dev/null 2>&1 || true
+# Reinforce timeout.
+# Note, we cannot set it just to $timeout for MT-test, because  CPU time
+# count for each thread and sum, so using multiplier.
+# CPU limits seems broken on ARM Darwin, so set it very high to allow MT tests to works.
+case `uname -a` in
+   CYGWIN* )      ;;
+   Darwin*arm64*) ulimit -t `expr $timeout \* 60` > /dev/null 2>&1 ;;
+   *)             ulimit -t `expr $timeout \* 3`  > /dev/null 2>&1 ;;
+esac
 
 # Use different kill on Unix and Cygwin
 case `uname -s` in
