@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_unix_cmake.sh 680027 2024-03-08 18:23:04Z ivanov $
+# $Id: check_make_unix_cmake.sh 689432 2024-10-28 18:10:27Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -534,10 +534,10 @@ if test "\$NCBI_CHECK_SETLIMITS" != "0"  -a  \$is_max_debug = false; then
    ulimit -n 8192
    if [ \$cygwin = false ]; then
        if test "\$NCBI_CHECK_TOOLS" = "regular"; then
-          ulimit -v 110000000
+          ulimit -v 200000000
        else
           # Increase memory limits if run under check tool
-          ulimit -v 150000000
+          ulimit -v 250000000
        fi
    fi
 fi
@@ -646,9 +646,13 @@ RunTest()
     if test -n "\$x_files"; then
         for i in \$x_files ; do
             x_copy="\$root_dir/src/\$x_work_dir_tail/\$i"
+            x_copy_alt="\$root_dir/\$x_work_dir_tail/\$i"
             if test -f "\$x_copy"  -o  -d "\$x_copy"; then
                 cp -rf "\$x_copy" "\$x_work_dir"
                 test -d "\$x_work_dir/\$i" &&  find "\$x_work_dir/\$i" -name .svn -print | xargs rm -rf
+            elif test -f "\$x_copy_alt"  -o  -d "\$x_copy_alt"; then
+                cp -rf "\$x_copy_alt" "\$x_work_dir"
+                test -d "\$x_work_dir/\$i" &&  find "\$x_work_dir/\$i" -name .svn -print | xargs rm -rf
             else
                 echo "[\$x_work_dir_tail] \$x_name: Warning:  The copied object \"\$x_copy\" should be a file or directory!"
                 continue
