@@ -1,7 +1,11 @@
-# $Id: ncbicxx_build_info.py 675074 2023-11-14 13:33:27Z ivanov $
+# $Id: ncbicxx_build_info.py 696192 2025-04-09 15:39:03Z ivanov $
 import ast
 from datetime import date, datetime, timedelta, timezone
-from distutils.sysconfig import parse_makefile, expand_makefile_vars
+try:
+    from distutils.sysconfig import parse_makefile, expand_makefile_vars
+except ImportError:
+    from sysconfig import expand_makefile_vars
+    from sysconfig.__main__ import _parse_makefile as parse_makefile
 import hashlib
 import mmap
 import os
@@ -327,7 +331,7 @@ class Collector(object):
             url = url.rstrip('\n')
         except subprocess.CalledProcessError:
             try:
-                cmd = [git, 'remote', 'show', 'origin'] 
+                cmd = [git, 'remote', 'show', '-n', 'origin'] 
                 with subprocess.Popen(cmd, stdout = subprocess.PIPE,
                                       stderr = subprocess.DEVNULL,
                                       universal_newlines = True,
