@@ -1,4 +1,4 @@
-// $Id: project_utilits.js 588746 2019-06-28 18:09:08Z gouriano $
+// $Id: project_utilits.js 635009 2021-07-26 14:39:58Z gouriano $
 ////////////////////////////////////////////////////////////////////////////////////
 // Shared part of new_project.wsf and import_project.wsf
 // global settings
@@ -12,9 +12,9 @@ var g_branch     = "toolkit/trunk/internal/c++";
 //var g_def_branch = "toolkit/trunk/c++";
 //var g_branch     = "toolkit/trunk/c++";
 
-// valid:   "150", "150x64", "140", "140x64"
-var g_def_msvcver = "150x64";
-var g_msvcver     = "150x64";
+// valid:   "160x64", "150", "150x64", "140", "140x64"
+var g_def_msvcver = "160x64";
+var g_msvcver     = "160x64";
 
 ////////////////////////////////////////////////////////////////////////////////////
 // Utility functions :
@@ -496,6 +496,7 @@ function SetMsvcVer(oArgs, flag)
 				&& msvcver != "120" && msvcver != "120x64"
 				&& msvcver != "140" && msvcver != "140x64"
 				&& msvcver != "150" && msvcver != "150x64"
+				                    && msvcver != "160x64"
            ) {
             WScript.Echo("ERROR: Unknown version of MSVC requested: " + msvcver);
             WScript.Quit(1);    
@@ -518,7 +519,10 @@ function GetMsvcFolder()
     if (g_msvcver == "150" || g_msvcver == "150x64") {
         return "vs2017";
     }
-    return "vs2017";
+    if (g_msvcver == "160x64") {
+        return "vs2019";
+    }
+    return "vs2019";
 }
 
 function GetFlaggedValue(oArgs, flag, default_val)
@@ -596,6 +600,8 @@ function GetDefaultSuffix()
         s = "vs2017";
     } else if (g_msvcver == "150x64") {
         s = "vs2017.64";
+    } else if (g_msvcver == "160x64") {
+        s = "vs2019.64";
     }
     return s;
 }
@@ -622,15 +628,17 @@ function GetPtbTargetSolutionArgs(oShell, ptb)
         s = " -ide 1500 -arch Win32";
     } else if (g_msvcver == "150x64") {
         s = " -ide 1500 -arch x64";
+    } else if (g_msvcver == "160x64") {
+        s = " -ide 1600 -arch x64";
     } else {
-        s = " -ide 1500 -arch x64";
+        s = " -ide 1600 -arch x64";
     }
     return s;
 }
 function GetTargetPlatform()
 {
-    if (g_msvcver == "110x64" ||
-        g_msvcver == "120x64" || g_msvcver == "140x64" || g_msvcver == "150x64") {
+    if (g_msvcver == "120x64" ||
+        g_msvcver == "140x64" || g_msvcver == "150x64" || g_msvcver == "160x64") {
         return "x64";
     }
     return "Win32";
