@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# $Id: post_build.py 626774 2021-03-03 22:13:03Z ucko $
+# $Id: post_build.py 693160 2025-02-06 20:30:16Z ucko $
 
 import glob, os, shutil, subprocess, sys
 
@@ -8,7 +8,10 @@ common_stems  = ('python_ncbi_dbapi', 'ncbi_xdbapi_*')
 unix_stems    = ('dbapi', 'dbapi_driver', 'ct_ftds*', 'tds_ftds*',
                  'xconne?t', 'xutil', 'xncbi')
 windows_stems = ('ncbi_core', 'ncbi_dbapi', 'ncbi_dbapi_driver')
-installations = ('3.7', '3.8', '3.9')
+installations = ['3.9', '3.11']
+
+if os.path.isdir('/opt/python-3.13'):
+    installations.append('3.13')
 
 os.mkdir(install_dir + '/lib')
 if os.path.exists(lib_dir + '/python_ncbi_dbapi.so'):
@@ -34,8 +37,8 @@ if sys.platform.startswith('linux'):
         if os.path.isdir(tmp_dir):
             shutil.rmtree(tmp_dir)
         os.mkdir(tmp_dir)
-        if 0 != subprocess.call((py_bin_dir + '/pip', 'install', '-t', tmp_dir,
-                                 'packit')):
+        if 0 != subprocess.call((py_bin_dir + '/pip' + x,
+                                 'install', '-t', tmp_dir, 'packit')):
             raise RuntimeError('Error installing packit')
         for x in ('python_ncbi_dbapi.cpp', 'python_ncbi_dbapi.hpp',
                   'setup.py'):
