@@ -1,4 +1,4 @@
-/*  $Id: blast_options_builder.cpp 617780 2020-10-06 16:24:16Z gouriano $
+/*  $Id: blast_options_builder.cpp 636927 2021-09-02 13:45:48Z fongah2 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -260,7 +260,8 @@ x_ProcessOneOption(CBlastOptionsHandle        & opts,
         	const list<int>& int_list = v.GetInteger_list();
         	list<TGi> gi_list;
         	ITERATE ( list<int>, it, int_list ) {
-        		gi_list.push_back(GI_FROM(int, *it));
+       			Uint4 gi_v = (Uint4) *it;
+        		gi_list.push_back(GI_FROM(Uint4, gi_v));
         	}
         	m_GiList = gi_list;
             #elif defined(NCBI_INT8_GI)
@@ -271,7 +272,8 @@ x_ProcessOneOption(CBlastOptionsHandle        & opts,
 			const list<int>& int_list = v.GetInteger_list();
         		list<TGi> gi_list;
         		ITERATE ( list<int>, it, int_list ) {
-        			gi_list.push_back(GI_FROM(int, *it));
+        			Uint4 gi_v = (Uint4) *it;
+        			gi_list.push_back(GI_FROM(Uint4, gi_v));
         		}
         		m_GiList = gi_list;
 		}
@@ -376,7 +378,8 @@ x_ProcessOneOption(CBlastOptionsHandle        & opts,
         	const list<int>& int_list = v.GetInteger_list();
         	list<TGi> gi_list;
         	ITERATE ( list<int>, it, int_list ) {
-        		gi_list.push_back(GI_FROM(int, *it));
+        		Uint4 gi_v = (Uint4) *it;
+        		gi_list.push_back(GI_FROM(Uint4, gi_v));
         	}
         	m_NegativeGiList = gi_list;
             #elif defined(NCBI_INT8_GI)
@@ -387,7 +390,8 @@ x_ProcessOneOption(CBlastOptionsHandle        & opts,
 			const list<int>& int_list = v.GetInteger_list();
         		list<TGi> gi_list;
         		ITERATE ( list<int>, it, int_list ) {
-        			gi_list.push_back(GI_FROM(int, *it));
+        			Uint4 gi_v = (Uint4) *it;
+        			gi_list.push_back(GI_FROM(Uint4, gi_v));
         		}
         		m_NegativeGiList = gi_list;
 		}
