@@ -1,4 +1,4 @@
-/*  $Id: blast_options_local_priv.hpp 566836 2018-07-09 13:56:34Z fongah2 $
+/*  $Id: blast_options_local_priv.hpp 687019 2024-09-03 13:55:47Z boratyng $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -201,6 +201,9 @@ public:
     int GetMismatchWindow() const;
     void SetMismatchWindow(int w);
 
+    bool GetChaining(void) const;
+    void SetChaining(bool ch);
+
     /******************* Hit saving options *************************/
     int GetHitlistSize() const;
     void SetHitlistSize(int s);
@@ -609,9 +612,9 @@ inline void
 CBlastOptionsLocal::SetWordSize(int ws)
 {
     m_LutOpts->word_size = ws;
-    if (m_LutOpts->lut_type == eCompressedAaLookupTable && ws <= 5)
+    if (m_LutOpts->lut_type == eCompressedAaLookupTable && ws <= 4)
 	m_LutOpts->lut_type = eAaLookupTable;
-    else if (m_LutOpts->lut_type == eAaLookupTable && ws > 5)
+    else if (m_LutOpts->lut_type == eAaLookupTable && ws > 4)
 	m_LutOpts->lut_type = eCompressedAaLookupTable;
 }
 
@@ -1249,6 +1252,18 @@ CBlastOptionsLocal::SetMismatchWindow(int w)
     m_ExtnOpts->mismatch_window = w;
 }
 
+inline bool
+CBlastOptionsLocal::GetChaining() const
+{
+    return m_ExtnOpts->chaining;
+}
+
+inline void
+CBlastOptionsLocal::SetChaining(bool ch)
+{
+    m_ExtnOpts->chaining = ch;
+}
+
 /******************* Hit saving options *************************/
 inline int
 CBlastOptionsLocal::GetHitlistSize() const
