@@ -1,4 +1,4 @@
-/*  $Id: blastn_args.cpp 631557 2021-05-19 13:52:41Z ivanov $
+/*  $Id: blastn_args.cpp 664744 2023-03-21 13:02:48Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -69,8 +69,7 @@ CBlastnAppArgs::CBlastnAppArgs()
     arg.Reset(m_StdCmdLineArgs);
     m_Args.push_back(arg);
 
-    arg.Reset(new CGenericSearchArgs(kQueryIsProtein, false, true,
-                                     false, false, false));
+    arg.Reset(new CGenericSearchArgs(eBlastTypeBlastn));
     m_Args.push_back(arg);
 
     arg.Reset(new CNuclArgs);
@@ -153,13 +152,13 @@ CBlastnNodeArgs::GetOutputStream()
 
 CBlastnNodeArgs::CBlastnNodeArgs(const string & input)
 {
-	m_InputStream = new CNcbiIstrstream(input.c_str(), input.length());
+	m_InputStream = new CNcbiIstrstream(input);
 }
 
 CBlastnNodeArgs::~CBlastnNodeArgs()
 {
 	if (m_InputStream) {
-		free(m_InputStream);
+		delete m_InputStream;
 		m_InputStream = NULL;
 	}
 }
