@@ -1,4 +1,4 @@
-/*  $Id: blastp_args.cpp 631557 2021-05-19 13:52:41Z ivanov $
+/*  $Id: blastp_args.cpp 638825 2021-10-07 12:19:31Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -149,13 +149,13 @@ CBlastpNodeArgs::GetOutputStream()
 
 CBlastpNodeArgs::CBlastpNodeArgs(const string & input)
 {
-	m_InputStream = new CNcbiIstrstream(input.c_str(), input.length());
+	m_InputStream = new CNcbiIstrstream(input);
 }
 
 CBlastpNodeArgs::~CBlastpNodeArgs()
 {
 	if (m_InputStream) {
-		free(m_InputStream);
+		delete m_InputStream;
 		m_InputStream = NULL;
 	}
 }
