@@ -1,4 +1,4 @@
-/*  $Id: blastx_args.cpp 631557 2021-05-19 13:52:41Z ivanov $
+/*  $Id: blastx_args.cpp 673087 2023-09-22 12:46:36Z madden $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -143,7 +143,9 @@ int
 CBlastxAppArgs::GetQueryBatchSize() const
 {
     bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
-    return blast::GetQueryBatchSize(eBlastx, m_IsUngapped, is_remote);
+    bool is_default = true;
+    return blast::GetQueryBatchSize(eBlastx, m_IsUngapped, is_remote, is_default, 
+          GetTask());
 }
 
 /// Get the input stream
@@ -164,13 +166,13 @@ CBlastxNodeArgs::GetOutputStream()
 
 CBlastxNodeArgs::CBlastxNodeArgs(const string & input)
 {
-	m_InputStream = new CNcbiIstrstream(input.c_str(), input.length());
+	m_InputStream = new CNcbiIstrstream(input);
 }
 
 CBlastxNodeArgs::~CBlastxNodeArgs()
 {
 	if (m_InputStream) {
-		free(m_InputStream);
+		delete m_InputStream;
 		m_InputStream = NULL;
 	}
 }
@@ -179,7 +181,9 @@ int
 CBlastxNodeArgs::GetQueryBatchSize() const
 {
     bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
-    return blast::GetQueryBatchSize(eBlastx, m_IsUngapped, is_remote);
+    bool split_by_query = true;
+    bool use_default = true;
+    return blast::GetQueryBatchSize(eBlastx, m_IsUngapped, is_remote, use_default, GetTask(), split_by_query);
 }
 
 CRef<CBlastOptionsHandle>
