@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c 683952 2024-06-10 12:29:05Z ivanov $
+/* $Id: blast_engine.c 695937 2025-04-03 19:37:55Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -79,7 +79,7 @@
 #define CONV_NUCL2PROT_COORDINATES(length) (length) / CODON_LENGTH
 
 NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
-NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 16;
+NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 17;
 NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 0;
 
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers
@@ -523,7 +523,7 @@ s_BlastSearchEngineOneContext(EBlastProgramType program_number,
             status = aux_struct->GetGappedScore(program_number, query,
                     query_info,
                     subject, gap_align, score_params, ext_params, hit_params,
-                    init_hitlist, &hsp_list, gapped_stats, NULL);
+                    word_params, init_hitlist, &hsp_list, gapped_stats, NULL);
         }
         else if (aux_struct->JumperGapped) {
             status = aux_struct->JumperGapped(subject, query, query_info,
