@@ -1,4 +1,4 @@
-/* $Id: blast_parameters.c 683930 2024-06-10 12:07:59Z ivanov $
+/* $Id: blast_parameters.c 687019 2024-09-03 13:55:47Z boratyng $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -119,9 +119,11 @@ BlastInitialWordParameters*
 BlastInitialWordParametersFree(BlastInitialWordParameters* parameters)
 
 {
-	sfree(parameters->cutoffs);
-	sfree(parameters);
-	return NULL;
+    if (parameters){
+        sfree(parameters->cutoffs);
+        sfree(parameters);
+    }    
+    return NULL;
 }
 
 /** Compute the default cutoff expect value for ungapped extensions
