@@ -1,4 +1,4 @@
-/*  $Id: blastfmtutil.cpp 500404 2016-05-04 14:59:01Z camacho $
+/*  $Id: blastfmtutil.cpp 668945 2023-06-15 14:10:29Z merezhuk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -202,9 +202,9 @@ CBlastFormatUtil::PrintAsciiPssm
     const SIZE_TYPE kQueryLength = pssm_with_params.GetPssm().GetQueryLength();
     _ASSERT(kQueryLength == 
             (SIZE_TYPE)pssm_with_params.GetPssm().GetNumColumns());
-    auto_ptr< TNcbiMatrixInt > pssm
+    unique_ptr< TNcbiMatrixInt > pssm
         (blast::CScorematPssmConverter::GetScores(pssm_with_params));
-    auto_ptr< TNcbiMatrixDouble > weighted_res_freqs
+    unique_ptr< TNcbiMatrixDouble > weighted_res_freqs
         (blast::CScorematPssmConverter::
             GetWeightedResidueFrequencies(pssm_with_params));
     vector<int> interval_sizes, num_matching_seqs;
@@ -457,7 +457,7 @@ s_MaskQuerySeq(CAlnVec& alnvec, string& query_seq,
     }
 
     vector<CRange<int> > masks_v;
-    int aln_stop = query_seq.size() - 1;
+    int aln_stop = static_cast<int>(query_seq.size()) - 1;
     ITERATE(ncbi::TMaskedQueryRegions, mask_iter, mask_info) {
         if ((*mask_iter)->GetFrame() != query_frame)
             continue;
