@@ -1,4 +1,4 @@
-/* $Id: blastxml_format.cpp 683928 2024-06-10 12:07:11Z ivanov $
+/* $Id: blastxml_format.cpp 670405 2023-07-24 18:54:39Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -498,7 +498,7 @@ s_SeqAlignSetToXMLHits(list <CRef<CHit> >& hits, const CSeq_align_set& alnset,
 	    else
 	    {
 		CNcbiOstrstream  one_hit_os;
-		unique_ptr<CObjectOStreamXml> xml_one_hit_os (new CObjectOStreamXml (one_hit_os,false));
+		unique_ptr<CObjectOStreamXml> xml_one_hit_os (new CObjectOStreamXml (one_hit_os,eNoOwnership));
 		xml_one_hit_os->SetEncoding(eEncoding_Ascii);
 		xml_one_hit_os->SetReferenceDTD(false);
 		xml_one_hit_os->Write( &(*new_hit), new_hit->GetThisTypeInfo() ); 
@@ -782,7 +782,7 @@ static bool s_SerializeAndSplitBy(const CSerialObject &object,
     end_part="</NOT SET>";
     CNcbiOstrstream one_iter_ss_os;
     {
-	unique_ptr<CObjectOStreamXml> xml_one_iter_os(new CObjectOStreamXml (one_iter_ss_os,false));
+	unique_ptr<CObjectOStreamXml> xml_one_iter_os(new CObjectOStreamXml (one_iter_ss_os,eNoOwnership));
 	xml_one_iter_os->SetEncoding(eEncoding_Ascii);
 	xml_one_iter_os->SetVerifyData( eSerialVerifyData_No );
 	xml_one_iter_os->SetReferenceDTD(add_reference_dtd);
