@@ -1,4 +1,4 @@
-/*  $Id: delta_unit_test.cpp 531968 2017-03-30 17:34:38Z grichenk $
+/*  $Id: delta_unit_test.cpp 636926 2021-09-02 13:40:12Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -194,11 +194,11 @@ struct CDeltaBlastTestFixture {
             }
             else {
                 BOOST_REQUIRE(seqid.GetGeneral().IsSetTag());
-                int new_tag = seqid.GetGeneral().GetTag().GetId();
+                Uint4 new_tag =(Uint4) (seqid.GetGeneral().GetTag().GetId());
 
-                if (new_tag != GI_TO(int, last_id)) {
+                if (new_tag != GI_TO(Uint4, last_id)) {
                     num_ids++;
-                    last_id = GI_FROM(int, new_tag);
+                    last_id = GI_FROM(Uint4, new_tag);
                 }
                 last_acc.clear();
                 last_ver = -1;
