@@ -1,4 +1,4 @@
-/*  $Id: traceback_unit_test.cpp 605350 2020-04-09 16:10:19Z ivanov $
+/*  $Id: traceback_unit_test.cpp 652745 2022-07-14 19:13:48Z merezhuk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -76,6 +76,10 @@
 #include <algo/blast/api/blast_seqinfosrc.hpp>
 #include <algo/blast/api/seqinfosrc_seqdb.hpp>
 
+#include <common/test_data_path.h>
+#include <corelib/ncbifile.hpp>
+#include <algorithm>
+
 #include "test_objmgr.hpp"
 #include "blast_test_util.hpp"
 
@@ -208,10 +212,10 @@ BOOST_AUTO_TEST_CASE(testBLASTNTraceBack) {
      const int k_num_hsps_end = 7;
 
      CSeq_id qid("gi|1945388");
-     auto_ptr<SSeqLoc> qsl(
+     unique_ptr<SSeqLoc> qsl(
          CTestObjMgr::Instance().CreateSSeqLoc(qid, eNa_strand_both));
      CSeq_id sid("gi|1732684");
-     auto_ptr<SSeqLoc> ssl(
+     unique_ptr<SSeqLoc> ssl(
          CTestObjMgr::Instance().CreateSSeqLoc(sid, eNa_strand_both));
 
      CBl2Seq blaster(*qsl, *ssl, eBlastn);
@@ -338,10 +342,10 @@ BOOST_AUTO_TEST_CASE(testBLASTNTraceBackLargeXDrop) {
      const int k_num_hsps_end = 1;
 
      CSeq_id qid("gi|42254502");
-     auto_ptr<SSeqLoc> qsl(
+     unique_ptr<SSeqLoc> qsl(
          CTestObjMgr::Instance().CreateSSeqLoc(qid, eNa_strand_both));
      CSeq_id sid("gi|34787366");
-     auto_ptr<SSeqLoc> ssl(
+     unique_ptr<SSeqLoc> ssl(
          CTestObjMgr::Instance().CreateSSeqLoc(sid, eNa_strand_both));
 
      CBlastNucleotideOptionsHandle opts_handle;
@@ -476,9 +480,9 @@ BOOST_AUTO_TEST_CASE(testBLASTPTraceBack) {
      CSeq_id qid("gi|42734333");
      CSeq_id sid("gi|30176631");
      
-     auto_ptr<SSeqLoc> qsl(
+     unique_ptr<SSeqLoc> qsl(
          CTestObjMgr::Instance().CreateSSeqLoc(qid, eNa_strand_unknown));
-     auto_ptr<SSeqLoc> ssl(
+     unique_ptr<SSeqLoc> ssl(
          CTestObjMgr::Instance().CreateSSeqLoc(sid, eNa_strand_unknown));
 
      CBlastProteinOptionsHandle opts_handle;
@@ -600,9 +604,9 @@ BOOST_AUTO_TEST_CASE(testTBLASTNTraceBack) {
      CSeq_id qid("gi|42734333");
      CSeq_id sid("gi|27902043");
      
-     auto_ptr<SSeqLoc> qsl(
+     unique_ptr<SSeqLoc> qsl(
          CTestObjMgr::Instance().CreateSSeqLoc(qid, eNa_strand_unknown));
-     auto_ptr<SSeqLoc> ssl(
+     unique_ptr<SSeqLoc> ssl(
          CTestObjMgr::Instance().CreateSSeqLoc(sid, eNa_strand_both));
 
      CTBlastnOptionsHandle opts_handle;
@@ -846,9 +850,9 @@ BOOST_AUTO_TEST_CASE(testFilterBlastResults_QueryCov) {
     CSeq_id qid("gi|42734333");
     CSeq_id sid("gi|30176631");
 
-    auto_ptr<SSeqLoc> qsl(
+    unique_ptr<SSeqLoc> qsl(
         CTestObjMgr::Instance().CreateSSeqLoc(qid, eNa_strand_unknown));
-    auto_ptr<SSeqLoc> ssl(
+    unique_ptr<SSeqLoc> ssl(
         CTestObjMgr::Instance().CreateSSeqLoc(sid, eNa_strand_unknown));
 
     CBlastProteinOptionsHandle opts_handle;
@@ -949,10 +953,10 @@ BOOST_AUTO_TEST_CASE(testFilterBlastResults_MaxHsps) {
      const int k_num_hsps_filtered = 4;
 
      CSeq_id qid("gi|1945388");
-     auto_ptr<SSeqLoc> qsl(
+     unique_ptr<SSeqLoc> qsl(
          CTestObjMgr::Instance().CreateSSeqLoc(qid, eNa_strand_both));
      CSeq_id sid("gi|1732684");
-     auto_ptr<SSeqLoc> ssl(
+     unique_ptr<SSeqLoc> ssl(
          CTestObjMgr::Instance().CreateSSeqLoc(sid, eNa_strand_both));
 
      CBl2Seq blaster(*qsl, *ssl, eBlastn);
@@ -1081,9 +1085,30 @@ BlastHSPList * s_GetHSPList(int num_hsps, int oid, int len, int range_len)
 	return hsp_list;
 }
 
+BOOST_AUTO_TEST_CASE(testUnit_TestsDataPath) {
+	string rel_path = "blast/algo/unit_tests/api/data/long_seqs";
+	std::replace( rel_path.begin(), rel_path.end(),'/', CDirEntry::GetPathSeparator() ); // SB-3398 fix sub directory path to be platform compatible
+
+	BlastSeqSrc* seqSrc = SeqDbBlastSeqSrcInit( CFile::ConcatPath( NCBI_GetTestDataPath(), rel_path)  , false );
+
+	if( seqSrc && BlastSeqSrcGetInitError(seqSrc) ) {
+	    cout << "testUnit_TestsDataPath: BlastSeqSrcGetInitError: "<<BlastSeqSrcGetInitError(seqSrc) << endl;
+	    cout << "testUnit_TestsDataPath: Can't access: "<< ( NCBI_GetTestDataPath()  + rel_path ) << endl;
+	}
+	BOOST_REQUIRE(seqSrc != NULL);
+	BOOST_REQUIRE(BlastSeqSrcGetInitError(seqSrc) ==  NULL);
+
+}
 
 BOOST_AUTO_TEST_CASE(testSetupPartialFetching) {
-	BlastSeqSrc* seqSrc = SeqDbBlastSeqSrcInit("data/long_seqs", false);
+	string rel_path = "blast/algo/unit_tests/api/data/long_seqs";
+	std::replace( rel_path.begin(), rel_path.end(),'/', CDirEntry::GetPathSeparator() ); // SB-3398 fix sub directory path to be platform compatible
+
+	BlastSeqSrc* seqSrc = SeqDbBlastSeqSrcInit( CFile::ConcatPath( NCBI_GetTestDataPath(), rel_path)  , false );
+
+	BOOST_REQUIRE(seqSrc != NULL);
+	BOOST_REQUIRE(BlastSeqSrcGetInitError(seqSrc) ==  NULL);
+
     const int k_num_hsps = 6;
 	BlastHSPList* hsp_list = (BlastHSPList*) calloc(1, sizeof(BlastHSPList));
 	hsp_list->oid = 4;
@@ -1120,7 +1145,14 @@ BOOST_AUTO_TEST_CASE(testSetupPartialFetching) {
 }
 
 BOOST_AUTO_TEST_CASE(testPartialFetchingMT) {
-	BlastSeqSrc* seqSrc = SeqDbBlastSeqSrcInit("data/long_seqs", false);
+	string rel_path = "blast/algo/unit_tests/api/data/long_seqs";
+	std::replace( rel_path.begin(), rel_path.end(),'/', CDirEntry::GetPathSeparator() ); // SB-3398 fix sub directory path to be platform compatible
+	
+	BlastSeqSrc* seqSrc = SeqDbBlastSeqSrcInit( CFile::ConcatPath( NCBI_GetTestDataPath(), rel_path)  , false );
+
+	BOOST_REQUIRE(seqSrc != NULL);
+	BOOST_REQUIRE(BlastSeqSrcGetInitError(seqSrc) ==  NULL);
+
     const int kMaxNum = 36;
     const int kRangeLen = 10000;
     int kOid = 4;
