@@ -1,4 +1,4 @@
-/*  $Id: blast_app_util.hpp 683535 2024-05-30 12:54:58Z ivanov $
+/*  $Id: blast_app_util.hpp 696147 2025-04-08 18:25:19Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -183,16 +183,18 @@ string RegisterOMDataLoader(CRef<CSeqDB> db_handle);
         exit_code = BLAST_INPUT_ERROR;                                      \
     }                                                                       \
     catch (const CSeqDBException& e) {                                      \
-		if (e.GetErrCode() == CSeqDBException::eOpenFileErr) {              \
+		if (e.GetErrCode() == CSeqDBException::eTooManyOpenFiles) {         \
 			string err_msg =                                                \
                 "Too many open files, please raise the open file limit";    \
         	LOG_POST(Error << "BLAST Database error: " << err_msg);         \
-            exit_code = BLAST_ENGINE_ERROR;                                 \
+            exit_code = BLAST_DATABASE_ERROR;                               \
 		}                                                                   \
-		else if (e.GetErrCode() == CSeqDBException::eFileErr){              \
+		else if (e.GetErrCode() == CSeqDBException::eMemoryMappingFailure){ \
 			string err_msg = "Database memory map file error";              \
+            if (!e.GetMsg().empty())                                        \
+                err_msg += " - " + e.GetMsg();                              \
         	LOG_POST(Error << "BLAST Database error: " << err_msg);         \
-            exit_code = BLAST_ENGINE_ERROR;                                 \
+            exit_code = BLAST_DATABASE_ERROR;                               \
 		}                                                                   \
         else {                                                              \
         	LOG_POST(Error << "BLAST Database error: " << e.GetMsg());      \
