@@ -1,4 +1,4 @@
-/*  $Id: blast_formatter.cpp 616448 2020-09-16 13:25:04Z fongah2 $
+/*  $Id: blast_formatter.cpp 634996 2021-07-26 13:00:45Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -113,7 +113,7 @@ void CBlastFormatterApp::Init()
 {
     HideStdArgs(fHideLogfile | fHideConffile | fHideFullVersion | fHideXmlHelp | fHideDryRun);
 
-    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
+    unique_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
 
     arg_desc->SetUsageContext(GetArguments().GetProgramBasename(), 
                   "Stand-alone BLAST formatter client, version " 
@@ -306,7 +306,7 @@ s_InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args,
             isRemote = true;
             LOG_POST(Info <<"Remote " << search_db->GetDatabaseName());
             }
-            catch (CException & e) {
+            catch (CException & ) {
             	SetDiagPostLevel(eDiag_Warning);
             	NCBI_THROW(CException, eUnknown, "Fail to initialize local or remote DB" );
             }
@@ -319,7 +319,7 @@ s_InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args,
         string genbank_loader = CGBDataLoader::RegisterInObjectManager
             (*( CObjectManager::GetInstance()), reader,CObjectManager::eNonDefault).GetLoader()->GetName();
         scope->AddDataLoader(genbank_loader, kGenbankLoaderPriority);
-    } catch (const CException& e) {
+    } catch (const CException& ) {
     	LOG_POST(Info << "Failed to add genbank dataloader");
     	// It's ok not to have genbank loader
     }
@@ -524,6 +524,8 @@ int CBlastFormatterApp::Run(void)
                 NCBI_RETHROW(e, CInputException, eInvalidInput,
                              "Invalid input format for BLAST Archive.");
             }
+            x_AddCmdOptions();
+            m_UsageReport.AddParam(CBlastUsageReport::eExitStatus, status);
     	    return status;
         }
 
