@@ -1,4 +1,4 @@
-/*  $Id: blastdb_aliastool.cpp 625561 2021-02-16 19:38:04Z ivanov $
+/*  $Id: blastdb_aliastool.cpp 651857 2022-06-27 12:07:41Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -147,7 +147,7 @@ void CBlastDBAliasApp::Init()
 {
     HideStdArgs(fHideConffile | fHideFullVersion | fHideXmlHelp | fHideDryRun);
 
-    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
+    unique_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
 
     arg_desc->SetUsageContext(GetArguments().GetProgramBasename(), 
                               "Application to create BLAST database aliases, version " 
@@ -214,8 +214,7 @@ void CBlastDBAliasApp::Init()
                              CArgDescriptions::eInputFile);
 
     arg_desc->SetDependency(kArgSeqIdList, CArgDescriptions::eRequires, kOutput);
-    arg_desc->SetDependency(kArgSeqIdList, CArgDescriptions::eExcludes,
-                            kArgGiList);
+    arg_desc->SetDependency(kArgSeqIdList, CArgDescriptions::eExcludes, kArgGiList);
 
     arg_desc->AddOptionalKey(kArgTaxIdListFile, "input_file",
                                  "Text taxonomy id file to restrict "
@@ -226,6 +225,15 @@ void CBlastDBAliasApp::Init()
     arg_desc->SetDependency(kArgTaxIdListFile, CArgDescriptions::eExcludes, kArgGiList);
     arg_desc->SetDependency(kArgTaxIdListFile, CArgDescriptions::eExcludes, kArgSeqIdList);
 
+    arg_desc->AddOptionalKey("oid_masks", "oid_masks",
+        		                 "Create alias db with pre-built oid masks\n"
+        		                 "0x01 Exclude Model", CArgDescriptions::eInteger);
+
+    arg_desc->SetDependency("oid_masks", CArgDescriptions::eRequires, kOutput);
+    arg_desc->SetDependency("oid_masks", CArgDescriptions::eExcludes, kArgGiList);
+    arg_desc->SetDependency("oid_masks", CArgDescriptions::eExcludes, kArgSeqIdList);
+    arg_desc->SetDependency("oid_masks", CArgDescriptions::eExcludes, kArgTaxIdListFile);
+
 #ifdef NCBI_TI
     arg_desc->AddFlag("process_as_tis", 
                       "Process all numeric ID lists as TIs instead of GIs", true);
@@ -386,26 +394,36 @@ CBlastDBAliasApp::CreateAliasFile() const
     }
 
     if (args[kArgDb].HasValue() && !args[kArgGiList].HasValue() &&
-        !args[kArgSeqIdList].HasValue()&& ! args[kArgTaxIdListFile].HasValue()) {
+        !args[kArgSeqIdList].HasValue() && ! args[kArgTaxIdListFile].HasValue() &&
+        !args["oid_masks"].HasValue()) {
 
-        NCBI_THROW(CInputException, eInvalidInput, "Either gilist or "
-                   "seqid_list must be specified if database name is used");
+        NCBI_THROW(CInputException, eInvalidInput, "Either gilist, seqid_list, taxidlist "
+                   "or oid_masks must be specified if database name is used");
     }
 
     if (args[kArgDbTitle].HasValue()) {
         title = args[kArgDbTitle].AsString();
     } else if (args[kArgDb].HasValue()) {
         _ASSERT(args[kArgGiList].HasValue() || args[kArgSeqIdList].HasValue() ||
-        		args[kArgTaxIdListFile].HasValue());
-        title = args[kArgDb].AsString() + " limited by ";
+        		args[kArgTaxIdListFile].HasValue() || args["oid_masks"].HasValue());
+        CSeqDB_Path orig_db(args[kArgDb].AsString());
+        string orig_db_name;
+        orig_db.FindBaseName().GetString(orig_db_name);
+        title = orig_db_name + " limited by ";
         if (args[kArgGiList]) {
             title += args[kArgGiList].AsString();
         }
         else if (args[kArgSeqIdList]){
             title += args[kArgSeqIdList].AsString();
-        } else {
+        }
+        else if (args[kArgTaxIdListFile]) {
             title += args[kArgTaxIdListFile].AsString();
         }
+        else  {
+        	if (args["oid_masks"].AsInteger() & EOidMaskType::fExcludeModel) {
+            	title += "exclude model oid masks";
+        	}
+        }
     }
     const CWriteDB::ESeqType seq_type = 
         args[kArgDbType].AsString() == "prot"
@@ -476,6 +494,9 @@ CBlastDBAliasApp::CreateAliasFile() const
                                  args[kArgDb].AsString(),
                                  seq_type, taxid_list,
                                  title, eTaxIdList);
+    } else if (args[kArgDb].HasValue() && args["oid_masks"]) {
+    	CWriteDB_CreateOidMaskDB(args[kArgDb].AsString(), args[kOutput].AsString(),
+    			                 seq_type, args["oid_masks"].AsInteger(), title);
     }
 
     if (args["vdblist"].HasValue() || args["vdblist_file"].HasValue()) {
@@ -679,8 +700,6 @@ void CBlastDBAliasApp::x_AddCmdOptions()
 	 }
 }
 
-
-
 #ifndef SKIP_DOXYGEN_PROCESSING
 int main(int argc, const char* argv[] /*, const char* envp[]*/)
 {
