@@ -1,4 +1,4 @@
-/*  $Id: blastdb_convert.cpp 611918 2020-07-13 16:49:16Z fongah2 $
+/*  $Id: blastdb_convert.cpp 622276 2020-12-22 16:57:24Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,7 @@ void CBlastdbConvertApp::Init()
 {
     HideStdArgs(fHideConffile | fHideFullVersion | fHideXmlHelp | fHideDryRun);
 
-    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
+    unique_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
 
     // Specify USAGE context
     arg_desc->SetUsageContext(GetArguments().GetProgramBasename(),
