@@ -1,4 +1,4 @@
-/*  $Id: blastdbcheck.cpp 611888 2020-07-13 11:50:02Z fongah2 $
+/*  $Id: blastdbcheck.cpp 660529 2022-12-19 15:35:44Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -144,7 +144,7 @@ void CBlastDbCheckApplication::Init(void)
     HideStdArgs(fHideLogfile | fHideConffile | fHideFullVersion | fHideXmlHelp | fHideDryRun);
 
     // Create command-line argument descriptions class
-    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
+    unique_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
 
     // Specify USAGE context
     arg_desc->SetUsageContext(GetArguments().GetProgramBasename(),
@@ -745,6 +745,8 @@ public:
                 // Don't check for right now
             } else if (tokens[0] == "MASKLIST") {
                 // Don't check for right now
+            } else if (tokens[0] == "TAXIDLIST") {
+                num_failures += x_CheckFile(name, dir, tokens);
             } else if (tokens[0] == "OIDLIST") {
                 oidlist = 0;
                 num_failures += x_CheckFile(name, dir, tokens);
@@ -765,6 +767,10 @@ public:
                 num_failures += x_CheckNumber(name, tokens, maxoid);
             } else if (tokens[0] == "MAXLEN") {
                 num_failures += x_CheckNumber(name, tokens, maxlen);
+            } else if (tokens[0] == "STATS_TOTLEN") {
+            	// STATS_TOTLEN is often not exact, skip check
+            } else if (tokens[0] == "STATS_NSEQ") {
+            	// STATS_NSEQ is often not exact, skip check
             } else {
                 Log(name, e_Brief) << "  [ERROR] unknown keyword encountered: " 
                                    << tokens[0] << endl;
