@@ -1,4 +1,4 @@
-/*  $Id: makeblastdb.cpp 631507 2021-05-19 13:47:23Z ivanov $
+/*  $Id: makeblastdb.cpp 699124 2025-06-30 15:41:20Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
  */
 
 #include <ncbi_pch.hpp>
+#include <serial/objostrjson.hpp>
 #include <algo/blast/api/version.hpp>
 #include <algo/blast/blastinput/blast_input_aux.hpp>
 #include <corelib/ncbiapp.hpp>
@@ -53,8 +54,10 @@
 #include <objtools/blast/seqdb_writer/writedb_error.hpp>
 #include <util/format_guess.hpp>
 #include <util/util_exception.hpp>
+#include <util/compress/stream_util.hpp> 
 #include <objtools/blast/seqdb_writer/build_db.hpp>
 
+#include <serial/objostrjson.hpp>
 #include <algo/blast/blastinput/blast_input.hpp>
 #include "../blast/blast_app_util.hpp"
 #include "masked_range_set.hpp"
@@ -62,6 +65,7 @@
 #include <iostream>
 #include <sstream>
 #include <fstream>
+#include <random>
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 USING_NCBI_SCOPE;
@@ -207,7 +211,7 @@ void CMakeBlastDBApp::Init()
 {
     HideStdArgs(fHideConffile | fHideFullVersion | fHideXmlHelp | fHideDryRun);
 
-    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
+    unique_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
 
     // Specify USAGE context
     arg_desc->SetUsageContext(GetArguments().GetProgramBasename(),
@@ -246,6 +250,9 @@ void CMakeBlastDBApp::Init()
                       "Create index of sequence hash values.",
                       true);
 
+    arg_desc->AddFlag("randomize",
+                      "Randomize input db seqs", true,  CArgDescriptions::fHidden);
+
 #if ((!defined(NCBI_COMPILER_WORKSHOP) || (NCBI_COMPILER_VERSION  > 550)) && \
      (!defined(NCBI_COMPILER_MIPSPRO)) )
     arg_desc->SetCurrentGroup("Sequence masking options");
@@ -296,7 +303,9 @@ void CMakeBlastDBApp::Init()
                             new CArgAllow_Integers(eBDB_Version4, eBDB_Version5));
     arg_desc->AddDefaultKey("max_file_sz", "number_of_bytes",
                             "Maximum file size for BLAST database files",
-                            CArgDescriptions::eString, "1GB");
+                            CArgDescriptions::eString, "3GB");
+    arg_desc->AddOptionalKey("metadata_output_prefix", "",
+    						"Path prefix for location of database files in metadata", CArgDescriptions::eString);
     arg_desc->AddOptionalKey("logfile", "File_Name",
                              "File to which the program log should be redirected",
                              CArgDescriptions::eOutputFile,
@@ -319,6 +328,9 @@ void CMakeBlastDBApp::Init()
              CArgDescriptions::eInputFile);
     arg_desc->SetDependency("taxid_map", CArgDescriptions::eRequires, "parse_seqids");
 
+    arg_desc->AddOptionalKey("oid_masks", "oid_masks",
+    		                 "0x01 Exclude Model", CArgDescriptions::eInteger);
+
     SetupArgDescriptions(arg_desc.release());
 }
 
@@ -376,21 +388,21 @@ static TTaxId s_GetTaxId(const CBioseq & bio)
     return ZERO_TAX_ID;
 }
 
-static bool s_GenerateTitle(const CBioseq & bio)
+static bool s_HasTitle(const CBioseq & bio)
 {
     if (! bio.CanGetDescr()) {
-        return true;
+        return false;
     }
 
     ITERATE(list< CRef< CSeqdesc > >, iter, bio.GetDescr().Get()) {
         const CSeqdesc & desc = **iter;
 
         if (desc.IsTitle()) {
-            return false;
+            return true;
         }
     }
 
-    return true;
+    return false;
 }
 
 class CSeqEntrySource : public IBioseqSource {
@@ -480,10 +492,8 @@ public:
         }
 
         m_bio = Begin(*m_entry);
-        for (CTypeIterator<CBioseq> it = Begin(*m_entry); it; ++it) {
-            m_scope->AddBioseq(*it);
-        }
         m_entry->Parentize();
+        m_scope->AddTopLevelSeqEntry(*m_entry);
     }
 
     virtual CConstRef<CBioseq> GetNext()
@@ -503,9 +513,18 @@ public:
                 }
             }
 
-            if (s_GenerateTitle(*m_bio)) {
-                sequence::CDeflineGenerator gen;
-                const string & title = gen.GenerateDefline(*m_bio , *m_scope);
+            const sequence::CDeflineGenerator::TUserFlags flags = sequence::CDeflineGenerator::fUseAutoDef;
+            sequence::CDeflineGenerator gen;
+            const string & title = gen.GenerateDefline(*m_bio , *m_scope, flags);
+            string old_title;
+            if (s_HasTitle(*m_bio)) {
+                for (auto& i: m_bio->SetDescr().Set()) {
+                    if (i->IsTitle()) {
+                        old_title = i->GetTitle();
+                        i->SetTitle(title);
+                    }
+                }
+            } else {
                 CRef<CSeqdesc> des(new CSeqdesc);
                 des->SetTitle(title);
                 CSeq_descr& desr(m_bio->SetDescr());
@@ -560,7 +579,7 @@ void CMakeBlastDBApp::x_AddSeqEntries(CNcbiIstream & data, TFormat fmt)
 
 class CRawSeqDBSource : public IRawSequenceSource {
 public:
-    CRawSeqDBSource(const string & name, bool protein, CBuildDatabase * outdb);
+    CRawSeqDBSource(const string & name, bool protein, CBuildDatabase * outdb, bool randomize);
 
     virtual ~CRawSeqDBSource()
     {
@@ -607,6 +626,7 @@ private:
     CRef<CSeqDBExpert> m_Source;
     const char * m_Sequence;
     int m_Oid;
+    vector<int> m_RandomOids;
 #if ((!defined(NCBI_COMPILER_WORKSHOP) || (NCBI_COMPILER_VERSION  > 550)) && \
      (!defined(NCBI_COMPILER_MIPSPRO)) )
     vector<CBlastDbBlob> m_Blobs;
@@ -617,7 +637,7 @@ private:
 #endif
 };
 
-CRawSeqDBSource::CRawSeqDBSource(const string & name, bool protein, CBuildDatabase * outdb)
+CRawSeqDBSource::CRawSeqDBSource(const string & name, bool protein, CBuildDatabase * outdb, bool randomize)
     : m_Sequence(NULL), m_Oid(0)
 {
     CSeqDB::ESeqType seqtype =
@@ -642,6 +662,16 @@ CRawSeqDBSource::CRawSeqDBSource(const string & name, bool protein, CBuildDataba
     for(int i = 0; i < (int)m_ColumnNames.size(); i++) {
         m_ColumnIds.push_back(m_Source->GetColumnId(m_ColumnNames[i]));
     }
+    if(randomize) {
+    	int num_oids = m_Source->GetNumOIDs();
+    	m_RandomOids.clear();
+        std::mt19937 rng(std::time(nullptr));
+    	for (int i=0; m_Source->CheckOrFindOID(i); i++) {
+    		m_RandomOids.push_back(i);
+    	}
+    	std::shuffle (m_RandomOids.begin(), m_RandomOids.end(), rng);
+    	m_RandomOids.push_back(num_oids);
+    }
 #endif
 }
 
@@ -653,7 +683,8 @@ CRawSeqDBSource::GetNext(CTempString               & sequence,
                          vector<int>               & column_ids,
                          vector<CTempString>       & column_blobs)
 {
-    if (! m_Source->CheckOrFindOID(m_Oid))
+	int oid = (m_RandomOids.size() > 0)? m_RandomOids[m_Oid]: m_Oid;
+    if (! m_Source->CheckOrFindOID(oid))
         return false;
 
     if (m_Sequence) {
@@ -663,12 +694,12 @@ CRawSeqDBSource::GetNext(CTempString               & sequence,
 
     int slength(0), alength(0);
 
-    m_Source->GetRawSeqAndAmbig(m_Oid, & m_Sequence, & slength, & alength);
+    m_Source->GetRawSeqAndAmbig(oid, & m_Sequence, & slength, & alength);
 
     sequence    = CTempString(m_Sequence, slength);
     ambiguities = CTempString(m_Sequence + slength, alength);
 
-    deflines = m_Source->GetHdr(m_Oid);
+    deflines = m_Source->GetHdr(oid);
 
 #if ((!defined(NCBI_COMPILER_WORKSHOP) || (NCBI_COMPILER_VERSION  > 550)) && \
      (!defined(NCBI_COMPILER_MIPSPRO)) )
@@ -676,7 +707,7 @@ CRawSeqDBSource::GetNext(CTempString               & sequence,
     ITERATE(vector<int>, algo_id, m_MaskIds) {
 
         CSeqDB::TSequenceRanges ranges;
-        m_Source->GetMaskData(m_Oid, *algo_id, ranges);
+        m_Source->GetMaskData(oid, *algo_id, ranges);
 
         SBlastDbMaskData mask_data;
         mask_data.algorithm_id = m_MaskIdMap[*algo_id];
@@ -695,11 +726,14 @@ CRawSeqDBSource::GetNext(CTempString               & sequence,
     m_Blobs.resize(column_ids.size());
 
     for(int i = 0; i < (int)column_ids.size(); i++) {
-        m_Source->GetColumnBlob(column_ids[i], m_Oid, m_Blobs[i]);
+        m_Source->GetColumnBlob(column_ids[i], oid, m_Blobs[i]);
         column_blobs[i] = m_Blobs[i].Str();
     }
 #endif
 
+    if (m_RandomOids.size()== 0) {
+    	m_Oid = oid;
+    }
     m_Oid ++;
 
     return true;
@@ -765,7 +799,7 @@ CMakeBlastDBApp::x_VerifyInputFilesType(const vector<CTempString>& filenames,
 
     // Guess the input data type
     for (size_t i = 0; i < filenames.size(); i++) {
-        const string & seq_file = filenames[i];
+        string seq_file = filenames[i];
 
         CFile input_file(seq_file);
         if ( !input_file.Exists() ) {
@@ -778,9 +812,27 @@ CMakeBlastDBApp::x_VerifyInputFilesType(const vector<CTempString>& filenames,
         }
 
         CNcbiIfstream f(seq_file.c_str(), ios::binary);
-        if(x_ConvertToSupportedType(x_GuessFileType(f)) != input_type) {
-            string error_msg = seq_file + " does not match input format type, default input type is FASTA";
-            NCBI_THROW(CInvalidDataException, eInvalidInput, error_msg);
+	unique_ptr<CDecompressIStream>  comp_seq_istream;
+	CNcbiIstream *data_in = & f;
+
+	// SB-4223 : support for compressed input
+	bool is_gz = false, is_bz2 = false, is_zst=false;
+	CCompressStream::EMethod method = CDecompressIStream::eGZipFile; 
+	if( (is_gz  = NStr::EndsWith( seq_file, ".gz", NStr::eNocase)) || 
+	    (is_bz2 = NStr::EndsWith( seq_file, ".bz2",NStr::eNocase)) ||    
+	    (is_zst = NStr::EndsWith( seq_file, ".zst",NStr::eNocase))    
+	    )
+	{
+	    if( is_bz2  ) method = CDecompressIStream::eBZip2;
+	    if( is_zst  ) method = CDecompressIStream::eZstd;
+	    comp_seq_istream.reset(new CDecompressIStream(f , method ));
+	    data_in =  comp_seq_istream.get();
+	}
+
+        if(input_type == eFasta && x_ConvertToSupportedType(x_GuessFileType( *data_in ) ) != eFasta) {
+            string msg = "\nInput file " + seq_file + " does NOT appear to be FASTA (processing anyway).\n" \
+                 + "Advise validating database with 'blastdbcheck -dbtype [prot|nucl] -db ${DBNAME}'\n";
+            ERR_POST(Warning << msg);
         }
     }
     return;
@@ -976,11 +1028,28 @@ void CMakeBlastDBApp::x_ProcessInputData(const string & paths,
             x_AddSequenceData(cin, build_fmt);
         }
         else {
-            x_VerifyInputFilesType(names, input_fmt);
+             x_VerifyInputFilesType(names, input_fmt);
             for (size_t i = 0; i < names.size(); i++) {
-                const string & seq_file = names[i];
+                string seq_file = names[i];
                 CNcbiIfstream f(seq_file.c_str(), ios::binary);
-                x_AddSequenceData(f, build_fmt);
+		CNcbiIstream *data_in = &f;  // uncompressed by default
+
+		unique_ptr<CDecompressIStream>  comp_seq_istream;
+		bool is_gz = false, is_bz2 = false, is_zst=false;
+		CCompressStream::EMethod method = CDecompressIStream::eGZipFile; // default gzip method if compressed
+
+		if( (is_gz  = NStr::EndsWith( seq_file, ".gz", NStr::eNocase)) || 
+		    (is_bz2 = NStr::EndsWith( seq_file, ".bz2",NStr::eNocase)) ||
+		    (is_zst = NStr::EndsWith( seq_file, ".zst",NStr::eNocase))    
+		  )
+		{
+		    if( is_bz2  ) method = CDecompressIStream::eBZip2;
+		    if( is_zst  ) method = CDecompressIStream::eZstd;
+		    comp_seq_istream.reset(new CDecompressIStream(f , method ));
+		    data_in =  comp_seq_istream.get();
+		}
+		// common part
+		x_AddSequenceData( *data_in , build_fmt);
             }
         }
     }
@@ -988,6 +1057,8 @@ void CMakeBlastDBApp::x_ProcessInputData(const string & paths,
         vector<string> blastdb;
         copy(names.begin(), names.end(), back_inserter(blastdb));
         CSeqDB::ESeqType seqtype = (is_protein ? CSeqDB::eProtein : CSeqDB::eNucleotide);
+        const CArgs& args = GetArgs();
+        bool randomize = args["randomize"].AsBoolean();
 
         vector<string> final_blastdb;
 
@@ -1026,8 +1097,8 @@ void CMakeBlastDBApp::x_ProcessInputData(const string & paths,
                     quoted,
                     is_protein ? CSeqDB::eProtein : CSeqDB::eNucleotide
             ));
-            const int numoids = indb->GetNumOIDs();
-            for (int oid = 0; oid < numoids; ++oid) {
+
+            for (int oid=0; indb->CheckOrFindOID(oid); oid++) {
                 CRef<CBlast_def_line_set> hdr = indb->GetHdr(oid);
                 ITERATE(CBlast_def_line_set::Tdata, itr, hdr->Get()) {
                     CRef<CBlast_def_line> bdl = *itr;
@@ -1044,7 +1115,7 @@ void CMakeBlastDBApp::x_ProcessInputData(const string & paths,
 
             m_DB->SetLeafTaxIds(leafTaxIds, true);
             CRef<IRawSequenceSource> raw(
-                    new CRawSeqDBSource(quoted, is_protein, m_DB)
+                    new CRawSeqDBSource(quoted, is_protein, m_DB, randomize)
             );
             m_DB->AddSequences(*raw);
         } else {
@@ -1060,6 +1131,10 @@ void CMakeBlastDBApp::x_BuildDatabase()
             GetEnvironment().Get("NCBI_MAKEBLASTDB_SKIP_UNVERIFIED_BIOSEQS");
     m_SkipUnver = (env_skip.empty() == false);
 
+    const string dont_scan_bioseq =
+        GetEnvironment().Get("NCBI_MAKEBLASTDB_DONT_SCAN_BIOSEQ_FOR_CFASTAREADER_USER_OBJECTS");
+    const bool scan_bioseq_4_cfastareader_usrobj = static_cast<bool>(dont_scan_bioseq.empty());
+
     const CArgs & args = GetArgs();
 
     // Get arguments to the CBuildDatabase constructor.
@@ -1078,6 +1153,10 @@ void CMakeBlastDBApp::x_BuildDatabase()
             "changing the output name, please provide a (different) database name "
             "using -" + kOutput);
     }
+    if (input_fmt != eBlastDb && args["randomize"].AsBoolean()) {
+        NCBI_THROW(CInvalidDataException, eInvalidInput,
+        		"Option randomize is only applicable to blastdb input type");
+    }
 
     vector<string> input_files;
     NStr::Split(dbname, kInputSeparators, input_files);
@@ -1145,6 +1224,10 @@ void CMakeBlastDBApp::x_BuildDatabase()
     	limit_defline = true;
     }
 #endif
+    Uint8 oid_masks = 0;
+    if(args["oid_masks"]) {
+    	oid_masks = args["oid_masks"].AsInteger();
+    }
     m_DB.Reset(new CBuildDatabase(dbname,
                                   title,
                                   is_protein,
@@ -1153,7 +1236,9 @@ void CMakeBlastDBApp::x_BuildDatabase()
                                   m_LogFile,
                                   long_seqids,
                                   dbver,
-                                  limit_defline));
+                                  limit_defline,
+                                  oid_masks,
+                                  scan_bioseq_4_cfastareader_usrobj));
 
 #if _BLAST_DEBUG
     if (args["verbose"]) {
@@ -1208,7 +1293,30 @@ void CMakeBlastDBApp::x_BuildDatabase()
 #endif
     x_ProcessInputData(args[kInput].AsString(), is_protein);
 
-    m_DB->EndBuild();
+    bool success = m_DB->EndBuild();
+    if(success) {
+    	string new_db = m_DB->GetOutputDbName();
+    	CSeqDB::ESeqType t = is_protein? CSeqDB::eProtein: CSeqDB::eNucleotide;
+    	CSeqDB sdb(new_db, t);
+        string output_prefix = args["metadata_output_prefix"]
+                ? args["metadata_output_prefix"].AsString()
+                : kEmptyStr;
+        if (!output_prefix.empty() && (output_prefix.back() != CFile::GetPathSeparator()))
+            output_prefix += CFile::GetPathSeparator();
+    	CRef<CBlast_db_metadata> m = sdb.GetDBMetaData(output_prefix);
+    	string extn (kEmptyStr);
+    	SeqDB_GetMetadataFileExtension(is_protein, extn);
+    	string metadata_filename = new_db + "." + extn;
+        ofstream out(metadata_filename.c_str());
+        unique_ptr<CObjectOStreamJson> json_out(new CObjectOStreamJson(out, eNoOwnership));
+        json_out->SetDefaultStringEncoding(eEncoding_Ascii);
+        json_out->PreserveKeyNames();
+        CConstObjectInfo obj_info(m, m->GetTypeInfo());
+        json_out->WriteObject(obj_info);
+        json_out->Flush();
+        out.flush();
+        out << NcbiEndl;
+    }
 }
 
 int CMakeBlastDBApp::Run(void)
