@@ -1,4 +1,4 @@
-/*  $Id: win_mask_app.cpp 617977 2020-10-08 18:29:47Z grichenk $
+/*  $Id: win_mask_app.cpp 656846 2022-09-29 01:34:59Z morgulis $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -100,7 +100,7 @@ CWinMaskApplication::CWinMaskApplication() {
 void CWinMaskApplication::Init(void)
 {
     HideStdArgs(fHideLogfile | fHideConffile | fHideVersion | fHideDryRun);
-    auto_ptr< CArgDescriptions > arg_desc( new CArgDescriptions );
+    unique_ptr< CArgDescriptions > arg_desc( new CArgDescriptions );
 
     // Set the program description
     arg_desc->SetUsageContext( GetArguments().GetProgramBasename(),
@@ -165,7 +165,11 @@ int CWinMaskApplication::Run (void)
                                         aConfig.Ids(),
                                         aConfig.ExcludeIds(),
                                         aConfig.UseBA(),
-                                        aConfig.GetMetaData() );
+                                        aConfig.GetMetaData(),
+                                        aConfig.MinScorePct(),
+                                        aConfig.ExtendScorePct(),
+                                        aConfig.ThresScorePct(),
+                                        aConfig.MaxScorePct() );
             cg();
         }
         else {
@@ -184,7 +188,11 @@ int CWinMaskApplication::Run (void)
                                         aConfig.Ids(),
                                         aConfig.ExcludeIds(),
                                         aConfig.UseBA(),
-                                        aConfig.GetMetaData() );
+                                        aConfig.GetMetaData(),
+                                        aConfig.MinScorePct(),
+                                        aConfig.ExtendScorePct(),
+                                        aConfig.ThresScorePct(),
+                                        aConfig.MaxScorePct() );
             cg();
         }
 
@@ -217,7 +225,11 @@ int CWinMaskApplication::Run (void)
                           aConfig.TMin_Count(),
                           aConfig.Discontig(),
                           aConfig.Pattern(),
-                          aConfig.UseBA() );
+                          aConfig.UseBA(),
+                          aConfig.MinScorePct(),
+                          aConfig.ExtendScorePct(),
+                          aConfig.ThresScorePct(),
+                          aConfig.MaxScorePct() );
     CRef< CSeq_entry > aSeqEntry( 0 );
     Uint4 total = 0, total_masked = 0;
     CSDustMasker * duster( 0 );
@@ -249,12 +261,12 @@ int CWinMaskApplication::Run (void)
                 _TRACE( "Sequence length " << len );
                 CSeqVector data =
                     bsh.GetSeqVector(CBioseq_Handle::eCoding_Iupac);
-                auto_ptr< CSeqMasker::TMaskList > mask_info( theMasker( data ) );
+                unique_ptr< CSeqMasker::TMaskList > mask_info( theMasker( data ) );
                 CSeqMasker::TMaskList dummy;
 
                 if( duster != 0 ) // Dust and merge with mask_info
                 {
-                    auto_ptr< CSeqMasker::TMaskList > dust_info( 
+                    unique_ptr< CSeqMasker::TMaskList > dust_info( 
                         (*duster)( data, *mask_info.get() ) );
                     CSeqMasker::MergeMaskInfo( mask_info.get(), dust_info.get() );
                 }
