@@ -1,6 +1,6 @@
 #! /bin/sh
 #############################################################################
-# $Id: cmClang.sh 612333 2020-07-20 14:49:13Z gouriano $
+# $Id: cmClang.sh 693540 2025-02-13 18:35:05Z ivanov $
 #   Compiler:   LLVM Clang
 #   Find Clang compiler of a specific version and
 #   call CMake configuration script
@@ -13,8 +13,9 @@ script_name=`basename $0`
 script_args="$@"
 
 ## Path to the compiler
-CXX="clang++"
-CC="clang"
+CXX=${NCBI_CLANG_CXX-clang++}
+CC=${NCBI_CLANG_CC-clang}
+CCNAME=${NCBI_CLANG_NAME-clang}
 
 Usage() {
     echo "USAGE:   $script_name [version] [configure-flags] | -h]"
@@ -50,8 +51,10 @@ fi
 
 # Look for the specified version in various reasonable places
 # (tuned for NCBI's installations).
-case "$cxx_version" in
-  [1-9]*)
+case "$CXX:$cxx_version" in
+  /* )
+     ;;
+  *:[1-9]*)
      if /usr/local/llvm/$cxx_version/bin/$CXX -dumpversion >/dev/null 2>&1; then
        CXX=/usr/local/llvm/$cxx_version/bin/$CXX
        CC=/usr/local/llvm/$cxx_version/bin/$CC
@@ -83,7 +86,20 @@ EOF
     exit 1
 fi
 
-gccver=7.3.0
+export CC CXX
+gccver=13.2.0
+#if [ -x ${script_dir}/toolchains/cmkTool.sh ]; then
+if true; then
+  export GCCVER="${gccver}"
+  export LIBICC="${libICC}"
+  toolchain=`exec ${script_dir}/toolchains/cmkTool.sh $CCNAME ${cxx_version}  "$@"`
+  result=$?
+  if test $result -ne 0; then
+    echo $toolchain 1>&2
+    exit $result
+  fi
+  export NCBI_TOOLCHAIN=$toolchain
+else
 if test -x /opt/ncbi/gcc/${gccver}/bin/gcc; then
   inc1="/opt/ncbi/gcc/${gccver}/include/c++/${gccver}"
   inc2="/opt/ncbi/gcc/${gccver}/include/c++/${gccver}/x86_64-redhat-linux-gnu"
@@ -95,8 +111,14 @@ if test -x /opt/ncbi/gcc/${gccver}/bin/gcc; then
   NCBI_COMPILER_EXE_LINKER_FLAGS="-L${libgnu} -B${libgnu} -L${lib64} -Wl,-rpath,${lib64}"
   NCBI_COMPILER_SHARED_LINKER_FLAGS="${NCBI_COMPILER_EXE_LINKER_FLAGS}"
 fi
+if test -n "$libICC"; then
+  NCBI_COMPILER_C_FLAGS="$NCBI_COMPILER_C_FLAGS -ffp-model=precise"
+  NCBI_COMPILER_CXX_FLAGS="$NCBI_COMPILER_CXX_FLAGS -ffp-model=precise"
+  NCBI_COMPILER_EXE_LINKER_FLAGS="-L$libICC -Wl,-rpath,$libICC -lintlc $NCBI_COMPILER_EXE_LINKER_FLAGS"
+  NCBI_COMPILER_SHARED_LINKER_FLAGS="${NCBI_COMPILER_EXE_LINKER_FLAGS}"
+fi
 export NCBI_COMPILER_C_FLAGS NCBI_COMPILER_CXX_FLAGS NCBI_COMPILER_EXE_LINKER_FLAGS NCBI_COMPILER_SHARED_LINKER_FLAGS
-export CC CXX
+fi
 
 if [ -z "$CMAKECFGRECURSIONGUARD" ]; then
   exec ${script_dir}/cmake-cfg-unix.sh --rootdir=$script_dir/../../.. --caller=$script_name "$@"
