@@ -1,6 +1,6 @@
 #!/bin/sh
 #############################################################################
-# $Id: cmake-cfg-xcode.sh 681359 2024-04-08 18:49:39Z ivanov $
+# $Id: cmake-cfg-xcode.sh 681224 2024-04-04 17:21:08Z ucko $
 #   Configure NCBI C++ toolkit for XCode using CMake build system.
 #   Author: Andrei Gourianov, gouriano@ncbi
 #############################################################################
@@ -192,6 +192,14 @@ while [ $# != 0 ]; do
         BUILD_TYPE=$types
       fi
       ;; 
+    -DNCBI_PTBCFG_CONFIGURATION_TYPES* )
+      types=${1#*=}
+      CMAKE_ARGS="$CMAKE_ARGS  -DNCBI_PTBCFG_CONFIGURATION_TYPES=$(Quote "${types}")"
+      cnt=`echo $types | tr ";" " " | wc -w | tr -d " "`
+      if [ "$cnt" = "1" ]; then
+        BUILD_TYPE=$types
+      fi
+      ;; 
     -D* | --debug-* | --log-* | --trace* )
       CMAKE_ARGS="$CMAKE_ARGS $1"
       ;; 
