@@ -1,4 +1,4 @@
-/* $Id: msvc_site.cpp 602953 2020-03-04 19:52:40Z gouriano $
+/* $Id: msvc_site.cpp 663834 2023-03-01 16:35:07Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -200,7 +200,7 @@ void CMsvcSite::InitializeLibChoices(void)
     }
 }
 
-bool CMsvcSite::IsProvided(const string& thing, bool deep) const
+bool CMsvcSite::IsProvided(const string& thing, bool deep, const SConfigInfo* cfg) const
 {
     if (thing.empty()) {
         return true;
@@ -243,21 +243,31 @@ bool CMsvcSite::IsProvided(const string& thing, bool deep) const
         if (components.empty()) {
             components.push_back(thing);
         }
-        // in at least one configuration all components must be ok
-        ITERATE(list<SConfigInfo>, config , GetApp().GetRegSettings().m_ConfigInfo) {
+        if (cfg != nullptr) {
             res = true;
             ITERATE(list<string>, p, components) {
                 const string& component = *p;
                 SLibInfo lib_info;
-                GetLibInfo(component, *config, &lib_info);
+                GetLibInfo(component, *cfg, &lib_info);
                 res = IsLibOk(lib_info);
-                if ( !res ) {
+            }
+        } else {
+            // in at least one configuration all components must be ok
+            ITERATE(list<SConfigInfo>, config , GetApp().GetRegSettings().m_ConfigInfo) {
+                res = true;
+                ITERATE(list<string>, p, components) {
+                    const string& component = *p;
+                    SLibInfo lib_info;
+                    GetLibInfo(component, *config, &lib_info);
+                    res = IsLibOk(lib_info);
+                    if ( !res ) {
+                        break;
+                    }
+                }
+                if (res) {
                     break;
                 }
             }
-            if (res) {
-                break;
-            }
         }
     }
     return res;
@@ -456,6 +466,21 @@ void CMsvcSite::GetConfigureDefines(list<string>* defines) const
                 *defines, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
 }
 
+void  CMsvcSite::CollectRequires(const list<string>& reqs) const
+{
+    m_Requires.insert(reqs.begin(), reqs.end());
+}
+
+void  CMsvcSite::GetRequiresInfo(const SConfigInfo& config, list<string>& list_enabled, list<string>& list_disabled) const
+{
+    for (const string& req : m_Requires) {
+        if ( (IsProvided(req, true, &config) || IsProvided(req, false, &config)) && IsLibEnabledInConfig(req, config)) {
+            list_enabled.push_back(req);
+        } else {
+            list_disabled.push_back(req);
+        }
+    }
+}
 
 bool CMsvcSite::IsLibWithChoice(const string& lib_id) const
 {
@@ -706,6 +731,13 @@ void CMsvcSite::GetStandardFeatures(list<string>& features) const
                 LIST_SEPARATOR, features, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
 }
 
+void CMsvcSite::GetConfigurableRequests(list<string>& features) const
+{
+    features.clear();
+    NStr::Split(x_GetConfigureEntry("ConfigurableRequests"),
+                LIST_SEPARATOR, features, NStr::fSplit_Tokenize);
+}
+
 //-----------------------------------------------------------------------------
 bool CMsvcSite::x_DirExists(const string& dir_name)
 {
