@@ -1,4 +1,4 @@
-/*  $Id: cgiapp.cpp 658284 2022-10-28 13:09:35Z grichenk $
+/*  $Id: cgiapp.cpp 689064 2024-10-17 16:34:01Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1080,6 +1080,17 @@ void CCgiApplication::x_OnEvent(CCgiRequestProcessor* pprocessor, EEvent event,
             CCgiRequestProcessor& processor = *pprocessor;
             const CCgiRequest& req = processor.GetContext().GetRequest();
 
+            // 'tracestate' and 'traceparent' values must be passed to the tracer before
+            // starting request since they are used to generate trace-id and parent-id.
+            auto& req_ctx = CDiagContext::GetRequestContext();
+            string s = req.GetRandomProperty("TRACESTATE");
+            if (!s.empty()) req_ctx.SetSrcTraceState(s);
+            s = req.GetRandomProperty("TRACEPARENT");
+            if (!s.empty()) req_ctx.SetSrcTraceParent(s);
+
+            // Set span kind for tracing, if any.
+            CDiagContext::GetRequestContext().SetTracerSpanKind(ITracerSpan::eKind_Server);
+
             // Print request start message
             if ( !CDiagContext::IsSetOldPostFormat() ) {
                 CExtraEntryCollector collector;
@@ -1096,7 +1107,48 @@ void CCgiApplication::x_OnEvent(CCgiRequestProcessor* pprocessor, EEvent event,
 
             // This will log ncbi_phid as a separate 'extra' message
             // if not yet logged.
-            CDiagContext::GetRequestContext().GetHitID();
+            req_ctx.GetHitID();
+
+            // Post request properties to the current tracer span, if any.
+            auto span = req_ctx.GetTracerSpan();
+            if ( span ) {
+                s = req.GetProperty(eCgi_ScriptName);
+                if (!s.empty()) span->SetName(s);
+                s = req.GetProperty(eCgi_ServerName);
+                if (!s.empty()) {
+                    span->SetAttribute(ITracerSpan::eServerAddress, s);
+                    s = req.GetProperty(eCgi_ServerPort);
+                    if (!s.empty()) span->SetAttribute(ITracerSpan::eServerPort, s);
+                }
+                s = processor.GetContext().GetSelfURL();
+                if (!s.empty()) {
+                    string args = req.GetRandomProperty("REDIRECT_QUERY_STRING", false);
+                    if ( args.empty() ) {
+                        args = req.GetProperty(eCgi_QueryString);
+                    }
+                    if ( !args.empty() ) {
+                        s += "?" + args;
+                    }
+                    span->SetAttribute(ITracerSpan::eUrl, s);
+                }
+                s = req.GetProperty(eCgi_RequestMethod);
+                if (!s.empty()) span->SetAttribute(ITracerSpan::eRequestMethod, s);
+                s = req.GetProperty(eCgi_ServerProtocol);
+                if (!s.empty()) {
+                    span->SetAttribute(ITracerSpan::eUrlScheme, s);
+                    // Although HttpScheme is deprecated by Opentelemetry in favor of UrlScheme,
+                    // elasticapm may expect it.
+                    span->SetAttribute(ITracerSpan::eHttpScheme, s);
+                }
+                s = req.GetProperty(eCgi_RemoteAddr);
+                if (!s.empty()) span->SetAttribute(ITracerSpan::eClientAddress, s);
+                s = req.GetProperty(eCgi_ContentType);
+                if (!s.empty()) span->SetHttpHeader(ITracerSpan::eRequest, "CONTENT_TYPE", s);
+                s = req.GetProperty(eCgi_ContentLength);
+                if (!s.empty()) span->SetHttpHeader(ITracerSpan::eRequest, "CONTENT_LENGTH", s);
+                s = req.GetProperty(eCgi_HttpUserAgent);
+                if (!s.empty()) span->SetHttpHeader(ITracerSpan::eRequest, "HTTP_USER_AGENT", s);
+            }
 
             // Check if ncbi_st cookie is set
             const CCgiCookie* st = req.GetCookies().Find(g_GetNcbiString(eNcbiStrings_Stat));
@@ -1142,6 +1194,11 @@ void CCgiApplication::x_OnEvent(CCgiRequestProcessor* pprocessor, EEvent event,
             }
             catch (const exception&) {
             }
+
+            auto span = CDiagContext::GetRequestContext().GetTracerSpan();
+            if ( span ) {
+                span->SetSpanStatus(event == eSuccess ? ITracerSpan::eSuccess : ITracerSpan::eError);
+            }
             break;
         }
     case eEndRequest:
@@ -1151,6 +1208,7 @@ void CCgiApplication::x_OnEvent(CCgiRequestProcessor* pprocessor, EEvent event,
             CCgiContext& cgi_ctx = processor.GetContext();
             CDiagContext& diag_ctx = GetDiagContext();
             CRequestContext& rctx = CDiagContext::GetRequestContext();
+
             // If an error status has been set by ProcessRequest, don't try
             // to check the output stream and change the status to 299/499.
             if ( !processor.GetErrorStatus() ) {
@@ -1800,6 +1858,15 @@ bool CCgiApplication::x_ProcessAdminRequest(CCgiRequestProcessor& processor)
     else if ( NStr::EqualNocase(cmd_name, "deep-health") ) {
         cmd = eAdmin_HealthDeep;
     }
+    else if ( NStr::EqualNocase(cmd_name, "healthz") ) {
+        cmd = eAdmin_HealthZ;
+    }
+    else if ( NStr::EqualNocase(cmd_name, "readyz") ) {
+        cmd = eAdmin_ReadyZ;
+    }
+    else if ( NStr::EqualNocase(cmd_name, "livez") ) {
+        cmd = eAdmin_LiveZ;
+    }
 
     // If the overriden method failed or refused to process a command,
     // fallback to the default processing which returns true for all known commands.
@@ -2532,7 +2599,7 @@ int CCgiRequestProcessor::OnException(std::exception& e, CNcbiOstream& os)
 
         // Message
         os << "ERROR:  " << status_str << " " HTTP_EOL HTTP_EOL;
-        os << NStr::HtmlEncode(message);
+        os << NStr::HtmlEncode(message) << HTTP_EOL HTTP_EOL;
 
         if ( dynamic_cast<CArgException*> (&e) ) {
             string ustr;
@@ -2570,6 +2637,12 @@ void CCgiRequestProcessor::SetHTTPStatus(unsigned int status, const string& reas
     else {
         CDiagContext::GetRequestContext().SetRequestStatus(status);
     }
+
+    auto span = CDiagContext::GetRequestContext().GetTracerSpan();
+    if ( span ) {
+        span->SetAttribute(ITracerSpan::eStatusCode, NStr::NumericToString(status));
+        span->SetAttribute(ITracerSpan::eStatusString, reason);
+    }
 }
 
 
