@@ -1,4 +1,4 @@
-/*  $Id: fcgi_run.cpp 675130 2023-11-15 13:51:04Z ivanov $
+/*  $Id: fcgi_run.cpp 696030 2025-04-07 12:59:48Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,15 @@
 #include <signal.h>
 
 
+#ifdef FCGIAPP
+#  include <cgi/fcgiapp_st.hpp>
+#  define CCgiApplication CFastCgiApplication
+#  ifndef HAVE_LIBFASTCGI
+#    define HAVE_LIBFASTCGI 1
+#  endif
+#endif
+
+
 #if !defined(HAVE_LIBFASTCGI)
 
 BEGIN_NCBI_SCOPE
