@@ -1,4 +1,4 @@
-/*  $Id: ncbicgi.cpp 562301 2018-04-19 20:55:38Z lavr $
+/*  $Id: ncbicgi.cpp 687014 2024-09-03 13:27:47Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,7 +42,6 @@
 #include <corelib/ncbi_strings.h>
 
 #include <cgi/cgi_exception.hpp>
-#include <cgi/ncbicgi.hpp>
 #include <cgi/cgi_serial.hpp>
 #include <cgi/cgi_session.hpp>
 #include <cgi/error_codes.hpp>
@@ -50,6 +49,8 @@
 #include <util/checksum.hpp>
 #include <util/ncbi_url.hpp>
 
+#include "cgi_impl.hpp"
+
 #include <algorithm>
 
 #include <stdio.h>
@@ -101,6 +102,11 @@ NCBI_PARAM_ENUM_DEF_EX(CCgiCookie::ECookieEncoding, CGI, Cookie_Encoding,
 typedef NCBI_PARAM_TYPE(CGI, Cookie_Encoding) TCookieEncoding;
 
 
+NCBI_PARAM_DECL(string, CGI, cookie_auth_token);
+NCBI_PARAM_DEF(string, CGI, cookie_auth_token, "WebCubbyUser");
+typedef NCBI_PARAM_TYPE(CGI, cookie_auth_token) TCookieAuthToken;
+
+
 // Helper function for encoding cookie name/value
 string CCgiCookie::x_EncodeCookie(const string& str,
                                   EFieldType ftype,
@@ -389,7 +395,7 @@ CCgiCookie* CCgiCookies::Add(const string& name,    const string& value,
             ck->SetPath(path);
             _VERIFY( m_Cookies.insert(ck).second );
         }
-    } catch (CCgiCookieException& ex) {
+    } catch (const CCgiCookieException& ex) {
         // This can only happen if cookie has empty name, ignore
         // Store/StoreAndError flags in this case.
         switch ( on_bad_cookie ) {
@@ -397,7 +403,7 @@ CCgiCookie* CCgiCookies::Add(const string& name,    const string& value,
             throw;
         case eOnBadCookie_StoreAndError:
         case eOnBadCookie_SkipAndError: {
-            CException& cex = ex;  // GCC 3.4.0 can't guess it for ERR_POST
+            const CException& cex = ex;  // GCC 3.4.0 can't guess it for ERR_POST
             ERR_POST_X(1, Severity(TCookieErrorSeverity::GetDefault()) << cex);
             return NULL;
         }
@@ -452,19 +458,19 @@ CCgiCookies::x_CheckField(const string&          str,
 {
     try {
         CCgiCookie::x_CheckField(str, ftype, banned_symbols, cookie_name);
-    } catch (CCgiCookieException& ex) {
+    } catch (const CCgiCookieException& ex) {
         switch ( on_bad_cookie ) {
         case eOnBadCookie_ThrowException:
             throw;
         case eOnBadCookie_SkipAndError: {
-            CException& cex = ex;  // GCC 3.4.0 can't guess it for ERR_POST
+            const CException& cex = ex;  // GCC 3.4.0 can't guess it for ERR_POST
             ERR_POST_X(2, Severity(TCookieErrorSeverity::GetDefault()) << cex);
             return eCheck_SkipInvalid;
         }
         case eOnBadCookie_Skip:
             return eCheck_SkipInvalid;
         case eOnBadCookie_StoreAndError: {
-            CException& cex = ex;  // GCC 3.4.0 can't guess it for ERR_POST
+            const CException& cex = ex;  // GCC 3.4.0 can't guess it for ERR_POST
             ERR_POST_X(3, Severity(TCookieErrorSeverity::GetDefault()) << cex);
             return eCheck_StoreInvalid;
         }
@@ -792,6 +798,23 @@ void CCgiCookies::Clear(void)
 }
 
 
+void CCgiCookies::SetAllCookiesSecure(bool value)
+{
+    m_AllSecure = value;
+    NON_CONST_ITERATE (TSet, cookie, m_Cookies) {
+        (*cookie)->SetSecure(value);
+    }
+}
+
+
+void CCgiCookies::SetAllCookiesHttpOnly(bool value)
+{
+    m_AllHttpOnly = value;
+    NON_CONST_ITERATE (TSet, cookie, m_Cookies) {
+        (*cookie)->SetHttpOnly(value);
+    }
+}
+
 
 ////////////////////////////////////////////////////////
 //  CTrackingEnvHolder
@@ -812,7 +835,7 @@ private:
 };
 
 
-// Must be in correspondence with variables checked in NcbiGetClientIP[Ex]()
+// Must be in correspondence with variables checked in NcbiGetCgiClientIP[Ex]()
 // (header: <connect/ext/ncbi_localnet.h>, source: connect/ext/ncbi_localnet.c,
 // library: [x]connext)
 static const char* kTrackingVars[] = 
@@ -945,30 +968,12 @@ static void s_AddEntry(TCgiEntries& entries, const string& name,
 }
 
 
-class CCgiEntries_Parser : public CUrlArgs_Parser
-{
-public:
-    CCgiEntries_Parser(TCgiEntries* entries,
-                       TCgiIndexes* indexes,
-                       bool indexes_as_entries);
-protected:
-    virtual void AddArgument(unsigned int position,
-                             const string& name,
-                             const string& value,
-                             EArgType arg_type);
-private:
-    TCgiEntries* m_Entries;
-    TCgiIndexes* m_Indexes;
-    bool         m_IndexesAsEntries;
-};
-
-
 CCgiEntries_Parser::CCgiEntries_Parser(TCgiEntries* entries,
                                        TCgiIndexes* indexes,
-                                       bool indexes_as_entries)
-    : m_Entries(entries),
-      m_Indexes(indexes),
-      m_IndexesAsEntries(indexes_as_entries  ||  !indexes)
+                                       CCgiRequest::TFlags flags)
+    : CUrlArgs_Parser(CCgiRequestTFlagsToTFlags(flags & ~(indexes ? 0 : CCgiRequest::fIndexesNotEntries))),
+      m_Entries(entries),
+      m_Indexes(indexes)
 {
     return;
 }
@@ -980,7 +985,7 @@ void CCgiEntries_Parser::AddArgument(unsigned int position,
                                      EArgType arg_type)
 {
     if (m_Entries  &&
-        (arg_type == eArg_Value  ||  m_IndexesAsEntries)) {
+        (arg_type == eArg_Value  ||  !(TFlagsToCCgiRequestTFlags(m_Flags) & CCgiRequest::fIndexesNotEntries))) {
         m_Entries->insert(TCgiEntries::value_type(
             name, CCgiEntry(value, kEmptyStr, position, kEmptyStr)));
     }
@@ -1122,7 +1127,7 @@ void CCgiRequest::x_Init
     try {
         m_Cookies.Add(GetProperty(eCgi_HttpCookie),
             TOnBadCookieParam::GetDefault());
-    } catch (CCgiCookieException& e) {
+    } catch (const CCgiCookieException& e) {
         NCBI_RETHROW(e, CCgiRequestException, eCookie,
                      "Error in parsing HTTP request cookies");
     }
@@ -1139,7 +1144,7 @@ void CCgiRequest::x_Init
     TCgiEntries::const_iterator empty_it = m_Entries.find(kEmptyStr);
     if (empty_it != m_Entries.end()) {
         // there is already empty name key
-        ERR_POST_X(5, "Encountered query parameter with empty name, "
+        ERR_POST_X(5, Warning << "Encountered query parameter with empty name, "
             "its value is: '" << empty_it->second << "'. ATTENTION: "
             "Because of this, check for image names will be disabled.");
         return;
@@ -1264,13 +1269,13 @@ void CCgiRequest::x_InitRequestContext(TFlags flags)
     if ( !rctx.IsSetHitID(CRequestContext::eHitID_Request) ) {
         if ((flags & fIgnorePageHitId) == 0) {
             string phid;
-            // Check if pageviewid is present. If not, generate one.
-            TCgiEntries::iterator phid_it = m_Entries.find(
-                g_GetNcbiString(eNcbiStrings_PHID));
-            if (phid_it != m_Entries.end()) {
-                phid = phid_it->second;
+            // Check if page hit id is present. If not, generate one.
+            auto phid_rg = m_Entries.equal_range(g_GetNcbiString(eNcbiStrings_PHID));
+            while (phid_rg.first != phid_rg.second) {
+                phid = phid_rg.first->second;
+                ++phid_rg.first;
             }
-            else {
+            if (phid.empty()) {
                 // Try HTTP_NCBI_PHID
                 phid = CRequestContext::SelectLastHitID(
                     GetRandomProperty("NCBI_PHID", true));
@@ -1289,6 +1294,13 @@ void CCgiRequest::x_InitRequestContext(TFlags flags)
             rctx.SetDtab(dtab);
         }
     }
+    if (!rctx.IsSetProperty("auth_token")) {
+        if (auto auth_token = TCookieAuthToken::GetDefault(); !auth_token.empty()) {
+            if (auto cookie = m_Cookies.Find(auth_token)) {
+                rctx.SetProperty("auth_token", cookie->GetValue());
+            }
+        }
+    }
 }
 
 
@@ -1310,11 +1322,7 @@ void CCgiRequest::x_ProcessQueryString(TFlags flags, const CNcbiArguments* args)
         }
 
         if ( query_string ) {
-            CCgiEntries_Parser parser(&m_Entries, &m_Indexes,
-                (flags & fIndexesNotEntries) == 0);
-            if (flags & fSemicolonIsNotArgDelimiter) {
-                parser.SetSemicolonIsNotArgDelimiter(true);
-            }
+            CCgiEntries_Parser parser(&m_Entries, &m_Indexes, flags);
             parser.SetQueryString(*query_string);
         }
     }
@@ -1366,7 +1374,9 @@ void CCgiRequest::x_ProcessInputStream(TFlags flags, CNcbiIstream* istr, int ifd
                 try {
                     ParseRemainingContent();
                 } NCBI_CATCH_ALL_X(8, "CCgiRequest: POST/PUT with no content type");
-                CStreamUtils::Pushback(*istr, pstr->data(), pstr->length());
+                if (pstr) {
+                    CStreamUtils::Pushback(*istr, pstr->data(), pstr->length());
+                }
                 m_Input    = istr;
                 // m_InputFD  = ifd; // would be exhausted
                 m_InputFD  = -1;
@@ -1491,7 +1501,7 @@ size_t CCgiRequest::GetContentLength(void) const
     size_t content_length;
     try {
         content_length = (size_t) NStr::StringToUInt(str);
-    } catch (CStringException& e) {
+    } catch (const CStringException& e) {
         NCBI_RETHROW(e, CCgiRequestException, eFormat,
                      "Malformed Content-Length value in HTTP request: " + str);
     }
@@ -1519,11 +1529,11 @@ void CCgiRequest::SetInputStream(CNcbiIstream* is, bool own, int fd)
 
 SIZE_TYPE CCgiRequest::ParseEntries(const string& str, TCgiEntries& entries)
 {
-    CCgiEntries_Parser parser(&entries, 0, true);
+    CCgiEntries_Parser parser(&entries, 0, 0);
     try {
         parser.SetQueryString(str);
     }
-    catch (CUrlParserException& ae) {
+    catch (const CUrlParserException& ae) {
         return ae.GetPos();
     }
     return 0;
@@ -1532,11 +1542,11 @@ SIZE_TYPE CCgiRequest::ParseEntries(const string& str, TCgiEntries& entries)
 
 SIZE_TYPE CCgiRequest::ParseIndexes(const string& str, TCgiIndexes& indexes)
 {
-    CCgiEntries_Parser parser(0, &indexes, false);
+    CCgiEntries_Parser parser(0, &indexes, fIndexesNotEntries);
     try {
         parser.SetQueryString(str);
     }
-    catch (CUrlParserException& ae) {
+    catch (const CUrlParserException& ae) {
         return ae.GetPos();
     }
     return 0;
@@ -1600,7 +1610,7 @@ CCgiSession& CCgiRequest::GetSession(ESessionCreateMode mode) const
 
     try {
         m_Session->Load();
-    } catch (CCgiSessionException& ex) {
+    } catch (const CCgiSessionException& ex) {
         if (ex.GetErrCode() != CCgiSessionException::eSessionId) {
             NCBI_RETHROW(ex, CCgiSessionException, eImplException, 
                          "Session implementation error");
@@ -1673,7 +1683,7 @@ void CCgiRequest::GetCGIEntries(CEntryCollector_Base& collector) const
         try {
             ival = NStr::StringToInt(val);
         }
-        catch (CStringException) {
+        catch (const CStringException&) {
             ERR_POST(Error << "Invalid argument size limit: " << *it);
             continue;
         }
@@ -1924,13 +1934,13 @@ EEncodingForm GetCharsetEncodingForm(const string& charset,
 
 CStringUTF8 CCgiEntry::GetValueAsUTF8(EOnCharsetError on_error) const
 {
-    CNcbiIstrstream is(GetValue().c_str());
+    CNcbiIstrstream is(GetValue());
     EEncodingForm enc = GetCharsetEncodingForm(x_GetCharset(), on_error);
     CStringUTF8 utf_str;
     try {
         ReadIntoUtf8(is, &utf_str, enc);
     }
-    catch (CException) {
+    catch (const CException&) {
         if (on_error == eCharsetError_Throw) {
             throw;
         }
