@@ -1,4 +1,4 @@
-/*  $Id: connect_misc.cpp 675422 2023-11-21 19:49:15Z ivanov $
+/*  $Id: connect_misc.cpp 682215 2024-04-24 17:30:55Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -127,8 +127,8 @@ struct SServiceDiscoveryImpl : protected CConnIniter
     shared_ptr<void> GetSingleServer(const string& service_name, SSocketAddress::SHost::EName name) const
     {
         if (auto address = SSocketAddress::Parse(service_name, name)) {
-            CServiceDiscovery::TServer server(move(address), 1.0);
-            return make_shared<CServiceDiscovery::TServers>(1, move(server));
+            CServiceDiscovery::TServer server(std::move(address), 1.0);
+            return make_shared<CServiceDiscovery::TServers>(1, std::move(server));
         }
 
         return {};
@@ -355,5 +355,6 @@ NCBI_EXPORT_FUNC_DEFINE(XCONNECT, mbedtls_strerror);
 NCBI_EXPORT_FUNC_DEFINE(XCONNECT, mbedtls_x509_crt_free);
 NCBI_EXPORT_FUNC_DEFINE(XCONNECT, mbedtls_x509_crt_init);
 NCBI_EXPORT_FUNC_DEFINE(XCONNECT, mbedtls_x509_crt_parse);
+NCBI_EXPORT_FUNC_DEFINE(XCONNECT, psa_crypto_init);
 
 END_NCBI_SCOPE
