@@ -1,4 +1,4 @@
-/* $Id: ncbi_ansi_ext.c 613452 2020-08-07 03:28:28Z lavr $
+/* $Id: ncbi_ansi_ext.c 667546 2023-05-16 20:13:01Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,34 +47,31 @@ size_t strnlen(const char* str, size_t maxlen)
 #endif /*HAVE_STRNLEN*/
 
 
-#ifndef HAVE_STRDUP
+#if !defined(HAVE_STRDUP)  &&  !defined(NCBI_COMPILER_MSVC)
 char* strdup(const char* str)
 {
     size_t size = strlen(str) + 1;
     char*   res = (char*) malloc(size);
-    if (res)
-        memcpy(res, str, size);
-    return res;
+    return res ? (char*) memcpy(res, str, size) : 0;
 }
-#endif /*HAVE_STRDUP*/
+#endif /*!HAVE_STRDUP && !NCBI_COMPILER_MSVC*/
 
 
 #ifndef HAVE_STRNDUP
 char* strndup(const char* str, size_t n)
 {
-    const char* end = n   ? (const char*) memchr(str, '\0', n) : 0;
-    size_t     size = end ? (size_t)(end - str)  : n;
-    char*       res = (char*) malloc(size + 1);
+    size_t size = strnlen(str, n);
+    char*   res = (char*) malloc(size + 1);
     if (res) {
         memcpy(res, str, size);
         res[size] = '\0';
     }
     return res;
 }
-#endif /*HAVE_STRNDUP*/
+#endif /*!HAVE_STRNDUP*/
 
 
-#ifndef HAVE_STRCASECMP
+#if !defined(NCBI_COMPILER_MSVC)  &&  !defined(HAVE_STRCASECMP)
 
 /* We assume that we're using ASCII-based charsets */
 int strcasecmp(const char* s1, const char* s2)
@@ -116,7 +113,7 @@ int strncasecmp(const char* s1, const char* s2, size_t n)
     return c1 - c2;
 }
 
-#endif /*HAVE_STRCASECMP*/
+#endif /*!NCBI_COMPILER_MSVC && !HAVE_STRCASECMP*/
 
 
 char* strupr(char* s)
@@ -208,8 +205,7 @@ double NCBI_simple_atof(const char* s, char** t)
         s++;
     if ((*s == '-'  ||  *s == '+')
         &&  (s[1] == '.'  ||  isdigit((unsigned char) s[1]))) {
-        n = *s == '-' ? 1/*true*/ : 0/*false*/;
-        s++;
+        n = *s++ == '-' ? 1/*true*/ : 0/*false*/;
     } else
         n = 0/*false*/;
 
@@ -247,3 +243,42 @@ double NCBI_simple_atof(const char* s, char** t)
         *t = e;
     return (double)(n ? -x : x);
 }
+
+
+#if 1
+
+int/*bool*/ NCBI_HasSpaces(const char* s, size_t n)
+{
+    while (n--) {
+        if (isspace((unsigned char) s[n]))
+            return 1/*true*/;
+    }
+    return 0/*false*/;
+}
+
+#else
+
+#ifdef __GNUC__
+inline
+#endif /*__GNUC__*/
+/* Takes a <ctype.h>-like classification _function_ */
+static int/*bool*/ x_HasClass(const char* s, size_t n, int (*isa)(int))
+{
+    while (n--) {
+        if (isa((unsigned char) s[n]))
+            return 1/*true*/;
+    }
+    return 0/*false*/;
+}
+
+
+/* undef to use a function variant */
+#ifdef   isspace
+#  undef isspace
+#endif /*isspace*/
+int/*bool*/ NCBI_HasSpaces(const char* s, size_t n)
+{
+    return x_HasClass(s, n, isspace);
+}
+
+#endif
