@@ -1,4 +1,4 @@
-/* $Id: ncbi_buffer.c 612591 2020-07-23 21:03:31Z lavr $
+/* $Id: ncbi_buffer.c 625782 2021-02-19 05:24:50Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -152,7 +152,7 @@ extern int/*bool*/ BUF_AppendEx(BUF* buf, void* base, size_t alloc_size,
     if (!*buf  &&  !BUF_SetChunkSize(buf, 0))
         return 0/*failure*/;
 
-    if (!(chunk = s_BUF_AllocChunk(0, (*buf)->unit)))
+    if (!(chunk = s_BUF_AllocChunk(0, 0)))
         return 0/*failure*/;
 
     assert(!chunk->data);
@@ -195,7 +195,7 @@ extern int/*bool*/ BUF_PrependEx(BUF* buf, void* base, size_t alloc_size,
     if (!*buf  &&  !BUF_SetChunkSize(buf, 0))
         return 0/*failure*/;
 
-    if (!(chunk = s_BUF_AllocChunk(0, (*buf)->unit)))
+    if (!(chunk = s_BUF_AllocChunk(0, 0)))
         return 0/*failure*/;
 
     assert(!chunk->data);
