@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_streambuf.cpp 652143 2022-07-01 14:35:45Z lavr $
+/* $Id: ncbi_conn_streambuf.cpp 684622 2024-06-27 18:45:39Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,8 +34,8 @@
 #include "ncbi_conn_streambuf.hpp"
 #include <corelib/ncbidbg.hpp>
 #include <corelib/ncbi_limits.hpp>
-#include <connect/ncbi_conn_exception.hpp>
 #include <connect/error_codes.hpp>
+#include <connect/ncbi_conn_exception.hpp>
 #include <stdlib.h>  // free()
 
 #define NCBI_USE_ERRCODE_X   Connect_Stream
@@ -429,6 +429,9 @@ CNcbiStreambuf* CConn_Streambuf::setbuf(CT_CHAR_TYPE* buf, streamsize buf_size)
 
 CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
 {
+    _ASSERT(CT_EQ_INT_TYPE(c, CT_EOF)  ||  pptr() >= epptr());
+    _ASSERT(pbase() <= pptr()  &&  pptr() <= epptr());
+
     if (!x_CheckConn(m_Conn))
         return CT_EOF;
 
@@ -445,7 +448,7 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
                 _ASSERT(m_Status != eIO_Success);
                 break;
             }
-            // update buffer content (get rid of the data just sent)
+            // update buffer contents (get rid of the data just sent)
             memmove(pbase(), pbase() + n_written, n_towrite - n_written);
             x_PPos += (CT_OFF_TYPE) n_written;
             pbump(-int(n_written));
@@ -500,6 +503,8 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
 
 streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
 {
+    _ASSERT(pbase() <= pptr()  &&  pptr() <= epptr());
+
     if (!x_CheckConn(m_Conn)  ||  m < 0)
         return 0;
 
@@ -711,7 +716,7 @@ streamsize CConn_Streambuf::x_Read(CT_CHAR_TYPE* buf, streamsize m)
 
 streamsize CConn_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 {
-    _ASSERT(egptr() >= gptr());
+    _ASSERT(gptr() <= gptr());
 
     return x_CheckConn(m_Conn) ? x_Read(buf, m) : 0;
 }
@@ -719,7 +724,7 @@ streamsize CConn_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 
 streamsize CConn_Streambuf::showmanyc(void)
 {
-#define POLLING  CConn_IOStream::kZeroTimeout
+#define POLLING  &CConn_IOStream::kZeroTimeout
 
     _ASSERT(gptr() >= egptr());
 
@@ -756,7 +761,7 @@ streamsize CConn_Streambuf::showmanyc(void)
         case eIO_Closed:
             return -1L;  // EOF
         case eIO_Success:
-            _ASSERT(0);
+            _TROUBLE;
             /*FALLTHRU*/
         default:
             if (x_IsThrowable(m_Status))
