@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_STREAMBUF__HPP
 #define CONNECT___NCBI_CONN_STREAMBUF__HPP
 
-/* $Id: ncbi_conn_streambuf.hpp 606942 2020-04-28 23:59:14Z lavr $
+/* $Id: ncbi_conn_streambuf.hpp 652084 2022-06-30 15:08:27Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -56,23 +56,28 @@ public:
                     const STimeout* timeout, size_t buf_size,
                     CConn_IOStream::TConn_Flags flags,
                     CT_CHAR_TYPE* ptr, size_t size);
-    virtual   ~CConn_Streambuf()    { Close();  delete[] m_WriteBuf; }
+    virtual   ~CConn_Streambuf();
 
     EIO_Status Open    (void);
-    CONN       GetCONN (void) const { return m_Conn;                 }
-    EIO_Status Close   (void)       { return x_Close(true);          }
-    EIO_Status Status  (EIO_Event direction = eIO_Open) const;
+    CONN       GetCONN (void) const { return m_Conn; }
+    EIO_Status Close   (void)       { return m_Conn ? x_Close(true) : eIO_Closed; }
+    EIO_Status Status  (EIO_Event direction) const;
 
-    /// Return the specified data "data" of size "size" into the underlying
+    /// Push the specified data "data" of size "size" back into the underlying
     /// connection CONN.
     /// If there is any non-empty pending input sequence (internal read buffer)
-    /// it will first be attempted to return to CONN.  That excludes any
-    /// initial read area ("ptr") that might have been specified in the ctor.
+    /// it will first be attempted to return to CONN.  Note that it may include
+    /// any initial read area ("ptr"), which could have been specified in the
+    /// ctor yet still unread, so potentially unwanted data copying can result.
     /// Any status different from eIO_Success means that nothing from "data"
     /// has been pushed back to the connection.
-    EIO_Status Pushback(const CT_CHAR_TYPE* data, streamsize size);
+    /// On success, the "push" argument (if true) updates the current put
+    /// position with "size" that helps maintain its correct value in streams,
+    /// which logically append data when writing (e.g. CConn_MemoryStream).
+    EIO_Status Pushback(const CT_CHAR_TYPE* data, streamsize size, bool push);
 
     /// @sa CConn_IOStream::Fetch
+    /// @note Return eIO_InvalidArg when there is no CONN
     EIO_Status Fetch   (const STimeout* timeout);
 
 protected:
@@ -106,7 +111,7 @@ protected:
     // operations on the call below.  Unfortunately, they seem to have
     // forgotten that xsgetn() is to optimize read operations for unbuffered
     // streams, in particular;  yet the new "secure" implementation falls
-    // back to use uflow() instead (causing byte-by-byte input) -- very
+    // back to use uflow() instead (causing a byte-by-byte input) -- very
     // inefficient.  We redefine the "secure" API to go the standard way here.
     virtual streamsize  _Xsgetn_s(CT_CHAR_TYPE* buf, size_t, streamsize n)
     { return xsgetn(buf, n); }
@@ -124,10 +129,11 @@ protected:
 
     streamsize  x_Read(CT_CHAR_TYPE* buf, streamsize n);
 
-    EIO_Status  x_Pushback(void);
+    EIO_Status  x_Pushback(void) THROWS_NONE;
 
 private:
     CONN              m_Conn;      // underlying connection handle
+    CONNECTOR         x_Connector; // CONNECTOR (if passed in)
 
     CT_CHAR_TYPE*     m_WriteBuf;  // I/O arena (set as 0 if unbuffered)
     CT_CHAR_TYPE*     m_ReadBuf;   // read buffer or &x_Buf (if unbuffered)
@@ -154,8 +160,8 @@ private:
 
     string            x_Message(const char*     method,
                                 const char*     message,
-                                EIO_Status      status = eIO_Success,
-                                const STimeout* timeout = 0/*kInfiniteTmo*/);
+                                EIO_Status      status  = eIO_Success,
+                                const STimeout* timeout = kInfiniteTimeout);
 
     SCONN_Callback    m_Cb;
 };
