@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_test.cpp 654831 2022-08-22 21:20:43Z lavr $
+/* $Id: ncbi_conn_test.cpp 693398 2025-02-11 14:11:53Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,9 +47,7 @@
 
 #define NCBI_WWW       DEF_CONN_HOST
 #define NCBI_WWW_BEMD  "www.be-md.ncbi.nlm.nih.gov"
-#define NCBI_WWW_STVA  "www.st-va.ncbi.nlm.nih.gov"
 #define NCBI_FWD_BEMD  "130.14.29.112"
-#define NCBI_FWD_STVA  "165.112.7.12"
 
 
 BEGIN_NCBI_SCOPE
@@ -89,10 +87,10 @@ CConnTest::CConnTest(const STimeout* timeout,
 }
 
 
-void CConnTest::SetTimeout(const STimeout* timeout)
+void CConnTest::SetTimeout(const STimeout* tmo)
 {
-    if (timeout) {
-        x_Timeout = timeout == kDefaultTimeout ? g_NcbiDefConnTimeout : *timeout;
+    if (tmo) {
+        x_Timeout = tmo == kDefaultTimeout ? g_NcbiDefConnTimeout : *tmo;
         m_Timeout = &x_Timeout;
     } else
         m_Timeout = kInfiniteTimeout/*0*/;
@@ -121,6 +119,9 @@ EIO_Status CConnTest::Execute(EStage& stage, string* reason)
         reason->clear();
     m_CheckPoint.clear();
 
+    if (stage == eNone)
+        return eIO_InvalidArg;
+
     int s = eNone + 1;
     EIO_Status status;
     do {
@@ -130,7 +131,7 @@ EIO_Status CConnTest::Execute(EStage& stage, string* reason)
         }
     } while (EStage(s++) < stage);
 
-    if (stage != eDns  &&  status != eIO_Success  &&  status != eIO_Interrupt)
+    if (stage > eHttp  &&  status != eIO_Success  &&  status != eIO_Interrupt)
         ExtraCheckOnFailure();
     return status;
 }
@@ -291,9 +292,7 @@ EIO_Status CConnTest::ExtraCheckOnFailure(void)
           "www.google.com", 0        },
         // 2. NCBI servers, explicitly
         { eURL_Https,
-          NCBI_WWW_BEMD,    NCBI_WWW }, // NCBI main
-        { eURL_Https,
-          NCBI_WWW_STVA,    NCBI_WWW }  // NCBI colo
+          NCBI_WWW_BEMD,    NCBI_WWW }  // NCBI main
     };
 
     m_CheckPoint.clear();
@@ -302,16 +301,16 @@ EIO_Status CConnTest::ExtraCheckOnFailure(void)
     AutoPtr<SConnNetInfo> net_info(ConnNetInfo_Create(0, eDebugPrintout_Data));
     if (!net_info) {
         PostCheck(eNone, 0/*main*/,
-                  eIO_Unknown, "Unable to create network info structure");
+                  eIO_Unknown, "Unable to build connection parameters");
         return eIO_Unknown;
     }
 
+    const STimeout* timeout = m_Timeout > kTimeout ? &kTimeout : m_Timeout;
     net_info->req_method = eReqMethod_Head;
-    net_info->timeout    = &kTimeout;
+    net_info->timeout    = timeout;
     net_info->max_try    = 0;
-    m_Timeout = 0;
 
-    CDeadline deadline(kTimeout.sec, kTimeout.usec * (kNanoSecondsPerSecond /
+    CDeadline deadline(timeout->sec, timeout->usec * (kNanoSecondsPerSecond /
                                                       kMicroSecondsPerSecond));
     time_t           sec;
     unsigned int nanosec;
@@ -328,8 +327,7 @@ EIO_Status CConnTest::ExtraCheckOnFailure(void)
         const char* vhost = kTests[n].vhost;
         if (host) {
             _ASSERT(::strlen(host) < sizeof(net_info->host) - 1);
-            unsigned int ip = vhost ? CSocketAPI::gethostbyname(host) : 0;
-            ::strcpy(net_info->host, ip ? CSocketAPI::ntoa(ip).c_str() : host);
+            ::strcpy(net_info->host, host);
         }
         if (vhost) {
             _ASSERT(::strlen(vhost) + 6 < sizeof(user_header) - 1);
@@ -383,19 +381,41 @@ EIO_Status CConnTest::DnsOkay(string* reason)
 {
     EIO_Status status;
     string result;
+    int step = 0;
 
-    PreCheck(eDns, 0/*main*/,
+    PreCheck(eDns, step/*main*/,
              "Checking whether NCBI is known to DNS");
 
     if (CSocketAPI::gethostbyname(NCBI_WWW) == 0) {
-        result = "Unable to resolve " NCBI_WWW;
         status = eIO_Unknown;
+        PreCheck(eDns, ++step/*sub*/,
+                 "Unable to resolve " NCBI_WWW "\n"
+                 "Checking for a proxy...");
+        AutoPtr<SConnNetInfo> net_info(ConnNetInfo_Create(0, m_DebugPrintout));
+        if (!net_info  ||  !net_info->http_proxy_host[0]) {
+            result = "No proxy detected in configuration";
+            // status = eIO_Unknown;
+        } else {
+            PreCheck(eDns, ++step/*sub*/,
+                     "Resolving \"" + string(net_info->http_proxy_host) + '"');
+            if (CSocketAPI::gethostbyname(net_info->http_proxy_host) == 0) {
+                result = "Unable to resolve proxy host \""
+                    + string(net_info->http_proxy_host) + '"';
+                // status = eIO_Unknown;
+            } else {
+                result = "OK";
+                status = eIO_Success;
+            }
+        }
     } else {
         result = "OK";
         status = eIO_Success;
     }
 
-    PostCheck(eDns, 0/*main*/, status, result);
+    PostCheck(eDns, step, status, result);
+
+    if (reason)
+        reason->swap(result);
     return status;
 }
 
@@ -408,7 +428,7 @@ EIO_Status CConnTest::HttpOkay(string* reason)
     AutoPtr<SConnNetInfo> net_info(ConnNetInfo_Create(0, m_DebugPrintout));
     if (!net_info) {
         PostCheck(eHttp, 0/*main*/,
-                  eIO_Unknown, "Unable to create network info structure");
+                  eIO_Unknown, "Unable to build connection parameters");
         return eIO_Unknown;
     }
 
@@ -442,13 +462,17 @@ EIO_Status CConnTest::HttpOkay(string* reason)
             // Redirect to https:// is expected
             temp += "Your HTTP connection appears to work but it does not seem"
                 " to be able to upgrade to secure HTTP (HTTPS), most likely"
-                " because of a malfunctioning HTTP proxy server in the way\n";
+                " because of an incompatibility of the encryption algorithms";
+            if (m_HttpProxy)
+                temp += " or a malfunctioning HTTP proxy server in the way\n";
+            else
+                temp += '\n';
         } else {
             bool host = NStr::CompareNocase(net_info->host, DEF_CONN_HOST) !=0;
             bool port = net_info->port != 0;
             if (host  ||  port) {
                 int n = 0;
-                temp += "Make sure that ";
+                temp += "Please make sure that ";
                 if (host) {
                     n++;
                     temp += "[" DEF_CONN_REG_SECTION "]" REG_CONN_HOST "=\"";
@@ -468,17 +492,47 @@ EIO_Status CConnTest::HttpOkay(string* reason)
         }
         bool auth = true;
         if (m_HttpProxy) {
-            temp += "Make sure that the HTTP proxy server \'";
+            temp += "Please make sure that the HTTP proxy server \'";
             temp += net_info->http_proxy_host;
             temp += ':';
             temp += NStr::UIntToString(net_info->http_proxy_port);
             temp += "' specified with ";
-            if (net_info->http_proxy_only) {
-                temp += "environment variable $http_proxy or $HTTP_PROXY";
+            if (net_info->http_proxy_mask) {
+                temp += "the environment variable";
+                if (net_info->http_proxy_mask == (fProxy_Http | fProxy_Https))
+                    temp += 's';
+                if (net_info->http_proxy_mask & fProxy_Http)
+                    temp += " 'http_proxy'";
+                if (net_info->http_proxy_mask == (fProxy_Http | fProxy_Https))
+                    temp += " and";
+                if (net_info->http_proxy_mask & fProxy_Https)
+                    temp += " 'https_proxy'";
                 auth = false;
             } else
                 temp += "[" DEF_CONN_REG_SECTION "]HTTP_PROXY_{HOST|PORT}";
             temp += " is correct";
+            if (!net_info->http_proxy_mask  &&   net_info->http_proxy_skip
+                &&  (getenv("http_proxy")  ||  getenv("https_proxy"))) {
+                temp += ": the proxy location was loaded this way because "
+                    "[" DEF_CONN_REG_SECTION "]" REG_CONN_HTTP_PROXY_SKIP
+                    " is set, and that prevents reading 'http_proxy' and/or"
+                    " 'https_proxy' from the environment";
+                auth = false;
+            }
+            if (auxdata->m_Redirect
+                &&  net_info->http_proxy_mask == fProxy_Http) {
+                temp += "; also note that (in addition to 'http_proxy') a"
+                    " setting of 'https_proxy' is usually required to properly"
+                    " forward secure HTTP (HTTPS) traffic";
+            }
+            if (net_info->http_proxy_mask) {
+                temp += "\nAlternatively, you can configure your HTTP proxy"
+                    " with [" DEF_CONN_REG_SECTION "]HTTP_PROXY_{HOST|PORT}"
+                    " (both must be set) and by disabling loading it from the"
+                    " environment by setting "
+                    "[" DEF_CONN_REG_SECTION "]" REG_CONN_HTTP_PROXY_SKIP "=1";
+                auth = true;
+            }
         } else {
             if (net_info->http_proxy_host[0]  ||  net_info->http_proxy_port) {
                 temp += "Note that your HTTP proxy seems to have been"
@@ -491,11 +545,20 @@ EIO_Status CConnTest::HttpOkay(string* reason)
                         + string(net_info->http_proxy_host) + '\'';
                 }
                 temp += ")\n";
+            } else if (net_info->http_proxy_skip
+                       &&  (getenv("http_proxy")  ||  getenv("https_proxy"))) {
+                temp += "Even though you seem to have 'http_proxy' and/or"
+                    " 'https_proxy' defined in your environment, "
+                    "[" DEF_CONN_REG_SECTION "]" REG_CONN_HTTP_PROXY_SKIP
+                    " is also set and, thus, prevents reading them";
+                auth = false;
+            } else {
+                temp += "If your network access requires the use of an HTTP"
+                    " proxy server, please contact your network administrator,"
+                    " and set"
+                    " [" DEF_CONN_REG_SECTION "]{HTTP_PROXY_{HOST|PORT}"
+                    " (both must be set) in your configuration accordingly";
             }
-            temp += "If your network access requires the use of an HTTP proxy"
-                " server, please contact your network administrator, and set"
-                " [" DEF_CONN_REG_SECTION "]{HTTP_PROXY_{HOST|PORT} (both must"
-                " be set) in your configuration accordingly";
         }
         if (auth) {
             temp += "; and if your proxy server requires authorization, please"
@@ -506,11 +569,13 @@ EIO_Status CConnTest::HttpOkay(string* reason)
         if (*net_info->user  ||  *net_info->pass) {
             temp += "Make sure there are no stray values for ["
                 DEF_CONN_REG_SECTION "]{" REG_CONN_USER "|" REG_CONN_PASS
-                "} in your configuration -- NCBI services neither require"
-                " nor use them\n";
+                "} in your configuration -- basic NCBI services neither"
+                " require nor use them\n";
+        }
+        if (status == eIO_NotSupported) {
+            temp += "Your application may also need to have the Secure Socket"
+                " Layer (SSL) support turned on\n";
         }
-        if (status == eIO_NotSupported)
-            temp += "Your application may also need to have SSL support on\n";
     }
 
     PostCheck(eHttp, 0/*main*/, status, temp);
@@ -673,7 +738,8 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
 }
 
 
-EIO_Status CConnTest::x_GetFirewallConfiguration(const SConnNetInfo* net_info)
+EIO_Status CConnTest::x_GetFirewallConfiguration(const SConnNetInfo* net_info,
+                                                 bool fwbports)
 {
     static const char kFWDUrl[] = "/IEB/ToolBox/NETWORK/fwd_check.cgi";
 
@@ -717,10 +783,12 @@ EIO_Status CConnTest::x_GetFirewallConfiguration(const SConnNetInfo* net_info)
              (!m_Firewall  &&  !(4444 <= cp.port  &&  cp.port <= 4544)))) {
             fb = true;
         }
-        if ( fb  &&  net_info  &&  net_info->firewall == eFWMode_Firewall)
+        if (net_info  &&  net_info->firewall == eFWMode_Firewall  &&  fb)
             continue;
-        if (!fb  &&  net_info  &&  net_info->firewall == eFWMode_Fallback)
+        if (net_info  &&  net_info->firewall == eFWMode_Fallback
+            &&  (!fb  ||  (fwbports  &&  !SERV_IsFirewallPort(cp.port)))) {
             continue;
+        }
         cp.status = okay ? eIO_Success : eIO_NotSupported;
         if (!fb)
             m_Fwd.push_back(cp);
@@ -756,47 +824,58 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             " way that it allows outbound connections to the port range ["
             NCBI_AS_STRING(CONN_FWD_PORT_MIN) ".."
             NCBI_AS_STRING(CONN_FWD_PORT_MAX)
-            "] (inclusive) at the two fixed NCBI addresses, "
-            NCBI_FWD_BEMD " and " NCBI_FWD_STVA ".\n"
+            "] (inclusive) at the fixed NCBI address " NCBI_FWD_BEMD ".\n"
             "To set that up correctly, please have your network administrator"
             " read the following (if they have not already done so):"
             " " CONN_FWD_URL "\n";
     } else {
         temp += "This is an obsolescent mode that requires keeping a wide port"
-            " range [4444..4544] (inclusive) open to let through connections"
-            " to the entire NCBI site (130.14.xxx.xxx/165.112.xxx.xxx) -- this"
+            " range [4444..4544] (inclusive) open to allow through connections"
+            " to the entire NCBI site (130.14.xxx.xxx) -- this"
             " mode was designed for unrestricted networks when firewall port"
             " blocking had not been an issue\n";
     }
+    char fwbset[256] = { '\0' };
     if (m_Firewall) {
         _ASSERT(net_info);
         switch (net_info->firewall) {
         case eFWMode_Adaptive:
-            temp += "Also, there are usually a few additional ports such as "
+            temp += "Also, there usually exist a few additional ports such as "
                 NCBI_AS_STRING(CONN_PORT_SSH) " and "
-                NCBI_AS_STRING(CONN_PORT_HTTPS)
-                " at " NCBI_FWD_BEMD ", which can be used if connections to"
-                " the ports in the range described above, have failed\n";
+                NCBI_AS_STRING(CONN_PORT_HTTPS) " at " NCBI_FWD_BEMD
+                ", which can be used if connections to the ports in the range"
+                " shown above, have failed\n";
             break;
         case eFWMode_Firewall:
             temp += "Furthermore, your configuration explicitly forbids to use"
                 " any fallback firewall ports that may exist to improve"
-                " reliability of connection experience\n";
+                " reliability of the connectivity with NCBI\n";
             break;
         case eFWMode_Fallback:
-            temp += "There are usually a few backup connection ports such as "
-                NCBI_AS_STRING(CONN_PORT_SSH) " and "
-                NCBI_AS_STRING(CONN_PORT_HTTPS)
-                " at " NCBI_FWD_BEMD ", which can be used as a failover if"
-                " connections to the port range above fail.  However, your "
-                " configuration explicitly requests that only those fallback"
-                " firewall ports (if any exist) are to be used for"
-                " connections:  this also implies that no conventional ports"
-                " from the default range will be used\n";
+            temp += "Normally, there are a few additional backup connection"
+                " ports such as " NCBI_AS_STRING(CONN_PORT_SSH) " and "
+                NCBI_AS_STRING(CONN_PORT_HTTPS) " at " NCBI_FWD_BEMD ", which"
+                " can be used as a failover should connections to the above"
+                " port range be unsuccessful.  However, your configuration"
+                " explicitly requests that only";
+            SERV_PrintFirewallPorts(fwbset, sizeof(fwbset), eFWMode_Fallback);
+            if (*fwbset  &&  strcmp(fwbset, "0") != 0) {
+                temp += " the following fallback port(s) may be used for"
+                    " connections: \"";
+                temp += fwbset;
+                temp += "\".";
+            } else {
+                *fwbset = '\0';
+                temp += " those fallback firewall ports (if any active) are to"
+                    " be used for connections.";
+            }
+            temp += " This also implies that no conventional ports from the"
+                " default range need to be configured, and none are going to"
+                " be utilized\n";
             break;
         default:
             temp += "Internal program error, please report!\n";
-            _ASSERT(0);
+            _TROUBLE;
             break;
         }
     } else {
@@ -804,7 +883,7 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             " firewall imposing a fine-grained control over which hosts and"
             " ports the outbound connections are allowed to use\n";
     }
-    if (m_HttpProxy  &&  net_info  &&  !net_info->http_proxy_only) {
+    if (m_HttpProxy && net_info && net_info->http_proxy_mask != fProxy_Http) {
         temp += "Connections to the aforementioned ports will be made via an"
             " HTTP proxy at '";
         temp += net_info->http_proxy_host;
@@ -813,10 +892,10 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
         temp += "'";
         if (net_info->http_proxy_leak) { 
             temp += ".  If that is unsuccessful, a link bypassing the proxy"
-                " will then be attempted";
-        }
+                " will then be attempted\n";
+        } else
+            temp += '\n';
     }
-    temp += '\n';
 
     PreCheck(eFirewallConnPoints, 0/*main*/, temp);
 
@@ -824,7 +903,7 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
              "Obtaining current NCBI " +
              string(m_Firewall ? "firewall settings" : "service entries"));
 
-    EIO_Status status = x_GetFirewallConfiguration(net_info.get());
+    EIO_Status status = x_GetFirewallConfiguration(net_info.get(), !!*fwbset);
 
     if (status == eIO_Interrupt)
         temp = kCanceled;
@@ -865,7 +944,16 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             empty = true;
         if (empty) {
             status = eIO_Unknown;
-            temp = "No connection ports found, please contact " + HELP_EMAIL;
+            temp = "No connection ports were found";
+            if (*fwbset) {
+                temp += ": it is quite possible that none of the fallback"
+                    " ports supported by NCBI match the restrictive set [";
+                temp += fwbset;
+                temp += "] imposed by your configuration.  Please make sure "
+                    "[" DEF_CONN_REG_SECTION "]" REG_CONN_FIREWALL " lists"
+                    " compatible value(s)";
+            } else
+                temp += ", please contact " + HELP_EMAIL;
         }
     } else if (status == eIO_Timeout) {
         temp = x_TimeoutMsg();
@@ -903,8 +991,8 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
         if (status == eIO_Success) {
             if (!m_Firewall  &&  !m_FwdFB.empty()) {
                 status = eIO_Unknown;
-                temp = "Fallback port(s) found in non-firewall mode,"
-                    " please contact " + HELP_EMAIL;
+                temp = "Fallback port(s) found in non-firewall mode, please"
+                    " contact " + HELP_EMAIL;
             } else if (m_Firewall != firewall) {
                 status = eIO_Unknown;
                 temp  = "Firewall ";
@@ -1118,8 +1206,8 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                     temp = x_TimeoutMsg();
                 else
                     temp.clear();
-                if (m_HttpProxy  &&  !(net_info->http_proxy_only |
-                                       net_info->http_proxy_leak)) {
+                if (m_HttpProxy  &&  !net_info->http_proxy_leak
+                    &&  net_info->http_proxy_mask != fProxy_Http) {
                     temp += "Your HTTP proxy '";
                     temp += net_info->http_proxy_host;
                     temp += ':';
@@ -1132,7 +1220,10 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                         temp += " and let them read: " CONN_FWD_URL;
                         url = true;
                     }
-                    temp += '\n';
+                    temp += "\nYou can also try setting "
+                        "[" DEF_CONN_REG_SECTION "]" REG_CONN_HTTP_PROXY_LEAK
+                        "=1 in your configuration to allow a proxy bypass"
+                        " retry logic for non-HTTP traffic\n";
                 }
                 temp += "The network port required for this connection to"
                     " succeed may be blocked/diverted at your firewall;";
@@ -1193,6 +1284,14 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                 m_Fwd.clear();
             } else
                 note = false;
+            char fwbset[256] = { '\0' };
+            SERV_PrintFirewallPorts(fwbset, sizeof(fwbset),
+                                    EFWMode(net_info->firewall));
+            if (*fwbset  &&  strcmp(fwbset, "0") != 0) {
+                temp += " [";
+                temp += fwbset;
+                temp += ']';
+            }
         }
         if (note  &&  status != eIO_Interrupt) {
             temp += "\nYou may want to read this link for more information: "
@@ -1340,9 +1439,8 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                 if (!net_info->firewall)
                     temp += " (Connection Relay)";
                 temp += " Daemon reports negotitation error";
-                if (net_info  &&  m_HttpProxy
-                    &&  !(net_info->http_proxy_only |
-                          net_info->http_proxy_leak)) {
+                if (net_info  &&  m_HttpProxy  &&  !net_info->http_proxy_leak
+                    &&  net_info->http_proxy_mask != fProxy_Http) {
                     temp += ", which usually means that an intermediate HTTP"
                         " proxy '";
                     temp += net_info->http_proxy_host;
@@ -1481,16 +1579,24 @@ EIO_Status CConnTest::x_CheckTrap(string* reason)
 }
 
 
-bool CConnTest::IsNcbiInhouseClient(void)
+bool CConnTest::IsNcbiInhouseClient(void) THROWS_NONE
 {
     static const STimeout kFast = { 2, 0 };
-    CConn_HttpStream http("https:///Service/getenv.cgi",
-                          fHTTP_KeepHeader | fHTTP_NoAutoRetry, &kFast);
-    char line[256];
-    if (!http  ||  !http.getline(line, sizeof(line)))
-        return false;
-    int code;
-    return !(::sscanf(line, "HTTP/%*d.%*d %d ", &code) < 1  ||  code != 200);
+    try {
+        CConn_HttpStream http("https:///Service/getenv.cgi",
+                              fHTTP_KeepHeader | fHTTP_NoAutoRetry, &kFast);
+        char line[256];
+        int  code;
+        if (http.getline(line, sizeof(line))
+            &&  ::sscanf(line, "HTTP/%*d.%*d %d ", &code) >= 1
+            &&  code == 200) {
+            return true;
+        }
+    }
+    catch (...) {
+        ;
+    }
+    return false;
 }
 
 
