@@ -1,4 +1,4 @@
-/* $Id: ncbi_connection.c 606402 2020-04-21 14:50:28Z lavr $
+/* $Id: ncbi_connection.c 690149 2024-11-15 18:05:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -39,6 +39,7 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_Conn
 
+
 #define CONNECTION_MAGIC     0xEFCDAB09
 
 
@@ -48,35 +49,35 @@
 
 /* Standard logging message
  */
-#define CONN_LOG_EX(subcode, func_name, level, message, status)            \
-  do {                                                                     \
-      const char* ststr = ((EIO_Status) status != eIO_Success              \
-                           ? IO_StatusStr((EIO_Status) status)             \
-                           : "");                                          \
-      const char* ctype = (conn  &&  conn->meta.get_type                   \
-                           ? conn->meta.get_type(conn->meta.c_get_type)    \
-                           : 0);                                           \
-      char* descr = (conn  &&  conn->meta.descr                            \
-                     ? conn->meta.descr(conn->meta.c_descr)                \
-                     : 0);                                                 \
-      char stbuf[80];                                                      \
-      if ((EIO_Status) status == eIO_Timeout  &&  timeout) {               \
-          assert(timeout != kDefaultTimeout);                              \
-          sprintf(stbuf, "%s[%u.%06u]", ststr,                             \
-                  (unsigned int)(timeout->sec + timeout->usec / 1000000),  \
-                  (unsigned int)               (timeout->usec % 1000000)); \
-          assert(strlen(stbuf) < sizeof(stbuf));                           \
-          ststr = stbuf;                                                   \
-      }                                                                    \
-      CORE_LOGF_X(subcode, level,                                          \
-                  ("[CONN_" #func_name "(%s%s%s)]  %s%s%s",                \
-                   ctype  &&  *ctype ? ctype : "UNDEF",                    \
-                   descr  &&  *descr ? "; "  : "", descr ? descr : "",     \
-                   message,                                                \
-                   ststr  &&  *ststr ? ": "  : "",                         \
-                   ststr             ? ststr : ""));                       \
-      if (descr)                                                           \
-          free(descr);                                                     \
+#define CONN_LOG_EX(subcode, func_name, level, message, status)         \
+  do {                                                                  \
+      const char* ststr = ((EIO_Status) status != eIO_Success           \
+                           ? IO_StatusStr((EIO_Status) status)          \
+                           : "");                                       \
+      const char* ctype = (conn  &&  conn->meta.get_type                \
+                           ? conn->meta.get_type(conn->meta.c_get_type) \
+                           : 0);                                        \
+      char* descr = (conn  &&  conn->meta.descr                         \
+                     ? conn->meta.descr(conn->meta.c_descr)             \
+                     : 0);                                              \
+      char stbuf[80];                                                   \
+      if ((EIO_Status) status == eIO_Timeout  &&  timeout) {            \
+          assert(timeout != kDefaultTimeout);                           \
+          sprintf(stbuf, "%s[%u.%06u]", ststr,                          \
+                  timeout->usec / 1000000 + timeout->sec,               \
+                  timeout->usec % 1000000);                             \
+          assert(strlen(stbuf) < sizeof(stbuf));                        \
+          ststr = stbuf;                                                \
+      }                                                                 \
+      CORE_LOGF_X(subcode, level,                                       \
+                  ("[CONN_" #func_name "(%s%s%s)]  %s%s%s",             \
+                   ctype  &&  *ctype ? ctype : "UNDEF",                 \
+                   descr  &&  *descr ? "; "  : "", descr ? descr : "",  \
+                   message,                                             \
+                   ststr  &&  *ststr ? ": "  : "",                      \
+                   ststr             ? ststr : ""));                    \
+      if (descr)                                                        \
+          free(descr);                                                  \
   } while (0)
 
 #define CONN_LOG(s_c, f_n, lvl, msg)  CONN_LOG_EX(s_c, f_n, lvl, msg, status)
@@ -85,9 +86,9 @@
 #  define CONN_CALLTRACE(func_name)                             \
   do {                                                          \
       static const STimeout* timeout = 0/*dummy*/;              \
-      char handle[80];                                          \
-      sprintf(handle, "0x%p", conn);                            \
-      CONN_LOG_EX(0, func_name, eLOG_Trace, handle, 0);         \
+      char x_handle[80];                                        \
+      sprintf(x_handle, "0x%p", conn);                          \
+      CONN_LOG_EX(0, func_name, eLOG_Trace, x_handle, 0);       \
   } while (0)
 #else
 #  define CONN_CALLTRACE(func_name)  /*((void) 0)*/
@@ -109,9 +110,11 @@
       }                                                                 \
       if (conn->magic != CONNECTION_MAGIC) {                            \
           static const STimeout* timeout = 0/*dummy*/;                  \
-          CONN_LOG_EX(subcode, func_name, eLOG_Critical,                \
-                      "Corrupt connection handle", 0);                  \
+          char x_errmsg[80];                                            \
+          sprintf(x_errmsg, "Corrupt connection handle 0x%p", conn);    \
+          CONN_LOG_EX(subcode, func_name, eLOG_Critical, x_errmsg, 0);  \
           assert(0);                                                    \
+          return retval;                                                \
       }                                                                 \
   } while (0)
 
@@ -124,7 +127,7 @@
 #endif /*_DEBUG*/
 
 
-/* Private flags, must not cross with ECONN_Flags defined in the header
+/* Private flags, MUST NOT cross with ECONN_Flags defined in the header
  */
 enum ECONN_InternalFlag {
     fCONN_Flush = 1024           /* auto-flush was successful                */
@@ -134,17 +137,18 @@ enum ECONN_InternalFlag {
 /* Connection state
  */
 typedef enum {
-    eCONN_Unusable = -1,         /* iff !conn->meta.list                     */
+    eCONN_Unusable = -1,         /* iff !conn->meta.list (uninited state)    */
     eCONN_Closed   =  0,         /* "Open" can be attempted                  */
     eCONN_Open     =  1,         /* operational state (I/O allowed)          */
     eCONN_Bad      =  2,         /* non-operational (I/O not allowed)        */
-    eCONN_Cancel   =  3,         /* NB: |= eCONN_Open, user-canceled         */
-    eCONN_Corrupt  =  5          /* NB: |= eCONN_Open, non-operational       */
+    eCONN_Corrupt  =  3,         /* NB: |= eCONN_Open|eCONN_Bad, no more I/O */
+    eCONN_Canceled =  5          /* NB: |= eCONN_Open, user-canceled         */
 } ECONN_State;
 
 
 /* Connection internal data.
- * NOTE:  meta *must* come first!
+ *
+ * NOTE:  "meta" *must* come first!  (see ncbi_conn_streambuf.cpp)
  */
 typedef struct SConnectionTag {
     SMetaConnector  meta;        /* VTable of operations and list            */
@@ -154,16 +158,17 @@ typedef struct SConnectionTag {
     EIO_Status      r_status;    /* I/O status of last read                  */
     EIO_Status      w_status;    /* I/O status of last write                 */
 
-    BUF             buf;         /* storage for peek data                    */
+    BUF             buf;         /* storage for peek/pushback data           */
 
     void*           data;        /* user data pointer                        */
 
     /* "[o|r|w|c]_timeout" is either 0 (kInfiniteTimeout), kDefaultTimeout
-       (to use connector-specific one), or points to "[oo|rr|ww|cc]_timeout" */
-    const STimeout* o_timeout;   /* timeout on open                          */
-    const STimeout* r_timeout;   /* timeout on read                          */
-    const STimeout* w_timeout;   /* timeout on write                         */
-    const STimeout* c_timeout;   /* timeout on close                         */
+       (to use connector-specific one), or points to "[oo|rr|ww|cc]_timeout";
+       timeouts are "as is" (may be not normalized) for performance reasons  */
+    const STimeout*  o_timeout;  /* timeout on open                          */
+    const STimeout*  r_timeout;  /* timeout on read                          */
+    const STimeout*  w_timeout;  /* timeout on write                         */
+    const STimeout*  c_timeout;  /* timeout on close                         */
     STimeout        oo_timeout;  /* storage for "o_timeout"                  */
     STimeout        rr_timeout;  /* storage for "r_timeout"                  */
     STimeout        ww_timeout;  /* storage for "w_timeout"                  */
@@ -172,12 +177,26 @@ typedef struct SConnectionTag {
     TNCBI_BigCount  r_pos;       /* read and ...                             */
     TNCBI_BigCount  w_pos;       /*          ... write positions             */
 
-    SCONN_Callback  cb[CONN_N_CALLBACKS+1]; /* +1 includes dummy no-callback */
+    SCONN_Callback  cb[CONN_N_CALLBACKS+1]; /* +1 includes catchall callback */
 
     unsigned int    magic;       /* magic cookie for integrity checks        */
 } SConnection;
 
 
+/*ARGSUSED*/
+static EIO_Status x_CatchallCallback(CONN conn,TCONN_Callback type,void* data)
+{
+    static const STimeout* timeout = 0/*dummy*/;
+    char errmsg[80];
+    sprintf(errmsg, "Unknown callback #%u for 0x%p, assume corruption",
+            type, conn);
+    CONN_LOG_EX(37, CALLBACK, eLOG_Critical, errmsg, 0);
+    assert(0);
+    conn->state = eCONN_Corrupt;
+    return eIO_InvalidArg;
+}
+
+
 static size_t x_CB2IDX(ECONN_Callback type)
 {
     size_t idx;
@@ -201,7 +220,7 @@ static size_t x_CB2IDX(ECONN_Callback type)
         idx = 5;
         break;
     default:
-        /* to flag an API error */
+        /* to flag an API error via x_CatchallCallback() */
         idx = CONN_N_CALLBACKS;
         break;
     }
@@ -211,6 +230,7 @@ static size_t x_CB2IDX(ECONN_Callback type)
 
 static EIO_Status x_Callback(CONN conn, ECONN_Callback type, unsigned int flag)
 {
+    int/*bool*/    io = (type | flag) & eIO_ReadWrite;
     size_t         idx = x_CB2IDX(type);
     FCONN_Callback func;
     void*          data;
@@ -219,12 +239,19 @@ static EIO_Status x_Callback(CONN conn, ECONN_Callback type, unsigned int flag)
     assert(!flag  ||  (type == eCONN_OnTimeout
                        &&  (flag | eIO_ReadWrite) == eIO_ReadWrite));
     assert(conn  &&  idx < CONN_N_CALLBACKS);
+    if (conn->state != eCONN_Open  &&  io)
+        return eIO_Unknown;
     if (!(func = conn->cb[idx].func))
         return type == eCONN_OnTimeout ? eIO_Timeout : eIO_Success;
     data = conn->cb[idx].data;
     status = (*func)(conn, (TCONN_Callback) type | flag, data);
-    if (status == eIO_Interrupt)
-        conn->state = eCONN_Cancel;
+    if (status == eIO_Success) {
+        if (type == eCONN_OnOpen)
+            status = eIO_Reserved;
+        else if (conn->state != eCONN_Open  &&  io)
+            status = eIO_Unknown;
+    } else if (status == eIO_Interrupt)
+        conn->state = eCONN_Canceled;
     return status;
 }
 
@@ -232,27 +259,28 @@ static EIO_Status x_Callback(CONN conn, ECONN_Callback type, unsigned int flag)
 static EIO_Status x_Flush(CONN conn, const STimeout* timeout,
                           int/*bool*/ isflush)
 {
-    EIO_Status status;
+    assert(isflush  ||  !(conn->flags & fCONN_Flush));
+    for (;;) {
+        EIO_Status status;
+        if ((status = x_Callback(conn, eCONN_OnFlush, 0)) != eIO_Success)
+            return status;
 
-    if ((status = x_Callback(conn, eCONN_OnFlush, 0)) != eIO_Success)
-        return status;
-
-    /* call current connector's "FLUSH" method */
-    if (conn->meta.flush) {
+        if (!conn->meta.flush)
+            break;
+        /* call current connector's "FLUSH" method */
         if (timeout == kDefaultTimeout)
             timeout  = conn->meta.default_timeout;
         assert(timeout != kDefaultTimeout);
-        for (;;) {
-            status = conn->meta.flush(conn->meta.c_flush, timeout);
-            if (isflush)
-                conn->w_status = status;
-            if (status == eIO_Success)
-                break;
-            if (status == eIO_Timeout)
-                status  = x_Callback(conn, eCONN_OnTimeout, eIO_ReadWrite);
-            if (status != eIO_Success)
-                return status;
-        }
+        status = conn->meta.flush(conn->meta.c_flush, timeout);
+        if (isflush)
+            conn->w_status = status;
+        if (status == eIO_Success)
+            break;
+
+        if (status == eIO_Timeout)
+            status  = x_Callback(conn, eCONN_OnTimeout, eIO_ReadWrite);
+        if (status != eIO_Success)
+            return status;
     }
     conn->flags |= fCONN_Flush;
     return eIO_Success;
@@ -268,14 +296,17 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
     assert(!close  ||  !connector);
     assert(!conn->meta.list == !(conn->state != eCONN_Unusable));
 
-    /* flush connection first, if open */
-    status = conn->meta.list  &&  conn->state == eCONN_Open
+    /* flush connection first, if open & not flushed */
+    status = conn->meta.list 
+        &&  conn->state == eCONN_Open
+        &&  !(conn->flags & fCONN_Flush)
         ? x_Flush(conn, conn->c_timeout, 0/*no-flush*/) : eIO_Success;
 
     for (x_conn = conn->meta.list;  x_conn;  x_conn = x_conn->next) {
+        assert(conn->state != eCONN_Unusable);
         if (x_conn == connector) {
             assert(!close);
-            /* reinit with the same and the only connector - allowed */
+            /* re-init with the same and the only connector is allowed */
             if (!x_conn->next  &&  x_conn == conn->meta.list)
                 break;
             status = eIO_NotSupported;
@@ -286,6 +317,7 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
     }
 
     if (conn->meta.list) {
+        assert(conn->state != eCONN_Unusable);
         /* erase unread data */
         BUF_Erase(conn->buf);
 
@@ -317,6 +349,7 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
                 }
             }
         }
+        /* HERE: conn->state = eCONN_Closed; */
 
         if (!x_conn) {
             /* entirely new connector - remove the old connector stack first */
@@ -325,12 +358,14 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
             memset(&conn->meta, 0, sizeof(conn->meta));
             conn->state = eCONN_Unusable;
         } else if (status != eIO_Success) {
+            /* the incumbent connector's re-init failed */
             conn->state = eCONN_Bad;
+            assert(!close);
             return status;
         } else
             conn->state = eCONN_Closed;
     } else if (!connector)
-        status = eIO_Closed;
+        status = eIO_Closed;  /*NB: conn->state == eCONN_Unusable */
 
     if (!x_conn  &&  connector) {
         assert(conn->state == eCONN_Unusable);
@@ -342,7 +377,7 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
         assert(conn->meta.default_timeout != kDefaultTimeout);
     }
 
-    assert(conn->state != eCONN_Open  &&  conn->state != eCONN_Bad);
+    assert(conn->state == eCONN_Unusable  ||  conn->state == eCONN_Closed);
     return status;
 }
 
@@ -355,11 +390,12 @@ static EIO_Status s_Open(CONN conn)
     switch (conn->state) {
     case eCONN_Unusable:
         return eIO_InvalidArg;
-    case eCONN_Cancel:
+    case eCONN_Canceled:
         return eIO_Interrupt;
-    case eCONN_Bad:
     case eCONN_Corrupt:
         return eIO_Unknown;
+    case eCONN_Bad:
+        return eIO_Closed;
     default:
         break;
     }
@@ -368,25 +404,32 @@ static EIO_Status s_Open(CONN conn)
     conn->w_pos = 0;
 
     if (conn->meta.open) {
-        /* call current connector's "OPEN" method */
-        int/*bool*/ nocb = !conn->cb[x_CB2IDX(eCONN_OnOpen)].func;
-        if ((status = x_Callback(conn, eCONN_OnOpen, 0)) == eIO_Success) {
+        for (;;) {
+            int/*bool*/ nocb = 1/*true = no callback*/;
+            if ((status = x_Callback(conn, eCONN_OnOpen, 0)) == eIO_Reserved) {
+                status = eIO_Success;
+                nocb = 0/*false*/;
+            } else if (status != eIO_Success)
+                break;
+
+            /* call current connector's "OPEN" method */
             timeout = (conn->o_timeout == kDefaultTimeout
                        ? conn->meta.default_timeout
                        : conn->o_timeout);
             assert(timeout != kDefaultTimeout);
             status = conn->meta.open(conn->meta.c_open, timeout);
-            /* OnTimeout gets called only if OnOpen was there */
-            if (status == eIO_Timeout  &&  !nocb) {
-                status  = x_Callback(conn, eCONN_OnTimeout, eIO_Open);
-                if (status == eIO_Success)
-                    status  = eIO_Timeout;
-            }
+            if (status != eIO_Timeout  ||  nocb)
+                break;
+
+            /* eCONN_OnTimeout gets called only if eCONN_OnOpen was there */
+            status = x_Callback(conn, eCONN_OnTimeout, eIO_Open);
+            if (status != eIO_Success)
+                break;
         }
     } else
         status  = eIO_NotSupported;
     if (status == eIO_Success) {
-        conn->flags   &= (TCONN_Flags)(~fCONN_Flush);
+        conn->flags   |= fCONN_Flush;
         conn->r_status = eIO_Success;
         conn->w_status = eIO_Success;
         conn->state    = eCONN_Open;
@@ -395,7 +438,7 @@ static EIO_Status s_Open(CONN conn)
         if (conn->state == eCONN_Closed)
             conn->state  = eCONN_Bad;
     }
-    return status == eIO_Closed ? eIO_Unknown : status;
+    return status;
 }
 
 
@@ -422,6 +465,7 @@ extern EIO_Status CONN_CreateEx
             conn->w_timeout = kDefaultTimeout;
             conn->c_timeout = kDefaultTimeout;
             conn->magic     = CONNECTION_MAGIC;
+            conn->cb[CONN_N_CALLBACKS].func = x_CatchallCallback;
             if ((status = x_ReInit(conn, connector, 0)) != eIO_Success) {
                 conn->magic = (unsigned int)(-1);
                 free(conn);
@@ -465,8 +509,9 @@ extern const char* CONN_GetType(CONN conn)
 {
     CONN_NOT_NULL_EX(6, GetType, 0);
 
-    return conn->state == eCONN_Unusable  ||  !conn->meta.list  ||
-        !conn->meta.get_type ? 0 : conn->meta.get_type(conn->meta.c_get_type);
+    assert(!conn->meta.list == !(conn->state != eCONN_Unusable));
+    return !conn->meta.list  ||  !conn->meta.get_type
+        ? 0 : conn->meta.get_type(conn->meta.c_get_type);
 }
 
 
@@ -474,14 +519,16 @@ extern char* CONN_Description(CONN conn)
 {
     CONN_NOT_NULL_EX(7, Description, 0);
 
-    return conn->state == eCONN_Unusable  ||  !conn->meta.list  ||
-        !conn->meta.descr ? 0 : conn->meta.descr(conn->meta.c_descr);
+    assert(!conn->meta.list == !(conn->state != eCONN_Unusable));
+    return !conn->meta.list  ||  !conn->meta.descr
+        ? 0 : conn->meta.descr(conn->meta.c_descr);
 }
 
 
 extern TNCBI_BigCount CONN_GetPosition(CONN conn, EIO_Event event)
 {
     static const STimeout* timeout = 0/*dummy*/;
+    TNCBI_BigCount pos;
     char errbuf[80];
 
     CONN_NOT_NULL_EX(30, GetPosition, 0);
@@ -490,19 +537,22 @@ extern TNCBI_BigCount CONN_GetPosition(CONN conn, EIO_Event event)
     case eIO_Open:
         conn->r_pos = 0;
         conn->w_pos = 0;
+        pos = 0;
         break;
     case eIO_Read:
-        return conn->r_pos;
+        pos = conn->r_pos;
+        break;
     case eIO_Write:
-        return conn->w_pos;
+        pos = conn->w_pos;
+        break;
     default:
         sprintf(errbuf, "Unknown direction #%u", (unsigned int) event);
         CONN_LOG_EX(31, GetPosition, eLOG_Error, errbuf, 0);
         assert(0);
+        pos = 0;
         break;
     }
-
-    return 0;
+    return conn->state == eCONN_Unusable ? 0 : pos;
 }
 
 
@@ -618,7 +668,7 @@ extern EIO_Status CONN_Wait
     /* perform open, if not opened yet */
     if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
         return status;
-    assert((conn->state & eCONN_Open)  &&  conn->meta.list);
+    assert(conn->state == eCONN_Open  &&  conn->meta.list);
 
     /* check if there is a PEEK'ed data in the input */
     if (event == eIO_Read  &&  BUF_Size(conn->buf))
@@ -639,8 +689,8 @@ extern EIO_Status CONN_Wait
         ELOG_Level level;
         switch (status) {
         case eIO_Timeout:
-            if (!timeout)
-                level = eLOG_Error; /*impossible*/
+            if (!timeout/*==kInfiniteTimeout,impossible*/)
+                level = eLOG_Critical;
             else if (timeout->sec | timeout->usec)
                 level = eLOG_Trace;
             else
@@ -665,33 +715,33 @@ extern EIO_Status CONN_Wait
 
 static EIO_Status s_CONN_Write
 (CONN         conn,
- const void*  buf,
+ const void*  data,
  const size_t size,
  size_t*      n_written)
 {
-    const STimeout* timeout = 0/*dummy*/;
+    const STimeout* timeout;
     EIO_Status status;
 
     assert(*n_written == 0);
 
-    /* check if the write method is specified at all */
-    if (!conn->meta.write) {
-        status = eIO_NotSupported;
-        CONN_LOG(16, Write, eLOG_Critical, "Cannot write data");
-        return status;
-    }
+    for (timeout = 0; ; timeout = 0) {
+        FConnectorWrite writefunc;
 
-    for (;;) {
         if ((status = x_Callback(conn, eCONN_OnWrite, 0)) != eIO_Success)
             break;
 
         /* call current connector's "WRITE" method */
+        if (!(writefunc = conn->meta.write)) {
+            status = eIO_NotSupported;
+            CONN_LOG(16, Write, eLOG_Critical, "Cannot write data");
+            return status;
+        }
         timeout = (conn->w_timeout == kDefaultTimeout
                    ? conn->meta.default_timeout
                    : conn->w_timeout);
         assert(timeout != kDefaultTimeout);
-        status = conn->meta.write(conn->meta.c_write, buf, size,
-                                  n_written, timeout);
+        status = writefunc(conn->meta.c_write, data, size,
+                           n_written, timeout);
         assert(status != eIO_Success  ||  *n_written  ||  !size);
         assert(*n_written <= size);
         conn->w_status = status;
@@ -703,6 +753,7 @@ static EIO_Status s_CONN_Write
         }
         if (!size  ||  status != eIO_Timeout)
             break;
+
         status = x_Callback(conn, eCONN_OnTimeout, eIO_Write);
         if (status != eIO_Success)
             break;
@@ -716,7 +767,7 @@ static EIO_Status s_CONN_Write
             ELOG_Level level;
             if (status != eIO_Timeout  ||  conn->w_timeout == kDefaultTimeout)
                 level = eLOG_Error;
-            else if (timeout  &&  (timeout->sec | timeout->usec))
+            else if (!timeout/*impossible*/ || (timeout->sec | timeout->usec))
                 level = eLOG_Warning;
             else
                 level = eLOG_Trace;
@@ -729,7 +780,7 @@ static EIO_Status s_CONN_Write
 
 static EIO_Status s_CONN_WritePersist
 (CONN         conn,
- const void*  buf,
+ const void*  data,
  const size_t size,
  size_t*      n_written)
 {
@@ -739,9 +790,12 @@ static EIO_Status s_CONN_WritePersist
 
     do {
         size_t x_written = 0;
-        status = s_CONN_Write(conn, (char*) buf + *n_written,
+        status = s_CONN_Write(conn, (char*) data + *n_written,
                               size - *n_written, &x_written);
         *n_written += x_written;
+        assert(*n_written <= size);
+        if (!size)
+            break;
         if (*n_written == size)
             return conn->flags & fCONN_Supplement ? status : eIO_Success;
     } while (status == eIO_Success);
@@ -752,32 +806,34 @@ static EIO_Status s_CONN_WritePersist
 
 extern EIO_Status CONN_Write
 (CONN            conn,
- const void*     buf,
+ const void*     data,
  size_t          size,
  size_t*         n_written,
  EIO_WriteMethod how)
 {
     EIO_Status status;
 
+    if (!n_written) {
+        assert(0);
+        return eIO_InvalidArg;
+    }
+    *n_written = 0;
+    if (size  &&  !data)
+        return eIO_InvalidArg;
+
     CONN_NOT_NULL(18, Write);
 
-    if (!n_written)
-        return eIO_InvalidArg;
-    *n_written = 0;
-    if (size  &&  !buf)
-        return eIO_InvalidArg;
-
-    /* open connection, if not yet opened */
+    /* perform open, if not opened yet */
     if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
         return status;
-    assert((conn->state & eCONN_Open)  &&  conn->meta.list);
+    assert(conn->state == eCONN_Open  &&  conn->meta.list);
 
     switch (how) {
     case eIO_WritePlain:
-        status = s_CONN_Write(conn, buf, size, n_written);
+        status = s_CONN_Write(conn, data, size, n_written);
         break;
     case eIO_WritePersist:
-        return s_CONN_WritePersist(conn, buf, size, n_written);
+        return s_CONN_WritePersist(conn, data, size, n_written);
     default:
         assert(0);
         return eIO_NotSupported;
@@ -794,17 +850,22 @@ extern EIO_Status CONN_Pushback
  const void* data,
  size_t      size)
 {
+    if (size  &&  !data) {
+        assert(0);
+        return eIO_InvalidArg;
+    }
+
     CONN_NOT_NULL(19, Pushback);
 
     if (conn->state == eCONN_Unusable)
         return eIO_InvalidArg;
 
+    if (conn->state == eCONN_Canceled)
+        return eIO_Interrupt;
+
     if (conn->state != eCONN_Open)
         return eIO_Closed;
 
-    if (!conn->meta.read)
-        return eIO_NotSupported;
-
     return BUF_Pushback(&conn->buf, data, size) ? eIO_Success : eIO_Unknown;
 }
 
@@ -819,7 +880,7 @@ extern EIO_Status CONN_Flush
     /* perform open, if not opened yet */
     if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
         return status;
-    assert((conn->state & eCONN_Open)  &&  conn->meta.list);
+    assert(conn->state == eCONN_Open  &&  conn->meta.list);
 
     status = x_Flush(conn, conn->w_timeout, 1/*flush*/);
     if (status != eIO_Success) {
@@ -845,30 +906,30 @@ static EIO_Status s_CONN_Read
  size_t*      n_read,
  int/*bool*/  peek)
 {
-    const STimeout* timeout = 0/*dummy*/;
+    const STimeout* timeout;
     EIO_Status status;
 
     assert(*n_read == 0);
 
-    /* check if the read method is specified at all */
-    if (!conn->meta.read) {
-        status = eIO_NotSupported;
-        CONN_LOG(22, Read, eLOG_Critical, "Cannot read data");
-        return status;
-    }
-
-    if (conn->state == eCONN_Cancel)
-        status = eIO_Interrupt;
-    else for (;;) {
+    for (timeout = 0; ; timeout = 0) {
+        FConnectorRead readfunc;
         size_t x_read;
 
+        /* keep flushing pending unwritten output data, if any */
+        if (!(conn->flags & (fCONN_Untie | fCONN_Flush))) {
+            status = x_Flush(conn, conn->r_timeout, 0/*no-flush*/);
+            if (status == eIO_Interrupt)
+                break;
+            /* else ignore any other status here (success or error) */
+        }
+
         /* read data from the internal peek/pushback buffer, if any */
         if (size) {
             x_read = (peek
                       ? BUF_Peek(conn->buf, buf, size - *n_read)
                       : BUF_Read(conn->buf, buf, size - *n_read));
             *n_read += x_read;
-            if (*n_read == size) {
+            if (x_read  &&  (*n_read == size  ||  !peek)) {
                 status = eIO_Success;
                 break;
             }
@@ -878,14 +939,24 @@ static EIO_Status s_CONN_Read
         if ((status = x_Callback(conn, eCONN_OnRead, 0)) != eIO_Success)
             break;
 
-        x_read = 0;
         /* call current connector's "READ" method */
+        if (!(readfunc = conn->meta.read)) {
+            if (!*n_read) {
+                status = eIO_NotSupported;
+                CONN_LOG(22, Read, eLOG_Critical, "Cannot read data");
+            } else {
+                status = eIO_Closed;
+                assert(peek);
+            }
+            break;
+        }
         timeout = (conn->r_timeout == kDefaultTimeout
                    ? conn->meta.default_timeout
                    : conn->r_timeout);
         assert(timeout != kDefaultTimeout);
-        status = conn->meta.read(conn->meta.c_read, buf, size - *n_read,
-                                 &x_read, timeout);
+        x_read = 0;
+        status = readfunc(conn->meta.c_read, buf, size - *n_read,
+                          &x_read, timeout);
         assert(status != eIO_Success  ||  x_read  ||  !size);
         assert(x_read <= size - *n_read);
         conn->r_status = status;
@@ -894,16 +965,18 @@ static EIO_Status s_CONN_Read
             *n_read     += x_read;
             conn->r_pos += x_read;
             /* save data in the internal peek buffer */
-            if (peek  &&  !BUF_Write(&conn->buf, buf, x_read)) {
+            if (conn->state == eCONN_Open  &&  peek
+                &&  !BUF_Write(&conn->buf, buf, x_read)) {
+                status = eIO_Unknown;
                 CONN_LOG_EX(32, Read, eLOG_Critical,
                             "Cannot save peek data", 0);
-                conn->state = eCONN_Corrupt;
-                status = eIO_Unknown;
+                conn->state  = eCONN_Corrupt;
             }
             break;
         }
         if (!size  ||  *n_read  ||  status != eIO_Timeout)
             break;
+
         status = x_Callback(conn, eCONN_OnTimeout, eIO_Read);
         if (status != eIO_Success)
             break;
@@ -917,7 +990,7 @@ static EIO_Status s_CONN_Read
             ELOG_Level level;
             if (status != eIO_Timeout  ||  conn->r_timeout == kDefaultTimeout)
                 level = eLOG_Error;
-            else if (timeout  &&  (timeout->sec | timeout->usec))
+            else if (!timeout/*impossible*/ || (timeout->sec | timeout->usec))
                 level = eLOG_Warning;
             else
                 level = eLOG_Trace;
@@ -940,21 +1013,18 @@ static EIO_Status s_CONN_ReadPersist
 
     assert(*n_read == 0);
 
-    for (;;) {
+    do {
         size_t x_read = 0;
         status = s_CONN_Read(conn, (char*) buf + *n_read,
-                             size - *n_read, &x_read, 0/*no peek*/);
+                             size - *n_read, &x_read, 0/*read*/);
         *n_read += x_read;
+        assert(*n_read <= size);
+        if (!size)
+            break;
         if (*n_read == size)
             return conn->flags & fCONN_Supplement ? status : eIO_Success;
+    } while (status == eIO_Success);
 
-        if (status != eIO_Success)
-            break;
-
-        /* keep flushing any pending unwritten output data */
-        if (!(conn->flags & (fCONN_Untie | fCONN_Flush)))
-            x_Flush(conn, conn->r_timeout, 0/*no-flush*/);
-    }
     return status;
 }
 
@@ -968,30 +1038,28 @@ extern EIO_Status CONN_Read
 {
     EIO_Status status;
 
-    CONN_NOT_NULL(24, Read);
-
-    if (!n_read)
+    if (!n_read) {
+        assert(0);
         return eIO_InvalidArg;
+    }
     *n_read = 0;
     if (size  &&  !buf)
         return eIO_InvalidArg;
 
+    CONN_NOT_NULL(24, Read);
+
     /* perform open, if not opened yet */
     if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
-        return status;
-    assert((conn->state & eCONN_Open)  &&  conn->meta.list);
-
-    /* flush pending unwritten output data (if any) */
-    if (!(conn->flags & (fCONN_Untie | fCONN_Flush)))
-        x_Flush(conn, conn->r_timeout, 0/*no-flush*/);
+        return status == eIO_Closed ? eIO_Unknown : status;
+    assert(conn->state == eCONN_Open  &&  conn->meta.list);
 
     /* now do read */
     switch (how) {
     case eIO_ReadPeek:
-        status = s_CONN_Read(conn, buf, size, n_read, 1/*i.e.peek*/);
+        status = s_CONN_Read(conn, buf, size, n_read, 1/*peek*/);
         break;
     case eIO_ReadPlain:
-        status = s_CONN_Read(conn, buf, size, n_read, 0/*i.e.read*/);
+        status = s_CONN_Read(conn, buf, size, n_read, 0/*read*/);
         break;
     case eIO_ReadPersist:
         return s_CONN_ReadPersist(conn, buf, size, n_read);
@@ -1017,67 +1085,70 @@ extern EIO_Status CONN_ReadLine
     int/*bool*/ done;
     size_t      len;
 
-    CONN_NOT_NULL(25, ReadLine);
-
-    if (!n_read)
+    if (!n_read) {
+        assert(0);
         return eIO_InvalidArg;
-    *n_read = 0;
-    if (size) {
-        if (!line)
-            return eIO_InvalidArg;
-        *line = '\0';
     }
+    *n_read = 0;
+    if (!size  ||  !line)
+        return eIO_InvalidArg;
+
+    CONN_NOT_NULL(25, ReadLine);
 
     /* perform open, if not opened yet */
     if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
-        return status;
-    assert((conn->state & eCONN_Open)  &&  conn->meta.list);
+        return status == eIO_Closed ? eIO_Unknown : status;
+    assert(conn->state == eCONN_Open  &&  conn->meta.list);
 
     len = 0;
     done = 0/*false*/;
     do {
-        size_t i;
         char   w[1024];
-        size_t x_read = 0;
-        size_t x_size = BUF_Size(conn->buf);
-        char*  x_buf  = size - len < sizeof(w) ? w : line + len;
-        if (x_size == 0  ||  x_size > sizeof(w))
-            x_size  = sizeof(w);
+        size_t i, x_size, x_read = 0;
+        char*  x_buf = size - len < sizeof(w) ? w : line + len;
 
-        /* keep flushing any pending unwritten output data then read */
-        if (!(conn->flags & (fCONN_Untie | fCONN_Flush)))
-            x_Flush(conn, conn->r_timeout, 0/*no-flush*/);
-        status = s_CONN_Read(conn, x_buf, size ? x_size : 0, &x_read, 0);
+        if (!(x_size = BUF_Size(conn->buf))  ||  x_size > sizeof(w))
+            x_size = sizeof(w);
+        status = s_CONN_Read(conn, x_buf, x_size, &x_read, 0);
+        assert(x_read <= x_size);
 
-        for (i = 0;  i < x_read  &&  len < size;  ++i) {
-            char c = x_buf[i];
+        i = 0;
+        while (i < x_read  &&  len < size) {
+            char c = x_buf[i++];
             if (c == '\n') {
+                status = eIO_Success;
                 done = 1/*true*/;
-                ++i;
                 break;
             }
             if (x_buf == w)
                 line[len] = c;
             ++len;
         }
+        if (len >= size) {
+            /* out of room */
+            assert(!done  &&  len);
+            done = 1/*true*/;
+        }
         if (i < x_read) {
-            assert(done  ||  len >= size);
-            if (!BUF_Pushback(&conn->buf, x_buf + i, x_read - i)) {
+            /* pushback excess */
+            assert(done);
+            if (conn->state == eCONN_Open
+                &&  !BUF_Pushback(&conn->buf, x_buf + i, x_read - i)) {
                 static const STimeout* timeout = 0/*dummy*/;
+                status = eIO_Unknown;
                 CONN_LOG_EX(35, ReadLine, eLOG_Critical,
                             "Cannot pushback extra data", 0);
-                conn->state = eCONN_Corrupt;
-                status = eIO_Unknown;
-            } else
-                status = eIO_Success;
+                conn->state  = eCONN_Corrupt;
+            }
             break;
         }
-    } while (!done  &&  len < size  &&  status == eIO_Success);
+    } while (!done  &&  status == eIO_Success);
+
     if (len < size)
         line[len] = '\0';
     *n_read = len;
 
-    return status;
+    return done  &&  !(conn->flags & fCONN_Supplement) ? eIO_Success : status;
 }
 
 
@@ -1091,14 +1162,14 @@ extern EIO_Status CONN_Status(CONN conn, EIO_Event dir)
     if (conn->state == eCONN_Unusable)
         return eIO_InvalidArg;
 
+    if (conn->state == eCONN_Canceled)
+        return eIO_Interrupt;
+
     if (conn->state == eCONN_Corrupt)
         return eIO_Unknown;
 
-    if (conn->state == eCONN_Cancel)
-        return eIO_Interrupt;
-
     if (conn->state != eCONN_Open)
-        return eIO_Closed;
+        return dir == eIO_Read ? eIO_Unknown : eIO_Closed;
 
     switch (dir) {
     case eIO_ReadWrite:
@@ -1121,7 +1192,7 @@ extern EIO_Status CONN_Status(CONN conn, EIO_Event dir)
     }
     return conn->meta.status
         ? conn->meta.status(conn->meta.c_status, dir)
-        : eIO_NotSupported;
+        : eIO_Success;
 }
 
 
@@ -1155,7 +1226,7 @@ extern EIO_Status CONN_SetCallback
         static const STimeout* timeout = 0/*dummy*/;
         char errbuf[80];
         sprintf(errbuf, "Unknown callback type #%u", (unsigned int) type);
-        CONN_LOG_EX(29, SetCallback, eLOG_Error, errbuf, eIO_InvalidArg);
+        CONN_LOG_EX(29, SetCallback, eLOG_Critical, errbuf, eIO_InvalidArg);
         assert(0);
         return eIO_InvalidArg;
     }
@@ -1177,19 +1248,20 @@ extern EIO_Status CONN_GetSOCK(CONN conn, SOCK* sock)
     EIO_Status status;
     CONNECTOR  x_conn;
 
-    CONN_NOT_NULL(36, GetSOCK);
-
     if (!sock)
         return eIO_InvalidArg;
     *sock = 0;
 
+    CONN_NOT_NULL(36, GetSOCK);
+
     /* perform open, if not opened yet */
     if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
         return status;
-    assert((conn->state & eCONN_Open)  &&  conn->meta.list);
+    assert(conn->state == eCONN_Open  &&  conn->meta.list);
 
     x_conn = conn->meta.list;
-    if (x_conn  &&  x_conn->meta  &&  x_conn->meta->get_type) {
+    if (x_conn->meta  &&  x_conn->meta->get_type) {
+        /* check to see if it's SOCK-based at the lowest level */
         const char* type = x_conn->meta->get_type(x_conn->meta->c_get_type);
         if (type == g_kNcbiSockNameAbbr
             ||  ((type = strrchr(type, '/')) != 0
@@ -1214,7 +1286,7 @@ extern EIO_Status CONN_SetFlags(CONN conn, TCONN_Flags flags)
         return eIO_InvalidArg;
 
     flags &= (TCONN_Flags)(~fCONN_Flush);
-    flags |= conn->flags & fCONN_Flush;
+    flags |=  conn->flags & fCONN_Flush;
     conn->flags = flags;
     return eIO_Success;
 }
