@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNSSL__H
 #define CONNECT___NCBI_CONNSSL__H
 
-/* $Id: ncbi_connssl.h 629075 2021-04-09 14:32:17Z ivanov $
+/* $Id: ncbi_connssl.h 630479 2021-05-03 22:24:04Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,9 +43,9 @@ extern "C" {
 
 
 typedef enum {
-    /* NB: Must be divisible by 100 decimal */
+    /* NB: Must be a positive 32-bit int divisible by 100 decimal */
     eNcbiCred_GnuTls  = 0x484FFB94,
-    eNcbiCred_MbedTls = 0xC12CC114
+    eNcbiCred_MbedTls = 0x412CC144
 } ENcbiCred;
 
 
@@ -133,6 +133,33 @@ struct SOCKSSL_struct {
 };
 
 
+/* Internal certificate credentials management routines */
+
+#if defined(HAVE_LIBMBEDTLS)  ||  defined(NCBI_CXX_TOOLKIT)
+
+NCBI_CRED NcbiCreateMbedTlsCertCredentials(const void* cert,
+                                           size_t      certsz,
+                                           const void* pkey,
+                                           size_t      pkeysz);
+
+void NcbiDeleteMbedTlsCertCredentials(NCBI_CRED cred);
+
+#endif /*HAVE_LIBMBEDTLS || NCBI_CXX_TOOLKIT*/
+
+
+#ifdef HAVE_LIBGNUTLS
+
+NCBI_CRED NcbiCreateGnuTlsCertCredentials(const void* cert,
+                                          size_t      certsz,
+                                          const void* pkey,
+                                          size_t      pkeysz);
+
+
+void NcbiDeleteGnuTlsCertCredentials(NCBI_CRED cred);
+
+#endif /*HAVE_LIBGNUTLS*/
+
+
 #ifdef __cplusplus
 }  /* extern "C" */
 #endif
