@@ -1,4 +1,4 @@
-/* $Id: ncbi_core_cxx.cpp 668369 2023-06-03 01:48:35Z lavr $
+/* $Id: ncbi_core_cxx.cpp 695804 2025-04-01 13:23:45Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,7 @@
 #include <corelib/ncbidbg.hpp>
 #include <corelib/ncbi_param.hpp>
 #include <corelib/request_ctx.hpp>
+#include <corelib/ncbi_safe_static.hpp>
 #include <connect/error_codes.hpp>
 #include <connect/ncbi_core_cxx.hpp>
 #include <connect/ncbi_monkey.hpp>
@@ -57,19 +58,19 @@ BEGIN_NCBI_SCOPE
 
 
 NCBI_PARAM_DECL  (bool, CONN, TRACE_REG);
-NCBI_PARAM_DEF_EX(bool, CONN, TRACE_REG,
-                  false, eParam_Default, CONN_TRACE_REG);
-static NCBI_PARAM_TYPE (CONN, TRACE_REG) s_TraceReg;
+NCBI_PARAM_DEF_EX(bool, CONN, TRACE_REG, false, eParam_Default, CONN_TRACE_REG);
+using TTraceReg = NCBI_PARAM_TYPE(CONN, TRACE_REG);
+static CSafeStatic<TTraceReg> s_TraceReg;
 
 NCBI_PARAM_DECL  (bool, CONN, TRACE_LOG);
-NCBI_PARAM_DEF_EX(bool, CONN, TRACE_LOG,
-                  false, eParam_Default, CONN_TRACE_LOG);
-static NCBI_PARAM_TYPE (CONN, TRACE_LOG) s_TraceLog;
+NCBI_PARAM_DEF_EX(bool, CONN, TRACE_LOG, false, eParam_Default, CONN_TRACE_LOG);
+using TTraceLog = NCBI_PARAM_TYPE (CONN, TRACE_LOG);
+static CSafeStatic<TTraceLog> s_TraceLog;
 
 NCBI_PARAM_DECL  (bool, CONN, TRACE_LOCK);
-NCBI_PARAM_DEF_EX(bool, CONN, TRACE_LOCK,
-                  false, eParam_Default, CONN_TRACE_LOCK);
-static NCBI_PARAM_TYPE (CONN, TRACE_LOCK) s_TraceLock;
+NCBI_PARAM_DEF_EX(bool, CONN, TRACE_LOCK, false, eParam_Default, CONN_TRACE_LOCK);
+using TTraceLock = NCBI_PARAM_TYPE (CONN, TRACE_LOCK);
+static CSafeStatic<TTraceLock> s_TraceLock;
 
 
 static TCORE_Set s_CORE_Set = 0;
@@ -107,7 +108,7 @@ static string x_Reg(const char* section, const char* name,
             x_storage = ", <Persistent>";
             break;
         default:
-            x_storage = ", <" + NStr::IntToString(int(storage)) + '>';
+            x_storage = ", <#" + NStr::IntToString(int(storage)) + '>';
             break;
         }
     }
@@ -120,11 +121,11 @@ static int s_REG_Get(void* user_data,
                       const char* section, const char* name,
                       char* value, size_t value_size) THROWS_NONE
 {
-    if (s_TraceReg.Get()) {
+    if (s_TraceReg->Get()) {
         _TRACE("s_REG_Get(" + NStr::PtrToString(user_data) + ", "
                + x_Reg(section, name) + ')');
     }
-    int result = 0/*assume error, including truncation*/;
+    int result = 0/*assume failure, including truncation*/;
     try {
         string item
             = static_cast<const IRegistry*> (user_data)->Get(section, name);
@@ -136,17 +137,16 @@ static int s_REG_Get(void* user_data,
                 result = 1/*success*/;
             strncpy0(value, item.data(), len);
 
-            if (s_TraceReg.Get()) {
+            if (s_TraceReg->Get()) {
                 _TRACE("s_REG_Get(" + NStr::PtrToString(user_data) + ", "
                        + x_Reg(section, name) + ") = \"" + string(value)
                        + (result ? "\"" : "\" <Truncated>"));
             }
         } else
-            result = -1/*unmodified*/;
+            result = -1/*success,unmodified*/;
     }
     NCBI_CATCH_ALL_X(1, "s_REG_Get(" + NStr::PtrToString(user_data) + ", "
-                     + x_Reg(section, name)
-                     + ") failed");
+                     + x_Reg(section, name) + ") failed");
     return result;
 }
 }
@@ -157,12 +157,12 @@ static int s_REG_Set(void* user_data,
                      const char* section, const char* name,
                      const char* value, EREG_Storage storage) THROWS_NONE
 {
-    if (s_TraceReg.Get()) {
+    if (s_TraceReg->Get()) {
         _TRACE("s_REG_" + string(value ? "Set" : "Unset") + '('
                + NStr::PtrToString(user_data) + ", "
                + x_Reg(section, name, value ? value : "", storage) + ')');
     }
-    int result = 0;
+    int result = 0/*assume failure*/;
     try {
         IRWRegistry* reg = static_cast<IRWRegistry*> (user_data);
         result = value
@@ -189,7 +189,7 @@ extern "C" {
 static void s_REG_Cleanup(void* user_data) THROWS_NONE
 {
 
-    if (s_TraceReg.Get())
+    if (s_TraceReg->Get())
         _TRACE("s_REG_Cleanup(" + NStr::PtrToString(user_data) + ')');
     try {
         static_cast<const IRegistry*> (user_data)->RemoveReference();
@@ -202,7 +202,7 @@ static void s_REG_Cleanup(void* user_data) THROWS_NONE
 
 extern REG REG_cxx2c(IRWRegistry* reg, bool pass_ownership)
 {
-    if (s_TraceReg.Get())
+    if (s_TraceReg->Get())
         _TRACE("REG_cxx2c(" + NStr::PtrToString(reg) + ')');
     if (!reg)
         return 0;
@@ -216,7 +216,7 @@ extern REG REG_cxx2c(IRWRegistry* reg, bool pass_ownership)
 
 extern REG REG_cxx2c(const IRWRegistry* reg, bool pass_ownership)
 {
-    if (s_TraceReg.Get())
+    if (s_TraceReg->Get())
         _TRACE("REG_cxx2c(const " + NStr::PtrToString(reg) + ')');
     if (!reg)
         return 0;
@@ -256,7 +256,7 @@ static string x_Log(ELOG_Level level)
             x_level = "Fatal";
             break;
         default:
-            x_level = NStr::IntToString(int(level));
+            x_level = '#' + NStr::IntToString(int(level));
             break;
     }
     return x_level;
@@ -267,7 +267,9 @@ extern "C" {
 static void s_LOG_Handler(void*             /*data*/,
                           const SLOG_Message* mess) THROWS_NONE
 {
-    if (s_TraceLog.Get())
+    static const char kOutOfMemory[] = "Ouch! Out of memory";
+
+    if (s_TraceLog->Get())
         _TRACE("s_LOG_Handler(" + x_Log(mess->level) + ')');
     try {
         EDiagSev level;
@@ -302,7 +304,17 @@ static void s_LOG_Handler(void*             /*data*/,
                               mess->module);
         CNcbiDiag diag(info, level);
         diag.SetErrorCode(mess->err_code, mess->err_subcode);
-        diag << mess->message;
+        size_t message_len;
+        if (mess->message  &&  *mess->message) {
+            message_len = strlen(mess->message);
+            do {
+                if (!isspace((unsigned char) mess->message[message_len - 1]))
+                    break;
+            } while (--message_len > 0);
+        } else
+            message_len = mess->message ? 0 : sizeof(kOutOfMemory) - 1;
+        if (message_len)
+            diag << string(mess->message ? mess->message : kOutOfMemory, message_len);
         if (mess->raw_size) {
             typedef AutoPtr< char, ArrayDeleter<char> >  TTempCharPtr;
             TTempCharPtr buf;
@@ -336,7 +348,7 @@ static void s_LOG_Handler(void*             /*data*/,
 
 extern LOG LOG_cxx2c(void)
 {
-    if (s_TraceLog.Get())
+    if (s_TraceLog->Get())
         _TRACE("LOG_cxx2c()");
     return LOG_Create(0, s_LOG_Handler, 0, 0);
 }
@@ -367,7 +379,7 @@ static string x_Lock(EMT_Lock how)
         x_how = "TryLockRead";
         break;
     default:
-        x_how = NStr::IntToString(int(how));
+        x_how = '#' + NStr::IntToString(int(how));
         break;
     }
     return x_how;
@@ -377,7 +389,7 @@ static string x_Lock(EMT_Lock how)
 extern "C" {
 static int/*bool*/ s_LOCK_Handler(void* user_data, EMT_Lock how) THROWS_NONE
 {
-    if (s_TraceLock.Get()) {
+    if (s_TraceLock->Get()) {
         _TRACE("s_LOCK_Handler(" + NStr::PtrToString(user_data) + ", "
                + x_Lock(how) + ')');
     }
@@ -386,30 +398,29 @@ static int/*bool*/ s_LOCK_Handler(void* user_data, EMT_Lock how) THROWS_NONE
         switch (int(how)) {
         case eMT_Lock:
             lock->WriteLock();
-            break;
+            return 1/*success*/;
         case eMT_LockRead:
             lock->ReadLock();
-            break;
+            return 1/*success*/;
         case eMT_Unlock:
             lock->Unlock();
-            break;
+            return 1/*success*/;
         case eMT_TryLock:
-            if (!lock->TryWriteLock())
-                return 0/*false*/;
+            if (lock->TryWriteLock())
+                return 1/*success*/;
             break;
         case eMT_TryLockRead:
-            if (!lock->TryReadLock())
-                return 0/*false*/;
+            if (lock->TryReadLock())
+                return 1/*success*/;
             break;
         default:
-            NCBI_THROW(CCoreException, eCore, "Lock used with unknown op #" +
-                       NStr::UIntToString((unsigned int) how));
+            NCBI_THROW(CCoreException, eCore, "Lock used with unknown op #"
+                       + NStr::IntToString(int(how)));
         }
-        return 1/*true*/;
     }
     NCBI_CATCH_ALL_X(5, "s_LOCK_Handler(" + NStr::PtrToString(user_data) + ", "
                      + x_Lock(how) + ") failed");
-    return 0/*false*/;
+    return 0/*failure*/;
 }
 }
 
@@ -417,7 +428,7 @@ static int/*bool*/ s_LOCK_Handler(void* user_data, EMT_Lock how) THROWS_NONE
 extern "C" {
 static void s_LOCK_Cleanup(void* user_data) THROWS_NONE
 {
-    if (s_TraceLock.Get())
+    if (s_TraceLock->Get())
         _TRACE("s_LOCK_Cleanup(" + NStr::PtrToString(user_data) + ')');
     try {
         delete static_cast<CRWLock*> (user_data);
@@ -430,7 +441,7 @@ static void s_LOCK_Cleanup(void* user_data) THROWS_NONE
 
 extern MT_LOCK MT_LOCK_cxx2c(CRWLock* lock, bool pass_ownership)
 {
-    if (s_TraceLock.Get())
+    if (s_TraceLock->Get())
         _TRACE("MT_LOCK_cxx2c(" + NStr::PtrToString(lock) + ')');
     return MT_LOCK_Create(static_cast<void*> (lock ? lock : new CRWLock),
                           s_LOCK_Handler,
@@ -485,6 +496,12 @@ static char* s_GetRequestID(ENcbiRequestID reqid)
     case eNcbiRequestID_HitID:
         id = CDiagContext::GetRequestContext().GetNextSubHitID();
         break;
+    case eNcbiTraceState:
+        id = CDiagContext::GetRequestContext().GetTraceState();
+        break;
+    case eNcbiTraceParent:
+        id = CDiagContext::GetRequestContext().GetTraceParent();
+        break;
     default:
         return 0;
     }
@@ -605,23 +622,32 @@ static void s_SetMonkeyHooks(EMonkeyHookSwitch hook_switch)
         g_MONKEY_SockHasSocket = s_MonkeySockHasSocket;
         break;
     default:
+        _TROUBLE;
         break;
     }
 }
 #endif //NCBI_MONKEY
 
 
+#ifdef NCBI_POSIX_THREADS
 extern "C" {
 static void x_PreFork(void)
 {
     CORE_LOCK_WRITE;
 }
 
-static void x_PostFork(void)
+static void x_PostForkParent(void)
 {
     CORE_UNLOCK;
 }
+
+static void x_PostForkChild(void)
+{
+    if (!g_CORE_SkipPostForkChildUnlock)
+        CORE_UNLOCK;
+}
 } // extern "C"
+#endif // NCBI_POSIX_THREADS
 
 
 enum EConnectInit {
@@ -718,7 +744,7 @@ static void s_Init(const IRWRegistry* reg  = 0,
         if (x_set  &&  atexit(s_Fini) != 0)
             err |= 1;
 #ifdef NCBI_POSIX_THREADS
-        if (pthread_atfork(x_PreFork, x_PostFork, x_PostFork) != 0)
+        if (pthread_atfork(x_PreFork, x_PostForkParent, x_PostForkChild) != 0)
             err |= 2;
 #endif // NCBI_POSIX_THREADS
         if (err) {
