@@ -1,4 +1,4 @@
-/* $Id: ncbi_ftp_connector.c 662326 2023-01-27 16:15:31Z lavr $
+/* $Id: ncbi_ftp_connector.c 689727 2024-11-05 14:13:24Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -52,12 +52,6 @@
 #include <connect/ncbi_ftp_connector.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include <time.h>
-
-#if !defined(NCBI_OS_MSWIN)  &&  !defined(__CYGWIN__)
-#  define _timezone timezone
-#  define _daylight daylight
-#endif /*!NCBI_OS_MSWIN && !__CYGWIN__*/
 
 #define NCBI_USE_ERRCODE_X   Connect_FTP
 
@@ -1414,6 +1408,9 @@ static EIO_Status x_FTPParseMdtm(SFTPConnector* xxx, char* timestamp)
     static const int kDay[12] = {31,  0, 31, 30, 31, 30,
                                  31, 31, 30, 31, 30, 31};
     char* frac = strchr(timestamp, '.');
+#ifndef HAVE_TIMEGM
+    time_t tzdiff;
+#endif /*HAVE_TIMEGM*/
     int field[6], n;
     struct tm tm;
     char buf[80];
@@ -1437,7 +1434,7 @@ static EIO_Status x_FTPParseMdtm(SFTPConnector* xxx, char* timestamp)
      *    input format specifiers in the format string (%-based);
      * 2. None to any spaces are allowed to match a space in the format, whilst
      *    an MDTM response must not contain even a single space. */
-    for (n = 0;  n < 6;  ++n) {
+    for (n = 0;  n < (int)(sizeof(field)/sizeof(field[0]));  ++n) {
         len = n ? 2 : 4/*year*/;
         if (len != strlen(strncpy0(buf, timestamp, len))  ||
             len != strspn(buf, kDigits)) {
@@ -1453,7 +1450,7 @@ static EIO_Status x_FTPParseMdtm(SFTPConnector* xxx, char* timestamp)
     if (field[1] < 1  ||  field[1] > 12)
         return eIO_Unknown;
     tm.tm_mon   = field[1] - 1;
-    if (field[2] < 1  ||  field[2] > (field[1] != 2
+    if (field[2] < 1  ||  field[2] > (field[1] != 2/*Feb*/
                                       ? kDay[tm.tm_mon]
                                       : 28 +
                                       (!(field[0] % 4)  &&
@@ -1471,24 +1468,26 @@ static EIO_Status x_FTPParseMdtm(SFTPConnector* xxx, char* timestamp)
     if (field[5] < 0  ||  field[5] > 60) /* allow one leap second */
         return eIO_Unknown;
     tm.tm_sec   = field[5];
+
 #ifdef HAVE_TIMEGM
-    tm.tm_isdst = -1;
     if ((t = timegm(&tm)) == (time_t)(-1))
         return eIO_Unknown;
 #else
-    tm.tm_isdst =  0;
+    n = tm.tm_hour;
+    tm.tm_isdst = -1;
     if ((t = mktime(&tm)) == (time_t)(-1))
         return eIO_Unknown;
-#  if !defined(NCBI_OS_DARWIN)  &&  !defined(NCBI_OS_BSD)
-    /* NB: timezone information is unavailable on Darwin or BSD :-/ */
-    if (t >= _timezone)
-        t -= _timezone;
-    if (t >= _daylight  &&  tm.tm_isdst > 0)
-        t -= _daylight;
-#  endif /*!NCBI_OS_DARWIN && !NCBI_OS_BSD*/
+
+    tzdiff = UTIL_Timezone();
+
+    if (t >= tzdiff)
+        t -= tzdiff;
+    if (tm.tm_isdst > 0  &&  n == tm.tm_hour)
+        t += 3600;
 #endif /*HAVE_TIMEGM*/
+
     n = sprintf(buf, "%lu%s%-.9s", (unsigned long) t,
-                frac  &&  *frac ? "." : "", frac ? frac : "");
+                &"."[!frac || !*frac], frac ? frac : "");
     if (n <= 0  ||  !BUF_Write(&xxx->rbuf, buf, (size_t) n))
         return eIO_Unknown;
     return eIO_Success;
@@ -1977,8 +1976,11 @@ static EIO_Status s_VT_Open
     assert(!xxx->what  &&  !xxx->data  &&  !xxx->cntl);
     do {
         assert(!BUF_Size(xxx->wbuf)  &&  !BUF_Size(xxx->rbuf));
-        if (xxx->info->debug_printout)
+        if (xxx->info->debug_printout) {
+            CORE_LOCK_READ;
             ConnNetInfo_Log(xxx->info, eLOG_Note, CORE_GetLOG());
+            CORE_UNLOCK;
+        }
         status = SOCK_CreateEx(xxx->info->host, xxx->info->port, timeout,
                                &xxx->cntl, 0, 0, fSOCK_KeepAlive
                                | (xxx->flag & fFTP_LogControl
@@ -2408,7 +2410,7 @@ extern CONNECTOR s_CreateConnector(const SConnNetInfo*  info,
     xinfo->stateless = 0;
     xinfo->lb_disable = 0;
     xinfo->http_proxy_leak = 0;
-    if (!(flag & fFTP_UseProxy)  ||  xinfo->http_proxy_only) {
+    if (!(flag & fFTP_UseProxy)  ||  xinfo->http_proxy_mask == fProxy_Http) {
         xinfo->http_proxy_host[0] = '\0';
         xinfo->http_proxy_port    =   0;
         xinfo->http_proxy_user[0] = '\0';
