@@ -1,4 +1,4 @@
-/* $Id: ncbi_gnutls.c 645948 2022-02-25 14:51:41Z lavr $
+/* $Id: ncbi_gnutls.c 688079 2024-09-24 21:39:23Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -89,7 +89,7 @@ static int gcry_user_mutex_destroy(void** lock)
             *lock = 0;
         return 0;
     }
-    CORE_LOG_X(49, eLOG_Warning, "NULL MT_LOCK deinit in GCRYPT");
+    CORE_LOG_X(40, eLOG_Warning, "NULL MT_LOCK deinit in GCRYPT");
     return EINVAL;
 }
 static int gcry_user_mutex_lock(void** lock)
@@ -131,7 +131,7 @@ static int gtls_user_mutex_deinit(void** lock)
             *lock = 0;
         return 0;
     }
-    CORE_LOG_X(49, eLOG_Warning, "NULL MT_LOCK deinit in GNUTLS");
+    CORE_LOG_X(40, eLOG_Warning, "NULL MT_LOCK deinit in GNUTLS");
     return EINVAL;
 }
 static int gtls_user_mutex_lock(void** lock)
@@ -207,7 +207,7 @@ static void x_GnuTlsLogger(int level, const char* message)
     if (message[len - 1] == '\n')
         --len;
     CORE_LOGF_X(21, eLOG_Note,
-                ("GNUTLS%d: %.*s", level, (int) len, message));
+                ("GNUTLS%02d: %.*s", level, (int) len, message));
 }
 
 
@@ -225,7 +225,7 @@ static EIO_Status x_RetryStatus(SOCK sock, EIO_Event direction)
             : eIO_Closed;
     } else
         status = SOCK_Status(sock, direction);
-    return status == eIO_Success ? eIO_Timeout : status;
+    return status;
 }
 
 
@@ -250,6 +250,7 @@ static EIO_Status x_AlertToStatus(gnutls_alert_description_t alert,
         status = eIO_Unknown;
         break;
     }
+    assert(status != eIO_Success);
     return status;
 }
 
@@ -294,7 +295,7 @@ static EIO_Status x_ErrorToStatus(int* error, gnutls_session_t session,
     else
         status = *error == GNUTLS_E_SESSION_EOF ? eIO_Closed : eIO_Unknown;
 
-    assert(status != eIO_Success);
+    assert(*error == GNUTLS_E_AGAIN  ||  status != eIO_Success);
     CORE_TRACEF(("GNUTLS error %d%s -> CONNECT GNUTLS status %s",
                  *error, gnutls_error_is_fatal(*error) ? "(fatal)" : "",
                  IO_StatusStr(status)));
@@ -360,11 +361,15 @@ static void* s_GnuTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
     size_t len;
     int err;
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACE("GnuTlsCreate(): Enter");
+
     if (end == GNUTLS_SERVER) {
         CORE_LOG_X(22, eLOG_Critical,
                    "Server-side SSL not yet supported with GNUTLS");
         *error = NCBI_NOTSUPPORTED;
-        return 0;
+        session = 0;
+        goto out;
     }
 
     CORE_LOCK_READ;
@@ -381,15 +386,14 @@ static void* s_GnuTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
                      acred ? "Invalid format" : "Not initialized"));
         /*FIXME: there's a NULL(data)-terminated array of credentials */
         *error = acred ? EINVAL : NCBI_NOTSUPPORTED;
-        return 0;
+        session = 0;
+        goto out;
     }
 
-    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
-        CORE_TRACE("GnuTlsCreate(): Enter");
-
     if ((err = gnutls_init(&session, end)) != GNUTLS_E_SUCCESS/*0*/) {
         *error = err;
-        return 0;
+        session = 0;
+        goto out;
     }
 
     /* Check CONN_GNUTLS_PRIORITY or [CONN]GNUTLS_PRIORITY */
@@ -420,18 +424,19 @@ static void* s_GnuTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
                                                 ctx->host, len))       != 0)) {
         gnutls_deinit(session);
         *error = err;
-        return 0;
+        session = 0;
+        goto out;
     }
 
     gnutls_transport_set_pull_function(session, x_GnuTlsPull);
     gnutls_transport_set_push_function(session, x_GnuTlsPush);
     gnutls_transport_set_ptr(session, ctx);
-    gnutls_session_set_ptr(session, ctx);
 
 #  if LIBGNUTLS_VERSION_NUMBER >= 0x030000
     gnutls_handshake_set_timeout(session, 0);
 #  endif /*LIBGNUTLS_VERSION_NUMBER>=3.0.0*/
 
+out:
     CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
         CORE_TRACEF(("GnuTlsCreate(): Leave(%p)", session));
 
@@ -454,22 +459,23 @@ static EIO_Status s_GnuTlsOpen(void* session, int* error, char** desc)
     if (x_error < 0) {
         status = x_ErrorToStatus(&x_error, (gnutls_session_t) session,
                                  eIO_Open);
-        assert(status != eIO_Success);
+        if (status == eIO_Success)
+            status  = eIO_Unknown;
         *error = x_error;
         if (desc)
             *desc = 0;
     } else {
+        status = eIO_Success;
         if (desc) {
 #  if LIBGNUTLS_VERSION_NUMBER >= 0x030110
             char* temp = gnutls_session_get_desc((gnutls_session_t) session);
             if (temp) {
-                *desc = strdup(temp);
+                *desc = *temp ? strdup(temp) : 0;
                 gnutls_free(temp);
             } else
 #  endif /*LIBGNUTLS_VERSION_NUMBER>=3.1.10*/
                 *desc = 0;
         }
-        status = eIO_Success;
     }
 
     CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
@@ -504,8 +510,7 @@ static void x_set_errno(gnutls_session_t session, int error)
 }
 
 
-static ssize_t x_GnuTlsPull(gnutls_transport_ptr_t ptr,
-                            void* buf, size_t size)
+static ssize_t x_GnuTlsPull(gnutls_transport_ptr_t ptr, void* buf, size_t size)
 {
     SNcbiSSLctx* ctx = (SNcbiSSLctx*) ptr;
     FSSLPull pull = s_Pull;
@@ -531,9 +536,38 @@ static ssize_t x_GnuTlsPull(gnutls_transport_ptr_t ptr,
 }
 
 
-static ssize_t x_GnuTlsPush(gnutls_transport_ptr_t ptr,
-                            const void* data, size_t size)
+static EIO_Status s_GnuTlsRead(void* session, void* buf,
+                               size_t n_todo, size_t* n_done, int* error)
 {
+    EIO_Status status;
+    ssize_t    x_read;
+
+    assert(session  &&  buf  &&  n_todo > 0);
+
+again:
+    x_read = gnutls_record_recv((gnutls_session_t) session, buf, n_todo);
+    assert(x_read < 0  ||  (size_t) x_read <= n_todo);
+
+    if (x_read <= 0) {
+        int x_error = (int) x_read;
+        status = x_ErrorToStatus(&x_error, (gnutls_session_t) session,
+                                 eIO_Read);
+        if (status == eIO_Success  &&  x_error == GNUTLS_E_AGAIN)
+            goto again;
+        *error = x_error;
+        x_read = 0;
+    } else
+        status = eIO_Success;
+
+    *n_done = (size_t) x_read;
+    return status;
+}
+
+
+static ssize_t x_GnuTlsPush(gnutls_transport_ptr_t ptr,
+                            const void* x_data, size_t size)
+{
+    const unsigned char* data = (const unsigned char*) x_data;
     SNcbiSSLctx* ctx = (SNcbiSSLctx*) ptr;
     FSSLPush push = s_Push;
     EIO_Status status;
@@ -551,9 +585,9 @@ static ssize_t x_GnuTlsPush(gnutls_transport_ptr_t ptr,
             } else {
                 assert(status == eIO_Success);
                 assert(x_written <= size);
-                n_written += (ssize_t)            x_written;
-                size      -=                      x_written;
-                data       = (const char*) data + x_written;
+                n_written += (ssize_t) x_written;
+                size      -=           x_written;
+                data      +=           x_written;
             }
         } while (size);
         x_set_errno((gnutls_session_t) ctx->sess, 0);
@@ -569,38 +603,13 @@ static ssize_t x_GnuTlsPush(gnutls_transport_ptr_t ptr,
 }
 
 
-static EIO_Status s_GnuTlsRead(void* session, void* buf, size_t n_todo,
-                               size_t* n_done, int* error)
-{
-    EIO_Status status;
-    ssize_t    x_read;
-
-    assert(session);
-
-    x_read = gnutls_record_recv((gnutls_session_t) session, buf, n_todo);
-    assert(x_read < 0  ||  (size_t) x_read <= n_todo);
-
-    if (x_read <= 0) {
-        int x_error = (int) x_read;
-        status = x_ErrorToStatus(&x_error, (gnutls_session_t) session,
-                                 eIO_Read);
-        *error = x_error;
-        x_read = 0;
-    } else
-        status = eIO_Success;
-
-    *n_done = (size_t) x_read;
-    return status;
-}
-
-
-static EIO_Status x_GnuTlsWrite(void* session, const void* data, size_t n_todo,
-                                size_t* n_done, int* error)
+static EIO_Status x_GnuTlsWrite(void* session, const unsigned char* data,
+                                size_t n_todo, size_t* n_done, int* error)
 {
     EIO_Status status;
     ssize_t    x_written;
 
-    assert(session);
+    assert(data  &&  n_todo > 0);
 
     x_written = gnutls_record_send((gnutls_session_t) session, data, n_todo);
     assert(x_written < 0  ||  (size_t) x_written <= n_todo);
@@ -609,6 +618,8 @@ static EIO_Status x_GnuTlsWrite(void* session, const void* data, size_t n_todo,
         int x_error = (int) x_written;
         status = x_ErrorToStatus(&x_error, (gnutls_session_t) session,
                                  eIO_Write);
+        if (status == eIO_Success)
+            status  = eIO_Unknown;
         *error = x_error;
         x_written = 0;
     } else
@@ -619,15 +630,18 @@ static EIO_Status x_GnuTlsWrite(void* session, const void* data, size_t n_todo,
 }
 
 
-static EIO_Status s_GnuTlsWrite(void* session, const void* data, size_t n_todo,
-                                size_t* n_done, int* error)
+static EIO_Status s_GnuTlsWrite(void* session, const void* x_data,
+                                size_t n_todo, size_t* n_done, int* error)
 {
+    const unsigned char* data = (const unsigned char*) x_data;
     size_t max_size = gnutls_record_get_max_size((gnutls_session_t) session);
     EIO_Status status;
 
+    assert(session  &&  data  &&  n_todo > 0);
+
     *n_done = 0;
 
-    do {
+    for (;;) {
         size_t x_todo = n_todo > max_size ? max_size : n_todo;
         size_t x_done;
         status = x_GnuTlsWrite(session, data, x_todo, &x_done, error);
@@ -640,8 +654,10 @@ static EIO_Status s_GnuTlsWrite(void* session, const void* data, size_t n_todo,
         if (x_todo != x_done)
             break;
         n_todo  -= x_done;
-        data     = (const char*) data + x_done;
-    } while (n_todo);
+        if (!n_todo)
+            break;
+        data    += x_done;
+    }
 
     return *n_done ? eIO_Success : status;
 }
@@ -649,6 +665,7 @@ static EIO_Status s_GnuTlsWrite(void* session, const void* data, size_t n_todo,
 
 static EIO_Status s_GnuTlsClose(void* session, int how, int* error)
 {
+    EIO_Status status;
     int x_error;
 
     assert(session);
@@ -661,14 +678,16 @@ static EIO_Status s_GnuTlsClose(void* session, int how, int* error)
                          ? GNUTLS_SHUT_RDWR
                          : GNUTLS_SHUT_WR);
 
-    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
-        CORE_TRACEF(("GnuTlsClose(%p): Leave", session));
-
     if (x_error != GNUTLS_E_SUCCESS) {
         *error = x_error;
-        return eIO_Unknown;
-    }
-    return eIO_Success;
+        status = eIO_Unknown;
+    } else
+        status = eIO_Success;
+
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsClose(%p): Leave(%d)", session, status));
+
+    return status;
 }
 
 
@@ -740,14 +759,16 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
 {
     gnutls_anon_client_credentials_t acred;
     gnutls_certificate_credentials_t xcred;
+    int/*bool*/ inited = 0/*false*/;
     const char* version;
-    int/*bool*/ inited;
     EIO_Status status;
     const char* val;
     char buf[32];
 
     assert(!s_GnuTlsCredAnon);
 
+    CORE_TRACE("GnuTlsInit(): Enter");
+
     version = gnutls_check_version(0);
     if (strcasecmp(GNUTLS_VERSION, version) != 0) {
         CORE_LOGF_X(26, eLOG_Critical,
@@ -756,10 +777,10 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
         assert(0);
     }
 
-    CORE_TRACE("GnuTlsInit(): Enter");
-
-    if (!pull  ||  !push)
-        return eIO_InvalidArg;
+    if (!pull  ||  !push) {
+        status = eIO_InvalidArg;
+        goto out;
+    }
 
     /* Check CONN_[GNU]TLS_LOGLEVEL or [CONN][GNU]TLS_LOGLEVEL */
     val = ConnNetInfo_GetValueInternal(0, "GNU" REG_CONN_TLS_LOGLEVEL,
@@ -770,13 +791,11 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
                                            buf, sizeof(buf),
                                            DEF_CONN_TLS_LOGLEVEL);
     }
-    CORE_LOCK_READ;
     if (!val  ||  !*val)
         val = getenv("GNUTLS_DEBUG_LEVEL"); /* GNUTLS proprietary setting */
     if (val  &&  *val) {
         ELOG_Level level;
         s_GnuTlsLogLevel = atoi(val);
-        CORE_UNLOCK;
         if (s_GnuTlsLogLevel) {
             gnutls_global_set_log_function(x_GnuTlsLogger);
             if (val == buf)
@@ -786,13 +805,11 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
             level = eLOG_Trace;
         CORE_LOGF_X(27, level, ("GNUTLS V%s (LogLevel=%d)",
                                 version, s_GnuTlsLogLevel));
-    } else
-        CORE_UNLOCK;
+    }
 
     CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
         CORE_TRACE("GnuTlsInit(): Go-on");
 
-    inited = 0/*false*/;
     if ((status = x_InitLocking()) != eIO_Success)
         goto out;
 
@@ -803,13 +820,11 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
     }
     inited = 1/*true*/;
     if (gnutls_anon_allocate_client_credentials(&acred) != 0) {
-        gnutls_global_deinit();
         status = eIO_Unknown;
         goto out;
     }
     if (gnutls_certificate_allocate_credentials(&xcred) != 0) {
         gnutls_anon_free_client_credentials(acred);
-        gnutls_global_deinit();
         status = eIO_Unknown;
         goto out;
     }
@@ -819,17 +834,19 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
     s_GnuTlsCredCert = xcred;
     s_Pull           = pull;
     s_Push           = push;
+    status           = eIO_Success;
+
+out:
+    if (status != eIO_Success) {
+        gnutls_global_set_log_level(s_GnuTlsLogLevel = 0);
+        gnutls_global_set_log_function(0);
+        if (inited)
+            gnutls_global_deinit();
+    }
 
     CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
-        CORE_TRACE("GnuTlsInit(): Leave");
+        CORE_TRACEF(("GnuTlsInit(): Leave(%d)", status));
 
-    return eIO_Success;
-
- out:
-    gnutls_global_set_log_level(s_GnuTlsLogLevel = 0);
-    gnutls_global_set_log_function(0);
-    if (inited)
-        gnutls_global_deinit();
     return status;
 }
 
