@@ -1,4 +1,4 @@
-/* $Id: ncbi_heapmgr.c 604960 2020-04-05 16:23:08Z lavr $
+/* $Id: ncbi_heapmgr.c 689714 2024-11-05 04:57:39Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -119,6 +119,7 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_HeapMgr
 
+
 #if defined(NCBI_OS_MSWIN)  &&  defined(_WIN64)
 /* Disable ptr->long conversion warning (even on explicit cast!) */
 #  pragma warning (disable : 4311)
@@ -150,11 +151,11 @@
 typedef struct HEAP_PACKED {
     SHEAP_Block head;         /* Block head, a free block has the following: */
     TNCBI_Size  prevfree;     /* Heap index for prev (smaller) free block    */
-    TNCBI_Size  nextfree;     /* Heap index for next (bigger) free block     */
+    TNCBI_Size  nextfree;     /* Heap index for next (larger) free block     */
 } SHEAP_HeapBlock;
 
 
-struct SHEAP_tag {
+struct SHEAP_tag {            /* NB: OOB=Out-Of-Bounds, i.e. pointing outside*/
     SHEAP_HeapBlock* base;    /* Base of heap extent:         !base == !size */
     TNCBI_Size       size;    /* # blocks in the heap extent: !base == !size */
     TNCBI_Size       used;    /* # of blocks used (size - used = free blocks)*/
@@ -173,7 +174,7 @@ static int/*bool*/ s_HEAP_fast = 1/*true*/;
 
 #define _HEAP_ALIGN_EX(a, b)  ((((unsigned long)(a) + ((b) - 1)) / (b)) * (b))
 #define _HEAP_ALIGN_2(a, b)   (( (unsigned long)(a) + ((b) - 1)) \
-                               & (unsigned long)(~((b) - 1)))
+                               & (unsigned long)(~((b) - 1)/*or -(b)*/))
 #define _HEAP_SIZESHIFT       4
 #define HEAP_BLOCKS(s)        ((s) >> _HEAP_SIZESHIFT)
 #define HEAP_EXTENT(b)        ((b) << _HEAP_SIZESHIFT)
