@@ -1,4 +1,4 @@
-/* $Id: ncbi_http_session.cpp 612999 2020-07-30 19:22:55Z sadyrovr $
+/* $Id: ncbi_http_session.cpp 680626 2024-03-23 01:54:52Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -71,7 +71,7 @@ static const char* kReservedHeaders[]  = {
     "NCBI-PHID"
 };
 
-static  CSafeStatic<CHttpHeaders::THeaderValues> kEmptyValues;
+static CSafeStatic<CHttpHeaders::THeaderValues> kEmptyValues;
 static const char kHttpHeaderDelimiter = ':';
 
 
@@ -146,7 +146,7 @@ void CHttpHeaders::ClearAll(void)
 }
 
 
-void s_ParseHttpHeader(const CTempString& from, CHttpHeaders::THeaders& to)
+static void s_ParseHttpHeader(const CTempString& from, CHttpHeaders::THeaders& to)
 {
     list<CTempString> lines;
     NStr::Split(from, HTTP_EOL, lines,
@@ -238,7 +238,7 @@ void CHttpFormData::SetContentType(EContentType content_type)
 {
     if (!m_Providers.empty()  &&  content_type != eMultipartFormData) {
         NCBI_THROW(CHttpSessionException, eBadContentType,
-            "The requested Content-Type cannot be used with the form data.");
+            "Requested Content-Type cannot be used with the form data");
     }
     m_ContentType = content_type;
 }
@@ -250,7 +250,7 @@ void CHttpFormData::AddEntry(CTempString entry_name,
 {
     if ( entry_name.empty() ) {
         NCBI_THROW(CHttpSessionException, eBadFormDataName,
-            "Form data entry name must not be empty.");
+            "Form data entry name must not be empty");
     }
     TValues& values = m_Entries[entry_name];
     SFormData entry;
@@ -265,7 +265,7 @@ void CHttpFormData::AddProvider(CTempString             entry_name,
 {
     if ( entry_name.empty() ) {
         NCBI_THROW(CHttpSessionException, eBadFormDataName,
-            "Form data entry name must not be empty.");
+            "Form data entry name must not be empty");
     }
     m_ContentType = eMultipartFormData;
     m_Providers[entry_name].push_back(Ref(provider));
@@ -294,11 +294,11 @@ public:
     {
         try {
             CNcbiIfstream in(m_FileName.c_str(), ios_base::binary);
-            NcbiStreamCopy(out, in);
+            NcbiStreamCopyThrow(out, in);
         }
         catch (...) {
             NCBI_THROW(CHttpSessionException, eBadFormData,
-                "Failed to POST file: " + m_FileName);
+                "Failed to POST file " + m_FileName);
         }
     }
 
@@ -410,9 +410,8 @@ void CHttpFormData::WriteFormData(CNcbiOstream& out) const
         ITERATE(TEntries, values, m_Entries) {
             if (values->second.size() > 1) {
                 NCBI_THROW(CHttpSessionException, eBadFormData,
-                    string("No multiple values per entry are allowed "
-                    "in URL-encoded form data, entry name '") +
-                    values->first + "' ");
+                    "Multiple values not allowed in URL-encoded form data, "
+                    " entry '" + values->first + '\'');
             }
             args.SetValue(values->first, values->second.back().m_Value);
         }
@@ -464,12 +463,12 @@ bool CHttpFormData::IsEmpty(void) const
 
 
 CHttpResponse::CHttpResponse(CHttpSession_Base&   session,
-                             const CUrl&     url,
+                             const CUrl&          url,
                              shared_ptr<iostream> stream)
     : m_Session(&session),
       m_Url(url),
       m_Location(url),
-      m_Stream(move(stream)),
+      m_Stream(std::move(stream)),
       m_Headers(new CHttpHeaders),
       m_StatusCode(0)
 {
@@ -481,9 +480,9 @@ CNcbiIstream& CHttpResponse::ContentStream(void) const
     _ASSERT(m_Stream);
     if ( !CanGetContentStream() ) {
         NCBI_THROW(CHttpSessionException, eBadStream,
-            string("Content stream is not available for status '") +
-            NStr::NumericToString(m_StatusCode) + " " +
-            m_StatusText + "'");
+            "Content stream not available for status '"
+            + NStr::NumericToString(m_StatusCode) + ' '
+            + m_StatusText + '\'');
     }
     return *m_Stream;
 }
@@ -494,9 +493,9 @@ CNcbiIstream& CHttpResponse::ErrorStream(void) const
     _ASSERT(m_Stream);
     if ( CanGetContentStream() ) {
         NCBI_THROW(CHttpSessionException, eBadStream,
-            string("Error stream is not available for status '") +
-            NStr::NumericToString(m_StatusCode) + " " +
-            m_StatusText + "'");
+            "Error stream not available for status '"
+            + NStr::NumericToString(m_StatusCode) + ' '
+            + m_StatusText + '\'');
     }
     return *m_Stream;
 }
@@ -510,14 +509,44 @@ bool CHttpResponse::CanGetContentStream(void) const
 
 void CHttpResponse::x_Update(CHttpHeaders::THeaders headers, int status_code, string status_text)
 {
+    // Prevent collecting multiple headers on redirects.
     m_Headers->m_Headers.swap(headers);
     m_StatusCode = status_code;
-    m_StatusText = move(status_text);
+    m_StatusText = std::move(status_text);
     const auto& cookies = m_Headers->GetAllValues(CHttpHeaders::eSetCookie);
     m_Session->x_SetCookies(cookies, &m_Location);
 }
 
 
+///////////////////////////////////////////////////////
+//  CTlsCertCredentials::
+//
+
+
+CTlsCertCredentials::CTlsCertCredentials(const CTempStringEx& cert, const CTempStringEx& pkey)
+    : m_Cert(cert), m_PKey(pkey)
+{
+    if (cert.HasZeroAtEnd()) m_Cert.push_back('\0');
+    if (pkey.HasZeroAtEnd()) m_PKey.push_back('\0');
+}
+
+CTlsCertCredentials::~CTlsCertCredentials(void)
+{
+    if ( m_Cred ) {
+        NcbiDeleteTlsCertCredentials(m_Cred);
+    }
+}
+
+
+NCBI_CRED CTlsCertCredentials::GetNcbiCred(void) const
+{
+    if ( !m_Cred ) {
+        m_Cred = NcbiCreateTlsCertCredentials(m_Cert.data(), m_Cert.size(), m_PKey.data(), m_PKey.size());
+    }
+    return m_Cred;
+}
+
+
 ///////////////////////////////////////////////////////
 //  CHttpRequest::
 //
@@ -525,13 +554,9 @@ void CHttpResponse::x_Update(CHttpHeaders::THeaders headers, int status_code, st
 
 unsigned short SGetHttpDefaultRetries::operator()(void) const
 {
-#define _STR(s)  #s
-#define  STR(s)  _STR(s)
     char buf[16];
-    ConnNetInfo_GetValueInternal(0, REG_CONN_MAX_TRY,
-                                 buf, sizeof(buf), STR(DEF_CONN_MAX_TRY));
-#undef   STR
-#undef  _STR
+    ConnNetInfo_GetValueInternal(0, REG_CONN_MAX_TRY, buf, sizeof(buf),
+                                 NCBI_AS_STRING(DEF_CONN_MAX_TRY));
     int maxtry = atoi(buf);
     return (unsigned short)(maxtry ? maxtry - 1 : 0);
 }
@@ -546,17 +571,16 @@ unsigned short x_RetriesToMaxtry(unsigned short retries)
 
 CHttpRequest::CHttpRequest(CHttpSession_Base& session,
                            const CUrl&   url,
-                           EReqMethod    method)
+                           EReqMethod    method,
+                           const CHttpParam& param)
     : m_Session(&session),
       m_Url(url),
-      m_IsService(url.IsService()),
       m_Method(method),
       m_Headers(new CHttpHeaders),
-      m_Timeout(CTimeout::eDefault),
-      m_Deadline(CTimeout::eDefault),
-      m_RetryProcessing(ESwitch::eDefault),
-      m_AdjustUrl(0)
+      m_AdjustUrl(0),
+      m_Credentials(session.GetCredentials())
 {
+    SetParam(param);
 }
 
 
@@ -693,7 +717,7 @@ CHttpResponse CHttpRequest::Execute(void)
         if ( !m_Response ) {
             if (m_Stream) {
                 NCBI_THROW(CHttpSessionException, eBadRequest,
-                    "An attempt to execute HTTP request already being executed");
+                    "Attempt to execute HTTP request already being executed");
             }
 
             m_Session->x_StartRequest(protocol, *this, have_data);
@@ -724,7 +748,7 @@ CNcbiOstream& CHttpRequest::ContentStream(void)
     if ( !m_Stream ) {
         if (m_Response) {
             NCBI_THROW(CHttpSessionException, eBadRequest,
-                "An attempt to execute HTTP request already being executed");
+                "Attempt to execute HTTP request already being executed");
         }
 
         m_Session->x_StartRequest(m_Session->GetProtocol(), *this, false);
@@ -765,15 +789,67 @@ void CHttpRequest::x_AdjustHeaders(bool use_form_data)
 void CHttpRequest::x_UpdateResponse(CHttpHeaders::THeaders headers, int status_code, string status_text)
 {
     if (m_Response) {
-        m_Response->x_Update(move(headers), status_code, move(status_text));
+        m_Response->x_Update(std::move(headers), status_code, std::move(status_text));
     }
 }
 
 
+void CHttpRequest::x_SetProxy(SConnNetInfo& net_info)
+{
+    CHttpProxy proxy = GetProxy();
+    // Try per-session proxy, if any.
+    if ( proxy.IsEmpty() ) proxy = m_Session->GetProxy();
+    if ( proxy.IsEmpty() ) return;
+
+    if (proxy.GetHost().size() > CONN_HOST_LEN) {
+        NCBI_THROW(CHttpSessionException, eConnFailed,
+            "Proxy host length exceeds " NCBI_AS_STRING(CONN_HOST_LEN));
+    }
+    memcpy(net_info.http_proxy_host, proxy.GetHost().c_str(), proxy.GetHost().size() + 1);
+    net_info.http_proxy_port = proxy.GetPort();
+
+    if (proxy.GetUser().size() > CONN_USER_LEN) {
+        NCBI_THROW(CHttpSessionException, eConnFailed,
+            "Proxy user length exceeds " NCBI_AS_STRING(CONN_USER_LEN));
+    }
+    memcpy(net_info.http_proxy_user, proxy.GetUser().c_str(), proxy.GetUser().size() + 1);
+
+    if (proxy.GetPassword().size() > CONN_PASS_LEN) {
+        NCBI_THROW(CHttpSessionException, eConnFailed,
+            "Proxy password length exceeds " NCBI_AS_STRING(CONN_PASS_LEN));
+    }
+    memcpy(net_info.http_proxy_pass, proxy.GetPassword().c_str(), proxy.GetPassword().size() + 1);
+}
+
+
+// Interface for the HTTP connector's adjust callback
+struct SAdjustData {
+    CHttpRequest* m_Request;  // NB: don't use after request has been sent!
+    const bool    m_IsService;
+
+    SAdjustData(CHttpRequest* request, bool is_service)
+        : m_Request(request), m_IsService(is_service)
+    { }
+};
+
+
+static void s_Cleanup(void* user_data)
+{
+    SAdjustData* adjust_data = reinterpret_cast<SAdjustData*>(user_data);
+    delete adjust_data;
+}
+
+
 void CHttpRequest::x_InitConnection(bool use_form_data)
 {
-    SConnNetInfo* net_info = ConnNetInfo_Create(
-        m_Url.IsService() ? m_Url.GetService().c_str() : 0);
+    bool is_service = m_Url.IsService();
+    unique_ptr<SConnNetInfo, void (*)(SConnNetInfo*)> net_info
+        (ConnNetInfo_Create(is_service ? m_Url.GetService().c_str() : 0),
+         ConnNetInfo_Destroy);
+    if (!net_info  ||  (is_service  &&  !net_info->svc[0])) {
+        NCBI_THROW(CHttpSessionException, eConnFailed,
+            "Failed to create SConnNetInfo");
+    }
     if (m_Session->GetProtocol() == CHttpSession::eHTTP_11) {
         net_info->http_version = 1;
     }
@@ -798,52 +874,55 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
 
     if ( !m_Timeout.IsDefault() ) {
         STimeout sto;
-        ConnNetInfo_SetTimeout(net_info, g_CTimeoutToSTimeout(m_Timeout, sto));
+        ConnNetInfo_SetTimeout(net_info.get(), g_CTimeoutToSTimeout(m_Timeout, sto));
     }
     if ( !m_Retries.IsNull() ) {
         net_info->max_try = x_RetriesToMaxtry(m_Retries);
     }
+    if ( m_Credentials ) {
+        net_info->credentials = m_Credentials->GetNcbiCred();
+    }
+    x_SetProxy(*net_info);
 
     m_Response.Reset(new CHttpResponse(*m_Session, m_Url));
-    if ( !m_Url.IsService() ) {
+    unique_ptr<SAdjustData> adjust_data(new SAdjustData(this, is_service));
+    if ( !is_service ) {
         // Connect using HTTP.
-        m_IsService = false;
         m_Stream.reset(new CConn_HttpStream(
             m_Url.ComposeUrl(CUrlArgs::eAmp_Char),
-            net_info,
+            net_info.get(),
             headers.c_str(),
             sx_ParseHeader,
-            this,
+            adjust_data.get(),
             sx_Adjust,
-            0, // cleanup
+            s_Cleanup,
             // Always set AdjustOnRedirect flag - to send correct cookies.
             m_Session->GetHttpFlags() | fHTTP_AdjustOnRedirect));
     }
     else {
         // Try to resolve service name.
-        m_IsService = true;
         SSERVICE_Extra x_extra;
         memset(&x_extra, 0, sizeof(x_extra));
-        x_extra.data = this;
+        x_extra.data = adjust_data.get();
         x_extra.adjust = sx_Adjust;
+        x_extra.cleanup = s_Cleanup;
         x_extra.parse_header = sx_ParseHeader;
         x_extra.flags = m_Session->GetHttpFlags() | fHTTP_AdjustOnRedirect;
-        ConnNetInfo_SetUserHeader(net_info, headers.c_str());
+        ConnNetInfo_OverrideUserHeader(net_info.get(), headers.c_str());
         m_Stream.reset(new CConn_ServiceStream(
-            m_Url.GetService(), // Ignore other fields now, set them in sx_Adjust
+            m_Url.GetService(), // Ignore other fields for now, set them in sx_Adjust (called with failure_count == -1 on open)
             fSERV_Http,
-            net_info,
+            net_info.get(),
             &x_extra));
     }
+    adjust_data.release();
     m_Response->m_Stream = m_Stream;
-    ConnNetInfo_Destroy(net_info);
 }
 
 
-void CHttpRequest::x_InitConnection2(shared_ptr<iostream> stream, bool is_service)
+void CHttpRequest::x_InitConnection2(shared_ptr<iostream> stream)
 {
-    m_Stream = move(stream);
-    m_IsService = is_service;
+    m_Stream = std::move(stream);
     m_Response.Reset(new CHttpResponse(*m_Session, m_Url, m_Stream));
 }
 
@@ -867,101 +946,138 @@ void CHttpRequest::x_AddCookieHeader(const CUrl& url, bool initial)
 
 
 // CConn_HttpStream callback for header parsing.
-// user_data must contain CHttpRequest*.
+// user_data must contain SAdjustData*.
+// See the explanation for callback sequence in sx_Adjust.
 EHTTP_HeaderParse CHttpRequest::sx_ParseHeader(const char* http_header,
                                                void*       user_data,
                                                int         server_error)
 {
-    if ( !user_data ) return eHTTP_HeaderContinue;
-    CHttpRequest* req = reinterpret_cast<CHttpRequest*>(user_data);
-    _ASSERT(req);
+    if ( !user_data ) return eHTTP_HeaderError;
+
+    SAdjustData* adj = reinterpret_cast<SAdjustData*>(user_data);
+
+    CHttpRequest*       req  = adj->m_Request;
     CRef<CHttpResponse> resp = req->m_Response;
-    // Response can be NULL, e.g. if an exception was thrown while
-    // initializing the request.
-    if ( resp ) {
-        // Prevent collecting multiple headers on redirects.
-        CHttpHeaders::THeaders headers;
-        s_ParseHttpHeader(http_header, headers);
-        
-        // Parse status code/text.
-        const char* eol = strstr(http_header, HTTP_EOL);
-        string status = eol ? string(http_header, eol - http_header) : http_header;
-        int status_code = 0;
-        string status_text;
-        if ( NStr::StartsWith(status, "HTTP/") ) {
-            int text_pos = 0;
-            sscanf(status.c_str(), "%*s %d %n", &status_code, &text_pos);
-            if (text_pos > 0) {
-                status_text = status.substr(text_pos);
-            }
-        }
+
+    CConn_HttpStream_Base* http
+        = dynamic_cast<CConn_HttpStream_Base*>(req->m_Stream.get());
+    _ASSERT(http);
+
+    CHttpHeaders::THeaders headers;
+    _ASSERT(http_header == http->GetHTTPHeader());
+    s_ParseHttpHeader(http->GetHTTPHeader(), headers);
+
+    // Capture status code/text.
+    resp->x_Update(headers, http->GetStatusCode(), http->GetStatusText());
 
-        resp->x_Update(headers, status_code, status_text);
-    }
     // Always read response body - normal content or error.
     return eHTTP_HeaderContinue;
 }
 
 
 // CConn_HttpStream callback for handling retries and redirects.
-// user_data must contain CHttpRequest*.
+// Reset and re-fill headers on redirects (failure_count == 0).
+// user_data must contain SAdjustData*.
+//
+// For HTTP streams, the callbacks are coming in the following order:
+//   1. *while* establishing a connection with the specified HTTP server (or
+//      through the chain of redirected-to server(s), therein) sx_ParseHeader
+//      gets called for every HTTP response received from the tried HTTP
+//      server(s);  and sx_Adjust gets called for every redirect (with
+//      failure_count == 0) or HTTP request failure (failure_count contains a
+//      positive connection attempt number);
+//   2. *after* the entire HTTP response has been read out, sx_Adjust is called
+//      again with failure_count == -1 to request the next URL.
+//      NOTE that by this time CHttpRequest* may no longer be valid.
+// For service streams:
+//   1. *before* establishing HTTP connection, sx_Adjust is called with
+//      failure_count == -1 to request SConnNetInfo's setup for the 1st found
+//      HTTP server;  then sx_ParseHeader and sx_Adjust are called the same way
+//      as for the HTTP streams above (in 1.) in the process of establishing
+//      the HTTP session;  however, if the negotiation fails, sx_Adjust may be
+//      called for the next HTTP server for the service (w/failure_count == -1)
+//      etc -- this process repeats until a satisfactory conneciton is made;
+//   2. once the HTTP data exchange has occurred, sx_Adjust is NOT called at
+//      the end of the HTTP data stream.
+// Note that adj->m_Request can only be considered valid at either of the steps
+// number 1 above because those actions get performed in the valid CHttpRequest
+// context, namely from CHttpRequest::Execute().  Once that call is finished,
+// CHttpRequest may no longer be accessible.
+//
 int/*bool*/ CHttpRequest::sx_Adjust(SConnNetInfo* net_info,
                                     void*         user_data,
                                     unsigned int  failure_count)
 {
-    if ( !user_data )
-        return  1;
-    // Reset and re-fill headers on redirects (failure_count == 0).
-    CHttpRequest* req = reinterpret_cast<CHttpRequest*>(user_data);
-    _ASSERT(req);
-    if (failure_count == (unsigned int)(-1)  &&  !req->m_IsService)
+    if ( !user_data ) return 0; // error, stop
+
+    SAdjustData* adj = reinterpret_cast<SAdjustData*>(user_data);
+    if (failure_count == (unsigned int)(-1)  &&  !adj->m_IsService)
         return -1; // no new URL
+
+    CHttpRequest*       req  = adj->m_Request;
     CRef<CHttpResponse> resp = req->m_Response;
-    _ASSERT(resp);
 
-    // On the following errors do not retry, abort the request.
-    switch ( resp->GetStatusCode() ) {
-    case 400:
-    case 403:
-    case 404:
-    case 405:
-    case 406:
-    case 410:
-        return 0; // false
-    default:
-        break;
+    if (failure_count  &&  failure_count != (unsigned int)(-1)) {
+        // On the following errors do not retry, abort the request.
+        switch ( resp->GetStatusCode() ) {
+        case 400:
+        case 403:
+        case 404:
+        case 405:
+        case 406:
+        case 410:
+            return 0; // stop
+        default:
+            break;
+        }
+        if (!adj->m_IsService)
+            return 1; // ok to retry
     }
+    _ASSERT(!failure_count/*redirect*/  ||  adj->m_IsService);
 
-    // Update location if it's different from the original url.
-    char* loc = ConnNetInfo_URL(net_info);
+    // Update location if it's different from the original URL.
+    auto loc = make_c_unique(ConnNetInfo_URL(net_info));
     if (loc) {
-        CUrl url(loc);
-        if (failure_count == (unsigned int)(-1)  &&  req->m_IsService) {
+        CUrl url(loc.get());
+        if (failure_count) {
+            _ASSERT(adj->m_IsService);
             bool adjust;
-            if (req->m_AdjustUrl) {
+            if (req->m_AdjustUrl)
                 adjust = req->m_AdjustUrl->AdjustUrl(url);
-            }
             else {
-                url.Adjust(req->m_Url, CUrl::fScheme_Replace |
-                                       CUrl::fPath_Append    |
-                                       CUrl::fArgs_Merge);
+                url.Adjust(req->m_Url,
+                           CUrl::fScheme_Replace |
+                           CUrl::fPath_Append    |
+                           CUrl::fArgs_Merge);
                 adjust = true;
             }
             if ( adjust ) {
-                ConnNetInfo_ParseURL(net_info, url.ComposeUrl(CUrlArgs::eAmp_Char).c_str());
-                // Re-read the url and save it in the response.
-                free(loc);
-                loc = ConnNetInfo_URL(net_info);
+                // Re-read the url and save that in the response.
+                string new_url = url.ComposeUrl(CUrlArgs::eAmp_Char);
+                if (!ConnNetInfo_ParseURL(net_info, new_url.c_str())) {
+                    NCBI_THROW(CHttpSessionException, eConnFailed,
+                        "Cannot parse URL " + new_url);
+                }
+                if (!(loc = make_c_unique(ConnNetInfo_URL(net_info)))) {
+                    NCBI_THROW(CHttpSessionException, eConnFailed,
+                        "Cannot obtain updated URL");
+                }
             }
         }
-        resp->m_Location.SetUrl(loc);
-        free(loc);
+        resp->m_Location.SetUrl(loc.get());
+    } else {
+        NCBI_THROW(CHttpSessionException, eConnFailed,
+            "Cannot obtain original URL");
     }
+
     // Discard old cookies, add those for the new location.
     req->x_AddCookieHeader(resp->m_Location, false);
     string headers = req->m_Headers->GetHttpHeader();
-    ConnNetInfo_OverrideUserHeader(net_info, headers.c_str());
-    return 1; // true
+    if (!ConnNetInfo_OverrideUserHeader(net_info, headers.c_str())) {
+        NCBI_THROW(CHttpSessionException, eConnFailed,
+            "Cannot set HTTP header(s)");
+    }
+    return 1; // proceed
 }
 
 
@@ -994,21 +1110,36 @@ CHttpRequest& CHttpRequest::SetRetryProcessing(ESwitch on_off)
 }
 
 
+void CHttpRequest::SetParam(const CHttpParam& param)
+{
+    m_Timeout = param.GetTimeout();
+    m_Retries = param.GetRetries();
+    m_Proxy = param.GetProxy();
+    m_Deadline = param.GetDeadline();
+    m_RetryProcessing = param.GetRetryProcessing();
+    Headers().Merge(param.GetHeaders());
+    if (param.GetHeaders().HasValue(CHttpHeaders::eContentType)) {
+        Headers().SetValue(CHttpHeaders::eContentType,
+            param.GetHeaders().GetValue(CHttpHeaders::eContentType));
+    }
+}
+
+
 ///////////////////////////////////////////////////////
 //  CHttpSession_Base::
 //
 
 
-CHttpSession_Base::CHttpSession_Base(void)
-    : m_Protocol(eHTTP_10),
+CHttpSession_Base::CHttpSession_Base(EProtocol protocol)
+    : m_Protocol(protocol),
       m_HttpFlags(0)
 {
 }
 
 
-CHttpRequest CHttpSession_Base::NewRequest(const CUrl& url, ERequestMethod method)
+CHttpRequest CHttpSession_Base::NewRequest(const CUrl& url, ERequestMethod method, const CHttpParam& param)
 {
-    return CHttpRequest(*this, url, EReqMethod(method));
+    return CHttpRequest(*this, url, EReqMethod(method), param);
 }
 
 
@@ -1068,7 +1199,7 @@ DEFINE_STATIC_FAST_MUTEX(s_SessionMutex);
 
 
 void CHttpSession_Base::x_SetCookies(const CHttpHeaders::THeaderValues& cookies,
-                                const CUrl*                        url)
+                                     const CUrl*                        url)
 {
     CFastMutexGuard lock(s_SessionMutex);
     ITERATE(CHttpHeaders::THeaderValues, it, cookies) {
@@ -1092,6 +1223,76 @@ string CHttpSession_Base::x_GetCookies(const CUrl& url) const
 }
 
 
+void CHttpSession_Base::SetCredentials(shared_ptr<CTlsCertCredentials> cred)
+{
+    if (m_Credentials) {
+        NCBI_THROW(CHttpSessionException, eOther,
+            "Session credentials already set");
+    }
+    m_Credentials = cred;
+}
+
+
+CHttpParam::CHttpParam(void)
+    : m_Headers(new CHttpHeaders),
+      m_Timeout(CTimeout::eDefault),
+      m_Deadline(CTimeout::eDefault),
+      m_RetryProcessing(ESwitch::eDefault)
+{
+}
+
+
+CHttpParam& CHttpParam::SetHeaders(const CHttpHeaders& headers)
+{
+    m_Headers->Assign(headers);
+    return *this;
+}
+
+
+CHttpParam& CHttpParam::SetHeader(CHttpHeaders::EHeaderName header, CTempString value)
+{
+    m_Headers->SetValue(header, value);
+    return *this;
+}
+
+
+CHttpParam& CHttpParam::AddHeader(CHttpHeaders::EHeaderName header, CTempString value)
+{
+    m_Headers->AddValue(header, value);
+    return *this;
+}
+
+
+CHttpParam& CHttpParam::SetTimeout(const CTimeout& timeout)
+{
+    m_Timeout = timeout;
+    return *this;
+}
+
+
+CHttpParam& CHttpParam::SetRetries(THttpRetries retries)
+{
+    m_Retries = retries;
+    return *this;
+}
+
+
+CHttpParam& CHttpParam::SetCredentials(shared_ptr<CTlsCertCredentials> credentials)
+{
+    m_Credentials = credentials;
+    return *this;
+}
+
+
+CHttpResponse g_HttpGet(const CUrl& url, const CHttpParam& param)
+{
+    CRef<CHttpSession> session(new CHttpSession);
+    session->SetCredentials(param.GetCredentials());
+    CHttpRequest req = session->NewRequest(url, CHttpSession::eGet, param);
+    return req.Execute();
+}
+
+
 CHttpResponse g_HttpGet(const CUrl&     url,
                         const CTimeout& timeout,
                         THttpRetries    retries)
@@ -1115,6 +1316,27 @@ CHttpResponse g_HttpGet(const CUrl&         url,
 }
 
 
+CHttpResponse g_HttpPost(const CUrl& url,
+    CTempString       data,
+    const CHttpParam& param)
+{
+    CRef<CHttpSession> session(new CHttpSession);
+    session->SetCredentials(param.GetCredentials());
+    CHttpRequest req = session->NewRequest(url, CHttpSession::ePost, param);
+
+    if (!param.GetHeaders().HasValue(CHttpHeaders::eContentType)) {
+        req.Headers().SetValue(CHttpHeaders::eContentType,
+            kContentType_FormUrlEnc);
+    }
+
+    if (!data.empty()) {
+        req.ContentStream() << data;
+    }
+
+    return req.Execute();
+}
+
+
 CHttpResponse g_HttpPost(const CUrl&     url,
                          CTempString     data,
                          CTempString     content_type,
@@ -1161,6 +1383,27 @@ CHttpResponse g_HttpPost(const CUrl&         url,
 }
 
 
+CHttpResponse g_HttpPut(const CUrl&       url,
+                        CTempString       data,
+                        const CHttpParam& param)
+{
+    CRef<CHttpSession> session(new CHttpSession);
+    session->SetCredentials(param.GetCredentials());
+    CHttpRequest req = session->NewRequest(url, CHttpSession::ePut, param);
+
+    if (!param.GetHeaders().HasValue(CHttpHeaders::eContentType)) {
+        req.Headers().SetValue(CHttpHeaders::eContentType,
+            kContentType_FormUrlEnc);
+    }
+
+    if (!data.empty()) {
+        req.ContentStream() << data;
+    }
+
+    return req.Execute();
+}
+
+
 CHttpResponse g_HttpPut(const CUrl&     url,
                         CTempString     data,
                         CTempString     content_type,
@@ -1215,6 +1458,7 @@ CHttpResponse g_HttpPut(const CUrl&         url,
 const char* CHttpSessionException::GetErrCodeString(void) const
 {
     switch (GetErrCode()) {
+    case eConnFailed:       return "Connection failed";
     case eBadRequest:       return "Bad request";
     case eBadContentType:   return "Bad Content-Type";
     case eBadFormDataName:  return "Bad form data name";
