@@ -1,4 +1,4 @@
-/* $Id: ncbi_ipv6.c 598940 2019-12-17 16:10:55Z lavr $
+/* $Id: ncbi_ipv6.c 689712 2024-11-05 00:54:28Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -181,8 +181,8 @@ extern int/*bool*/ NcbiIPv4ToIPv6(TNCBI_IPv6Addr* addr,
 }
 
 
-/* Parse "str" as an IP address, and return 0 if failed, otherwise a pointer to
- * the first non-parsed char (which is neither a digit nor a dot) and "dst"
+/* Parse "str" as an IPv4 address, and return 0 if failed, otherwise a pointer
+ * to the first non-parsed char (which is neither a digit nor a dot) and "dst"
  * updated with the just read IPv4 address in network byte order.
  */
 static const char* x_StringToIPv4(unsigned int* dst,
@@ -623,6 +623,7 @@ static const char* s_StringToAddr(TNCBI_IPv6Addr* addr,
     const char* tmp;
     size_t n;
 
+    assert(how);
     if (!addr)
         return 0/*failure*/;
     memset(addr, 0, sizeof(*addr));
@@ -716,13 +717,13 @@ extern int/*bool*/ NcbiIsInIPv6Network(const TNCBI_IPv6Addr* base,
 
     for (n = 0;  n < sizeof(addr->octet);  ++n) {
         unsigned char mask;
-        if (!bits)
+        if (!bits) {
             mask  =                  0;
-        else if (8 > bits) {
-            mask  = (unsigned char)(~0 << (8 - bits));
+        } else if (8 > bits) {
+            mask  = (unsigned char)(~0U << (8 - bits));
             bits  =                  0;
         } else {
-            mask  = (unsigned char)(~0);
+            mask  = (unsigned char)(~0U);
             bits -=                  8;
         }
         if ((addr->octet[n] & mask) ^ base->octet[n])
@@ -740,10 +741,10 @@ extern int/*bool*/ NcbiIPv6Subnet(TNCBI_IPv6Addr* addr, unsigned int bits)
     if (addr) {
         size_t n;
         for (n = 0;  n < sizeof(addr->octet);  ++n) {
-            if (!bits)
+            if (!bits) {
                 addr->octet[n] = 0;
-            else if (8 > bits) {
-                unsigned char mask = (unsigned char)(~0 << (8 - bits));
+            } else if (8 > bits) {
+                unsigned char mask = (unsigned char)(~0U << (8 - bits));
                 if (addr->octet[n] &= mask)
                     zero = 0/*false*/;
                 bits  = 0;
@@ -757,3 +758,33 @@ extern int/*bool*/ NcbiIPv6Subnet(TNCBI_IPv6Addr* addr, unsigned int bits)
 
     return !zero;
 }
+
+
+extern int/*bool*/ NcbiIPv6Suffix(TNCBI_IPv6Addr* addr, unsigned int bits)
+{
+    int/*bool*/ zero = 1/*true*/;
+
+    if (addr) {
+        size_t n;
+        if (bits < sizeof(addr->octet) * 8)
+            bits = sizeof(addr->octet) * 8 - bits;
+        else
+            bits = 0;
+        for (n = 0;  n < sizeof(addr->octet);  ++n) {
+            if (!bits) {
+                if (addr->octet[n])
+                    zero = 0/*false*/;
+            } else if (8 > bits) {
+                unsigned char mask = (unsigned char)(~0U << (8 - bits));
+                if (addr->octet[n] &= ~mask)
+                    zero = 0/*false*/;
+                bits  = 0;
+            } else {
+                bits -= 8;
+                addr->octet[n] = 0;
+            }
+        }
+    }
+
+    return !zero;
+}
