@@ -1,4 +1,4 @@
-/* $Id: ncbi_linkerd.c 660664 2022-12-21 19:07:13Z lavr $
+/* $Id: ncbi_linkerd.c 689714 2024-11-05 04:57:39Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,10 +43,10 @@
 #include <connect/ncbi_http_connector.h>
 
 #include <ctype.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <time.h>
 
-
 #define NCBI_USE_ERRCODE_X   Connect_NamerdLinkerd
 
 
@@ -74,9 +74,12 @@ enum ELINKERD_Subcodes {
 /* LINKERD_TODO - "temporarily" support plain "linkerd" on Unix only */
 #if defined(NCBI_OS_UNIX)  &&  !defined(NCBI_OS_CYGWIN)
 #  define DEF_LINKERD_HOST       "linkerd"
-#else
-#  define DEF_LINKERD_HOST       \
+#elif 1/*defined(_DEBUG)  &&  !defined(NDEBUG)*/
+#  define DEF_LINKERD_HOST                                              \
     "pool.linkerd-proxy.service.bethesda-dev.consul.ncbi.nlm.nih.gov"
+#else
+#  define DEF_LINKERD_HOST                                              \
+    "pool.linkerd-proxy.service.bethesda-prod.consul.ncbi.nlm.nih.gov"
 #endif
 
 #define REG_LINKERD_PORT         REG_CONN_PORT
@@ -123,8 +126,8 @@ static int/*bool*/ s_Resolve(SERV_ITER iter)
     struct SLINKERD_Data* data     = (struct SLINKERD_Data*) iter->data;
     SConnNetInfo*         net_info = data->net_info;
     char                  hostport[80/*IP:port*/];
+    ESERV_Type            xtype, atype;
     char*                 infostr;
-    ESERV_Type            atype;
     const char*           type;
     unsigned int          host;
     size_t                size;
@@ -147,26 +150,26 @@ static int/*bool*/ s_Resolve(SERV_ITER iter)
     switch (net_info->req_method) {
     case eReqMethod_Any:
     case eReqMethod_Any11:
-        atype = fSERV_Http;
+        xtype = fSERV_Http;
         break;
     case eReqMethod_Get:
     case eReqMethod_Get11:
-        atype = fSERV_HttpGet;
+        xtype = fSERV_HttpGet;
         break;
     case eReqMethod_Post:
     case eReqMethod_Post11:
-        atype = fSERV_HttpPost;
+        xtype = fSERV_HttpPost;
         break;
     default:
-        atype = SERV_GetImplicitServerTypeInternal(iter->name);
+        xtype = SERV_GetImplicitServerTypeInternal(iter->name);
         break;
     }
     /* make sure the server type matches */
-    if ((data->types  &&  !(atype = (ESERV_Type)(atype & data->types)))
+    if (!(atype = data->types ? (ESERV_Type)(xtype & data->types) : xtype)
         ||  !*(type = SERV_TypeStr(atype))) {
         CORE_LOGF_X(eLSub_BadData, eLOG_Error,
-                    ("[%s]  Cannot match request method (%d)"
-                     " with a server type", iter->name, net_info->req_method));
+                    ("[%s]  Cannot match request method (%d) to server type"
+                     " (%d)", iter->name, net_info->req_method, xtype));
         return 0/*failure*/;
     }
 
@@ -196,9 +199,9 @@ static int/*bool*/ s_Resolve(SERV_ITER iter)
     size += sizeof(kDescrFmt) + sizeof(LINKERD_VHOST_DOMAIN) + 40/*L,R,T,$*/
         + strlen(type) + strlen(net_info->path) + strlen(hostport);
     if ( ! (infostr = (char*) malloc(size))) {
-        CORE_LOGF_X(eLSub_Alloc, eLOG_Critical,
-                    ("[%s]  Failed to allocate for server descriptor",
-                     iter->name));
+        CORE_LOGF_ERRNO_X(eLSub_Alloc, eLOG_Critical, errno,
+                          ("[%s]  Failed to allocate for server descriptor",
+                           iter->name));
         return 0/*failure*/;
     }
     verify((size_t)
@@ -414,28 +417,28 @@ static int/*bool*/ x_SetupConnectionParams(SERV_ITER iter, int* do_namerd)
               (*do_namerd < 0  &&  !(*do_namerd
                                      = SERV_IsMapperConfiguredInternal
                                      (iter->name, REG_CONN_NAMERD_ENABLE))))) {
-            namerd  = x_SetupFromNamerd(iter, do_namerd);
+            namerd = x_SetupFromNamerd(iter, do_namerd);
             if (!namerd)
                 return 0/*failed*/;
         } else
-            namerd  = 0;
-        if (namerd <= 0) {
-            if (!namerd  &&  iter->arglen) {
-                assert(iter->arg);
-                CORE_LOGF_X(eLSub_BadData, eLOG_Warning,
-                            ("[%s]  LINKERD does not support argument affinity"
-                             ": %s%s%s%s%s, use at your own risk!", iter->name,
-                             iter->arg, &"="[!iter->val], &"\""[!iter->val],
-                             iter->val ? iter->val : "",  &"\""[!iter->val]));
-            }
-            net_info->scheme = eURL_Http;
             namerd = 0;
-        } else
-            assert(net_info->scheme);
     } else
-        namerd = 0;
+        namerd  = 0;
+    if (namerd <= 0) {
+        if ((!namerd  ||  !*do_namerd)  &&  iter->arglen) {
+            assert(iter->arg);
+            CORE_LOGF_X(eLSub_BadData, eLOG_Warning,
+                        ("[%s]  LINKERD does not support argument affinity"
+                         ": %s%s%s%s%s, use at your own risk!", iter->name,
+                         iter->arg, &"="[!iter->val], &"\""[!iter->val],
+                         iter->val ? iter->val : "",  &"\""[!iter->val]));
+        }
+        net_info->scheme = eURL_Http;
+        namerd  = 0;
+    } else
+        assert(net_info->scheme);
 
-    /* N.B. Proxy configuration (including 'http_proxy' env. var. detected and
+    /* N.B. Proxy configuration (including $http_proxy environment detected and
        parsed by the toolkit) may be used to override the default host:port for
        Linkerd.  But connections via Linkerd should be made directly to the
        Linkerd's host:port as the authority part of the URL, not by using the
@@ -443,7 +446,7 @@ static int/*bool*/ x_SetupConnectionParams(SERV_ITER iter, int* do_namerd)
        'net_info->http_proxy_host' (and same for port). */
     if (!net_info->http_proxy_host[0]  ||
         !net_info->http_proxy_port     ||
-        !net_info->http_proxy_only) {
+        net_info->http_proxy_mask != fProxy_Http) {
         int n;
         if ( ! ConnNetInfo_GetValueService(DEF_LINKERD_REG_SECTION,
                                            REG_LINKERD_HOST,
@@ -513,7 +516,6 @@ static int/*bool*/ x_SetupConnectionParams(SERV_ITER iter, int* do_namerd)
             return 0/*failed*/;
         }
     }
-
     return 1/*succeeded*/;
 }
 
@@ -559,16 +561,17 @@ extern const SSERV_VTable* SERV_LINKERD_Open(SERV_ITER           iter,
     }
 
     if ( ! (data = (struct SLINKERD_Data*) calloc(1, sizeof(*data)))) {
-        CORE_LOGF_X(eLSub_Alloc, eLOG_Critical,
-                    ("[%s]  Failed to allocate for SLINKERD_Data",iter->name));
+        CORE_LOGF_ERRNO_X(eLSub_Alloc, eLOG_Critical, errno,
+                          ("[%s]  Failed to allocate for SLINKERD_Data",
+                           iter->name));
         return 0;
     }
     iter->data = data;
     data->types = types;
 
     if ( ! (data->net_info = ConnNetInfo_Clone(net_info))) {
-        CORE_LOGF_X(eLSub_Alloc, eLOG_Critical,
-                    ("[%s]  Failed to clone net_info", iter->name));
+        CORE_LOGF_ERRNO_X(eLSub_Alloc, eLOG_Critical, errno,
+                          ("[%s]  Failed to clone net_info", iter->name));
         s_Close(iter);
         return 0;
     }
