@@ -1,4 +1,4 @@
-/* $Id: ncbi_localip.c 611694 2020-07-09 16:35:41Z lavr $
+/* $Id: ncbi_localip.c 689714 2024-11-05 04:57:39Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -57,6 +57,7 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_LocalIP
 
+
 #if defined(NCBI_OS_UNIX)
 #  define NcbiSys_access    access
 #elif defined(NCBI_OS_MSWIN)
@@ -97,10 +98,32 @@ static int/*bool*/ s_Inited = 0/*false*/;
 static SIPRange s_LocalIP[256 + 1] = { { eIPRange_None } };
 
 
-#ifdef HAVE_LIBCONNEXT
-
-
-#  include "ext/ncbi_localip.c"
+static const struct {
+    EIPRangeType t;
+    unsigned int a;
+    unsigned int b;
+} kLocalIP[] = {
+    /* localnet (localhost):  127/8 */
+#if defined(IN_CLASSA) && defined(IN_CLASSA_NET) && defined(IN_CLASSA_NSHIFT)
+#  if IN_CLASSA_MAX <= IN_LOOPBACKNET
+#    error "IN_LOOPBACKNET is out of range"
+#  endif /*IN_CLASSA_MAX<=IN_LOOPBACKNET*/
+    { eIPRange_Network, IN_LOOPBACKNET << IN_CLASSA_NSHIFT, IN_CLASSA_NET },
+#else
+    { eIPRange_Network, INADDR_LOOPBACK - 1,                0xFF000000    },
+#endif /*IN_CLASSA && IN_CLASSA_NET && IN_CLASSA_NSHIFT*/
+    /* from assigned IP ranges */
+    { eIPRange_Range,   0x820E0800, 0x820E09FF }, /* 130.14.{8|9}.0/24       */
+    { eIPRange_Range,   0x820E0B00, 0x820E0CFF }, /* 130.14.1{1|2}.0/24      */
+    { eIPRange_Range,   0x820E1400, 0x820E1AFF }, /* 130.14.20.0..27.255     */
+    { eIPRange_Range,   0x820E1B40, 0x820E1BFF }, /*  w/o 130.14.27.0/26     */
+    { eIPRange_Network, 0x820E1D00, 0xFFFFFF00 }, /* 130.14.29.yyy           */
+    { eIPRange_Network, 0xA5700700, 0xFFFFFF00 }, /* 165.112.7.zzz (colo)    */
+    /* from private IP ranges */
+    { eIPRange_Network, 0x0A0A0000, 0xFFFF0000 }, /* 10.10/16 from cl.A      */
+    { eIPRange_Network, 0x0A140000, 0xFFFF0000 }, /* 10.20/16 from cl.A      */
+    { eIPRange_None }
+};
 
 
 static const SIPRange* x_IsOverlappingRange(const SIPRange* start,
@@ -219,11 +242,12 @@ static int/*bool*/ xx_LoadLocalIPs(CONN conn, const char* source)
         if (n < SizeOf(s_LocalIP))
             s_LocalIP[n].type = eIPRange_None;
         n -= 2;  /* compensate for auto-added localnet(s) */
-        CORE_LOGF(eLOG_Trace, ("%s: Done loading local IP specs, %u line%s,"
-                               " %u entr%s (%u domain%s)", source,
-                               lineno, &"s"[lineno == 1],
-                               (unsigned int) n, n == 1 ? "y" : "ies",
-                               domain, &"s"[domain == 1]));
+        CORE_LOGF(eLOG_Trace,
+                  ("%s: Done loading local IP specs, %u line%s,"
+                   " %u entr%s (%u domain%s)", source,
+                   lineno, &"s"[lineno == 1],
+                   (unsigned int) n, n == 1 ? "y" : "ies",
+                   domain, &"s"[domain == 1]));
         return 1/*true*/;
     }
     return 0/*false*/;
@@ -232,15 +256,17 @@ static int/*bool*/ xx_LoadLocalIPs(CONN conn, const char* source)
 
 static int/*bool*/ x_LoadLocalIPs(CONNECTOR c, const char* source)
 {
-    CONN conn;
     int/*bool*/ loaded = 0/*false*/;
-    CORE_LOGF(eLOG_Trace,
-              ("Loading local IP specs from \"%s\"", source));
-    if (c  &&  CONN_Create(c, &conn) == eIO_Success) {
-        loaded = xx_LoadLocalIPs(conn, source);
-        CONN_Close(conn);
-    } else if (c  &&  c->destroy)
-        c->destroy(c);
+    if (c) {
+        CONN conn;
+        CORE_LOGF(eLOG_Trace,
+                  ("Loading local IP specs from \"%s\"", source));
+        if (CONN_Create(c, &conn) == eIO_Success) {
+            loaded = xx_LoadLocalIPs(conn, source);
+            CONN_Close(conn);
+        } else if (c->destroy)
+            c->destroy(c);
+    }
     return loaded;
 }
 
@@ -261,11 +287,11 @@ static void s_LoadLocalIPs(void)
         for (n = 0;  n < SizeOf(kFile);  ++n) {
             if (n  ||  !*file)
                 file = kFile[n];
-            else if (strcasecmp(file, REG_CONN_LOCAL_IPS_DISABLE) == 0)
+            else if (strcasecmp(file, DEF_CONN_LOCAL_IPS_DISABLE) == 0)
                 break;
             errno = 0;
-            if (NcbiSys_access(file, R_OK) == 0  &&
-                x_LoadLocalIPs(FILE_CreateConnector(file, 0), file)) {
+            if (NcbiSys_access(file, R_OK) == 0
+                &&  x_LoadLocalIPs(FILE_CreateConnector(file, 0), file)) {
                 return;
             }
             if (errno == ENOENT  &&  file != buf) {
@@ -281,23 +307,25 @@ static void s_LoadLocalIPs(void)
             if (file == buf)
                 break;
         }
-        net_info = ConnNetInfo_Create(REG_CONN_LOCAL_IPS);
-        if (net_info)
-            n = strcasecmp(net_info->svc, REG_CONN_LOCAL_IPS_DISABLE) ? 1 : 0;
-        else
+        net_info = ConnNetInfo_Create(DEF_CONN_LOCAL_IPS);
+        if (!net_info)
             n = 0;
+        else if (strcasecmp(net_info->svc, DEF_CONN_LOCAL_IPS) == 0)
+            n = 1;
+        else
+            n = strcasecmp(net_info->svc, DEF_CONN_LOCAL_IPS_DISABLE) ? 2 : 0;
         /* Build a pass-thru HTTP connector here to save on a dispatcher hit */
-        if (n  &&  ConnNetInfo_SetupStandardArgs(net_info, net_info->svc)  &&
-            x_LoadLocalIPs(HTTP_CreateConnector(net_info,
-                                                "User-Agent: ncbi_localip",
-                                                fHTTP_NoAutoRetry |
-                                                fHTTP_SuppressMessages),
-                           net_info->svc)) {
+        if (n  &&  ConnNetInfo_SetupStandardArgs(net_info, net_info->svc)
+            &&  x_LoadLocalIPs(HTTP_CreateConnector(net_info,
+                                                    "User-Agent: ncbi_localip",
+                                                    fHTTP_NoAutoRetry |
+                                                    fHTTP_SuppressMessages),
+                               net_info->svc)) {
             ConnNetInfo_Destroy(net_info);
             return;
         }
         ConnNetInfo_Destroy(net_info);
-        if (net_info) {
+        if (net_info  &&  n < 2) {
 #  ifdef NCBI_OS_LINUX
             level = n ? eLOG_Trace   : eLOG_Note;
 #  else
@@ -309,7 +337,7 @@ static void s_LoadLocalIPs(void)
         level = eLOG_Critical;
     if (level != eLOG_Note) {
         CORE_LOG_X(1, level,
-                   "Cannot load local IP specs from " REG_CONN_LOCAL_IPS);
+                   "Cannot load local IP specs from " DEF_CONN_LOCAL_IPS);
     }
 
     CORE_LOG(eLOG_Warning, "Using default local IPv4 specs");
@@ -324,15 +352,6 @@ static void s_LoadLocalIPs(void)
 }
 
 
-#else
-
-
-#  define s_LoadLocalIPs()  /*void*/
-
-
-#endif /*HAVE_LIBCONNEXT*/
-
-
 extern void NcbiInitLocalIP(void)
 {
     /*CORE_LOCK_WRITE;*/
