@@ -1,4 +1,4 @@
-/* $Id: ncbi_mbedtls.c 629075 2021-04-09 14:32:17Z ivanov $
+/* $Id: ncbi_mbedtls.c 688079 2024-09-24 21:39:23Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,29 +35,26 @@
 #include "ncbi_priv.h"
 #include "ncbi_servicep.h"
 #include <connect/ncbi_mbedtls.h>
+#include <connect/ncbi_tls.h>
 #include <stdlib.h>
 
+#define NCBI_USE_ERRCODE_X   Connect_TLS
+
+
 #if defined(HAVE_LIBMBEDTLS)  ||  defined(NCBI_CXX_TOOLKIT)
 
-#  ifdef HAVE_LIBMBEDTLS /* external */
-#    include <mbedtls/ctr_drbg.h>
-#    include <mbedtls/debug.h>
-#    include <mbedtls/entropy.h>
-#    include <mbedtls/error.h>
-#    include <mbedtls/net_sockets.h>
-#    include <mbedtls/ssl.h>
-#    include <mbedtls/threading.h>
-#    include <mbedtls/version.h>
-#  else
-#    include "mbedtls/mbedtls/ctr_drbg.h"
-#    include "mbedtls/mbedtls/debug.h"
-#    include "mbedtls/mbedtls/entropy.h"
-#    include "mbedtls/mbedtls/error.h"
-#    include "mbedtls/mbedtls/net_sockets.h"
-#    include "mbedtls/mbedtls/ssl.h"
-#    include "mbedtls/mbedtls/threading.h"
-#    include "mbedtls/mbedtls/version.h"
-#  endif /*HAVE_LIBMBEDTLS*/
+#  include <mbedtls/ctr_drbg.h>
+#  include <mbedtls/debug.h>
+#  include <mbedtls/entropy.h>
+#  include <mbedtls/error.h>
+#  include <mbedtls/pk.h>
+#  include <mbedtls/net_sockets.h>
+#  include <mbedtls/ssl.h>
+#  include <mbedtls/threading.h>
+#  include <mbedtls/version.h>
+#  ifdef MBEDTLS_PSA_CRYPTO_C
+#    include <psa/crypto.h>
+#  endif /*MBEDTLS_PSA_CRYPTO_C*/
 
 #  if   defined(ENOTSUP)
 #    define NCBI_NOTSUPPORTED  ENOTSUP
@@ -67,6 +64,21 @@
 #    define NCBI_NOTSUPPORTED  EINVAL
 #  endif
 
+#else
+
+#  define mbedtls_x509_crt    void
+#  define mbedtls_pk_context  void
+
+#endif /*HAVE_LIBMBEDTLS || NCBI_CXX_TOOLKIT*/
+
+
+struct SNcbiMbedTlsCred {
+    mbedtls_x509_crt*   cert;
+    mbedtls_pk_context* pkey;
+};
+
+
+#if defined(HAVE_LIBMBEDTLS)  ||  defined(NCBI_CXX_TOOLKIT)
 
 #  if defined(MBEDTLS_THREADING_ALT)  &&  defined(NCBI_THREADS)
 #    ifdef MBEDTLS_THREADING_PTHREAD
@@ -74,22 +86,29 @@
 #    endif /*MBEDTLS_THREADING_PTHREAD*/
 static void mbtls_user_mutex_init(MT_LOCK* lock)
 {
-    if (lock)
-        *lock = MT_LOCK_AddRef(g_CORE_MT_Lock);
+    *lock = MT_LOCK_AddRef(g_CORE_MT_Lock);
 }
 static void mbtls_user_mutex_deinit(MT_LOCK* lock)
 {
-    if (lock) {
-        g_CORE_MT_Lock = MT_LOCK_Delete(*lock);
-        *lock = 0;
-    }
+    if (*lock) {
+        if (!(*lock = MT_LOCK_Delete(*lock))) {
+            /* NB: Do not use CORE_SetLOCK() here! */
+            g_CORE_MT_Lock = 0;
+        } else
+            *lock = 0;
+    } else
+        CORE_LOG_X(20, eLOG_Warning, "NULL MT_LOCK deinit in MBEDTLS");
 }
 static int mbtls_user_mutex_lock(MT_LOCK* lock)
 {
     if (lock) {
-        switch (MT_LOCK_Do((MT_LOCK)(*lock), eMT_Lock)) {
+        switch (MT_LOCK_Do(*lock, eMT_Lock)) {
         case -1:
+#ifdef MBEDTLS_ERR_THREADING_FEATURE_UNAVAILABLE
             return MBEDTLS_ERR_THREADING_FEATURE_UNAVAILABLE;
+#else
+            return MBEDTLS_ERR_PLATFORM_FEATURE_UNSUPPORTED;
+#endif
         case  0:
             return MBEDTLS_ERR_THREADING_MUTEX_ERROR;
         case  1:
@@ -103,9 +122,13 @@ static int mbtls_user_mutex_lock(MT_LOCK* lock)
 static int mbtls_user_mutex_unlock(MT_LOCK* lock)
 {
     if (lock) {
-        switch (MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock)) {
+        switch (MT_LOCK_Do(*lock, eMT_Unlock)) {
         case -1:
+#ifdef MBEDTLS_ERR_THREADING_FEATURE_UNAVAILABLE
             return MBEDTLS_ERR_THREADING_FEATURE_UNAVAILABLE;
+#else
+            return MBEDTLS_ERR_PLATFORM_FEATURE_UNSUPPORTED;
+#endif
         case  0:
             return MBEDTLS_ERR_THREADING_MUTEX_ERROR;
         case  1:
@@ -147,7 +170,7 @@ static int         x_MbedTlsPush  (void*, const unsigned char*, size_t);
 #  endif /*__cplusplus*/
 
 
-static int                      s_MbedTlsLogLevel;
+static volatile int             s_MbedTlsLogLevel;
 static mbedtls_entropy_context  s_MbedTlsEntropy;
 static mbedtls_ctr_drbg_context s_MbedTlsCtrDrbg;
 static mbedtls_ssl_config       s_MbedTlsConf;
@@ -166,7 +189,8 @@ static void x_MbedTlsLogger(void* unused, int level,
         return;
     if (message[len - 1] == '\n')
         --len;
-    CORE_LOGF(eLOG_Note, ("MBEDTLS%d: %.*s", level, (int) len, message));
+    CORE_LOGF_X(1, eLOG_Note,
+                ("MBEDTLS%d: %.*s", level, (int) len, message));
 }
 
 
@@ -184,7 +208,7 @@ static EIO_Status x_RetryStatus(SOCK sock, EIO_Event direction)
             : eIO_Closed;
     } else
         status = SOCK_Status(sock, direction);
-    return status == eIO_Success ? eIO_Timeout : status;
+    return status;
 }
 
 
@@ -201,7 +225,11 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
 
     if (!error)
         return eIO_Success;
+#if MBEDTLS_VERSION_MAJOR >= 3
+    sock = ((SNcbiSSLctx*)(session->MBEDTLS_PRIVATE(p_bio)))->sock;
+#else
     sock = ((SNcbiSSLctx*)(session->p_bio))->sock;
+#endif
     switch (error) {
     case MBEDTLS_ERR_SSL_WANT_READ:
         status = x_RetryStatus(sock, direction);
@@ -212,11 +240,21 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
     case MBEDTLS_ERR_SSL_TIMEOUT:
         status = eIO_Timeout;
         break;
+#ifdef MBEDTLS_ERR_THREADING_FEATURE_UNAVAILABLE
     case MBEDTLS_ERR_THREADING_FEATURE_UNAVAILABLE:
+#endif
+#ifdef MBEDTLS_ERR_SSL_NO_USABLE_CIPHERSUITE
     case MBEDTLS_ERR_SSL_NO_USABLE_CIPHERSUITE:
+#endif
+    case MBEDTLS_ERR_CIPHER_FEATURE_UNAVAILABLE:
     case MBEDTLS_ERR_SSL_FEATURE_UNAVAILABLE:
+#ifdef MBEDTLS_ERR_SSL_UNKNOWN_CIPHER
     case MBEDTLS_ERR_SSL_UNKNOWN_CIPHER:
+#endif
     case MBEDTLS_ERR_SSL_INTERNAL_ERROR:
+#ifdef MBEDTLS_ERR_PLATFORM_FEATURE_UNSUPPORTED
+    case MBEDTLS_ERR_PLATFORM_FEATURE_UNSUPPORTED:
+#endif
         status = eIO_NotSupported;
         break;
     case MBEDTLS_ERR_THREADING_BAD_INPUT_DATA:
@@ -251,9 +289,12 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
         break;
     }
 
-    assert(status != eIO_Success);
-    CORE_TRACEF(("MBEDTLS error %d -> CONNECT MBEDTLS status %s",
-                 error, IO_StatusStr(status)));
+    assert(error == MBEDTLS_ERR_SSL_WANT_READ   ||
+           error == MBEDTLS_ERR_SSL_WANT_WRITE  ||
+           status != eIO_Success);
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MBEDTLS error %d -> CONNECT MBEDTLS status %s",
+                     error, IO_StatusStr(status)));
     return status;
 }
 
@@ -292,8 +333,10 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
 
     {{
         int x_err = error ? error : errno;
-        CORE_TRACEF(("CONNECT MBEDTLS status %s -> %s %d",
-                     IO_StatusStr(status), error ? "error" : "errno", x_err));
+        CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+            CORE_TRACEF(("CONNECT MBEDTLS status %s -> %s %d",
+                         IO_StatusStr(status),
+                         error ? "error" : "errno", x_err));
         if (!error)
             errno = x_err; /* restore errno that might have been clobbered */
     }}
@@ -323,34 +366,40 @@ static void* s_MbedTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
     int end = (side == eSOCK_Client
                ? MBEDTLS_SSL_IS_CLIENT
                : MBEDTLS_SSL_IS_SERVER);
+    struct SNcbiMbedTlsCred* xcred;
     mbedtls_ssl_context* session;
-    void** xcred;
     int err;
 
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACE("MbedTlsCreate(): Enter");
+
     if (end == MBEDTLS_SSL_IS_SERVER) {
-        CORE_LOG(eLOG_Error, "Server-side SSL not yet supported with MBEDTLS");
-        *error = 0;
-        return 0;
+        CORE_LOG_X(2, eLOG_Critical,
+                   "Server-side SSL not yet supported with MBEDTLS");
+        *error = NCBI_NOTSUPPORTED;
+        session = 0;
+        goto out;
     }
 
     if (ctx->cred) {
         if (ctx->cred->type != eNcbiCred_MbedTls  ||  !ctx->cred->data) {
             /*FIXME: there's a NULL(data)-terminated array of credentials */
-            CORE_LOGF(eLOG_Error,
-                      ("%s credentials in MBEDTLS session",
-                       ctx->cred->type != eNcbiCred_MbedTls
-                       ? "Foreign"
-                       : "Empty"));
-            *error = 0;
-            return 0;
+            CORE_LOGF_X(3, eLOG_Error,
+                        ("%s credentials in MBEDTLS session",
+                         ctx->cred->type != eNcbiCred_MbedTls
+                         ? "Foreign"
+                         : "Empty"));
+            *error = EINVAL;
+            session = 0;
+            goto out;
         }
-        xcred = ctx->cred->data;
+        xcred = (struct SNcbiMbedTlsCred*) ctx->cred->data;
     } else
         xcred = 0;
 
     if (!(session = (mbedtls_ssl_context*) malloc(sizeof(*session)))) {
         *error = errno;
-        return 0;
+        goto out;
     }
     mbedtls_ssl_init(session);
 
@@ -359,75 +408,85 @@ static void* s_MbedTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
          &&  (err = mbedtls_ssl_set_hostname(session, ctx->host)) != 0)  ||
         (xcred
          &&  (err = mbedtls_ssl_set_hs_own_cert
-              (session,
-               (mbedtls_x509_crt*)   xcred[0],
-               (mbedtls_pk_context*) xcred[1])) != 0)) {
+              (session, xcred->cert, xcred->pkey))                != 0)) {
         mbedtls_ssl_free(session);
         free(session);
         *error = err;
-        return 0;
+        session = 0;
+        goto out;
     }
 
     mbedtls_ssl_set_bio(session, ctx, x_MbedTlsPush, x_MbedTlsPull, 0);
- 
+
+out:
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsCreate(): Leave(%p)", session));
+
     return session;
 }
 
 
+static char* x_MbedTlsDesc(const mbedtls_ssl_context* session)
+{
+    const char* alpn     = mbedtls_ssl_get_alpn_protocol(session);
+    size_t      alpn_len = alpn ? strlen(alpn) : 0;
+    const char* sslv     = mbedtls_ssl_get_version(session);
+    size_t      sslv_len = sslv ? strlen(sslv) : 0;
+    const char* ciph     = mbedtls_ssl_get_ciphersuite(session);
+    size_t      ciph_len = ciph ? strlen(ciph) : 0;
+    size_t           len = alpn_len + sslv_len + ciph_len;
+    char*       str;
+    if (!len  ||  !(str = (char*) malloc(len + 3/*seps+EOL*/)))
+        return 0;
+    len = 0;
+    if (alpn_len) {
+        memcpy(str, alpn, alpn_len);
+        len = alpn_len;
+    }
+    if (sslv_len) {
+        if (len)
+            str[len++] = '/';
+        memcpy(str + len, sslv, sslv_len);
+        len += sslv_len;
+    }
+    if (ciph_len) {
+        if (len)
+            str[len++] = '/';
+        memcpy(str + len, ciph, ciph_len);
+        len += ciph_len;
+    }
+    str[len] = '\0';
+    return str;
+}
+
+
 static EIO_Status s_MbedTlsOpen(void* session, int* error, char** desc)
 {
     EIO_Status status;
     int x_error;
 
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsOpen(%p): Enter", session));
+
     x_error = mbedtls_ssl_handshake((mbedtls_ssl_context*) session);
 
     if (x_error < 0) {
         status = x_ErrorToStatus(x_error, (mbedtls_ssl_context*) session,
                                  eIO_Open);
+        if (status == eIO_Success)
+            status  = eIO_Unknown;
         *error = x_error;
         if (desc)
             *desc = 0;
     } else {
-        if (desc) {
-            const char* alpn
-                = mbedtls_ssl_get_alpn_protocol((const mbedtls_ssl_context*)
-                                                session);
-            size_t alpn_len = alpn ? strlen(alpn) : 0;
-            const char* sslv 
-                = mbedtls_ssl_get_version      ((const mbedtls_ssl_context*)
-                                                session);
-            size_t sslv_len = sslv ? strlen(sslv) : 0;
-            const char* ciph
-                = mbedtls_ssl_get_ciphersuite  ((const mbedtls_ssl_context*)
-                                                session);
-            size_t ciph_len = ciph ? strlen(ciph) : 0;
-            size_t len = alpn_len + sslv_len + ciph_len;
-            if (!len)
-                *desc = 0;
-            else if ((*desc = (char*) malloc(len + 3/*seps+EOL*/)) != 0) {
-                char* ptr = *desc;
-                if (alpn_len) {
-                    memcpy(ptr, alpn, alpn_len);
-                    ptr += alpn_len;
-                }
-                if (sslv_len) {
-                    if (ptr != *desc)
-                        *ptr++ = '/';
-                    memcpy(ptr, sslv, sslv_len);
-                    ptr += sslv_len;
-                }
-                if (ciph_len) {
-                    if (ptr != *desc)
-                        *ptr++ = '/';
-                    memcpy(ptr, ciph, ciph_len);
-                    ptr += ciph_len;
-                }
-                *ptr = '\0';
-            }
-        }
         status = eIO_Success;
+        if (desc)
+            *desc = x_MbedTlsDesc((mbedtls_ssl_context*) session);
     }
 
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsOpen(%p): Leave(%d)", session, status));
+
     return status;
 }
 
@@ -462,6 +521,34 @@ static int x_MbedTlsPull(void* ctx, unsigned char* buf, size_t size)
 }
 
 
+static EIO_Status s_MbedTlsRead(void* session, void* buf,
+                                size_t n_todo, size_t* n_done, int* error)
+{
+    EIO_Status status;
+    int        x_read;
+
+    assert(session  &&  buf  &&  n_todo > 0);
+
+again:
+    x_read = mbedtls_ssl_read((mbedtls_ssl_context*) session,
+                              (unsigned char*) buf, n_todo);
+    assert(x_read < 0  ||  (size_t) x_read <= n_todo);
+
+    if (x_read <= 0) {
+        status = x_ErrorToStatus(x_read, (mbedtls_ssl_context*) session,
+                                 eIO_Read);
+        if (status == eIO_Success  &&  x_read == MBEDTLS_ERR_SSL_WANT_READ)
+            goto again;
+        *error = x_read;
+        x_read = 0;
+    } else
+        status = eIO_Success;
+
+    *n_done = (size_t) x_read;
+    return status;
+}
+
+
 static int x_MbedTlsPush(void* ctx, const unsigned char* data, size_t size)
 {
     SOCK sock = ((SNcbiSSLctx*) ctx)->sock;
@@ -482,7 +569,7 @@ static int x_MbedTlsPush(void* ctx, const unsigned char* data, size_t size)
                 assert(x_written <= size);
                 n_written += (ssize_t) x_written;
                 size      -=           x_written;
-                data       = data  +   x_written;
+                data      +=           x_written;
             }
         } while (size);
         return (int) n_written;
@@ -494,46 +581,22 @@ static int x_MbedTlsPush(void* ctx, const unsigned char* data, size_t size)
 }
 
 
-static EIO_Status s_MbedTlsRead(void* session, void* buf, size_t n_todo,
-                               size_t* n_done, int* error)
-{
-    EIO_Status status;
-    int        x_read;
-
-    assert(session);
-
-    x_read = mbedtls_ssl_read((mbedtls_ssl_context*) session,
-                              (unsigned char*) buf, n_todo);
-    assert(x_read < 0  ||  (size_t) x_read <= n_todo);
-
-    if (x_read <= 0) {
-        status = x_ErrorToStatus(x_read, (mbedtls_ssl_context*) session,
-                                 eIO_Read);
-        *error = x_read;
-        x_read = 0;
-    } else
-        status = eIO_Success;
-
-    *n_done = (size_t) x_read;
-    return status;
-}
-
-
-static EIO_Status x_MbedTlsWrite(void* session, const void* data,
+static EIO_Status x_MbedTlsWrite(void* session, const unsigned char* data,
                                  size_t n_todo, size_t* n_done, int* error)
 {
     EIO_Status status;
     int        x_written;
 
-    assert(session);
+    assert(data  &&  n_todo > 0);
 
-    x_written = mbedtls_ssl_write((mbedtls_ssl_context*) session,
-                                  (const unsigned char*) data, n_todo);
+    x_written = mbedtls_ssl_write((mbedtls_ssl_context*) session, data, n_todo);
     assert(x_written < 0  ||  (size_t) x_written <= n_todo);
 
     if (x_written <= 0) {
         status = x_ErrorToStatus(x_written, (mbedtls_ssl_context*) session,
                                  eIO_Write);
+        if (status == eIO_Success)
+            status  = eIO_Unknown;
         *error = x_written;
         x_written = 0;
     } else
@@ -544,16 +607,19 @@ static EIO_Status x_MbedTlsWrite(void* session, const void* data,
 }
 
 
-static EIO_Status s_MbedTlsWrite(void* session, const void* data,
+static EIO_Status s_MbedTlsWrite(void* session, const void* x_data,
                                  size_t n_todo, size_t* n_done, int* error)
 {
-    size_t max_size
-        = mbedtls_ssl_get_max_frag_len((mbedtls_ssl_context*) session);
+    const unsigned char* data = (const unsigned char*) x_data;
+    size_t max_size = mbedtls_ssl_get_max_out_record_payload
+        ((mbedtls_ssl_context*) session);
     EIO_Status status;
 
+    assert(session  &&  data  &&  n_todo > 0);
+
     *n_done = 0;
 
-    do {
+    for (;;) {
         size_t x_todo = n_todo > max_size ? max_size : n_todo;
         size_t x_done;
         status = x_MbedTlsWrite(session, data, x_todo, &x_done, error);
@@ -566,26 +632,38 @@ static EIO_Status s_MbedTlsWrite(void* session, const void* data,
         if (x_todo != x_done)
             break;
         n_todo  -= x_done;
-        data     = (const char*) data + x_done;
-    } while (n_todo);
+        if (!n_todo)
+            break;
+        data    += x_done;
+    }
 
     return *n_done ? eIO_Success : status;
 }
 
 
+/*ARGSUSED*/
 static EIO_Status s_MbedTlsClose(void* session, int how/*unused*/, int* error)
 {
+    EIO_Status status;
     int x_error;
 
     assert(session);
 
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsClose(%p): Enter", session));
+
     x_error = mbedtls_ssl_close_notify((mbedtls_ssl_context*) session);
 
     if (x_error) {
         *error = x_error;
-        return eIO_Unknown;
-    }
-    return eIO_Success;
+        status = eIO_Unknown;
+    } else
+        status = eIO_Success;
+
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsClose(%p): Leave(%d)", session, status));
+
+    return status;
 }
 
 
@@ -593,8 +671,14 @@ static void s_MbedTlsDelete(void* session)
 {
     assert(session);
 
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsDelete(%p): Enter", session));
+
     mbedtls_ssl_free((mbedtls_ssl_context*) session);
 
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsDelete(%p): Leave", session));
+
     free(session);
 }
 
@@ -619,19 +703,20 @@ static EIO_Status x_InitLocking(void)
 #  ifdef MBEDTLS_THREADING_PTHREAD
     status = eIO_Success;
 #  elif defined(MBEDTLS_THREADING_ALT)  &&  defined(NCBI_THREADS)
-    MT_LOCK lk = CORE_GetLOCK();
-    if (MT_LOCK_Do(lk, eMT_Lock) != -1) {
+    int locked;
+    MT_LOCK lock = CORE_GetLOCK();
+    if ((locked = MT_LOCK_Do(lock, eMT_Lock)) > 0) {
         mbedtls_threading_set_alt(mbtls_user_mutex_init,
                                   mbtls_user_mutex_deinit,
                                   mbtls_user_mutex_lock,
                                   mbtls_user_mutex_unlock);
-        MT_LOCK_Do(lk, eMT_Unlock);
+        MT_LOCK_Do(lock, eMT_Unlock);
         status = eIO_Success;
     } else
-        status = lk ? eIO_Success : eIO_NotSupported;
+        status = !locked ? eIO_Unknown : lock ? eIO_Success : eIO_NotSupported;
 #  elif !defined(NCBI_NO_THREADS)  &&  defined(_MT)
-    CORE_LOG(eLOG_Critical,
-             "MBEDTLS locking uninited: Unknown threading model");
+    CORE_LOG_X(4, eLOG_Critical,
+               "MBEDTLS locking uninited: Unknown threading model");
     status = eIO_NotSupported;
 #  else
     status = eIO_Success;
@@ -641,8 +726,23 @@ static EIO_Status x_InitLocking(void)
 }
 
 
-static void x_FreeLocking(void)
+/*ARGSUSED*/
+static void x_MbedTlsExit(int/*bool*/ nopsa)
 {
+    s_Push = 0;
+    s_Pull = 0;
+
+    mbedtls_ctr_drbg_free(&s_MbedTlsCtrDrbg);
+    mbedtls_entropy_free(&s_MbedTlsEntropy);
+#ifdef MBEDTLS_PSA_CRYPTO_C
+    if (!nopsa)
+        mbedtls_psa_crypto_free();
+#endif /*MBEDTLS_PSA_CRYPTO_C*/
+    mbedtls_ssl_config_free(&s_MbedTlsConf);
+    mbedtls_debug_set_threshold(s_MbedTlsLogLevel = 0);
+    memset(&s_MbedTlsCtrDrbg, 0, sizeof(s_MbedTlsCtrDrbg));
+    memset(&s_MbedTlsEntropy, 0, sizeof(s_MbedTlsEntropy));
+    memset(&s_MbedTlsConf,    0, sizeof(s_MbedTlsConf));
 #  if defined(MBEDTLS_THREADING_ALT)  &&  defined(NCBI_THREADS)
     mbedtls_threading_free_alt();
 #  endif /*MBEDTLS_THREADING_ALT && NCBI_THREADS*/
@@ -659,21 +759,28 @@ static EIO_Status s_MbedTlsInit(FSSLPull pull, FSSLPush push)
         "Embedded "
 #  endif /*HAVE_LIBMBEDTLS*/
         "MBEDTLS";
+#ifdef MBEDTLS_PSA_CRYPTO_C
+    psa_status_t psa_status;
+#endif /*MBEDTLS_PSA_CRYPTO_C*/
     EIO_Status status;
     char version[80];
     const char* val;
     char buf[32];
 
+    CORE_TRACE("MbedTlsInit(): Enter");
+
     mbedtls_version_get_string(version);
     if (strcasecmp(MBEDTLS_VERSION_STRING, version) != 0) {
-        CORE_LOGF(eLOG_Critical,
-                  ("%s version mismatch: %s headers vs. %s runtime",
-                   kMbedTls, MBEDTLS_VERSION_STRING, version));
+        CORE_LOGF_X(5, eLOG_Critical,
+                    ("%s version mismatch: %s headers vs. %s runtime",
+                     kMbedTls, MBEDTLS_VERSION_STRING, version));
         assert(0);
     }
 
-    if (!pull  ||  !push)
-        return eIO_InvalidArg;
+    if (!pull  ||  !push) {
+        status = eIO_InvalidArg;
+        goto out;
+    }
 
     mbedtls_ssl_config_init(&s_MbedTlsConf);
     mbedtls_ssl_config_defaults(&s_MbedTlsConf,
@@ -681,34 +788,42 @@ static EIO_Status s_MbedTlsInit(FSSLPull pull, FSSLPush push)
                                 MBEDTLS_SSL_TRANSPORT_STREAM,
                                 MBEDTLS_SSL_PRESET_DEFAULT);
     mbedtls_ssl_conf_authmode(&s_MbedTlsConf, MBEDTLS_SSL_VERIFY_NONE);
+#if MBEDTLS_VERSION_NUMBER == 0x03060000
+    /* The above line can otherwise be ineffective. */
+    mbedtls_ssl_conf_max_tls_version(&s_MbedTlsConf,
+                                     MBEDTLS_SSL_VERSION_TLS1_2);
+#endif
 
-    val = ConnNetInfo_GetValueInternal(0, "MBEDTLS_LOGLEVEL",
-                                       buf, sizeof(buf), 0);
+    /* Check CONN_[MBED]TLS_LOGLEVEL or [CONN][MBED]TLS_LOGLEVEL */
+    val = ConnNetInfo_GetValueInternal(0, "MBED" REG_CONN_TLS_LOGLEVEL,
+                                       buf, sizeof(buf),
+                                       DEF_CONN_TLS_LOGLEVEL);
     if (!val  ||  !*val) {
-        val = ConnNetInfo_GetValueInternal(0, "TLS_LOGLEVEL",
-                                           buf, sizeof(buf), 0);
+        val = ConnNetInfo_GetValueInternal(0, REG_CONN_TLS_LOGLEVEL,
+                                           buf, sizeof(buf),
+                                           DEF_CONN_TLS_LOGLEVEL);
     }
-    CORE_LOCK_READ;
     if (val  &&  *val) {
         ELOG_Level level;
         s_MbedTlsLogLevel = atoi(val);
-        CORE_UNLOCK;
         if (s_MbedTlsLogLevel) {
             mbedtls_debug_set_threshold(s_MbedTlsLogLevel);
             mbedtls_ssl_conf_dbg(&s_MbedTlsConf, x_MbedTlsLogger, 0);
             level = eLOG_Note;
         } else
             level = eLOG_Trace;
-        CORE_LOGF(level, ("%s V%s (LogLevel=%d)",
-                          kMbedTls, version, s_MbedTlsLogLevel));
-    } else
-        CORE_UNLOCK;
+        CORE_LOGF_X(6, level, ("%s V%s (LogLevel=%d)",
+                               kMbedTls, version, s_MbedTlsLogLevel));
+    }
+
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACE("MbedTlsInit(): Go-on");
 
     if ((status = x_InitLocking()) != eIO_Success) {
-        mbedtls_ssl_config_free(&s_MbedTlsConf);
         mbedtls_debug_set_threshold(s_MbedTlsLogLevel = 0);
+        mbedtls_ssl_config_free(&s_MbedTlsConf);
         memset(&s_MbedTlsConf, 0, sizeof(s_MbedTlsConf));
-        return status;
+        goto out;
     }
 
     mbedtls_entropy_init(&s_MbedTlsEntropy);
@@ -716,35 +831,51 @@ static EIO_Status s_MbedTlsInit(FSSLPull pull, FSSLPush push)
 
     if (mbedtls_ctr_drbg_seed(&s_MbedTlsCtrDrbg, mbedtls_entropy_func,
                               &s_MbedTlsEntropy, 0, 0) != 0) {
-        s_MbedTlsExit();
-        return eIO_Unknown;
+        x_MbedTlsExit(1/*nopsa*/);
+        status = eIO_Unknown;
+        goto out;
     }
     mbedtls_ssl_conf_rng(&s_MbedTlsConf,
                          mbedtls_ctr_drbg_random, &s_MbedTlsCtrDrbg);
 
+#ifdef MBEDTLS_PSA_CRYPTO_C
+    if ((psa_status = psa_crypto_init()) != PSA_SUCCESS) {
+        extern int psa_generic_status_to_mbedtls(psa_status_t status);
+        int err = psa_generic_status_to_mbedtls(psa_status);
+        char errbuf[80];
+        mbedtls_strerror(err, errbuf, sizeof(errbuf));
+        CORE_LOG_ERRNO_EXX(13, eLOG_Error, psa_status, errbuf,
+                           "Platform Security Architecture (PSA) failed to initialize");
+        x_MbedTlsExit(1/*nopsa*/);
+        status = eIO_Unknown;
+        goto out;
+    }
+#endif /*MBEDTLS_PSA_CRYPTO_C*/
+
     s_Pull = pull;
     s_Push = push;
-    return eIO_Success;
+    status = eIO_Success;
+
+out:
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACEF(("MbedTlsInit(): Leave(%d)", status));
+
+    return status;
 }
 
 
 /* NB: Called under a lock */
 static void s_MbedTlsExit(void)
 {
-    s_Push = 0;
-    s_Pull = 0;
+    CORE_DEBUG_ARG(if (s_MbedTlsLogLevel))
+        CORE_TRACE("MbedTlsExit(): Enter");
 
-    mbedtls_ctr_drbg_free(&s_MbedTlsCtrDrbg);
-    mbedtls_entropy_free(&s_MbedTlsEntropy);
-    mbedtls_ssl_config_free(&s_MbedTlsConf);
-    mbedtls_debug_set_threshold(s_MbedTlsLogLevel = 0);
-    memset(&s_MbedTlsCtrDrbg, 0, sizeof(s_MbedTlsCtrDrbg));
-    memset(&s_MbedTlsEntropy, 0, sizeof(s_MbedTlsEntropy));
-    memset(&s_MbedTlsConf,    0, sizeof(s_MbedTlsConf));
-    x_FreeLocking();
+    x_MbedTlsExit(0/*psa as well*/);
+
+    CORE_TRACE("MbedTlsExit(): Leave");
 }
 
- 
+
 static const char* s_MbedTlsError(void* session/*unused*/, int error,
                                   char* buf, size_t size)
 {
@@ -752,18 +883,15 @@ static const char* s_MbedTlsError(void* session/*unused*/, int error,
     return buf;
 }
 
-
 #else
 
-
 /*ARGSUSED*/
 static EIO_Status s_MbedTlsInit(FSSLPull unused_pull, FSSLPush unused_push)
 {
-    CORE_LOG(eLOG_Critical, "Unavailable feature MBEDTLS");
+    CORE_LOG_X(7, eLOG_Critical, "Unavailable feature MBEDTLS");
     return eIO_NotSupported;
 }
 
-
 #endif /*HAVE_LIBMBEDTLS || NCBI_CXX_TOOLKIT*/
 
 
@@ -784,30 +912,144 @@ extern SOCKSSL NcbiSetupMbedTls(void)
 #endif /*HAVE_LIBMBEDTLS || NCBI_CXX_TOOLKIT*/
     };
 #if !defined(HAVE_LIBMBEDTLS)  &&  !defined(NCBI_CXX_TOOLKIT)
-    CORE_LOG(eLOG_Warning, "Unavailable feature MBEDTLS");
+    CORE_LOG_X(8, eLOG_Warning, "Unavailable feature MBEDTLS");
 #endif /*!HAVE_LIBMBEDTLS && !NCBI_CXX_TOOLKIT*/
     return &kMbedTlsOps;
 }
 
 
+#define ALIGN2(s, a)  ((((s) + ((a) - 1)) / (a)) * (a))
+#define ALIGN(s)      ALIGN2(s, sizeof(double))
+
+
 extern NCBI_CRED NcbiCredMbedTls(void* xcert, void* xpkey)
 {
-    struct SNcbiCred* cred;
-    size_t size = sizeof(*cred);
+    struct SNcbiCred*        cred;
+    struct SNcbiMbedTlsCred* xcred;
+    size_t size = ALIGN(sizeof(*cred));
     if (xcert  &&  xpkey) {
         size <<= 1;
-        size  += 2 * sizeof(void*);
+        size  += sizeof(*xcred);
     }
-    cred = (NCBI_CRED) malloc(size);
+    cred = (NCBI_CRED) calloc(1, size);
     if (cred) {
-        memset(cred, 0, size);
         cred->type = eNcbiCred_MbedTls;
         if (xcert  &&  xpkey) {
-            void** data = (void**)((char*) cred + 2 * sizeof(*cred));
-            data[0]     = xcert;
-            data[1]     = xpkey;
-            cred->data  = data;
+            xcred = (struct SNcbiMbedTlsCred*)
+                ((char*) cred + 2*ALIGN(sizeof(*cred)));
+            xcred->cert = (mbedtls_x509_crt*)   xcert;
+            xcred->pkey = (mbedtls_pk_context*) xpkey;
+            cred->data  = xcred;
         }
     }
     return cred;
 }
+
+
+#if defined(HAVE_LIBMBEDTLS)  ||  defined(NCBI_CXX_TOOLKIT)
+
+void NcbiDeleteMbedTlsCertCredentials(NCBI_CRED cred)
+{
+    if (cred->type / 100 == eNcbiCred_MbedTls / 100  &&  !(cred->type % 100)) {
+        struct SNcbiMbedTlsCred* xcred = (struct SNcbiMbedTlsCred*) cred->data;
+        mbedtls_x509_crt_free(xcred->cert);
+        mbedtls_pk_free      (xcred->pkey);
+        memset(xcred, 0, sizeof(*xcred));
+    } else {
+        char who[80];
+        switch (cred->type / 100) {
+        case eNcbiCred_GnuTls / 100:
+            strcpy(who, "GNUTLS");
+            break;
+        case eNcbiCred_MbedTls / 100:
+            strcpy(who, "MBEDTLS");
+            break;
+        default:
+            sprintf(who, "TLS 0x%08X", cred->type);
+            break;
+        }
+        CORE_LOGF_X(9, eLOG_Critical,
+                    ("Deleting unknown certificate credentials (%s/%u)",
+                     who, cred->type % 100));
+        assert(0);
+    }
+    cred->type = (ENcbiCred) 0;
+    cred->data = 0;
+    free(cred);
+}
+
+
+NCBI_CRED NcbiCreateMbedTlsCertCredentials(const void* cert,
+                                           size_t      certsz,
+                                           const void* pkey,
+                                           size_t      pkeysz)
+{
+    struct SNcbiCred*    ncbi_cred;
+    struct SNcbiMbedTlsCred* xcred;
+    const size_t size = (2*ALIGN(sizeof(*ncbi_cred))
+                         + ALIGN(sizeof(*xcred))
+                         + ALIGN(sizeof(*xcred->cert))
+                         +       sizeof(*xcred->pkey));
+    CORE_DEBUG_ARG(char tmp[1024];)
+    char   errbuf[80];
+    int    err;
+
+    if (!(ncbi_cred = (NCBI_CRED) calloc(1, size))) {
+        CORE_LOGF_ERRNO_X(10, eLOG_Error, errno,
+                          ("Cannot allocate NCBI_CRED (%lu bytes)",
+                           (unsigned long) size));
+        return 0;
+    }
+
+    xcred = (struct SNcbiMbedTlsCred*)
+        ((char*) ncbi_cred   + 2*ALIGN(sizeof(*ncbi_cred)));
+    xcred->cert = (mbedtls_x509_crt*)
+        ((char*)     xcred   +   ALIGN(sizeof(*xcred)));
+    xcred->pkey = (mbedtls_pk_context*)
+        ((char*) xcred->cert +   ALIGN(sizeof(*xcred->cert)));
+    ncbi_cred->type = eNcbiCred_MbedTls;
+    ncbi_cred->data = xcred;
+
+    /* these are not technically necessary as they just zero up the memory */
+    mbedtls_x509_crt_init(xcred->cert);
+    mbedtls_pk_init      (xcred->pkey);
+
+    err = mbedtls_x509_crt_parse(xcred->cert,
+                                 (const unsigned char*) cert, certsz ? certsz
+                                 : strlen((const char*) cert) + 1);
+    if (err) {
+        mbedtls_strerror(err, errbuf, sizeof(errbuf) - 1);
+        CORE_LOG_ERRNO_EXX(11, eLOG_Error, err, errbuf,
+                           "mbedTLS cannot parse X.509 certificate");
+        goto out;
+    }
+    CORE_DEBUG_ARG(err = mbedtls_x509_crt_info(tmp, sizeof(tmp),
+                                               "", xcred->cert));
+    CORE_TRACEF(("Certificate loaded%s%s",
+                 err > 0 ? ":\n" : "",
+                 err > 0 ? tmp   : ""));
+
+    err = mbedtls_pk_parse_key(xcred->pkey,
+                               (const unsigned char*) pkey, pkeysz ? pkeysz
+                               : strlen((const char*) pkey) + 1, 0, 0
+#if MBEDTLS_VERSION_MAJOR >= 3
+                               , mbedtls_ctr_drbg_random, &s_MbedTlsCtrDrbg
+#endif
+                               );
+    if (err) {
+        mbedtls_strerror(err, errbuf, sizeof(errbuf) - 1);
+        CORE_LOG_ERRNO_EXX(12, eLOG_Error, err, errbuf,
+                           "mbedTLS cannot parse private key");
+        goto out;
+    }
+    CORE_TRACEF(("Private key loaded: %s",
+                 mbedtls_pk_get_name(xcred->pkey)));
+
+    return ncbi_cred;
+
+ out:
+    NcbiDeleteMbedTlsCertCredentials(ncbi_cred);
+    return 0;
+}
+
+#endif /*HAVE_LIBMBEDTLS || NCBI_CXX_TOOLKIT*/
