@@ -1,4 +1,4 @@
-/* $Id: ncbi_pipe.cpp 663805 2023-03-01 05:54:36Z lavr $
+/* $Id: ncbi_pipe.cpp 695807 2025-04-01 13:24:13Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,7 @@
 #include <corelib/ncbi_param.hpp>
 #include <corelib/ncbi_system.hpp>
 #include <corelib/stream_utils.hpp>
+#include <corelib/ncbi_safe_static.hpp>
 #include <connect/error_codes.hpp>
 #include <connect/ncbi_pipe.hpp>
 
@@ -255,7 +256,7 @@ public:
     EIO_Status Open(const string& cmd, const vector<string>& args,
                     CPipe::TCreateFlags create_flags,
                     const string&       current_dir,
-                    const char* const   env[],
+                    const char* const   envp[],
                     size_t              pipe_size);
     void       OpenSelf(void);
     EIO_Status Close(int* exitcode, const STimeout* timeout);
@@ -322,7 +323,7 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
                              const vector<string>& args,
                              CPipe::TCreateFlags   create_flags,
                              const string&         current_dir,
-                             const char* const     env[],
+                             const char* const     envp[],
                              size_t                pipe_size)
 {
     DEFINE_STATIC_FAST_MUTEX(s_Mutex);
@@ -343,16 +344,34 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
     try {
         // Prepare command line to run
         string cmd_line = x_CommandLine(cmd, args);
+        string x_cmd_line;
+
+        if (cmd.find_first_of(":\\/") == NPOS) {
+            CNcbiApplicationGuard app = CNcbiApplicationAPI::InstanceGuard();
+            AutoPtr<CNcbiEnvironment> env(app
+                                          ? &app->SetEnvironment()
+                                          : new CNcbiEnvironment,
+                                          app
+                                          ? eNoOwnership
+                                          : eTakeOwnership);
+            const string& comspec = env->Get("COMSPEC");
+            if (!comspec.empty()  &&  NStr::CompareNocase(comspec, cmd) != 0) {
+                x_cmd_line = CExec::QuoteArg(comspec) + " /C " + cmd_line;
+            }
+        }
+        if (x_cmd_line.empty()) {
+            x_cmd_line = cmd_line;
+        }
 
         // Convert environment array to block form
         AutoPtr< TXChar, ArrayDeleter<TXChar> > env_block;
-        if (env) {
+        if (envp) {
             // Count block size:
             // it should have one zero char at least
             size_t size = 1; 
             int    count = 0;
-            while (env[count]) {
-                size += strlen(env[count++]) + 1/*'\0'*/;
+            while (envp[count]) {
+                size += strlen(envp[count++]) + 1/*'\0'*/;
             }
             // Allocate memory
             TXChar* block = new TXChar[size];
@@ -361,13 +380,13 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
             // Copy environment strings
             for (int i = 0;  i < count;  ++i) {
 #if defined(NCBI_OS_MSWIN)  &&  defined(_UNICODE)
-                TXString tmp = _T_XSTRING(env[i]);
+                TXString tmp = _T_XSTRING(envp[i]);
                 size_t n = tmp.size();
                 memcpy(block, tmp.data(), n * sizeof(TXChar));
                 block[n++] = _TX('\0');
 #else
-                size_t n = strlen(env[i]) + 1;
-                memcpy(block, env[i], n);
+                size_t n = strlen(envp[i]) + 1;
+                memcpy(block, envp[i], n);
 #endif // NCBI_OS_MSWIN && _UNICODE
                 block += n;
             }
@@ -447,7 +466,7 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
         }
 
         // Create child process
-        STARTUPINFO sinfo;
+        STARTUPINFO         sinfo;
         PROCESS_INFORMATION pinfo;
         ::ZeroMemory(&pinfo, sizeof(pinfo));
         ::ZeroMemory(&sinfo, sizeof(sinfo));
@@ -461,8 +480,10 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
 #  else
         DWORD dwCreationFlags = 0;
 #  endif // _UNICODE
+        if (create_flags & CPipe::fNewGroup)
+            dwCreationFlags |= CREATE_NEW_PROCESS_GROUP;
         if (!::CreateProcess(NULL,
-                             (LPTSTR)(_T_XCSTRING(cmd_line)),
+                             (LPTSTR)(_T_XCSTRING(x_cmd_line)),
                              NULL, NULL, TRUE,
                              dwCreationFlags, env_block.get(),
                              current_dir.empty()
@@ -916,9 +937,14 @@ CPipe::TChildPollMask CPipeHandle::x_Poll(CPipe::TChildPollMask mask,
 // CPipeHandle -- Unix version
 //
 
+
+// Use poll(2) (now default) instead of select(2) (formerly)
 NCBI_PARAM_DECL  (bool, CONN, PIPE_USE_POLL);
 NCBI_PARAM_DEF_EX(bool, CONN, PIPE_USE_POLL,
                   true, eParam_Default, CONN_PIPE_USE_POLL);
+using TUsePoll = NCBI_PARAM_TYPE(CONN, PIPE_USE_POLL);
+static CSafeStatic<TUsePoll> s_UsePoll;
+
 
 class CPipeHandle
 {
@@ -929,7 +955,7 @@ public:
                     const vector<string>& args,
                     CPipe::TCreateFlags   create_flags,
                     const string&         current_dir,
-                    const char* const     env[],
+                    const char* const     envp[],
                     size_t                /*pipe_size*/);
     void       OpenSelf(void);
     EIO_Status Close(int* exitcode, const STimeout* timeout);
@@ -981,9 +1007,7 @@ CPipeHandle::CPipeHandle(void)
     : m_ChildStdIn(-1), m_ChildStdOut(-1), m_ChildStdErr(-1),
       m_Pid((TPid)(-1)), m_Flags(0), m_SelfHandles(false)
 {
-    static NCBI_PARAM_TYPE(CONN, PIPE_USE_POLL) use_poll_param;
-
-    m_UsePoll = use_poll_param.Get();
+    m_UsePoll = s_UsePoll->Get();
     ERR_POST_ONCE(Trace << "CPipeHandle using poll(): "
                   + NStr::BoolToString(m_UsePoll));
 }
@@ -998,8 +1022,8 @@ CPipeHandle::~CPipeHandle()
 
 
 // Auxiliary function to exit from forked process with reporting errno
-// on errors into the specified file descriptor 
-static void s_Exit(int status, int fd)
+// on errors into the specified file descriptor
+[[noreturn]] static void s_Exit(int status, int fd)
 {
     int errcode = errno;
     (void) ::write(fd, &errcode, sizeof(errcode));
@@ -1015,21 +1039,26 @@ static void s_Exit(int status, int fd)
 // and errno is set appropriately.
 
 static int s_ExecShell(const char* command,
-                       char *const argv[], char *const envp[])
+                       char *const argv[], char *const envp[],
+                       const CNcbiEnvironment& xenv)
 {
-    static const char kShell[] = "/bin/sh";
+    static const char kDefaultShell[] = "/bin/sh";
 
-    // Count number of arguments
+    // Count the number of arguments
     size_t i;
     for (i = 0;  argv[i];  ++i);
     ++i; // last NULL element
-    _ASSERT(i > 1);
+    assert(i > 1);
+
+    // Figure out the shell first
+    string xshell = xenv.Get("SHELL");
+    const char* shell = xshell.empty() ? kDefaultShell : xshell.c_str();
 
     // Construct argument list for the shell
     const char** args = new const char*[i + 1];
     AutoPtr< const char*, ArrayDeleter<const char*> > args_ptr(args);
 
-    args[0] = kShell;
+    args[0] = shell;
     args[1] = command;
     // NB: skip argv[0]
     for (;  i > 1;  --i) {
@@ -1037,17 +1066,17 @@ static int s_ExecShell(const char* command,
     }
 
     // Execute the shell
-    return ::execve(kShell, (char**) args, envp);
+    return ::execve(shell, (char**) args, envp);
 }
 
 
-// Note this is executing in a separate process: no environment locks necessary
+// Note this is executing in a separate process: no app/env locks necessary
 static int s_ExecVPE(const char* file, char* const argv[], char* const envp[])
 {
     // CAUTION (security):  the current directory is in the path on purpose
     //                      (see man execlp), and also to be in-sync with the
     //                      default behavior on MS-Win.
-    static const char* kPathDefault = ":/bin:/usr/bin";
+    static const char kDefaultPath[] = ":/bin:/usr/bin";
 
     // If file name is not specified
     if (!file  ||  *file == '\0') {
@@ -1055,17 +1084,25 @@ static int s_ExecVPE(const char* file, char* const argv[], char* const envp[])
         return -1;
     }
 
+    CNcbiApplication*         app = CNcbiApplication::Instance();
+    AutoPtr<CNcbiEnvironment> env(app
+                                  ? &app->SetEnvironment()
+                                  : new CNcbiEnvironment,
+                                  app
+                                  ? eNoOwnership
+                                  : eTakeOwnership);
+
     // If the file name contains path
     if (strchr(file, '/')) {
         ::execve(file, argv, envp);
-        return errno == ENOEXEC ? s_ExecShell(file, argv, envp) : -1;
+        return errno == ENOEXEC ? s_ExecShell(file, argv, envp, *env) : -1;
     }
 
     // Get the PATH environment variable
-    const char* path = getenv("PATH");
-    if (!path) {
-        path = kPathDefault;
-    }
+    const string& xpath = env->Get("PATH");
+    const char* path = xpath.empty() ? kDefaultPath : xpath.c_str();
+    assert(path[0]); // NB: strlen(path) >= 1
+
     size_t file_len = strlen(file) + 1/*'\0'*/;
     char* buf = new char[strlen(path) + 1/*'/'*/ + file_len];
     AutoPtr< char, ArrayDeleter<char> > buf_ptr(buf);
@@ -1075,9 +1112,9 @@ static int s_ExecVPE(const char* file, char* const argv[], char* const envp[])
         const char* next = strchr(path, ':');
         size_t len = next ? (size_t)(next - path) : strlen(path)/*last part*/;
         if (len) {
-            // Copy directory name into the buffer
+            // Copy the directory name into the buffer
             memmove(buf, path, len);
-            // Add slash if needed
+            // Add a slash if needed
             if (buf[len - 1] != '/') {
                 buf[len++]    = '/';
             }
@@ -1087,27 +1124,29 @@ static int s_ExecVPE(const char* file, char* const argv[], char* const envp[])
             buf[1] = '/';
             len = 2;
         }
-        // Add file name
+        // Add the file name
         memcpy(buf + len, file, file_len);
 
-        // Try to execute file with the generated name
+        // Try to execute the file by the generated name
         int error;
         ::execve(buf, argv, envp);
         if ((error = errno) == ENOEXEC) {
-            return s_ExecShell(buf, argv, envp);
+            return s_ExecShell(buf, argv, envp, *env);
         }
         switch (error) {
         case EACCES:
-            // Permission denied.  Memorize this fact and try next path.
+            // Access denied.  Memorize this fact and try next path element.
             eacces_err = true;
             /*FALLTHRU*/
         case ENOENT:
         case ENOTDIR:
             // Try next path element
             break;
+        case 0:
+            assert(0);
+            /*FALLTHRU*/
         default:
             // We found an executable file, but could not execute it
-            _ASSERT(error);
             return -1;
         }
         if (!next) {
@@ -1115,10 +1154,10 @@ static int s_ExecVPE(const char* file, char* const argv[], char* const envp[])
         }
         path = next + 1;
     }
-
     if (eacces_err) {
         errno = EACCES;
     }
+
     return -1;
 }
 
@@ -1155,7 +1194,7 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
                              const vector<string>& args,
                              CPipe::TCreateFlags   create_flags,
                              const string&         current_dir,
-                             const char* const     env[],
+                             const char* const     envp[],
                              size_t                /*unused*/)
 
 {
@@ -1232,9 +1271,24 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
         ::fcntl(status_pipe[1], F_SETFD, 
                 ::fcntl(status_pipe[1], F_GETFD, 0) | FD_CLOEXEC);
 
+        // Prepare program arguments
+        size_t i;
+        const char** argv = new const char*[args.size() + 2];
+        AutoPtr< const char*, ArrayDeleter<const char*> > argv_ptr(argv);
+        argv[i = 0] = cmd.c_str();
+        for (auto&& arg : args) {
+            argv[++i] = arg.c_str();
+        }
+        argv[++i] = 0;
+
+        CORE_LOCK_WRITE;
+        g_CORE_SkipPostForkChildUnlock = 1/*true*/;
+
         // Fork off a child process
         switch (m_Pid = ::fork()) {
         case (TPid)(-1):
+            g_CORE_SkipPostForkChildUnlock = 0/*false*/;
+            CORE_UNLOCK;
             PIPE_THROW(errno,
                        "Failed fork()");
             /*NOTREACHED*/
@@ -1299,35 +1353,28 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
                 ::signal(SIGPIPE, SIG_DFL);
             }
 
-            // Prepare program arguments
-            size_t i;
-            const char** x_args = new const char*[args.size() + 2];
-            AutoPtr< const char*, ArrayDeleter<const char*> > p_args(x_args);
-            x_args[i = 0] = cmd.c_str();
-            for (auto&& arg : args) {
-                x_args[++i] = arg.c_str();
-            }
-            x_args[++i] = 0;
-
             // Change current working directory if specified
             if (!current_dir.empty()  &&  current_dir != ".") {
                 (void) ::chdir(current_dir.c_str());
             }
             // Execute the program
             int status;
-            if (env) {
-                status = execvpe(cmd.c_str(),
-                                 const_cast<char**>(x_args),
-                                 const_cast<char**>(env));
+            if (envp) {
+                status = execvpe(argv[0],
+                                 const_cast<char**>(argv),
+                                 const_cast<char**>(envp));
             } else {
-                status = ::execvp(cmd.c_str(),
-                                  const_cast<char**>(x_args));
+                status = ::execvp(argv[0],
+                                  const_cast<char**>(argv));
             }
             s_Exit(status, status_pipe[1]);
 
             // *** CHILD PROCESS DOES NOT CONTINUE BEYOND THIS LINE ***
         }
 
+        g_CORE_SkipPostForkChildUnlock = 0/*false*/;
+        CORE_UNLOCK;
+
         // Close unused pipes' ends
         if (!IS_SET(create_flags, CPipe::fStdIn_Close)) {
             ::close(pipe_in[0]);
@@ -1865,7 +1912,7 @@ CPipe::CPipe(const string&         cmd,
              const vector<string>& args,
              TCreateFlags          create_flags,
              const string&         current_dir,
-             const char* const     env[],
+             const char* const     envp[],
              size_t                pipe_size)
     : m_PipeSize(pipe_size),
       m_PipeHandle(0), m_ReadHandle(eStdOut),
@@ -1874,7 +1921,7 @@ CPipe::CPipe(const string&         cmd,
 {
     unique_ptr<CPipeHandle> pipe_handle_ptr(new CPipeHandle);
     EIO_Status status = pipe_handle_ptr->Open(cmd, args, create_flags,
-                                              current_dir, env, pipe_size);
+                                              current_dir, envp, pipe_size);
     if (status != eIO_Success) {
         NCBI_THROW(CPipeException, eOpen,
                    "[CPipe::CPipe]  Failed: " + string(IO_StatusStr(status)));
@@ -1894,7 +1941,7 @@ EIO_Status CPipe::Open(const string&         cmd,
                        const vector<string>& args,
                        TCreateFlags          create_flags,
                        const string&         current_dir,
-                       const char* const     env[],
+                       const char* const     envp[],
                        size_t                pipe_size)
 {
     _ASSERT(m_PipeHandle);
@@ -1904,7 +1951,7 @@ EIO_Status CPipe::Open(const string&         cmd,
 
     m_ReadHandle = eStdOut;
     EIO_Status status = m_PipeHandle->Open(cmd, args, create_flags,
-                                           current_dir, env, m_PipeSize);
+                                           current_dir, envp, m_PipeSize);
     m_ReadStatus  = status;
     m_WriteStatus = status;
     return status;
@@ -2110,7 +2157,7 @@ CPipe::EFinish CPipe::ExecWait(const string&           cmd,
                                CNcbiOstream&           err,
                                int&                    exit_code,
                                const string&           current_dir,
-                               const char* const       env[],
+                               const char* const       envp[],
                                CPipe::IProcessWatcher* watcher,
                                const STimeout*         kill_timeout,
                                size_t                  pipe_size)
@@ -2127,7 +2174,7 @@ CPipe::EFinish CPipe::ExecWait(const string&           cmd,
     EIO_Status status = pipe.Open(cmd, args, 
                                   fStdErr_Open | fSigPipe_Restore
                                   | fNewGroup | fKillOnClose,
-                                  current_dir, env);
+                                  current_dir, envp);
     if (status != eIO_Success) {
         NCBI_THROW(CPipeException, eOpen,
                    "[CPipe::ExecWait]  Cannot execute '"
