@@ -1,4 +1,4 @@
-/* $Id: ncbi_priv.c 645948 2022-02-25 14:51:41Z lavr $
+/* $Id: ncbi_priv.c 695803 2025-04-01 13:23:35Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,6 +53,7 @@ FNcbiGetAppName     g_CORE_GetAppName        = 0;
 FNcbiGetReferer     g_CORE_GetReferer        = 0;
 FNcbiGetRequestID   g_CORE_GetRequestID      = 0;
 FNcbiGetRequestDtab g_CORE_GetRequestDtab    = 0;
+int/*bool*/         g_CORE_SkipPostForkChildUnlock = 0/*false*/;
 
 #ifdef NCBI_MONKEY
 FMonkeySend         g_MONKEY_Send            = 0;
@@ -107,20 +108,27 @@ extern int g_NCBI_CoreCheckUnlock(void)
 extern const char* g_CORE_Sprintf(const char* fmt, ...)
 {
     static const size_t buf_size = 4096;
-    char*   buf;
     va_list args;
+    char*   buf;
+    int     len;
 
     if (!(buf = (char*) malloc(buf_size)))
         return 0;
     *buf = '\0';
 
     va_start(args, fmt);
-#ifdef HAVE_VSNPRINTF
-    vsnprintf(buf, buf_size, fmt, args);
+#ifndef HAVE_VSNPRINTF
+    len = vsprintf (buf,           fmt, args);
 #else
-    vsprintf (buf,           fmt, args);
+    len = vsnprintf(buf, buf_size, fmt, args);
+    if (len < 0  ||  buf_size <= len) {
+        memcpy(&buf[buf_size - 4], "...", 4);
+        len = (int)(buf_size - 1);
+    } else
 #endif /*HAVE_VSNPRINTF*/
-    assert(strlen(buf) < buf_size);
+    if (len <= 0)
+        *buf = '\0', len = 0;
+    assert(0 <= len  &&  len < buf_size);
     va_end(args);
     return buf;
 }
