@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_PRIV__H
 #define CONNECT___NCBI_PRIV__H
 
-/* $Id: ncbi_priv.h 668280 2023-06-01 15:35:47Z lavr $
+/* $Id: ncbi_priv.h 695808 2025-04-01 13:24:22Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,6 +53,7 @@
  *                     g_CORE_GetReferer
  *                     g_CORE_GetRequestID
  *                     g_CORE_GetRequestDtab
+ *                     g_CORE_SkipPostForkChildUnlock
  *
  */
 
@@ -92,16 +93,16 @@ extern "C" {
 /* Here are only error codes used in C sources.  For error codes used in
  * C++ sources (in C++ Toolkit) see include/connect/error_codes.hpp.
  */
-NCBI_C_DEFINE_ERRCODE_X(Connect_Conn,          301,  36);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Conn,          301,  37);
 NCBI_C_DEFINE_ERRCODE_X(Connect_Socket,        302, 166);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Util,          303,  13);
-NCBI_C_DEFINE_ERRCODE_X(Connect_LBSM,          304,  30);
-NCBI_C_DEFINE_ERRCODE_X(Connect_FTP,           305,  13);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Util,          303,  15);
+NCBI_C_DEFINE_ERRCODE_X(Connect_LBSM,          304,  31);
+NCBI_C_DEFINE_ERRCODE_X(Connect_FTP,           305,  14);
 NCBI_C_DEFINE_ERRCODE_X(Connect_SMTP,          306,  33);
 NCBI_C_DEFINE_ERRCODE_X(Connect_HTTP,          307,  26);
 NCBI_C_DEFINE_ERRCODE_X(Connect_Service,       308,  10);
 NCBI_C_DEFINE_ERRCODE_X(Connect_HeapMgr,       309,  34);
-NCBI_C_DEFINE_ERRCODE_X(Connect_TLS,           310,  50);
+NCBI_C_DEFINE_ERRCODE_X(Connect_TLS,           310,  50);  /* mbedTLS: 1-20; GNUTLS: 21-40; TLS: 41-50 */
 NCBI_C_DEFINE_ERRCODE_X(Connect_Mghbn,         311,  16);
 NCBI_C_DEFINE_ERRCODE_X(Connect_Crypt,         312,   5);
 NCBI_C_DEFINE_ERRCODE_X(Connect_LocalIP,       313,   5);
@@ -377,6 +378,17 @@ typedef const char* (*FNcbiGetRequestDtab)(void);
 extern NCBI_XCONNECT_EXPORT FNcbiGetRequestDtab g_CORE_GetRequestDtab;
 
 
+/******************************************************************************
+ * Prevent pthread_atfork() from unlocking in the child fork handler.  This may
+ * be necessary if the parent is an MT process and the child is headed for exec()
+ * (as in this case the repertoir of calls limited to only async-signal-safe ones
+ * because the child might have inherited inconsistent locks due to activity
+ * in the other threads -- so having unlock in place can lead to a deadlock).
+ */
+extern int/*bool*/ g_CORE_SkipPostForkChildUnlock/* = 0(Off) by default*/;
+
+
+
 /******************************************************************************
  *  Miscellanea
  */
