@@ -1,4 +1,4 @@
-/* $Id: ncbi_service_connector.c 656746 2022-09-27 15:28:18Z lavr $
+/* $Id: ncbi_service_connector.c 688165 2024-09-25 17:43:50Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -56,16 +56,16 @@ typedef struct SServiceConnectorTag {
     EIO_Status          status;         /* Status of last I/O                */
 
     TSERV_TypeOnly      types;          /* Server types w/o any specials     */
+
     unsigned short      retry;          /* Open retry count since last okay  */
 
     ticket_t            ticket;         /* Network byte order (none if zero) */
-
     unsigned int        host;           /* Parsed connection info... (n.b.o) */
     unsigned short      port;           /*                       ... (h.b.o) */
 
     /* flags */
     unsigned            reset:1;        /* Non-zero if iter was just reset   */
-    unsigned            warned:1;       /* Non-zero when needed adj via HTTP */
+    unsigned            warned:1;       /* Non-zero when adjusted via HTTP   */
     unsigned            unused:5;
     unsigned            secure:1;       /* Set when must start SSL on SOCK   */
 
@@ -154,16 +154,17 @@ static EHTTP_HeaderParse s_ParseHeader(const char* header,
             return eHTTP_HeaderSuccess;
         header_parse = eHTTP_HeaderError;
     }
-    uuu->retry = 0;
+    if (user_callback_enabled  &&  !uuu->warned)
+        uuu->retry = 0;
 
     while (header  &&  *header) {
         if (strncasecmp(header, HTTP_CONNECTION_INFO,
                         sizeof(HTTP_CONNECTION_INFO) - 1) == 0) {
-            if (uuu->host)
-                break/*failed - duplicate connection info*/;
+            if (user_callback_enabled  ||  uuu->host)
+                break/*failed - unexpected / duplicate connection info*/;
             header += sizeof(HTTP_CONNECTION_INFO) - 1;
             while (*header  &&  isspace((unsigned char)(*header)))
-                header++;
+                ++header;
             if (strncasecmp(header, kStateless, kSLen) == 0  &&
                 (!header[kSLen]  ||  isspace((unsigned char) header[kSLen]))) {
                 /* Special keyword for switching into stateless mode */
@@ -315,6 +316,7 @@ static int/*bool*/ s_IsContentTypeDefined(const char*         service,
 
 
 static const char* s_AdjustNetParams(const char*    service,
+                                     const char*    svcname,
                                      SConnNetInfo*  net_info,
                                      EReqMethod     req_method,
                                      const char*    cgi_path,
@@ -367,6 +369,11 @@ static const char* s_AdjustNetParams(const char*    service,
             retval = "";
     }
 
+    if (!ConnNetInfo_SetupStandardArgs(net_info, svcname)) {
+        if (retval  &&  *retval)
+            free((void*) retval);
+        retval = 0;
+    }
     return retval;
 }
 
@@ -437,7 +444,7 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
     flags |= (net_info->debug_printout == eDebugPrintout_Data
               ? fSOCK_LogOn : fSOCK_LogDefault);
     if (net_info->http_proxy_host[0]  &&  net_info->http_proxy_port
-        &&  !net_info->http_proxy_only) {
+        &&  net_info->http_proxy_mask != fProxy_Http) {
         /* NB: ideally, should have pushed data:size here if proxy not buggy */
         *status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
         assert(!sock ^ !(*status != eIO_Success));
@@ -474,7 +481,7 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
         if (size)
             tempf &= (TSOCK_Flags)(~fSOCK_Secure);
         if (!proxy  &&  net_info->debug_printout) {
-            net_info->scheme = eURL_Unspec;
+            assert(net_info->scheme == eURL_Unspec);
             net_info->req_method = eReqMethod_Any;
             net_info->external = 0;
             net_info->firewall = 0;
@@ -484,7 +491,7 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
             net_info->http_push_auth = 0;
             net_info->http_proxy_leak = 0;
             net_info->http_proxy_skip = 0;
-            net_info->http_proxy_only = 0;
+            net_info->http_proxy_mask = 0;
             net_info->user[0] = '\0';
             net_info->pass[0] = '\0';
             net_info->path[0] = '\0';
@@ -497,7 +504,9 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
                 free((void*) net_info->http_referer);
                 net_info->http_referer = 0;
             }
+            CORE_LOCK_READ;
             ConnNetInfo_Log(net_info, eLOG_Note, CORE_GetLOG());
+            CORE_UNLOCK;
         }
         memset(&init, 0, sizeof(init));
         init.data = data;
@@ -587,15 +596,15 @@ extern "C" {
 }
 #endif /*__cplusplus*/
 
-/*ARGSUSED*/
 /* NB: This callback is only for services called via direct HTTP */
 static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
                             void*         data,
                             unsigned int  n)
 {
     SServiceConnector* uuu = (SServiceConnector*) data;
-    const char*        user_header;
+    SERV_ITER          iter = uuu->iter;
     char*              iter_header;
+    const char*        user_header;
     SSERV_InfoCPtr     info;
 
     assert(n  ||  uuu->extra.adjust);
@@ -611,15 +620,14 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
     if (uuu->retry >= uuu->net_info->max_try)
         return 0/*failure - too many errors*/;
     uuu->retry++;
-
     if (!(info = s_GetNextInfo(uuu, 1/*http*/)))
         return 0/*failure - not adjusted*/;
 
-    iter_header = SERV_Print(uuu->iter, 0/*net_info*/, 0);
+    iter_header = SERV_Print(iter, 0/*net_info*/, 0);
     switch (info->type) {
     case fSERV_Ncbid:
         user_header = "Connection-Mode: STATELESS\r\n"; /*default*/
-        user_header = s_AdjustNetParams(uuu->name, net_info,
+        user_header = s_AdjustNetParams(uuu->name, iter->name, net_info,
                                         eReqMethod_Post,
                                         NCBID_WEBPATH,
                                         SERV_NCBID_ARGS(&info->u.ncbid),
@@ -632,7 +640,7 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
     case fSERV_HttpGet:
     case fSERV_HttpPost:
         user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-        user_header = s_AdjustNetParams(uuu->name, net_info,
+        user_header = s_AdjustNetParams(uuu->name, iter->name, net_info,
                                         info->type == fSERV_HttpPost
                                         ?  eReqMethod_Post
                                         : (info->type == fSERV_HttpGet
@@ -648,9 +656,8 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
     case fSERV_Firewall:
     case fSERV_Standalone:
         user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-        user_header = s_AdjustNetParams(uuu->name, net_info,
-                                        eReqMethod_Any,
-                                        uuu->net_info->path, 0,
+        user_header = s_AdjustNetParams(uuu->name, iter->name, net_info,
+                                        eReqMethod_Any, uuu->net_info->path, 0,
                                         ConnNetInfo_GetArgs(uuu->net_info),
                                         user_header, info->mime_t,
                                         info->mime_s, info->mime_e,
@@ -760,15 +767,16 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
                         EIO_Status*        status)
 {
     int/*bool*/ but_last = 0/*false*/;
-    const char* user_header; /* either static "" or non-empty dynamic string */
+    SERV_ITER   iter = uuu->iter;
     char*       iter_header;
+    const char* user_header; /* either static "" or non-empty dynamic string */
     EReqMethod  req_method;
 
     *status = eIO_Success;
     assert(net_info->firewall  ||  info);
     ConnNetInfo_DeleteUserHeader(net_info, kHttpHostTag);
     if (!net_info->http_referer)
-        x_SetDefaultReferer(net_info, uuu->iter);
+        x_SetDefaultReferer(net_info, iter);
     if ((!net_info->firewall  &&  info->type != fSERV_Firewall)
         || (info  &&  ((info->type  & fSERV_Http)  ||
                        (info->type == fSERV_Ncbid  &&  net_info->stateless)))){
@@ -782,15 +790,16 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         switch (info->type) {
         case fSERV_Ncbid:
             /* Connection directly to NCBID, add NCBID-specific tags */
-            if (info->mode & fSERV_Secure)
-                net_info->scheme = eURL_Https;
+            net_info->scheme
+                = info->mode & fSERV_Secure ? eURL_Https : eURL_Http;
             req_method  = eReqMethod_Any; /* replaced with GET if aux HTTP */
             user_header = net_info->stateless
                 /* Connection request with data */
                 ? "Connection-Mode: STATELESS\r\n" /*default*/
                 /* We will be waiting for conn-info back */
                 : "Connection-Mode: STATEFUL\r\n";
-            user_header = s_AdjustNetParams(uuu->name, net_info, req_method,
+            user_header = s_AdjustNetParams(uuu->name, iter->name, net_info,
+                                            req_method,
                                             NCBID_WEBPATH,
                                             SERV_NCBID_ARGS(&info->u.ncbid),
                                             0, user_header, info->mime_t,
@@ -800,12 +809,15 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         case fSERV_HttpGet:
         case fSERV_HttpPost:
             /* Connection directly to a CGI */
-            net_info->stateless = 1/*true*/;
+            net_info->scheme
+                = info->mode & fSERV_Secure ? eURL_Https : eURL_Http;
             req_method  = info->type == fSERV_HttpGet
                 ? eReqMethod_Get : (info->type == fSERV_HttpPost
                                     ? eReqMethod_Post : eReqMethod_Any);
+            net_info->stateless = 1/*true*/;
             user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-            user_header = s_AdjustNetParams(uuu->name, net_info, req_method,
+            user_header = s_AdjustNetParams(uuu->name, iter->name, net_info,
+                                            req_method,
                                             SERV_HTTP_PATH(&info->u.http),
                                             SERV_HTTP_ARGS(&info->u.http),
                                             0, user_header, info->mime_t,
@@ -813,6 +825,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
             break;
         case fSERV_Standalone:
             if (!net_info->stateless) {
+                net_info->scheme = eURL_Unspec;
                 assert(!uuu->descr);
                 uuu->descr = x_HostPort(net_info->host, net_info->port);
                 return s_SocketConnectorBuilder(net_info, uuu->descr, status,
@@ -823,7 +836,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
             if (!net_info->scheme)
                 net_info->scheme = eURL_Https;
             user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-            user_header = s_AdjustNetParams(uuu->name, net_info,
+            user_header = s_AdjustNetParams(uuu->name, iter->name, net_info,
                                             eReqMethod_Any, 0, 0,
                                             0, user_header, info->mime_t,
                                             info->mime_s, info->mime_e, 0);
@@ -870,7 +883,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         user_header = (net_info->stateless
                        ? "Client-Mode: STATELESS_ONLY\r\n" /*default*/
                        : "Client-Mode: STATEFUL_CAPABLE\r\n");
-        user_header = s_AdjustNetParams(uuu->name, net_info,
+        user_header = s_AdjustNetParams(uuu->name, iter->name, net_info,
                                         req_method, 0, 0,
                                         0, user_header, mime_t,
                                         mime_s, mime_e, 0);
@@ -878,7 +891,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
     if (!user_header)
         return 0;
 
-    if ((iter_header = SERV_Print(uuu->iter, net_info, but_last)) != 0) {
+    if ((iter_header = SERV_Print(iter, net_info, but_last)) != 0) {
         size_t uh_len;
         if ((uh_len = strlen(user_header)) > 0) {
             char*  ih;
@@ -903,9 +916,6 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
     if (user_header && !ConnNetInfo_OverrideUserHeader(net_info, user_header))
         return 0;
 
-    if (!ConnNetInfo_SetupStandardArgs(net_info, uuu->iter->name))
-        return 0;
-
     ConnNetInfo_ExtendUserHeader
         (net_info, "User-Agent: NCBIServiceConnector/"
          NCBI_DISP_VERSION
@@ -929,7 +939,9 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         uuu->port = 0;
         uuu->ticket = 0;
         uuu->secure = 0;
-        assert(!net_info->req_method);
+        assert(net_info->scheme == eURL_Https  ||
+               net_info->scheme == eURL_Http);
+        assert(!net_info->req_method/*Any*/);
         net_info->req_method = eReqMethod_Get;
         c = HTTP_CreateConnectorEx(net_info,
                                    fHTTP_Flushable | fHTTP_NoAutoRetry,
@@ -973,8 +985,6 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         }
         if (!uuu->host  ||  !uuu->port) {
             /* no connection info found */
-            if (!net_info->scheme)
-                net_info->scheme = eURL_Http;
             ConnNetInfo_SetArgs(net_info, 0);
             assert(!uuu->descr);
             return 0;
@@ -986,12 +996,11 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
                          uuu->name, uuu->port));
         }
         ConnNetInfo_DeleteUserHeader(net_info, uuu->user_header);
+        net_info->scheme = eURL_Unspec;
         SOCK_ntoa(uuu->host, net_info->host, sizeof(net_info->host));
         net_info->port = uuu->port;
         assert(!uuu->descr);
         uuu->descr = x_HostPort(net_info->host, net_info->port);
-        if (net_info->http_proxy_host[0]  &&  net_info->http_proxy_port)
-            net_info->scheme = uuu->net_info->scheme;
         return s_SocketConnectorBuilder(net_info, uuu->descr, status,
                                         &uuu->ticket,
                                         uuu->ticket ? sizeof(uuu->ticket) : 0,
@@ -1123,7 +1132,7 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
             net_info->http_proxy_pass[0] = '\0';
             net_info->http_proxy_leak    =   0;
             net_info->http_proxy_skip    =   0;
-            net_info->http_proxy_only    =   1;
+            net_info->http_proxy_mask    =   fProxy_Http;
         }
 
         c = s_Open(uuu, timeout, info, net_info, &status);
@@ -1273,21 +1282,27 @@ extern CONNECTOR SERVICE_CreateConnectorEx
  const SSERVICE_Extra* extra)
 {
     SConnNetInfo*      x_net_info;
-    char*              x_name;
     size_t             len;
     CONNECTOR          ccc;
     SServiceConnector* xxx;
 
-    if (!(x_name = SERV_ServiceName(name)))
+    if (!net_info) {
+        char* x_name = SERV_ServiceName(name);
+        if (!x_name)
+            return 0;
+        x_net_info = ConnNetInfo_CreateInternal(x_name);
+    } else
+        x_net_info = ConnNetInfo_Clone(net_info);
+    if (!x_net_info)
         return 0;
 
     if (!(ccc = (SConnector*) malloc(sizeof(SConnector)))) {
-        free(x_name);
+        ConnNetInfo_Destroy(x_net_info);
         return 0;
     }
     len = strlen(name);
-    if (!(xxx = (SServiceConnector*) calloc(1, sizeof(*xxx) + len + 1))) {
-        free(x_name);
+    if (!(xxx = (SServiceConnector*) calloc(1, sizeof(*xxx) + len))) {
+        ConnNetInfo_Destroy(x_net_info);
         free(ccc);
         return 0;
     }
@@ -1299,14 +1314,6 @@ extern CONNECTOR SERVICE_CreateConnectorEx
     ccc->setup    = s_Setup;
     ccc->destroy  = s_Destroy;
 
-    x_net_info    = (net_info
-                     ? ConnNetInfo_Clone(net_info)
-                     : ConnNetInfo_CreateInternal(x_name));
-    free(x_name);
-    if (!x_net_info) {
-        s_Destroy(ccc);
-        return 0;
-    }
     xxx->net_info = x_net_info;
     xxx->types    = (TSERV_TypeOnly) types;
     /* NB: zero'ed block, no need to copy the trailing '\0' */
