@@ -1,4 +1,4 @@
-/* $Id: ncbi_service_cxx.cpp 641147 2021-11-24 20:22:45Z lavr $
+/* $Id: ncbi_service_cxx.cpp 682979 2024-05-14 19:12:05Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -57,7 +57,7 @@ static string x_HostOfInfo(const SSERV_Info*     info,
         return xx_host;
     }
 
-    _ASSERT(0);
+    _TROUBLE;
     return kEmptyStr;
 }
 
