@@ -1,4 +1,4 @@
-/* $Id: ncbi_usage_report.cpp 602851 2020-03-03 18:47:23Z ivanov $
+/* $Id: ncbi_usage_report.cpp 689723 2024-11-05 13:17:36Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,8 +34,8 @@
 #include <corelib/ncbistr.hpp>
 #include <corelib/ncbi_param.hpp>
 #include <connect/ncbi_usage_report.hpp>
-#include <sstream>
 #include <atomic>
+#include <sstream>
 
 BEGIN_NCBI_SCOPE
 
@@ -49,11 +49,20 @@ BEGIN_NCBI_SCOPE
 const char* kDefault_URL = "https://www.ncbi.nlm.nih.gov/stat";
 
 // Usage logging is disabled by default.
-const bool  kDefault_IsEnabled = false;
+const bool kDefault_IsEnabled = false;
 
 // Maximum number of threads for asynchronous reporting.
 const unsigned kDefault_MaxQueueSize = 100;
 
+// Default connection timeout (negative == use Connect API settings)
+const double kDefault_ConnTimeout = -1;
+
+// Default connection max retry count (negative == use Connect API settings)
+const int kDefault_ConnMaxTry = -1;
+
+// Default Wait() timeout (negative == infinite timeout)
+const double kDefault_WaitTimeout = -1;
+
 
 /////////////////////////////////////////////////////////////////////////////
 //  Global parameters
@@ -74,6 +83,15 @@ NCBI_PARAM_DEF_EX(string,   USAGE_REPORT, AppVersion, "", eParam_NoThread, NCBI_
 NCBI_PARAM_DECL(unsigned,   USAGE_REPORT, MaxQueueSize);
 NCBI_PARAM_DEF_EX(unsigned, USAGE_REPORT, MaxQueueSize, kDefault_MaxQueueSize, eParam_NoThread, NCBI_USAGE_REPORT_MAXQUEUESIZE);
 
+NCBI_PARAM_DECL(double,     USAGE_REPORT, ConnTimeout);
+NCBI_PARAM_DEF_EX(double,   USAGE_REPORT, ConnTimeout, kDefault_ConnTimeout, eParam_NoThread, NCBI_USAGE_REPORT_CONN_TIMEOUT);
+
+NCBI_PARAM_DECL(int,        USAGE_REPORT, ConnMaxTry);
+NCBI_PARAM_DEF_EX(int,      USAGE_REPORT, ConnMaxTry, kDefault_ConnMaxTry, eParam_NoThread, NCBI_USAGE_REPORT_CONN_MAX_TRY);
+
+NCBI_PARAM_DECL(double,     USAGE_REPORT, WaitTimeout);
+NCBI_PARAM_DEF_EX(double,   USAGE_REPORT, WaitTimeout, kDefault_WaitTimeout, eParam_NoThread, NCBI_USAGE_REPORT_WAIT_TIMEOUT);
+
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -81,9 +99,12 @@ NCBI_PARAM_DEF_EX(unsigned, USAGE_REPORT, MaxQueueSize, kDefault_MaxQueueSize, e
 /// CUsageReportAPI::
 ///
 
-// Global default parameters to report
+// Globals 
+
+// Default parameters to report
 static atomic<CUsageReport::TWhat> gs_DefaultParams(CUsageReport::fDefault);
 
+
 void CUsageReportAPI::SetDefaultParameters(TWhat what)
 {
     gs_DefaultParams = what;
@@ -103,13 +124,25 @@ CUsageReportAPI::TWhat CUsageReportAPI::GetDefaultParameters(void)
 
 static bool gs_IsEnabled = NCBI_PARAM_TYPE(USAGE_REPORT, Enabled)::GetDefault();
 
-bool CUsageReportAPI::IsEnabled()
+bool CUsageReportAPI::IsEnabled(void)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
-    return gs_IsEnabled;
-#else
-    return false;
+    if (!gs_IsEnabled) {
+        return false;
+    };
+    const char* do_not_track = ::getenv("DO_NOT_TRACK");
+    if (!do_not_track) {
+        return true;
+    }
+    try {
+        if (!NStr::StringToBool(do_not_track)) {
+            return true;
+        }
+    }
+    catch (CStringException&) {
+    }
 #endif
+    return false;
 }
 
 void CUsageReportAPI::SetEnabled(bool enable)
@@ -124,32 +157,17 @@ void CUsageReportAPI::SetURL(const string& url)
     NCBI_PARAM_TYPE(USAGE_REPORT, URL)::SetDefault(url);
 }
 
+string CUsageReportAPI::GetURL(void)
+{
+    return NCBI_PARAM_TYPE(USAGE_REPORT, URL)::GetDefault();
+}
+
 void CUsageReportAPI::SetAppName(const string& name)
 {
     NCBI_PARAM_TYPE(USAGE_REPORT, AppName)::SetDefault(name);
 }
 
-void CUsageReportAPI::SetAppVersion(const string& version)
-{
-    NCBI_PARAM_TYPE(USAGE_REPORT, AppVersion)::SetDefault(version);
-}
-
-void CUsageReportAPI::SetAppVersion(const CVersionInfo& version)
-{
-    SetAppVersion(version.Print());
-}
-
-void CUsageReportAPI::SetMaxQueueSize(unsigned n)
-{
-    NCBI_PARAM_TYPE(USAGE_REPORT, MaxQueueSize)::SetDefault(n ? n : kDefault_MaxQueueSize);
-}
-
-string CUsageReportAPI::GetURL()
-{
-    return NCBI_PARAM_TYPE(USAGE_REPORT, URL)::GetDefault();
-}
-
-string CUsageReportAPI::GetAppName()
+string CUsageReportAPI::GetAppName(void)
 {
     string name;
     CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
@@ -162,7 +180,7 @@ string CUsageReportAPI::GetAppName()
     return name;
 }
 
-string CUsageReportAPI::GetAppVersion()
+string CUsageReportAPI::GetAppVersion(void)
 {
     string version;
     CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
@@ -175,17 +193,92 @@ string CUsageReportAPI::GetAppVersion()
     return version;
 }
 
-unsigned CUsageReportAPI::GetMaxQueueSize()
+void CUsageReportAPI::SetAppVersion(const string& version)
+{
+    NCBI_PARAM_TYPE(USAGE_REPORT, AppVersion)::SetDefault(version);
+}
+
+void CUsageReportAPI::SetAppVersion(const CVersionInfo& version)
+{
+    SetAppVersion(version.Print());
+}
+
+unsigned CUsageReportAPI::GetMaxQueueSize(void)
 {
     return NCBI_PARAM_TYPE(USAGE_REPORT, MaxQueueSize)::GetDefault();
 }
 
+void CUsageReportAPI::SetMaxQueueSize(unsigned int n)
+{
+    NCBI_PARAM_TYPE(USAGE_REPORT, MaxQueueSize)::SetDefault(n ? n : kDefault_MaxQueueSize);
+}
+
+CTimeout CUsageReportAPI::GetTimeout()
+{
+    double v = NCBI_PARAM_TYPE(USAGE_REPORT, ConnTimeout)::GetDefault();
+    if (v < 0) {
+        return CTimeout(CTimeout::eDefault);
+    }
+    return CTimeout(v);
+}
+
+void CUsageReportAPI::SetTimeout(const CTimeout& timeout)
+{
+    double v;
+    if (timeout.IsFinite()) {
+        v = timeout.GetAsDouble();
+    } 
+    else if (timeout.IsDefault()) {
+        v = kDefault_ConnTimeout;
+    } 
+    else if (timeout.IsZero()) {
+        v = 0;
+    }
+    else {
+        NCBI_THROW(CCoreException, eInvalidArg, "Only default or finite timeouts are supported");
+    }
+    NCBI_PARAM_TYPE(USAGE_REPORT, ConnTimeout)::SetDefault(v);
+}
+
+int CUsageReportAPI::GetRetries()
+{
+    return NCBI_PARAM_TYPE(USAGE_REPORT, ConnMaxTry)::GetDefault();
+}
+
+void CUsageReportAPI::SetRetries(int retries)
+{
+    NCBI_PARAM_TYPE(USAGE_REPORT, ConnMaxTry)::SetDefault(retries);
+}
+
+bool s_CheckConnection(const string& url)
+{
+#if defined(NCBI_USAGE_REPORT_SUPPORTED)
+    // Silent mode -- discard all diagnostic messages from CHttpSession during this call.
+    // Affects current thread/function only.
+    CDiagCollectGuard diag_guard;
+    CTimeout timeout = CUsageReportAPI::GetTimeout();
+    int tries = CUsageReportAPI::GetRetries();
+    CHttpSession session;
+    CHttpResponse response = session.Get(url, timeout, tries < 0 ? THttpRetries(null) : THttpRetries(static_cast<unsigned short>(tries)));
+    return response.GetStatusCode() == 200;
+#else
+    return false;
+#endif
+}
+
+bool CUsageReportAPI::CheckConnection()
+{
+    return s_CheckConnection(GetURL());
+}
+
+
 
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// CUsageReportParameters::
 ///
 
+#if defined(NCBI_USAGE_REPORT_SUPPORTED)
 #ifdef _DEBUG
     inline void s_CheckName(const string& name)
     {
@@ -198,6 +291,7 @@ unsigned CUsageReportAPI::GetMaxQueueSize()
 #else 
     #define CHECK_USAGE_REPORT_PARAM_NAME(name) 
 #endif
+#endif
 
 CUsageReportParameters& CUsageReportParameters::Add(const string& name, const string& value)
 {
@@ -213,7 +307,7 @@ CUsageReportParameters& CUsageReportParameters::Add(const string& name, const ch
     return Add(name, string(value));
 }
 
-string CUsageReportParameters::ToString() const
+string CUsageReportParameters::ToString(void) const
 {
     string result;
 
@@ -263,17 +357,17 @@ void CUsageReportJob::x_CopyFrom(const CUsageReportJob& other)
 /// Helpers
 ///
 
-static string s_GetOS()
+#if defined(NCBI_USAGE_REPORT_SUPPORTED)
+static string s_GetOS(void)
 {
     // Check NCBI_OS first, configure can define OS name already
-#if defined(NCBI_OS)
+#if   defined(NCBI_OS)
     return NCBI_OS;
-#endif
     // Fallback, try to guess
-#if defined(_WIN32)
+#elif defined(_WIN32)
     return "MSWIN";         // Windows
 #elif defined(__CYGWIN__)
-    return "CYGWIN";        // Windows (Cygwin POSIX under Microsoft Window)
+    return "CYGWIN";        // Windows (Cygwin POSIX under Microsoft Windows)
 #elif defined(__linux__)
     return "LINUX";
 #elif defined(__FreeBSD__)
@@ -298,15 +392,17 @@ static string s_GetOS()
     return "SOLARIS";
 #elif defined(unix) || defined(__unix__) || defined(__unix)
     return "UNIX";
+#else
+    return kEmptyStr;       // Unknown environment
 #endif
-    return string();        // Unknown environment
 }
 
-static string s_GetHost()
+static string s_GetHost(void)
 {
     CDiagContext& ctx = GetDiagContext();
     return ctx.GetHost();
 }
+#endif
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -314,7 +410,7 @@ static string s_GetHost()
 /// CUsageReport::
 ///
 
-// Usage MT-protection for CUsageReport
+// MT-protection guard for CUsageReport
 #define MT_GUARD std::lock_guard<std::mutex> mt_usage_guard(m_Usage_Mutex)
 
 
@@ -361,21 +457,26 @@ CUsageReport::CUsageReport(TWhat what, const string& url, unsigned max_queue_siz
     m_URL          = url.empty() ? CUsageReportAPI::GetURL() : url;
     m_MaxQueueSize = max_queue_size ? max_queue_size : CUsageReportAPI::GetMaxQueueSize();
 
+    // Init statictics counters
+    m_CountTotal = 0;
+    m_CountSent  = 0;
+
     // Enable reporter
-    m_IsEnabled   = true;
     m_IsFinishing = false;
+    m_IsWaiting   = false;
+    m_IsEnabled   = true;
 
 #endif
 }
 
-CUsageReport::~CUsageReport(void)
+CUsageReport::~CUsageReport()
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
     Finish();
 #endif
 }
 
-bool CUsageReport::IsEnabled()
+bool CUsageReport::IsEnabled(void)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
     return !m_IsFinishing  &&  m_IsEnabled  &&  CUsageReportAPI::IsEnabled();
@@ -388,7 +489,6 @@ bool CUsageReport::IsEnabled()
 bool CUsageReport::x_Send(const string& extra_params)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
-
     // Silent mode -- discard all diagnostic messages from CHttpSession during this call.
     // Affects current thread/function only.
     CDiagCollectGuard diag_guard;
@@ -397,8 +497,11 @@ bool CUsageReport::x_Send(const string& extra_params)
     if (!extra_params.empty()) {
         url += '&' + extra_params;
     }
+    CTimeout timeout = CUsageReportAPI::GetTimeout();
+    int tries = CUsageReportAPI::GetRetries();
+
     CHttpSession session;
-    CHttpResponse response = session.Get(url);
+    CHttpResponse response = session.Get(url, timeout, tries < 0 ? THttpRetries(null) : THttpRetries(static_cast<unsigned short>(tries)));
     return response.GetStatusCode() == 200;
 #else
     return false;
@@ -438,6 +541,7 @@ void CUsageReport::x_SendAsync(TJobPtr job_ptr)
 #endif
 }
 
+// MT-safe
 void CUsageReport::Send(void)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
@@ -449,6 +553,7 @@ void CUsageReport::Send(void)
 #endif
 }
 
+// MT-safe
 void CUsageReport::Send(CUsageReportParameters& params)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
@@ -462,7 +567,8 @@ void CUsageReport::Send(CUsageReportParameters& params)
 #endif
 }
 
-unsigned CUsageReport::GetQueueSize()
+// MT-safe
+unsigned CUsageReport::GetQueueSize(void)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
     MT_GUARD;
@@ -471,7 +577,8 @@ unsigned CUsageReport::GetQueueSize()
     return 0;
 #endif
 }
-
+ 
+// MT-safe
 void CUsageReport::ClearQueue(void)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
@@ -492,10 +599,36 @@ void CUsageReport::x_ClearQueue(void)
 #endif
 }
 
-void CUsageReport::Wait(void)
+// MT-safe
+void CUsageReport::Wait(EWait how, CTimeout timeout)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
 
+    {{
+        MT_GUARD;
+        if (m_IsFinishing) {
+            // Finishing, nothing to wait, queue is empty
+            return;
+        }
+        if (m_IsWaiting) {
+            // Wait() already happening in the another thread, do nothing
+            return;
+        }
+        // Resolve default timeout
+        if (timeout.IsDefault()) {
+            double v = NCBI_PARAM_TYPE(USAGE_REPORT, WaitTimeout)::GetDefault();
+            if (v < 0) {
+                timeout.Set(CTimeout::eInfinite);
+            } else{
+                timeout.Set(v);
+            }
+        }
+        // Set Wait mode
+        m_WaitMode = how;
+        m_WaitDeadline = CDeadline(timeout);
+        m_IsWaiting = true;
+    }}    
+
     while (true) {
         if (m_IsFinishing) {
             // Finishing, nothing to wait, queue is empty
@@ -526,7 +659,12 @@ void CUsageReport::Wait(void)
 
             // Check queue size
             MT_GUARD;
+            if (!m_IsWaiting) {
+                // Wait has canceled in the thread handler
+                return;
+            }
             if (m_Queue.empty()) {
+                m_IsWaiting = false;
                 return;
             }
         }}
@@ -534,6 +672,7 @@ void CUsageReport::Wait(void)
 #endif
 }
 
+// MT-safe
 void CUsageReport::Finish(void)
 {
 #if defined(NCBI_USAGE_REPORT_SUPPORTED)
@@ -563,7 +702,8 @@ void CUsageReport::x_ThreadHandler(void)
         // Awaiting for a signal from the main thread to proceed
         m_ThreadSignal.wait(signal_lock);
 
-        // Process all jobs in the queue
+        // Process jobs in the queue
+
         while (true) {
             // Check on finishing flag between processing jobs
             if (m_IsFinishing) {
@@ -572,6 +712,15 @@ void CUsageReport::x_ThreadHandler(void)
             TJobPtr job = nullptr;
             {{
                 MT_GUARD;
+                // Check Wait() conditions, if any, and cancel it if required
+                if (m_IsWaiting) {
+                    if ((m_WaitMode == eSkipIfNoConnection  &&  m_CountTotal > 0  &&  m_CountSent == 0) ||
+                        m_WaitDeadline.IsExpired() ) {
+                        m_IsWaiting = false;
+                        break;
+                    }
+                }
+                // Get next job from the queue
                 if (!m_Queue.empty()) {
                     if (IsEnabled()) {
                         job = m_Queue.front();
@@ -586,6 +735,11 @@ void CUsageReport::x_ThreadHandler(void)
                 // Send report
                 job->x_SetState(CUsageReportJob::eRunning);
                 bool res = x_Send(job->ToString());
+                // Increase counters
+                ++m_CountTotal;
+                if (res) {
+                    ++m_CountSent;
+                }
                 // Set result state
                 job->x_SetState(res ? CUsageReportJob::eCompleted : CUsageReportJob::eFailed);
             } 
@@ -598,5 +752,15 @@ void CUsageReport::x_ThreadHandler(void)
 #endif
 }
 
+// MT-safe
+bool CUsageReport::CheckConnection()
+{
+#if defined(NCBI_USAGE_REPORT_SUPPORTED)
+    return s_CheckConnection(m_URL);
+#else
+    return false;
+#endif
+}
+
 
 END_NCBI_SCOPE
