@@ -1,4 +1,4 @@
-/* $Id: ncbi_util.c 668369 2023-06-03 01:48:35Z lavr $
+/* $Id: ncbi_util.c 689726 2024-11-05 14:10:58Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -74,9 +74,13 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_Util
 
+
 #define NCBI_USE_PRECOMPILED_CRC32_TABLES 1
 
 
+static const char kOutOfMemory[] = "Ouch! Out of memory";
+
+
 #if 0
 static char* x_getenv(const char* name)
 {
@@ -169,7 +173,6 @@ extern int/*bool*/ CORE_SetLOGFILE_NAME_Ex
                           ("Cannot open \"%s\"", logfile));
         return 0/*false*/;
     }
-
     CORE_SetLOGFILE_Ex(fp, cut_off, fatal_err, 1/*auto_close*/);
     return 1/*true*/;
 }
@@ -210,7 +213,7 @@ extern size_t UTIL_PrintableStringSize(const char* data, size_t size)
             size += 3;
     }
     return size +
-        (((size + UTIL_PRINTABLE_WIDTH - 1) / UTIL_PRINTABLE_WIDTH) << 1);
+        (((size + UTIL_PRINTABLE_WIDTH_MIN - 1) / UTIL_PRINTABLE_WIDTH_MIN)*2);
 }
 
 
@@ -223,8 +226,8 @@ extern char* UTIL_PrintableStringEx(const char* src, size_t size, char* dst,
         return 0;
     if (!size)
         size = strlen(src);
-    if (width < UTIL_PRINTABLE_WIDTH  &&  width)
-        width = UTIL_PRINTABLE_WIDTH;
+    if (width < UTIL_PRINTABLE_WIDTH_MIN  &&  width)
+        width = UTIL_PRINTABLE_WIDTH_MIN;
 
     for (w = dst, s = src;  size;  --size, ++s) {
         unsigned char c = (unsigned char)(*s);
@@ -389,7 +392,7 @@ extern const char* NcbiMessagePlusError
             UTIL_ReleaseBuffer(descr);
         else if (release < 0)
             UTIL_ReleaseBufferOnHeap(descr);
-        return "Ouch! Out of memory";
+        return kOutOfMemory;
     }
 
     if (message) {
@@ -428,7 +431,7 @@ extern char* LOG_ComposeMessage
     static const char kRawData_Beg[] =
         "\n#################### [BEGIN] Raw Data (%lu byte%s):%s";
     static const char kRawData_End[] =
-        "\n#################### [_END_] Raw Data\n";
+        "\n#################### [_END_] Raw Data";
 
     /* Calculated length of ... */
     size_t datetime_len  = 0;
@@ -441,7 +444,7 @@ extern char* LOG_ComposeMessage
     size_t total_len;
 
     const char* level;
-    char *str, *s, datetime[32];
+    char *str, *s, datetime[40];
 
     /* Adjust formatting flags */
     if (mess->level == eLOG_Trace  &&  !(flags & fLOG_None))
@@ -456,33 +459,21 @@ extern char* LOG_ComposeMessage
 
     /* Pre-calculate total message length */
     if ((flags & fLOG_DateTime) != 0) {
-#ifdef NCBI_OS_MSWIN /*Should be compiler-dependent but C-Tkit lacks it*/
-        _strdate(&datetime[datetime_len]);
-        datetime_len += strlen(&datetime[datetime_len]);
-        datetime[datetime_len++] = ' ';
-        _strtime(&datetime[datetime_len]);
-        datetime_len += strlen(&datetime[datetime_len]);
-        datetime[datetime_len++] = ' ';
-        datetime[datetime_len]   = '\0';
-#else /*NCBI_OS_MSWIN*/
         static const char timefmt[] = "%m/%d/%y %H:%M:%S ";
-        struct tm* tm;
-#  ifdef NCBI_CXX_TOOLKIT
-        time_t t = time(0);
-#    ifdef HAVE_LOCALTIME_R
-        struct tm temp;
-        localtime_r(&t, &temp);
-        tm = &temp;
-#    else /*HAVE_LOCALTIME_R*/
-        tm = localtime(&t);
-#    endif/*HAVE_LOCALTIME_R*/
-#  else /*NCBI_CXX_TOOLKIT*/
-        struct tm temp;
-        Nlm_GetDayTime(&temp);
-        tm = &temp;
-#  endif /*NCBI_CXX_TOOLKIT*/
-        datetime_len = strftime(datetime, sizeof(datetime), timefmt, tm);
-#endif /*NCBI_OS_MSWIN*/
+        struct tm tm;
+#ifdef NCBI_CXX_TOOLKIT
+        time_t now = time(0);
+#  ifdef HAVE_LOCALTIME_R
+        localtime_r(&now, &tm);
+#  else /*HAVE_LOCALTIME_R*/
+        CORE_LOCK_WRITE;
+        tm = *localtime(&now);
+        CORE_UNLOCK;
+#  endif/*HAVE_LOCALTIME_R*/
+#else /*NCBI_CXX_TOOLKIT*/
+        Nlm_GetDayTime(&tm);
+#endif /*NCBI_CXX_TOOLKIT*/
+        datetime_len = strftime(datetime, sizeof(datetime), timefmt, &tm);
     }
     if ((flags & fLOG_Level) != 0
         &&  (mess->level != eLOG_Note  ||  !(flags & fLOG_OmitNoteLevel))) {
@@ -499,8 +490,14 @@ extern char* LOG_ComposeMessage
     }
     if ((flags & fLOG_FileLine) != 0  &&  mess->file  &&  *mess->file)
         file_line_len = 12 + strlen(mess->file) + 11;
-    if (mess->message  &&  *mess->message)
+    if (mess->message  &&  *mess->message) {
         message_len = strlen(mess->message);
+        do {
+            if (!isspace((unsigned char) mess->message[message_len - 1]))
+                break;
+        } while (--message_len > 0);
+    } else if (!mess->message)
+        message_len = sizeof(kOutOfMemory) - 1;
     if (mess->raw_size) {
         if (mess->raw_data) {
             data_len = UTIL_PrintableStringSize((const char*)
@@ -549,7 +546,7 @@ extern char* LOG_ComposeMessage
         *s++ = ' ';
     }
     if (message_len) {
-        memcpy(s, mess->message, message_len);
+        memcpy(s, mess->message ? mess->message : kOutOfMemory, message_len);
         s += message_len;
     }
     if (data_len) {
@@ -595,13 +592,17 @@ static void s_LOG_FileHandler(void* data, const SLOG_Message* mess)
     if (mess->level >= logdata->cut_off  ||
         mess->level >= logdata->fatal_err) {
         char* str = LOG_ComposeMessage(mess, s_LogFormatFlags);
+        FILE* log = logdata->fp;
         if (str) {
             size_t len = strlen(str);
             str[len++] = '\n';
-            fwrite(str, len, 1, logdata->fp);
-            fflush(logdata->fp);
+            fwrite(str, len, 1, log);
             free(str);
+        } else {
+            fputs(kOutOfMemory, log);
+            fputc('\n', log);
         }
+        fflush(log);
         if (mess->level >= logdata->fatal_err) {
 #ifdef NDEBUG
             fflush(0);
@@ -655,7 +656,7 @@ extern void LOG_ToFILE_Ex
     if (logdata) {
         logdata->fp         = fp;
         logdata->cut_off    = cut_off;
-        logdata->fatal_err  = fatal_err;
+        logdata->fatal_err  = fatal_err < eLOG_Fatal ? fatal_err : eLOG_Fatal;
         logdata->auto_close = auto_close;
         LOG_Reset(lg, logdata, s_LOG_FileHandler, s_LOG_FileCleanup);
     } else {
@@ -713,6 +714,21 @@ extern const char* CORE_GetAppName(void)
 
 
 
+/******************************************************************************
+ *  CORE_GetPlatform
+ */
+
+extern const char* CORE_GetPlatform(void)
+{
+#ifndef NCBI_CXX_TOOLKIT
+    return Nlm_PlatformName();
+#else
+    return HOST;
+#endif /*NCBI_CXX_TOOLKIT*/
+}
+
+
+
 /******************************************************************************
  *  CORE_GetNcbiRequestID
  */
@@ -741,6 +757,12 @@ extern char* CORE_GetNcbiRequestID(ENcbiRequestID reqid)
             break;
         id = x_getenv("NCBI_LOG_HIT_ID");
         break;
+    case eNcbiTraceState:
+        id = x_getenv("HTTP_TRACESTATE");
+        break;
+    case eNcbiTraceParent:
+        id = x_getenv("HTTP_TRACEPARENT");
+        break;
     default:
         id = 0;
         goto out;
@@ -754,21 +776,6 @@ extern char* CORE_GetNcbiRequestID(ENcbiRequestID reqid)
 
 
 
-/******************************************************************************
- *  CORE_GetPlatform
- */
-
-extern const char* CORE_GetPlatform(void)
-{
-#ifndef NCBI_CXX_TOOLKIT
-    return Nlm_PlatformName();
-#else
-    return HOST;
-#endif /*NCBI_CXX_TOOLKIT*/
-}
-
-
-
 /****************************************************************************
  * CORE_GetUsername
  */
@@ -1416,6 +1423,35 @@ extern int/*bool*/ UTIL_HelpRequested(int argc, char** argv)
 }
 
 
+#if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_CYGWIN)
+#  define timezone  _timezone
+#endif /*NCBI_OS_MSWIN || NCBI_OS_CYGWIN*/
+
+extern time_t UTIL_Timezone(void)
+{
+#if defined(NCBI_OS_DARWIN)  ||  defined(NCBI_OS_BSD)
+    static time_t timezone = (time_t)(-1);
+
+    /* NB: timezone information is unavailable on Darwin or BSD :-/ */
+    if (timezone == (time_t)(-1)) {
+        time_t now = time(0);
+        struct tm tm;
+#  ifdef HAVE_LOCALTIME_R
+        gmtime_r(&now, &tm);
+#  else
+        CORE_LOCK_WRITE;
+        tm = *gmtime(&now);
+        CORE_UNLOCK;
+#  endif /*HAVE_LOCALTIME_R*/
+        assert(tm.tm_isdst == 0);
+        timezone  = mktime(&tm) - now;
+    }
+#endif /*NCBI_OS_DARWIN || NCBI_OS_BSD*/
+
+    return timezone;
+}
+
+
 #ifdef NCBI_OS_MSWIN
 
 
