@@ -1,4 +1,4 @@
-/*  $Id: grid_worker.cpp 638391 2021-09-29 18:15:12Z sadyrovr $
+/*  $Id: grid_worker.cpp 672203 2023-08-31 19:21:12Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -592,7 +592,7 @@ int SGridWorkerNodeImpl::Run(
 
     ITERATE(vector<string>, it, vhosts) {
         if (auto address = SSocketAddress::Parse(NStr::TruncateSpaces(*it))) {
-            m_Masters.insert(move(address));
+            m_Masters.insert(std::move(address));
         }
     }
 
