@@ -1,4 +1,4 @@
-/*  $Id: netservice_params.cpp 620111 2020-11-16 19:50:00Z sadyrovr $
+/*  $Id: netservice_params.cpp 672203 2023-08-31 19:21:12Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,6 +51,7 @@ NCBI_PARAM_DEF(int, netservice_api, max_find_lbname_retries, 3);
 NCBI_PARAM_DEF(string, netcache_api, fallback_server, "");
 NCBI_PARAM_DEF(int, netservice_api, max_connection_pool_size, 0); // unlimited
 NCBI_PARAM_DEF(bool, netservice_api, connection_data_logging, false);
+NCBI_PARAM_DEF(bool, netservice_api, error_on_unexpected_reply, false);
 NCBI_PARAM_DEF(bool, netservice_api, warn_on_unexpected_reply, false);
 NCBI_PARAM_DEF(unsigned, server, max_wait_for_servers, 24 * 60 * 60);
 NCBI_PARAM_DEF(bool, server, stop_on_job_errors, true);
@@ -83,7 +84,7 @@ CConfig* CConfigRegistry::GetSubConfig(const string& section) const
     if (const CConfig::TParamTree* tree = m_Config->GetTree()) {
         if (const CConfig::TParamTree* sub_tree = tree->FindSubNode(section)) {
             unique_ptr<CConfig> sub_config(new CConfig(sub_tree));
-            auto result = m_SubConfigs.emplace(section, move(sub_config));
+            auto result = m_SubConfigs.emplace(section, std::move(sub_config));
             return result.first->second.get();
         }
     }
@@ -291,7 +292,7 @@ void CSynRegistry::CAlert::Report(ostream& os) const
 {
     lock_guard<mutex> lock(m_Mutex);
     for (const auto& alert : m_Alerts) {
-        os << "Alert_" << alert.first << ": \"" << alert.second << '"' << endl;
+        os << "Alert_" << alert.first << ": " << alert.second << endl;
     }
 }
 
@@ -367,7 +368,7 @@ TType CSynRegistry::TGet(const SRegSynonyms& sections, SRegSynonyms names, TType
                 }
 
                 ERR_POST(Warning << msg);
-                m_Alert->Set(NStr::JsonEncode(msg));
+                m_Alert->Set(NStr::JsonEncode(msg, NStr::eJsonEnc_Quoted));
             }
         }
     }
