@@ -1,4 +1,4 @@
-/*  $Id: wn_main_loop.cpp 606785 2020-04-27 16:12:51Z sadyrovr $
+/*  $Id: wn_main_loop.cpp 637657 2021-09-15 19:17:14Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -131,7 +131,7 @@ SWorkerNodeJobContextImpl::SWorkerNodeJobContextImpl(
     m_ProgressMsgThrottler(1),
     m_NetScheduleExecutor(worker_node->m_NSExecutor),
     m_NetCacheAPI(worker_node->m_NetCacheAPI),
-    m_JobGeneration(worker_node->m_CurrentJobGeneration),
+    m_JobGeneration(0),
     m_CommitExpiration(0, 0),
     m_Deadline(0, 0)
 {
@@ -234,22 +234,23 @@ void SWorkerNodeJobContextImpl::PutProgressMessage(const string& msg,
     }
 
     try {
-        if (m_Job.progress_msg.empty()) {
+        if (!overwrite) {
+            // Cached progress_msg (blob ID) might be outdated, refreshing it
             m_NetScheduleExecutor.GetProgressMsg(m_Job);
 
-            if (m_Job.progress_msg.empty()) {
-                m_Job.progress_msg =
-                        m_NetCacheAPI.PutData(msg.data(), msg.length());
-
-                m_NetScheduleExecutor.PutProgressMsg(m_Job);
-
+            if (!m_Job.progress_msg.empty()) {
                 return;
             }
         }
 
-        if (overwrite) {
+        if (CNetCacheKey::IsValidKey(m_Job.progress_msg, m_NetCacheAPI.GetCompoundIDPool())) {
             m_NetCacheAPI.PutData(m_Job.progress_msg, msg.data(), msg.length());
+        } else {
+            m_Job.progress_msg = m_NetCacheAPI.PutData(msg.data(), msg.length());
         }
+
+        // Cached progress_msg (blob ID) might differ from the actual, syncing them
+        m_NetScheduleExecutor.PutProgressMsg(m_Job);
     }
     catch (exception& ex) {
         ERR_POST_X(6, "Couldn't send a progress message: " << ex.what());
@@ -293,11 +294,11 @@ CNetScheduleAdmin::EShutdownLevel SWorkerNodeJobContextImpl::GetShutdownLevel()
             switch (job_status) {
             case CNetScheduleAPI::eRunning:
                 if (pause_mode == eNSQ_WithPullback) {
-                    m_WorkerNode->SetJobPullbackTimer(
-                            m_WorkerNode->m_DefaultPullbackTimeout);
+                    auto default_timeout = m_WorkerNode->m_SuspendResume.GetLock()->GetDefaultPullbackTimeout();
+                    m_WorkerNode->m_SuspendResume.GetLock()->SetJobPullbackTimer(default_timeout);
                     LOG_POST("Pullback request from the server, "
                             "(default) pullback timeout=" <<
-                            m_WorkerNode->m_DefaultPullbackTimeout);
+                            default_timeout);
                 }
                 /* FALL THROUGH */
 
@@ -324,7 +325,8 @@ CNetScheduleAdmin::EShutdownLevel SWorkerNodeJobContextImpl::GetShutdownLevel()
                     ": " << ex.what());
         }
 
-    if (m_WorkerNode->CheckForPullback(m_JobGeneration)) {
+    if ((m_JobGeneration != m_WorkerNode->m_SuspendResume->GetCurrentJobGeneration()) &&
+                m_WorkerNode->m_SuspendResume.GetLock()->IsJobPullbackTimerExpired()) {
         LOG_POST("Pullback timeout for " << m_Job.job_id);
         return CNetScheduleAdmin::eShutdownImmediate;
     }
@@ -351,6 +353,7 @@ void SWorkerNodeJobContextImpl::ResetJobContext()
             (m_Job.mask & CNetScheduleAPI::eExclusiveJob) != 0;
 
     m_RequestContext->Reset();
+    m_JobGeneration = m_WorkerNode->m_SuspendResume->GetCurrentJobGeneration();
 }
 
 void CWorkerNodeJobContext::RequestExclusiveMode()
@@ -642,37 +645,24 @@ void* CMainLoopThread::Main()
 
 CNetScheduleGetJob::EState CMainLoopThread::CImpl::CheckState()
 {
-    if (CGridGlobals::GetInstance().IsShuttingDown())
-        return eStopped;
-
     EState ret = eWorking;
 
-    for (;;) {
-        void* event;
-
-        while ((event = SwapPointers(&m_WorkerNode->m_SuspendResumeEvent,
-                NO_EVENT)) != NO_EVENT) {
-            if (event == SUSPEND_EVENT) {
-                if (!m_WorkerNode->m_TimelineIsSuspended) {
-                    // Stop the timeline.
-                    m_WorkerNode->m_TimelineIsSuspended = true;
-                    ret = eRestarted;
-                }
-            } else { /* event == RESUME_EVENT */
-                if (m_WorkerNode->m_TimelineIsSuspended) {
-                    // Resume the timeline.
-                    m_WorkerNode->m_TimelineIsSuspended = false;
-                }
-            }
-        }
-
-        if (!m_WorkerNode->m_TimelineIsSuspended) {
-            return ret;
+    while (!CGridGlobals::GetInstance().IsShuttingDown()) {
+        switch (m_WorkerNode->m_SuspendResume->CheckState()) {
+            case SSuspendResume::eRunning:
+                return ret;
+            case SSuspendResume::eSuspending:
+                ret = eRestarted;
+                break;
+            case SSuspendResume::eSuspended:
+                break;
         }
 
         m_WorkerNode->m_NSExecutor->
             m_NotificationHandler.WaitForNotification(m_Timeout);
     }
+
+    return eStopped;
 }
 
 CNetServer CMainLoopThread::CImpl::ReadNotifications()
