@@ -1,4 +1,4 @@
-/*  $Id: ncbi_param.cpp 606111 2020-04-20 13:34:35Z grichenk $
+/*  $Id: ncbi_param.cpp 651600 2022-06-21 13:53:08Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -118,7 +118,7 @@ namespace {
     {
         return s_ConfigDumpDisabled.Get() == 0
             && s_ConfigDump
-            && CDiagContextThreadData::IsInitialized();
+            && CDiagContext::IsMainThreadDataInitialized();
     }
 #endif
 }
@@ -176,7 +176,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
 
     if ( section  &&  *section ) {
         CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
-        if ( app  &&  app->HasLoadedConfig() ) {
+        if ( app  &&  app->FinishedLoadingConfig() ) {
             const string& s = app->GetConfig().Get(section, variable);
             if ( !s.empty() ) {
                 try {
@@ -264,7 +264,7 @@ int NCBI_XNCBI_EXPORT g_GetConfigInt(const char* section,
 
     if ( section  &&  *section ) {
         CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
-        if ( app  &&  app->HasLoadedConfig() ) {
+        if ( app  &&  app->FinishedLoadingConfig() ) {
             const string& s = app->GetConfig().Get(section, variable);
             if ( !s.empty() ) {
                 try {
@@ -348,7 +348,7 @@ double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
 
     if ( section  &&  *section ) {
         CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
-        if ( app  &&  app->HasLoadedConfig() ) {
+        if ( app  &&  app->FinishedLoadingConfig() ) {
             const string& s = app->GetConfig().Get(section, variable);
             if ( !s.empty() ) {
                 try {
@@ -428,7 +428,7 @@ string NCBI_XNCBI_EXPORT g_GetConfigString(const char* section,
 
     if ( section  &&  *section ) {
         CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
-        if ( app  &&  app->HasLoadedConfig() ) {
+        if ( app  &&  app->FinishedLoadingConfig() ) {
             const string& v = app->GetConfig().Get(section, variable);
             if ( !v.empty() ) {
 #ifdef NCBI_PARAM_ENABLE_CONFIG_DUMP
