@@ -1,4 +1,4 @@
-/* $Id: ncbi_process.cpp 579349 2019-02-01 16:07:58Z ivanov $
+/* $Id: ncbi_process.cpp 664691 2023-03-20 13:48:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -85,7 +85,7 @@ BEGIN_NCBI_SCOPE
 
 const unsigned long CProcessBase::kDefaultKillTimeout = 1000;
 const unsigned long CProcessBase::kInfiniteTimeoutMs  = kMax_ULong;
-const unsigned long               kWaitPrecision      = 100;
+const unsigned long               kWaitPrecisionMs    = 100;
 
 
 
@@ -304,25 +304,46 @@ static string s_ErrnoToString()
 }
 #endif
 
+
 TPid CCurrentProcess::Fork(CProcess::TForkFlags flags)
 {
 #ifdef NCBI_OS_UNIX
+
+    // Issue a warning if not a single thread is used that
+    // can lead to a not-async safe calls (and potential problems)
+    bool use_async_safe = CCurrentProcess::GetThreadCount() > 1;
+    if (!(flags & fFF_Exec)  &&  use_async_safe) {
+        ERR_POST_X(3, Warning << "It is not safe to call Fork() from a multithreaded program");
+    }
     TPid pid = ::fork();
-    if (pid == 0)
-        // Only update PID and UID in the child process.
-        CDiagContext::UpdateOnFork((flags & fFF_UpdateDiag) != 0 ?
-                                   CDiagContext::fOnFork_ResetTimer | CDiagContext::fOnFork_PrintStart : 0);
+    if (pid == 0) {
+        // child process:
+        // Update Diag API after fork, except fFF_Exec cases
+        if (!(flags & fFF_Exec)) {
+            CDiagContext::TOnForkFlags f = 0;
+            if (flags & fFF_UpdateDiag) {
+                f |= (CDiagContext::fOnFork_ResetTimer | CDiagContext::fOnFork_PrintStart);
+            }
+            if (use_async_safe) {
+                f |= CDiagContext::fOnFork_AsyncSafe;
+            }
+            // Update PID and UID in the child process
+            CDiagContext::UpdateOnFork(f);
+        }
+    }
     else if (pid == (TPid)(-1) && (flags & fFF_AllowExceptions) != 0) {
         NCBI_THROW_FMT(CCoreException, eCore,
                        "CCurrentProcess::Fork(): Cannot fork: " << s_ErrnoToString());
     }
     return pid;
+
 #else
     NCBI_THROW(CCoreException, eCore,
                "CCurrentProcess::Fork() not implemented on this platform");
 #endif
 }
 
+
 #ifdef NCBI_OS_UNIX
 namespace {
     class CSafeRedirect
@@ -619,7 +640,7 @@ bool CProcess::Kill(unsigned long timeout)
     // Check process termination within the timeout
     unsigned long x_timeout = timeout;
     for (;;) {
-        TPid reap = waitpid(pid, static_cast<int*>(NULL), WNOHANG);
+        TPid reap = waitpid(pid, 0, WNOHANG);
         if (reap) {
             if (reap != (TPid)(-1)) {
                 _ASSERT(reap == pid);
@@ -632,7 +653,7 @@ bool CProcess::Kill(unsigned long timeout)
             if (kill(pid, 0) < 0)
                 return true;
         }
-        unsigned long x_sleep = kWaitPrecision;
+        unsigned long x_sleep = kWaitPrecisionMs;
         if (x_sleep > x_timeout) {
             x_sleep = x_timeout;
         }
@@ -649,9 +670,9 @@ bool CProcess::Kill(unsigned long timeout)
     if ( !timeout ) {
         return res <= 0;
     }
-    SleepMilliSec(kWaitPrecision);
+    SleepMilliSec(kWaitPrecisionMs);
     // Reap the zombie (if child) up from the system
-    waitpid(pid, static_cast<int*>(NULL), WNOHANG);
+    waitpid(pid, 0, WNOHANG);
 
     // Check whether the process cannot be killed
     // (most likely due to a kernel problem)
@@ -808,7 +829,7 @@ bool CProcess::Kill(unsigned long timeout)
                 terminated = true;
                 break;
             }
-            unsigned long x_sleep = kWaitPrecision;
+            unsigned long x_sleep = kWaitPrecisionMs;
             if (x_sleep > linger_timeout) {
                 x_sleep = linger_timeout;
             }
@@ -941,7 +962,7 @@ bool CProcess::KillGroupById(TPid pgid, unsigned long timeout)
     unsigned long x_timeout = timeout;
     for (;;) {
         // Reap the zombie (if group leader is a child) up from the system
-        TPid reap = waitpid(pgid, static_cast<int*>(NULL), WNOHANG);
+        TPid reap = waitpid(pgid, 0, WNOHANG);
         if (reap) {
             if (reap != (TPid)(-1)) {
                 _ASSERT(reap == pgid);
@@ -955,7 +976,7 @@ bool CProcess::KillGroupById(TPid pgid, unsigned long timeout)
                 return true;
             }
         }
-        unsigned long x_sleep = kWaitPrecision;
+        unsigned long x_sleep = kWaitPrecisionMs;
         if (x_sleep > x_timeout) {
             x_sleep = x_timeout;
         }
@@ -972,9 +993,9 @@ bool CProcess::KillGroupById(TPid pgid, unsigned long timeout)
     if ( !timeout ) {
         return res <= 0;
     }
-    SleepMilliSec(kWaitPrecision);
+    SleepMilliSec(kWaitPrecisionMs);
     // Reap the zombie (if group leader is a child) up from the system
-    waitpid(pgid, static_cast<int*>(NULL), WNOHANG);
+    waitpid(pgid, 0, WNOHANG);
     // Check whether the process cannot be killed
     // (most likely due to a kernel problem)
     return kill(-pgid, 0) < 0;
@@ -1024,7 +1045,7 @@ int CProcess::Wait(unsigned long timeout, CExitInfo* info) const
                 }
                 break;
             }
-            unsigned long x_sleep = kWaitPrecision;
+            unsigned long x_sleep = kWaitPrecisionMs;
             if (x_sleep > timeout) {
                 x_sleep = timeout;
             }
@@ -1059,7 +1080,8 @@ int CProcess::Wait(unsigned long timeout, CExitInfo* info) const
     if (::GetExitCodeProcess(hProcess, &x_status)) {
         if (x_status == STILL_ACTIVE) {
             if (allow_wait  &&  timeout) {
-                DWORD tv = (timeout == kInfiniteTimeoutMs ? INFINITE : (DWORD)timeout);
+                DWORD tv = (timeout == kInfiniteTimeoutMs
+                            ? INFINITE : (DWORD) timeout);
                 DWORD ws = ::WaitForSingleObject(hProcess, tv);
                 switch(ws) {
                 case WAIT_TIMEOUT:
