@@ -1,4 +1,4 @@
-/*  $Id: ncbi_stack.cpp 613327 2020-08-05 15:07:15Z grichenk $
+/*  $Id: ncbi_stack.cpp 671746 2023-08-23 15:14:23Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,7 +38,7 @@
 #  include <sys/ucontext.h> // for additional test below
 #endif
 
-#if defined(HAVE_LIBBACKWARD_CPP) \
+#if defined(HAVE_BACKWARD_HPP) \
     &&  (defined(HAVE_LIBDW) || defined(NCBI_OS_DARWIN))
 #  include "ncbi_stack_libbackward.cpp"
 #elif defined HAVE_LIBUNWIND
