@@ -1,4 +1,4 @@
-/*  $Id: ncbi_url.cpp 658652 2022-11-04 17:10:53Z sadyrovr $
+/*  $Id: ncbi_url.cpp 684243 2024-06-17 13:59:53Z satskyse $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -435,9 +435,21 @@ void CUrl::SetUrl(const string& orig_url, const IUrlEncoder* encoder)
 
     // Parse scheme and authority (user info, host, port).
     SIZE_TYPE pos = orig_url.find("//");
+
+    // The // can be in parameters part, so check that it is before the ?
+    bool    slashes_are_in_args = false;
+    if (pos != NPOS) {
+        SIZE_TYPE   question_mark_pos = orig_url.find("?");
+        if (question_mark_pos != NPOS) {
+            if (question_mark_pos < pos) {
+                slashes_are_in_args = true;
+            }
+        }
+    }
+
     SIZE_TYPE offset = 0;
     string authority;
-    if (pos != NPOS) {
+    if (pos != NPOS && slashes_are_in_args == false) {
         m_IsGeneric = true;
         // Host is present, split into scheme/host/path.
         if (pos > 0 && orig_url[pos - 1] == ':') {
@@ -573,16 +585,16 @@ string CUrl::ComposeUrl(CUrlArgs::EAmpEncoding amp_enc,
     }
     string url;
 
-    // Host or service name only require some special processing.
-    bool host_only =  (!m_Host.empty() || !m_Service.empty())
-        && m_Scheme.empty()
+    // Service- or host- name only require some special processing.
+    bool host_only = (IsService() || !m_Host.empty())
+        &&  m_Scheme.empty()
         && !m_IsGeneric
-        && m_User.empty()
-        && m_Password.empty()
-        && m_Port.empty()
-        && m_Path.empty()
-        && m_Fragment.empty() &&
-        !HaveArgs();
+        &&  m_User.empty()
+        &&  m_Password.empty()
+        &&  m_Port.empty()
+        &&  m_Path.empty()
+        &&  m_Fragment.empty()
+        && !HaveArgs();
     if (host_only && IsService()) {
         // Do not add ncbilb:// if only service name is set.
         return NStr::URLEncode(m_Service, NStr::eUrlEnc_ProcessMarkChars);
@@ -616,7 +628,7 @@ string CUrl::ComposeUrl(CUrlArgs::EAmpEncoding amp_enc,
     if ( have_user_info ) {
         url += "@";
     }
-    if ( !m_Service.empty() ) {
+    if ( IsService() ) {
         url += NStr::URLEncode(m_Service, NStr::eUrlEnc_ProcessMarkChars);
     }
     else if ( !m_Host.empty() ) {
@@ -803,7 +815,7 @@ IUrlEncoder* CUrl::GetDefaultEncoder(void)
 bool CUrl::IsEmpty(void) const
 {
     // At least one of host, service or path must be present.
-    return m_Host.empty()  &&  m_Service.empty()  &&  m_Path.empty();
+    return m_Host.empty()  &&  !IsService()  &&  m_Path.empty();
 }
 
 
