@@ -1,4 +1,4 @@
-/*  $Id: ncbiapp.cpp 671295 2023-08-11 12:10:38Z ivanov $
+/*  $Id: ncbiapp.cpp 685024 2024-07-10 13:26:23Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -195,15 +195,35 @@ CNcbiApplicationAPI::CNcbiApplicationAPI(const SBuildInfo& build_info)
     // Create an empty registry
     m_Config.Reset(new CNcbiRegistry);
 
+    // Phone Home Policy
+    m_PhoneHomePolicy = nullptr;
+    m_PhoneHomePolicy_Ownership = eNoOwnership;
+
     m_DryRun = false;
 }
 
+
 void CNcbiApplicationAPI::ExecuteOnExitActions()
 {
     m_OnExitActions.ExecuteActions();
 }
 
 
+void CNcbiApplicationAPI::SetPhoneHomePolicy(IPhoneHomePolicy* policy, ENcbiOwnership ownership)
+{ 
+    if (m_PhoneHomePolicy) {
+        m_PhoneHomePolicy->Finish();
+    }
+    if (m_PhoneHomePolicy  &&  m_PhoneHomePolicy != policy  &&  m_PhoneHomePolicy_Ownership==eTakeOwnership) {
+        delete m_PhoneHomePolicy;
+    }
+    m_PhoneHomePolicy = policy;
+    if (m_PhoneHomePolicy) {
+        m_PhoneHomePolicy->Apply(this);
+    }
+};
+
+
 CNcbiApplicationAPI::~CNcbiApplicationAPI(void)
 {
     CThread::sm_IsExiting = true;
@@ -689,12 +709,18 @@ void CNcbiApplicationAPI::x_TryMain(EAppDiagStream diag,
                 NCBI_RETHROW_SAME(e, "Application's execution failed");
             }
             catch (const CException& e) {
-                CRef<CRequestContext> cur_ctx(&GetDiagContext().GetRequestContext());
-                if (cur_ctx != &e.GetRequestContext()) {
-                    GetDiagContext().SetRequestContext(&e.GetRequestContext());
+                CRef<CRequestContext> cur_ctx(&CDiagContext::GetRequestContext());
+                CRef<CRequestContext> e_ctx(&e.GetRequestContext());
+                if (cur_ctx != e_ctx) {
+                    CDiagContext::SetRequestContext(e_ctx);
                     NCBI_REPORT_EXCEPTION_X(16,
                                             "CException thrown", e);
-                    GetDiagContext().SetRequestContext(cur_ctx);
+                    auto e_span = e_ctx->GetTracerSpan();
+                    if ( e_span ) {
+                        e_span->SetSpanStatus(ITracerSpan::eError);
+                        e_span->EndSpan();
+                    }
+                    CDiagContext::SetRequestContext(cur_ctx);
                 }
                 NCBI_REPORT_EXCEPTION_X(16,
                                         "Application's execution failed", e);
@@ -711,6 +737,13 @@ void CNcbiApplicationAPI::x_TryMain(EAppDiagStream diag,
             *exit_code = m_DryRun ? DryRun() : Run();
         }
     }
+    auto span = CDiagContext::GetRequestContext().GetTracerSpan();
+    if ( span ) {
+        if (*exit_code != 0 || *got_exception) {
+            span->SetSpanStatus(ITracerSpan::eError);
+        }
+        span->EndSpan();
+    }
     x_LogOptions(eStopEvent);
     GetDiagContext().SetGlobalAppState(eDiagAppState_AppEnd);
 
@@ -854,7 +887,7 @@ int CNcbiApplicationAPI::AppMain
     if (appname.empty()) {
         if (!exepath.empty()) {
             CDirEntry::SplitPath(exepath, NULL, &appname);
-        } else if (argc > 0  &&  argv[0] != NULL  &&  *argv[0] != '\0') {
+        } else if (argc > 0  &&  argv  &&  argv[0] != NULL  &&  *argv[0] != '\0') {
             CDirEntry::SplitPath(argv[0], NULL, &appname);
         } else {
             appname = "ncbi";
