@@ -1,4 +1,4 @@
-/*  $Id: ncbiobj.cpp 580272 2019-02-12 17:41:00Z vasilche $
+/*  $Id: ncbiobj.cpp 667211 2023-05-10 17:44:44Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -184,17 +184,63 @@ typedef vector<TLastNewPtrMultipleInfo> TLastNewPtrMultiple;
 #ifdef NCBI_NO_THREADS
 static TLastNewPtrMultiple* s_LastNewPtrMultiple_ptr;
 #else
-static TTlsKey s_LastNewPtrMultiple_key;
-#endif
+static atomic<TTlsKey> s_LastNewPtrMultiple_key;
 
-#ifdef NCBI_POSIX_THREADS
-static
-void sx_EraseLastNewPtrMultiple(void* ptr)
+static inline
+TTlsKey sx_GetLastNewPtrMultipleCurrentKey(void)
 {
-    delete (TLastNewPtrMultiple*)ptr;
+    return s_LastNewPtrMultiple_key.load(memory_order_relaxed);
+}
+
+struct SEraseLastNewPtrMultiple {
+    static void sx_Cleanup(void* ptr)
+    {
+        TLastNewPtrMultiple* set = static_cast<TLastNewPtrMultiple*>(ptr);
+        auto key = sx_GetLastNewPtrMultipleCurrentKey();
+        delete set;
+#ifdef NCBI_WIN32_THREADS
+        TlsSetValue(key, nullptr);
+#else
+        pthread_setspecific(key, nullptr);
+#endif
+    }
+    ~SEraseLastNewPtrMultiple() {
+#ifdef NCBI_WIN32_THREADS
+        auto key = sx_GetLastNewPtrMultipleCurrentKey();
+        TLastNewPtrMultiple* set = (TLastNewPtrMultiple*)TlsGetValue(key);
+        delete set;
+        TlsSetValue(key, nullptr);
+#endif
+    }
+};
+
+static inline
+TTlsKey sx_GetLastNewPtrMultipleKey(void)
+{
+    auto key = sx_GetLastNewPtrMultipleCurrentKey();
+    if ( !key ) {
+        DEFINE_STATIC_FAST_MUTEX(s_InitMutex);
+        NCBI_NS_NCBI::CFastMutexGuard guard(s_InitMutex);
+        key = s_LastNewPtrMultiple_key.load(memory_order_acquire);
+        if ( !key ) {
+            do {
+#  ifdef NCBI_WIN32_THREADS
+                _VERIFY((key = TlsAlloc()) != DWORD(-1));
+#  else
+                _VERIFY(pthread_key_create(&key, SEraseLastNewPtrMultiple::sx_Cleanup)==0);
+#  endif
+            } while ( !key );
+#  ifndef NCBI_WIN32_THREADS
+            pthread_setspecific(key, 0);
+#  endif
+            s_LastNewPtrMultiple_key.store(key, memory_order_release);
+        }
+    }
+    return key;
 }
 #endif
 
+
 static
 TLastNewPtrMultiple& sx_GetLastNewPtrMultiple(void)
 {
@@ -205,36 +251,20 @@ TLastNewPtrMultiple& sx_GetLastNewPtrMultiple(void)
     }
     return *set;
 #else
-    if ( !s_LastNewPtrMultiple_key ) {
-        DEFINE_STATIC_FAST_MUTEX(s_InitMutex);
-        NCBI_NS_NCBI::CFastMutexGuard guard(s_InitMutex);
-        if ( !s_LastNewPtrMultiple_key ) {
-            TTlsKey key = 0;
-            do {
-#  ifdef NCBI_WIN32_THREADS
-                _VERIFY((key = TlsAlloc()) != DWORD(-1));
-#  else
-                _VERIFY(pthread_key_create(&key, sx_EraseLastNewPtrMultiple)==0);
-#  endif
-            } while ( !key );
-#  ifndef NCBI_WIN32_THREADS
-            pthread_setspecific(key, 0);
-#  endif
-            s_LastNewPtrMultiple_key = key;
-        }
-    }
+    auto key = sx_GetLastNewPtrMultipleKey();
     TLastNewPtrMultiple* set;
 #  ifdef NCBI_WIN32_THREADS
-    set = (TLastNewPtrMultiple*)TlsGetValue(s_LastNewPtrMultiple_key);
+    set = (TLastNewPtrMultiple*)TlsGetValue(key);
 #  else
-    set = (TLastNewPtrMultiple*)pthread_getspecific(s_LastNewPtrMultiple_key);
+    set = (TLastNewPtrMultiple*)pthread_getspecific(key);
 #  endif
     if ( !set ) {
         set = new TLastNewPtrMultiple();
 #  ifdef NCBI_WIN32_THREADS
-        TlsSetValue(s_LastNewPtrMultiple_key, set);
+        TlsSetValue(key, set);
+        static thread_local SEraseLastNewPtrMultiple s_Cleanup;
 #  else
-        pthread_setspecific(s_LastNewPtrMultiple_key, set);
+        pthread_setspecific(key, set);
 #  endif
     }
     return *set;
@@ -467,7 +497,7 @@ void* CObject::operator new(size_t size)
     GetSecondCounter(static_cast<CObject*>(ptr))->Set(eMagicCounterNew);
 #  endif// USE_COMPLEX_MASK
 #endif// USE_HEAPOBJ_LIST
-    static_cast<CObject*>(ptr)->m_Counter.Set(eMagicCounterNew);
+    static_cast<CObject*>(ptr)->m_Counter = eMagicCounterNew;
 #endif// USE_TLS_PTR
     return ptr;
 #endif
@@ -482,7 +512,7 @@ void CObject::operator delete(void* ptr)
 # ifdef _DEBUG
     TCount magic = sx_PopLastNewPtr(ptr);
     if ( !magic ) { // counter already initialized
-        magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+        magic = static_cast<CObject*>(ptr)->m_Counter;
     }
 # else// !_DEBUG
     // Just remove saved operator new info.
@@ -490,7 +520,7 @@ void CObject::operator delete(void* ptr)
 # endif// _DEBUG
 #else// !USE_TLS_PTR
 # ifdef _DEBUG
-    TCount magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+    TCount magic = static_cast<CObject*>(ptr)->m_Counter;
 # endif// _DEBUG
 #endif// USE_TLS_PTR
 
@@ -519,7 +549,7 @@ void CObject::operator delete(void* _DEBUG_ARG(ptr), void* /*place*/)
 #if !USE_TLS_PTR
 # ifdef _DEBUG
     CObject* objectPtr = static_cast<CObject*>(ptr);
-    TCount magic = objectPtr->m_Counter.Get();
+    TCount magic = objectPtr->m_Counter;
     // magic can be equal to:
     // 1. eMagicCounterDeleted when memory is freed after CObject destructor.
     // 2. 0 when memory is freed before CObject constructor.
@@ -547,7 +577,7 @@ void* CObject::operator new(size_t size, CObjectMemoryPool* memory_pool)
 #  if USE_COMPLEX_MASK
     GetSecondCounter(static_cast<CObject*>(ptr))->Set(eMagicCounterPoolNew);
 #  endif// USE_COMPLEX_MASK
-    static_cast<CObject*>(ptr)->m_Counter.Set(eMagicCounterPoolNew);
+    static_cast<CObject*>(ptr)->m_Counter = eMagicCounterPoolNew;
 #endif// USE_TLS_PTR
     return ptr;
 }
@@ -561,7 +591,7 @@ void CObject::operator delete(void* ptr, CObjectMemoryPool* memory_pool)
 # ifdef _DEBUG
     TCount magic = sx_PopLastNewPtr(ptr);
     if ( !magic ) { // counter already initialized
-        magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+        magic = static_cast<CObject*>(ptr)->m_Counter;
     }
 # else// !_DEBUG
     // Just remove saved operator new info.
@@ -569,7 +599,7 @@ void CObject::operator delete(void* ptr, CObjectMemoryPool* memory_pool)
 # endif// _DEBUG
 #else// !USE_TLS_PTR
 # ifdef _DEBUG
-    TCount magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+    TCount magic = static_cast<CObject*>(ptr)->m_Counter;
 # endif//_DEBUG
 #endif// USE_TLS_PTR
 
@@ -620,11 +650,11 @@ void CObject::InitCounter(void)
         switch ( type ) {
         case eMagicCounterNew:
             // allocated in heap
-            m_Counter.Set(eInitCounterInHeap);
+            m_Counter = eInitCounterInHeap;
             break;
         case eMagicCounterPoolNew:
             // allocated in memory pool
-            m_Counter.Set(eInitCounterInPool);
+            m_Counter = eInitCounterInPool;
             break;
         default:
             ERR_POST_X(1, ObjFatal << "CObject::InitCounter: "
@@ -632,23 +662,23 @@ void CObject::InitCounter(void)
                        " at "<<StackTrace);
             // something is broken in TLS data
             // mark as not in heap
-            m_Counter.Set(eInitCounterNotInHeap);
+            m_Counter = eInitCounterNotInHeap;
             break;
         }
     }
     else {
         // surely not in heap
-        m_Counter.Set(eInitCounterNotInHeap);
+        m_Counter = eInitCounterNotInHeap;
     }
 #else
     // This code can't use Get(), which may block waiting for an
     // update that will never happen.
     // ATTENTION:  this code can cause UMR (Uninit Mem Read) -- it's okay here!
-    TCount main_counter = m_Counter.m_Value;
+    TCount main_counter = m_Counter;
     if ( main_counter != eMagicCounterNew &&
          main_counter != eMagicCounterPoolNew ) {
         // takes care of statically allocated case
-        m_Counter.Set(eInitCounterNotInHeap);
+        m_Counter = eInitCounterNotInHeap;
     }
     else {
         bool inStack = false;
@@ -690,15 +720,15 @@ void CObject::InitCounter(void)
         
         if ( inStack ) {
             // surely not in heap
-            m_Counter.Set(eInitCounterInStack);
+            m_Counter = eInitCounterInStack;
         }
         else if ( main_counter == eMagicCounterNew ) {
             // allocated in heap
-            m_Counter.Set(eInitCounterInHeap);
+            m_Counter = eInitCounterInHeap;
         }
         else {
             // allocated in memory pool
-            m_Counter.Set(eInitCounterInPool);
+            m_Counter = eInitCounterInPool;
         }
     }
 #endif
@@ -719,7 +749,7 @@ CObject::CObject(const CObject& /*src*/)
 
 CObject::~CObject(void)
 {
-    TCount count = m_Counter.Get();
+    TCount count = m_Counter;
     if ( ObjectStateUnreferenced(count) ) {
         // reference counter is zero -> ok
     }
@@ -748,7 +778,7 @@ CObject::~CObject(void)
     else {
         final_magic = eMagicCounterDeleted;
     }
-    m_Counter.Set(final_magic);
+    m_Counter = final_magic;
 }
 
 
@@ -778,7 +808,7 @@ void CObject::CheckReferenceOverflow(TCount count) const
 
 void CObject::DeleteThis(void)
 {
-    TCount count = m_Counter.Get();
+    TCount count = m_Counter;
     // Counter could be changed by some other thread,
     // we should take care of that.
     if ( (count & eInitCounterInHeap) == eInitCounterInHeap ) {
@@ -809,7 +839,7 @@ void CObject::RemoveLastReference(TCount count) const
 
     // Error here
     // restore original value
-    count = m_Counter.Add(eCounterStep);
+    count = (m_Counter += eCounterStep);
     // bad object
     if ( ObjectStateValid(count) ) {
         ERR_POST_X(4, ObjFatal << "CObject::RemoveLastReference: "
@@ -829,11 +859,11 @@ void CObject::RemoveLastReference(TCount count) const
 
 void CObject::ReleaseReference(void) const
 {
-    TCount count = m_Counter.Add(-eCounterStep);
+    TCount count = (m_Counter -= eCounterStep);
     if ( ObjectStateValid(count) ) {
         return;
     }
-    m_Counter.Add(eCounterStep); // undo
+    m_Counter += eCounterStep; // undo
 
     // error
     if ( count == eMagicCounterDeleted ||
@@ -855,7 +885,7 @@ void CObject::DoNotDeleteThisObject(void)
 {
     TCount count;
 #if USE_TLS_PTR
-    count = m_Counter.Get();
+    count = m_Counter;
     if ( ObjectStateValid(count) ) {
         if ( ObjectStateCanBeDeleted(count) ) {
             NCBI_THROW(CObjectException, eHeapState,
@@ -868,11 +898,11 @@ void CObject::DoNotDeleteThisObject(void)
 #else
     {{
         CFastMutexGuard LOCK(sm_ObjectMutex);
-        count = m_Counter.Get();
+        count = m_Counter;
         if ( ObjectStateValid(count) ) {
             // valid and unreferenced
             // reset all 'in heap' flags -> make it non-heap without signature
-            m_Counter.Add(-int(count & eStateBitsInHeapMask));
+            m_Counter -= (count & eStateBitsInHeapMask);
             return;
         }
     }}
@@ -898,7 +928,7 @@ void CObject::DoDeleteThisObject(void)
 #ifndef USE_SINGLE_ALLOC
     TCount count;
 #if USE_TLS_PTR
-    count = m_Counter.Get();
+    count = m_Counter;
     if ( ObjectStateValid(count) ) {
         if ( !ObjectStateCanBeDeleted(count) ) {
             NCBI_THROW(CObjectException, eHeapState,
@@ -911,13 +941,13 @@ void CObject::DoDeleteThisObject(void)
 #else
     {{
         CFastMutexGuard LOCK(sm_ObjectMutex);
-        count = m_Counter.Get();
+        count = m_Counter;
         // DoDeleteThisObject is not allowed for stack objects
         static const TCount eCheckBits = eStateBitsValid | eStateBitsHeapSignature;
         if ( (count & eCheckBits) == eCheckBits ) {
             if ( !(count & eStateBitsInHeap) ) {
                 // set 'in heap' flag
-                m_Counter.Add(eStateBitsInHeap);
+                m_Counter += eStateBitsInHeap;
             }
             return;
         }
@@ -1237,10 +1267,10 @@ CWeakObject::~CWeakObject(void)
 inline
 bool CWeakObject::x_AddWeakReference(CObject* obj)
 {
-    CObject::TCount newCount = obj->m_Counter.Add(CObject::eCounterStep);
+    CObject::TCount newCount = obj->m_Counter += CObject::eCounterStep;
 
     if ( CObject::ObjectStateReferencedOnlyOnce(newCount) ) {
-        obj->m_Counter.Add(-CObject::eCounterStep);
+        obj->m_Counter -= CObject::eCounterStep;
         return false;
     }
     return true;
