@@ -1,4 +1,4 @@
-/*  $Id: request_ctx.cpp 664132 2023-03-08 14:20:02Z ivanov $
+/*  $Id: request_ctx.cpp 689783 2024-11-06 16:29:40Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -174,9 +174,12 @@ CRequestContext::CRequestContext(TContextFlags flags)
       m_IsRunning(false),
       m_AutoIncOnPost(false),
       m_Flags(flags),
+      m_DisabledAppLog(CDiagContext::GetDisabledAppLog()),
+      m_DisabledAppLogEvents(CDiagContext::GetDisabledAppLogEvents()),
       m_OwnerTID(-1),
       m_IsReadOnly(false),
-      m_Version(sm_VersionCounter.Add(1))
+      m_Version(sm_VersionCounter.Add(1)),
+      m_SpanKind(ITracerSpan::eKind_NotSet)
 {
     x_LoadEnvContextProperties();
 }
@@ -320,9 +323,24 @@ void CRequestContext::SetClientIP(const string& client)
     if (!x_CanModify()) return;
     x_SetProp(eProp_ClientIP);
 
-    // Verify IP
-    string ip = NStr::TruncateSpaces(client);
-    if ( !NStr::IsIPAddress(ip) ) {
+    // Verify IP (tolerating optional enclosing square brackets, [...])
+    CTempStringEx ip = NStr::TruncateSpaces_Unsafe(client);
+    size_t len = ip.size();
+    bool bad;
+    if (!len) {
+        bad = true;
+    } else if (ip[0] != '[') {
+        bad = false;
+    } else if (len < 3  ||  ip[len - 1] != ']') {
+        bad = true;
+    } else {
+        ip = ip.substr(1, len - 2);
+        bad = false;
+    }
+    if (!bad  &&  !NStr::IsIPAddress(ip)) {
+        bad = true;
+    }
+    if ( bad ) {
         m_ClientIP = kBadIP;
         x_Modify();
         ERR_POST_X(25, "Bad client IP value: " << ip);
@@ -346,6 +364,8 @@ void CRequestContext::StartRequest(void)
     GetRequestTimer().Restart();
     m_IsRunning = true;
     x_LogHitID();
+    // Save the tracer to make sure the same one is used to stop request.
+    m_Tracer = sm_Tracer;
     if (m_Tracer) {
         m_Tracer->OnRequestStart(*this);
     }
@@ -357,6 +377,7 @@ void CRequestContext::StopRequest(void)
     if (!x_CanModify()) return;
     if (m_Tracer) {
         m_Tracer->OnRequestStop(*this);
+        m_Tracer.reset();
     }
     if ((m_HitIDLoggedFlag & fLoggedOnRequest) == 0) {
         // Hit id has not been set or logged yet. Try to log the default one.
@@ -589,6 +610,8 @@ CRef<CRequestContext> CRequestContext::Clone(void) const
     ret->m_IsRunning = m_IsRunning;
     ret->m_AutoIncOnPost = m_AutoIncOnPost;
     ret->m_Flags = m_Flags;
+    ret->m_DisabledAppLog = m_DisabledAppLog;
+    ret->m_DisabledAppLogEvents = m_DisabledAppLogEvents;
     ret->m_IsReadOnly = m_IsReadOnly;
     // NOTE: Do not clone m_Version - the new context gets its own unique version.
     return ret;
@@ -763,6 +786,15 @@ void CRequestContext::x_UpdateStdContextProp(CTempString name) const
 }
 
 
+
+shared_ptr<IRequestTracer> CRequestContext::sm_Tracer;
+
+void CRequestContext::SetRequestTracer(const shared_ptr<IRequestTracer>& tracer)
+{
+    sm_Tracer = tracer;
+}
+
+
 NCBI_PARAM_DECL(string, Context, Fields);
 NCBI_PARAM_DEF_EX(string, Context, Fields, "",
                   eParam_NoThread,
