@@ -1,4 +1,4 @@
-/* $Id: test_mt.cpp 626913 2021-03-05 14:17:23Z grichenk $
+/* $Id: test_mt.cpp 690567 2024-11-25 15:21:13Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,7 @@
 #include <corelib/ncbimtx.hpp>
 #include <corelib/ncbi_system.hpp>
 #include <corelib/ncbi_param.hpp>
+#include <corelib/ncbi_test.hpp>
 #include <math.h>
 #include <common/test_assert.h>  /* This header must go last */
 
@@ -53,7 +54,7 @@ unsigned int  s_NumThreads = 34;
 int           s_SpawnBy    = 6;
 
 // Next test thread index
-static volatile unsigned int s_NextIndex = 0;
+static atomic<unsigned int> s_NextIndex = 0;
 
 #define TESTAPP_LOG_POST(x)  do { ++m_LogMsgCount; LOG_POST(x); } while (0)
 
@@ -534,8 +535,7 @@ int CThreadedApp::Run(void)
 
     unsigned int seed = GetArgs()["seed"]
         ? static_cast<unsigned int>(GetArgs()["seed"].AsInteger())
-        : (static_cast<unsigned int>(CCurrentProcess::GetPid()) ^
-           static_cast<unsigned int>(time(NULL)) % 1000000);
+        : CNcbiTest::GetRandomSeed();
     TESTAPP_LOG_POST("Randomization seed value: " << seed);
     srand(seed);
 
