@@ -1,4 +1,4 @@
-/*  $Id: version.cpp 657706 2022-10-14 15:58:58Z gouriano $
+/*  $Id: version.cpp 684652 2024-06-28 14:11:08Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -309,50 +309,60 @@ void ParseVersionString(const string&  vstr,
 
     // all other normal formats
 
-    const char* version_pattern = "version";
-
-    pos = lo_vstr.find(version_pattern);
-    if (pos == string::npos) {
-        version_pattern = "v.";
-        pos = lo_vstr.find(version_pattern);
-
-        if (pos == string::npos) {
-            version_pattern = "ver";
-            pos = lo_vstr.find(version_pattern);
-
-            if (pos == string::npos) {
-                version_pattern = "";
-                // find the first space-digit and assume it's version
-                const char* ch = vstr_str;
-                for (; *ch; ++ch) {
-                    if (isdigit((unsigned char)(*ch))) {
-                        if (ch == vstr_str) {
-                            // check if it's version
-                            const char* ch2 = ch + 1;
-                            for (;*ch2; ++ch2) {
-                                if (!isdigit((unsigned char)(*ch2))) {
-                                    break;
-                                }
-                            } // for
-                            if (*ch2 == '.') {
-                                pos = ch - vstr_str;
-                                break;
-                            } else {
-                                continue;
-                            }
-                        } else {
-                            if (isspace((unsigned char) ch[-1])) {
-                                pos = ch - vstr_str;
-                                break;
-                            }
-                        }
-                    } // if digit
-                    
-                } // for
+    static const char* version_patterns[] = {
+        "version",
+        "v.",
+        "ver",
+        "v",
+        nullptr
+    };
+    const char* version_pattern = kEmptyCStr;
+    for (const char** ppattern = version_patterns;  *ppattern != nullptr;
+         ++ppattern) {
+        pos = lo_vstr.rfind(*ppattern);
+        if (pos != NPOS) {
+            // sanity check
+            auto pos2
+                = lo_vstr.find_first_not_of(" .", pos + strlen(*ppattern));
+            if ( !isdigit(lo_vstr[pos2]) ) {
+                pos = NPOS;
+                continue;
             }
+            version_pattern = *ppattern;
+            break;
         }
     }
 
+    if (pos == string::npos) {
+        // find the first space-digit and assume it's version
+        const char* ch = vstr_str;
+        for (; *ch; ++ch) {
+            if (isdigit((unsigned char)(*ch))) {
+                if (ch == vstr_str) {
+                    // check if it's version
+                    const char* ch2 = ch + 1;
+                    for (;*ch2; ++ch2) {
+                        if (!isdigit((unsigned char)(*ch2))) {
+                            break;
+                        }
+                    } // for
+                    if (*ch2 == '.') {
+                        pos = ch - vstr_str;
+                        break;
+                    } else {
+                        continue;
+                    }
+                } else {
+                    if (isspace((unsigned char) ch[-1])) {
+                        pos = ch - vstr_str;
+                        break;
+                    }
+                }
+            } // if digit
+                    
+        } // for
+    }
+
 
     if (pos != string::npos) {
         int pname_end = (int)(pos - 1);
@@ -493,6 +503,7 @@ string SBuildInfo::ExtraName(EExtra key)
     case eProductionVersion:       return "Production-Version";
     case eBuiltAs:                 return "Built-As";
     case eRevision:                return "Revision";
+    case eGitBranch:               return "GitBranch";
     }
     return "Unknown";
 }
@@ -513,6 +524,7 @@ string SBuildInfo::ExtraNameAppLog(EExtra key)
     case eProductionVersion:       return "ncbi_app_prod_version";
     case eBuiltAs:                 return "ncbi_app_built_as";
     case eRevision:                return "ncbi_app_revision";
+    case eGitBranch:               return "ncbi_app_gitbranch";
     }
     return "ncbi_app_unk";
 }
