@@ -1,4 +1,4 @@
-/*  $Id: dbapi_impl_connection.cpp 669609 2023-07-05 19:26:32Z ivanov $
+/*  $Id: dbapi_impl_connection.cpp 684655 2024-06-28 14:39:22Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -435,6 +435,11 @@ string CConnection::GetDriverName(void) const
     return GetCDriverContext().GetDriverName();
 }
 
+string CConnection::GetVersionString(void) const
+{
+    return "0.0";
+}
+
 void CConnection::x_RecordServer(const CDBServer& server)
 {
     CWriteLockGuard guard(eEmptyGuard);
