@@ -1,4 +1,4 @@
-/* $Id: ncbi_win_hook.cpp 547700 2017-10-03 15:05:21Z satskyse $
+/* $Id: ncbi_win_hook.cpp 663858 2023-03-01 20:24:53Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -349,7 +349,7 @@ namespace NWinHook
         bool IsPatched(const void* addr);
         DWORD GetRVAFromExportSection(
             HMODULE hmodOriginal,
-            PSTR    pszFuncName
+            PCSTR   pszFuncName
             );
 
     private:
@@ -612,7 +612,7 @@ namespace NWinHook
     ////////////////////////////////////////////////////////////////////////////
     DWORD CKernell32::GetRVAFromExportSection(
         HMODULE hmodOriginal,
-        PSTR    pszFuncName
+        PCSTR   pszFuncName
         )
     {
         DWORD rva = 0;
@@ -1242,7 +1242,7 @@ namespace NWinHook
                 BOOL result = pHook->UnHookImport();
 
                 if (result == FALSE) {
-                    LOG_POST_X(4, Warning << pHook->GetFuncName() <<
+                    ERR_POST_X(4, Warning << pHook->GetFuncName() <<
                                " wasn't unhooked in " << NCBI_CURRENT_FUNCTION);
                 }
             }
@@ -1303,7 +1303,7 @@ namespace NWinHook
             BOOL result = UnHookImport();
 
             if (result == FALSE) {
-                LOG_POST_X(4, Warning <<
+                ERR_POST_X(4, Warning <<
                            "Import is not unhooked in " << NCBI_CURRENT_FUNCTION);
             }
         }
@@ -1917,7 +1917,7 @@ namespace NWinHook
                                      );
 
                 if (bResult == FALSE) {
-                    LOG_POST_X(4, Warning
+                    ERR_POST_X(4, Warning
                                << "LoadLibraryA is not hooked in "
                                << NCBI_CURRENT_FUNCTION
                                );
@@ -1931,7 +1931,7 @@ namespace NWinHook
                                      ) || bResult;
 
                 if (bResult == FALSE) {
-                    LOG_POST_X(4, Warning
+                    ERR_POST_X(4, Warning
                                << "LoadLibraryW is not hooked in "
                                << NCBI_CURRENT_FUNCTION
                                );
@@ -1945,7 +1945,7 @@ namespace NWinHook
                                      ) || bResult;
 
                 if (bResult == FALSE) {
-                    LOG_POST_X(4, Warning
+                    ERR_POST_X(4, Warning
                                << "LoadLibraryExA is not hooked in "
                                << NCBI_CURRENT_FUNCTION
                                );
@@ -1959,7 +1959,7 @@ namespace NWinHook
                                      ) || bResult;
 
                 if (bResult == FALSE) {
-                    LOG_POST_X(4, Warning
+                    ERR_POST_X(4, Warning
                                << "LoadLibraryExW is not hooked in "
                                << NCBI_CURRENT_FUNCTION
                                );
@@ -1973,7 +1973,7 @@ namespace NWinHook
                                      ) || bResult;
 
                 if (bResult == FALSE) {
-                    LOG_POST_X(4, Warning
+                    ERR_POST_X(4, Warning
                                << "GetProcAddress is not hooked in"
                                << NCBI_CURRENT_FUNCTION
                                );
@@ -2100,7 +2100,7 @@ namespace NWinHook
             BOOL result = pHook->HookImport();
 
             if (result == FALSE) {
-                LOG_POST_X(4, Warning << pszFuncName
+                ERR_POST_X(4, Warning << pszFuncName
                            << " is not hooked in "
                            << NCBI_CURRENT_FUNCTION
                            );
@@ -2345,7 +2345,7 @@ namespace NWinHook
         m_Hooked = (result == TRUE);
 
         if (!m_Hooked) {
-            LOG_POST_X(4, Warning
+            ERR_POST_X(4, Warning
                        << "ExitProcess is not hooked in "
                        << NCBI_CURRENT_FUNCTION
                        );
