@@ -1,4 +1,4 @@
-/* $Id: bcp.cpp 563801 2018-05-15 15:55:00Z ucko $
+/* $Id: bcp.cpp 637239 2021-09-09 13:34:08Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,8 +35,8 @@
 #include <dbapi/error_codes.hpp>
 #include <string.h>
 
-#include <odbcss.h>
-// #include <sqlncli.h>
+// #include <odbcss.h>
+#include <msodbcsql.h>
 
 #include "odbc_utils.hpp"
 
@@ -61,6 +61,7 @@ BEGIN_NCBI_SCOPE
 #define DBDATETIME4_mins(x) ((x)->nummins)
 #define DBNUMERIC_val(x) ((x)->val)
 #define SQL_VARLEN_DATA (-10)
+typedef SQL_SS_TIMESTAMPOFFSET_STRUCT DBTIMESTAMP;
 
 /////////////////////////////////////////////////////////////////////////////
 //
@@ -97,11 +98,11 @@ bool CODBC_BCPInCmd::Bind(unsigned int column_num, CDB_Object* param_ptr)
 
 
 int
-CODBC_BCPInCmd::x_GetBCPDataType(EDB_Type type)
+CODBC_BCPInCmd::x_GetBCPDataType(const CDB_Object& param)
 {
     int bcp_datatype = 0;
 
-    switch (type) {
+    switch (param.GetType()) {
     case eDB_Int:
         bcp_datatype = SQLINT4;
         break;
@@ -117,15 +118,17 @@ CODBC_BCPInCmd::x_GetBCPDataType(EDB_Type type)
     case eDB_Char:
     case eDB_VarChar:
     case eDB_LongChar:
-#ifdef UNICODE
-        bcp_datatype = SQLNCHAR;
-#else
-        bcp_datatype = SQLCHARACTER;
-#endif
+        if (static_cast<const CDB_String&>(param).GetBulkInsertionEnc()
+            == eBulkEnc_RawBytes) {
+            bcp_datatype = SQLCHARACTER;
+        } else {
+            bcp_datatype = SQLNCHAR;
+        }
         break;
     case eDB_Binary:
     case eDB_VarBinary:
     case eDB_LongBinary:
+    case eDB_VarBinaryMax:
         bcp_datatype = SQLBINARY;
         break;
         /*
@@ -151,22 +154,29 @@ CODBC_BCPInCmd::x_GetBCPDataType(EDB_Type type)
     case eDB_DateTime:
         bcp_datatype = SQLDATETIME;
         break;
-#ifdef SQLDATETIME2
+#ifdef SQLDATETIME2N
     case eDB_BigDateTime:
-        bcp_datatype = SQLDATETIME2;
+        bcp_datatype = SQLDATETIME2N;
         break;
 #endif
     case eDB_Text:
+        if (static_cast<const CDB_Text&>(param).GetEncoding()
+            == eBulkEnc_RawBytes) {
+            bcp_datatype = SQLTEXT;
+        } else {
+            bcp_datatype = SQLNTEXT;
+        }
+        break;
     case eDB_VarCharMax:
-//TODO: Make different type depending on type of underlying column
-/*#ifdef UNICODE
-        bcp_datatype = SQLNTEXT;
-#else*/
-        bcp_datatype = SQLTEXT;
-//#endif
+        if (static_cast<const CDB_VarCharMax&>(param).GetEncoding()
+            == eBulkEnc_RawBytes) {
+            bcp_datatype = SQLCHARACTER;
+        } else {
+            bcp_datatype = SQLNCHAR;
+        }
         break;
     case eDB_Image:
-    case eDB_VarBinaryMax:
+    // case eDB_VarBinaryMax:
         bcp_datatype = SQLIMAGE;
         break;
     case eDB_Bit:
@@ -181,38 +191,54 @@ CODBC_BCPInCmd::x_GetBCPDataType(EDB_Type type)
 
 
 size_t
-CODBC_BCPInCmd::x_GetDataTermSize(EDB_Type type)
+CODBC_BCPInCmd::x_GetDataTermSize(const CDB_Object& param)
 {
-    switch (type) {
+    EBulkEnc bulk_enc;
+    switch (param.GetType()) {
     case eDB_Char:
     case eDB_VarChar:
     case eDB_LongChar:
-    case eDB_Text:
-    case eDB_VarCharMax:
-        return sizeof(odbc::TChar);
-    default:
+        bulk_enc = static_cast<const CDB_String&>(param).GetBulkInsertionEnc();
         break;
+    case eDB_Text:
+        bulk_enc = static_cast<const CDB_Text&>(param).GetEncoding();
+        break;
+    case eDB_VarCharMax:
+        bulk_enc = static_cast<const CDB_VarCharMax&>(param).GetEncoding();
+        break;
+    default:
+        return 0;
     }
 
-    return 0;
+    return bulk_enc == eBulkEnc_RawBytes ? 1 : sizeof(odbc::TChar);
 }
 
 
-void*
-CODBC_BCPInCmd::x_GetDataTerminator(EDB_Type type)
+const void*
+CODBC_BCPInCmd::x_GetDataTerminator(const CDB_Object& param)
 {
-    switch (type) {
+    EBulkEnc bulk_enc;
+    switch (param.GetType()) {
     case eDB_Char:
     case eDB_VarChar:
     case eDB_LongChar:
-    case eDB_Text:
-    case eDB_VarCharMax:
-        return _T_NCBI_ODBC("");
-    default:
+        bulk_enc = static_cast<const CDB_String&>(param).GetBulkInsertionEnc();
         break;
+    case eDB_Text:
+        bulk_enc = static_cast<const CDB_Text&>(param).GetEncoding();
+        break;
+    case eDB_VarCharMax:
+        bulk_enc = static_cast<const CDB_VarCharMax&>(param).GetEncoding();
+        break;
+    default:
+        return NULL;
     }
 
-    return NULL;
+    if (bulk_enc == eBulkEnc_RawBytes) {
+        return kEmptyCStr;
+    } else {
+        return _T_NCBI_ODBC("");
+    }
 }
 
 
@@ -259,9 +285,9 @@ bool CODBC_BCPInCmd::x_AssignParams(void* pb)
                              static_cast<LPCBYTE>(const_cast<void*>(x_GetDataPtr(data_type, pb))),
                              0,
                              static_cast<DBINT>(x_GetBCPDataSize(data_type)),
-                             static_cast<LPCBYTE>(x_GetDataTerminator(data_type)),
-                             static_cast<INT>(x_GetDataTermSize(data_type)),
-                             x_GetBCPDataType(data_type),
+                             static_cast<LPCBYTE>(x_GetDataTerminator(param)),
+                             static_cast<INT>(x_GetDataTermSize(param)),
+                             x_GetBCPDataType(param),
                              i + 1);
 
                 m_HasBlob = m_HasBlob || CDB_Object::IsBlobType(data_type);
@@ -443,18 +469,18 @@ bool CODBC_BCPInCmd::x_AssignParams(void* pb)
                 pb = (void*) (dt + 1);
             }
             break;
-#if defined(SQLDATETIME2)
-            case CDB_BigDateTime: {
+#if defined(SQLDATETIME2N)
+            case eDB_BigDateTime: {
                 CDB_BigDateTime& val = dynamic_cast<CDB_BigDateTime&> (param);
                 CTime            lt  = val.GetCTime().GetLocalTime();
                 DBTIMESTAMP*     dt  = (DBTIMESTAMP*) pb;
-                dt.year     = lt.Year();
-                dt.month    = lt.Month();
-                dt.day      = lt.Day();
-                dt.hour     = lt.Hour();
-                dt.minute   = lt.Minute();
-                dt.second   = lt.Second();
-                dt.fraction = lt.Nanosecond() / 100 * 100;
+                dt->year     = lt.Year();
+                dt->month    = lt.Month();
+                dt->day      = lt.Day();
+                dt->hour     = lt.Hour();
+                dt->minute   = lt.Minute();
+                dt->second   = lt.Second();
+                dt->fraction = lt.NanoSecond() / 100 * 100;
                 r = bcp_colptr(GetHandle(), (BYTE*) dt, i + 1)
                     == SUCCEED &&
                     bcp_collen(GetHandle(), sizeof(DBTIMESTAMP), i + 1)
@@ -475,10 +501,9 @@ bool CODBC_BCPInCmd::x_AssignParams(void* pb)
                 else {
                     r = bcp_bind(GetHandle(), (BYTE*) NULL, 0, (DBINT) val.Size(),
                                  static_cast<LPCBYTE>(x_GetDataTerminator
-                                                      (param.GetType())),
-                                 static_cast<INT>(x_GetDataTermSize
-                                                  (param.GetType())),
-                                 x_GetBCPDataType(param.GetType()),
+                                                      (param)),
+                                 static_cast<INT>(x_GetDataTermSize(param)),
+                                 x_GetBCPDataType(param),
                                  i + 1);
                 }
             }
