@@ -1,4 +1,4 @@
-/* $Id: public.cpp 620241 2020-11-18 15:39:15Z ucko $
+/* $Id: public.cpp 684655 2024-06-28 14:39:22Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -577,6 +577,12 @@ string CDB_Connection::GetDriverName(void) const
     return m_ConnImpl->GetDriverName();
 }
 
+string CDB_Connection::GetVersionString(void) const
+{
+    CHECK_CONNECTION(m_ConnImpl);
+    return m_ConnImpl->GetVersionString();
+}
+
 void CDB_Connection::FinishOpening(void)
 {
     CHECK_CONNECTION(m_ConnImpl);
