@@ -1,4 +1,4 @@
-/*  $Id: html.cpp 435421 2014-05-15 19:29:47Z ucko $
+/*  $Id: html.cpp 691463 2024-12-18 19:21:57Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -583,11 +583,14 @@ CNcbiOstream& CHTMLBlockElement::PrintEnd(CNcbiOstream& out, TMode mode)
     switch (mode) {
         case ePlainText:
             {{
-            // Add a newline iff no node on the path to the last descendant
+            // Add a newline if no node on the path to the last descendant
             // is also a block element. We only need one break.
             CNCBINode* node = this;
             while (node->HaveChildren()) {
                 node = node->Children().back();
+                if (!node) {
+                    break;
+                }
                 if (dynamic_cast<CHTMLBlockElement*>(node)) {
                     return out;
                 }
