@@ -1,4 +1,4 @@
-/*  $Id: page.cpp 626608 2021-03-02 14:23:58Z ivanov $
+/*  $Id: page.cpp 696095 2025-04-08 12:02:18Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -387,7 +387,7 @@ void CHTMLPage::x_LoadTemplate(CNcbiIstream& is, string& str)
     if ( m_TemplateSize ) {
         str.reserve(m_TemplateSize);
     }
-    while ( is ) {
+    while (is.good()) {
         is.read(buf, sizeof(buf));
         if (m_TemplateSize == 0  &&  is.gcount() > 0
             &&  str.size() == str.capacity()) {
@@ -423,7 +423,7 @@ CNCBINode* CHTMLPage::x_PrintTemplate(CNcbiIstream& is, CNcbiOstream* out,
     char   buf[kBufferSize];
     unique_ptr<CNCBINode> node(new CNCBINode);
 
-    while (is) {
+    while (is.good()) {
         is.read(buf, sizeof(buf));
         str.append(buf, (SIZE_TYPE)is.gcount());
         SIZE_TYPE pos = str.rfind('\n');
@@ -573,6 +573,7 @@ void CHTMLPage::x_LoadTemplateLib(CNcbiIstream& istrm, SIZE_TYPE size,
         if ( !file_name.empty() ) {
             Int8 x_size = CFile(file_name).GetLength();
             if (x_size == 0) {
+                if ( caching ) delete pstr;
                 return;
             } else if (x_size < 0) {
                 NCBI_THROW(CHTMLException, eTemplateAccess,
@@ -643,7 +644,7 @@ void CHTMLPage::x_LoadTemplateLib(CNcbiIstream& istrm, SIZE_TYPE size,
         } else {
             // Use faster block read
             char buf[kBufferSize];
-            while (is) {
+            while (is->good()) {
                 is->read(buf, sizeof(buf));
                 if (pstr->size() == pstr->capacity()  &&
                     is->gcount() > 0) {
