@@ -1,5 +1,5 @@
 
-/*  $Id: test_jsonwrapp.cpp 620381 2020-11-20 19:00:55Z gouriano $
+/*  $Id: test_jsonwrapp.cpp 671770 2023-08-23 18:53:20Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -788,6 +788,17 @@ BOOST_AUTO_TEST_CASE(s_JsonWrapp)
         BOOST_CHECK(docs.GetArray().at(1).GetValue().IsBool());
         BOOST_CHECK(!docs.GetArray().at(1).GetValue().GetBool());
 
+        char buf[128];
+        strcpy(buf, "[\"utf8 string expected\", false, null ]");
+        docs.ParseMutableString(buf);
+        cout << docs;
+        BOOST_CHECK(docs.ReadSucceeded());
+        BOOST_CHECK(docs.IsArray());
+        BOOST_CHECK(docs.GetArray().size() == 3);
+        BOOST_CHECK(docs.GetArray().at(1).IsValue());
+        BOOST_CHECK(docs.GetArray().at(1).GetValue().IsBool());
+        BOOST_CHECK(!docs.GetArray().at(1).GetValue().GetBool());
+
 // --------------------------------------------------------------------------
 // new document from array
         CJson_Document doc0(arr);
@@ -888,6 +899,16 @@ BOOST_AUTO_TEST_CASE(s_JsonWrapp)
         ofstream ofs(filename.c_str());
         ofs << doc;
     }
+    {
+        CJson_Document doc2;
+        BOOST_CHECK(doc2.Read(filename));
+        cout << doc2;
+    }
+    {
+        CJson_Document doc2;
+        BOOST_CHECK(doc2.ReadBuffered(filename));
+        cout << doc2;
+    }
 
 // --------------------------------------------------------------------------
 // SAX parsing
@@ -947,6 +968,7 @@ BOOST_AUTO_TEST_CASE(s_JsonWrapp)
             "},\"required\": [\"id\", \"name\", \"price\"]}"
             );
         CJson_Document sd(str_schema);
+        BOOST_CHECK(sd.IsSchema());
         CJson_Schema sc(sd);
         CJson_Document testdoc;
         testdoc.SetObject().insert("id", 1);
