@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 #define OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 
-/*  $Id: seq_id_tree.hpp 662785 2023-02-06 21:49:17Z vasilche $
+/*  $Id: seq_id_tree.hpp 694387 2025-03-03 21:14:20Z sadyrovr $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -554,9 +554,14 @@ private:
         }
         x_FindMatchByAcc(id_list, acc, ver_ptr);
     }
+    enum EAllowFields {
+        eAnyFields,
+        eOnlyName
+    };
     void x_FindMatchByName(TSeq_id_MatchList& id_list,
                            const string& name,
-                           const CTextseq_id* tid = 0) const;
+                           const CTextseq_id* tid = nullptr,
+                           EAllowFields allow_fields = eAnyFields) const;
 
     void x_FindRevMatchByAccPacked(TSeq_id_MatchList& id_list,
                                    const string& acc,
