@@ -1,4 +1,4 @@
-/*  $Id: local_taxon.cpp 684231 2024-06-17 12:05:48Z ivanov $
+/*  $Id: local_taxon.cpp 692705 2025-01-27 19:36:51Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,21 +33,18 @@
 #include <ncbi_pch.hpp>
 
 #include <objects/taxon1/local_taxon.hpp>
-
 #include <objects/seqfeat/Org_ref.hpp>
-
 #include <objects/taxon1/taxon1.hpp>
 
 #include <serial/serial.hpp>
 #include <serial/objistr.hpp>
 
 #include <db/sqlite/sqlitewrapp.hpp>
+#include <corelib/ncbi_safe_static.hpp>
 
 BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 
-CLocalTaxon::TNodes CLocalTaxon::s_DummyNodes;
-CLocalTaxon::TNodeRef CLocalTaxon::s_InvalidNode = CLocalTaxon::s_DummyNodes.end();
 
 void CLocalTaxon::AddArguments(CArgDescriptions& arg_desc)
 {
@@ -91,10 +88,17 @@ CLocalTaxon::~CLocalTaxon()
 {
 }
 
+CLocalTaxon::TNodeRef CLocalTaxon::GetInvalidNode()
+{
+    static CSafeStatic<CLocalTaxon::TNodes> s_DummyNodes;
+    static CLocalTaxon::TNodeRef s_InvalidNode = s_DummyNodes->end();
+    return s_InvalidNode;
+}
+
 CLocalTaxon::STaxidNode::STaxidNode()
     : taxid(INVALID_TAX_ID)
     , is_valid(false)
-    , parent(s_InvalidNode)
+    , parent(GetInvalidNode())
     , genetic_code(-1)
 {
 }
@@ -118,7 +122,7 @@ CLocalTaxon::TTaxid CLocalTaxon::GetParent(TTaxid taxid)
     if (m_SqliteConn.get()) {
         x_Cache(taxid);
         TNodeRef parent = m_Nodes.find(taxid)->second.parent;
-        return parent == s_InvalidNode ? ZERO_TAX_ID : parent->first;
+        return parent == GetInvalidNode() ? ZERO_TAX_ID : parent->first;
     } else {
         return m_TaxonConn->GetParent(taxid);
     }
@@ -126,18 +130,25 @@ CLocalTaxon::TTaxid CLocalTaxon::GetParent(TTaxid taxid)
 
 string CLocalTaxon::GetRank(TTaxid taxid)
 {
+    if (taxid <= 0) {
+        return string();
+    }
+
     if (m_SqliteConn.get()) {
         x_Cache(taxid);
         return m_Nodes.find(taxid)->second.rank;
-    } else {
-        string rank_name;
-        auto node = m_TaxonConn->GetTreeIterator(taxid)->GetNode();
+    }
+
+    string rank_name;
+    auto it = m_TaxonConn->GetTreeIterator(taxid);
+    if (it) {
+        auto node = it->GetNode();
         if (node) {
             TTaxRank rank_id = node->GetRank();
             m_TaxonConn->GetRankName(rank_id, rank_name);
         }
-        return rank_name;
     }
+    return rank_name;
 }
 
 string CLocalTaxon::GetScientificName(TTaxid taxid)
@@ -254,6 +265,25 @@ CConstRef<objects::COrg_ref> CLocalTaxon::GetOrgRef(TTaxid taxid)
     }
 }
 
+CConstRef<objects::COrg_ref> CLocalTaxon::GetOrgRef(TTaxid taxid,
+                                                    bool& is_species,
+                                                    bool& is_uncultured,
+                                                    string& blast_name)
+{
+    if (m_SqliteConn.get()) {
+        x_Cache(taxid, true);
+
+        // we fake some data here
+        TNodes::const_iterator it = m_Nodes.find(taxid);
+        is_species = (it->second.rank == "species");
+        is_uncultured = false;
+        blast_name = it->second.scientific_name;
+        return it->second.org_ref;
+    } else {
+        return m_TaxonConn->GetOrgRef(taxid, is_species, is_uncultured, blast_name);
+    }
+}
+
 CLocalTaxon::TTaxid CLocalTaxon::GetAncestorByRank(TTaxid taxid, const string &rank)
 {
     if (m_SqliteConn.get()) {
@@ -265,12 +295,14 @@ CLocalTaxon::TTaxid CLocalTaxon::GetAncestorByRank(TTaxid taxid, const string &r
             }
         }
         return ZERO_TAX_ID;
-    } else if (m_fallback) {
-            if (!m_TaxonConn.get()) {
-                m_TaxonConn.reset(new CTaxon1);
-                m_TaxonConn->Init();
-            }
+    } else {
+        if (m_fallback && !m_TaxonConn.get()) {
+            m_TaxonConn.reset(new CTaxon1);
+            m_TaxonConn->Init();
+        }
+        if (m_TaxonConn.get()) {
             return m_TaxonConn->GetAncestorByRank(taxid, rank.c_str());
+        }
     }
 NCBI_THROW(CException, eUnknown, "CLocalTaxon: neither local nor remote connections available");
 }
@@ -315,17 +347,17 @@ CLocalTaxon::TLineage CLocalTaxon::GetLineage(TTaxid taxid)
 CLocalTaxon::TTaxid CLocalTaxon::Join(TTaxid taxid1, TTaxid taxid2)
 {
     if (m_SqliteConn.get()) {
-        TLineage lineage1 = GetLineage(taxid1),
-                 lineage2 = GetLineage(taxid2);
-        TLineage::const_iterator it1 = lineage1.begin(),
-                                 it2 = lineage2.begin();
-        for (; it1 != lineage1.end() && it2 != lineage2.end() && *it1 == *it2;
+        TLineage lineage1 = GetLineage(taxid1);
+        TLineage lineage2 = GetLineage(taxid2);
+        TLineage::const_iterator it1 = lineage1.begin();
+        TLineage::const_iterator it2 = lineage2.begin();
+        CLocalTaxon::TTaxid join_taxid = 0;
+        for ( ;  it1 != lineage1.end()  &&  it2 != lineage2.end()  &&
+              *it1 == *it2;
                ++it1, ++it2) {
+            join_taxid = *it1;
         }
-        if (it1 == lineage1.end()) {
-            return 0;
-        }
-        return *--it1;
+        return join_taxid;
     } else {
         return m_TaxonConn->Join(taxid1, taxid2);
     }
@@ -514,7 +546,7 @@ void CLocalTaxon::x_GetLineage(TTaxid taxid, TInternalLineage &lineage)
         return;
     }
     lineage.push_front(it);
-    while(lineage.front()->second.parent != s_InvalidNode) {
+    while(lineage.front()->second.parent != GetInvalidNode()) {
         lineage.push_front(lineage.front()->second.parent);
     }
 }
