@@ -1,4 +1,4 @@
-/* $Id: taxon1.cpp 624673 2021-02-03 13:43:41Z ivanov $
+/* $Id: taxon1.cpp 663830 2023-03-01 16:19:54Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -138,9 +138,9 @@ CTaxon1::Init(const STimeout* timeout, unsigned reconnect_attempts, unsigned cac
             ( (tmp=getenv("NI_SERVICE_NAME_TAXONOMY")) != NULL ) ) {
             m_pchService = tmp;
         }
-        auto_ptr<CConn_ServiceStream> pServer;
-        auto_ptr<CObjectOStream> pOut;
-        auto_ptr<CObjectIStream> pIn;
+        unique_ptr<CConn_ServiceStream> pServer;
+        unique_ptr<CObjectOStream> pOut;
+        unique_ptr<CObjectIStream> pIn;
 	pNi = ConnNetInfo_Create( m_pchService );
 	if( pNi == NULL ) {
 	    SetLastError( "ERROR: Init(): Unable to create net info" );
@@ -1394,7 +1394,7 @@ CTaxon1::GetTaxId4GI(TGi gi, TTaxId& tax_id_out )
         } else { // Internal: wrong respond type
             SetLastError( "INTERNAL: TaxService response type is not Id4gi" );
         }
-    } else if( GetLastError() != NULL && NStr::EqualNocase( GetLastError(), "ERROR: id4gi: No taxid for this gi") ) {
+    } else if( NStr::EqualNocase( GetLastError(), "ERROR: id4gi: No taxid for this gi") ) {
         tax_id_out = ZERO_TAX_ID;
         return true;
     }
@@ -1488,9 +1488,9 @@ CTaxon1::SendRequest( CTaxon1_req& req, CTaxon1_resp& resp, bool bShouldReconnec
             m_pIn = NULL;
             m_pServer = NULL;
             try {
-                auto_ptr<CObjectOStream> pOut;
-                auto_ptr<CObjectIStream> pIn;
-                auto_ptr<CConn_ServiceStream>
+                unique_ptr<CObjectOStream> pOut;
+                unique_ptr<CObjectIStream> pIn;
+                unique_ptr<CConn_ServiceStream>
                     pServer( new CConn_ServiceStream(m_pchService, fSERV_Any,
                                                      0, 0, m_timeout) );
 
