@@ -1,4 +1,4 @@
-/*  $Id: bioseq_info.cpp 598334 2019-12-06 18:14:42Z vasilche $
+/*  $Id: bioseq_info.cpp 667638 2023-05-18 17:59:55Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -158,11 +158,9 @@ void CBioseq_Info::x_DoUpdate(TNeedUpdateFlags flags)
         if ( !m_Seq_dataChunks.empty() ) {
             x_LoadChunks(m_Seq_dataChunks);
         }
+        CFastMutexGuard guard(m_SeqMap_Mtx);
         if ( m_SeqMap ) {
-            CFastMutexGuard guard(m_SeqMap_Mtx);
-            if ( m_SeqMap ) {
-                m_SeqMap->x_UpdateSeq_inst(m_Object->SetInst());
-            }
+            m_SeqMap->x_UpdateSeq_inst(m_Object->SetInst());
         }
     }
     if ( flags & fNeedUpdate_assembly ) {
@@ -321,10 +319,14 @@ void CBioseq_Info::x_SetObject(const CBioseq_Info& info,
         x_DSMapObject(m_Object, GetDataSource());
     }
     m_Id = info.m_Id;
-    if ( info.m_SeqMap ) {
-        m_SeqMap = info.m_SeqMap->CloneFor(*m_Object);
-        m_SeqMap->m_Bioseq = this;
-    }
+    {{
+        CFastMutexGuard guard(info.m_SeqMap_Mtx);
+        if ( info.m_SeqMap ) {
+            CFastMutexGuard guard2(m_SeqMap_Mtx);
+            m_SeqMap = info.m_SeqMap->CloneFor(*m_Object);
+            m_SeqMap->m_Bioseq = this;
+        }
+    }}
     if ( info.IsSetAnnot() ) {
         x_SetAnnot(info, copy_map);
     }
@@ -1242,22 +1244,16 @@ void CBioseq_Info::x_AttachMap(CSeqMap& seq_map)
 
 const CSeqMap& CBioseq_Info::GetSeqMap(void) const
 {
-    const CSeqMap* ret = m_SeqMap.GetPointer();
-    if ( !ret ) {
-        CFastMutexGuard guard(m_SeqMap_Mtx);
-        ret = m_SeqMap.GetPointer();
-        if ( !ret ) {
-            m_SeqMap = CSeqMap::CreateSeqMapForBioseq(*m_Object);
-            m_SeqMap->m_Bioseq = const_cast<CBioseq_Info*>(this);
-            ret = m_SeqMap.GetPointer();
-            _ASSERT(ret);
-        }
+    CFastMutexGuard guard(m_SeqMap_Mtx);
+    if ( !m_SeqMap ) {
+        m_SeqMap = CSeqMap::CreateSeqMapForBioseq(*m_Object);
+        m_SeqMap->m_Bioseq = const_cast<CBioseq_Info*>(this);
     }
-    return *ret;
+    return *m_SeqMap;
 }
 
 
-int CBioseq_Info::GetTaxId(void) const
+TTaxId CBioseq_Info::GetTaxId(void) const
 {
     const COrg_ref* org_ref = 0;
     if ( const CSeqdesc* desc_src = x_SearchFirstDesc(1<<CSeqdesc::e_Source) ) {
@@ -1267,13 +1263,13 @@ int CBioseq_Info::GetTaxId(void) const
         org_ref = &desc_org->GetOrg();
     }
     else {
-        return 0;
+        return ZERO_TAX_ID;
     }
     try {
         return org_ref->GetTaxId();
     }
     catch ( CException& /*ignored*/ ) {
-        return 0;
+        return ZERO_TAX_ID;
     }
 }
 
