@@ -1,4 +1,4 @@
-/*  $Id: bioseq_set_info.cpp 606971 2020-04-29 13:54:33Z vasilche $
+/*  $Id: bioseq_set_info.cpp 643673 2022-01-14 15:54:47Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -210,6 +210,12 @@ void CBioseq_set_Info::x_AddBioseqChunkId(TChunkId id)
 }
 
 
+const CBioseq_set_Info::TChunkIds& CBioseq_set_Info::x_GetBioseqChunkIds() const
+{
+    return m_BioseqChunks;
+}
+
+
 void CBioseq_set_Info::x_DoUpdate(TNeedUpdateFlags flags)
 {
     if (flags & (fNeedUpdate_bioseq|fNeedUpdate_core|fNeedUpdate_children)) {
