@@ -1,4 +1,4 @@
-/*  $Id: data_loader.cpp 659592 2022-11-30 18:45:10Z grichenk $
+/*  $Id: data_loader.cpp 679239 2024-02-20 19:05:51Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -557,6 +557,25 @@ CDataLoader::GetSequenceHashFound(const CSeq_id_Handle& idh)
 }
 
 
+void CDataLoader::GetBulkIds(const TIds& ids, TLoaded& loaded, TBulkIds& ret)
+{
+    size_t count = ids.size();
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    TIds seq_ids;
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] ) {
+            continue;
+        }
+        ret[i].clear();
+        GetIds(ids[i], ret[i]);
+        if ( !ret[i].empty() ) {
+            loaded[i] = true;
+        }
+    }
+}
+
+
 void CDataLoader::GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret)
 {
     size_t count = ids.size();
