@@ -1,4 +1,4 @@
-/*  $Id: data_source.cpp 669340 2023-06-27 13:19:52Z ivanov $
+/*  $Id: data_source.cpp 686741 2024-08-26 15:21:55Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -584,16 +584,13 @@ CDataSource::GetSeq_entry_Lock(const CBlobIdKey& blob_id)
     TSeq_entry_Lock ret;
     {{
         try {
-            ret.first = m_Loader->GetBlobById(blob_id);
+            ret.second = m_Loader->GetBlobById(blob_id);
+            ret.first = ret.second;
         }
         catch ( CLoaderException& exc ) {
             exc.SetFailedCall(s_FormatCall("GetBlobById", blob_id));
             throw;
         }
-        TMainLock::TWriteLockGuard guard(m_DSMainLock);
-        if ( ret.first ) {
-            x_SetLock(ret.second, ConstRef(&ret.first->GetTSE_Info()));
-        }
     }}
     return ret;
 }
@@ -993,7 +990,7 @@ CDataSource::x_GetRecords(const CSeq_id_Handle& idh,
     if ( m_Loader ) {
         CDataLoader::TTSE_LockSet tse_set2;
         try {
-            tse_set2 = move(m_Loader->GetRecords(idh, choice));
+            tse_set2 = m_Loader->GetRecords(idh, choice);
         }
         catch ( CLoaderException& exc ) {
             exc.SetFailedCall(s_FormatCall("GetRecords", idh, choice));
@@ -1091,7 +1088,7 @@ void CDataSource::GetTSESetWithOrphanAnnots(const TSeq_idSet& ids,
         // collect set of TSEs with orphan annotations
         CDataLoader::TTSE_LockSet tse_set;
         try {
-            tse_set = move(m_Loader->GetOrphanAnnotRecordsNA(ids, sel, processed_nas));
+            tse_set = m_Loader->GetOrphanAnnotRecordsNA(ids, sel, processed_nas);
         }
         catch ( CLoaderException& exc ) {
             exc.SetFailedCall(s_FormatCall("GetOrphanAnnotRecordsNA(", ids, sel, processed_nas));
@@ -1149,7 +1146,7 @@ void CDataSource::GetTSESetWithBioseqAnnots(const CBioseq_Info& bioseq,
         // external annotations
         CDataLoader::TTSE_LockSet tse_set2;
         try {
-            tse_set2 = move(m_Loader->GetExternalAnnotRecordsNA(bioseq, sel, processed_nas));
+            tse_set2 = m_Loader->GetExternalAnnotRecordsNA(bioseq, sel, processed_nas);
         }
         catch ( CLoaderException& exc ) {
             exc.SetFailedCall(s_FormatCall("GetExternalAnnotRecordsNA", bioseq, sel, processed_nas));
@@ -1691,6 +1688,36 @@ int CDataSource::GetSequenceState(const CSeq_id_Handle& idh)
 }
 
 
+void CDataSource::GetBulkIds(const TIds& ids, TLoaded& loaded, TBulkIds& ret)
+{
+    size_t count = ids.size(), remaining = 0;
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] ) {
+            continue;
+        }
+        SSeqMatch_DS match = x_GetSeqMatch(ids[i]);
+        if ( match ) {
+            ret[i] = match.m_Bioseq->GetId();
+            loaded[i] = true;
+        }
+        else {
+            ++remaining;
+        }
+    }
+    if ( remaining && m_Loader ) {
+        try {
+            m_Loader->GetBulkIds(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetBulkIds", ids));
+            throw;
+        }
+    }
+}
+
+
 void CDataSource::GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret)
 {
     size_t count = ids.size(), remaining = 0;
@@ -2001,9 +2028,20 @@ void CDataSource::GetBlobs(TSeqMatchMap& match_map)
             // bulk chunk loading
             vector<CConstRef<CTSE_Chunk_Info>> chunks;
             for ( auto& tse_set : tse_sets ) {
+                auto& idh = tse_set.first;
+                CSeq_id_Handle::TMatches hset;
+                if ( idh.HaveMatchingHandles() ) {
+                    idh.GetMatchingHandles(hset, eAllowWeakMatch);
+                }
                 for ( auto& tse_lock : tse_set.second ) {
                     if ( tse_lock->HasSplitInfo() ) {
-                        tse_lock->GetSplitInfo().x_AddChunksForGetRecords(chunks, tse_set.first);
+                        auto& split_info = tse_lock->GetSplitInfo();
+                        split_info.x_AddChunksForGetRecords(chunks, idh);
+                        for ( auto& hit : hset ) {
+                            if ( hit == idh ) // already checked
+                                continue;
+                            split_info.x_AddChunksForGetRecords(chunks, hit);
+                        }
                     }
                 }
             }
