@@ -1,4 +1,4 @@
-/*  $Id: scope.cpp 659592 2022-11-30 18:45:10Z grichenk $
+/*  $Id: scope.cpp 679239 2024-02-20 19:05:51Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -661,6 +661,27 @@ void CScope::UpdateAnnotIndex(void)
 
 
 /// Bulk retrieval methods
+CScope::TBulkIds CScope::GetBulkIds(const TSeq_id_Handles& idhs,
+                                    TGetFlags flags)
+{
+    TBulkIds results;
+    GetBulkIds(&results, idhs, flags);
+    return results;
+}
+
+
+void CScope::GetBulkIds(TBulkIds* results,
+                        const TSeq_id_Handles& idhs,
+                        TGetFlags flags)
+{
+    if ( !results ) {
+        NCBI_THROW(CCoreException, eNullPtr,
+                   "CScope::GetBulkIds: null results pointer");
+    }
+    return m_Impl->GetBulkIds(*results, idhs, flags);
+}
+
+
 CScope::TSeq_id_Handles CScope::GetAccVers(const TSeq_id_Handles& idhs,
                                            TGetFlags flags)
 {
