@@ -1,4 +1,4 @@
-/*  $Id: seq_map.cpp 667637 2023-05-18 17:58:23Z vasilche $
+/*  $Id: seq_map.cpp 680566 2024-03-21 14:36:08Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1167,7 +1167,7 @@ CConstRef<CSeqMap> CSeqMap::GetSeqMapForSeq_loc(const CSeq_loc& loc,
         if ( loc.IsInt() ) {
             const CSeq_interval& locint = loc.GetInt();
             if ( locint.GetFrom() == 0 &&
-                 (!locint.IsSetStrand() || IsForward(locint.GetStrand())) ) {
+                 (!locint.IsSetStrand() || !IsReverse(locint.GetStrand())) ) {
                 CBioseq_Handle bh = scope->GetBioseqHandle(locint.GetId());
                 if ( bh ) {
                     if ( bh.GetBioseqLength() == locint.GetTo()+1 ) {
