@@ -1,4 +1,4 @@
-/*  $Id: seq_loc_util.cpp 569939 2018-08-31 13:32:44Z bollin $
+/*  $Id: seq_loc_util.cpp 671392 2023-08-14 14:23:37Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -609,7 +609,7 @@ bool BadSeqLocSortOrder
             }
             last_range = lit.GetRange();
         }
-    } catch (CException) {
+    } catch (CException&) {
         // exception will be thrown if references far sequence and not remote fetching
     }
     return false;
