@@ -1,4 +1,4 @@
-/*  $Id: seqalignfilter.cpp 613601 2020-08-10 18:31:02Z zaretska $
+/*  $Id: seqalignfilter.cpp 655764 2022-09-08 17:43:28Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -448,7 +448,8 @@ void CSeqAlignFilter::x_ReadExtraGis(CConstRef<CSeq_align> sa, vector<TGi>& vec_
                 string str_id = score_entry->GetId().GetStr();
                 if (str_id == "use_this_gi")
                 {
-                    TGi gi = GI_FROM(CScore::C_Value::TInt, score_entry->GetValue().GetInt());
+                	Uint4 gi_v = (Uint4) (score_entry->GetValue().GetInt());
+                    TGi gi = GI_FROM(Uint4, gi_v);
                     vec_extra_gis.push_back(gi);
                 }
             }
@@ -513,13 +514,13 @@ CRef<CSeqDB> CSeqAlignFilter::PrepareSeqDB(const string& fname_db, bool is_prot,
 
 void CSeqAlignFilter::ReadSeqalignSet(const string& fname, CSeq_align_set& aln)
 {
-    auto_ptr<CObjectIStream> asn_in(CObjectIStream::Open(fname, eSerial_AsnText));
+    unique_ptr<CObjectIStream> asn_in(CObjectIStream::Open(fname, eSerial_AsnText));
     *asn_in >> aln;
 }
 
 void CSeqAlignFilter::WriteSeqalignSet(const string& fname, const CSeq_align_set& aln)
 {
-    auto_ptr<CObjectOStream> asn_out(CObjectOStream::Open(fname, eSerial_AsnText));
+    unique_ptr<CObjectOStream> asn_out(CObjectOStream::Open(fname, eSerial_AsnText));
     *asn_out << aln;
 }
 
@@ -707,10 +708,18 @@ static CRef<CSeq_align> s_ModifySeqAlnWithFilteredSeqIDs(CRef<CBlast_def_line_se
         bool has_match = s_IncludeDeflineTaxid(defline, taxids);
         CRef<CSeq_id> seqID;
         string textSeqID;
-        if(has_match) {
-            const CBioseq::TId& cur_id = (*iter)->GetSeqid();
-            seqID = FindBestChoice(cur_id, CSeq_id::WorstRank);
-            CAlignFormatUtil::GetTextSeqID(seqID, &textSeqID);
+        TGi cur_gi;
+        if(has_match) {            
+            const CBioseq::TId& cur_id = (*iter)->GetSeqid();            
+            seqID = FindBestChoice(cur_id, CSeq_id::WorstRank);            
+
+            cur_gi =  FindGi(cur_id);                                   
+            if(cur_gi != ZERO_GI) {            
+                textSeqID = NStr::NumericToString(cur_gi);                
+            }   
+            else {
+                CAlignFormatUtil::GetTextSeqID(seqID, &textSeqID);                
+            }
 
             list<string> use_this_seq;
             CAlignFormatUtil::GetUseThisSequence(*in_align,use_this_seq);  
@@ -721,8 +730,8 @@ static CRef<CSeq_align> s_ModifySeqAlnWithFilteredSeqIDs(CRef<CBlast_def_line_se
         if(has_match) {
             if(sa_copy.Empty()) {                
                 sa_copy = s_UpdateSubjectInSeqalign(in_align,seqID);
-            }
-            if(seqID->IsGi()) {
+            }            
+            if(cur_gi != ZERO_GI) {            
                 useThisSeqs.push_back("gi:" + textSeqID);
             }
             else {
