@@ -1,4 +1,4 @@
-/*  $Id: taxFormat.cpp 613840 2020-08-13 14:11:10Z zaretska $
+/*  $Id: taxFormat.cpp 696198 2025-04-09 15:42:12Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -56,6 +56,74 @@ USING_SCOPE(sequence);
 BEGIN_SCOPE(align_format)
 
 
+MAKE_CONST_MAP(kMapTagToString, ct::tagStrNocase, ct::tagStrNocase,
+{
+    //kTaxBrowserURL s_TagToConstString("TAX_BROWSER_URL")
+    { "TAX_BROWSER_URL", "<@protocol@>//www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi"},
+    //kBlastNameLink s_TagToConstString("BLAST_NAME_LINK")
+    {"BLAST_NAME_LINK" ,"<a href=\"<@taxBrowserURL@>?id=<@bl_taxid@>\" target=\"lnktx<@rid@>\" title=\"Show taxonomy info for <@blast_name@> (taxid <@bl_taxid@>)\"><@blast_name@></a>"},
+    //kOrgReportTable       s_TagToConstString("ORG_REPORT_TABLE")   
+    { "ORG_REPORT_TABLE", "<table><caption><h2>Organism Report</h2></caption><tr><th>Accession</th><th>Descr</th><th>Score</th><th>E-value</th></tr><@table_rows@></table><@taxidToSeqsMap@>"},
+    //kOrgReportOrganismHeader s_TagToConstString("ORG_REPORT_ORG_HEADER")
+     { "ORG_REPORT_ORG_HEADER", "<tr><th colspan=\"4\"><a href=\"<@taxBrowserURL@>?id=<@taxid@>\" name=\"<@taxid@>\" title=\"Show taxonomy info for <@scientific_name@> (taxid <@taxid@>)\" target=\"lnktx<@rid@>\"><@scientific_name@> <@common_name@> [<@blast_name_link@>] taxid <@taxid@></th></tr>"},
+    //kOrgReportOrganismHeaderNoTaxConnect s_TagToConstString("ORG_REPORT_ORG_HEADER_NO_TAX_CON")
+    { "ORG_REPORT_ORG_HEADER_NO_TAX_CON", "<tr><th colspan=\"4\"><a href=\"<@taxBrowserURL@>?id=<@taxid@>\" name=\"<@taxid@>\" title=\"Show taxonomy info for <@scientific_name@> (taxid <@taxid@>)\" target=\"lnktx<@rid@>\"><@scientific_name@> <@common_name@> [<@blast_name@>]</th></tr>"},
+    //*****kOrgReportTableHeader   s_TagToConstString("ORG_REPORT_TABLE_HEADER")
+    { "ORG_REPORT_TABLE_HEADER", "<tr><th>Accession</th><th>Description</th><th>Score</th><th>E-value</th></tr>"},
+    //kOrgReportTableRow       s_TagToConstString("ORG_REPORT_TABLE_ROW") 
+    { "ORG_REPORT_TABLE_ROW",  "<tr><td><a title=\"Show report for <@acc@>\" target=\"lnktx<@rid@>\" href=\"<@protocol@>//www.ncbi.nlm.nih.gov/protein/<@acc@>?report=fwwwtax&amp;log$=taxrep&amp;RID=<@rid@>\"><@acc@></a></td><td><@descr_abbr@></td><td><@score@></td><td><@evalue@></td></tr>"},
+    //kTaxIdToSeqsMap          s_TagToConstString("TAXID_TO_SEQ_MAP")
+    { "TAXID_TO_SEQ_MAP", "<input type=\"hidden\" id=\"txForSeq_<@taxid@>\" value=\"<@giList@>\" />"},
+    // kLineageReportTable          s_TagToConstString("LINAGE_REPORT_TABLE")
+    { "LINAGE_REPORT_TABLE", "<table><caption><h2>Linage Report</h2><caption><@table_rows@></table>"},
+    //kLineageReportTableHeader    s_TagToConstString("LINAGE_REPORT_TABLE_HEADER")
+    {"LINAGE_REPORT_TABLE_HEADER", "<tr><th>Organism</th><th>Blast Name</th><th>Score</th><th>Number of Hits</th><th>Description</th></tr>"},
+    //kLineageReportOrganismHeader s_TagToConstString("LINAGE_REPORT_ORG_HEADER") 
+    {"LINAGE_REPORT_ORG_HEADER", "<tr><td><@depth@><a href=\"//<@taxBrowserURL@>?id=<@taxid@>\" title=\"Show taxonomy info for <@scientific_name@> (taxid <@taxid@>)\" target=\"lnktx<@rid@>\"><@scientific_name@></a><td><@blast_name_link@></td><td colspan =\"3\"></td></tr>"},
+    //kLineageReportTableRow       s_TagToConstString("LINAGE_REPORT_TABLE_ROW")
+    {"LINAGE_REPORT_TABLE_ROW", "<tr><td><@depth@><a href=\"//<@taxBrowserURL@>?id=<@taxid@>\" title=\"Show taxonomy info for <@scientific_name@> (taxid <@taxid@>)\" target=\"lnktx<@rid@>\"><@scientific_name@></a></td><td><@blast_name_link@></td><td><@score@></td><td><a href=\"#<@taxid@>\" title=\"Show organism report for <@scientific_name@>\"><@numhits@></a></td><td><a title=\"Show report for <@acc@> <@descr_abbr@>\" target=\"lnktx<@rid@>\" href=\"<@protocol@>//www.ncbi.nlm.nih.gov/protein/<@acc@>?report=genbank&amp;log$=taxrep&amp;RID=<@rid@>\"><@descr_abbr@></a></td></tr>"},
+    //kTaxonomyReportTable          s_TagToConstString("TAXONOMY_REPORT_TABLE")
+    { "TAXONOMY_REPORT_TABLE", "<table><caption><h2>Taxonomy Report</h2><caption><@table_rows@></table>"},
+    //kTaxonomyReportTableHeader    s_TagToConstString("TAXONOMY_REPORT_TABLE_HEADER") 
+    { "TAXONOMY_REPORT_TABLE_HEADER", "<tr><th>Taxonomy</th><th>Number of hits</th><th>Number of organisms</th><th>Description</th></tr>"}, 
+    //kTaxonomyReportOrganismHeader s_TagToConstString("TAXONOMY_REPORT_ORG_HEADER")
+    {"TAXONOMY_REPORT_ORG_HEADER", "<tr><td><@depth@><a href=\"//<@taxBrowserURL@>?id=<@taxid@>\" title=\"Show taxonomy info for <@scientific_name@> (taxid <@taxid@>)\" target=\"lnktx<@rid@>\"><@scientific_name@></a></td><td><@numhits@></td><td><@numOrgs@></td><td><@descr_abbr@></td></tr>"}, 
+    //kTaxonomyReportTableRow       s_TagToConstString("TAXONOMY_REPORT_TABLE_ROW")
+    { "TAXONOMY_REPORT_TABLE_ROW", "<tr><td><@depth@><a href=\"//<@taxBrowserURL@>?id=<@taxid@>\" title=\"Show taxonomy info for <@scientific_name@> (taxid <@taxid@>)\" target=\"lnktx<@rid@>\"><@scientific_name@></a></td><td><@numhits@></td><td><@numOrgs@></td><td><@descr_abbr@></td></tr>"},
+    // kOrgReportTxtTable          s_TagToConstString("ORG_REPORT_TXT_TABLE")
+    {"ORG_REPORT_TXT_TABLE", "<@org_report_caption@>\n<@acc_hd@><@descr_hd@><@score_hd@><@evalue_hd@>\n<@table_rows@>"}, 
+    //kOrgReportTxtOrganismHeader s_TagToConstString("ORG_REPORT_TXT_ORG_HEADER")
+    {"ORG_REPORT_TXT_ORG_HEADER", "<@scientific_name@> <@common_name@> [<@blast_name_link@>] taxid <@taxid@>"},     
+    //kOrgReportTxtOrganismHeaderNoTaxConnect s_TagToConstString("ORG_REPORT_TXT_ORG_HEADER_NO_TAX_CON")
+    {"ORG_REPORT_TXT_ORG_HEADER_NO_TAX_CON", "<@scientific_name@> <@common_name@> [<@blast_name@>]"},
+    //kOrgReportTxtTableHeader    s_TagToConstString("ORG_REPORT_TXT_TABLE_HEADER") 
+    {"ORG_REPORT_TXT_TABLE_HEADER", " <@acc_hd@><@descr_hd@><@score_hd@><@evalue_hd@>\n"},
+    //kOrgReportTxtTableRow       s_TagToConstString("ORG_REPORT_TXT_TABLE_ROW") 
+    {"ORG_REPORT_TXT_TABLE_ROW", " <@acc@><@descr_text@><@score@><@evalue@>\n"},
+    //kOrgReportTxtTableCaption s_TagToConstString("ORG_REPORT_TXT_TABLE_CAPTION") 
+    {"ORG_REPORT_TXT_TABLE_CAPTION", "Organism Report"},
+    //kOrgAccTxtTableHeader  s_TagToConstString("ORG_ACC_TXT_TABLE_HEADER") 
+    { "ORG_ACC_TXT_TABLE_HEADER", "Accession"},
+    //kOrgDescrTxtTableHeader  s_TagToConstString("ORG_DESCR_TXT_TABLE_HEADER") 
+    { "ORG_DESCR_TXT_TABLE_HEADER", "Description"},
+    //kOrgScoreTxtTableHeader s_TagToConstString("ORG_SCORE_TXT_TABLE_HEADER") 
+    { "ORG_SCORE_TXT_TABLE_HEADER", "Score"},
+    //kOrgEValueTxtTableHeader s_TagToConstString("ORG_EVALUE_TXT_TABLE_HEADER") 
+    { "ORG_EVALUE_TXT_TABLE_HEADER", "E-value"},
+});
+
+static string  s_TagToConstString( const string tag_name) 
+{
+  string mappedStr;
+
+  auto cit = kMapTagToString.find(tag_name);  
+  
+ if( cit != kMapTagToString.end()) {
+    mappedStr = cit->second;      
+  } 
+  return mappedStr;  
+}
+
 CTaxFormat::CTaxFormat(const CSeq_align_set& seqalign, 
                        CScope& scope,
                        unsigned int displayOption,
@@ -69,6 +137,16 @@ CTaxFormat::CTaxFormat(const CSeq_align_set& seqalign,
       m_ConnectToTaxServer(connectToTaxServer),
       m_LineLength(lineLength) //used for text output
       
+{
+    x_InitTaxFormat();
+    x_InitTaxInfoMap();    //Sets m_BlastResTaxInfo
+    if(m_ConnectToTaxServer) {        
+        x_LoadTaxTree();
+    }
+}
+
+
+void CTaxFormat::x_InitTaxFormat(void)      
 {
     m_TaxClient = NULL;    
     m_TaxTreeLoaded = false;    
@@ -89,10 +167,6 @@ CTaxFormat::CTaxFormat(const CSeq_align_set& seqalign,
     if(m_ConnectToTaxServer) {        
         x_InitTaxClient();        
     }
-    x_InitTaxInfoMap();    //Sets m_BlastResTaxInfo
-    if(m_ConnectToTaxServer) {        
-        x_LoadTaxTree();
-    }
     
     //set config file
     m_ConfigFile = new CNcbiIfstream(".ncbirc");
@@ -100,32 +174,54 @@ CTaxFormat::CTaxFormat(const CSeq_align_set& seqalign,
     if(m_Reg) {
         m_TaxBrowserURL = m_Reg->Get("TAX_BROWSER","BLASTFMTUTIL");
     }
-    m_TaxBrowserURL = (m_TaxBrowserURL.empty()) ? kTaxBrowserURL : m_TaxBrowserURL;
+    m_TaxBrowserURL = (m_TaxBrowserURL.empty()) ? s_TagToConstString("TAX_BROWSER_URL") : m_TaxBrowserURL;
     m_TaxBrowserURL = CAlignFormatUtil::MapTemplate(m_TaxBrowserURL,"protocol",m_Protocol);
 
     
         
     m_TaxFormatTemplates = new STaxFormatTemplates;        
 
-    m_TaxFormatTemplates->blastNameLink = kBlastNameLink;
-    m_TaxFormatTemplates->orgReportTable = (m_DisplayOption == eHtml) ? kOrgReportTable : kOrgReportTxtTable;
-    m_TaxFormatTemplates->orgReportOrganismHeader = (m_DisplayOption == eHtml) ? kOrgReportOrganismHeader : kOrgReportTxtOrganismHeader; ///< Template for displaying header
-    m_TaxFormatTemplates->orgReportTableHeader  = (m_DisplayOption == eHtml) ? kOrgReportTableHeader : kOrgReportTxtTableHeader; ///< Template for displaying header
-    m_TaxFormatTemplates->orgReportTableRow = (m_DisplayOption == eHtml) ? kOrgReportTableRow : kOrgReportTxtTableRow;
+    m_TaxFormatTemplates->blastNameLink = s_TagToConstString("BLAST_NAME_LINK");
+    m_TaxFormatTemplates->orgReportTable = (m_DisplayOption == eHtml) ? s_TagToConstString("ORG_REPORT_TABLE") : s_TagToConstString("ORG_REPORT_TXT_TABLE");
+    m_TaxFormatTemplates->orgReportOrganismHeader = (m_DisplayOption == eHtml) ? s_TagToConstString("ORG_REPORT_ORG_HEADER") : s_TagToConstString("ORG_REPORT_TXT_ORG_HEADER"); ///< Template for displaying header
+    m_TaxFormatTemplates->orgReportTableHeader  = (m_DisplayOption == eHtml) ? s_TagToConstString("ORG_REPORT_ORG_TABLE_HEADER") : s_TagToConstString("ORG_REPORT_TXT_TABLE_HEADER"); ///< Template for displaying header
+    m_TaxFormatTemplates->orgReportTableRow = (m_DisplayOption == eHtml) ? s_TagToConstString("ORG_REPORT_TABLE_ROW") : s_TagToConstString("ORG_REPORT_TXT_TABLE_ROW");
 
-    m_TaxFormatTemplates->taxIdToSeqsMap = kTaxIdToSeqsMap;
+    m_TaxFormatTemplates->taxIdToSeqsMap = s_TagToConstString("TAXID_TO_SEQ_MAP");
 
-    m_TaxFormatTemplates->lineageReportTable = kLineageReportTable;
-    m_TaxFormatTemplates->lineageReportOrganismHeader = kLineageReportOrganismHeader; ///< Template for displaying header
-    m_TaxFormatTemplates->lineageReportTableHeader  = kLineageReportTableHeader; ///< Template for displaying header
-    m_TaxFormatTemplates->lineageReportTableRow = kLineageReportTableRow;
+    m_TaxFormatTemplates->lineageReportTable = s_TagToConstString("LINAGE_REPORT_TABLE");
+    m_TaxFormatTemplates->lineageReportOrganismHeader = s_TagToConstString("LINAGE_REPORT_ORG_HEADER"); ///< Template for displaying header
+    m_TaxFormatTemplates->lineageReportTableHeader  = s_TagToConstString("LINAGE_REPORT_TABLE_HEADER"); ///< Template for displaying header
+    m_TaxFormatTemplates->lineageReportTableRow = s_TagToConstString("LINAGE_REPORT_TABLE_ROW");
 
-    m_TaxFormatTemplates->taxonomyReportTable           = kTaxonomyReportTable;
-    m_TaxFormatTemplates->taxonomyReportOrganismHeader  = kTaxonomyReportOrganismHeader;    
-    m_TaxFormatTemplates->taxonomyReportTableHeader     = kTaxonomyReportTableHeader;
-    m_TaxFormatTemplates->taxonomyReportTableRow        = kTaxonomyReportTableRow;
+    m_TaxFormatTemplates->taxonomyReportTable           = s_TagToConstString("TAXONOMY_REPORT_TABLE");
+    m_TaxFormatTemplates->taxonomyReportOrganismHeader  = s_TagToConstString("TAXONOMY_REPORT_ORG_HEADER");    
+    m_TaxFormatTemplates->taxonomyReportTableHeader     = s_TagToConstString("TAXONOMY_REPORT_TABLE_HEADER");
+    m_TaxFormatTemplates->taxonomyReportTableRow        = s_TagToConstString("TAXONOMY_REPORT_TABLE_ROW");
 }
 
+CTaxFormat::CTaxFormat(list< pair<string,TTaxId> > &accessionTaxidList,    
+                       CScope& scope,                    
+                       unsigned int displayOption,
+                       bool connectToTaxServer,
+                       unsigned int lineLength)
+                      
+                      
+    : m_AccessionTaxidList(accessionTaxidList),    
+      m_Scope(scope),  
+      m_DisplayOption(displayOption),
+      m_ConnectToTaxServer(connectToTaxServer),
+      m_LineLength(lineLength) //used for text output
+      
+{    
+    x_InitTaxFormat();
+    x_InitTaxInfoMapFromAccList();    //Sets m_BlastResTaxInfo
+    if(m_ConnectToTaxServer) {        
+        x_LoadTaxTree();
+    }
+}
+
+
 CTaxFormat::~CTaxFormat()
 {   
     
@@ -353,15 +449,18 @@ public:
 
 void CTaxFormat::DisplayOrgReport(CNcbiOstream& out) 
 {
-    string orgReportData;  
-    string taxIdToSeqsMap;
+    string orgReportData;      
+
+    if(m_TaxIdToSeqsMap.empty()) {
+        x_InitTaxIdToSeqsMap();
+    }
 
     for(size_t i = 0; i < m_BlastResTaxInfo->orderedTaxids.size(); i++) {
         TTaxId taxid = m_BlastResTaxInfo->orderedTaxids[i];
         STaxInfo seqsForTaxID = m_BlastResTaxInfo->seqTaxInfoMap[taxid];
         
         string orgHeader = (m_ConnectToTaxServer) ? m_TaxFormatTemplates->orgReportOrganismHeader : 
-                                                    (m_DisplayOption == eHtml) ? kOrgReportOrganismHeaderNoTaxConnect : kOrgReportTxtOrganismHeaderNoTaxConnect;        
+                                                    (m_DisplayOption == eHtml) ? s_TagToConstString("ORG_REPORT_ORG_HEADER_NO_TAX_CON") : s_TagToConstString("ORG_REPORT_TXT_ORG_HEADER_NO_TAX_CON");        
         orgHeader = x_MapTaxInfoTemplate(orgHeader,seqsForTaxID);
         string prevTaxid,nextTaxid, hidePrevTaxid, hideNextTaxid, hideTop;
         const string kDisabled = "disabled=\"disabled\"";
@@ -395,29 +494,28 @@ void CTaxFormat::DisplayOrgReport(CNcbiOstream& out)
 
         string oneOrgInfo = orgHeader + orgReportSeqInfo;
         orgReportData += oneOrgInfo; 
-
-        //string taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->taxIdToSeqsMap,"giList",seqsForTaxID.giList);
-        string taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->taxIdToSeqsMap,"giList",seqsForTaxID.accList);
-        taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(taxIdToSeqsRow,"taxid", TAX_ID_TO(TIntId, taxid));
-        taxIdToSeqsMap += taxIdToSeqsRow;
     }   
     orgReportData = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->orgReportTable,"table_rows",orgReportData);
     if(m_DisplayOption == eText) {
-        string reportCaption = CAlignFormatUtil::AddSpaces(kOrgReportTxtTableCaption,m_LineLength,CAlignFormatUtil::eSpacePosToCenter | CAlignFormatUtil::eAddEOLAtLineStart);
+        string orgReportTxtTableCaption = s_TagToConstString("ORG_REPORT_TXT_TABLE_CAPTION");
+        string reportCaption = CAlignFormatUtil::AddSpaces(orgReportTxtTableCaption,m_LineLength,CAlignFormatUtil::eSpacePosToCenter | CAlignFormatUtil::eAddEOLAtLineStart);
         orgReportData = CAlignFormatUtil::MapTemplate(orgReportData,"org_report_caption",reportCaption);
-        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"acc_hd",kOrgAccTxtTableHeader,m_MaxAccLength);                
-        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"descr_hd",kOrgDescrTxtTableHeader,m_MaxDescrLength,CAlignFormatUtil::eSpacePosToCenter);                
-        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"score_hd",kOrgScoreTxtTableHeader,m_MaxScoreLength);
-        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"evalue_hd",kOrgEValueTxtTableHeader,m_MaxEvalLength);        
+        string orgAccTxtTableHeader = s_TagToConstString("ORG_ACC_TXT_TABLE_HEADER");
+        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"acc_hd",orgAccTxtTableHeader,m_MaxAccLength);                
+        string orgDescrTxtTableHeader = s_TagToConstString("ORG_DESCR_TXT_TABLE_HEADER");
+        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"descr_hd",orgDescrTxtTableHeader,m_MaxDescrLength,CAlignFormatUtil::eSpacePosToCenter);                
+        string orgScoreTxtTableHeader = s_TagToConstString("ORG_SCORE_TXT_TABLE_HEADER");
+        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"score_hd",orgScoreTxtTableHeader,m_MaxScoreLength);
+        string orgEValueTxtTableHeader = s_TagToConstString("ORG_EVALUE_TXT_TABLE_HEADER");
+        orgReportData = CAlignFormatUtil::MapSpaceTemplate(orgReportData,"evalue_hd",orgEValueTxtTableHeader,m_MaxEvalLength);        
     }
     else {        
-        orgReportData = CAlignFormatUtil::MapTemplate(orgReportData,"taxidToSeqsMap",taxIdToSeqsMap);    
+        orgReportData = CAlignFormatUtil::MapTemplate(orgReportData,"taxidToSeqsMap",m_TaxIdToSeqsMap);    
     }
     out << orgReportData;
 }
 
  
-
 void CTaxFormat::DisplayLineageReport(CNcbiOstream& out) 
 {
     x_InitLineageReport();    
@@ -428,7 +526,6 @@ void CTaxFormat::DisplayLineageReport(CNcbiOstream& out)
             string lngReportTableRow;
             unsigned int depth = 0;
             STaxInfo alnSeqTaxInfo = *iter;
-//            taxInfo.lineage.push_back(iter->taxid);            
             for(size_t i = 0; i< alnSeqTaxInfo.lineage.size();i++) {
                 TTaxId taxid = alnSeqTaxInfo.lineage[i];
                 if(header) {                                        
@@ -452,11 +549,11 @@ void CTaxFormat::DisplayLineageReport(CNcbiOstream& out)
             header = false;            
      }
      string lngReportData = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->lineageReportTable,"table_rows",lngReportRows);
+     lngReportData = CAlignFormatUtil::MapTemplate(lngReportData,"taxidToSeqsMap",m_TaxIdToSeqsMap);    
+    
      out << lngReportData;
 }
 
-
-
 void CTaxFormat::DisplayTaxonomyReport(CNcbiOstream& out) 
 {
     x_InitTaxReport();    
@@ -536,6 +633,10 @@ void CTaxFormat::x_InitLineageReport(void)
         x_InitOrgTaxMetaData();
     }
     x_InitLineageMetaData();
+
+    if(m_TaxIdToSeqsMap.empty()) {
+        x_InitTaxIdToSeqsMap();
+    }
 }
 
 
@@ -567,7 +668,7 @@ void CTaxFormat::x_LoadTaxTree(void)
                     }
                     STaxInfo &taxInfo = GetAlignTaxInfo(tax_id);
                     taxInfo.taxid = newTaxid;
-                    for(size_t j = 0; i < taxInfo.seqInfoList.size(); j++) {
+                    for(size_t j = 0; j < taxInfo.seqInfoList.size(); j++) {
                         SSeqInfo* seqInfo = taxInfo.seqInfoList[j];                        
                         seqInfo->taxid = newTaxid;
                     }        
@@ -598,10 +699,20 @@ void CTaxFormat::x_LoadTaxTree(void)
         else {
             const string& err = m_TaxClient->GetLastError();
             NCBI_THROW(CException, eUnknown,"Cannot connect to tax server. " + err);
-        }
-    }
+        }     
+    }           
 }
 
+void CTaxFormat::x_InitTaxIdToSeqsMap(void)
+{
+    for(size_t i = 0; i < m_BlastResTaxInfo->orderedTaxids.size(); i++) {
+        TTaxId taxid = m_BlastResTaxInfo->orderedTaxids[i];
+        STaxInfo seqsForTaxID = m_BlastResTaxInfo->seqTaxInfoMap[taxid];
+        string taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->taxIdToSeqsMap,"giList",seqsForTaxID.accList);
+        taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(taxIdToSeqsRow,"taxid", TAX_ID_TO(TIntId, taxid));
+        m_TaxIdToSeqsMap += taxIdToSeqsRow;
+    }
+}
 
 void CTaxFormat::x_PrintTaxInfo(vector <TTaxId> taxids, string title)
 {
@@ -786,10 +897,14 @@ bool CTaxFormat::isTaxidInAlign(TTaxId taxid)
 
 void CTaxFormat::x_InitTextFormatInfo(CTaxFormat::SSeqInfo *seqInfo)
 {
-    m_MaxAccLength = max(max(m_MaxAccLength,(unsigned int)seqInfo->label.size()),(unsigned int)kOrgAccTxtTableHeader.size());    
-    m_MaxDescrLength = max(max(m_MaxDescrLength,(unsigned int)seqInfo->title.size()),(unsigned int)kOrgDescrTxtTableHeader.size());        
-    m_MaxScoreLength = max(max(m_MaxScoreLength,(unsigned int)seqInfo->bit_score.size()),(unsigned int)kOrgScoreTxtTableHeader.size());    
-    m_MaxEvalLength = max(max(m_MaxEvalLength,(unsigned int)seqInfo->evalue.size()),(unsigned int)kOrgEValueTxtTableHeader.size());    
+    string orgAccTxtTableHeader = s_TagToConstString("ORG_ACC_TXT_TABLE_HEADER");
+    m_MaxAccLength = max(max(m_MaxAccLength,(unsigned int)seqInfo->label.size()),(unsigned int)orgAccTxtTableHeader.size());    
+    string orgDescrTxtTableHeader = s_TagToConstString("ORG_DESCR_TXT_TABLE_HEADER");
+    m_MaxDescrLength = max(max(m_MaxDescrLength,(unsigned int)seqInfo->title.size()),(unsigned int)orgDescrTxtTableHeader.size());        
+    string orgScoreTxtTableHeader = s_TagToConstString("ORG_SCORE_TXT_TABLE_HEADER");
+    m_MaxScoreLength = max(max(m_MaxScoreLength,(unsigned int)seqInfo->bit_score.size()),(unsigned int)orgScoreTxtTableHeader.size());    
+    string orgEValueTxtTableHeader = s_TagToConstString("ORG_EVALUE_TXT_TABLE_HEADER");
+    m_MaxEvalLength = max(max(m_MaxEvalLength,(unsigned int)seqInfo->evalue.size()),(unsigned int)orgEValueTxtTableHeader.size());    
     m_MaxDescrLength =  m_LineLength - m_MaxAccLength - m_MaxScoreLength - m_MaxEvalLength - 4;//4 for spaces in between            
  }
 
@@ -896,7 +1011,7 @@ void CTaxFormat::x_InitTaxInfoMap(void)
                             TTaxId taxid;
                             CAlignFormatUtil::GetDisplayIds(handle,*seqInfo->seqID,use_this_seqid,&dispGI,&taxid,&dispSeqID);                            
                             if(dispGI == ZERO_GI && !dispSeqID.empty()) {
-                                dispSeqID = seqInfo->label; //accesion without version
+                                dispSeqID = seqInfo->label; //accession without version
                             }                                
                         }                                        
                         seqInfo->displGi = dispGI;
@@ -921,6 +1036,17 @@ void CTaxFormat::x_InitTaxInfoMap(void)
     }                
 }
 
+
+void CTaxFormat::x_InitTaxInfoMapFromAccList(void)
+{
+    m_BlastResTaxInfo = new SBlastResTaxInfo;    
+    std::list<std::pair<std::string,TTaxId> >::iterator  at_it;    
+    for( at_it = m_AccessionTaxidList.begin(); at_it != m_AccessionTaxidList.end(); at_it++){
+      x_InitBlastDBTaxInfo(*at_it);        
+    }    
+}
+
+
 void CTaxFormat::x_InitBlastDBTaxInfo(CTaxFormat::SSeqInfo *seqInfo)    
 {
     TTaxId taxid = seqInfo->taxid;
@@ -950,6 +1076,43 @@ void CTaxFormat::x_InitBlastDBTaxInfo(CTaxFormat::SSeqInfo *seqInfo)
         m_BlastResTaxInfo->seqTaxInfoMap[taxid].seqInfoList.push_back(seqInfo);
     }    
 }
+       
+
+
+void CTaxFormat::x_InitBlastDBTaxInfo(pair<string,TTaxId> &accTaxidPair)    
+{
+    TTaxId taxid = accTaxidPair.second;
+    string accession = accTaxidPair.first;
+
+    SSeqInfo *seqInfo = new SSeqInfo;
+	seqInfo->gi = ZERO_GI;	
+	seqInfo->label =  accession;
+    
+    seqInfo->dispSeqID = seqInfo->label;    
+    seqInfo->taxid = taxid;	
+
+    if(m_BlastResTaxInfo->seqTaxInfoMap.count(taxid) == 0){		                            
+        SSeqDBTaxInfo taxInfo;
+        CSeqDB::GetTaxInfo(taxid, taxInfo);
+
+        STaxInfo seqsForTaxID;
+        seqsForTaxID.taxid = taxid;
+        seqsForTaxID.commonName = taxInfo.common_name;
+        seqsForTaxID.scientificName = taxInfo.scientific_name;
+        seqsForTaxID.blastName = taxInfo.blast_name;                    
+        seqsForTaxID.giList = NStr::NumericToString(seqInfo->gi);        
+        seqsForTaxID.accList = accession;
+        x_InitBlastNameTaxInfo(seqsForTaxID);
+        seqsForTaxID.seqInfoList.push_back(seqInfo);
+		m_BlastResTaxInfo->seqTaxInfoMap.insert(CTaxFormat::TSeqTaxInfoMap::value_type(taxid,seqsForTaxID));  
+        m_BlastResTaxInfo->orderedTaxids.push_back(taxid);
+	}
+    else {        
+        m_BlastResTaxInfo->seqTaxInfoMap[taxid].accList += ", ";
+        m_BlastResTaxInfo->seqTaxInfoMap[taxid].accList += accession;
+        m_BlastResTaxInfo->seqTaxInfoMap[taxid].seqInfoList.push_back(seqInfo);
+    }    
+}
 
 
 string CTaxFormat::x_MapSeqTemplate(string seqTemplate, STaxInfo &taxInfo)
@@ -1024,6 +1187,5 @@ string CTaxFormat::x_MapTaxInfoTemplate(string tableRowTemplate,STaxInfo &taxInf
 }
 
 
-
 END_SCOPE(align_format)
 END_NCBI_SCOPE
