@@ -1,4 +1,4 @@
-/*  $Id: showdefline_unit_test.cpp 631033 2021-05-12 14:09:56Z ivanov $
+/*  $Id: showdefline_unit_test.cpp 663678 2023-02-27 15:02:25Z fongah2 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -71,7 +71,7 @@ struct CShowBlastDeflineTest : public CShowBlastDefline {
     static void GetDeflineInfo(CBlastOM::ELocation location)
     {
         CNcbiIfstream is("data/showdefline-cppunit.aln");
-        auto_ptr<CObjectIStream> in(CObjectIStream::Open(eSerial_AsnText, is));
+        unique_ptr<CObjectIStream> in(CObjectIStream::Open(eSerial_AsnText, is));
         CRef<CSeq_annot> san(new CSeq_annot);
         *in >> *san;
         
@@ -96,9 +96,9 @@ struct CShowBlastDeflineTest : public CShowBlastDefline {
         
         int i = 0;
         ITERATE(CSeq_align_set::Tdata, iter, seqalign->Get()){
-            auto_ptr<CShowBlastDefline::SScoreInfo>
+            unique_ptr<CShowBlastDefline::SScoreInfo>
                 si(sbd.x_GetScoreInfo(**iter, 1));
-            auto_ptr<CShowBlastDefline::SDeflineInfo>
+            unique_ptr<CShowBlastDefline::SDeflineInfo>
                 dl(sbd.x_GetDeflineInfo(si->id, si->use_this_seqid, 1));
             CShowBlastDeflineTest::TestData(dl.get(), si.get(), i);
             i++;
@@ -114,7 +114,7 @@ struct CShowBlastDeflineTest : public CShowBlastDefline {
                   int index)
     {
         string defline[] = {"adenosine deaminase [Rattus norvegicus]", 
-                            "adenosine deaminase [Mus musculus]"};
+                            "adenosine deaminase isoform 1 [Mus musculus]"};
         string evalue_string[] = {"0.0", "0.0"};
         string bit_string[] = {"734", "677"};
         int sum_n[] = {1, 1};
