@@ -1,4 +1,4 @@
-/*  $Id: alnmerger.cpp 601341 2020-02-05 20:27:33Z vasilche $
+/*  $Id: alnmerger.cpp 670464 2023-07-25 17:29:42Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -163,7 +163,7 @@ CAlnMixMerger::x_Merge()
 
     first_refseq = true; // mark the first loop
 
-    x_SetTaskTotal(m_Matches.size());
+    x_SetTaskTotal(int(m_Matches.size()));
 
     while (true) {
 
@@ -279,7 +279,7 @@ CAlnMixMerger::x_Merge()
 
         // save the match info into the segments map
         if (seq1) {
-            x_SetTaskCompleted(m_MatchIdx++);
+            x_SetTaskCompleted(int(m_MatchIdx++));
 
             // this match is used, erase from seq1 list
             if ( !first_refseq ) {
@@ -818,7 +818,7 @@ CAlnMixMerger::x_Merge()
             }
         }
     }
-    x_SetTaskCompleted(m_MatchIdx);
+    x_SetTaskCompleted(int(m_MatchIdx));
     
     m_AlnMixSequences->BuildRows();
     m_AlnMixSegments->Build(m_MergeFlags & fGapJoin,
@@ -1156,9 +1156,9 @@ void
 CAlnMixMerger::x_CreateDenseg()
 {
     int numrow  = 0,
-        numrows = m_Rows.size();
+        numrows = int(m_Rows.size());
     int numseg  = 0,
-        numsegs = m_AlnMixSegments->m_Segments.size();
+        numsegs = int(m_AlnMixSegments->m_Segments.size());
     int num     = numrows * numsegs;
 
     m_DS = new CDense_seg();
@@ -1207,7 +1207,7 @@ CAlnMixMerger::x_CreateDenseg()
             starts[offset + start_its_i->first->m_RowIdx] =
                 start_its_i->second->first;
 
-            if (start_its_i->second->first != -1) {
+            if (start_its_i->second->first != kInvalidSeqPos) {
                 row_empty[start_its_i->first->m_RowIdx] = false;
             }
         }
