@@ -1,4 +1,4 @@
-/*  $Id: score_builder_base.cpp 667197 2023-05-10 13:36:27Z mozese2 $
+/*  $Id: score_builder_base.cpp 672447 2023-09-07 19:05:43Z mozese2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1025,7 +1025,7 @@ void CScoreBuilderBase::AddSplignScores(const CSeq_align& align,
     typedef TSpliced::TExons TExons;
     const TExons & exons (spliced.GetExons());
 
-    size_t matches (0),
+    TSeqPos matches (0),
         aligned_query_bases (0),  // matches, mismatches and indels
         aln_length_exons (0),
         aln_length_gaps (0),
