@@ -1,4 +1,4 @@
-/*  $Id: sparse_aln.cpp 601341 2020-02-05 20:27:33Z vasilche $
+/*  $Id: sparse_aln.cpp 671390 2023-08-14 14:22:17Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -699,7 +699,7 @@ string& CSparseAln::GetAlnSeqString(TNumrow row,
         ++it;
         is_first_seg = false;
     }
-    if (translate  &&  aln_range.GetLength() >= trim_from + trim_to) {
+    if (translate  &&  (size_t)aln_range.GetLength() >= trim_from + trim_to) {
         buffer.resize((aln_range.GetLength() - trim_from - trim_to) / 3);
     }
     return buffer;
