@@ -1,4 +1,4 @@
-/*  $Id: seqdb.cpp 683929 2024-06-10 12:07:52Z ivanov $
+/*  $Id: seqdb.cpp 696151 2025-04-08 18:26:06Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,7 @@
 
 BEGIN_NCBI_SCOPE
 
-const string CSeqDB::kOidNotFound("OID not found");
+const char* CSeqDB::kOidNotFound = "OID not found";
 
 /// Helper function to translate enumerated type to character.
 ///
@@ -643,7 +643,6 @@ CSeqDB::GetDate(const string   & dbname,
 {
     vector<string> vols;
     CSeqDB::FindVolumePaths(dbname, seqtype, vols);
-    string fmt = "b d, Y  H:m P";
     CTime retv;
     char date[128];
     ITERATE(vector<string>, vol, vols) {
@@ -658,7 +657,7 @@ CSeqDB::GetDate(const string   & dbname,
             f.read(s, 4);
             offset = SeqDB_GetStdOrd((Uint4 *) s);
             f.read(date, offset);
-            CTime d(string(date), fmt);
+            CTime d(string(date), kBlastDbDateFormat);
             if (retv.IsEmpty() || d > retv) {
                 retv = d;
             }
@@ -855,6 +854,7 @@ bool CSeqDB::GiToPig(TGi gi, int & pig) const
 {
     ////m_Impl->Verify();
     bool rv = false;
+    pig = -1;
 
     int oid(0);
 
@@ -1728,8 +1728,7 @@ CRef<CBlast_db_metadata> CSeqDB::GetDBMetaData(string user_path)
 	m->SetNumber_of_sequences(num_seqs);
 
 	CTimeFormat timeFmt = CTimeFormat::GetPredefined(CTimeFormat::eISO8601_DateTimeSec);
-	string fmt = "b d, Y  H:m P";
-	CTime date(GetDate(), fmt);
+	CTime date(GetDate(), kBlastDbDateFormat);
 	m->SetLast_updated(date.AsString(timeFmt));
 
 	Int8 disk_bytes(0), cached_bytes(0);
