@@ -1,4 +1,4 @@
-/*  $Id: seqdbalias.cpp 651939 2022-06-28 12:33:29Z fongah2 $
+/*  $Id: seqdbalias.cpp 696148 2025-04-08 18:25:33Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -311,7 +311,9 @@ void CSeqDBAliasNode::x_ResolveNames(char prot_nucl, CSeqDBLockHold & locked)
             ostringstream oss;
             oss << "No alias or index file found for " << p_or_n
                 << " database [" << m_DBList[i].GetBasePathS()
-                << "] in search path [" << search_path << "]";
+                << "] in search path [" << search_path << "]. "
+                << "Please verify the spelling of the BLAST "
+                << "database and its molecule type.";
             
             string msg(oss.str());
             
