@@ -1,4 +1,4 @@
-/*  $Id: seqdbcommon.cpp 672619 2023-09-12 12:55:53Z ivanov $
+/*  $Id: seqdbcommon.cpp 696135 2025-04-08 18:23:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -2638,6 +2638,7 @@ void SeqDB_GetFileExtensions(bool db_is_protein, vector<string>& extn, EBlastDbV
     }
     extn.push_back(kExtnMol + "pi");   // ISAM PIG index file
     extn.push_back(kExtnMol + "pd");   // ISAM PIG data file
+    extn.push_back(kExtnMol + "js");   // BLASTDB metadata
     if (dbver == eBDB_Version5) {
     	vector<string> lmdbs;
     	SeqDB_GetLMDBFileExtensions(db_is_protein, lmdbs);
@@ -2652,6 +2653,7 @@ void SeqDB_GetFileExtensions(bool db_is_protein, vector<string>& extn, EBlastDbV
     extn.push_back(kExtnMol + "hd");   // ISAM sequence hash data file
     extn.push_back(kExtnMol + "ti");   // ISAM trace id index file
     extn.push_back(kExtnMol + "td");   // ISAM trace id data file
+    extn.push_back(kExtnMol + "xm");   // OID mask file to exclude models
 }
 
 
