@@ -1,4 +1,4 @@
-/*  $Id: seqdbgilistset.cpp 618513 2020-10-21 16:07:12Z grichenk $
+/*  $Id: seqdbgilistset.cpp 695456 2025-03-24 13:01:59Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -131,7 +131,7 @@ public:
             (int(m_GisOids.size() * sizeof(m_GisOids[0])) +
              int(m_TisOids.size() * sizeof(m_TisOids[0])));
 
-        atlas.RegisterExternal(m_VectorMemory, vector_size, locked);
+        //atlas.RegisterExternal(m_VectorMemory, vector_size, locked);
         // TODO m_VectorMemory for seqid_list
     }
 
