@@ -1,4 +1,4 @@
-/*  $Id: seqdbvol.cpp 631553 2021-05-19 13:52:19Z ivanov $
+/*  $Id: seqdbvol.cpp 696152 2025-04-08 18:26:16Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,6 +43,7 @@
 #include <serial/objostrasnb.hpp>
 #include <serial/serial.hpp>
 #include <corelib/ncbimtx.hpp>
+#include <corelib/ncbi_safe_static.hpp>
 
 #include <sstream>
 
@@ -79,8 +80,10 @@ CSeqDBVol::CSeqDBVol(CSeqDBAtlas        & atlas,
     : m_Atlas        (atlas),
       m_IsAA         (prot_nucl == 'p'),
       m_VolName      (name),
+	  m_HasGis       (true),
       m_TaxCache     (256),
       m_MemBit       (0),
+      m_OidMaskType  (0),
       m_VolStart     (vol_start),
       m_VolEnd       (0),
       m_DeflineCache (256),
@@ -92,6 +95,7 @@ CSeqDBVol::CSeqDBVol(CSeqDBAtlas        & atlas,
 {
     if (user_list) {
         m_UserGiList.Reset(user_list);
+        m_OidMaskType = static_cast<int>(m_UserGiList->GetMaskOpts());
     }
     if (neg_list) {
         m_NegativeList.Reset(neg_list);
@@ -159,9 +163,13 @@ CSeqDBVol::x_OpenGiFile(void) const{
     if (m_IsamGi.NotEmpty()) {
     	m_IsamGi->AddReference();
     }
-    else if (CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'n') &&
-			   m_Idx->GetNumOIDs() != 0) {
+    else if ((m_Idx->GetNumOIDs() != 0) && m_HasGis) {
+    	try {
         m_IsamGi = new CSeqDBIsam(m_Atlas, m_VolName, (m_IsAA?'p':'n'), 'n', eGiId);
+    	} catch(CException &) {
+    		m_IsamGi.Reset();
+    		m_HasGis = false;
+    	}
     }
 }
 
@@ -230,8 +238,8 @@ CSeqDBVol::x_UnleaseTiFile(void) const{
 
 void
 CSeqDBVol::x_OpenHashFile(void) const{
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
+    DEFINE_STATIC_FAST_MUTEX(mtx);
+    CFastMutexGuard mtx_guard(mtx);
     if (!m_HashFileOpened &&
          CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'h') &&
          m_Idx->GetNumOIDs() != 0) {
@@ -247,8 +255,8 @@ CSeqDBVol::x_OpenHashFile(void) const{
 
 void
 CSeqDBVol::x_OpenOidFile(void) const{
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
+    DEFINE_STATIC_FAST_MUTEX(mtx);
+    CFastMutexGuard mtx_guard(mtx);
     if (!m_OidFileOpened &&
          CSeqDBGiIndex::IndexExists(m_VolName, (m_IsAA?'p':'n')) &&
          m_Idx->GetNumOIDs() != 0) {
@@ -335,6 +343,9 @@ int CSeqDBVol::GetSeqLengthApprox(int oid) const
     return (whole_bytes * 4) + (oid & 0x03);
 }
 
+/// Translation table type
+typedef vector<Uint1> TTable;
+
 /// Build NA2 to NcbiNA4 translation table
 ///
 /// This builds a translation table for nucleotide data.  The table
@@ -345,17 +356,15 @@ int CSeqDBVol::GetSeqLengthApprox(int oid) const
 ///
 /// @return
 ///    The NA2 to NA4 translation table
-static vector<Uint1>
-s_SeqDBMapNA2ToNA4Setup()
+static void
+s_SeqDBMapNA2ToNA4Setup(TTable& translated)
 {
-    vector<Uint1> translated;
     translated.resize(512);
 
     Uint1 convert[16] = { 0x11,  0x12, 0x14, 0x18,
                           0x21,  0x22, 0x24, 0x28,
                           0x41,  0x42, 0x44, 0x48,
                           0x81,  0x82, 0x84, 0x88 };
-
     Int2 pair1 = 0;
     Int2 pair2 = 0;
 
@@ -367,10 +376,17 @@ s_SeqDBMapNA2ToNA4Setup()
             translated[index+1] = convert[pair2];
         }
     }
-
-    return translated;
 }
 
+class CNA2ToNA4MapTable
+{
+public:
+	CNA2ToNA4MapTable () {
+		s_SeqDBMapNA2ToNA4Setup(storage);
+	}
+	TTable storage;
+};
+
 /// Convert sequence data from NA2 to NA4 format
 ///
 /// This uses a translation table to convert nucleotide data.  The
@@ -388,7 +404,8 @@ s_SeqDBMapNA2ToNA4(const char   * buf2bit,
                    vector<char> & buf4bit,
                    int            base_length)
 {
-    static vector<Uint1> expanded = s_SeqDBMapNA2ToNA4Setup();
+    static CSafeStatic<CNA2ToNA4MapTable> t;
+    TTable& expanded = t.Get().storage;
 
     int estimated_length = (base_length + 1)/2;
     int bytes = 0;
@@ -434,15 +451,12 @@ s_SeqDBMapNA2ToNA4(const char   * buf2bit,
 ///
 /// @return
 ///    The NA2 to NA8 translation table
-static vector<Uint1>
-s_SeqDBMapNA2ToNA8Setup()
+static void
+s_SeqDBMapNA2ToNA8Setup(TTable& translated)
 {
     // Builds a table; each two bit slice holds 0,1,2 or 3.  These are
     // converted to whole bytes containing 1,2,4, or 8, respectively.
 
-    vector<Uint1> translated;
-    translated.reserve(1024);
-
     for(int i = 0; i<256; i++) {
         int p1 = (i >> 6) & 0x3;
         int p2 = (i >> 4) & 0x3;
@@ -454,10 +468,16 @@ s_SeqDBMapNA2ToNA8Setup()
         translated.push_back(1 << p3);
         translated.push_back(1 << p4);
     }
-
-    return translated;
 }
 
+class CNA2ToNA8MapTable
+{
+public:
+	CNA2ToNA8MapTable () {
+		s_SeqDBMapNA2ToNA8Setup(storage);
+	}
+	TTable storage;
+};
 /// Convert sequence data from NA2 to NA8 format
 ///
 /// This uses a translation table to convert nucleotide data.  The
@@ -485,7 +505,8 @@ s_SeqDBMapNA2ToNA8(const char        * buf2bit,
     // data and is maintained to point at the next unused byte of
     // input data.
 
-    static vector<Uint1> expanded = s_SeqDBMapNA2ToNA8Setup();
+    static CSafeStatic<CNA2ToNA8MapTable> t;
+    TTable& expanded = t.Get().storage;
 
     int pos = range.begin;
 
@@ -1535,9 +1556,8 @@ int CSeqDBVol::GetAmbigPartialSeq(int                oid,
 	    NCBI_THROW(CSeqDBException, eFileErr, "Error: could not get sequence or range.");
 	}
 
-
-    int num_ranges = partial_ranges->size();
-    if ((*partial_ranges)[num_ranges - 1].second > base_length) {
+    int num_ranges = static_cast<int>(partial_ranges->size());
+    if ((*partial_ranges)[num_ranges - 1].second > static_cast<TSeqPos>(base_length)) {
         NCBI_THROW(CSeqDBException, eFileErr, "Error: region beyond sequence range.");
     }
 
@@ -1630,14 +1650,21 @@ int CSeqDBVol::x_GetAmbigSeq(int                oid,
         //
         // 1. No range is specified by the user.
         // 2. We have cached ranges.
-
-        TRangeCache::iterator rciter = m_RangeCache.find(oid);
+        TRangeList range_set;
         bool use_range_set = true;
+        {
+    	CFastMutexGuard mtx_guard(m_MtxCachedRange);
+        TRangeCache::iterator rciter = m_RangeCache.find(oid);
         if (region
          || rciter == m_RangeCache.end()
          || rciter->second->GetRanges().empty()
-         || CSeqDBRangeList::ImmediateLength() >= base_length)
+         || CSeqDBRangeList::ImmediateLength() >= base_length) {
             use_range_set = false;
+        }
+        else {
+            range_set = rciter->second->GetRanges();
+        }
+        }
 
         if (!use_range_set) {
             s_SeqDBMapNA2ToNA8(tmp, seq, range);
@@ -1648,8 +1675,6 @@ int CSeqDBVol::x_GetAmbigSeq(int                oid,
         } else {
 
             _ASSERT (!region);
-            const TRangeList & range_set = rciter->second->GetRanges();
-
             // Place 'fence' sentinel bytes around each range; this is done
             // before any of the range data is mapped so that the range data
             // is free to replace the sentinel bytes if needed; that would
@@ -1706,7 +1731,7 @@ void SeqDB_UnpackAmbiguities(const CTempString & sequence,
 
     const char * seq_buffer = sequence.data();
 
-    int whole_bytes = sequence.length() - 1;
+    int whole_bytes = static_cast<int>(sequence.length()) - 1;
     int remainder = sequence[whole_bytes] & 3;
     int base_length = (whole_bytes * 4) + remainder;
 
@@ -1915,6 +1940,19 @@ bool s_IncludeDefline_Taxid(const CBlast_def_line & def, const set<TTaxId> & use
 	return false;
 }
 
+bool s_IncludeDefline_MaskFilter(const CBlast_def_line & def, Uint8 mask)
+{
+    ITERATE(list< CRef<CSeq_id> >, seqid, def.GetSeqid()) {
+    	if (mask & EOidMaskType::fExcludeModel) {
+    		if((*seqid)->IdentifyAccession() & CSeq_id::fAcc_predicted) {
+    			return false;
+    		}
+    	}
+    }
+	return true;
+}
+
+
 bool s_IncludeDefline_NegativeTaxid(const CBlast_def_line & def, const set<TTaxId> & user_tax_ids)
 {
 	CBlast_def_line::TTaxIds taxid_set = def.GetTaxIds();
@@ -1956,7 +1994,7 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
 
     bool id_filter = x_HaveIdFilter();
 
-    if (id_filter || m_MemBit) {
+    if (id_filter || m_MemBit || m_OidMaskType) {
         // Create the memberships mask (should this be fixed to allow
         // membership bits greater than 32?)
 
@@ -2014,6 +2052,10 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
             	if(have_memb && (!m_NegativeList.Empty()) && (m_NegativeList->GetNumTaxIds() > 0)) {
                		have_memb = s_IncludeDefline_NegativeTaxid(defline, m_NegativeList->GetTaxIdsList());
             	}
+
+            	if(have_memb && (m_OidMaskType != 0)) {
+               		have_memb = s_IncludeDefline_MaskFilter(defline, m_OidMaskType);
+            	}
             }
 
             if (! have_memb) {
@@ -2068,7 +2110,7 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
 
     bool id_filter = x_HaveIdFilter();
 
-    if (id_filter || m_MemBit) {
+    if (id_filter || m_MemBit || m_OidMaskType) {
         // Create the memberships mask (should this be fixed to allow
         // membership bits greater than 32?)
 
@@ -2126,6 +2168,11 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
             	if(have_memb && (!m_NegativeList.Empty()) && (m_NegativeList->GetNumTaxIds() > 0)) {
                		have_memb = s_IncludeDefline_NegativeTaxid(defline, m_NegativeList->GetTaxIdsList());
             	}
+
+            	if(have_memb && (m_OidMaskType != 0)) {
+               		have_memb = s_IncludeDefline_MaskFilter(defline, m_OidMaskType);
+            	}
+
             }
 
             if (! have_memb) {
@@ -3355,7 +3402,7 @@ int CSeqDBVol::GetColumnId(const string   & title,
 
     for(size_t i = 0; i < m_Columns.size(); i++) {
         if (m_Columns[i]->GetTitle() == title) {
-            return i;
+            return static_cast<int>(i);
         }
     }
 
