@@ -1,4 +1,4 @@
-/*  $Id: seqdb_perf.cpp 618513 2020-10-21 16:07:12Z grichenk $
+/*  $Id: seqdb_perf.cpp 639174 2021-10-14 12:36:57Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -137,7 +137,8 @@ CSeqDBPerfApp::x_ScanDatabase()
     }
     LOG_POST(Info << "Will go over " << oids2iterate.size() << " sequences");
 
-#if defined(NCBI_COMPILER_GCC) && (NCBI_COMPILER_VERSION >= 900)
+#if (defined(NCBI_COMPILER_GCC) && (NCBI_COMPILER_VERSION >= 900)) || \
+    (defined(NCBI_COMPILER_ICC) && (NCBI_COMPILER_VERSION >= 2100))
     #pragma omp parallel default(none) num_threads(m_DbHandles.size()) \
                          shared(oids2iterate,kScanUncompressed) if(m_DbHandles.size() > 1)
 #else
@@ -208,7 +209,8 @@ CSeqDBPerfApp::x_InitApplicationData()
 
 
     if (args["multi_threaded_creation"]) {
-#if defined(NCBI_COMPILER_GCC) && (NCBI_COMPILER_VERSION >= 900)
+#if (defined(NCBI_COMPILER_GCC) && (NCBI_COMPILER_VERSION >= 900)) || \
+    (defined(NCBI_COMPILER_ICC) && (NCBI_COMPILER_VERSION >= 2100))
         #pragma omp parallel default(none) shared(kDbName, kNumThreads, kSeqType) num_threads(kNumThreads)
 #else
         #pragma omp parallel default(none) shared(kDbName, kNumThreads) num_threads(kNumThreads)
@@ -282,7 +284,7 @@ void CSeqDBPerfApp::Init()
 {
     HideStdArgs(fHideConffile | fHideFullVersion | fHideXmlHelp | fHideDryRun);
 
-    auto_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
+    unique_ptr<CArgDescriptions> arg_desc(new CArgDescriptions);
 
     // Specify USAGE context
     arg_desc->SetUsageContext(GetArguments().GetProgramBasename(),
