@@ -1,4 +1,4 @@
-/*  $Id: build_db.cpp 631653 2021-05-20 02:38:30Z vakatov $
+/*  $Id: build_db.cpp 674875 2023-11-07 15:39:13Z camacho $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -471,7 +471,8 @@ bool CBuildDatabase::x_EditAndAddBioseq(CConstRef<objects::CBioseq>   bs,
                                         bool						  add_pig)
 {
     CRef<CBlast_def_line_set> headers =
-        CWriteDB::ExtractBioseqDeflines(*bs, m_ParseIDs, m_LongIDs);
+        CWriteDB::ExtractBioseqDeflines(*bs, m_ParseIDs, m_LongIDs, 
+                m_ScanBioseq4CFastaReaderUsrObjct);
 
     x_EditHeaders(headers);
 
@@ -706,8 +707,7 @@ CFastaBioseqSource::CFastaBioseqSource(CNcbiIstream & fasta_file,
     m_LineReader.Reset(new CBufferedLineReader(fasta_file));
     typedef CFastaReader::EFlags TFlags;
 
-    int iflags = CFastaReader::fAllSeqIds |
-                 CFastaReader::fForceType;
+    int iflags = CFastaReader::fForceType;
 
     if (is_protein) {
         iflags |= CFastaReader::fAssumeProt;
@@ -717,7 +717,7 @@ CFastaBioseqSource::CFastaBioseqSource(CNcbiIstream & fasta_file,
     }
 
     if (parse_ids) {
-        iflags |= CFastaReader::fAllSeqIds | CFastaReader::fRequireID;
+        iflags |= CFastaReader::fRequireID;
         // parse bare accessions
         if (!long_ids) {
             iflags |= CFastaReader::fParseRawID;
@@ -736,6 +736,15 @@ CFastaBioseqSource::CFastaBioseqSource(CNcbiIstream & fasta_file,
     m_FastaReader->IgnoreProblem(ILineError::eProblem_TooManyAmbiguousResidues);
     m_FastaReader->IgnoreProblem(ILineError::eProblem_TooLong);
 
+    CNcbiApplication* app = CNcbiApplication::Instance();
+    if (app) {
+        const CNcbiRegistry& registry = app->GetConfig();
+        const string& value = registry.Get("BLAST", "MAX_SEQID_LENGTH");
+        if (!value.empty()) {
+            try { m_FastaReader->SetMaxIDLength(NStr::StringToUInt(value)); } 
+            catch (const exception&) {}  // Ignore errors
+        }
+    }
 }
 
 CFastaBioseqSource::~CFastaBioseqSource()
@@ -852,6 +861,13 @@ bool CBuildDatabase::AddSequences(IBioseqSource & src, bool add_pig)
                     continue;
        }
 
+       if (bs->GetLength() > 0x7fffffff)
+       {
+                   NCBI_THROW(CWriteDBException,
+                   eArgErr,
+                   "Sequences longer than 2,147,483,647 bases are not supported.  Offending sequence is " + bioseq_id);
+       }
+
         if ((bs->GetLength() == 0) || (!x_EditAndAddBioseq(bs, NULL, add_pig))){
             m_LogFile << "Ignoring sequence '" << bioseq_id
                       << "' as it has no sequence data" << endl;
@@ -1062,7 +1078,9 @@ CBuildDatabase::CBuildDatabase(const string         & dbname,
                                ostream              * logfile,
                                bool                   long_seqids,
                                EBlastDbVersion        dbver,
-                               bool                   limit_defline)
+                               bool                   limit_defline,
+                               Uint8                  oid_masks,
+                               bool scan_bioseq_4_cfastareader_usrobj)
     : m_IsProtein    (is_protein),
       m_KeepLinks    (false),
       m_KeepMbits    (false),
@@ -1077,7 +1095,9 @@ CBuildDatabase::CBuildDatabase(const string         & dbname,
       m_LongIDs      (long_seqids),
       m_FoundMatchingMasks(false),
       m_SkipCopyingGis(false),
-      m_SkipLargeGis(true)
+      m_SkipLargeGis(true),
+      m_OutputDbName(kEmptyStr),
+      m_ScanBioseq4CFastaReaderUsrObjct(scan_bioseq_4_cfastareader_usrobj)
 {
     CreateDirectories(dbname);
     const string output_dbname = CDirEntry::CreateAbsolutePath(dbname);
@@ -1104,11 +1124,13 @@ CBuildDatabase::CBuildDatabase(const string         & dbname,
                                   m_LongIDs,
                                   use_gi_mask,
                                   dbver,
-                                  limit_defline));
+                                  limit_defline,
+                                  oid_masks));
 
     // Standard 1 GB limit
 
     m_OutputDb->SetMaxFileSize(1000*1000*1000);
+    m_OutputDbName = output_dbname;
 }
 
 CBuildDatabase::CBuildDatabase(const string & dbname,
@@ -1120,7 +1142,9 @@ CBuildDatabase::CBuildDatabase(const string & dbname,
                                ostream      * logfile,
                                bool           long_seqids,
                                EBlastDbVersion        dbver,
-                               bool           limit_defline)
+                               bool           limit_defline,
+                               Uint8          oid_masks,
+                               bool scan_bioseq_4_cfastareader_usrobj)
     : m_IsProtein    (is_protein),
       m_KeepLinks    (false),
       m_KeepMbits    (false),
@@ -1135,7 +1159,8 @@ CBuildDatabase::CBuildDatabase(const string & dbname,
       m_LongIDs      (long_seqids),
       m_FoundMatchingMasks(false),
       m_SkipCopyingGis(false),
-      m_SkipLargeGis(true)
+      m_SkipLargeGis(true),
+      m_ScanBioseq4CFastaReaderUsrObjct(scan_bioseq_4_cfastareader_usrobj)
 {
     CreateDirectories(dbname);
     const string output_dbname = CDirEntry::CreateAbsolutePath(dbname);
@@ -1166,11 +1191,14 @@ CBuildDatabase::CBuildDatabase(const string & dbname,
                                   long_seqids,
                                   use_gi_mask,
                                   dbver,
-                                  limit_defline));
+                                  limit_defline,
+                                  oid_masks,
+                                  scan_bioseq_4_cfastareader_usrobj));
 
     // Standard 1 GB limit
 
     m_OutputDb->SetMaxFileSize(1000*1000*1000);
+    m_OutputDbName = output_dbname;
 }
 
 CBuildDatabase::~CBuildDatabase()
