@@ -1,4 +1,4 @@
-/*  $Id: taxid_set.cpp 610974 2020-06-26 12:59:33Z grichenk $
+/*  $Id: taxid_set.cpp 645777 2022-02-22 19:32:10Z boratyng $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -73,6 +73,14 @@ void CTaxIdSet::SetMappingFromFile(CNcbiIstream & f)
     m_Matched = (m_GlobalTaxId != kTaxIdNotSet) || m_TaxIdMap.empty();
 }
 
+
+void CTaxIdSet::AddTaxId(const CSeq_id& seqid, const TTaxId& taxid)
+{
+    string key = AccessionToKey(seqid.AsFastaString());
+    m_TaxIdMap[key] = taxid;
+}
+
+
 TTaxId CTaxIdSet::x_SelectBestTaxid(const objects::CBlast_def_line & defline) 
 {
     TTaxId retval = m_GlobalTaxId;
