@@ -1,4 +1,4 @@
-/* $Id: gene_qual_normalization.cpp 632626 2021-06-03 17:38:42Z ivanov $
+/* $Id: gene_qual_normalization.cpp 642578 2021-12-22 19:48:06Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -103,7 +103,7 @@ bool CCleanup::NormalizeGeneQuals(CSeq_feat& cds, CSeq_feat& gene)
     }
     CTempString locus;
     CTempString locus_tag;
-    CRef<CSeqFeatXref> gene_xref(NULL);
+    CRef<CSeqFeatXref> gene_xref;
     for (auto it : cds.SetXref()) {
         if (it->IsSetData() && it->GetData().IsGene()) {
             if (it->GetData().GetGene().IsSetLocus()) {
