@@ -117,7 +117,7 @@ class CSeq_feat_Handle;
 class CObjectManager;
 class CScope;
 
-class CNewCleanup_imp
+class NCBI_CLEANUP_EXPORT CNewCleanup_imp
 {
 public:
 
@@ -139,7 +139,7 @@ public:
 
     /// Main methods
 
-    void SetScope(CScope& scope) { m_Scope.Reset(&scope); }
+    void SetScope(CScope& scope);
 
     /// Basic Cleanup methods
 
@@ -223,19 +223,12 @@ public:
 
     static void AddNcbiCleanupObject(CSeq_descr& descr);
 
-private:
 
     // many more methods and variables ...
-
-    // We do not include the usual "x_" prefix for private functions
-    // because we want to be able to distinguish between higher-level
-    // functions like those just below, and the lower-level
-    // functions like those farther below.
-
+    // Note that these are not private. This is to facilitate testing
     void ChangeMade (CCleanupChange::EChanges e);
 
     void EnteringEntry(CSeq_entry& se);
-    void LeavingEntry (CSeq_entry& se);
 
     void SetGeneticCode (CBioseq& bs);
 
@@ -244,7 +237,9 @@ private:
     void SeqsetBC (CBioseq_set& bss);
     void ProtSeqBC (CBioseq& bs);
 
-    void SeqIdBC( CSeq_id &seq_id );
+    bool BasicCleanupSeqIds(CBioseq_Handle bsh);
+    bool SeqIdBC( CSeq_id &seq_id );
+
 
     void GBblockOriginBC( string& str);
     void GBblockBC (CGB_block& gbk);
@@ -270,7 +265,6 @@ private:
 
     void SeqLocBC( CSeq_loc &loc );
     void ConvertSeqLocWholeToInt( CSeq_loc &loc );
-    void SeqLocMixBC( CSeq_loc_mix & loc_mix );
 
     void SeqfeatBC (CSeq_feat& sf);
 
@@ -292,14 +286,14 @@ private:
 
     static bool x_IsCommentRedundantWithEC(const CSeq_feat& seqfeat, CScope& scope);
 
-    void DeltaExtBC( CDelta_ext & delta_ext, CSeq_inst &seq_inst );
+    void RemoveZeroLengthLiterals(CDelta_ext & delta_ext);
 
     void UserObjectBC( CUser_object &user_object );
 
     void PCRReactionSetBC( CPCRReactionSet &pcr_reaction_set );
     void SubSourceListBC(CBioSource& biosrc);
 
-    void MolInfoBC( CMolInfo &molinfo );
+    bool MolInfoBC( CMolInfo &molinfo );
     void CreateMissingMolInfo( CBioseq& seq );
 
     static bool IsInternalTranscribedSpacer(const string& name);
@@ -340,7 +334,7 @@ private:
     // data structures used for post-processing
 
     // recorded by x_NotePubdescOrAnnotPubs
-    typedef std::map<int, int> TMuidToPmidMap;
+    typedef std::map<TEntrezId, TEntrezId> TMuidToPmidMap;
     TMuidToPmidMap m_MuidToPmidMap;
     // recorded by x_RememberMuidThatMightBeConvertibleToPmid
     typedef std::vector< CRef<CPub> > TMuidPubContainer;
@@ -352,6 +346,7 @@ private:
     // at least one should remain so we can make the "old_label -> new_label" connection.
     typedef std::multimap< string, CRef<CPub> > TOldLabelToPubMap;
     TOldLabelToPubMap m_OldLabelToPubMap;
+    map<string,string> m_OldLabelToNewLabel;
     // remember label changes
     typedef std::map< CRef<CPub>, string > TPubToNewPubLabelMap;
     TPubToNewPubLabelMap m_PubToNewPubLabelMap;
@@ -401,10 +396,11 @@ private:
 //    void x_FlattenPubEquiv(CPub_equiv& pe);
 
     // Date-related
-    void x_DateStdBC( CDate_std& date );
+    bool DateStdBC(CDate_std& date);
 
     void x_AddReplaceQual(CSeq_feat& feat, const string& str);
 
+    void x_SeqPointBC(CSeq_point & seq_point);
     void x_SeqIntervalBC( CSeq_interval & seq_interval );
     void x_BothStrandBC( CSeq_loc &loc );
     void x_BothStrandBC( CSeq_interval & seq_interval );
@@ -423,10 +419,11 @@ private:
     void x_SubSourceBC( CSubSource & subsrc );
     void x_OrgModBC( COrgMod & orgmod );
 
-    void x_FixUnsetMolFromBiomol( CMolInfo& molinfo, CBioseq &bioseq );
-    void FixUnsetMolFromBiomol(CMolInfo::TBiomol biomol, CBioseq& bioseq);
+public:
+    void FixUnsetMolFromBiomol(CMolInfo::TBiomol biomol, CBioseq_Handle bsh);
 
-    void x_AddPartialToProteinTitle( CBioseq &bioseq );
+    void AddPartialToProteinTitle(CBioseq_Handle bsh);
+private:
 
     string x_ExtractSatelliteFromComment( string &comment );
 
@@ -445,7 +442,7 @@ private:
 
     void x_RemoveFlankingQuotes( string &val );
 
-    void x_MoveCdregionXrefsToProt (CCdregion& cds, CSeq_feat& seqfeat);
+    void x_MoveCdregionXrefsToProt(CSeq_feat& seqfeat);
     bool x_InGpsGenomic( const CSeq_feat& seqfeat );
 
     void x_AddNonCopiedQual(
@@ -467,7 +464,7 @@ private:
     void x_CompressStringSpacesMarkChanged( std::string &str );
     void x_StripSpacesMarkChanged( std::string& str );
     void x_RemoveSpacesBetweenTildesMarkChanged( std::string & str );
-    void x_TruncateSpacesMarkChanged( std::string & str );
+    bool x_TruncateSpacesMarkChanged( std::string & str );
     void x_TrimInternalSemicolonsMarkChanged( std::string & str );
 
     void x_PostSeqFeat( CSeq_feat& seq_feat );
@@ -487,17 +484,27 @@ private:
     // This function does that processing.
     void x_PostProcessing(void);
 
+    void xUpdateSeqfeatCitPub(CSeq_entry& entry);
+    void xUpdateSeqfeatCitPub(CBioseq_set& bioset);
+    void xUpdateSeqfeatCitPub(CBioseq& bioseq);
+    void xUpdateSeqfeatCitPub(CSeq_annot& annot);
+    bool xUpdateSeqfeatCitGenLabel(CPub& pub);
+
     // after cleaning bioseq and bioseq-set, need to clear empty descriptors
     void x_ClearEmptyDescr( CBioseq_set& bioseq_set );
     void x_ClearEmptyDescr( CBioseq& bioseq );
 
-    // removes single-strandedness from non-viral nucleotide sequences
-    void x_RemoveSingleStrand( CBioseq& bioseq );
 
+public:
+    // removes single-strandedness from non-viral nucleotide sequences
+    bool RemoveSingleStrand( CBioseq& bioseq );
+    void LabelMapAppend(const vector<string>& old_labels, const vector<string>& new_labels);
+
+private:
     // functions that prepare for post-processing while traversing
     void x_NotePubdescOrAnnotPubs( const CPub_equiv &pub_equiv );
     void x_NotePubdescOrAnnotPubs_RecursionHelper(
-        const CPub_equiv &pub_equiv, int &muid, int &pmid );
+        const CPub_equiv &pub_equiv, TEntrezId &muid, TEntrezId &pmid );
     void x_RememberPubOldLabel( CPub &pub );
     void x_RememberMuidThatMightBeConvertibleToPmid(CPub &pub );
     void x_RememberSeqFeatCitPubs( CPub &pub );
@@ -506,7 +513,6 @@ private:
     void AddMolInfo(CBioseq_set& set, const CMolInfo& mol);
     void AddMolInfo(CBioseq& seq, const CMolInfo& mol);
 
-private:
     void x_SortSeqDescs( CSeq_entry & seq_entry );
 
     void x_FixStructuredCommentKeywords(CSeq_descr& descr);
@@ -584,10 +590,9 @@ private:
 
     void x_ExtendedCleanupExtra(CSeq_entry_Handle seh);
 
-protected:
+private:
 
     // variables used for the whole cleaning process
-
     /// If set, holds all the cleanup changes that have occurred so far
     CRef<CCleanupChange>  m_Changes;
     /// See CCleanup::EValidOptions
@@ -601,6 +606,9 @@ protected:
     /// Set via m_Options to synchronize Cdregion genetic codes with BioSource
     bool                  m_SyncGenCodes { false };
 
+    //// Used to bypass x_CleanSeqFeatQuals for flatfile generator with inference in record.
+    bool                  m_HasInferenceQuals { false };
+
     void SetGlobalFlags(const CSeq_entry& se, bool reset = true);
     void SetGlobalFlags(const CSeq_submit& ss);
     void SetGlobalFlags(const CBioseq& bs, bool reset = true);
