@@ -1,4 +1,4 @@
-/*  $Id: reader_cache.cpp 610970 2020-06-26 12:56:46Z grichenk $
+/*  $Id: reader_cache.cpp 667972 2023-05-24 12:59:04Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -1148,7 +1148,7 @@ bool CCacheReader::LoadSeq_idBlob_ids(CReaderRequestResult& result,
 
 
 struct SCacheEntryAccessCount {
-    Uint8 load_count, save_count;
+    atomic<Uint8> load_count, save_count;
 
     void GoingToLoad() {
         ++load_count;
@@ -1456,15 +1456,7 @@ struct SPluginParams
     static
     TParams* FindSubNode(TParams* params, const string& name)
     {
-        if ( params ) {
-            for ( TParams::TNodeList_I it = params->SubNodeBegin();
-                  it != params->SubNodeEnd(); ++it ) {
-                if ( NStr::CompareNocase((*it)->GetKey(), name) == 0 ) {
-                    return static_cast<TParams*>(*it);
-                }
-            }
-        }
-        return 0;
+        return params? params->FindSubNode(name): 0;
     }
 
 
@@ -1472,15 +1464,7 @@ struct SPluginParams
     const TParams* FindSubNode(const TParams* params,
                                const string& name)
     {
-        if ( params ) {
-            for ( TParams::TNodeList_CI it = params->SubNodeBegin();
-                  it != params->SubNodeEnd(); ++it ) {
-                if ( NStr::CompareNocase((*it)->GetKey(), name) == 0 ) {
-                    return static_cast<const TParams*>(*it);
-                }
-            }
-        }
-        return 0;
+        return params? params->FindSubNode(name): 0;
     }
 
 
@@ -1644,7 +1628,7 @@ GetCacheParams(const SCacheInfo::TParams* src_params,
     else {
         section_name = NCBI_GBLOADER_READER_CACHE_PARAM_BLOB_SECTION;
     }
-    auto_ptr<SCacheInfo::TParams> section
+    unique_ptr<SCacheInfo::TParams> section
         (GetCacheParamsCopy(src_params, section_name));
     if ( !section.get() ) {
         // disabled
@@ -1673,7 +1657,7 @@ ICache* SCacheInfo::CreateCache(const TParams* params,
                                 EReaderOrWriter reader_or_writer,
                                 EIdOrBlob id_or_blob)
 {
-    auto_ptr<TParams> cache_params
+    unique_ptr<TParams> cache_params
         (GetCacheParams(params, reader_or_writer, id_or_blob));
     if ( !cache_params.get() ) {
         return 0;
@@ -1693,9 +1677,9 @@ void CCacheReader::InitializeCache(CReaderCacheManager& cache_manager,
         params->FindNode(NCBI_GBLOADER_READER_CACHE_DRIVER_NAME) : 0;
     ICache* id_cache = 0;
     ICache* blob_cache = 0;
-    auto_ptr<TParams> id_params
+    unique_ptr<TParams> id_params
         (GetCacheParams(reader_params, eCacheReader, eIdCache));
-    auto_ptr<TParams> blob_params
+    unique_ptr<TParams> blob_params
         (GetCacheParams(reader_params, eCacheReader, eBlobCache));
     _ASSERT(id_params.get());
     _ASSERT(blob_params.get());
