@@ -1,4 +1,4 @@
-/*  $Id: dispatcher.cpp 675188 2023-11-16 13:46:47Z ivanov $
+/*  $Id: dispatcher.cpp 681500 2024-04-09 17:35:06Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -85,6 +85,10 @@ static CGBRequestStatistics sx_Statistics[CGBRequestStatistics::eStats_Count] =
     CGBRequestStatistics("parsed", "SNP data"),
     CGBRequestStatistics("parsed", "split data"),
     CGBRequestStatistics("parsed", "chunk data"),
+    CGBRequestStatistics("attached", "blob data"),
+    CGBRequestStatistics("attached", "SNP data"),
+    CGBRequestStatistics("attached", "split data"),
+    CGBRequestStatistics("attached", "chunk data"),
     CGBRequestStatistics("loaded", "sequence hash"),
     CGBRequestStatistics("loaded", "sequence length"),
     CGBRequestStatistics("loaded", "sequence type")
@@ -723,6 +727,56 @@ namespace {
         return ret;
     }
 
+    class CCommandLoadBulkIds : public CReadDispatcherCommand
+    {
+    public:
+        typedef vector<CSeq_id_Handle> TKey;
+        typedef vector<bool> TLoaded;
+        typedef vector<CSeq_id_Handle> TIds;
+        typedef vector<TIds> TRet;
+        typedef CLoadLockSeqIds CLoadLock;
+        CCommandLoadBulkIds(CReaderRequestResult& result,
+                            const TKey& key, TLoaded& loaded, TRet& ret)
+            : CReadDispatcherCommand(result),
+              m_Key(key), m_Loaded(loaded), m_Ret(ret)
+            {
+            }
+
+        bool Execute(CReader& reader)
+            {
+                return reader.LoadBulkIds(GetResult(), m_Key, m_Loaded, m_Ret);
+            }
+        bool IsDone(void)
+            {
+                return sx_BulkIsDone<CLoadLock>(GetResult(), m_Key, m_Loaded);
+            }
+        string GetErrMsg(void) const
+            {
+                return "LoadBulkIds("+
+                    sx_DescribeError<CLoadLock>(GetResult(), m_Key, m_Loaded)+
+                    "): data not found";
+            }
+        string GetStatisticsDescription(void) const
+            {
+                return "bulkids("+
+                    sx_DescribeError<CLoadLock>(GetResult(), m_Key, m_Loaded)+
+                    ")";
+            }
+        CGBRequestStatistics::EStatType GetStatistics(void) const
+            {
+                return CGBRequestStatistics::eStat_Seq_idSeq_ids;
+            }
+        size_t GetStatisticsCount(void) const
+            {
+                return m_Key.size();
+            }
+        
+    private:
+        const TKey& m_Key;
+        TLoaded& m_Loaded;
+        TRet& m_Ret;
+    };
+
     class CCommandLoadAccVers : public CReadDispatcherCommand
     {
     public:
@@ -1144,7 +1198,7 @@ namespace {
             ret += result.GetRequestedId().AsString();
         }
         else {
-            int total = 0;
+            size_t total = 0;
             for (auto key : keys) {
                 CTSE_LoadLock lock = result.GetTSE_LoadLock(key);
                 CTSE_Info::TSeqIds ids;
@@ -1167,6 +1221,37 @@ namespace {
         return ret;
     }
 
+    string sx_ErrorSeqIds(CReaderRequestResult& result,
+                          const vector<pair<CBlob_id, vector<int>>>& keys)
+    {
+        string ret = "; seq-ids: { ";
+        if (result.GetRequestedId()) {
+            ret += result.GetRequestedId().AsString();
+        }
+        else {
+            size_t total = 0;
+            for ( auto& key : keys ) {
+                CTSE_LoadLock lock = result.GetTSE_LoadLock(key.first);
+                CTSE_Info::TSeqIds ids;
+                lock->GetBioseqsIds(ids);
+                if (!ids.empty()) {
+                    int cnt = 0;
+                    for (auto& id : ids) {
+                        if (++total > kMaxErrorSeqIds) continue;
+                        if (cnt++ > 0) ret += ", ";
+                        ret += id.AsString();
+                    }
+                }
+            }
+            if (total == 0) return "";
+            if (total > kMaxErrorSeqIds) {
+                ret += ", ... (+" + NStr::NumericToString(total - kMaxErrorSeqIds) + " more)";
+            }
+        }
+        ret += " }";
+        return ret;
+    }
+
     class CCommandLoadBlobState : public CReadDispatcherCommand
     {
     public:
@@ -1247,15 +1332,11 @@ namespace {
     };
 
     bool s_AllBlobsAreLoaded(CReaderRequestResult& result,
-                             const CLoadLockBlobIds& blobs,
-                             CReadDispatcher::TContentsMask mask,
-                             const SAnnotSelector* sel)
+                             const CReader::TBlobIds& infos,
+                             CReadDispatcher::TContentsMask mask = fBlobHasAll,
+                             const SAnnotSelector* sel = 0)
     {
-        _ASSERT(blobs.IsLoaded());
-
-        CFixedBlob_ids blob_ids = blobs.GetBlob_ids();
-        ITERATE ( CFixedBlob_ids, it, blob_ids ) {
-            const CBlob_Info& info = *it;
+        for ( auto& info : infos ) {
             if ( !info.Matches(mask, sel) ) {
                 continue;
             }
@@ -1266,6 +1347,15 @@ namespace {
         }
         return true;
     }
+    
+    bool s_AllBlobsAreLoaded(CReaderRequestResult& result,
+                             const CLoadLockBlobIds& blobs,
+                             CReadDispatcher::TContentsMask mask,
+                             const SAnnotSelector* sel)
+    {
+        _ASSERT(blobs.IsLoaded());
+        return s_AllBlobsAreLoaded(result, blobs.GetBlob_ids().Get(), mask, sel);
+    }
 
     class CCommandLoadBlobs : public CReadDispatcherCommand
     {
@@ -1308,6 +1398,7 @@ namespace {
         TMask m_Mask;
         const SAnnotSelector* m_Selector;
     };
+    
     class CCommandLoadSeq_idBlobs : public CReadDispatcherCommand
     {
     public:
@@ -1355,6 +1446,43 @@ namespace {
         const SAnnotSelector* m_Selector;
     };
 
+    class CCommandLoadBlobList : public CReadDispatcherCommand
+    {
+    public:
+        typedef CReader::TBlobIds TIds;
+        CCommandLoadBlobList(CReaderRequestResult& result,
+                             const TIds& ids)
+            : CReadDispatcherCommand(result),
+              m_Ids(ids)
+            {
+            }
+
+        bool IsDone(void)
+            {
+                return s_AllBlobsAreLoaded(GetResult(), m_Ids);
+            }
+        bool Execute(CReader& reader)
+            {
+                return reader.LoadBlobs(GetResult(), m_Ids);
+            }
+        string GetErrMsg(void) const
+            {
+                return "LoadBlobs(CLoadInfoBlob_ids): "
+                    "data not found";
+            }
+        CGBRequestStatistics::EStatType GetStatistics(void) const
+            {
+                return CGBRequestStatistics::eStat_LoadBlob;
+            }
+        string GetStatisticsDescription(void) const
+            {
+                return "blobs(...)";
+            }
+        
+    private:
+        const TIds& m_Ids;
+    };
+    
     class CCommandLoadBlob : public CReadDispatcherCommand
     {
     public:
@@ -1410,7 +1538,6 @@ namespace {
         typedef CBlob_id TKey;
         typedef CLoadLockBlob TLock;
         typedef int TChunkId;
-        typedef CTSE_Chunk_Info TChunkInfo;
         CCommandLoadChunk(CReaderRequestResult& result,
                           const TKey& key,
                           TChunkId chunk_id)
@@ -1458,7 +1585,6 @@ namespace {
         typedef CLoadLockBlob TLock;
         typedef int TChunkId;
         typedef vector<TChunkId> TChunkIds;
-        typedef CTSE_Chunk_Info TChunkInfo;
         CCommandLoadChunks(CReaderRequestResult& result,
                            const TKey& key,
                            const TChunkIds chunk_ids)
@@ -1529,6 +1655,104 @@ namespace {
         TChunkIds m_ChunkIds;
     };
 
+    class CCommandLoadBlobChunks : public CReadDispatcherCommand
+    {
+    public:
+        typedef int TChunkId;
+        typedef vector<TChunkId> TChunkIds;
+        typedef vector<pair<CBlob_id, TChunkIds>> TKey;
+        typedef vector<CLoadLockBlob> TLock;
+        CCommandLoadBlobChunks(CReaderRequestResult& result,
+                               const TKey& key)
+            : CReadDispatcherCommand(result),
+              m_Key(key)
+            {
+                for ( auto& blob : key ) {
+                    m_Lock.push_back(CLoadLockBlob(result, blob.first));
+                }
+            }
+
+        bool IsDone(void)
+            {
+                for ( size_t i = 0; i < m_Key.size(); ++i ) {
+                    for ( auto chunk : m_Key[i].second ) {
+                        if ( !m_Lock[i].IsLoadedChunk(chunk) ) {
+                            return false;
+                        }
+                    }
+                }
+                return true;
+            }
+        bool Execute(CReader& reader)
+            {
+                return reader.LoadChunks(GetResult(), m_Key);
+            }
+        string GetErrMsg(void) const
+            {
+                CNcbiOstrstream str;
+                str << "LoadChunks(";
+                int blob_cnt = 0;
+                for ( size_t i = 0; i < m_Key.size(); ++i ) {
+                    int chunk_cnt = 0;
+                    for ( auto& chunk : m_Key[i].second ) {
+                        if ( !m_Lock[i].IsLoadedChunk(chunk) ) {
+                            if ( chunk_cnt++ ) {
+                                str << ',';
+                            }
+                            else {
+                                if ( blob_cnt++ ) {
+                                    str << ", ";
+                                }
+                                str << "("<<m_Key[i].first.ToString() << "; chunks: {";
+                            }
+                            str << ' ' << chunk;
+                        }
+                    }
+                    if ( chunk_cnt ) {
+                        str << "})";
+                    }
+                }
+                str << sx_ErrorSeqIds(GetResult(), m_Key) + ": data not found";
+                return CNcbiOstrstreamToString(str);
+            }
+        CGBRequestStatistics::EStatType GetStatistics(void) const
+            {
+                return CGBRequestStatistics::eStat_LoadChunk;
+            }
+        string GetStatisticsDescription(void) const
+            {
+                CNcbiOstrstream str;
+                int blob_cnt = 0;
+                for ( auto& blob : m_Key ) {
+                    if ( blob_cnt++ ) {
+                        str << ',';
+                    }
+                    int cnt = 0;
+                    for ( auto id : blob.second ) {
+                        if ( id >= 0 && id < kMax_Int ) {
+                            if ( !cnt ) {
+                                str << "chunk(" << blob.first.ToString() << '.';
+                                cnt = 1;
+                            }
+                            else {
+                                str << ',';
+                            }
+                            str << id;
+                        }
+                    }
+                    if ( !cnt ) {
+                        str << "blob(" << blob.first.ToString();
+                    }
+                    str << ')';
+                }
+                return CNcbiOstrstreamToString(str);
+            }
+        
+    private:
+        TKey m_Key;
+        TLock m_Lock;
+    };
+
     class CCommandLoadBlobSet : public CReadDispatcherCommand
     {
     public:
@@ -1768,6 +1992,14 @@ void CReadDispatcher::LoadSequenceType(CReaderRequestResult& result,
 }
 
 
+void CReadDispatcher::LoadBulkIds(CReaderRequestResult& result,
+                                  const TIds& ids, TLoaded& loaded, TBulkIds& ret)
+{
+    CCommandLoadBulkIds command(result, ids, loaded, ret);
+    Process(command);
+}
+
+
 void CReadDispatcher::LoadAccVers(CReaderRequestResult& result,
                                   const TIds& ids, TLoaded& loaded, TIds& ret)
 {
@@ -1877,6 +2109,14 @@ void CReadDispatcher::LoadBlobs(CReaderRequestResult& result,
 }
 
 
+void CReadDispatcher::LoadBlobs(CReaderRequestResult& result,
+                                const TBlobInfos& blob_infos)
+{
+    CCommandLoadBlobList command(result, blob_infos);
+    Process(command);
+}
+
+
 void CReadDispatcher::LoadBlob(CReaderRequestResult& result,
                                const CBlob_id& blob_id)
 {
@@ -1910,6 +2150,14 @@ void CReadDispatcher::LoadChunks(CReaderRequestResult& result,
 }
 
 
+void CReadDispatcher::LoadChunks(CReaderRequestResult& result,
+                                 const TBlobChunkIds& chunk_ids)
+{
+    CCommandLoadBlobChunks command(result, chunk_ids);
+    Process(command);
+}
+
+
 void CReadDispatcher::LoadBlobSet(CReaderRequestResult& result,
                                   const TIds& seq_ids)
 {
