@@ -1,4 +1,4 @@
-/*  $Id: gbloader.cpp 603534 2020-03-12 14:02:02Z vasilche $
+/*  $Id: gbloader.cpp 686741 2024-08-26 15:21:55Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -68,6 +68,8 @@
 #include <corelib/plugin_manager_impl.hpp>
 #include <corelib/plugin_manager_store.hpp>
 
+#include <util/md5.hpp>
+
 #include <algorithm>
 
 
@@ -85,98 +87,50 @@ typedef NCBI_PARAM_TYPE(GENBANK, LOADER_PSG) TGenbankLoaderPsg;
 #define GBLOADER_NAME "GBLOADER"
 #define GBLOADER_HUP_NAME "GBLOADER-HUP"
 
-#define DEFAULT_ID_GC_SIZE 10000
-#define DEFAULT_ID_EXPIRATION_TIMEOUT 2*3600 // 2 hours
-
 CGBLoaderParams::CGBLoaderParams(void)
-    : m_ReaderName(),
-      m_ReaderPtr(0),
+    : m_ReaderPtr(0),
       m_ParamTree(0),
       m_Preopen(ePreopenByConfig),
-      m_HasHUPIncluded(false),
-      m_PSGNoSplit(false)
+      m_UsePSGInitialized(false),
+      m_UsePSG(false),
+      m_PSGNoSplit(false),
+      m_HasHUPIncluded(false)
 {
 }
 
 
 CGBLoaderParams::CGBLoaderParams(const string& reader_name)
-    : m_ReaderName(reader_name),
-      m_ReaderPtr(0),
-      m_ParamTree(0),
-      m_Preopen(ePreopenByConfig),
-      m_HasHUPIncluded(false),
-      m_PSGNoSplit(false)
+    : CGBLoaderParams()
 {
+    m_ReaderName = reader_name;
 }
 
 
 CGBLoaderParams::CGBLoaderParams(CReader* reader_ptr)
-    : m_ReaderName(),
-      m_ReaderPtr(reader_ptr),
-      m_ParamTree(0),
-      m_Preopen(ePreopenByConfig),
-      m_HasHUPIncluded(false),
-      m_PSGNoSplit(false)
+    : CGBLoaderParams()
 {
+    m_ReaderPtr = reader_ptr;
 }
 
 
 CGBLoaderParams::CGBLoaderParams(const TParamTree* param_tree)
-    : m_ReaderName(),
-      m_ReaderPtr(0),
-      m_ParamTree(param_tree),
-      m_Preopen(ePreopenByConfig),
-      m_HasHUPIncluded(false),
-      m_PSGNoSplit(false)
+    : CGBLoaderParams()
 {
+    m_ParamTree = param_tree;
 }
 
 
 CGBLoaderParams::CGBLoaderParams(EPreopenConnection preopen)
-    : m_ReaderName(),
-      m_ReaderPtr(0),
-      m_ParamTree(0),
-      m_Preopen(preopen),
-      m_HasHUPIncluded(false),
-      m_PSGNoSplit(false)
+    : CGBLoaderParams()
 {
+    m_Preopen = preopen;
 }
 
 
-CGBLoaderParams::~CGBLoaderParams(void)
-{
-}
+CGBLoaderParams::~CGBLoaderParams(void) = default;
 
-
-CGBLoaderParams::CGBLoaderParams(const CGBLoaderParams& params)
-    : m_ReaderName(params.m_ReaderName),
-      m_ReaderPtr(params.m_ReaderPtr),
-      m_ParamTree(params.m_ParamTree),
-      m_Preopen(params.m_Preopen),
-      m_HasHUPIncluded(params.m_HasHUPIncluded),
-      m_WebCookie(params.m_WebCookie),
-      m_LoaderName(params.m_LoaderName),
-      m_PSGServiceName(params.m_PSGServiceName),
-      m_PSGNoSplit(params.m_PSGNoSplit)
-{
-}
-
-
-CGBLoaderParams& CGBLoaderParams::operator=(const CGBLoaderParams& params)
-{
-    if ( this != &params ) {
-        m_ReaderName = params.m_ReaderName;
-        m_ReaderPtr = params.m_ReaderPtr;
-        m_ParamTree = params.m_ParamTree;
-        m_Preopen = params.m_Preopen;
-        m_HasHUPIncluded = params.m_HasHUPIncluded;
-        m_WebCookie = params.m_WebCookie;
-        m_LoaderName = params.m_LoaderName;
-        m_PSGServiceName = params.m_PSGServiceName;
-        m_PSGNoSplit = params.m_PSGNoSplit;
-    }
-    return *this;
-}
+CGBLoaderParams::CGBLoaderParams(const CGBLoaderParams&) = default;
+CGBLoaderParams& CGBLoaderParams::operator=(const CGBLoaderParams&) = default;
 
 
 void CGBLoaderParams::SetReaderPtr(CReader* reader_ptr)
@@ -197,22 +151,226 @@ void CGBLoaderParams::SetParamTree(const TPluginManagerParamTree* param_tree)
 }
 
 
+bool CGBLoaderParams::IsSetEnableSNP(void) const
+{
+    return !m_EnableSNP.IsNull();
+}
+
+bool CGBLoaderParams::GetEnableSNP(void) const
+{
+    return m_EnableSNP.GetValue();
+}
+
+void CGBLoaderParams::SetEnableSNP(bool enable)
+{
+    m_EnableSNP = enable;
+}
+
+bool CGBLoaderParams::IsSetEnableWGS(void) const
+{
+    return !m_EnableWGS.IsNull();
+}
+
+bool CGBLoaderParams::GetEnableWGS(void) const
+{
+    return m_EnableWGS.GetValue();
+}
+
+void CGBLoaderParams::SetEnableWGS(bool enable)
+{
+    m_EnableWGS = enable;
+}
+
+bool CGBLoaderParams::IsSetEnableCDD(void) const
+{
+    return !m_EnableCDD.IsNull();
+}
+
+bool CGBLoaderParams::GetEnableCDD(void) const
+{
+    return m_EnableCDD.GetValue();
+}
+
+void CGBLoaderParams::SetEnableCDD(bool enable)
+{
+    m_EnableCDD = enable;
+}
+
+
+NCBI_PARAM_DEF_EX(string, GENBANK, LOADER_METHOD, "",
+                  eParam_NoThread, GENBANK_LOADER_METHOD);
+typedef NCBI_PARAM_TYPE(GENBANK, LOADER_METHOD) TGenbankLoaderMethod;
+
+#define NCBI_GBLOADER_PARAM_LOADER_PSG = "loader_psg"
+#define NCBI_PSG_READER_NAME "psg"
+
+
+string CGBDataLoader::x_GetLoaderMethod(const TParamTree* params)
+{
+    string method = GetParam(params, NCBI_GBLOADER_PARAM_LOADER_METHOD);
+    if ( method.empty() ) {
+        // try config first
+        method = TGenbankLoaderMethod::GetDefault();
+    }
+    return method;
+}
+
+
+static bool s_CheckPSGMethod(const string& loader_method)
+{
+    bool use_psg = false;
+    if ( NStr::FindNoCase(loader_method, NCBI_PSG_READER_NAME) != NPOS ) {
+        vector<string> str_list;
+        NStr::Split(loader_method, ";", str_list);
+        for (auto s : str_list) {
+            if ( NStr::EqualNocase(s, NCBI_PSG_READER_NAME) ) {
+#ifdef HAVE_PSG_LOADER
+                if (str_list.size() == 1) {
+                    use_psg = true;
+                    break;
+                }
+                // PSG method can not be combined with any other methods.
+                NCBI_THROW(CLoaderException, eBadConfig,
+                           "'PSG' loader method can not be combined with other methods: '" + loader_method + "'");
+#else
+                // PSG method is not available
+                NCBI_THROW(CLoaderException, eBadConfig,
+                           "'PSG' loader method is not available: '" + loader_method + "'");
+#endif
+            }
+        }
+    }
+    return use_psg;
+}    
+
+
+static bool s_GetDefaultUsePSG()
+{
+    static atomic<bool> initialized;
+    static atomic<bool> loader_psg;
+    if ( !initialized.load(memory_order_acquire) ) {
+        bool new_value = false;
+#ifdef HAVE_PSG_LOADER
+        if ( TGenbankLoaderPsg::GetDefault() ) {
+            new_value = true;
+        }
+#endif
+        if ( !new_value ) {
+            new_value = s_CheckPSGMethod(TGenbankLoaderMethod::GetDefault());
+        }
+        loader_psg.store(new_value, memory_order_relaxed);
+        initialized.store(true, memory_order_release);
+    }
+    return loader_psg.load(memory_order_relaxed);
+}
+
+
+static bool s_GetDefaultUsePSG(const CGBLoaderParams::TParamTree* param_tree)
+{
+    if ( param_tree ) {
+        if ( const CGBLoaderParams::TParamTree* gb_params = CGBDataLoader::GetLoaderParams(param_tree) ) {
+#ifdef HAVE_LOADER_PSG
+            auto loader_psg_param = CGBDataLoader::GetParam(gb_params, NCBI_GBLOADER_PARAM_LOADER_PSG);
+            if ( !loader_psg_param.empty() ) {
+                try {
+                    return NStr::StringToBool(loader_psg_param);
+                }
+                catch (CStringException& ex) {
+                    NCBI_THROW_FMT(CConfigException, eInvalidParameter,
+                                   "Cannot init GenBank loader, incorrect parameter format: "
+                                   << NCBI_GBLOADER_PARAM_LOADER_PSG << " : " << loader_psg_param
+                                   << ". " << ex.what());
+                }
+            }
+#endif
+            string loader_method = CGBDataLoader::GetParam(gb_params, NCBI_GBLOADER_PARAM_LOADER_METHOD);
+            if ( !loader_method.empty() ) {
+                return s_CheckPSGMethod(loader_method);
+            }
+        }
+    }
+    return s_GetDefaultUsePSG();
+}
+
+
+bool CGBLoaderParams::GetUsePSG() const
+{
+    if ( m_UsePSGInitialized ) {
+        return m_UsePSG;
+    }
+    
+    string loader_method = GetLoaderMethod();
+    if (loader_method.empty()) {
+        loader_method = GetReaderName();
+    }
+    if (loader_method.empty()) {
+        // use default settings from config
+        m_UsePSG = s_GetDefaultUsePSG(GetParamTree());
+    }
+    else {
+        m_UsePSG = s_CheckPSGMethod(loader_method);
+    }
+    m_UsePSGInitialized = true;
+    return m_UsePSG;
+}
+
+/*
+void CGBDataLoader::SetLoaderMethod(CGBLoaderParams& params)
+{
+    string loader_method = params.GetLoaderMethod();
+    if (loader_method.empty()) {
+        loader_method = params.GetReaderName();
+    }
+    if (loader_method.empty()) {
+        unique_ptr<TParamTree> app_params;
+        const TParamTree* gb_params = 0;
+        if ( params.GetParamTree() ) {
+            gb_params = GetLoaderParams(params.GetParamTree());
+        }
+        else {
+            CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
+            if ( app ) {
+                app_params.reset(CConfig::ConvertRegToTree(app->GetConfig()));
+                gb_params = GetLoaderParams(app_params.get());
+            }
+        }
+
+        if ( !gb_params ) {
+            app_params.reset(new TParamTree);
+            gb_params = GetLoaderParams(app_params.get());
+        }
+
+        loader_method = GetParam(gb_params, NCBI_GBLOADER_PARAM_LOADER_METHOD);
+        if ( loader_method.empty() ) {
+            // try config first
+            loader_method = TGenbankLoaderMethod::GetDefault();
+        }
+        params.SetLoaderMethod(loader_method);
+    }
+
+    vector<string> str_list;
+    NStr::Split(loader_method, ";", str_list);
+    for (auto s : str_list) {
+        if (!NStr::EqualNocase(s, "psg")) continue;
+        if (str_list.size() == 1) {
+            TGenbankLoaderPsg::SetDefault(true);
+            break;
+        }
+        // PSG method can not be combined with any other methods.
+        NCBI_THROW(CLoaderException, eBadConfig,
+            "'PSG' loader method can not be combined with other methods: '" + loader_method + "'");
+    }
+}
+*/
+
 CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager& om,
     CReader*        reader_ptr,
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
-#if defined(HAVE_PSG_LOADER)
-    if (TGenbankLoaderPsg::GetDefault()) {
-        return CPSGDataLoader::RegisterInObjectManager(om, is_default, priority);
-    }
-#else
-    if (0) {}
-#endif
-    else {
-        return CGBDataLoader_Native::RegisterInObjectManager(om, reader_ptr, is_default, priority);
-    }
+    CGBLoaderParams params(reader_ptr);
+    return RegisterInObjectManager(om, params, is_default, priority);
 }
 
 
@@ -228,16 +386,8 @@ CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
-#if defined(HAVE_PSG_LOADER)
-    if (TGenbankLoaderPsg::GetDefault()) {
-        return CPSGDataLoader::RegisterInObjectManager(om, is_default, priority);
-    }
-#else
-    if (0) {}
-#endif
-    else {
-        return CGBDataLoader_Native::RegisterInObjectManager(om, reader_name, is_default, priority);
-    }
+    CGBLoaderParams params(reader_name);
+    return RegisterInObjectManager(om, params, is_default, priority);
 }
 
 
@@ -257,30 +407,30 @@ CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
                                    priority);
 }
 
+string CGBDataLoader::GetLoaderNameFromArgs(EIncludeHUP include_hup)
+{
+    return GBLOADER_HUP_NAME;
+}
 
 CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager& om,
-    EIncludeHUP     include_hup,
+    EIncludeHUP     /*include_hup*/,
     const string& web_cookie,
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
-#if defined(HAVE_PSG_LOADER)
-    if (TGenbankLoaderPsg::GetDefault()) {
-        return CPSGDataLoader::RegisterInObjectManager(om, is_default, priority);
-    }
-#else
-    if (0) {}
-#endif
-    else {
-        return CGBDataLoader_Native::RegisterInObjectManager(om, include_hup, web_cookie, is_default, priority);
-    }
+    CGBLoaderParams params;
+    params.SetHUPIncluded(true, web_cookie);
+    return RegisterInObjectManager(om, params, is_default, priority);
 }
 
 
-string CGBDataLoader::GetLoaderNameFromArgs(EIncludeHUP /*include_hup*/)
+string CGBDataLoader::GetLoaderNameFromArgs(EIncludeHUP include_hup,
+                                            const string& web_cookie)
 {
-    return GBLOADER_HUP_NAME;
+    CGBLoaderParams params;
+    params.SetHUPIncluded(true, web_cookie);
+    return GetLoaderNameFromArgs(params);
 }
 
 CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
@@ -294,6 +444,13 @@ CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
                                    is_default, priority);
 }
 
+string CGBDataLoader::GetLoaderNameFromArgs(const string& reader_name,
+                                            EIncludeHUP include_hup)
+{
+    return GBLOADER_HUP_NAME;
+}
+
+
 CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager& om,
     const string&   reader_name,
@@ -302,23 +459,19 @@ CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
-#if defined(HAVE_PSG_LOADER)
-    if (TGenbankLoaderPsg::GetDefault()) {
-        return CPSGDataLoader::RegisterInObjectManager(om, is_default, priority);
-    }
-#else
-    if (0) {}
-#endif
-    else {
-        return CGBDataLoader_Native::RegisterInObjectManager(om, reader_name, include_hup, web_cookie, is_default, priority);
-    }
+    CGBLoaderParams params(reader_name);
+    params.SetHUPIncluded(true, web_cookie);
+    return RegisterInObjectManager(om, params, is_default, priority);
 }
 
 
-string CGBDataLoader::GetLoaderNameFromArgs(const string& /*reader_name*/,
-                                            EIncludeHUP /*include_hup*/)
+string CGBDataLoader::GetLoaderNameFromArgs(const string& reader_name,
+                                            EIncludeHUP include_hup,
+                                            const string& web_cookie)
 {
-    return GBLOADER_HUP_NAME;
+    CGBLoaderParams params(reader_name);
+    params.SetHUPIncluded(true, web_cookie);
+    return GetLoaderNameFromArgs(params);
 }
 
 
@@ -328,20 +481,12 @@ CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
-#if defined(HAVE_PSG_LOADER)
-    if (TGenbankLoaderPsg::GetDefault()) {
-        return CPSGDataLoader::RegisterInObjectManager(om, param_tree, is_default, priority);
-    }
-#else
-    if (0) {}
-#endif
-    else {
-        return CGBDataLoader_Native::RegisterInObjectManager(om, param_tree, is_default, priority);
-    }
+    CGBLoaderParams params(&param_tree);
+    return RegisterInObjectManager(om, params, is_default, priority);
 }
 
 
-string CGBDataLoader::GetLoaderNameFromArgs(const TParamTree& /* params */)
+string CGBDataLoader::GetLoaderNameFromArgs(const TParamTree& param_tree)
 {
     return GBLOADER_NAME;
 }
@@ -353,16 +498,17 @@ CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
+    if ( params.GetUsePSG() ) {
 #if defined(HAVE_PSG_LOADER)
-    if (TGenbankLoaderPsg::GetDefault()) {
         return CPSGDataLoader::RegisterInObjectManager(om, params, is_default, priority);
-    }
 #else
-    if (0) {}
+        ERR_POST_X(3, Critical << "PSG Loader is requested but not available");
+        TRegisterLoaderInfo info;
+        info.Set(nullptr, false);
+        return info;
 #endif
-    else {
-        return CGBDataLoader_Native::RegisterInObjectManager(om, params, is_default, priority);
     }
+    return CGBDataLoader_Native::RegisterInObjectManager(om, params, is_default, priority);
 }
 
 
@@ -373,10 +519,14 @@ string CGBDataLoader::GetLoaderNameFromArgs(const CGBLoaderParams& params)
     }
     if (params.HasHUPIncluded()) {
         const string& web_cookie = params.GetWebCookie();
-        if (web_cookie.empty())
+        if (web_cookie.empty()) {
             return GBLOADER_HUP_NAME;
-        else
-            return GBLOADER_HUP_NAME + string("-") + web_cookie;
+        }
+        else {
+            CMD5 md5;
+            md5.Update(web_cookie.data(), web_cookie.size());
+            return GBLOADER_HUP_NAME + string("-") + md5.GetHexSum();
+        }
     } else {
         return GBLOADER_NAME;
     }
@@ -386,7 +536,8 @@ string CGBDataLoader::GetLoaderNameFromArgs(const CGBLoaderParams& params)
 CGBDataLoader::CGBDataLoader(const string& loader_name,
                              const CGBLoaderParams& params)
     : CDataLoader(loader_name),
-      m_HasHUPIncluded(params.HasHUPIncluded())
+      m_HasHUPIncluded(params.HasHUPIncluded()),
+      m_WebCookie(params.GetWebCookie())
 {
 }
 
@@ -396,35 +547,36 @@ CGBDataLoader::~CGBDataLoader(void)
 }
 
 
+CObjectManager::TPriority CGBDataLoader::GetDefaultPriority(void) const
+{
+    CObjectManager::TPriority priority = CDataLoader::GetDefaultPriority();
+    if ( HasHUPIncluded() ) {
+        // HUP data loader has lower priority by default
+        priority += 1;
+    }
+    return priority;
+}
+
+
+bool CGBDataLoader::GetTrackSplitSeq() const
+{
+    return true;
+}
+
+
 namespace {
     typedef CGBDataLoader::TParamTree TParams;
 
     TParams* FindSubNode(TParams* params,
                          const string& name)
     {
-        if ( params ) {
-            for ( TParams::TNodeList_I it = params->SubNodeBegin();
-                  it != params->SubNodeEnd(); ++it ) {
-                if ( NStr::CompareNocase((*it)->GetKey(), name) == 0 ) {
-                    return static_cast<TParams*>(*it);
-                }
-            }
-        }
-        return 0;
+        return params? params->FindSubNode(name): 0;
     }
 
     const TParams* FindSubNode(const TParams* params,
                                const string& name)
     {
-        if ( params ) {
-            for ( TParams::TNodeList_CI it = params->SubNodeBegin();
-                  it != params->SubNodeEnd(); ++it ) {
-                if ( NStr::CompareNocase((*it)->GetKey(), name) == 0 ) {
-                    return static_cast<const TParams*>(*it);
-                }
-            }
-        }
-        return 0;
+        return params? params->FindSubNode(name): 0;
     }
 }
 
@@ -435,7 +587,7 @@ CGBDataLoader::GetParamsSubnode(const TParamTree* params,
 {
     const TParamTree* subnode = 0;
     if ( params ) {
-        if ( NStr::CompareNocase(params->GetKey(), subnode_name) == 0 ) {
+        if ( params->KeyEqual(subnode_name) ) {
             subnode = params;
         }
         else {
@@ -452,7 +604,7 @@ CGBDataLoader::GetParamsSubnode(TParamTree* params,
 {
     _ASSERT(params);
     TParamTree* subnode = 0;
-    if ( NStr::CompareNocase(params->GetKey(), subnode_name) == 0 ) {
+    if ( params->KeyEqual(subnode_name) ) {
         subnode = params;
     }
     else {
@@ -523,23 +675,11 @@ string CGBDataLoader::GetParam(const TParamTree* params,
 }
 
 
-CDataLoader::TBlobId CGBDataLoader::GetBlobIdFromSatSatKey(int sat,
-    int sat_key,
-    int sub_sat) const
-{
-    CRef<CBlob_id> blob_id(new CBlob_id);
-    blob_id->SetSat(sat);
-    blob_id->SetSatKey(sat_key);
-    blob_id->SetSubSat(sub_sat);
-    return TBlobId(blob_id);
-}
-
-
 CConstRef<CSeqref> CGBDataLoader::GetSatSatkey(const CSeq_id_Handle& sih)
 {
     TBlobId id = GetBlobId(sih);
     if (id) {
-        const TRealBlobId& blob_id = GetRealBlobId(id);
+        TRealBlobId blob_id = GetRealBlobId(id);
         return ConstRef(new CSeqref(ZERO_GI, blob_id.GetSat(), blob_id.GetSatKey()));
     }
     return CConstRef<CSeqref>();
@@ -552,14 +692,14 @@ CConstRef<CSeqref> CGBDataLoader::GetSatSatkey(const CSeq_id& id)
 }
 
 
-const CGBDataLoader::TRealBlobId&
+CGBDataLoader::TRealBlobId
 CGBDataLoader::GetRealBlobId(const TBlobId& blob_id) const
 {
-    return dynamic_cast<const CBlob_id&>(*blob_id);
+    return x_GetRealBlobId(blob_id);
 }
 
 
-const CGBDataLoader::TRealBlobId&
+CGBDataLoader::TRealBlobId
 CGBDataLoader::GetRealBlobId(const CTSE_Info& tse_info) const
 {
     if (&tse_info.GetDataSource() != GetDataSource()) {
@@ -572,11 +712,7 @@ CGBDataLoader::GetRealBlobId(const CTSE_Info& tse_info) const
 
 bool CGBDataLoader::IsUsingPSGLoader(void)
 {
-#if defined(HAVE_PSG_LOADER)
-    return TGenbankLoaderPsg::GetDefault();
-#else
-    return false;
-#endif
+    return s_GetDefaultUsePSG();
 }
 
 
