@@ -1,4 +1,4 @@
-/*  $Id: reader_gicache.cpp 661188 2023-01-05 19:40:28Z vasilche $
+/*  $Id: reader_gicache.cpp 689701 2024-11-04 19:59:04Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -44,8 +44,13 @@
 #include <corelib/plugin_manager_impl.hpp>
 #include <corelib/plugin_manager_store.hpp>
 
-#include "gicache.h"
+//#define USE_GICACHE
 
+#ifdef USE_GICACHE
+# include "gicache.h"
+#else
+# define DEFAULT_GI_CACHE_PATH ""
+#endif
 
 #define NCBI_USE_ERRCODE_X   Objtools_Rd_GICache
 
@@ -75,13 +80,43 @@ CGICacheReader::CGICacheReader(const TPluginManagerParamTree* params,
 
 CGICacheReader::~CGICacheReader()
 {
+#ifdef USE_GICACHE
     CMutexGuard guard(m_Mutex);
     GICache_ReadEnd();
+#endif
 }
 
 
+#ifdef USE_GICACHE
+static
+void s_LogFunc(int severity, char* msg)
+{
+    /*
+      Severity values from gicache.c
+      #define SEV_NONE 0 
+      #define SEV_INFO 1
+      #define SEV_WARNING 2
+      #define SEV_ERROR 3
+      #define SEV_REJECT 4 
+      #define SEV_FATAL 5
+     */
+    static const EDiagSev sev[] = {
+        eDiag_Info,
+        eDiag_Info,
+        eDiag_Warning,
+        eDiag_Error,
+        eDiag_Critical,
+        eDiag_Fatal
+    };
+    ERR_POST(Severity(sev[min(size_t(severity), ArraySize(sev)-1)]) << msg);
+}
+#endif
+
+
 void CGICacheReader::x_Initialize(void)
 {
+    ERR_POST_ONCE("This app is using deprecated OM++ reader gicache. Please remove it from your configuration.");
+#ifdef USE_GICACHE
     string index = m_Path;
     if ( CFile(index).IsDir() ) {
         const char* file;
@@ -93,7 +128,9 @@ void CGICacheReader::x_Initialize(void)
         index = CFile::MakePath(index, file);
     }
     CMutexGuard guard(m_Mutex);
+    GICache_SetLogEx(s_LogFunc);
     GICache_ReadData(index.c_str());
+#endif
 }
 
 
@@ -140,6 +177,7 @@ bool CGICacheReader::LoadSeq_idAccVer(CReaderRequestResult& result,
 {
     if ( seq_id.IsGi() ) {
         CLoadLockAcc ids(result, seq_id);
+#ifdef USE_GICACHE
         char buffer[256];
         int got;
         {{
@@ -161,6 +199,7 @@ bool CGICacheReader::LoadSeq_idAccVer(CReaderRequestResult& result,
                 }
             }
         }
+#endif
     }
     // if any problem occurs -> fall back to regular reader
     return false;
