@@ -1,4 +1,4 @@
-/*  $Id: reader.cpp 675188 2023-11-16 13:46:47Z ivanov $
+/*  $Id: reader.cpp 681500 2024-04-09 17:35:06Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -658,6 +658,30 @@ bool CReader::LoadSequenceType(CReaderRequestResult& result,
 }
 
 
+bool CReader::LoadBulkIds(CReaderRequestResult& result,
+                          const TIds& ids, TLoaded& loaded, TBulkIds& ret)
+{
+    size_t count = ids.size();
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        CLoadLockSeqIds lock(result, ids[i]);
+        if ( !lock.IsLoaded() ) {
+            m_Dispatcher->LoadSeq_idSeq_ids(result, ids[i]);
+        }
+        if ( lock.IsLoaded() ) {
+            CFixedSeq_ids data = lock.GetSeq_ids();
+            if ( data.IsFound() ) {
+                ret[i] = data.Get();
+                loaded[i] = true;
+            }
+        }
+    }
+    return true;
+}
+
+
 bool CReader::LoadAccVers(CReaderRequestResult& result,
                           const TIds& ids, TLoaded& loaded, TIds& ret)
 {
@@ -978,6 +1002,17 @@ bool CReader::LoadChunks(CReaderRequestResult& result,
 }
 
 
+bool CReader::LoadChunks(CReaderRequestResult& result,
+                         const TBlobChunkIds& chunk_ids)
+{
+    bool ret = false;
+    for ( auto& blob_chunks : chunk_ids ) {
+        ret |= LoadChunks(result, blob_chunks.first, blob_chunks.second);
+    }
+    return ret;
+}
+
+
 bool CReader::LoadBlobSet(CReaderRequestResult& result,
                           const TSeqIds& seq_ids)
 {
@@ -989,6 +1024,44 @@ bool CReader::LoadBlobSet(CReaderRequestResult& result,
 }
 
 
+bool CReader::LoadBlobs(CReaderRequestResult& result,
+                        const TBlobIds& blob_infos)
+{
+    bool ret = false;
+    for ( auto& info : blob_infos ) {
+        const CBlob_id& blob_id = *info.GetBlob_id();
+        CLoadLockBlob blob(result, blob_id);
+        if ( blob.IsLoadedBlob() ) {
+            continue;
+        }
+        
+        if ( info.IsSetAnnotInfo() ) {
+            if ( info.GetAnnotInfo()->GetAnnotInfo().empty() ) {
+                // no actual annot info - only name
+                auto& names = info.GetAnnotInfo()->GetNamedAnnotNames();
+                if ( names.size() == 1 ) {
+                    CLoadLockSetter setter(blob);
+                    setter.GetTSE_LoadLock()->SetName(*names.begin());
+                    setter.AllowIncompleteLoading();
+                }
+            }
+            else {
+                CProcessor_AnnotInfo::LoadBlob(result, info);
+                _ASSERT(blob.IsLoadedBlob());
+                ret = true;
+                continue;
+            }
+        }
+        
+        m_Dispatcher->LoadBlob(result, blob_id);
+        if ( blob.IsLoadedBlob() ) {
+            ret = true;
+        }
+    }
+    return ret;
+}
+
+
 void CReader::SetAndSaveNoBlob(CReaderRequestResult& result,
                                const TBlobId& blob_id,
                                TChunkId chunk_id,
