@@ -1,4 +1,4 @@
-/*  $Id: request_result.cpp 667661 2023-05-18 18:47:34Z vasilche $
+/*  $Id: request_result.cpp 681497 2024-04-09 17:26:05Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -438,8 +438,7 @@ const CTSE_Split_Info& CLoadLockBlob::GetSplitInfo(void) const
 
 bool CLoadLockBlob::NeedsDelayedMainChunk(void) const
 {
-    return m_TSE_LoadLock && m_TSE_LoadLock->HasSplitInfo() &&
-        m_TSE_LoadLock->GetSplitInfo().x_NeedsDelayedMainChunk();
+    return m_TSE_LoadLock && m_TSE_LoadLock->x_NeedsDelayedMainChunk();
 }
 
 
@@ -666,6 +665,17 @@ void CLoadLockSetter::SetLoaded(void)
         if ( s_GetLoadTraceLevel() > 0 ) {
             LOG_POST(Info<<"GBLoader:"<<SBlobId(*m_TSE_LoadLock)<<" loaded");
         }
+        const CBlob_id* blob_id = dynamic_cast<const CBlob_id*>(&*m_TSE_LoadLock->GetBlobId());
+        if ( blob_id ) {
+            auto& tse_name = m_TSE_LoadLock->GetName();
+            if ( tse_name.IsNamed() ) {
+                // save the name into blob_id
+                const_cast<CBlob_id*>(blob_id)->SetTSEName(tse_name.GetName());
+            }
+            else if ( blob_id->HasTSEName() ) {
+                m_TSE_LoadLock->SetName(blob_id->GetTSEName());
+            }
+        }
         m_TSE_LoadLock.SetLoaded();
         TParent::SetLoaded(m_TSE_LoadLock, GBL::eExpire_normal);
         CReaderRequestResult& result =
