@@ -1,4 +1,4 @@
-/*  $Id: eutils_updater.cpp 676570 2023-12-18 19:55:37Z sadyrovr $
+/*  $Id: eutils_updater.cpp 691779 2025-01-02 19:05:32Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -39,8 +39,7 @@
 #include <objtools/eutils/efetch/PubmedBookArticle.hpp>
 #include <objtools/eutils/efetch/PubmedBookArticleSet.hpp>
 
-#include <objtools/eutils/api/efetch.hpp>
-#include <objtools/eutils/api/esearch.hpp>
+#include <objtools/eutils/esearch/ESearchResult.hpp>
 #include <objtools/eutils/esearch/IdList.hpp>
 #include <objects/pubmed/Pubmed_entry.hpp>
 #include <objects/medline/Medline_entry.hpp>
@@ -208,11 +207,13 @@ void SCitMatch::FillFromArticle(const CCit_art& A)
     }
 }
 
-class CECitMatch_Request : public CESearch_Request
+namespace
 {
-public:
-    CECitMatch_Request(CRef<CEUtils_ConnContext>& ctx) :
-        CESearch_Request("pubmed", ctx)
+
+struct CSearch_Request : CEUtils_Request
+{
+    CSearch_Request(CRef<CEUtils_ConnContext>& ctx) :
+        CEUtils_Request(ctx, "esearch.fcgi")
     {
     }
 
@@ -315,13 +316,34 @@ public:
         return true;
     }
 
-    TEntrezId GetResponse(EPubmedError& err)
+    string m_Term;
+
+    string GetQueryString() const override
+    {
+        string args = "db=pubmed&field=title&retmax=2&rettype=ulist&term=";
+        args += NStr::URLEncode(m_Term, NStr::eUrlEnc_ProcessMarkChars);
+        return args;
+    }
+
+    ESerialDataFormat GetSerialDataFormat() const override
+    {
+        return eSerial_Xml;
+    }
+
+    TEntrezId GetResponse(EPubmedError& err, string term)
     {
         err = EPubmedError::operational_error;
 
-        CRef<esearch::CESearchResult> result = this->GetESearchResult();
-        if (result && result->IsSetData()) {
-            auto& D = result->GetData();
+        m_Term = term;
+
+        CObjectIStream* is = GetObjIStream();
+        _ASSERT(is);
+        esearch::CESearchResult result;
+        *is >> result;
+        Disconnect();
+
+        if (result.IsSetData()) {
+            auto& D = result.GetData();
             if (D.IsInfo() && D.GetInfo().IsSetContent() && D.GetInfo().GetContent().IsSetIdList()) {
                 const auto& idList = D.GetInfo().GetContent().GetIdList();
                 if (idList.IsSetId()) {
@@ -349,9 +371,10 @@ public:
     }
 };
 
+}
 
-CEUtilsUpdater::CEUtilsUpdater(ENormalize norm) :
-    m_Ctx(new CEUtils_ConnContext), m_Norm(norm)
+CEUtilsUpdater::CEUtilsUpdater(ENormalize norm, EPubmedSource pmsrc) :
+    m_Ctx(new CEUtils_ConnContext), m_Norm(norm), m_pubmed_src(pmsrc)
 {
 }
 
@@ -371,14 +394,11 @@ TEntrezId CEUtilsUpdater::CitMatch(const CPub& pub, EPubmedError* perr)
 
 TEntrezId CEUtilsUpdater::CitMatch(const SCitMatch& cm, EPubmedError* perr)
 {
-    unique_ptr<CECitMatch_Request> req(new CECitMatch_Request(m_Ctx));
-    req->SetField("title");
-    req->SetRetMax(2);
-    req->SetUseHistory(false);
+    CSearch_Request req(m_Ctx);
     EPubmedError err = EPubmedError::citation_not_found;
 
     // clang-format off
-    constexpr array<eCitMatchFlags, 6> ruleset_single = {
+    static constexpr array<eCitMatchFlags, 6> ruleset_single = {
         e_J | e_V | e_P       | e_A | e_I,
         e_J | e_V | e_P       | e_A,
         e_J | e_V | e_P,
@@ -387,7 +407,7 @@ TEntrezId CEUtilsUpdater::CitMatch(const SCitMatch& cm, EPubmedError* perr)
                                 e_A       | e_T,
     };
 
-    constexpr array<eCitMatchFlags, 6> ruleset_in_press = {
+    static constexpr array<eCitMatchFlags, 6> ruleset_in_press = {
         e_J | e_V | e_P | e_Y | e_A,
         e_J | e_V | e_P | e_Y,
         e_J | e_V       | e_Y | e_A       | e_T,
@@ -398,16 +418,14 @@ TEntrezId CEUtilsUpdater::CitMatch(const SCitMatch& cm, EPubmedError* perr)
     // clang-format on
 
     const auto& ruleset = cm.InPress ? ruleset_in_press : ruleset_single;
-    const unsigned n       = cm.Option1 ? 6 : 5;
+    const unsigned n    = cm.Option1 ? 6 : 5;
 
     for (unsigned i = 0; i < n; ++i) {
         eCitMatchFlags r = ruleset[i];
 
         string term;
-        if (CECitMatch_Request::BuildSearchTerm(cm, r, term)) {
-            req->SetArgument("term", term);
-            req->SetRetType(CESearch_Request::eRetType_uilist);
-            TEntrezId pmid = req->GetResponse(err);
+        if (CSearch_Request::BuildSearchTerm(cm, r, term)) {
+            TEntrezId pmid = req.GetResponse(err, term);
             if (pmid != ZERO_ENTREZ_ID) {
                 return pmid;
             }
@@ -499,50 +517,104 @@ static void Normalize(CPub& pub)
     }
 }
 
+namespace
+{
+    struct CFetch_Request : CEUtils_Request {
+        TEntrezId m_pmid;
+        CFetch_Request(CRef<CEUtils_ConnContext>& ctx, TEntrezId pmid) :
+            CEUtils_Request(ctx, "efetch.fcgi"), m_pmid(pmid)
+        {
+            SetRequestMethod(CEUtils_Request::eHttp_Get);
+        }
+        string GetQueryString() const override
+        {
+            return "db=pubmed&retmode=xml&id="s + NStr::NumericToString(m_pmid);
+        }
+    };
+
+    struct CPubOneRequest : CEUtils_Request {
+        TEntrezId m_pmid;
+        CPubOneRequest(CRef<CEUtils_ConnContext>& ctx, TEntrezId pmid) :
+            CEUtils_Request(ctx, ""), m_pmid(pmid)
+        {
+            SetRequestMethod(CEUtils_Request::eHttp_Get);
+        }
+        string GetURL() const override
+        {
+            string s("https://pubmed.ncbi.nlm.nih.gov/api/pubone/pubmed/");
+            s += "pubmed_";
+            s += NStr::NumericToString(m_pmid);
+            return s;
+        }
+    };
+}
+
 CRef<CPubmed_entry> CEUtilsUpdater::x_GetPubmedEntry(TEntrezId pmid, EPubmedError* perr)
 {
-    unique_ptr<CEFetch_Request> req(
-        new CEFetch_Literature_Request(CEFetch_Literature_Request::eDB_pubmed, m_Ctx)
-    );
-
-    req->SetRequestMethod(CEUtils_Request::eHttp_Get);
-    req->GetId().AddId(NStr::NumericToString(pmid));
-    req->SetRetMode(CEFetch_Request::eRetMode_xml);
-
-    eutils::CPubmedArticleSet pas;
     string content;
-    req->Read(&content);
+    if (m_pubmed_src == EPubmedSource::EUtils) {
+        CFetch_Request req(m_Ctx, pmid);
+        req.Read(&content);
+    } else if (m_pubmed_src == EPubmedSource::PubOne) {
+        CPubOneRequest req(m_Ctx, pmid);
+        req.Read(&content);
+    }
+
     try {
+        eutils::CPubmedArticleSet pas;
         CNcbiIstrstream(content) >> MSerial_Xml >> pas;
-    } catch (const CSerialException&) {
-        if (perr) {
-            *perr = EPubmedError::operational_error;
+        if (pas.IsSetPP() && pas.GetPP().IsSetPP()) {
+            const auto& pp = pas.GetPP().GetPP();
+            if (! pp.empty()) {
+                const auto& ppf = *pp.front();
+                if (ppf.IsPubmedArticle()) {
+                    const eutils::CPubmedArticle& article = ppf.GetPubmedArticle();
+                    return article.ToPubmed_entry();
+                } else if (ppf.IsPubmedBookArticle()) {
+                    const eutils::CPubmedBookArticle& article = ppf.GetPubmedBookArticle();
+                    return article.ToPubmed_entry();
+                }
+            }
         }
-        return {};
-    } catch (...) {
         if (perr) {
             *perr = EPubmedError::citation_not_found;
         }
         return {};
-    }
-
-    const auto& pp = pas.GetPP().GetPP();
-    if (! pp.empty()) {
-        const auto& ppf = *pp.front();
-
-        CRef<CPubmed_entry> pme;
-        if (ppf.IsPubmedArticle()) {
-            const eutils::CPubmedArticle& article = ppf.GetPubmedArticle();
-            pme.Reset(article.ToPubmed_entry());
-        } else if (ppf.IsPubmedBookArticle()) {
-            const eutils::CPubmedBookArticle& article = ppf.GetPubmedBookArticle();
-            pme.Reset(article.ToPubmed_entry());
+    } catch (const CSerialException&) {
+        // not a <PubmedArticleSet>; continue
+    } catch (...) {
+        if (perr) {
+            *perr = EPubmedError::operational_error;
         }
-        return pme;
+        return {};
     }
 
+    try {
+        eutils::CPubmedBookArticleSet pbas;
+        CNcbiIstrstream(content) >> MSerial_Xml >> pbas;
+        if (pbas.IsSetPubmedBookArticle()) {
+            const auto& articles = pbas.GetPubmedBookArticle();
+            if (! articles.empty()) {
+                const eutils::CPubmedBookArticle& article = *articles.front();
+                return article.ToPubmed_entry();
+            }
+        }
+        if (perr) {
+            *perr = EPubmedError::citation_not_found;
+        }
+        return {};
+    } catch (const CSerialException&) {
+        // not a <PubmedBookArticleSet>; continue
+    } catch (...) {
+        if (perr) {
+            *perr = EPubmedError::operational_error;
+        }
+        return {};
+    }
+
+    // unrecognized content; fail
     if (perr) {
-        *perr = EPubmedError::citation_not_found;
+        *perr = EPubmedError::operational_error;
     }
     return {};
 }
@@ -580,6 +652,11 @@ CRef<CPub> CEUtilsUpdater::GetPubmedEntry(TEntrezId pmid, EPubmedError* perr)
     return x_GetPub(pmid, perr);
 }
 
+void CEUtilsUpdater::SetBaseURL(string url)
+{
+    CEUtils_Request::SetBaseURL(url);
+}
+
 CRef<CPub> CEUtilsUpdaterWithCache::GetPubmedEntry(TEntrezId pmid, EPubmedError* perr)
 {
     m_num_requests++;
