@@ -1,4 +1,4 @@
-/*  $Id: mail_report.cpp 632623 2021-06-03 17:38:11Z ivanov $
+/*  $Id: mail_report.cpp 692986 2025-02-04 14:14:56Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -117,8 +117,7 @@ void MakeMailReportPostReport(CSeq_table& table, CScope& scope)
     CRef<CSeqTable_column> pub_stat(new CSeqTable_column());
     pub_stat->SetHeader().SetTitle("is_unpublished");
     table.SetColumns().push_back(pub_stat);
-    CTaxon3 taxon3;
-    taxon3.Init();
+    CTaxon3 taxon3(CTaxon3::initialize::yes);
     CRef<CTaxon3_reply> reply = taxon3.SendOrgRefList(org_ref_list);
     if (reply) {
         ITERATE(CTaxon3_reply::TReply, reply_it, reply->GetReply()) {
@@ -156,13 +155,13 @@ void MakeMailReportPostReport(CSeq_table& table, CScope& scope)
 }
 
 
-void PrintReportLineHeader(CNcbiOstrstream& lines)
+void PrintReportLineHeader(ostringstream& lines)
 {
     lines << "Accession\tOld Name\tNew Name\n";
 }
 
 
-void ReportMailReportLine(CNcbiOstrstream& lines, const CSeq_table& table, size_t i, CScope* scope = NULL)
+void ReportMailReportLine(ostringstream& lines, const CSeq_table& table, size_t i, CScope* scope = nullptr)
 {
     string id;
     if (scope)
@@ -186,7 +185,7 @@ void ReportMailReportLine(CNcbiOstrstream& lines, const CSeq_table& table, size_
 
 string GetReportFromMailReportTable(const CSeq_table& table, CScope* scope)
 {
-    CNcbiOstrstream lines;
+    ostringstream lines;
 
     lines << "Failed Lookups\n";
     PrintReportLineHeader(lines);
@@ -212,7 +211,7 @@ string GetReportFromMailReportTable(const CSeq_table& table, CScope* scope)
         }
     }
 
-    return string(CNcbiOstrstreamToString(lines));
+    return lines.str();
 }
 
 
