@@ -1,4 +1,4 @@
-/* $Id: pub_fix.cpp 632623 2021-06-03 17:38:11Z ivanov $
+/* $Id: pub_fix.cpp 676260 2023-12-13 09:20:40Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,15 +55,12 @@
 
 #include "pub_fix_aux.hpp"
 
-#include <objects/mla/Title_msg.hpp>
-#include <objects/mla/Title_msg_list.hpp>
-#include <objects/mla/mla_client.hpp>
-
 #include <corelib/ncbi_message.hpp>
 #include <objtools/eutils/api/esearch.hpp>
 #include <objtools/eutils/esearch/IdList.hpp>
 #include <objtools/eutils/api/esummary.hpp>
 
+#include <cmath>
 
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
@@ -156,20 +153,7 @@ namespace fix_pub
 void MedlineToISO(CCit_art& cit_art)
 {
     if (cit_art.IsSetAuthors()) {
-
-        CAuth_list& auths = cit_art.SetAuthors();
-        if (auths.IsSetNames()) {
-            if (auths.GetNames().IsMl()) {
-                auths.ConvertMlToStandard();
-            }
-            else if (auths.GetNames().IsStd()) {
-                for (auto& auth : auths.SetNames().SetStd()) {
-                    if (auth->IsSetName() && auth->GetName().IsMl()) {
-                        auth = CAuthor::ConvertMlToStandard(*auth);
-                    }
-                }
-            }
-        }
+        cit_art.SetAuthors().ConvertMlToStd(true);
     }
 
     if (!cit_art.IsSetFrom() || !cit_art.GetFrom().IsJournal())
@@ -177,60 +161,6 @@ void MedlineToISO(CCit_art& cit_art)
 
     // from a journal - get iso_jta
     CCit_jour& journal = cit_art.SetFrom().SetJournal();
-
-    auto IsIso_jta = [](const CRef<CTitle::C_E>& title) -> bool { return title->IsIso_jta(); };
-
-    if (journal.IsSetTitle() && journal.GetTitle().IsSet()) {
-
-        auto& titles = journal.SetTitle().Set();
-
-        if (find_if(titles.begin(), titles.end(), IsIso_jta) == titles.end()) {
-            // no iso_jta
-
-            CTitle::C_E& first_title = *titles.front();
-            const string& title_str = journal.SetTitle().GetTitle(first_title);
-
-            CRef<CTitle> title_new(new CTitle);
-            CRef<CTitle::C_E> type_new(new CTitle::C_E);
-            type_new->SetIso_jta(title_str);
-            title_new->Set().push_back(type_new);
-
-            CRef<CTitle_msg> msg_new(new CTitle_msg);
-            msg_new->SetType(eTitle_type_iso_jta);
-            msg_new->SetTitle(*title_new);
-
-            CRef<CTitle_msg_list> msg_list_new;
-            try {
-                CMLAClient mla;
-                msg_list_new = mla.AskGettitle(*msg_new);
-            }
-            catch (exception &) {
-                // msg_list_new stays empty
-            }
-
-            if (msg_list_new.NotEmpty() && msg_list_new->IsSetTitles()) {
-
-                bool gotit = false;
-                for (auto& item : msg_list_new->GetTitles()) {
-                    const CTitle &cur_title = item->GetTitle();
-
-                    if (cur_title.IsSet()) {
-
-                        auto iso_jta_title = find_if(cur_title.Get().begin(), cur_title.Get().end(), IsIso_jta);
-                        if (iso_jta_title != cur_title.Get().end()) {
-                            gotit = true;
-                            first_title.SetIso_jta((*iso_jta_title)->GetIso_jta());
-                            break;
-                        }
-                    }
-
-                    if (gotit)
-                        break;
-                }
-            }
-        }
-    }
-
     if (journal.IsSetImp()) {
         // remove Eng language
         if (journal.GetImp().IsSetLanguage() && journal.GetImp().GetLanguage() == "Eng")
@@ -421,7 +351,7 @@ static void s_GetESearchIds(CESearch_Request& req,
         }
         req.Disconnect();
 
-        int sleepSeconds = sqrt(retry);
+        int sleepSeconds = int(sqrt(retry));
         if (sleepSeconds) {
             SleepSec(sleepSeconds);
         }
@@ -454,7 +384,7 @@ static bool s_IsIndexed(CRef<CEUtils_ConnContext> pContext,
         }
         request.Disconnect();
 
-        int sleepSeconds = sqrt(retry);
+        int sleepSeconds = int(sqrt(retry));
         if (sleepSeconds) {
             SleepSec(sleepSeconds);
         }
@@ -512,11 +442,13 @@ bool MUIsJournalIndexed(const string& journal)
         return false;
     }
 
-    if (ids.size() != 1) {
-        return false;
+    for (const string& id : ids) {
+        if (s_IsIndexed(pContext, id)) {
+            return true;
+        }
     }
 
-    return s_IsIndexed(pContext, ids.front());
+    return false;
 }
 
 
@@ -622,9 +554,13 @@ void PrintPub(const CCit_art& cit_art, bool found, bool auth, long muid, IMessag
         in_press = imprint->IsSetPrepub() && imprint->GetPrepub() == CImprint::ePrepub_in_press;
     }
 
+    ostringstream ostr;
+    ostr << last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page;
+    const string tail = ostr.str();
+
     if (auth) {
         ERR_POST_TO_LISTENER(err_log, eDiag_Error, err_Reference, err_Reference_MedArchMatchIgnored,
-            "Too many author name differences: " << muid << "|" << last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page);
+            "Too many author name differences: " << muid << "|" << tail);
         return;
     }
 
@@ -635,32 +571,32 @@ void PrintPub(const CCit_art& cit_art, bool found, bool auth, long muid, IMessag
 
         if (year && cur_year - year > YEAR_MAX_DIFF) {
             ERR_POST_TO_LISTENER(err_log, eDiag_Warning, err_Reference, err_Reference_OldInPress,
-                "encountered in-press article more than 2 years old: " << last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page);
+                "encountered in-press article more than " << YEAR_MAX_DIFF << " years old: " << tail);
         }
     }
 
     if (found) {
         ERR_POST_TO_LISTENER(err_log, eDiag_Info, err_Reference, err_Reference_SuccessfulPmidLookup,
-            muid << "|" << last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page);
+            muid << "|" << tail);
     }
     else if (MUIsJournalIndexed(title_str)) {
         if (muid) {
             ERR_POST_TO_LISTENER(err_log, eDiag_Warning, err_Reference, in_press ? err_Reference_PmidNotFoundInPress : err_Reference_PmidNotFound,
-                ">>" << muid << "<<|" << last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page);
+                ">>" << muid << "<<|" << tail);
         }
         else {
             ERR_POST_TO_LISTENER(err_log, eDiag_Warning, err_Reference, in_press ? err_Reference_PmidNotFoundInPress : err_Reference_PmidNotFound,
-                last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page);
+                tail);
         }
     }
     else {
         if (muid) {
             ERR_POST_TO_LISTENER(err_log, eDiag_Info, err_Reference, in_press ? err_Reference_NoPmidJournalNotInPubMedInPress : err_Reference_NoPmidJournalNotInPubMed,
-                ">>" << muid << "<<|" << last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page);
+                ">>" << muid << "<<|" << tail);
         }
         else {
             ERR_POST_TO_LISTENER(err_log, eDiag_Info, err_Reference, in_press ? err_Reference_NoPmidJournalNotInPubMedInPress : err_Reference_NoPmidJournalNotInPubMed,
-                last_name << " " << first_name << "|" << title_str << "|(" << year << ")|" << vol << "|" << page);
+                tail);
         }
     }
 }
@@ -671,6 +607,11 @@ bool IsFromBook(const CCit_art& art)
     return art.IsSetFrom() && art.GetFrom().IsBook();
 }
 
+static void MoveAuthors(CCit_art& to, CCit_art& from)
+{
+    to.SetAuthors(from.SetAuthors());
+    from.ResetAuthors();
+}
 
 static const size_t MAX_MATCH_COEFF = 3;
 
@@ -687,10 +628,10 @@ bool TenAuthorsCompare(CCit_art& cit_old, CCit_art& cit_new)
            num_of_authors = count_if(old_names.GetStr().begin(), old_names.GetStr().end(), StrNotEmpty);
 
     size_t match = 0;
-    for (auto& name : old_names.GetStr()) {
+    for (const string& name : old_names.GetStr()) {
 
         if (!name.empty()) {
-            if (NStr::FindNoCase(new_names.GetStr(), name) != nullptr) {
+            if (NStr::FindNoCase(new_names.GetStr(), name)) {
                 ++match;
             }
         }
@@ -704,8 +645,7 @@ bool TenAuthorsCompare(CCit_art& cit_old, CCit_art& cit_new)
 
     static const size_t MAX_AUTHORS = 10;
     if (min_num_of_authors > MAX_AUTHORS) {
-        cit_new.SetAuthors(cit_old.SetAuthors());
-        cit_old.ResetAuthors();
+        MoveAuthors(cit_new, cit_old);
     }
 
     return true;
@@ -715,7 +655,7 @@ size_t ExtractConsortiums(const CAuth_list::C_Names::TStd& names, CAuth_list::C_
 {
     size_t num_of_names = 0;
 
-    for (auto& name: names)
+    for (const auto& name : names)
     {
         const CAuthor& auth = *name;
         if (auth.IsSetName() && auth.GetName().IsName()) {
@@ -733,13 +673,125 @@ size_t ExtractConsortiums(const CAuth_list::C_Names::TStd& names, CAuth_list::C_
     return num_of_names;
 }
 
+static bool s_GetConsortia(const list<CRef<CAuthor>>& authors, list<string>& consortia)  {
+    
+    if (!consortia.empty()) {
+        consortia.clear();
+    }
+
+    for (const auto& pAuthor : authors) {
+        if (pAuthor && 
+            pAuthor->IsSetName() &&
+            pAuthor->GetName().IsConsortium()) {
+            consortia.push_back(pAuthor->GetName().GetConsortium());
+        }
+    }
+
+    if (!consortia.empty()) {
+        consortia.sort(PNocase());
+        return true;
+    }
+
+    return false;
+}
+
+
+static void s_TrimPrefixThe(CTempString& str)
+{
+    NStr::TrimPrefixInPlace(str, "The", NStr::eNocase);
+    NStr::TruncateSpacesInPlace(str, NStr::eTrunc_Begin);
+}
+
+static bool s_ConsortiaMatch(const list<string>& x, const list<string>& y)
+{
+    if (x.size() != y.size()) {
+        return false;
+    }
+
+
+    auto yit = begin(y);
+    for (auto xit = begin(x); xit != end(x); ++xit, ++yit) {
+        auto xval = NStr::TruncateSpaces_Unsafe(*xit);
+        auto yval = NStr::TruncateSpaces_Unsafe(*yit);
+
+        if (!NStr::EqualNocase(xval, yval)) {
+            const auto xval_starts_with_the = NStr::StartsWith(xval, "The", NStr::eNocase);
+            const auto yval_starts_with_the = NStr::StartsWith(yval, "The", NStr::eNocase);
+            if (xval_starts_with_the && !yval_starts_with_the) {
+                s_TrimPrefixThe(xval);
+                if (!NStr::EqualNocase(xval, yval)) {
+                    return false;
+                }
+            }
+            else if(yval_starts_with_the && !xval_starts_with_the) {
+                s_TrimPrefixThe(yval);
+                if (!NStr::EqualNocase(xval, yval)) {
+                    return false;
+                }
+            }
+            else {
+                return false;
+            }
+        }
+    }
+
+    return true;
+}
+
+static void s_ProcessConsortia(const CCit_art& old_cit, CCit_art& new_cit, IMessageListener* pListener) 
+{
+    if (!old_cit.IsSetAuthors() || 
+        !old_cit.GetAuthors().IsSetNames() ||
+        !old_cit.GetAuthors().GetNames().IsStd()) {
+        return;
+    }
+
+    if (!new_cit.IsSetAuthors() ||
+        !new_cit.GetAuthors().IsSetNames() ||
+        !new_cit.GetAuthors().GetNames().IsStd()) {
+        return;
+    }
+
+    list<string> old_consortia;
+    if (!s_GetConsortia(old_cit.GetAuthors().GetNames().GetStd(), old_consortia)) {
+        return;
+    }
+
+    list<string> new_consortia;
+    if (!s_GetConsortia(new_cit.GetAuthors().GetNames().GetStd(), new_consortia)) {
+        auto old_consortia_string = NStr::Join(old_consortia, ";");
+        ERR_POST_TO_LISTENER(pListener, eDiag_Warning, err_Reference, err_Reference_NoConsortAuthors,
+                "Publication as returned by MedArch lacks consortium authors of the original publication : \"" 
+                << old_consortia_string << "\".");
+        
+        auto& std_list = new_cit.SetAuthors().SetNames().SetStd();
+        transform(begin(old_consortia), 
+                  end(old_consortia), 
+                  back_inserter(std_list),
+                  [](const string& consortium) { 
+                    auto pAuthor = Ref(new CAuthor());
+                    pAuthor->SetName().SetConsortium(consortium);
+                    return pAuthor; });
+    }
+    else if (!s_ConsortiaMatch(old_consortia, new_consortia)) {
+        auto old_consortia_string = NStr::Join(old_consortia, ";");
+        auto new_consortia_string = NStr::Join(new_consortia, ";");
+        ERR_POST_TO_LISTENER(pListener, eDiag_Warning, err_Reference, err_Reference_DiffConsortAuthors,
+                    "Consortium author names differ. Original is \"" 
+                    << old_consortia_string 
+                    << "\". MedArch's is \"" 
+                    << new_consortia_string << "\".");
+
+    }
+}
+
 
 void GetFirstTenNames(const CAuth_list::C_Names::TStd& names, list<CTempString>& res)
 {
     static const size_t MAX_EXTRACTED = 10;
     size_t extracted = 0;
 
-    for (auto& name : names) {
+    for (const auto& name : names) {
         if (name->IsSetName() && name->GetName().IsName() && name->GetName().GetName().IsSetLast()) {
             res.push_back(name->GetName().GetName().GetLast());
             ++extracted;
@@ -756,8 +808,7 @@ bool TenAuthorsProcess(CCit_art& cit, CCit_art& new_cit, IMessageListener* err_l
 {
     if (!new_cit.IsSetAuthors() || !new_cit.GetAuthors().IsSetNames()) {
         if (cit.IsSetAuthors()) {
-            new_cit.SetAuthors(cit.SetAuthors());
-            cit.ResetAuthors();
+            MoveAuthors(new_cit, cit);
         }
         return true;
     }
@@ -812,7 +863,7 @@ bool TenAuthorsProcess(CCit_art& cit, CCit_art& new_cit, IMessageListener* err_l
     GetFirstTenNames(new_cit.GetAuthors().GetNames().GetStd(), new_author_names);
     size_t match = 0;
 
-    for (auto& name: cit.GetAuthors().GetNames().GetStd())
+    for (const auto& name: cit.GetAuthors().GetNames().GetStd())
     {
         const CAuthor& auth = *name;
         if (auth.IsSetName() && auth.GetName().IsName() && auth.GetName().GetName().IsSetLast()) {
@@ -877,8 +928,7 @@ bool TenAuthorsProcess(CCit_art& cit, CCit_art& new_cit, IMessageListener* err_l
     }
 
     if (replace_authors) {
-        new_cit.SetAuthors(cit.SetAuthors());
-        cit.ResetAuthors();
+        MoveAuthors(new_cit, cit);
     }
 
     return true;
@@ -893,7 +943,7 @@ void MergeNonPubmedPubIds(const CCit_art& cit_old, CCit_art& cit_new)
 
     const CArticleIdSet& old_ids = cit_old.GetIds();
 
-    for (auto& cur_id: old_ids.Get()) {
+    for (const auto& cur_id : old_ids.Get()) {
 
         if (!cur_id->IsDoi() && !cur_id->IsOther()) {
             continue;
@@ -1002,7 +1052,7 @@ void CPubFix::FixPubEquiv(CPub_equiv& pub_equiv)
         cit_arts;
 
     if (pub_equiv.IsSet()) {
-        for (auto& pub: pub_equiv.Set())
+        for (auto& pub : pub_equiv.Set())
         {
             if (pub->IsMuid()) {
                 muids.push_back(pub);
@@ -1059,7 +1109,7 @@ void CPubFix::FixPubEquiv(CPub_equiv& pub_equiv)
         oldpmid = pmids.front()->GetPmid();
 
         // check if more than one
-        for (auto& pub: pmids) {
+        for (const auto& pub : pmids) {
             if (pub->GetPmid() != oldpmid) {
                 ERR_POST_TO_LISTENER(m_err_log, eDiag_Warning, err_Reference, err_Reference_Multiple_pmid,
                     "Two different pmids in Pub-equiv [" << oldpmid << "] [" << pub->GetPmid() << "]");
@@ -1074,7 +1124,7 @@ void CPubFix::FixPubEquiv(CPub_equiv& pub_equiv)
         oldmuid = muids.front()->GetMuid();
 
         // check if more than one
-        for (auto& pub : muids) {
+        for (const auto& pub : muids) {
             if (pub->GetMuid() != oldmuid) {
                 ERR_POST_TO_LISTENER(m_err_log, eDiag_Warning, err_Reference, err_Reference_Multiple_pmid,
                     "Two different muids in Pub-equiv  [" << oldmuid << "] [" << pub->GetMuid() << "]");
@@ -1098,10 +1148,10 @@ void CPubFix::FixPubEquiv(CPub_equiv& pub_equiv)
 
         TEntrezId pmid = ZERO_ENTREZ_ID;
         try {
-            CMLAClient mla;
-            pmid = ENTREZ_ID_FROM(int, mla.AskCitmatchpmid(*new_pub));
-        }
-        catch (exception &) {
+            if (m_upd) {
+                pmid = m_upd->CitMatch(*new_pub);
+            }
+        } catch (...) {
             // pmid == 0
         }
 
@@ -1118,7 +1168,7 @@ void CPubFix::FixPubEquiv(CPub_equiv& pub_equiv)
             bool set_pmid = true;
             if (m_replace_cit) {
 
-                CRef<CCit_art> new_cit_art = FetchPubPmId(pmid);
+                CRef<CCit_art> new_cit_art = FetchPubPmId(pmid, m_upd);
 
                 if (new_cit_art.NotEmpty()) {
 
@@ -1128,10 +1178,10 @@ void CPubFix::FixPubEquiv(CPub_equiv& pub_equiv)
                         switch (outcome) {
                         case CAuthListValidator::eAccept_pubmed:
                             new_cit_is_valid = true;
+                            s_ProcessConsortia(*cit_art, *new_cit_art, m_err_log);
                             break;
                         case CAuthListValidator::eKeep_genbank:
-                            new_cit_art->SetAuthors(cit_art->SetAuthors());
-                            cit_art->ResetAuthors();
+                            MoveAuthors(*new_cit_art, *cit_art);
                             new_cit_is_valid = true;
                             break;
                         case CAuthListValidator::eFailed_validation:
@@ -1209,7 +1259,7 @@ void CPubFix::FixPubEquiv(CPub_equiv& pub_equiv)
     if (oldpmid != ZERO_ENTREZ_ID) {
         // have a pmid but no cit-art
 
-        CRef<CCit_art> new_cit_art = FetchPubPmId(oldpmid);
+        CRef<CCit_art> new_cit_art = FetchPubPmId(oldpmid, m_upd);
 
         if (new_cit_art.NotEmpty()) {
 
@@ -1262,17 +1312,17 @@ void CPubFix::FixPub(CPub& pub)
 
         TEntrezId pmid = ZERO_ENTREZ_ID;
         try {
-            CMLAClient mla;
-            pmid = ENTREZ_ID_FROM(int, mla.AskCitmatchpmid(pub));
-        }
-        catch (exception &) {
+            if (m_upd) {
+                pmid = m_upd->CitMatch(pub);
+            }
+        } catch (...) {
             // pmid == 0;
         }
 
         if (pmid > ZERO_ENTREZ_ID) {
             PrintPub(cit_art, true, false, ENTREZ_ID_TO(long, pmid), m_err_log);
             if (m_replace_cit) {
-                CRef<CCit_art> new_cit_art = FetchPubPmId(pmid);
+                CRef<CCit_art> new_cit_art = FetchPubPmId(pmid, m_upd);
 
                 if (new_cit_art.NotEmpty()) {
                     if (TenAuthorsProcess(cit_art, *new_cit_art, m_err_log)) {
@@ -1311,19 +1361,18 @@ void CPubFix::FixPub(CPub& pub)
     }
 }
 
-CRef<CCit_art> CPubFix::FetchPubPmId(TEntrezId pmid)
+CRef<CCit_art> CPubFix::FetchPubPmId(TEntrezId pmid, CEUtilsUpdater* upd)
 {
     CRef<CCit_art> cit_art;
-    if (pmid < ZERO_ENTREZ_ID)
+    if (!upd || pmid < ZERO_ENTREZ_ID) {
         return cit_art;
+    }
 
     CRef<CPub> pub;
     try {
-        CMLAClient mla;
-        pub = mla.AskGetpubpmid(CPubMedId(pmid));
-    }
-    catch (exception &) {
-        pub.Reset();
+        pub = upd->GetPub(pmid);
+    } catch (...) {
+        return cit_art;
     }
 
     if (pub.NotEmpty() && pub->IsArticle()) {
@@ -1336,6 +1385,7 @@ CRef<CCit_art> CPubFix::FetchPubPmId(TEntrezId pmid)
     return cit_art;
 }
 
+
 bool CAuthListValidator::enabled = true; // Verified in ID-6550, so set to use it by default
                                          // Setting it to false would lead to a few bugs
 bool CAuthListValidator::configured = false;
@@ -1365,10 +1415,14 @@ CAuthListValidator::EOutcome CAuthListValidator::validate(const CCit_art& gb_art
     if (pub_year < 1900 || pub_year > 3000) {
         throw logic_error("Publication from PubMed has invalid year: " + std::to_string(pub_year));
     }
-    gb_type = CAuth_list::C_Names::SelectionName(gb_art.GetAuthors().GetNames().Which());
-    get_lastnames(gb_art.GetAuthors(), removed, gb_auth_string);
-    pm_type = CAuth_list::C_Names::SelectionName(pm_art.GetAuthors().GetNames().Which());
-    get_lastnames(pm_art.GetAuthors(), added, pm_auth_string);
+    if (gb_art.IsSetAuthors()) {
+        gb_type = CAuth_list::C_Names::SelectionName(gb_art.GetAuthors().GetNames().Which());
+        get_lastnames(gb_art.GetAuthors(), removed, gb_auth_string);
+    }
+    if (pm_art.IsSetAuthors()) {
+        pm_type = CAuth_list::C_Names::SelectionName(pm_art.GetAuthors().GetNames().Which());
+        get_lastnames(pm_art.GetAuthors(), added, pm_auth_string);
+    }
     matched.clear();
     compare_lastnames();
     actual_matched_to_min = double(cnt_matched) / cnt_min;
@@ -1439,7 +1493,7 @@ void CAuthListValidator::DebugDump(CNcbiOstream& out) const
 void CAuthListValidator::dumplist(const char* hdr, const list<string>& lst, CNcbiOstream& out) const
 {
     out << lst.size() << " " << hdr << " authors:\n";
-    for (const auto& a : lst)
+    for (const string& a : lst)
         out << "    " << a << "\n";
 }
 
@@ -1492,7 +1546,7 @@ void CAuthListValidator::get_lastnames(const CAuth_list& authors, list<string>&
 
 void CAuthListValidator::get_lastnames(const CAuth_list::C_Names::TStd& authors, list<string>& lastnames)
 {
-    for (auto& name : authors) {
+    for (const auto& name : authors) {
         if (name->IsSetName() && name->GetName().IsName() && name->GetName().GetName().IsSetLast()) {
             string lname(name->GetName().GetName().GetLast());
             lastnames.push_back(NStr::ToLower(lname));
@@ -1503,7 +1557,7 @@ void CAuthListValidator::get_lastnames(const CAuth_list::C_Names::TStd& authors,
 void CAuthListValidator::get_lastnames(const CAuth_list::C_Names::TStr& authors, list<string>& lastnames)
 {
     const char* alpha = "abcdefghijklmnopqrstuvwxyz";
-    for (auto auth : authors) {
+    for (string auth : authors) {
         size_t eow = NStr::ToLower(auth).find_first_not_of(alpha);
         lastnames.push_back(auth.substr(0, eow));
     }
