@@ -1,4 +1,4 @@
-/*  $Id: remote_updater.cpp 677028 2024-01-03 14:08:01Z ivanov $
+/*  $Id: remote_updater.cpp 693395 2025-02-11 14:10:16Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -131,6 +131,8 @@ CRef<CPub> s_GetPubFrompmid(CEUtilsUpdater* upd, TEntrezId id, int maxAttempts,
 
 } // end anonymous namespace
 
+static const bool kUseBulkTaxonQuery = true;
+
 class CCachedTaxon3_impl
 {
 public:
@@ -162,10 +164,10 @@ public:
 
     void ReportStats(std::ostream& str)
     {
-        str << "CRemoteUpdater: cache_hits " << m_cache_hits << " out of " << m_num_requests << " requests\n";
+        str << "CTaxonomyUpdater: cache_hits " << m_cache_hits << " out of " << m_num_requests << " requests\n";
     }
 
-    CRef<COrg_ref> GetOrg(const COrg_ref& org, CRemoteUpdater::FLogger f_logger)
+    CRef<COrg_ref> GetOrg(const COrg_ref& org, CTaxonomyUpdater::FLogger f_logger)
     {
         CRef<COrg_ref> result;
         CRef<CT3Reply> reply = GetOrgReply(org, f_logger);
@@ -175,7 +177,7 @@ public:
         return result;
     }
 
-    CRef<CT3Reply> GetOrgReply(const COrg_ref& in_org, CRemoteUpdater::FLogger f_logger)
+    CRef<CT3Reply> GetOrgReply(const COrg_ref& in_org, CTaxonomyUpdater::FLogger f_logger)
     {
         m_num_requests++;
         std::ostringstream os;
@@ -214,14 +216,90 @@ public:
         return reply;
     }
 
-    CRef<CTaxon3_reply> SendOrgRefList(const vector<CRef<COrg_ref>>& query, CRemoteUpdater::FLogger logger)
+    CRef<CTaxon3_reply> SendOrgRefList(const vector<CRef<COrg_ref>>& query, CTaxonomyUpdater::FLogger logger)
     {
         CRef<CTaxon3_reply> result(new CTaxon3_reply);
-
-        for (const auto& it : query) {
-            result->SetReply().push_back(GetOrgReply(*it, logger));
+        if ( kUseBulkTaxonQuery ) {
+            size_t n = query.size();
+            m_num_requests += n;
+            // lookup cache first
+            vector<string> keys(n);
+            vector<CRef<CT3Reply>*> cache_refs(n);
+            map<string, pair<CRef<COrg_ref>, CRef<CT3Reply>*>> to_ask;
+            for ( size_t i = 0; i < n; ++i ) {
+                std::ostringstream os;
+                os << MSerial_AsnText << *query[i];
+                keys[i] = os.str();
+                CRef<CT3Reply>& reply = (*m_cache)[keys[i]];
+                cache_refs[i] = &reply;
+                if ( reply ) {
+                    m_cache_hits++;
+                }
+                else {
+                    to_ask[keys[i]] = make_pair(query[i], &reply);
+                }
+            }
+            if ( !to_ask.empty() ) { // there are cache misses
+                // prepare request of new orgs
+                vector<pair<CRef<COrg_ref>, CRef<CT3Reply>*>> to_ask_vec;
+                for ( const auto& it : to_ask ) {
+                    to_ask_vec.push_back(it.second);
+                }
+                size_t m = to_ask_vec.size();
+                CTaxon3_request whole_request;
+                auto& requests = whole_request.SetRequest();
+                for ( auto& it : to_ask_vec ) {
+                    CRef<CT3Request> rq(new CT3Request);
+                    rq->SetOrg(*it.first);
+                    requests.push_back(rq);
+                }
+                // invoke request
+                CStopWatch sw(CStopWatch::eStart);
+                CRef<CTaxon3_reply> whole_reply = m_taxon->SendRequest(whole_request);
+                LOG_POST(Info<<"Got "<<m<<" taxonomy in "<<sw.Elapsed()<<" s");
+                auto& replies = whole_reply->SetReply();
+                if ( replies.size() != m ) {
+                    // requests and replies mismatch
+                    if (logger) {
+                        const string& error_message =
+                            "Taxon update: got " + NStr::NumericToString(replies.size()) +
+                            " replies for " + NStr::NumericToString(m) +
+                            " requests";
+                        logger(error_message);
+                    }
+                    return result; // Error?
+                }
+                // store replies into cache
+                size_t i = 0;
+                for ( auto& reply : replies ) {
+                    auto& ask = to_ask_vec[i++];
+                    *ask.second = reply;
+                    if (reply->IsError() && logger) {
+                        auto& in_org = *ask.first;
+                        const string& error_message =
+                            "Taxon update: " +
+                            (in_org.IsSetTaxname() ? in_org.GetTaxname() : NStr::NumericToString(in_org.GetTaxId())) + ": " +
+                            reply->GetError().GetMessage();
+                        
+                        logger(error_message);
+                    } else if (reply->IsData() && reply->SetData().IsSetOrg()) {
+                        reply->SetData().SetOrg().ResetSyn();
+                        // next will reset 'attrib = specified'
+                        // RW-1380 why do we need to reset attrib 'specified' ?
+                        //reply->SetData().SetOrg().SetOrgname().SetFormalNameFlag(false);
+                    }
+                }
+            }
+            // update results with new cache content
+            for ( size_t i = 0; i < n; ++i ) {
+                result->SetReply().push_back(*cache_refs[i]);
+            }
+        }
+        else {
+            for (const auto& it : query) {
+                result->SetReply().push_back(GetOrgReply(*it, logger));
+            }
         }
-
         return result;
     }
 
@@ -232,7 +310,7 @@ protected:
     size_t                      m_cache_hits   = 0;
 };
 
-bool CRemoteUpdater::xUpdatePubPMID(list<CRef<CPub>>& arr, TEntrezId id)
+bool CPubmedUpdater::xUpdatePubPMID(list<CRef<CPub>>& arr, TEntrezId id)
 {
     auto pub = s_GetPubFrompmid(m_pubmed.get(), id, m_MaxMlaAttempts, m_pMessageListener);
     if (! (pub && pub->IsMedline())) {
@@ -257,7 +335,7 @@ bool CRemoteUpdater::xUpdatePubPMID(list<CRef<CPub>>& arr, TEntrezId id)
     // authors come back in a weird format that we need
     // to convert to ISO
     if (new_pub->IsSetAuthors())
-        CRemoteUpdater::ConvertToStandardAuthors(new_pub->SetAuthors());
+        CPubmedUpdater::ConvertToStandardAuthors(new_pub->SetAuthors());
 
     arr.clear();
     CRef<CPub> new_pmid(new CPub);
@@ -267,20 +345,20 @@ bool CRemoteUpdater::xUpdatePubPMID(list<CRef<CPub>>& arr, TEntrezId id)
     return true;
 }
 
-void CRemoteUpdater::SetMaxMlaAttempts(int maxAttempts)
+void CPubmedUpdater::SetMaxMlaAttempts(int maxAttempts)
 {
     m_MaxMlaAttempts = maxAttempts;
 }
 
-void CRemoteUpdater::SetTaxonTimeout(unsigned seconds, unsigned retries, bool exponential)
+void CTaxonomyUpdater::SetTaxonTimeout(unsigned seconds, unsigned retries, bool exponential)
 {
-    m_TaxonTimeoutSet  = true;
-    m_TaxonTimeout     = seconds;
-    m_TaxonAttempts    = retries;
-    m_TaxonExponential = exponential;
+    m_TimeoutSet  = true;
+    m_Timeout     = seconds;
+    m_Attempts    = retries;
+    m_Exponential = exponential;
 }
 
-bool CRemoteUpdater::xSetFromConfig()
+void CTaxonomyUpdater::xSetFromConfig()
 {
     // default update lambda function
     m_taxon_update = [this](const vector<CRef<COrg_ref>>& query) -> CRef<CTaxon3_reply>
@@ -291,6 +369,26 @@ bool CRemoteUpdater::xSetFromConfig()
         return copied;
     };
 
+    CNcbiApplicationAPI* app = CNcbiApplicationAPI::Instance();
+    if (app) {
+        const CNcbiRegistry& cfg = app->GetConfig();
+        if (cfg.HasEntry("RemoteTaxonomyUpdate")) {
+            const string sect = "RemoteTaxonomyUpdate";
+            int delay = cfg.GetInt(sect, "RetryDelay", 20);
+            if (delay < 0)
+                delay = 20;
+            int count = cfg.GetInt(sect, "RetryCount", 5);
+            if (count < 0)
+                count = 5;
+            bool exponential = cfg.GetBool(sect, "RetryExponentially", false);
+
+            SetTaxonTimeout(static_cast<unsigned>(delay), static_cast<unsigned>(count), exponential);
+        }
+    }
+}
+
+bool CPubmedUpdater::xSetFromConfig()
+{
     CNcbiApplicationAPI* app = CNcbiApplicationAPI::Instance();
     if (app) {
         const CNcbiRegistry& cfg = app->GetConfig();
@@ -305,19 +403,6 @@ bool CRemoteUpdater::xSetFromConfig()
             if (cfg.HasEntry(sect, "UseCache")) {
                 m_pm_use_cache = cfg.GetBool(sect, "UseCache", true);
             }
-        }
-
-        if (cfg.HasEntry("RemoteTaxonomyUpdate")) {
-            const string sect = "RemoteTaxonomyUpdate";
-            int delay = cfg.GetInt(sect, "RetryDelay", 20);
-            if (delay < 0)
-                delay = 20;
-            int count = cfg.GetInt(sect, "RetryCount", 5);
-            if (count < 0)
-                count = 5;
-            bool exponential = cfg.GetBool(sect, "RetryExponentially", false);
-
-            SetTaxonTimeout(static_cast<unsigned>(delay), static_cast<unsigned>(count), exponential);
             return true;
         }
     }
@@ -325,7 +410,7 @@ bool CRemoteUpdater::xSetFromConfig()
     return false;
 }
 
-void CRemoteUpdater::UpdateOrgFromTaxon(CSeqdesc& desc)
+void CTaxonomyUpdater::UpdateOrgFromTaxon(CSeqdesc& desc)
 {
     if (desc.IsOrg()) {
         xUpdateOrgTaxname(desc.SetOrg());
@@ -334,18 +419,18 @@ void CRemoteUpdater::UpdateOrgFromTaxon(CSeqdesc& desc)
     }
 }
 
-void CRemoteUpdater::xInitTaxCache()
+void CTaxonomyUpdater::xInitTaxCache()
 {
     if (! m_taxClient) {
         m_taxClient.reset(new CCachedTaxon3_impl);
-        if (m_TaxonTimeoutSet)
-            m_taxClient->InitWithTimeout(m_TaxonTimeout, m_TaxonAttempts, m_TaxonExponential);
+        if (m_TimeoutSet)
+            m_taxClient->InitWithTimeout(m_Timeout, m_Attempts, m_Exponential);
         else
             m_taxClient->Init();
     }
 }
 
-void CRemoteUpdater::xUpdateOrgTaxname(COrg_ref& org)
+void CTaxonomyUpdater::xUpdateOrgTaxname(COrg_ref& org)
 {
     std::lock_guard<std::mutex> guard(m_Mutex);
 
@@ -367,14 +452,8 @@ CRemoteUpdater& CRemoteUpdater::GetInstance()
     return instance;
 }
 
-CRemoteUpdater::CRemoteUpdater(FLogger logger, CEUtilsUpdater::ENormalize norm) :
-    m_logger{ logger }, m_pm_normalize(norm)
-{
-    xSetFromConfig();
-}
-
-CRemoteUpdater::CRemoteUpdater(IObjtoolsListener* pMessageListener, CEUtilsUpdater::ENormalize norm) :
-    m_pMessageListener(pMessageListener), m_pm_normalize(norm)
+CTaxonomyUpdater::CTaxonomyUpdater(IObjtoolsListener* pMessageListener) :
+    m_pMessageListener(pMessageListener)
 {
     if (m_pMessageListener) {
         m_logger = [this](const string& error_message) {
@@ -384,17 +463,48 @@ CRemoteUpdater::CRemoteUpdater(IObjtoolsListener* pMessageListener, CEUtilsUpdat
     xSetFromConfig();
 }
 
+CTaxonomyUpdater::CTaxonomyUpdater(FLogger logger) :
+    m_logger(logger)
+{
+    xSetFromConfig();
+}
+
+CTaxonomyUpdater::~CTaxonomyUpdater()
+{
+}
+
+CPubmedUpdater::CPubmedUpdater(IObjtoolsListener* pMessageListener, CEUtilsUpdater::ENormalize norm) :
+    m_pMessageListener(pMessageListener), m_pm_normalize(norm)
+{
+    xSetFromConfig();
+}
+
+CRemoteUpdater::CRemoteUpdater(CTaxonomyUpdater::FLogger logger, CEUtilsUpdater::ENormalize norm) :
+    m_taxon(logger), m_pubmed(nullptr, norm)
+{
+}
+
+CRemoteUpdater::CRemoteUpdater(IObjtoolsListener* pMessageListener, CEUtilsUpdater::ENormalize norm) :
+    m_taxon(pMessageListener), m_pubmed(pMessageListener, norm)
+{
+}
+
 CRemoteUpdater::~CRemoteUpdater()
 {
 }
 
-void CRemoteUpdater::ClearCache()
+void CTaxonomyUpdater::ClearCache()
 {
     std::lock_guard<std::mutex> guard(m_Mutex);
 
     if (m_taxClient) {
         m_taxClient->ClearCache();
     }
+}
+
+void CPubmedUpdater::ClearCache()
+{
+    std::lock_guard<std::mutex> guard(m_Mutex);
 
     if (m_pm_use_cache && m_pubmed) {
         auto* upd = dynamic_cast<CEUtilsUpdaterWithCache*>(m_pubmed.get());
@@ -404,14 +514,14 @@ void CRemoteUpdater::ClearCache()
     }
 }
 
-void CRemoteUpdater::UpdatePubReferences(CSeq_entry_EditHandle& obj)
+void CPubmedUpdater::UpdatePubReferences(CSeq_entry_EditHandle& obj)
 {
     for (CBioseq_CI it(obj); it; ++it) {
         xUpdatePubReferences(it->GetEditHandle().SetDescr());
     }
 }
 
-void CRemoteUpdater::UpdatePubReferences(CSerialObject& obj)
+void CPubmedUpdater::UpdatePubReferences(CSerialObject& obj)
 {
     if (obj.GetThisTypeInfo()->IsType(CSeq_entry::GetTypeInfo())) {
         CSeq_entry* entry = static_cast<CSeq_entry*>(&obj);
@@ -432,7 +542,7 @@ void CRemoteUpdater::UpdatePubReferences(CSerialObject& obj)
     }
 }
 
-void CRemoteUpdater::xUpdatePubReferences(CSeq_entry& entry)
+void CPubmedUpdater::xUpdatePubReferences(CSeq_entry& entry)
 {
     if (entry.IsSet()) {
         for (auto& it : entry.SetSet().SetSeq_set()) {
@@ -446,7 +556,7 @@ void CRemoteUpdater::xUpdatePubReferences(CSeq_entry& entry)
     xUpdatePubReferences(entry.SetDescr());
 }
 
-void CRemoteUpdater::xUpdatePubReferences(CSeq_descr& seq_descr)
+void CPubmedUpdater::xUpdatePubReferences(CSeq_descr& seq_descr)
 {
     std::lock_guard<std::mutex> guard(m_Mutex);
 
@@ -463,7 +573,7 @@ void CRemoteUpdater::xUpdatePubReferences(CSeq_descr& seq_descr)
                 m_pubmed.reset(new CEUtilsUpdater(m_pm_normalize));
             }
             if (! m_pm_url.empty()) {
-                CEUtils_Request::SetBaseURL(m_pm_url);
+                m_pubmed->SetBaseURL(m_pm_url);
             }
             if (m_pm_interceptor) {
                 m_pubmed->SetPubInterceptor(m_pm_interceptor);
@@ -540,7 +650,7 @@ namespace
     }
 }
 
-void CRemoteUpdater::UpdateOrgFromTaxon(CSeq_entry& entry)
+void CTaxonomyUpdater::UpdateOrgFromTaxon(CSeq_entry& entry)
 {
     TOrgMap org_to_update;
 
@@ -552,12 +662,20 @@ void CRemoteUpdater::UpdateOrgFromTaxon(CSeq_entry& entry)
 
     if (! m_taxClient) {
         m_taxClient.reset(new CCachedTaxon3_impl);
-        if (m_TaxonTimeoutSet)
-            m_taxClient->InitWithTimeout(m_TaxonTimeout, m_TaxonAttempts, m_TaxonExponential);
+        if (m_TimeoutSet)
+            m_taxClient->InitWithTimeout(m_Timeout, m_Attempts, m_Exponential);
         else
             m_taxClient->Init();
     }
 
+    if ( kUseBulkTaxonQuery ) {
+        vector<CRef<COrg_ref>> reflist;
+        for (auto& it : org_to_update) {
+            reflist.push_back(it.second.org_ref);
+        }
+        m_taxClient->SendOrgRefList(reflist, m_logger);
+    }
+
     for (auto& it : org_to_update) {
         vector<CRef<COrg_ref>> reflist;
         reflist.push_back(it.second.org_ref);
@@ -573,7 +691,7 @@ void CRemoteUpdater::UpdateOrgFromTaxon(CSeq_entry& entry)
 }
 
 
-void CRemoteUpdater::ConvertToStandardAuthors(CAuth_list& auth_list)
+void CPubmedUpdater::ConvertToStandardAuthors(CAuth_list& auth_list)
 {
     if (! auth_list.IsSetNames()) {
         return;
@@ -602,7 +720,7 @@ void CRemoteUpdater::ConvertToStandardAuthors(CAuth_list& auth_list)
 }
 
 
-void CRemoteUpdater::PostProcessPubs(CSeq_entry& obj)
+void CPubmedUpdater::PostProcessPubs(CSeq_entry& obj)
 {
     if (obj.IsSet()) {
         for (CRef<CSeq_entry>& it : obj.SetSet().SetSeq_set()) {
@@ -617,7 +735,7 @@ void CRemoteUpdater::PostProcessPubs(CSeq_entry& obj)
     }
 }
 
-void CRemoteUpdater::PostProcessPubs(CPubdesc& pubdesc)
+void CPubmedUpdater::PostProcessPubs(CPubdesc& pubdesc)
 {
     if (! pubdesc.IsSetPub())
         return;
@@ -629,7 +747,7 @@ void CRemoteUpdater::PostProcessPubs(CPubdesc& pubdesc)
     }
 }
 
-void CRemoteUpdater::PostProcessPubs(CSeq_entry_EditHandle& obj)
+void CPubmedUpdater::PostProcessPubs(CSeq_entry_EditHandle& obj)
 {
     for (CBioseq_CI bioseq_it(obj); bioseq_it; ++bioseq_it) {
         for (CSeqdesc_CI desc_it(bioseq_it->GetEditHandle(), CSeqdesc::e_Pub); desc_it; ++desc_it) {
@@ -638,12 +756,12 @@ void CRemoteUpdater::PostProcessPubs(CSeq_entry_EditHandle& obj)
     }
 }
 
-void CRemoteUpdater::SetPubmedClient(CEUtilsUpdater* pubmedUpdater)
+void CPubmedUpdater::SetPubmedClient(CEUtilsUpdater* pubmedUpdater)
 {
     m_pubmed.reset(pubmedUpdater);
 }
 
-CConstRef<CTaxon3_reply> CRemoteUpdater::SendOrgRefList(const vector<CRef<COrg_ref>>& list)
+CConstRef<CTaxon3_reply> CTaxonomyUpdater::SendOrgRefList(const vector<CRef<COrg_ref>>& list)
 {
     std::lock_guard<std::mutex> guard(m_Mutex);
 
@@ -653,13 +771,18 @@ CConstRef<CTaxon3_reply> CRemoteUpdater::SendOrgRefList(const vector<CRef<COrg_r
     return reply;
 }
 
-void CRemoteUpdater::ReportStats(std::ostream& os)
+void CTaxonomyUpdater::ReportStats(std::ostream& os)
 {
     std::lock_guard<std::mutex> guard(m_Mutex);
 
     if (m_taxClient) {
         m_taxClient->ReportStats(os);
     }
+}
+
+void CPubmedUpdater::ReportStats(std::ostream& os)
+{
+    std::lock_guard<std::mutex> guard(m_Mutex);
 
     if (m_pm_use_cache && m_pubmed) {
         auto* upd = dynamic_cast<CEUtilsUpdaterWithCache*>(m_pubmed.get());
