@@ -1,4 +1,4 @@
-/*  $Id: text_object_description.cpp 629259 2021-04-13 13:28:40Z ivanov $
+/*  $Id: text_object_description.cpp 692986 2025-02-04 14:14:56Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -85,7 +85,7 @@ static CConstRef<CSeq_id> GetBestId(const CBioseq& bioseq)
 
 static string GetTextObjectDescription(CBioseq_set_Handle bssh)
 {
-    CNcbiOstrstream result_strm;
+    ostringstream result_strm;
 
     CBioseq_set::EClass bioseq_set_class = bssh.IsSetClass() ? bssh.GetClass() : CBioseq_set::eClass_not_set;
     if (bioseq_set_class == CBioseq_set::eClass_segset || bioseq_set_class == CBioseq_set::eClass_nuc_prot) {
@@ -139,7 +139,7 @@ static string GetTextObjectDescription(CBioseq_set_Handle bssh)
         }
     }
 
-    return CNcbiOstrstreamToString(result_strm);
+    return result_strm.str();
 }
 
 
@@ -150,7 +150,7 @@ static void GetSeqFeatLabel(const CSeq_feat& seq_feat, string& label)
     feature::GetLabel(seq_feat, &label, feature::fFGL_Content);
     size_t pos;
     if (seq_feat.GetData().IsRna() && !label.empty() && (string::npos != (pos = label.find("RNA-")))) {
-        label = CTempString(label).substr(pos + 4);
+        label = label.substr(pos + 4);
     }
     string number = "/number=";
     if (!label.empty()
