@@ -1,4 +1,4 @@
-/*  $Id: cigar_formatter.cpp 562945 2018-04-30 16:39:32Z kans $
+/*  $Id: cigar_formatter.cpp 692982 2025-02-04 14:14:33Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -52,7 +52,7 @@ CCIGAR_Formatter::CCIGAR_Formatter(const CSeq_align&    aln,
                                    CScope*              scope,
                                    TCIGARFlags          flags)
     : m_Align(aln),
-      m_CurAlign(NULL),
+      m_CurAlign(nullptr),
       m_Scope(scope),
       m_Flags(flags),
       m_IsFirstSubalign(true),
@@ -126,7 +126,7 @@ void CCIGAR_Formatter::x_FormatAlignmentRows(const CSeq_align& sa,
                 StartSubAlignment();
                 x_FormatAlignmentRows(**it, width_inverted);
                 EndSubAlignment();
-                m_CurAlign = NULL;
+                m_CurAlign = nullptr;
                 m_IsFirstSubalign = false;
             }
             break;
@@ -159,7 +159,7 @@ void CCIGAR_Formatter::x_FormatLine(bool width_inverted)
     if (m_TargetRow == m_RefRow) {
         return;
     }
-    CNcbiOstrstream cigar;
+    stringstream cigar;
     m_LastType = 0;
     TSeqPos last_count = 0;
 
@@ -222,7 +222,7 @@ void CCIGAR_Formatter::x_FormatLine(bool width_inverted)
         TRange tgt_piece = m_AlnMap->GetRange(m_TargetRow, seg);
         CAlnMap::TSegTypeFlags ref_flags = m_AlnMap->GetSegType(m_RefRow, seg);
         CAlnMap::TSegTypeFlags tgt_flags = m_AlnMap->GetSegType(m_TargetRow, seg);
-        //The type and count are guaranteed set by one of the if/else cases below.  
+        //The type and count are guaranteed set by one of the if/else cases below.
         char type = 'X'; // Guaranteed set. Pacify compiler.
         TSeqPos count = 0;  // Guaranteed set. Pacify compiler.
         TSignedSeqPos frameshift = 0;
@@ -342,11 +342,10 @@ void CCIGAR_Formatter::x_FormatLine(bool width_inverted)
             last_frameshift = frameshift;
         }
     }
-    CNcbiOstrstream aln_out;
 
     AddSegment(cigar, m_LastType, last_count);
-    string cigar_string = CNcbiOstrstreamToString(cigar);
 
+    string cigar_string = cigar.str();
     AddRow(cigar_string);
 
     EndRow();
