@@ -1,4 +1,4 @@
-/*  $Id: defline_item.cpp 647299 2022-03-23 16:47:41Z kans $
+/*  $Id: defline_item.cpp 693505 2025-02-13 14:43:47Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -94,6 +94,9 @@ void CDeflineItem::x_GatherInfo(CBioseqContext& ctx)
     if ( ctx.Config().ShowDeflineModifiers() ) {
         flags |= sequence::CDeflineGenerator::fShowModifiers;
     }
+    if ( ctx.Config().LeavePrefixSuffix() ) {
+        flags |= sequence::CDeflineGenerator::fLeavePrefixSuffix;
+    }
     if ( ctx.UsingSeqEntryIndex() ) {
         CRef<CSeqEntryIndex> idx = ctx.GetSeqEntryIndex();
         CBioseq_Handle bsh = scope.GetBioseqHandle(*bioseq);
