@@ -1,4 +1,4 @@
-/*  $Id: genbank_formatter.cpp 683835 2024-06-06 12:08:29Z ivanov $
+/*  $Id: genbank_formatter.cpp 692982 2025-02-04 14:14:33Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -82,7 +82,8 @@ BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
 CGenbankFormatter::CGenbankFormatter(void) :
-    m_uFeatureCount( 0 ), m_bHavePrintedSourceFeatureJavascript(false)
+    m_uFeatureCount( 0 ), m_bHavePrintedSourceFeatureJavascript(false),
+    m_bSourceDescriptorDone(false)
 {
     SetIndent(string(12, ' '));
     SetFeatIndent(string(21, ' '));
@@ -214,7 +215,7 @@ void s_PrintLocAsJavascriptArray(
 {
     CBioseq_Handle &bioseq_handle = ctx.GetHandle();
 
-    CNcbiOstrstream result; // will hold complete printed location
+    ostringstream result; // will hold complete printed location
     result << "[";
 
     // special case for when the location is just a point with "lim tr"
@@ -227,7 +228,7 @@ void s_PrintLocAsJavascriptArray(
         const TSeqPos point = loc.GetPnt().GetPoint();
         // Note the "+2"
         result << "[" << (point+1) << ", " << (point+2) << "]]";
-        text_os << (string)CNcbiOstrstreamToString(result);
+        text_os << result.str();
         return;
     }
 
@@ -266,7 +267,7 @@ void s_PrintLocAsJavascriptArray(
         is_first = false;
     }
     result << "]";
-    text_os << (string)CNcbiOstrstreamToString(result);
+    text_os << result.str();
 }
 
 static
@@ -282,24 +283,14 @@ string s_GetAccessionWithoutPeriod(
     return accn;
 }
 
-static string s_get_anchor_html(const string & sAnchorName, TGi iGi )
-{
-    CNcbiOstrstream result;
-
-    result << "<a name=\"" << sAnchorName << "_"
-        << iGi << "\"></a>";
-
-    return (string)CNcbiOstrstreamToString(result);
-}
-
 static string s_get_anchor_html(const string & sAnchorName, CBioseqContext *ctx )
 {
-    CNcbiOstrstream result;
+    ostringstream result;
 
     result << "<a name=\"" << sAnchorName << "_"
         << ctx->GetAccession() << "\"></a>";
 
-    return (string)CNcbiOstrstreamToString(result);
+    return result.str();
 }
 
 void CGenbankFormatter::EndSection
@@ -333,6 +324,7 @@ void CGenbankFormatter::EndSection
     // New record, so reset
     m_FeatureKeyToLocMap.clear();
     m_bHavePrintedSourceFeatureJavascript = false;
+    m_bSourceDescriptorDone = false;
 }
 
 
@@ -379,7 +371,7 @@ void CGenbankFormatter::FormatLocus
     CBioseqContext& ctx = *locus.GetContext();
 
     list<string> l;
-    CNcbiOstrstream locus_line;
+    ostringstream locus_line;
 
     const char* units = "bp";
     if ( !ctx.IsProt() ) {
@@ -429,7 +421,7 @@ void CGenbankFormatter::FormatLocus
 
     const bool is_html = GetContext().GetConfig().DoHTML() ;
 
-    string locus_line_str = CNcbiOstrstreamToString(locus_line);
+    string locus_line_str = locus_line.str();
     if ( is_html ) {
         TryToSanitizeHtml( locus_line_str );
     }
@@ -519,7 +511,7 @@ void CGenbankFormatter::FormatVersion
     IFlatTextOStream& text_os = s_WrapOstreamIfCallbackExists(p_text_os, version, orig_text_os);
 
     list<string> l;
-    CNcbiOstrstream version_line;
+    ostringstream version_line;
 
     if ( version.GetAccession().empty() ) {
         l.push_back("VERSION");
@@ -531,7 +523,7 @@ void CGenbankFormatter::FormatVersion
                 version_line << "  GI:" << version.GetGi();
             }
         }
-        string version_line_str = CNcbiOstrstreamToString(version_line);
+        string version_line_str = version_line.str();
         if( version.GetContext()->Config().DoHTML() ) {
             TryToSanitizeHtml( version_line_str );
         }
@@ -560,7 +552,7 @@ void CGenbankFormatter::FormatGenomeProject(
 
     if ( ! gp.GetProjectNumbers().empty() ) {
 
-        CNcbiOstrstream project_line;
+        ostringstream project_line;
         project_line << "Project: ";
 
         const bool is_html = GetContext().GetConfig().DoHTML();
@@ -579,7 +571,7 @@ void CGenbankFormatter::FormatGenomeProject(
             }
         }
 
-        string project_line_str = CNcbiOstrstreamToString(project_line);
+        string project_line_str = project_line.str();
         if( gp.GetContext()->Config().DoHTML() ) {
             TryToSanitizeHtml( project_line_str );
         }
@@ -683,11 +675,11 @@ void CGenbankFormatter::FormatSegment
     IFlatTextOStream& text_os = s_WrapOstreamIfCallbackExists(p_text_os, seg, orig_text_os);
 
     list<string> l;
-    CNcbiOstrstream segment_line;
+    ostringstream segment_line;
 
     segment_line << seg.GetNum() << " of " << seg.GetCount();
 
-    Wrap(l, "SEGMENT", CNcbiOstrstreamToString(segment_line));
+    Wrap(l, "SEGMENT", segment_line.str());
     text_os.AddParagraph(l, seg.GetObject());
 
     text_os.Flush();
@@ -720,7 +712,7 @@ void CGenbankFormatter::x_FormatSourceLine
 (list<string>& l,
  const CSourceItem& source) const
 {
-    CNcbiOstrstream source_line;
+    ostringstream source_line;
 
     string prefix = source.IsUsingAnamorph() ? " (anamorph: " : " (";
 
@@ -728,7 +720,7 @@ void CGenbankFormatter::x_FormatSourceLine
     if ( !source.GetCommon().empty() ) {
         source_line << prefix << source.GetCommon() << ")";
     }
-    string line = CNcbiOstrstreamToString(source_line);
+    string line = source_line.str();
 
     if( source.GetContext()->Config().DoHTML() ) {
         TryToSanitizeHtml(line);
@@ -898,7 +890,7 @@ void CGenbankFormatter::x_Reference
  const CReferenceItem& ref,
  CBioseqContext& ctx) const
 {
-    CNcbiOstrstream ref_line;
+    ostringstream ref_line;
 
     int serial = ref.GetSerial();
     CPubdesc::TReftype reftype = ref.GetReftype();
@@ -922,7 +914,7 @@ void CGenbankFormatter::x_Reference
     } else {
         x_FormatRefLocation(ref_line, ref.GetLoc(), " to ", "; ", ctx);
     }
-    string ref_line_str = CNcbiOstrstreamToString(ref_line);
+    string ref_line_str = ref_line.str();
     if( ref.GetContext()->Config().DoHTML() ) {
         TryToSanitizeHtml( ref_line_str );
     }
@@ -1110,7 +1102,7 @@ CGenbankFormatter::x_LocusHtmlPrefix( string &first_line, CBioseqContext& ctx )
         return;
     }
 
-    CNcbiOstrstream result;
+    ostringstream result;
 
     // determine what sections we have.
 
@@ -1227,7 +1219,7 @@ CGenbankFormatter::x_LocusHtmlPrefix( string &first_line, CBioseqContext& ctx )
     result << "<pre class=\"genbank\">";
 
     result << first_line;
-    first_line = CNcbiOstrstreamToString(result);
+    first_line = result.str();
 }
 
 void
@@ -1243,7 +1235,7 @@ CGenbankFormatter::x_GetFeatureSpanAndScriptStart(
     const int feat_type_count = ( m_FeatureKeyToLocMap[strKey]++ );
 
     // The span
-    CNcbiOstrstream pre_feature_html;
+    ostringstream pre_feature_html;
     pre_feature_html << "<span id=\"feature_" << ctx.GetAccession()
         << "_" << strKey << "_" << feat_type_count << "\" class=\"feature\">";
 
@@ -1268,7 +1260,7 @@ CGenbankFormatter::x_GetFeatureSpanAndScriptStart(
     s_PrintLocAsJavascriptArray( ctx, pre_feature_html, feat_loc );
     pre_feature_html << ");</script>";
 
-    string temp = CNcbiOstrstreamToString(pre_feature_html);
+    string temp = pre_feature_html.str();
     text_os.AddLine(temp, nullptr, IFlatTextOStream::eAddNewline_No);
 }
 
@@ -1474,14 +1466,11 @@ bool s_GetLinkFeatureKey(
     {
         return false;
     }
-    if ( strRawKey == "source" ) {
-        const string& loc = feat.GetLoc().GetString();
-        if (NStr::StartsWith(loc, "1..") && NStr::Find(loc, ",") == NPOS) {
-            return false;
-        }
-        if (NStr::Find(loc, "order") != NPOS || NStr::Find(loc, "join") != NPOS) {
-            return false;
-        }
+
+    // ID-7962 : The first "source" item comes from a descriptor, not a feature, and
+    // hyperlink for it is not created.
+    if ( strRawKey == "source" && uItemNumber == 0) {
+        return false;
     }
 
     TGi iGi = ZERO_GI;
@@ -1616,6 +1605,7 @@ void CGenbankFormatter::FormatFeature
 {
     CRef<IFlatTextOStream> p_text_os;
     IFlatTextOStream* text_os = nullptr;
+
     {
         // this works differently from the others because we have to check
         // the underlying type
@@ -1639,9 +1629,12 @@ void CGenbankFormatter::FormatFeature
 
     CConstRef<CFlatFeature> feat = f.Format();
 
-    if ( feat->GetKey() != "source" ) {
-        ++ m_uFeatureCount;
-    }
+    // ID-7962 : Do not increment feature count for the source descriptor - this would tell the
+    // s_GetLinkFeatureKey function below to not create a hyperlink.
+    if ( feat->GetKey() != "source" || m_uFeatureCount > 0 || m_bSourceDescriptorDone)
+        ++m_uFeatureCount;
+    else
+        m_bSourceDescriptorDone = true;
 
     const string& strKey = feat->GetKey();
     string fkey = strKey;
@@ -1668,7 +1661,7 @@ void CGenbankFormatter::FormatFeature
     // on the already generated flat file view (which itself can be either a full sequence,
     // or a location on it).
     // In case 2, the absolute offsets included in the link are adjusted to the relative offsets
-    // by the Javascript responsible for howing the highlights.
+    // by the Javascript responsible for showing the highlights.
     string strFeatKey;
     if (s_GetLinkFeatureKey(f, *feat, fkey, strFeatKey, m_uFeatureCount))
     {
@@ -1708,7 +1701,7 @@ void CGenbankFormatter::FormatBasecount
 
     list<string> l;
 
-    CNcbiOstrstream bc_line;
+    ostringstream bc_line;
 
     bc_line.setf(IOS_BASE::right, IOS_BASE::adjustfield);
     bc_line
@@ -1719,7 +1712,7 @@ void CGenbankFormatter::FormatBasecount
     if ( bc.GetOther() > 0 ) {
         bc_line << setw(7) << bc.GetOther() << " others";
     }
-    Wrap(l, "BASE COUNT", CNcbiOstrstreamToString(bc_line));
+    Wrap(l, "BASE COUNT", bc_line.str());
     text_os.AddParagraph(l, bc.GetObject());
 
     text_os.Flush();
@@ -2026,11 +2019,11 @@ void CGenbankFormatter::FormatSequence
                 }
 
                 // build gap size text and "Expand Ns" link
-                CNcbiOstrstream gap_link;
+                ostringstream gap_link;
                 GetContext().GetConfig().GetHTMLFormatter().FormatGapLink(gap_link, gap_size,
                                                                           seq.GetContext()->GetAccession(),
                                                                           seq.GetContext()->IsProt());
-                text_os.AddLine( (string)CNcbiOstrstreamToString(gap_link) );
+                text_os.AddLine(gap_link.str());
             } else {
                 // create a fake total so we stop before the next gap
                 TSeqPos fake_total = distance_until_next_significant_gap;
