@@ -1,4 +1,4 @@
-/*  $Id: genome_project_item.cpp 615610 2020-09-01 20:52:14Z kans $
+/*  $Id: genome_project_item.cpp 692982 2025-02-04 14:14:33Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -80,12 +80,12 @@ const CGenomeProjectItem::TDBLinkLineVec & CGenomeProjectItem::GetDBLinkLines()
 /***************************************************************************/
 
 static string
-s_JoinLinkableStrs(const CUser_field_Base::C_Data::TStrs &strs, 
+s_JoinLinkableStrs(const CUser_field_Base::C_Data::TStrs &strs,
                    const string &url_prefix, const string &alt_prefix, const bool is_html )
 {
     const char * pchPrefix = "";
 
-    CNcbiOstrstream result;
+    ostringstream result;
     ITERATE( CUser_field_Base::C_Data::TStrs, str_iter, strs ) {
         const string &id = *str_iter;
         if( id.empty() ) {
@@ -107,8 +107,8 @@ s_JoinLinkableStrs(const CUser_field_Base::C_Data::TStrs &strs,
         }
         pchPrefix = ", ";
     }
-    
-    return CNcbiOstrstreamToString( result );
+
+    return result.str();
 }
 
 static string
@@ -116,7 +116,7 @@ s_JoinNumbers( const CUser_field_Base::C_Data::TInts & ints, const string & sepa
 {
     const char * pchPrefix = "";
 
-    CNcbiOstrstream result;
+    ostringstream result;
     ITERATE( CUser_field_Base::C_Data::TInts, int_iter, ints ) {
         if( *int_iter == 0 ) {
             continue;
@@ -125,7 +125,7 @@ s_JoinNumbers( const CUser_field_Base::C_Data::TInts & ints, const string & sepa
         result << *int_iter;
         pchPrefix = separator.c_str();
     }
-    return CNcbiOstrstreamToString( result );
+    return result.str();
 }
 
 namespace {
@@ -177,7 +177,7 @@ namespace {
                 // unknown prefix type
                 return kDefaultPrefixOrder;
             }
-            
+
             return find_iter->second;
         }
     };
@@ -193,8 +193,8 @@ void CGenomeProjectItem::x_GatherInfo(CBioseqContext& ctx)
 {
     const bool bHtml = ctx.Config().DoHTML();
 
-    const CUser_object *genome_projects_user_obje = NULL;
-    const CUser_object *dblink_user_obj = NULL;
+    const CUser_object* genome_projects_user_obje = nullptr;
+    const CUser_object* dblink_user_obj = nullptr;
 
     // extract all the useful user objects
     for (CSeqdesc_CI desc(ctx.GetHandle(), CSeqdesc::e_User);  desc;  ++desc) {
@@ -218,7 +218,7 @@ void CGenomeProjectItem::x_GatherInfo(CBioseqContext& ctx)
     }
 
     // process GenomeProjectsDB
-    if( genome_projects_user_obje != NULL ) {
+    if (genome_projects_user_obje) {
         ITERATE (CUser_object::TData, uf_it, genome_projects_user_obje->GetData()) {
             const CUser_field& field = **uf_it;
             if ( field.IsSetLabel()  &&  field.GetLabel().IsStr() ) {
@@ -246,7 +246,7 @@ void CGenomeProjectItem::x_GatherInfo(CBioseqContext& ctx)
     // ( we have these temporary vectors because we can't push straight to m_DBLinkLines
     //  because we have to sort them in case they're out of order in the ASN.1 )
     vector<string> dblinkLines;
-    if( dblink_user_obj != NULL ) {
+    if (dblink_user_obj) {
         ITERATE (CUser_object::TData, uf_it, dblink_user_obj->GetData()) {
             const CUser_field& field = **uf_it;
             if ( field.IsSetLabel()  &&  field.GetLabel().IsStr() && field.CanGetData() ) {
@@ -264,14 +264,14 @@ void CGenomeProjectItem::x_GatherInfo(CBioseqContext& ctx)
                 typedef CUser_field::C_Data TFieldData;
                 const TFieldData & field_data = field.GetData();
 
-                if( dbLinkLabelInfo.allow_text && 
+                if( dbLinkLabelInfo.allow_text &&
                     (field_data.IsStrs() || field_data.IsStr()) )
                 {
-                    const TFieldData::TStrs * pStrs = NULL;
+                    const TFieldData::TStrs* pStrs = nullptr;
 
-                    // auto_ptr just used to destroy the pStrs if it's
+                    // unique_ptr just used to destroy the pStrs if it's
                     // dynamically created.
-                    auto_ptr<TFieldData::TStrs> pStrsDestroyer;
+                    unique_ptr<TFieldData::TStrs> pStrsDestroyer;
 
                     if(  field_data.IsStrs() ) {
                         pStrs = & field_data.GetStrs();
@@ -286,23 +286,23 @@ void CGenomeProjectItem::x_GatherInfo(CBioseqContext& ctx)
                     if (NStr::Equal (label, "Sequence Read Archive")) {
                         alt_url = "https://www.ncbi.nlm.nih.gov/Traces/sra/sra.cgi?analysis=";
                     }
-                    string dblinkValue = s_JoinLinkableStrs( 
+                    string dblinkValue = s_JoinLinkableStrs(
                         *pStrs, dbLinkLabelInfo.url, alt_url, bHtml );
                     if( ! dblinkValue.empty() ) {
-                        dblinkLines.push_back( 
+                        dblinkLines.push_back(
                             pchNormalizedDbLinkLabel + string(": ") + dblinkValue );
                         if( bHtml ) {
                             TryToSanitizeHtml( dblinkLines.back() );
                         }
                     }
 
-                } else if( dbLinkLabelInfo.allow_numeric && 
+                } else if( dbLinkLabelInfo.allow_numeric &&
                     (field_data.IsInts() || field_data.IsInt()) )
                 {
 
-                    const TFieldData::TInts * pInts = NULL;
+                    const TFieldData::TInts* pInts = nullptr;
                     // destroys pInts if it's dynamically created
-                    auto_ptr<TFieldData::TInts> pIntsDestroyer;
+                    unique_ptr<TFieldData::TInts> pIntsDestroyer;
 
                     if( field_data.IsInts() ) {
                         pInts = & field_data.GetInts();
@@ -314,8 +314,8 @@ void CGenomeProjectItem::x_GatherInfo(CBioseqContext& ctx)
 
                     string dblinkValue = s_JoinNumbers( *pInts, ", " );
                     if( ! dblinkValue.empty() ) {
-                        dblinkLines.push_back( 
-                            pchNormalizedDbLinkLabel + string(": ") + 
+                        dblinkLines.push_back(
+                            pchNormalizedDbLinkLabel + string(": ") +
                             dblinkValue );
                         // No need to sanitize; it's just numbers, commas, and spaces
                     }
