@@ -1,4 +1,4 @@
-/*  $Id: locus_item.cpp 665431 2023-04-04 17:43:43Z stakhovv $
+/*  $Id: locus_item.cpp 692982 2025-02-04 14:14:33Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -165,7 +165,7 @@ static void s_AddLocusSuffix(string &basename, CBioseqContext& ctx)
     }
 
     // Add the suffix
-    CNcbiOstrstream locus;
+    ostringstream locus;
     locus << basename;
 
     size_t suffix_len = NStr::NumericToString(numsegs).length();
@@ -174,7 +174,7 @@ static void s_AddLocusSuffix(string &basename, CBioseqContext& ctx)
         locus << 'S';
     }
     locus << setfill('0') << setw(suffix_len) << ctx.GetPartNumber();
-    basename = CNcbiOstrstreamToString(locus);
+    basename = locus.str();
 }
 
 static string s_FastaGetOriginalID (CBioseqContext& ctx)
