@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_IMPL_HPP
 #define OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_IMPL_HPP
 
-/*  $Id: psg_client_impl.hpp 675129 2023-11-15 13:50:52Z ivanov $
+/*  $Id: psg_client_impl.hpp 689293 2024-10-23 20:12:39Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -69,7 +69,7 @@ struct SPSG_RStream : private SPSG_BlobReader, public CRStream
 {
     template <class... TArgs>
     SPSG_RStream(TArgs&&... args) :
-        SPSG_BlobReader(forward<TArgs>(args)...),
+        SPSG_BlobReader(std::forward<TArgs>(args)...),
         CRStream(this, size(), data())
     {}
 };
@@ -98,15 +98,18 @@ private:
 struct SPSG_UserArgsBuilder
 {
     SPSG_UserArgsBuilder() { x_UpdateCache(); }
-    void SetQueueArgs(SPSG_UserArgs queue_args) { m_QueueArgs = move(queue_args); x_UpdateCache(); }
+    void SetQueueArgs(SPSG_UserArgs queue_args) { m_QueueArgs = std::move(queue_args); x_UpdateCache(); }
     void Build(ostream& os, const SPSG_UserArgs& request_args);
+    void BuildRaw(ostringstream& os, const SPSG_UserArgs& request_args);
 
 private:
+    void x_MergeOthers(SPSG_UserArgs& combined_args);
     void x_UpdateCache();
 
     struct MergeValues; // A function-like class
     static bool Merge(SPSG_UserArgs& higher_priority, const SPSG_UserArgs& lower_priority);
 
+    static SPSG_UserArgs s_ReadIniArgs();
     static const SPSG_UserArgs& s_GetIniArgs();
     
     SPSG_UserArgs m_QueueArgs;
@@ -125,7 +128,7 @@ struct CPSG_Queue::SImpl
 
     bool RejectsRequests() const { return m_Service.ioc.RejectsRequests(); }
     void SetRequestFlags(CPSG_Request::TFlags request_flags) { m_RequestFlags = request_flags; }
-    void SetUserArgs(SPSG_UserArgs user_args) { m_UserArgsBuilder.GetLock()->SetQueueArgs(move(user_args)); }
+    void SetUserArgs(SPSG_UserArgs user_args) { m_UserArgsBuilder.GetLock()->SetQueueArgs(std::move(user_args)); }
 
     static TApiLock GetApiLock() { return CService::GetMap(); }
 
@@ -148,7 +151,7 @@ private:
         static shared_ptr<TMap> GetMap();
     };
 
-    string x_GetAbsPathRef(shared_ptr<const CPSG_Request> user_request);
+    string x_GetAbsPathRef(shared_ptr<const CPSG_Request> user_request, const CPSG_Request::TFlags& flags, bool raw);
 
     CService m_Service;
     CPSG_Request::TFlags m_RequestFlags = CPSG_Request::eDefaultFlags;
