@@ -1,4 +1,4 @@
-/*  $Id: psg_client_transport.cpp 684281 2024-06-18 13:28:36Z ivanov $
+/*  $Id: psg_client_transport.cpp 697646 2025-05-19 15:54:29Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,7 +51,6 @@
 #define __STDC_FORMAT_MACROS
 
 #include <corelib/request_status.hpp>
-#include <corelib/ncbi_cookies.hpp>
 
 #include "psg_client_transport.hpp"
 
@@ -61,9 +60,9 @@ PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, rd_buf_size,                   64 *
 PSG_PARAM_VALUE_DEF_MIN(size_t,         PSG, wr_buf_size,                   64 * 1024,          1024    );
 PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, max_concurrent_streams,        100,                10      );
 PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, max_concurrent_submits,        150,                1       );
-PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, max_sessions,                  40,                 1       );
+PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, max_sessions,                  1,                  1       );
 PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, max_concurrent_requests_per_server, 500,           100     );
-PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, num_io,                        6,                  1       );
+PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, num_io,                        1,                  1       );
 PSG_PARAM_VALUE_DEF_MIN(unsigned,       PSG, reader_timeout,                12,                 1       );
 PSG_PARAM_VALUE_DEF_MIN(double,         PSG, rebalance_time,                10.0,               1.0     );
 PSG_PARAM_VALUE_DEF_MIN(size_t,         PSG, requests_per_io,               1,                  1       );
@@ -73,6 +72,7 @@ NCBI_PARAM_DEF(double,   PSG, competitive_after,      0.0);
 NCBI_PARAM_DEF(unsigned, PSG, request_retries,        2);
 NCBI_PARAM_DEF(unsigned, PSG, refused_stream_retries, 2);
 NCBI_PARAM_DEF(string,   PSG, request_user_args,      "");
+NCBI_PARAM_DEF(string,   PSG, multivalued_user_args,  "");
 NCBI_PARAM_DEF(bool,     PSG, user_request_ids,       false);
 NCBI_PARAM_DEF(unsigned, PSG, localhost_preference,   1);
 NCBI_PARAM_DEF(bool,     PSG, fail_on_unknown_items,  false);
@@ -84,6 +84,8 @@ NCBI_PARAM_DEF(double,   PSG, stats_period,           0.0);
 NCBI_PARAM_DEF_EX(string,   PSG, service,               "PSG2",             eParam_Default,     NCBI_PSG_SERVICE);
 NCBI_PARAM_DEF_EX(string,   PSG, auth_token_name,       "WebCubbyUser",     eParam_Default,     NCBI_PSG_AUTH_TOKEN_NAME);
 NCBI_PARAM_DEF_EX(string,   PSG, auth_token,            "",                 eParam_Default,     NCBI_PSG_AUTH_TOKEN);
+NCBI_PARAM_DEF_EX(string,   PSG, admin_auth_token_name, "AdminAuthToken",   eParam_Default,     NCBI_PSG_ADMIN_AUTH_TOKEN_NAME);
+NCBI_PARAM_DEF_EX(string,   PSG, admin_auth_token,      "",                 eParam_Default,     NCBI_PSG_ADMIN_AUTH_TOKEN);
 
 NCBI_PARAM_DEF(double,   PSG, throttle_relaxation_period,                  0.0);
 NCBI_PARAM_DEF(unsigned, PSG, throttle_by_consecutive_connection_failures, 0);
@@ -139,6 +141,32 @@ SPSG_ArgsBase::SArg<SPSG_ArgsBase::eChunkType>::TType SPSG_ArgsBase::SArg<SPSG_A
     return { SPSG_ArgsBase::eUnknownChunk, cref(value) };
 };
 
+string SPSG_Env::GetCookie(const string& name) const
+{
+    if (!m_Cookies) {
+        m_Cookies.emplace();
+        m_Cookies->Add(CHttpCookies::eHeader_Cookie, Get("HTTP_COOKIE"), nullptr);
+    }
+
+    for (const auto& cookie : *m_Cookies) {
+        if (cookie.GetName() == name) {
+            return NStr::URLDecode(cookie.GetValue());
+        }
+    }
+
+    return {};
+}
+
+string SPSG_Params::GetCookie(function<string()> get_auth_token)
+{
+    auto combine = [](auto p, auto v) { return v.empty()? v : p.Get() + '=' + NStr::URLEncode(v); };
+
+    auto admin_cookie = combine(admin_auth_token_name, admin_auth_token.Get());
+    auto hup_cookie = combine(auth_token_name, auth_token.Get().empty() ? get_auth_token() : auth_token.Get());
+
+    return admin_cookie.empty() ? hup_cookie : hup_cookie.empty() ? admin_cookie : admin_cookie + "; " + hup_cookie;
+}
+
 unsigned SPSG_Params::s_GetRequestTimeout(double io_timer_period)
 {
     auto value = TPSG_RequestTimeout::GetDefault();
@@ -173,30 +201,13 @@ unsigned SPSG_Params::s_GetCompetitiveAfter(double io_timer_period, double timeo
     return static_cast<unsigned>(timeout / io_timer_period);
 }
 
-string SPSG_Params::GetAuthToken()
-{
-    string rv = TPSG_AuthToken(TPSG_AuthToken::eGetDefault);
-
-    if (rv.empty()) {
-        CHttpCookies cookies;
-        cookies.Add(CHttpCookies::eHeader_Cookie, CNcbiEnvironment().Get("HTTP_COOKIE"), nullptr);
-
-        for (const auto& cookie : cookies) {
-            if (cookie.GetName() == auth_token_name.Get()) {
-                return NStr::URLDecode(cookie.GetValue());
-            }
-        }
-    }
-
-    return rv;
-}
-
 void SDebugPrintout::Print(SSocketAddress address, const string& path, const string& sid, const string& phid, const string& ip, SUv_Tcp::TPort port)
 {
     ostringstream os;
 
     if (!ip.empty()) os << ";IP=" << ip;
     if (port)        os << ";PORT=" << port;
+    if (m_Params.proxy) os << ";PROXY=" << m_Params.proxy;
 
     ERR_POST(Message << id << ": " << address << path << ";SID=" << sid << ";PHID=" << phid << os.str());
 }
@@ -482,19 +493,19 @@ void SPSG_StatsCounters::Apply(EGroup start_with, TArgs&&... args)
     // This method is always called with start_with == eRequest (so, all cases are run with one call, one by one).
     // This switch usage however makes compilers warn if any enum value is missing/not handled
     switch (start_with) {
-        case eRequest:          TWhat::template Func<eRequest>          (forward<TArgs>(args)...);
-        case eReplyItem:        TWhat::template Func<eReplyItem>        (forward<TArgs>(args)...);
-        case eSkippedBlob:      TWhat::template Func<eSkippedBlob>      (forward<TArgs>(args)...);
-        case eReplyItemStatus:  TWhat::template Func<eReplyItemStatus>  (forward<TArgs>(args)...);
-        case eMessage:          TWhat::template Func<eMessage>          (forward<TArgs>(args)...);
-        case eRetries:          TWhat::template Func<eRetries>          (forward<TArgs>(args)...);
+        case eRequest:          TWhat::template Func<eRequest>          (std::forward<TArgs>(args)...);
+        case eReplyItem:        TWhat::template Func<eReplyItem>        (std::forward<TArgs>(args)...);
+        case eSkippedBlob:      TWhat::template Func<eSkippedBlob>      (std::forward<TArgs>(args)...);
+        case eReplyItemStatus:  TWhat::template Func<eReplyItemStatus>  (std::forward<TArgs>(args)...);
+        case eMessage:          TWhat::template Func<eMessage>          (std::forward<TArgs>(args)...);
+        case eRetries:          TWhat::template Func<eRetries>          (std::forward<TArgs>(args)...);
     }
 }
 
 template <class... TArgs>
 void SPSG_StatsCounters::Report(TArgs&&... args)
 {
-    Apply<SReport>(eRequest, m_Data, forward<TArgs>(args)...);
+    Apply<SReport>(eRequest, m_Data, std::forward<TArgs>(args)...);
 }
 
 SPSG_StatsAvgTime::SPSG_StatsAvgTime() :
@@ -602,13 +613,15 @@ void SPSG_Stats::Report()
     }
 }
 
-string SPSG_Reply::SState::GetError()
+SPSG_Message SPSG_Reply::SState::GetMessage(EDiagSev min_severity)
 {
-    if (m_Messages.empty()) return {};
+    for (auto& m : m_Messages) {
+        if (m && ((min_severity == eDiag_Trace) || ((m.severity != eDiag_Trace) && (m.severity >= min_severity)))) {
+            return exchange(m, SPSG_Message{});
+        }
+    }
 
-    auto rv = m_Messages.back();
-    m_Messages.pop_back();
-    return rv;
+    return {};
 }
 
 void SPSG_Reply::SState::Reset()
@@ -618,7 +631,17 @@ void SPSG_Reply::SState::Reset()
     m_Messages.clear();
 }
 
-EPSG_Status SPSG_Reply::SState::FromRequestStatus(int status)
+int SPSG_Reply::SState::SStatus::From(EPSG_Status status)
+{
+    switch (status) {
+        case EPSG_Status::eSuccess:         return CRequestStatus::e200_Ok;
+        case EPSG_Status::eForbidden:       return CRequestStatus::e403_Forbidden;
+        case EPSG_Status::eNotFound:        return CRequestStatus::e404_NotFound;
+        default:                            return CRequestStatus::e400_BadRequest;
+    }
+}
+
+EPSG_Status SPSG_Reply::SState::SStatus::From(int status)
 {
     switch (status) {
         case CRequestStatus::e200_Ok:               return EPSG_Status::eSuccess;
@@ -669,7 +692,7 @@ void SPSG_Reply::SetComplete()
     queue->NotifyOne();
 }
 
-void SPSG_Reply::SetFailed(string message, EPSG_Status status)
+void SPSG_Reply::SetFailed(string message, SState::SStatus status)
 {
     if (auto items_locked = items.GetLock()) {
         for (auto& item : *items_locked) {
@@ -731,9 +754,8 @@ shared_ptr<void> SPSG_Request::SContext::Set()
     return guard;
 }
 
-SPSG_Request::SPSG_Request(string p, CPSG_Request::TFlags f, shared_ptr<SPSG_Reply> r, CRef<CRequestContext> c, const SPSG_Params& params) :
-    full_path(move(p)),
-    flags(f),
+SPSG_Request::SPSG_Request(string p, shared_ptr<SPSG_Reply> r, CRef<CRequestContext> c, const SPSG_Params& params) :
+    full_path(std::move(p)),
     reply(r),
     context(c),
     m_State(&SPSG_Request::StatePrefix),
@@ -773,6 +795,12 @@ SPSG_Request::EStateResult SPSG_Request::StatePrefix(const char*& data, size_t&
         if (!len) return eContinue;
     }
 
+    if (reply->raw && !index) {
+        m_State = &SPSG_Request::StateData;
+        m_Buffer.data_to_read = numeric_limits<size_t>::max();
+        return eContinue;
+    }
+
     // Check failed
     const auto message = "Protocol error: prefix mismatch";
 
@@ -800,7 +828,7 @@ SPSG_Request::EStateResult SPSG_Request::StateArgs(const char*& data, size_t& le
     const auto& size_str = args.GetValue("size");
     const auto size = size_str.empty() ? 0ul : stoul(size_str);
 
-    m_Buffer.args = move(args);
+    m_Buffer.args = std::move(args);
 
     if (size) {
         m_State = &SPSG_Request::StateData;
@@ -849,6 +877,11 @@ EDiagSev s_GetSeverity(const string& severity)
     return eDiag_Error;
 }
 
+auto s_GetCode(const string& code)
+{
+    return code.empty() ? optional<int>{} : atoi(code.c_str());
+}
+
 SPSG_Request::EStateResult SPSG_Request::Add()
 {
     auto context_guard = context.Set();
@@ -901,7 +934,7 @@ SPSG_Request::EStateResult SPSG_Request::Add()
             }
 
             if (to_create) {
-                item_locked->args = move(args);
+                item_locked->args = std::move(args);
             }
 
             if (update_result == eNewItem) {
@@ -950,7 +983,7 @@ SPSG_Request::EUpdateResult SPSG_Request::UpdateItem(SPSG_Args::EItemType item_t
         if (const auto status = get_status(); can_retry_503(status, "Server returned a meta with status 503")) {
             return eRetry503;
         } else if (status) {
-            item.state.SetStatus(SPSG_Reply::SState::FromRequestStatus(status), true);
+            item.state.SetStatus(status, true);
         }
 
         if ((item_type != SPSG_Args::eBlob) || item.chunks.empty()) {
@@ -971,17 +1004,16 @@ SPSG_Request::EUpdateResult SPSG_Request::UpdateItem(SPSG_Args::EItemType item_t
 
     if (chunk_type.first & SPSG_Args::eMessage) {
         auto severity = s_GetSeverity(args.GetValue("severity"));
+        auto code = s_GetCode(args.GetValue("code"));
 
-        if (severity == eDiag_Warning) {
-            ERR_POST(Warning << chunk);
-        } else if (severity == eDiag_Info) {
-            ERR_POST(Info << chunk);
+        if (severity <= eDiag_Warning) {
+            item.state.AddMessage(std::move(chunk), severity, code);
         } else if (severity == eDiag_Trace) {
-            ERR_POST(Trace << chunk);
+            _DEBUG_CODE(item.state.AddMessage(std::move(chunk), severity, code););
         } else if (const auto status = get_status(); can_retry_503(status, chunk.c_str())) {
             return eRetry503;
         } else {
-            item.state.AddError(move(chunk), SPSG_Reply::SState::FromRequestStatus(status));
+            item.state.AddError(std::move(chunk), status, severity, code);
         }
 
         if (auto stats = reply->stats.lock()) stats->IncCounter(SPSG_Stats::eMessage, severity);
@@ -1003,7 +1035,7 @@ SPSG_Request::EUpdateResult SPSG_Request::UpdateItem(SPSG_Args::EItemType item_t
 
         if (item.chunks.size() <= index) item.chunks.resize(index + 1);
 
-        item.chunks[index] = move(chunk);
+        item.chunks[index] = std::move(chunk);
     }
 
     const bool is_not_reply = item_type != SPSG_Args::eReply;
@@ -1025,6 +1057,16 @@ SPSG_Request::EUpdateResult SPSG_Request::UpdateItem(SPSG_Args::EItemType item_t
 }
 
 
+struct SPSG_NcbiPeerID
+{
+    static const string& Get()
+    {
+        static const string ncbi_peer_id(GetDiagContext().GetStringUID());
+        return ncbi_peer_id;
+    }
+};
+
+
 #define HTTP_STATUS_HEADER ":status"
 
 
@@ -1034,12 +1076,12 @@ template <class... TNgHttp2Cbs>
 SPSG_IoSession::SPSG_IoSession(SPSG_Server& s, const SPSG_Params& params, SPSG_AsyncQueue& queue, uv_loop_t* loop, TNgHttp2Cbs&&... callbacks) :
     SUvNgHttp2_SessionBase(
             loop,
-            TAddrNCred(s.address, SUvNgHttp2_Tls::TCred()),
+            TAddrNCred{{s.address, SUvNgHttp2_Tls::TCred()}, params.proxy},
             TPSG_RdBufSize::GetDefault(),
             TPSG_WrBufSize::GetDefault(),
             TPSG_Https::GetDefault(),
             TPSG_MaxConcurrentStreams::GetDefault(),
-            forward<TNgHttp2Cbs>(callbacks)...),
+            std::forward<TNgHttp2Cbs>(callbacks)...),
     server(s),
     m_Params(params),
     m_Headers{{
@@ -1048,6 +1090,7 @@ SPSG_IoSession::SPSG_IoSession(SPSG_Server& s, const SPSG_Params& params, SPSG_A
         { ":authority", m_Authority },
         { ":path" },
         { "user-agent", SUvNgHttp2_UserAgent::Get() },
+        { "ncbi-peer-id", SPSG_NcbiPeerID::Get() },
         { "http_ncbi_sid" },
         { "http_ncbi_phid" },
         { "cookie" },
@@ -1113,6 +1156,16 @@ bool SPSG_Request::Fail(SPSG_Processor::TId processor_id, const SUvNgHttp2_Error
     return true;
 }
 
+void SPSG_Request::ConvertRaw()
+{
+    if (m_Buffer.args_buffer.empty() && !m_Buffer.chunk.empty()) {
+        m_Buffer.args = "item_id=1&item_type=unknown&chunk_type=data_and_meta&n_chunks=1"s;
+        Add();
+        m_Buffer.args = "item_id=0&item_type=reply&chunk_type=meta&n_chunks=2"s;
+        Add();
+    }
+}
+
 bool SPSG_IoSession::Fail(SPSG_Processor::TId processor_id, shared_ptr<SPSG_Request> req, const SUvNgHttp2_Error& error, bool refused_stream)
 {
     auto context_guard = req->context.Set();
@@ -1146,6 +1199,10 @@ int SPSG_IoSession::OnStreamClose(nghttp2_session*, int32_t stream_id, uint32_t
                     ERR_POST("Request for " << GetId() << " failed with " << error);
                 }
             } else {
+                if (req->reply->raw) {
+                    req->ConvertRaw();
+                }
+
                 req->OnReplyDone(processor_id)->SetComplete();
                 server.throttling.AddSuccess();
                 PSG_THROTTLING_TRACE("Server '" << GetId() << "' processed request '" <<
@@ -1172,7 +1229,7 @@ int SPSG_IoSession::OnHeader(nghttp2_session*, const nghttp2_frame* frame, const
 
         if (auto it = m_Requests.find(stream_id); it != m_Requests.end()) {
             const auto request_status = static_cast<CRequestStatus::ECode>(atoi(status_str));
-            const auto status = SPSG_Reply::SState::FromRequestStatus(request_status);
+            const auto status = SPSG_Reply::SState::SStatus(request_status);
 
             if (status != EPSG_Status::eSuccess) {
                 if (auto [processor_id, req] = it->second.Get(); req) {
@@ -1198,7 +1255,7 @@ bool SPSG_IoSession::ProcessRequest(SPSG_TimedRequest timed_req, SPSG_Processor:
     const auto& path = req->full_path;
     const auto& session_id = context.GetSessionID();
     const auto& sub_hit_id = context.GetNextSubHitID();
-    const auto& cookie = m_Params.GetCookie(req->flags, [&]() { return context.GetProperty("auth_token"); });
+    const auto& cookie = m_Params.GetCookie([&]() { return context.GetProperty("auth_token"); });
     const auto& client_ip = context.GetClientIP();
     auto headers_size = m_Headers.size();
 
@@ -1220,7 +1277,7 @@ bool SPSG_IoSession::ProcessRequest(SPSG_TimedRequest timed_req, SPSG_Processor:
 
         // Do not reset all requests unless throttling has been activated
         if (RetryFail(processor_id, req, error) && server.throttling.Active()) {
-            Reset(move(error));
+            Reset(std::move(error));
         }
 
         return false;
@@ -1229,7 +1286,7 @@ bool SPSG_IoSession::ProcessRequest(SPSG_TimedRequest timed_req, SPSG_Processor:
     req->submitted_by.Set(GetInternalId());
     req->reply->debug_printout << server.address << path << session_id << sub_hit_id << client_ip << m_Tcp.GetLocalPort() << endl;
     PSG_IO_SESSION_TRACE(this << '/' << stream_id << " submitted");
-    m_Requests.emplace(stream_id, move(timed_req));
+    m_Requests.emplace(stream_id, std::move(timed_req));
     return Send();
 }
 
@@ -1335,7 +1392,7 @@ SPSG_ThrottleParams::SPSG_ThrottleParams() :
 
 SPSG_Throttling::SPSG_Throttling(const SSocketAddress& address, SPSG_ThrottleParams p, uv_loop_t* l) :
     m_Address(address),
-    m_Stats(move(p)),
+    m_Stats(std::move(p)),
     m_Active(eOff),
     m_Timer(this, s_OnTimer, Configured(), 0)
 {
@@ -1555,7 +1612,7 @@ void SPSG_IoImpl::OnQueue(uv_async_t* handle)
             next_server();
 
         // If failed to submit
-        } else if (!session->ProcessRequest(*move(timed_req), processor_id, move(req))) {
+        } else if (!session->ProcessRequest(*std::move(timed_req), processor_id, std::move(req))) {
             PSG_IO_TRACE("Server '" << session->GetId() << "' failed to get request '" << req_id << "' with rate = " << target_rate);
             next_server();
 
@@ -1567,7 +1624,10 @@ void SPSG_IoImpl::OnQueue(uv_async_t* handle)
 
             // Add new session if needed and allowed to
             if (session->IsFull() && (distance(session, server.sessions.end()) == 1)) {
-                if (server.sessions.size() >= TPSG_MaxSessions::GetDefault()) {
+                const auto single_server_single_session = m_Sessions.size() == 1 && TPSG_MaxSessions::GetDefault() == 1;
+                const auto max_sessions = single_server_single_session ? 2 : TPSG_MaxSessions::GetDefault();
+
+                if (server.sessions.size() >= max_sessions) {
                     PSG_IO_TRACE("Server '" << server->address << "' reached session limit");
                     ignore_server();
                 } else {
@@ -1580,7 +1640,7 @@ void SPSG_IoImpl::OnQueue(uv_async_t* handle)
 
     if (req) {
         // Do not need to signal here
-        m_Queue.Emplace(*move(timed_req));
+        m_Queue.Emplace(*std::move(timed_req));
     }
 
     if (!remaining_submits) {
@@ -1893,7 +1953,7 @@ bool SPSG_IoCoordinator::AddRequest(shared_ptr<SPSG_Request> req, const atomic_b
     }
 
     const auto idx = (m_RequestCounter++ / params.requests_per_io) % m_Io.size();
-    m_Queues[idx].Emplace(move(req));
+    m_Queues[idx].Emplace(std::move(req));
     m_Queues[idx].Signal();
     return true;
 }
