@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_TRANSPORT__HPP
 #define OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_TRANSPORT__HPP
 
-/*  $Id: psg_client_transport.hpp 676359 2023-12-14 18:25:59Z ivanov $
+/*  $Id: psg_client_transport.hpp 697646 2025-05-19 15:54:29Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -64,6 +64,7 @@
 #include <connect/services/netservice_api.hpp>
 #include <corelib/ncbi_param.hpp>
 #include <corelib/ncbi_url.hpp>
+#include <corelib/ncbi_cookies.hpp>
 
 
 BEGIN_NCBI_SCOPE
@@ -198,6 +199,14 @@ struct SPSG_Chunk : string
     SPSG_Chunk& operator=(const SPSG_Chunk&) = delete;
 };
 
+struct SPSG_Env : CNcbiEnvironment
+{
+    string GetCookie(const string& name) const;
+
+private:
+    mutable optional<CHttpCookies> m_Cookies;
+};
+
 struct SPSG_Params
 {
     TPSG_DebugPrintout debug_printout;
@@ -211,10 +220,13 @@ struct SPSG_Params
     TPSG_RefusedStreamRetries refused_stream_retries;
     TPSG_UserRequestIds user_request_ids;
     TPSG_AuthTokenName auth_token_name;
-    string auth_token;
+    TPSG_AuthToken auth_token;
+    TPSG_AdminAuthTokenName admin_auth_token_name;
+    TPSG_AdminAuthToken admin_auth_token;
+    SSocketAddress proxy;
     TPSG_PsgClientMode client_mode;
 
-    SPSG_Params() :
+    SPSG_Params(SPSG_Env env = {}) :
         debug_printout(TPSG_DebugPrintout::eGetDefault),
         max_concurrent_submits(TPSG_MaxConcurrentSubmits::eGetDefault),
         max_concurrent_requests_per_server(TPSG_MaxConcurrentRequestsPerServer::eGetDefault),
@@ -226,23 +238,18 @@ struct SPSG_Params
         refused_stream_retries(TPSG_RefusedStreamRetries::eGetDefault),
         user_request_ids(TPSG_UserRequestIds::eGetDefault),
         auth_token_name(TPSG_AuthTokenName::eGetDefault),
-        auth_token(GetAuthToken()),
+        auth_token([&](string v) { return v.empty() ? env.GetCookie(auth_token_name) : v; }),
+        admin_auth_token_name(TPSG_AdminAuthTokenName::eGetDefault),
+        admin_auth_token([&](string v) { return v.empty() ? env.GetCookie(admin_auth_token_name) : v; }),
+        proxy(SSocketAddress::Parse(env.Get("HTTP_PROXY"), SSocketAddress::SHost::EName::eOriginal)),
         client_mode(TPSG_PsgClientMode::eGetDefault)
     {}
 
-    template <class TGet>
-    string GetCookie(const CPSG_Request::TFlags& request_flags, TGet get)
-    {
-        if (auto include_hup = request_flags & CPSG_Request::fIncludeHUP; !include_hup) return {};
-        auto rv = auth_token.empty() ? get() : auth_token;
-        return rv.empty() ? rv : auth_token_name.Get() + '=' + NStr::URLEncode(rv);
-    }
+    string GetCookie(function<string()> get_auth_token);
 
 private:
     static unsigned s_GetRequestTimeout(double io_timer_period);
     static unsigned s_GetCompetitiveAfter(double io_timer_period, double timeout);
-
-    string GetAuthToken();
 };
 
 struct SDebugPrintout
@@ -250,7 +257,7 @@ struct SDebugPrintout
     const string id;
 
     SDebugPrintout(string i, const SPSG_Params& params) :
-        id(move(i)),
+        id(std::move(i)),
         m_Params(params)
     {
         if (IsPerf()) m_Events.reserve(20);
@@ -268,11 +275,11 @@ private:
     template <class ...TArgs>
     void Process(TArgs&&... args)
     {
-        if (IsPerf()) return Event(forward<TArgs>(args)...);
+        if (IsPerf()) return Event(std::forward<TArgs>(args)...);
 
         if (m_Params.debug_printout == EPSG_DebugPrintout::eNone) return;
 
-        Print(forward<TArgs>(args)...);
+        Print(std::forward<TArgs>(args)...);
     }
 
     enum EType { eSend = 1000, eReceive, eClose, eRetry, eFail };
@@ -310,14 +317,14 @@ template <class TArg, class ...TRest>
 struct SDebugPrintout::SPack : SPack<TRest...>
 {
     SPack(SPack<TRest...>&& base, const TArg* arg) :
-        SPack<TRest...>(move(base)),
+        SPack<TRest...>(std::move(base)),
         m_Arg(arg)
     {}
 
     template <class TNextArg>
     SPack<TNextArg, TArg, TRest...> operator<<(const TNextArg& next_arg)
     {
-        return { move(*this), &next_arg };
+        return { std::move(*this), &next_arg };
     }
 
     void operator<<(ostream& (*)(ostream&)) { Process(); }
@@ -326,7 +333,7 @@ protected:
     template <class ...TArgs>
     void Process(TArgs&&... args)
     {
-        SPack<TRest...>::Process(*m_Arg, forward<TArgs>(args)...);
+        SPack<TRest...>::Process(*m_Arg, std::forward<TArgs>(args)...);
     }
 
 private:
@@ -344,7 +351,7 @@ struct SDebugPrintout::SPack<TArg>
     template <class TNextArg>
     SPack<TNextArg, TArg> operator<<(const TNextArg& next_arg)
     {
-        return { move(*this), &next_arg };
+        return { std::move(*this), &next_arg };
     }
 
     void operator<<(ostream& (*)(ostream&)) { Process(); }
@@ -353,7 +360,7 @@ protected:
     template <class ...TArgs>
     void Process(TArgs&&... args)
     {
-        m_DebugPrintout->Process(*m_Arg, forward<TArgs>(args)...);
+        m_DebugPrintout->Process(*m_Arg, std::forward<TArgs>(args)...);
     }
 
 private:
@@ -369,35 +376,57 @@ struct SPSG_Reply
 {
     struct SState
     {
+        struct SStatus
+        {
+            template <typename T, enable_if_t<is_convertible_v<T, int>, int> = 0>
+            SStatus(T h) : m_Http(static_cast<int>(h)), m_Psg(From(m_Http)) {}
+            SStatus(EPSG_Status p) : m_Http(From(p)), m_Psg(p) {}
+
+            int GetHttpCode() const { return m_Http; }
+            operator EPSG_Status() const { return m_Psg; }
+
+            auto CanBeChangedTo(const SStatus& other) const { return m_Psg < other.m_Psg; }
+
+        private:
+            static int From(EPSG_Status status);
+            static EPSG_Status From(int status);
+
+            int m_Http;
+            EPSG_Status m_Psg;
+        };
+
         SPSG_CV<> change;
 
         SState() : m_InProgress(true), m_Status(EPSG_Status::eSuccess) {}
 
-        EPSG_Status GetStatus() const volatile { return m_Status; }
-        string GetError();
+        SStatus GetStatus() const volatile { return m_Status; }
+        SPSG_Message GetMessage(EDiagSev min_severity);
         const volatile atomic_bool& InProgress() const volatile { return m_InProgress; }
 
-        void SetStatus(EPSG_Status status, bool reset) volatile
+        void SetStatus(SStatus status, bool reset) volatile
         {
-            EPSG_Status expected = m_Status;
-            while (((expected < status) || reset) && !m_Status.compare_exchange_weak(expected, status));
+            SStatus expected = m_Status;
+            while (((expected.CanBeChangedTo(status)) || reset) && !m_Status.compare_exchange_weak(expected, status));
         }
 
-        void AddError(string message, EPSG_Status status = EPSG_Status::eError)
+        void AddError(string message, SStatus status = EPSG_Status::eError, EDiagSev severity = eDiag_Error, optional<int> code = nullopt)
         {
-            m_Messages.emplace_front(move(message));
+            AddMessage(std::move(message), severity, code);
             SetStatus(status, false);
         }
 
+        void AddMessage(string message, EDiagSev severity, optional<int> code)
+        {
+            m_Messages.emplace_back(SPSG_Message{std::move(message), severity, code});
+        }
+
         void SetComplete() volatile { if (m_InProgress.exchange(false)) change.NotifyOne(); }
         void Reset();
 
-        static EPSG_Status FromRequestStatus(int status);
-
     private:
         atomic_bool m_InProgress;
-        atomic<EPSG_Status> m_Status;
-        deque<string> m_Messages;
+        atomic<SStatus> m_Status;
+        deque<SPSG_Message> m_Messages;
     };
 
     struct SItem
@@ -413,6 +442,7 @@ struct SPSG_Reply
         void Reset();
     };
 
+    const bool raw;
     SThreadSafe<list<SItem::TTS>> items;
     SThreadSafe<list<SItem::TTS*>> new_items;
     SItem::TTS reply_item;
@@ -420,14 +450,15 @@ struct SPSG_Reply
     shared_ptr<TPSG_Queue> queue;
     weak_ptr<SPSG_Stats> stats;
 
-    SPSG_Reply(string id, const SPSG_Params& params, shared_ptr<TPSG_Queue> q, weak_ptr<SPSG_Stats> s = weak_ptr<SPSG_Stats>()) :
-        debug_printout(move(id), params),
-        queue(move(q)),
-        stats(move(s))
+    SPSG_Reply(string id, const SPSG_Params& params, shared_ptr<TPSG_Queue> q, weak_ptr<SPSG_Stats> s = weak_ptr<SPSG_Stats>(), bool r = false) :
+        raw(r),
+        debug_printout(std::move(id), params),
+        queue(std::move(q)),
+        stats(std::move(s))
     {}
 
     void SetComplete();
-    void SetFailed(string message, EPSG_Status status = EPSG_Status::eError);
+    void SetFailed(string message, SState::SStatus status = EPSG_Status::eError);
     optional<SItem::TTS*> GetNextItem(CDeadline deadline);
     void Reset();
 };
@@ -496,13 +527,12 @@ struct SPSG_Request
     };
 
     const string full_path;
-    CPSG_Request::TFlags flags;
     shared_ptr<SPSG_Reply> reply;
     SContext context;
     SPSG_Submitter submitted_by;
     SPSG_Processor processed_by;
 
-    SPSG_Request(string p, CPSG_Request::TFlags f, shared_ptr<SPSG_Reply> r, CRef<CRequestContext> c, const SPSG_Params& params);
+    SPSG_Request(string p, shared_ptr<SPSG_Reply> r, CRef<CRequestContext> c, const SPSG_Params& params);
 
     enum EStateResult { eContinue, eStop, eRetry };
     EStateResult OnReplyData(SPSG_Processor::TId processor_id, const char* data, size_t len)
@@ -533,6 +563,8 @@ struct SPSG_Request
     bool Fail(SPSG_Processor::TId processor_id, const SUvNgHttp2_Error& error, bool refused_stream = false);
     void Reset();
 
+    void ConvertRaw();
+
 private:
     EStateResult StatePrefix(const char*& data, size_t& len);
     EStateResult StateArgs  (const char*& data, size_t& len);
@@ -561,7 +593,7 @@ private:
 
 struct SPSG_TimedRequest
 {
-    SPSG_TimedRequest(shared_ptr<SPSG_Request> r) : m_Id(SPSG_Processor::GetNextId()), m_Request(move(r)) {}
+    SPSG_TimedRequest(shared_ptr<SPSG_Request> r) : m_Id(SPSG_Processor::GetNextId()), m_Request(std::move(r)) {}
 
     SPSG_TimedRequest(SPSG_TimedRequest&&) = default;
     SPSG_TimedRequest& operator=(SPSG_TimedRequest&&) = default;
@@ -602,10 +634,10 @@ struct SPSG_AsyncQueue : SUv_Async, SPSG_AsyncQueuesRef
 private:
     static auto s_Pop(list<SPSG_TimedRequest>& queue)
     {
-        auto timed_req = move(queue.front());
+        auto timed_req = std::move(queue.front());
         auto p = timed_req.Get();
         queue.pop_front();
-        return tuple_cat(make_tuple(make_optional(move(timed_req))), move(p));
+        return tuple_cat(make_tuple(make_optional(std::move(timed_req))), std::move(p));
     }
 
 public:
@@ -620,7 +652,7 @@ public:
     template <class... TArgs>
     void Emplace(TArgs&&... args)
     {
-        m_Queue.GetLock()->emplace_back(forward<TArgs>(args)...);
+        m_Queue.GetLock()->emplace_back(std::forward<TArgs>(args)...);
     }
 
     auto GetLockedQueue() { return m_Queue.GetLock(); }
@@ -721,11 +753,11 @@ struct SPSG_Server
     SPSG_Throttling throttling;
 
     SPSG_Server(SSocketAddress a, double r, int as, SPSG_ThrottleParams p, uv_loop_t* l) :
-        address(move(a)),
+        address(std::move(a)),
         rate(r),
         available_streams(as),
         stats(0),
-        throttling(address, move(p), l)
+        throttling(address, std::move(p), l)
     {}
 };
 
@@ -799,7 +831,7 @@ protected:
             const uint8_t* value, size_t valuelen, uint8_t flags);
 
 private:
-    enum EHeaders { eMethod, eScheme, eAuthority, ePath, eUserAgent, eSessionID, eSubHitID, eCookie, eClientIP, eSize };
+    enum EHeaders { eMethod, eScheme, eAuthority, ePath, eUserAgent, ePeerID, eSessionID, eSubHitID, eCookie, eClientIP, eSize };
 
     SPSG_Submitter::TId GetInternalId() const { return this; }
 
@@ -828,7 +860,7 @@ struct SPSG_Thread : public TImpl
 {
     template <class... TArgs>
     SPSG_Thread(SUv_Barrier& start_barrier, SUv_Barrier& stop_barrier, uint64_t timeout, uint64_t repeat, TArgs&&... args) :
-        TImpl(forward<TArgs>(args)...),
+        TImpl(std::forward<TArgs>(args)...),
         m_Timer(this, s_OnTimer, timeout, repeat),
         m_Thread(s_Execute, this, ref(start_barrier), ref(stop_barrier))
     {}
@@ -902,7 +934,7 @@ struct SPSG_Servers : protected deque<SPSG_Server>
     template <class... TArgs>
     void emplace_back(TArgs&&... args)
     {
-        TBase::emplace_back(forward<TArgs>(args)...);
+        TBase::emplace_back(std::forward<TArgs>(args)...);
         ++m_Size;
     }
 
@@ -1101,10 +1133,10 @@ struct SPSG_DiscoveryImpl
     SPSG_DiscoveryImpl(CServiceDiscovery service, shared_ptr<SPSG_Stats> stats, const SPSG_Params& params, SPSG_Servers::TTS& servers, SPSG_AsyncQueuesRef queues_ref) :
         m_Params(params),
         m_NoServers(params, servers),
-        m_Service(move(service)),
-        m_Stats(move(stats)),
+        m_Service(std::move(service)),
+        m_Stats(std::move(stats)),
         m_Servers(servers),
-        m_QueuesRef(move(queues_ref))
+        m_QueuesRef(std::move(queues_ref))
     {}
 
 protected:
