@@ -1,4 +1,4 @@
-/*  $Id: agp_validate_reader.cpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: agp_validate_reader.cpp 670366 2023-07-23 17:37:28Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,7 @@ CAgpValidateReader::CAgpValidateReader(CAgpErrEx& agpErr, CMapCompLen& comp2len,
   m_CheckCompNames=false; // checkCompNames;
   m_unplaced=false;
   m_explicit_scaf=false;
-  m_row_output=NULL;
+  m_row_output=nullptr;
 
   Reset(false);
 }
@@ -149,7 +149,7 @@ void CAgpValidateReader::OnComment()
   // No other callbacks invoked for this line.
   m_CommentLineCount++;
 
-  if(m_row_output) m_row_output->SaveRow(m_line, CRef<CAgpRow>(), NULL);
+  if(m_row_output) m_row_output->SaveRow(m_line, CRef<CAgpRow>(), nullptr);
 }
 
 void CAgpValidateReader::OnGapOrComponent()
@@ -214,7 +214,7 @@ void CAgpValidateReader::OnGapOrComponent()
       m_last_scaf_start_line=m_line_num;
       m_last_scaf_start_is_obj=false;
     }
-    if(m_row_output) m_row_output->SaveRow(m_line, m_this_row, NULL);
+    if(m_row_output) m_row_output->SaveRow(m_line, m_this_row, nullptr);
   }
   else { // component line
     m_CompCount++;
@@ -460,7 +460,7 @@ void CAgpValidateReader::OnGapOrComponent()
       m_AgpErr->Msg(CAgpErrEx::W_BreakingGapSameCompId, sameComId_otherScaf);
     }
 
-    if(m_row_output && !row_saved) m_row_output->SaveRow(m_line, m_this_row, NULL);
+    if(m_row_output && !row_saved) m_row_output->SaveRow(m_line, m_this_row, nullptr);
   }
 
   CAgpErrEx* errEx = static_cast<CAgpErrEx*>(GetErrorHandler());
@@ -503,7 +503,7 @@ void CAgpValidateReader::OnScaffoldEnd()
 
       TMapStrInt::iterator it = m_comp2len->find( m_prev_row->GetComponentId() );
       if( it!=m_comp2len->end() ) {
-        int len = it->second;
+        TAgpLen len = it->second;
         if(m_prev_component_beg!=1 || m_prev_component_end<len ) {
           m_AgpErr->Msg( CAgpErrEx::W_UnSingleCompNotInFull,
             " (" + i2s(m_prev_component_end-m_prev_component_beg+1) + " out of " + i2s(len)+ " bp)",
@@ -1143,7 +1143,7 @@ void CAgpValidateReader::x_PrintTotals(CNcbiOstream& out, bool use_xml) // witho
 
   if(m_ObjCount) {
     x_PrintPatterns(m_objNamePatterns, "Object names",
-      m_CheckObjLen ? m_comp2len->m_count : 0, NULL,
+      m_CheckObjLen ? m_comp2len->m_count : 0, nullptr,
       out, use_xml
     );
   }
@@ -1157,7 +1157,7 @@ void CAgpValidateReader::x_PrintTotals(CNcbiOstream& out, bool use_xml) // witho
     }
     bool hasSuspicious = x_PrintPatterns(compNamePatterns, "Component names",
       m_CheckObjLen ? 0 : m_comp2len->m_count,
-      m_comp2len == &m_scaf2len ? " Scaffold from component AGP" : NULL,
+      m_comp2len == &m_scaf2len ? " Scaffold from component AGP" : nullptr,
       out, use_xml
     );
     if(!m_CheckCompNames && hasSuspicious ) {
@@ -1594,7 +1594,7 @@ void CAgpValidateReader::CIdsNotInAgp::Print(CNcbiOstream& out, const string& ms
     }
   }
   else {
-    m_reader.x_PrintPatterns(m_patterns, NcbiEmptyString, 0, NULL, out, false);
+    m_reader.x_PrintPatterns(m_patterns, NcbiEmptyString, 0, nullptr, out, false);
   }
 }
 
@@ -1606,7 +1606,7 @@ void CAgpValidateReader::CIdsNotInAgp::PrintXml(CNcbiOstream& out, const string&
   for(set<string>::iterator it = m_ids.begin();  it != m_ids.end(); ++it) {
     out << " <name>" << NStr::XmlEncode(*it) << "</name>\n";
   }
-  m_reader.x_PrintPatterns(m_patterns, NcbiEmptyString, 0, NULL, out, true);
+  m_reader.x_PrintPatterns(m_patterns, NcbiEmptyString, 0, nullptr, out, true);
   out << "</MissingSeqNames>\n";
 }
 
@@ -1641,7 +1641,7 @@ int CValuesCount::x_byCount( value_type* a, value_type* b )
 }
 
 //// class CCompSpans - stores data for all preceding components
-CCompSpans::TCheckSpan CCompSpans::CheckSpan(int span_beg, int span_end, bool isPlus)
+CCompSpans::TCheckSpan CCompSpans::CheckSpan(TAgpPos span_beg, TAgpPos span_end, bool isPlus)
 {
   // The lowest priority warning (to be ignored for draft seqs)
   TCheckSpan res( begin(), CAgpErrEx::W_DuplicateComp );
@@ -1670,7 +1670,7 @@ void CCompSpans::AddSpan(const CCompVal& span)
 }
 
 //// class CMapCompLen
-int CMapCompLen::AddCompLen(const string& acc, int len, bool increment_count)
+TAgpLen CMapCompLen::AddCompLen(const string& acc, TAgpLen len, bool increment_count)
 {
   TMapStrInt::value_type acc_len(acc, len);
   TMapStrIntResult insert_result = insert(acc_len);
