@@ -1,5 +1,5 @@
 /*
- * $Id: aln_scanner_fastagap.cpp 632526 2021-06-02 17:25:01Z ivanov $
+ * $Id: aln_scanner_fastagap.cpp 665205 2023-03-30 13:36:19Z stakhovv $
  *
  * ===========================================================================
  *
@@ -84,10 +84,10 @@ CAlnScannerFastaGap::xImportAlignmentData(
 //  ----------------------------------------------------------------------------
 {
     bool waitingForSeqData = false;
-    vector<int> expectedDataSizes;
-    int currentDataLineIndex = 0;
+    vector<unsigned> expectedDataSizes;
+    unsigned currentDataLineIndex = 0;
     bool processingFirstSequence = true;
-    int expectedNumLines = 0;
+    unsigned expectedNumLines = 0;
 
     string line;
     int lineNumber = 0;
@@ -96,7 +96,6 @@ CAlnScannerFastaGap::xImportAlignmentData(
 
         string seqId;
         string defLine;
-        string seqData;
 
         if (waitingForSeqData) {
             if (line.empty()  ||  NStr::StartsWith(line, ">")) {
@@ -112,7 +111,7 @@ CAlnScannerFastaGap::xImportAlignmentData(
                 else {
 
                     if (!expectedNumLines) {
-                        expectedNumLines = expectedDataSizes.size();
+                        expectedNumLines = static_cast<unsigned>(expectedDataSizes.size());
                     }
                     auto currentDataSize = seqData.size();
                     auto expectedDataSize = (currentDataLineIndex < expectedNumLines) ?
