@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.cpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: bed_reader.cpp 697928 2025-05-27 18:27:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -414,7 +414,7 @@ bool CBedReader::xDetermineLikelyColumnCount(
             continue;
         }
 
-        CBedColumnData columnData(SReaderLine(bufferLineNumber, line));
+        CBedColumnData columnData(TReaderLine(bufferLineNumber, line));
         if (realColumnCount == 0 ) {
             realColumnCount = columnData.ColumnCount();
         }
@@ -541,7 +541,7 @@ void CBedReader::xPostProcessAnnot(
 //  ----------------------------------------------------------------------------
 bool
 CBedReader::xParseTrackLine(
-    const string& strLine)
+    const CTempString& strLine)
 //  ----------------------------------------------------------------------------
 {
     CReaderMessage warning(
@@ -574,7 +574,7 @@ CBedReader::xParseTrackLine(
 //  ----------------------------------------------------------------------------
 bool
 CBedReader::xParseFeature(
-    const SReaderLine& lineData,
+    const TReaderLine& lineData,
     CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
@@ -848,6 +848,7 @@ void CBedReader::xSetFeatureDisplayData(
                 "score",
                 NStr::StringToInt(columnData[4],
                     NStr::fConvErr_NoThrow|NStr::fAllowTrailingSymbols) );
+            feature->AddOrReplaceQualifier("score", columnData[4]);
         }
         else {
             display_data->AddField(
@@ -1782,6 +1783,9 @@ CBedReader::xReadBedRecordRaw(
     //  parse
     NStr::Split(linecopy, " \t", columns, NStr::fSplit_MergeDelimiters);
     xCleanColumnValues(columns);
+    if (mRealColumnCount == 0) {
+        mRealColumnCount = columns.size();
+    }
     if (columns.size() != mRealColumnCount) {
         CReaderMessage error(
             eDiag_Error,
