@@ -1,4 +1,4 @@
-/*  $Id: gff3_reader.cpp 632532 2021-06-02 17:25:46Z ivanov $
+/*  $Id: gff3_reader.cpp 697955 2025-05-28 15:49:57Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,13 +55,15 @@
 #include <objects/seqfeat/Genetic_code.hpp>
 #include <objects/seqfeat/RNA_ref.hpp>
 #include <objects/seqfeat/Feat_id.hpp>
+#include <objects/seqfeat/Imp_feat.hpp>
 
 #include <objtools/readers/gff3_reader.hpp>
-#include "gff3_location_merger.hpp"
+#include <objtools/readers/gff3_location_merger.hpp>
 
 #include "reader_message_handler.hpp"
 
 #include <algorithm>
+#include <util/regexp/ctre/ctre.hpp>
 
 BEGIN_NCBI_SCOPE
 BEGIN_objects_SCOPE
@@ -101,6 +103,15 @@ string CGff3Reader::xNextGenericId()
     return string("generic") + NStr::IntToString(msGenericIdCounter++);
 }
 
+//  ----------------------------------------------------------------------------
+string CGff3ReadRecord::GetOneAttribute(const string& attr)
+//  ----------------------------------------------------------------------------
+{
+    string val;
+    GetAttribute(attr, val);
+    return val;
+}
+
 //  ----------------------------------------------------------------------------
 bool CGff3ReadRecord::AssignFromGff(
     const string& strRawInput )
@@ -112,9 +123,19 @@ bool CGff3ReadRecord::AssignFromGff(
     string id, parent;
     GetAttribute("ID", id);
     GetAttribute("Parent", parent);
-    if (id.empty()  &&  parent.empty()) {
-        m_Attributes["ID"] = CGff3Reader::xNextGenericId();
+
+    if (! parent.empty() && parent == id) {
+        throw CReaderMessage(eDiag_Error, 0,
+                "Bad data line: "
+                "ID and Parent have the same value \"" + parent + "\"");
     }
+
+    if (m_strType == "protein_coding_gene" ||
+        m_strType == "ncRNA_gene") {
+        SetType("gene");
+        return true;
+    }
+
     if (m_strType == "pseudogene") {
         SetType("gene");
         m_Attributes["pseudo"] = "true";
@@ -193,7 +214,7 @@ string CGff3ReadRecord::x_NormalizedAttributeKey(
 
 //  ----------------------------------------------------------------------------
 CGff3Reader::CGff3Reader(
-    unsigned int uFlags,
+    TReaderFlags uFlags,
     const string& name,
     const string& title,
     SeqIdResolver resolver,
@@ -201,7 +222,7 @@ CGff3Reader::CGff3Reader(
 //  ----------------------------------------------------------------------------
     CGff2Reader( uFlags, name, title, resolver, pRL )
 {
-    mpLocations.reset(new CGff3LocationMerger(uFlags, resolver, 0));
+    mpLocations.reset(new CGff3LocationMerger(uFlags, resolver, 0, pRL));
     CGff2Record::ResetId();
 }
 
@@ -287,7 +308,7 @@ void CGff3Reader::xProcessAlignmentData(
 //  ----------------------------------------------------------------------------
 bool
 CGff3Reader::xParseFeature(
-    const string& line,
+    const CTempString& line,
     CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
@@ -302,6 +323,23 @@ CGff3Reader::xParseFeature(
         if (!pRecord->AssignFromGff(line)) {
             return false;
         }
+
+        string id = pRecord->GetOneAttribute("ID");
+        string parent = pRecord->GetOneAttribute("Parent");
+
+        if (! parent.empty()) {
+            if (mIDsAlreadySeen.count(parent) == 0) {
+cerr << "Parent " << parent << " not yet seen for ID " << id << endl;
+                /*
+                throw CReaderMessage(eDiag_Error, 0,
+                            "Bad data line: "
+                    "Parent \"" + parent + " not yet found as ID\"");
+                */
+            }
+        }
+        if (! id.empty()) {
+            mIDsAlreadySeen.insert(id);
+        }
     }
     catch(CObjReaderLineException& err) {
         ProcessError(err, pEC);
@@ -317,12 +355,12 @@ CGff3Reader::xParseFeature(
     }
 
     //no support for multiparented features in genbank mode:
-    if (this->IsInGenbankMode()  &&  pRecord->IsMultiParent()) {
+    if (pRecord->IsMultiParent()) {
         AutoPtr<CObjReaderLineException> pErr(
             CObjReaderLineException::Create(
             eDiag_Fatal,
             0,
-            "Multiparented features are not supported in Genbank mode"));
+            "This GFF3 reader does not support multiparented features"));
         ProcessError(*pErr, pEC);
     }
 
@@ -402,8 +440,12 @@ bool CGff3Reader::xUpdateAnnotFeature(
     if (recType == "gene") {
          return xUpdateAnnotGene(gffRecord, pFeature, annot, pEC);
     }
-    if (recType == "mrna") {
-        return xUpdateAnnotMrna(gffRecord, pFeature, annot, pEC);
+    if (NStr::EndsWith(recType, "rna")) {
+        return xUpdateAnnotRna(gffRecord, pFeature, annot, pEC);
+    }
+    // allow exon before VDJC gene segment to not crash as data error
+    if (NStr::EndsWith(recType, "_gene_segment")) {
+        return xUpdateAnnotRna(gffRecord, pFeature, annot, pEC);
     }
     if (recType == "region") {
         return xUpdateAnnotRegion(gffRecord, pFeature, annot, pEC);
@@ -456,70 +498,27 @@ bool CGff3Reader::xUpdateAnnotExon(
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-    list<string> parents;
-    if (record.GetAttribute("Parent", parents)) {
-        for (list<string>::const_iterator it = parents.begin(); it != parents.end();
-                ++it) {
-            const string& parentId = *it;
-            CRef<CSeq_feat> pParent;
-            if (!x_GetFeatureById(parentId, pParent)) {
-                xAddPendingExon(parentId, record);
-                return true;
-            }
-            if (pParent->GetData().IsRna()) {
-                xVerifyExonLocation(parentId, record);
-            }
-            if (pParent->GetData().IsGene()) {
-                if  (!xInitializeFeature(record, pFeature)) {
-                    return false;
-                }
-                return xAddFeatureToAnnot(pFeature, annot);
-            }
-            IdToFeatureMap::iterator fit = m_MapIdToFeature.find(parentId);
-            if (fit != m_MapIdToFeature.end()) {
-                CRef<CSeq_feat> pParent = fit->second;
-                if (!record.UpdateFeature(m_iFlags, pParent)) {
-                    return false;
-                }
-            }
-        }
-    }
-    return true;
-}
-
-
-//  ----------------------------------------------------------------------------
-bool CGff3Reader::xJoinLocationIntoRna(
-    const CGff2Record& record,
-    //CRef<CSeq_feat> pFeature,
-    //CSeq_annot& annot,
-    ILineErrorListener* pEC)
-//  ----------------------------------------------------------------------------
-{
-    list<string> parents;
-    if (!record.GetAttribute("Parent", parents)) {
-        return true;
-    }
-    for (list<string>::const_iterator it = parents.begin(); it != parents.end();
-            ++it) {
-        const string& parentId = *it;
+    string parentId;
+    if (record.GetAttribute("Parent", parentId)) {
         CRef<CSeq_feat> pParent;
         if (!x_GetFeatureById(parentId, pParent)) {
-            // Danger:
-            // We don't know whether the CDS parent is indeed an RNA and it could
-            //  possible be a gene.
-            // If the parent is indeed a gene then gene construction will have to
-            //  purge this pending exon (or it will cause a sanity check to fail
-            //  during post processing).
-            xAddPendingExon(parentId, record);
-            continue;
+            return true;
         }
-        if (!pParent->GetData().IsRna()) {
-            continue;
+        if (pParent->GetData().IsRna()) {
+            xVerifyExonLocation(parentId, record);
         }
-        xVerifyExonLocation(parentId, record);
-        if (!record.UpdateFeature(m_iFlags, pParent)) {
-            return false;
+        if (pParent->GetData().IsGene()) {
+            if (!xInitializeFeature(record, pFeature)) {
+                return false;
+            }
+            return xAddFeatureToAnnot(pFeature, annot);
+        }
+        IdToFeatureMap::iterator fit = m_MapIdToFeature.find(parentId);
+        if (fit != m_MapIdToFeature.end()) {
+            CRef<CSeq_feat> pParent = fit->second;
+            if (!record.UpdateFeature(m_iFlags, pParent)) {
+                return false;
+            }
         }
     }
     return true;
@@ -534,14 +533,19 @@ bool CGff3Reader::xUpdateAnnotCds(
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-    if (!xJoinLocationIntoRna(record, pEC)) {
-        return false;
-    }
     xVerifyCdsParents(record);
 
     string cdsId = xMakeRecordId(record);
     mpLocations->AddRecordForId(cdsId, record);
 
+    string parentId;
+    record.GetAttribute("Parent", parentId);
+
+    if (!parentId.empty()) {
+        mCdsParents.insert(parentId);
+        mpLocations->AddRecordForId(parentId, record);
+    }
+
     auto pExistingFeature = m_MapIdToFeature.find(cdsId);
     if (pExistingFeature != m_MapIdToFeature.end()) {
         return true;
@@ -551,8 +555,6 @@ bool CGff3Reader::xUpdateAnnotCds(
     xInitializeFeature(record, pFeature);
     xAddFeatureToAnnot(pFeature, annot);
 
-    string parentId;
-    record.GetAttribute("Parent", parentId);
     if (!parentId.empty()) {
         xFeatureSetQualifier("Parent", parentId, pFeature);
         xFeatureSetXrefParent(parentId, pFeature);
@@ -730,7 +732,7 @@ bool CGff3Reader::xUpdateAnnotGeneric(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff3Reader::xUpdateAnnotMrna(
+bool CGff3Reader::xUpdateAnnotRna(
     const CGff2Record& record,
     CRef<CSeq_feat> pFeature,
     CSeq_annot& annot,
@@ -778,12 +780,6 @@ bool CGff3Reader::xUpdateAnnotMrna(
     mrnaLoc->Assign(pFeature->GetLocation().GetInt());
     mMrnaLocs[strId] = mrnaLoc;
 
-    list<CGff2Record> pendingExons;
-    xGetPendingExons(strId, pendingExons);
-    for (auto exonRecord: pendingExons) {
-        CRef< CSeq_feat > pFeature(new CSeq_feat);
-        xUpdateAnnotExon(exonRecord, pFeature, annot, pEC);
-    }
     if (! xAddFeatureToAnnot(pFeature, annot)) {
         return false;
     }
@@ -813,13 +809,6 @@ bool CGff3Reader::xUpdateAnnotGene(
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
-    // address corner case:
-    // parent of CDS is a gene but the DCS is listed before the gene so at the
-    //  time we did not know the parent would be a gene.
-    // remedy: throw out any collected cds locations that were meant for RNA
-    //  construction.
-    list<CGff2Record> pendingExons;
-    xGetPendingExons(strId, pendingExons);
     return true;
 }
 
@@ -862,24 +851,17 @@ void CGff3Reader::xVerifyCdsParents(
 //  ----------------------------------------------------------------------------
 {
     string id;
-    string parents;
+    string parentId;
     if (!record.GetAttribute("ID", id)) {
         return;
     }
-    record.GetAttribute("Parent", parents);
-    if (parents.empty()) {
-        CReaderMessage error(
-            eDiag_Error,
-            m_uLineNumber,
-            "Bad data line: CDS record is missing mandatory Parent attribute.");
-        throw error;
-    }
+    record.GetAttribute("Parent", parentId);
     map<string, string>::iterator it = mCdsParentMap.find(id);
     if (it == mCdsParentMap.end()) {
-        mCdsParentMap[id] = parents;
+        mCdsParentMap[id] = parentId;
         return;
     }
-    if (it->second == parents) {
+    if (it->second == parentId) {
         return;
     }
     CReaderMessage error(
@@ -889,6 +871,15 @@ void CGff3Reader::xVerifyCdsParents(
     throw error;
 }
 
+
+//  ----------------------------------------------------------------------------
+bool CGff3Reader::xHasCdsChild(const string& id) const
+//  ----------------------------------------------------------------------------
+{
+    return (mCdsParents.find(id) != mCdsParents.end());
+}
+
+
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xReadInit()
 //  ----------------------------------------------------------------------------
@@ -897,12 +888,13 @@ bool CGff3Reader::xReadInit()
         return false;
     }
     mCdsParentMap.clear();
+    mIDsAlreadySeen.clear();
     return true;
 }
 
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xIsIgnoredFeatureType(
-    const string& featureType)
+    const CTempString& featureType)
 //  ----------------------------------------------------------------------------
 {
     typedef CStaticArraySet<string, PNocase> STRINGARRAY;
@@ -1015,33 +1007,19 @@ CGff3Reader::xInitializeFeature(
 }
 
 //  ----------------------------------------------------------------------------
-void
-CGff3Reader::xAddPendingExon(
-    const string& rnaId,
-    const CGff2Record& exonRecord)
-//  ----------------------------------------------------------------------------
+
+static bool s_TreatAsRna(const CSeqFeatData& featData)
 {
-    PENDING_EXONS::iterator it = mPendingExons.find(rnaId);
-    if (it == mPendingExons.end()) {
-        mPendingExons[rnaId] = list<CGff2Record>();
+    if (featData.IsRna()) {
+        return true;
     }
-    mPendingExons[rnaId].push_back(exonRecord);
+    const auto subtype = featData.GetSubtype();
+    return (subtype == CSeqFeatData::eSubtype_C_region ||
+            subtype == CSeqFeatData::eSubtype_D_segment ||
+            subtype == CSeqFeatData::eSubtype_J_segment ||
+            subtype == CSeqFeatData::eSubtype_V_segment);
 }
 
-//  ----------------------------------------------------------------------------
-void
-CGff3Reader::xGetPendingExons(
-    const string& rnaId,
-    list<CGff2Record>& pendingExons)
-//  ----------------------------------------------------------------------------
-{
-    PENDING_EXONS::iterator it = mPendingExons.find(rnaId);
-    if (it == mPendingExons.end()) {
-        return;
-    }
-    pendingExons.swap(mPendingExons[rnaId]);
-    mPendingExons.erase(rnaId);
-}
 
 //  ----------------------------------------------------------------------------
 void CGff3Reader::xPostProcessAnnot(
@@ -1052,29 +1030,63 @@ void CGff3Reader::xPostProcessAnnot(
         xProcessAlignmentData(annot);
         return;
     }
-    if (!mCurrentFeatureCount) {
+    if (! mCurrentFeatureCount) {
         return;
     }
 
-    for (const auto& it: mPendingExons) {
-        CReaderMessage warning(
-            eDiag_Warning,
-            m_uLineNumber,
-            "Bad data line: Record references non-existent Parent=" + it.first);
-        m_pMessageHandler->Report(warning);
-    }
-
-    //location fixup:
-    for (auto itLocation: mpLocations->LocationMap()) {
-        auto id = itLocation.first;
+    // location fixup:
+    for (auto itLocation : mpLocations->LocationMap()) {
+        auto id        = itLocation.first;
         auto itFeature = m_MapIdToFeature.find(id);
         if (itFeature == m_MapIdToFeature.end()) {
             continue;
         }
-        CRef<CSeq_loc> pNewLoc(new CSeq_loc);
+        CRef<CSeq_loc>    pNewLoc(new CSeq_loc);
+        CRef<CSeq_feat>   pFeature = itFeature->second;
         CCdregion::EFrame frame;
+        const auto&       locs = itLocation.second;
+
+        if (pFeature->GetData().IsImp() &&
+            pFeature->GetData().GetImp().IsSetKey() &&
+            pFeature->GetData().GetImp().GetKey() == "misc_RNA") {
+            // RW-143: if this has a child (pseudogenic) CDS, convert to mRNA
+            if (xHasCdsChild(id)) {
+                pFeature->SetData().SetRna().SetType(CRNA_ref::eType_mRNA);
+            }
+        }
+
+        if (s_TreatAsRna(pFeature->GetData())) {
+            // check for exons
+            list<CGff3LocationRecord> exonLocs;
+            for (auto record : locs) {
+                if (record.mType == "exon") {
+                    exonLocs.push_back(record);
+                }
+            }
+            // prioritize exons over UTR and CDS child features
+            if (! exonLocs.empty()) {
+                mpLocations->MergeLocation(pNewLoc, frame, exonLocs);
+                pFeature->SetLocation(*pNewLoc);
+                continue;
+            }
+        } else if (pFeature->GetData().IsGene()) {
+
+            list<CGff3LocationRecord> geneLocs;
+            for (auto record : locs) {
+                if (record.mType == "gene") {
+                    geneLocs.push_back(record);
+                }
+            }
+
+            if (! geneLocs.empty()) { // should never be empty
+                mpLocations->MergeLocation(pNewLoc, frame, geneLocs);
+                pFeature->SetLocation(*pNewLoc);
+                continue;
+            }
+        }
+
+
         mpLocations->MergeLocation(pNewLoc, frame, itLocation.second);
-        CRef<CSeq_feat> pFeature = itFeature->second;
         pFeature->SetLocation(*pNewLoc);
         if (pFeature->GetData().IsCdregion()) {
             auto& cdrData = pFeature->SetData().SetCdregion();
@@ -1088,7 +1100,7 @@ void CGff3Reader::xPostProcessAnnot(
 
 //  ----------------------------------------------------------------------------
 void CGff3Reader::xProcessSequenceRegionPragma(
-    const string& pragma)
+    const CTempString& pragma)
 //  ----------------------------------------------------------------------------
 {
     TSeqPos sequenceSize(0);
@@ -1114,6 +1126,9 @@ void CGff3Reader::xProcessSequenceRegionPragma(
         }
     }
     mpLocations->SetSequenceSize(tokens[1], sequenceSize);
+    auto resolvedId = mSeqIdResolve(tokens[1], m_iFlags, true)->AsFastaString();
+    mpLocations->SetSequenceSize(resolvedId, sequenceSize);
+
 }
 
 //  ----------------------------------------------------------------------------
@@ -1123,5 +1138,22 @@ TSeqPos CGff3Reader::SequenceSize() const
     return mpLocations->SequenceSize();
 }
 
+//  ----------------------------------------------------------------------------
+TSeqPos CGff3Reader::GetSequenceSize(
+    const string& seqId) const
+//  ----------------------------------------------------------------------------
+{
+    return mpLocations->GetSequenceSize(seqId);
+}
+
+//  ----------------------------------------------------------------------------
+void CGff3Reader::xValidateAnnot(
+    const CSeq_annot& )
+//  ----------------------------------------------------------------------------
+{
+    mpLocations->Validate();
+}
+
+
 END_objects_SCOPE
 END_NCBI_SCOPE
