@@ -1,5 +1,5 @@
 /*
- * $Id: gtf_location_merger.cpp 674645 2023-11-01 12:38:20Z ivanov $
+ * $Id: gtf_location_merger.cpp 688878 2024-10-11 15:45:40Z foleyjp $
  *
  * ===========================================================================
  *
@@ -326,6 +326,7 @@ CGtfLocationMerger::MergeLocationDefault(
     for (auto& location: locations) {
         mix.AddSeqLoc(*location.GetLocation());
     }
+    pSeqloc = pSeqloc->Merge(CSeq_loc::fMerge_AbuttingOnly, nullptr);
     return pSeqloc;
 }
 
@@ -344,7 +345,7 @@ CGtfLocationMerger::MergeLocationForCds(
     for (auto& location: locations) {
         mix.AddSeqLoc(*location.GetLocation());
     }
-    pSeqloc = pSeqloc->Merge(CSeq_loc::fMerge_All, 0);
+    pSeqloc = pSeqloc->Merge(CSeq_loc::fMerge_AbuttingOnly, nullptr);
     return pSeqloc;
 }
 
@@ -433,6 +434,7 @@ CGtfLocationMerger::MergeLocationForGene(
             pSeqloc->SetMix(*pMix);
         }
     }
+    pSeqloc = pSeqloc->Merge(CSeq_loc::fMerge_AbuttingOnly, nullptr);
     return pSeqloc;
 }
 
