@@ -1,4 +1,4 @@
- /*  $Id: gvf_reader.cpp 663694 2023-02-27 16:15:10Z ludwigf $
+ /*  $Id: gvf_reader.cpp 697928 2025-05-27 18:27:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -208,7 +208,7 @@ CGvfReader::xProcessData(
 //  ----------------------------------------------------------------------------
 bool
 CGvfReader::xParseFeature(
-    const string& line,
+    const CTempString& line,
     CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
@@ -557,7 +557,7 @@ bool CGvfReader::xFeatureSetVariation(
 
 //  ----------------------------------------------------------------------------
 bool CGvfReader::xParseStructuredComment(
-    const string& strLine)
+    const CTempString& strLine)
 //  ----------------------------------------------------------------------------
 {
     if ( !CGff2Reader::xParseStructuredComment( strLine) ) {
