@@ -1,4 +1,4 @@
-/*  $Id: idmapper_config.cpp 666851 2023-05-02 19:22:01Z foleyjp $
+/*  $Id: idmapper_config.cpp 694116 2025-02-25 18:57:56Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,9 +47,27 @@ USING_SCOPE(objects);
 
 CIdMapperConfig::CIdMapperConfig(CNcbiIstream& istr,
                                  const std::string& strContext,
+                                 bool bInvert,
+                                 ILineErrorListener* pErrors) 
+    : CIdMapperConfig(istr, strContext, bInvert, true, pErrors) // localOnly == true
+{
+}
+
+
+CIdMapperConfig::CIdMapperConfig(const std::string& strContext,
                                  bool bInvert,
                                  ILineErrorListener* pErrors)
-    : CIdMapper(strContext, bInvert, pErrors)
+    : CIdMapperConfig(strContext, bInvert, true, pErrors) // localOnly == true
+{
+}
+
+
+CIdMapperConfig::CIdMapperConfig(CNcbiIstream& istr,
+                                 const std::string& strContext,
+                                 bool bInvert,
+                                 bool localOnly,
+                                 ILineErrorListener* pErrors)
+    : CIdMapperConfig(strContext, bInvert, localOnly, pErrors)
 {
     Initialize(istr);
 }
@@ -57,20 +75,20 @@ CIdMapperConfig::CIdMapperConfig(CNcbiIstream& istr,
 
 CIdMapperConfig::CIdMapperConfig(const std::string& strContext,
                                  bool bInvert,
+                                 bool localOnly,
                                  ILineErrorListener* pErrors)
-    : CIdMapper(strContext, bInvert, pErrors)
+    : CIdMapper(strContext, bInvert, pErrors), m_LocalOnly(localOnly)
 {
 }
 
-
 void CIdMapperConfig::Initialize(CNcbiIstream& istr)
 {
     string buffer;
-    {{
-         CNcbiOstrstream os;
+    {
+         ostringstream os;
          NcbiStreamCopy(os, istr);
-         buffer = string(CNcbiOstrstreamToString(os));
-     }}
+         buffer = os.str();
+    }
 
     CMemoryRegistry reg;
     try {
@@ -235,8 +253,13 @@ CIdMapperConfig::SourceHandle(
     const string& strId )
 //  ============================================================================
 {
-    CSeq_id source( CSeq_id::e_Local, strId );
-    return CSeq_id_Handle::GetHandle( source );
+    CRef<CSeq_id> pSource;
+    if (m_LocalOnly) {
+        pSource = Ref(new CSeq_id(CSeq_id::e_Local, strId));
+    } else {
+        pSource = Ref(new CSeq_id(strId, CSeq_id::fParse_AnyRaw | CSeq_id::fParse_AnyLocal));
+    }
+    return CSeq_id_Handle::GetHandle(*pSource);
 };
 
 //  ============================================================================
