@@ -1,4 +1,4 @@
-/*  $Id: idmapper_gcassembly.cpp 665431 2023-04-04 17:43:43Z stakhovv $
+/*  $Id: idmapper_gcassembly.cpp 692986 2025-02-04 14:14:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -281,12 +281,12 @@ void CIdMapperGCAssembly::x_AddAliasMappings(const CGC_Sequence& seq,
 
                 default:
                     _ASSERT(false);
-                    CNcbiOstrstream str;
+                    ostringstream str;
                     str << MSerial_AsnText << **it;
                     NCBI_THROW(CIdMapperException, eBadSeqId,
                         "Unhandled ID type in GC-Assembly: " +
                         NStr::PrintableString(
-                        (string)CNcbiOstrstreamToString(str),
+                        str.str(),
                         NStr::fNewLine_Quote));
                 }
             }
