@@ -1,4 +1,4 @@
-/*  $Id: rm_reader.cpp 625986 2021-02-22 21:26:47Z mozese2 $
+/*  $Id: rm_reader.cpp 692986 2025-02-04 14:14:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -551,7 +551,7 @@ CRef<CSeq_feat> CRepeatToFeat::operator()(const IRepeatRegion& repeat)
                 tag->SetDb("taxon");
                 tag->SetTag().SetId(specificity);
                 feat->SetDbxref().push_back(tag);
-                if (fRemoveRedundancy  &&  m_Library  &&
+                if ((m_Flags & fRemoveRedundancy) && m_Library &&
                     m_Library->TestSpecificityMatchesName(
                             specificity,
                             specificity_name)) {
@@ -685,7 +685,7 @@ CRef<CSeq_feat> CRepeatToFeat::operator()(const IRepeatRegion& repeat)
             // the possibility the annotation is remapped, the original
             // query identifier is preserved.
 
-            CNcbiOstrstream comment;
+            ostringstream comment;
             const char eq('='), sep(' ');
 
             comment << "source=RepeatMasker";
@@ -728,7 +728,7 @@ CRef<CSeq_feat> CRepeatToFeat::operator()(const IRepeatRegion& repeat)
                 comment << sep
                         << "rpt_length" << eq << extra.GetRptLength();
             }
-            feat->SetComment(CNcbiOstrstreamToString(comment));
+            feat->SetComment(comment.str());
         }
     }
 
