@@ -1,4 +1,4 @@
-/*  $Id: wiggle_reader.cpp 653402 2022-07-27 16:13:42Z ludwigf $
+/*  $Id: wiggle_reader.cpp 697928 2025-05-27 18:27:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -519,7 +519,7 @@ CRef<CSeq_graph> CWiggleReader::xMakeGraph()
     b_graph.SetMin(stat.AsByte(stat.m_Min));
     b_graph.SetMax(stat.AsByte(stat.m_Max));
     b_graph.SetAxis(0);
-    vector<char>& bytes = b_graph.SetValues();
+    CByte_graph::TValues& bytes = b_graph.SetValues();
 
     if ( m_Values.empty() ) {
         graph->SetNumval(0);
@@ -813,7 +813,7 @@ void CWiggleReader::xSetChrom(
 
 //  ----------------------------------------------------------------------------
 bool CWiggleReader::xParseBrowserLine(
-    const string& line,
+    const CTempString& line,
     CSeq_annot&)
 //  ----------------------------------------------------------------------------
 {
@@ -825,7 +825,7 @@ bool CWiggleReader::xParseBrowserLine(
 
 //  ----------------------------------------------------------------------------
 bool CWiggleReader::xParseTrackLine(
-    const string& line)
+    const CTempString& line)
 //  ----------------------------------------------------------------------------
 {
     if (!xIsTrackLine(line)) {
