@@ -1,4 +1,4 @@
-/*  $Id: datatool.cpp 624387 2021-01-28 19:47:11Z ivanov $
+/*  $Id: datatool.cpp 662316 2023-01-27 14:31:36Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -649,7 +649,7 @@ bool CDataTool::GenerateCode(bool undo)
         string ex;
         ex = generator.GetConfig().Get("-","_export");
         if (ex.empty()) {
-            ex = opt;
+            ex = NStr::TruncateSpaces(opt);
         }
         CClassCode::SetExportSpecifier(ex);
     }
@@ -662,7 +662,7 @@ bool CDataTool::GenerateCode(bool undo)
                 string root = opt;
                 if (root.empty()) {
                     // default
-                    root = "http://www.ncbi.nlm.nih.gov/IEB/ToolBox/CPP_DOC/lxr/source";
+                    root = "https://www.ncbi.nlm.nih.gov/IEB/ToolBox/CPP_DOC/lxr/source";
                 }
                 CClassCode::SetDocRootURL(root);
             }
