@@ -1,4 +1,4 @@
-/*  $Id: enumtype.cpp 624723 2021-02-03 18:51:43Z ivanov $
+/*  $Id: enumtype.cpp 696022 2025-04-07 12:25:50Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -223,7 +223,7 @@ void CEnumDataType::PrintXMLSchema(CNcbiOstream& out, int indent, bool /*content
             }
         }
     }
-    if (IsASNDataSpec()) {
+    if (IsASNDataSpec() || inAttlist) {
         string tmp = "<xs:attribute name=\"" + value + "\" use=\"" + use + "\"" + form;
         const CDataMember* mem = GetDataMember();
         if (!inAttlist) {
