@@ -1,4 +1,4 @@
-/*  $Id: xsdparser.cpp 649692 2022-05-09 15:47:05Z gouriano $
+/*  $Id: xsdparser.cpp 696024 2025-04-07 12:26:07Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1280,13 +1280,24 @@ void XSDParser::ParseEnumeration(DTDAttribute& att)
 // http://www.w3.org/TR/2004/REC-xmlschema-1-20041028/structures.html#key-vv
 {
     TToken tok = GetRawAttributeSet();
-    att.SetType(DTDAttribute::eEnum);
+    if (att.GetType() == DTDAttribute::eInteger ||
+        att.GetType() == DTDAttribute::eBigInt ||
+        att.GetType() == DTDAttribute::eIntEnum) {
+        att.SetType(DTDAttribute::eIntEnum);
+    } else {
+        att.SetType(DTDAttribute::eEnum);
+    }
     int id = 0;
+    bool hasint = false;
     if (GetAttribute("intvalue")) {
         id = NStr::StringToInt(m_Value);
         att.SetType(DTDAttribute::eIntEnum);
+        hasint = true;
     }
     if (GetAttribute("value")) {
+        if (!hasint && att.GetType() == DTDAttribute::eIntEnum) {
+            id = NStr::StringToInt(m_Value);
+        }
         string v(m_ValuePrefix);
         if (!v.empty()) {
             v += ':';
