@@ -1,4 +1,4 @@
-/*  $Id: objistr.cpp 673980 2023-10-16 13:16:36Z ivanov $
+/*  $Id: objistr.cpp 674289 2023-10-23 14:30:08Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -308,6 +308,16 @@ EFixNonPrint CObjectIStream::x_GetFixCharsMethodDefault(void) const
     return s_SerialFixChars->Get();
 }
 
+EFixNonPrint CObjectIStream::x_FixCharsMethod(void) const {
+    if (GetDataFormat() == eSerial_AsnText || GetDataFormat() == eSerial_AsnBinary) {
+        if (GetStackDepth() > 1 && FetchFrameFromTop(1).HasTypeInfo() &&
+            FetchFrameFromTop(1).GetTypeInfo()->GetDataSpec() != EDataSpec::eASN) {
+            return eFNP_Allow;
+        }
+    }
+    return m_FixMethod;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // skip unknown members setup
 
