@@ -1,4 +1,4 @@
-/*  $Id: objistrasn.cpp 574303 2018-11-08 19:59:04Z gouriano $
+/*  $Id: objistrasn.cpp 651126 2022-06-10 12:57:23Z ludwigf $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -108,11 +108,12 @@ bool CObjectIStreamAsn::EndOfData(void)
     return false;
 }
 
-string CObjectIStreamAsn::GetPosition(void) const
+void CObjectIStreamAsn::Location(string& loc_type, size_t& loc) const
 {
-    return "line "+NStr::SizetToString(m_Input.GetLine());
+    loc_type = "line";
+    loc = m_Input.GetLine();
 }
-
+ 
 inline
 bool CObjectIStreamAsn::FirstIdChar(char c)
 {
