@@ -1,4 +1,4 @@
-/*  $Id: objistrasnb.cpp 574758 2018-11-19 13:27:42Z gouriano $
+/*  $Id: objistrasnb.cpp 643328 2022-01-10 13:54:54Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1580,7 +1580,7 @@ void CObjectIStreamAsnBinary::SkipAnyContent(void)
 }
 
 set<TTypeInfo>
-CObjectIStreamAsnBinary::GuessDataType(set<TTypeInfo>& known_types,
+CObjectIStreamAsnBinary::GuessDataType(const set<TTypeInfo>& known_types,
                                        size_t max_length,
                                        size_t max_bytes)
 {
