@@ -1,4 +1,4 @@
-/*  $Id: objistrjson.cpp 624386 2021-01-28 19:46:31Z ivanov $
+/*  $Id: objistrjson.cpp 651126 2022-06-10 12:57:23Z ludwigf $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -102,9 +102,10 @@ bool CObjectIStreamJson::EndOfData(void)
     return false;
 }
 
-string CObjectIStreamJson::GetPosition(void) const
+void CObjectIStreamJson::Location(string& loc_type, size_t& loc) const
 {
-    return "line "+NStr::SizetToString(m_Input.GetLine());
+    loc_type = "line";
+    loc = m_Input.GetLine();
 }
 
 void CObjectIStreamJson::SetDefaultStringEncoding(EEncoding enc)
@@ -898,6 +899,16 @@ TMemberIndex CObjectIStreamJson::BeginClassMember(const CClassTypeInfo* classTyp
             UndoClassMember();
             return last;
         }
+        if (!m_GotNameless && classType->GetDataSpec() == EDataSpec::eJSON) {
+            if (CanSkipUnknownMembers()) {
+                SetFailFlags(fUnknownValue);
+                SkipAnyContent();
+                m_ExpectValue = false;
+                return BeginClassMember(classType);
+            } else {
+                UnexpectedMember(tagName, classType->GetMembers());
+            }
+        }
     } else {
         if (classType->GetMembers().GetItemInfo(ind)->GetId().HasNotag()) {
             TopFrame().SetNotag();
