@@ -1,4 +1,4 @@
-/*  $Id: objistrxml.cpp 604400 2020-03-27 12:53:54Z ivanov $
+/*  $Id: objistrxml.cpp 686785 2024-08-27 14:36:26Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -56,7 +56,7 @@ CObjectIStream* CObjectIStream::CreateObjectIStreamXml()
 
 CObjectIStreamXml::CObjectIStreamXml(void)
     : CObjectIStream(eSerial_Xml),
-      m_TagState(eTagOutside), m_Attlist(false),
+      m_TagState(eTagOutside), m_LeadingWs(0), m_Attlist(false),
       m_StdXml(false), m_Doctype_found(false), m_IsNil(false),
       m_Encoding( eEncoding_Unknown ),
       m_StringEncoding( eEncoding_UTF8 ),
@@ -67,7 +67,7 @@ CObjectIStreamXml::CObjectIStreamXml(void)
 
 CObjectIStreamXml::CObjectIStreamXml(CNcbiIstream& in, EOwnership deleteIn)
     : CObjectIStream(eSerial_Xml),
-      m_TagState(eTagOutside), m_Attlist(false),
+      m_TagState(eTagOutside), m_LeadingWs(0), m_Attlist(false),
       m_StdXml(false), m_Doctype_found(false), m_IsNil(false),
       m_Encoding( eEncoding_Unknown ),
       m_StringEncoding( eEncoding_UTF8 ),
@@ -88,6 +88,7 @@ void CObjectIStreamXml::ResetState(void)
         return;
     }
     m_TagState = eTagOutside;
+    m_LeadingWs = 0;
     m_LastTag.clear();
     m_RejectedTag.clear();
     m_Attlist = false;
@@ -127,9 +128,10 @@ bool CObjectIStreamXml::EndOfData(void)
     return false;
 }
 
-string CObjectIStreamXml::GetPosition(void) const
+void CObjectIStreamXml::Location(string& loc_type, size_t& loc) const
 {
-    return "line "+NStr::SizetToString(m_Input.GetLine());
+    loc_type = "line";
+    loc = m_Input.GetLine();
 }
 
 void CObjectIStreamXml::SetEnforcedStdXml(bool set)
@@ -242,7 +244,9 @@ char CObjectIStreamXml::SkipWSAndComments(void)
 {
     _ASSERT(OutsideTag());
     for ( ;; ) {
+        Int8 before = m_Input.GetStreamPosAsInt8();
         char c = m_Input.SkipSpaces();
+        m_LeadingWs += m_Input.GetStreamPosAsInt8() - before;
         switch ( c ) {
         case '\t':
             m_Input.SkipChar();
@@ -253,7 +257,14 @@ char CObjectIStreamXml::SkipWSAndComments(void)
             m_Input.SkipEndOfLine(c);
             continue;
         case '<':
-// http://www.w3.org/TR/REC-xml/#dt-comment
+// https://www.w3.org/TR/xml11/#sec-pi
+            if ( m_Input.PeekChar(1) == '?') {
+                m_Input.SkipChar();
+                Found_lt();
+                SkipQDecl();
+                return SkipWSAndComments();
+            }
+// https://www.w3.org/TR/xml11/#sec-comments
             if ( m_Input.PeekChar(1) == '!' &&
                  m_Input.PeekChar(2) == '-' &&
                  m_Input.PeekChar(3) == '-' ) {
@@ -437,16 +448,19 @@ CTempString CObjectIStreamXml::ReadName(char c)
 
     // save beginning of tag name
     const char* ptr = m_Input.GetCurrentPos();
+    m_LastTag = string(ptr+iColon, i-iColon);
+    string ns_prefix;
+    if (iColon > 1) {
+        ns_prefix = string(ptr, iColon-1);
+    }
+    m_Input.SkipChars(i);
 
     // check end of tag name
-    m_Input.SkipChars(i);
     if (c == '\n' || c == '\r') {
         m_Input.SkipChar();
         m_Input.SkipEndOfLine(c);
     }
-    m_LastTag = CTempString(ptr+iColon, i-iColon);
     if (iColon > 1) {
-        string ns_prefix( CTempString(ptr, iColon-1));
         if (ns_prefix == "xmlns") {
             string value;
             ReadAttributeValue(value, true);
@@ -505,7 +519,7 @@ CTempString CObjectIStreamXml::ReadName(char c)
         char ch = SkipWS();
         return IsEndOfTagChar(ch) ? CTempString() : ReadName(ch);
     }
-    return CTempString(ptr+iColon, i-iColon);
+    return m_LastTag;
 }
 
 CTempString CObjectIStreamXml::RejectedName(void)
@@ -545,13 +559,8 @@ void CObjectIStreamXml::SkipQDecl(void)
         string value;
         ReadAttributeValue(value);
         if (tagName == "encoding") {
-            if (NStr::CompareNocase(value.c_str(),"UTF-8") == 0) {
-                m_Encoding = eEncoding_UTF8;
-            } else if (NStr::CompareNocase(value.c_str(),"ISO-8859-1") == 0) {
-                m_Encoding = eEncoding_ISO8859_1;
-            } else if (NStr::CompareNocase(value.c_str(),"Windows-1252") == 0) {
-                m_Encoding = eEncoding_Windows_1252;
-            } else {
+            m_Encoding = CUtf8::StringToEncoding(value);
+            if (m_Encoding == eEncoding_Unknown || m_Encoding > eEncoding_Windows_1252) {
                 ThrowError(fFormatError, "unsupported encoding: " + value);
             }
             break;
@@ -586,6 +595,12 @@ string CObjectIStreamXml::ReadFileHeader(void)
     }
     
     m_Doctype_found = false;
+    if (GetStackDepth() > 0) {
+        const CObjectStack::TFrame& top = TopFrame();
+        if (top.HasTypeInfo()) {
+            m_Doctype_found = top.GetTypeInfo()->GetDataSpec() == EDataSpec::eDTD;
+        }
+    }
     for ( ;; ) {
         switch ( BeginOpeningTag() ) {
         case '?':
@@ -596,7 +611,7 @@ string CObjectIStreamXml::ReadFileHeader(void)
                 m_Input.SkipChar();
                 CTempString tagName = ReadName(m_Input.PeekChar());
                 if ( tagName == "DOCTYPE" ) {
-                    m_Doctype_found = true;
+//                    m_Doctype_found = true;
                     ReadName(SkipWS());
                     // skip the rest of !DOCTYPE
                     for ( ;; ) {
@@ -1282,15 +1297,22 @@ bool CObjectIStreamXml::ReadCDSection(string& str)
 void CObjectIStreamXml::ReadTagData(string& str, EStringType type)
 /*
     White Space Handling:
-    http://www.w3.org/TR/2000/REC-xml-20001006#sec-white-space
+    https://www.w3.org/TR/xml/#sec-white-space
+    https://www.w3.org/TR/xml11/#sec-white-space
 
     End-of-Line Handling
-    http://www.w3.org/TR/2000/REC-xml-20001006#sec-line-ends
+    https://www.w3.org/TR/xml/#sec-line-ends
+    https://www.w3.org/TR/xml11/#sec-line-ends
     
     Attribute-Value Normalization
-    http://www.w3.org/TR/2000/REC-xml-20001006#AVNormalize
+    https://www.w3.org/TR/xml/#AVNormalize
+    https://www.w3.org/TR/xml11/#AVNormalize
 */
 {
+    if (m_LeadingWs != 0) {
+        str.append(m_LeadingWs, ' ');
+        m_LeadingWs = 0;
+    }
     BeginData();
     bool encoded = false;
     bool CR = false;
@@ -1652,7 +1674,7 @@ bool CObjectIStreamXml::ThisTagIsSelfClosed(void)
     if (InsideOpeningTag()) {
         return EndOpeningTagSelfClosed();
     }
-    return false;
+    return SelfClosedTag();
 }
 
 
@@ -2214,6 +2236,10 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
                 TMemberIndex ind = first+1;
                 if (classType->GetMemberInfo(ind)->GetId().HasNotag()) {
                     TopFrame().SetNotag();
+                    if (x_IsStdXml() && !GetEnforcedStdXml()) {
+                        ETypeFamily type = GetRealTypeFamily(classType->GetMemberInfo(ind)->GetTypeInfo());
+                        m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
+                    }
                     return ind;
                 }
                 if ( NextTagIsClosing() )
@@ -2276,7 +2302,7 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
 
     TMemberIndex ind = classType->GetMembers().Find(tagName);
     if (ind == kInvalidMember) {
-        ind = classType->GetMembers().FindDeep(tagName);
+        ind = classType->GetMembers().FindDeep(tagName, pos);
         if (ind != kInvalidMember && ind >= pos) {
             TopFrame().SetNotag();
             UndoClassMember();
@@ -2355,6 +2381,7 @@ void CObjectIStreamXml::UndoClassMember(void)
     if (InsideOpeningTag()) {
         m_RejectedTag = m_LastTag;
         m_TagState = eTagOutside;
+        m_LeadingWs = 0;
 #if defined(NCBI_SERIAL_IO_TRACE)
     cout << ", Undo= " << m_LastTag;
 #endif
