@@ -1,4 +1,4 @@
-/*  $Id: objostr.cpp 607093 2020-04-30 12:29:36Z grichenk $
+/*  $Id: objostr.cpp 674289 2023-10-23 14:30:08Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -284,6 +284,15 @@ EFixNonPrint CObjectOStream::x_GetFixCharsMethodDefault(void) const
     return s_SerialFixChars->Get();
 }
 
+EFixNonPrint CObjectOStream::x_FixCharsMethod(void) const {
+    if (GetDataFormat() == eSerial_AsnText || GetDataFormat() == eSerial_AsnBinary) {
+        if (GetStackDepth() > 1 && FetchFrameFromTop(1).HasTypeInfo() &&
+            FetchFrameFromTop(1).GetTypeInfo()->GetDataSpec() != EDataSpec::eASN) {
+            return eFNP_Allow;
+        }
+    }
+    return m_FixMethod;
+}
 
 /////////////////////////////////////////////////////////////////////////////
 
