@@ -1,4 +1,4 @@
-/*  $Id: objostrxml.cpp 608769 2020-05-20 19:18:29Z vasilche $
+/*  $Id: objostrxml.cpp 696021 2025-04-07 12:25:29Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -233,18 +233,9 @@ void CObjectOStreamXml::WriteFileHeader(TTypeInfo type)
 {
     if (m_UseXmlDecl) {
         m_Output.PutString("<?xml version=\"1.0");
-        switch (m_Encoding) {
-        default:
-            break;
-        case eEncoding_UTF8:
-            m_Output.PutString("\" encoding=\"UTF-8");
-            break;
-        case eEncoding_ISO8859_1:
-            m_Output.PutString("\" encoding=\"ISO-8859-1");   
-            break;
-        case eEncoding_Windows_1252:
-            m_Output.PutString("\" encoding=\"Windows-1252");   
-            break;
+        if (m_Encoding != eEncoding_Unknown) {
+            m_Output.PutString("\" encoding=\"");
+            m_Output.PutString(CUtf8::EncodingToString(m_Encoding));
         }
         m_Output.PutString("\"?>");
     }
@@ -706,7 +697,7 @@ void CObjectOStreamXml::WriteDouble2(double data, unsigned digits)
         if (m_FastWriteDouble) {
             width = NStr::DoubleToStringPosix(data, digits, buffer, sizeof(buffer));
         } else {
-            width = sprintf(buffer, "%.*g", (int)digits, data);
+            width = snprintf(buffer, sizeof(buffer)/sizeof(char), "%.*g", (int)digits, data);
             char* dot = strchr(buffer,',');
             if (dot) {
                 *dot = '.'; // enforce C locale
@@ -984,6 +975,9 @@ void CObjectOStreamXml::WriteNullPointer(void)
         }
         return;
     }
+    if (m_EndTag) {
+        return;
+    }
     m_SpecialCaseWrite = nillable ? eWriteAsNil : eWriteAsDefault;
     x_SpecialCaseWrite();
     m_SpecialCaseWrite = eWriteAsNormal;
@@ -1473,6 +1467,11 @@ void CObjectOStreamXml::BeginClassMember(TTypeInfo memberType,
             } else {
                 needTag = !id.HasNotag() && !id.HasAnyContent() && type != eTypeFamilyContainer;
                 m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
+                if (needTag && m_SkipNextTag) {
+                    if (memberType->HasNamespaceName() && m_NsNameToPrefix.find(memberType->GetNamespaceName()) == m_NsNameToPrefix.end()) {
+                        needTag = m_SkipNextTag = false;
+                    }
+                }
             }
             if (needTag) {
                 OpenStackTag(0);
@@ -1619,6 +1618,12 @@ void CObjectOStreamXml::BeginChoiceVariant(const CChoiceTypeInfo* choiceType,
         } else {
             needTag = !id.HasNotag() && !id.HasAnyContent() && type != eTypeFamilyContainer;
             m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
+            if (needTag && m_SkipNextTag) {
+                TTypeInfo var_type = choiceType->GetItemInfo(id.GetName())->GetTypeInfo();
+                if (var_type->HasNamespaceName() && m_NsNameToPrefix.find(var_type->GetNamespaceName()) == m_NsNameToPrefix.end()) {
+                    needTag = m_SkipNextTag = false;
+                }
+            }
         }
         if (needTag) {
             OpenStackTag(0);
