@@ -1,4 +1,4 @@
-/*  $Id: csraloader_impl.cpp 673877 2023-10-12 13:16:52Z ivanov $
+/*  $Id: csraloader_impl.cpp 672765 2023-09-14 17:30:28Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -582,7 +582,7 @@ CCSRADataLoader_Impl::GetRefSeqInfo(const CSeq_id_Handle& idh)
         if ( !file.first ) {
             return TRefLock();
         }
-        return TRefLock(file.first->GetRefSeqInfo(idh), move(file.second));
+        return TRefLock(file.first->GetRefSeqInfo(idh), std::move(file.second));
     }
     TRefLock ret;
     NON_CONST_ITERATE ( TFixedFiles, it, m_FixedFiles ) {
@@ -840,7 +840,7 @@ CCSRADataLoader_Impl::GetRecordsOnce(CDataSource* data_source,
             }
         }
         if ( need_align && ref_info ) {
-            if ( CRef<CCSRABlobId> blob_id = GetBlobId(TRefLock(ref_info, move(file.second)), CCSRABlobId::eBlobType_annot) ) {
+            if ( CRef<CCSRABlobId> blob_id = GetBlobId(TRefLock(ref_info, std::move(file.second)), CCSRABlobId::eBlobType_annot) ) {
                 CDataLoader::TTSE_Lock tse_lock = GetBlobById(data_source, *blob_id);
                 tse_lock->x_LoadChunk(kMainChunkId);
                 int chunk_id = ref_info->GetAnnotChunkId(ref_pos);
