@@ -1,4 +1,4 @@
-/*  $Id: snploader_impl.cpp 673882 2023-10-12 13:17:25Z ivanov $
+/*  $Id: snploader_impl.cpp 689425 2024-10-28 16:19:39Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -553,9 +553,8 @@ void CSNPBlobId::FromString(CTempString str)
 }
 
 
-bool CSNPBlobId::operator<(const CBlobId& id) const
+bool CSNPBlobId::operator<(const CSNPBlobId& id2) const
 {
-    const CSNPBlobId& id2 = dynamic_cast<const CSNPBlobId&>(id);
     if ( m_NAIndex != id2.m_NAIndex ) {
         return m_NAIndex < id2.m_NAIndex;
     }
@@ -581,17 +580,16 @@ bool CSNPBlobId::operator<(const CBlobId& id) const
 }
 
 
-bool CSNPBlobId::operator==(const CBlobId& id) const
+bool CSNPBlobId::operator==(const CSNPBlobId& id2) const
 {
-    const CSNPBlobId& id2 = dynamic_cast<const CSNPBlobId&>(id);
     return m_NAIndex == id2.m_NAIndex &&
-        m_NAVersion == id2.m_NAVersion &&
-        m_SeqIndex == id2.m_SeqIndex &&
-        m_FilterIndex == id2.m_FilterIndex &&
-        m_IsPrimaryTrack == id2.m_IsPrimaryTrack &&
-        m_IsPrimaryTrackGraph == id2.m_IsPrimaryTrackGraph &&
-        m_Accession == id2.m_Accession &&
-        m_SeqId == id2.m_SeqId;
+           m_NAVersion == id2.m_NAVersion &&
+           m_SeqIndex == id2.m_SeqIndex &&
+           m_FilterIndex == id2.m_FilterIndex &&
+           m_IsPrimaryTrack == id2.m_IsPrimaryTrack &&
+           m_IsPrimaryTrackGraph == id2.m_IsPrimaryTrackGraph &&
+           m_Accession == id2.m_Accession &&
+           m_SeqId == id2.m_SeqId;
 }
 
 
@@ -884,19 +882,13 @@ CSNPDataLoader_Impl::GetOrphanAnnotRecordsOnce(CDataSource* ds,
                 // default SNP track
                 string acc_ver = s_GetAccVer(id);
                 if ( !acc_ver.empty() ) {
-                    string na_acc;
-                    try {
-                        // add default SNP track
-                        if ( GetDebugLevel() >= eDebug_resolve ) {
-                            LOG_POST_X(13, Info<<"CSNPDataLoader:PTIS: resolving "<<acc_ver);
-                        }
-                        na_acc = m_PTISClient->GetPrimarySnpTrackForAccVer(acc_ver);
-                        if ( GetDebugLevel() >= eDebug_resolve ) {
-                            LOG_POST_X(13, Info<<"CSNPDataLoader:PTIS: "<<acc_ver<<" primary SNP track is "<<na_acc);
-                        }
+                    // add default SNP track
+                    if ( GetDebugLevel() >= eDebug_resolve ) {
+                        LOG_POST_X(13, Info<<"CSNPDataLoader:PTIS: resolving "<<acc_ver);
                     }
-                    catch ( CException& exc ) {
-                        ERR_POST_X(2, "CSNPDataLoader: failed to add PTIS track for "<<acc_ver<<": "<<exc);
+                    string na_acc = m_PTISClient->GetPrimarySnpTrackForAccVer(acc_ver);
+                    if ( GetDebugLevel() >= eDebug_resolve ) {
+                        LOG_POST_X(13, Info<<"CSNPDataLoader:PTIS: "<<acc_ver<<" primary SNP track is "<<na_acc);
                     }
                     if ( !na_acc.empty() ) {
                         size_t filter_index = sx_ExtractFilterIndex(na_acc);
