@@ -1,4 +1,4 @@
-/*  $Id: bamgraph.cpp 538108 2017-06-07 17:46:42Z vasilche $
+/*  $Id: bamgraph.cpp 679608 2024-02-28 18:12:53Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -459,12 +459,13 @@ vector<Uint8> CBam2Seq_graph::CollectRawAccessCoverage(CBamRawDb& bam_raw_db)
 vector<Uint8> CBam2Seq_graph::CollectEstimatedCoverage(const CBamHeader& header,
                                                        const CBamIndex& bam_index)
 {
-    m_GraphBinSize = kEstimatedGraphBinSize;
+    auto bin_size = bam_index.GetMinBinSize();
+    m_GraphBinSize = bin_size;
     size_t ref_index = header.GetRefIndex(GetRefLabel());
     TSeqPos length = header.GetRefLength(ref_index);
     vector<uint64_t> ret = bam_index.CollectEstimatedCoverage(ref_index);
     if ( length == 0 || length == kInvalidSeqPos ) {
-        length = TSeqPos(ret.size())*kEstimatedGraphBinSize;
+        length = TSeqPos(ret.size())*bin_size;
     }
     m_TotalRange.SetFrom(0).SetToOpen(length);
     m_AlignCount = 0;
@@ -608,8 +609,8 @@ CRef<CSeq_annot> CBam2Seq_graph::MakeSeq_annot(const vector<Uint8>& cov,
     graph->SetLoc().SetWhole(*SerialClone(*m_RefId));
     graph->SetComp(GetGraphBinSize());
     graph->SetNumval(TSeqPos(cov.size()));
-    vector<char>* vvb = 0;
-    vector<int>* vvi = 0;
+    CByte_graph::TValues* vvb = 0;
+    CInt_graph::TValues* vvi = 0;
     int MAX = 0;
     if ( GetGraphValueType() == eGraphValueType_byte ) {
         MAX = 254;
