@@ -1,4 +1,4 @@
-/*  $Id: vdbread.cpp 681697 2024-04-12 12:53:38Z ivanov $
+/*  $Id: vdbread.cpp 683856 2024-06-06 17:12:19Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -72,6 +72,7 @@
 #include <objects/seqset/seqset__.hpp>
 #include <objects/seqres/seqres__.hpp>
 #include <sra/error_codes.hpp>
+#include <util/random_gen.hpp>
 
 #include <cstring>
 #include <algorithm>
@@ -103,13 +104,30 @@ NCBI_PARAM_DECL(int, VDB, DEBUG);
 NCBI_PARAM_DEF_EX(int, VDB, DEBUG, 0, eParam_NoThread, VDB_DEBUG);
 
 
-static int s_GetDebugLevel(void)
+static int& s_GetDebugLevelVar(void)
 {
     static int value = NCBI_PARAM_TYPE(VDB, DEBUG)::GetDefault();
     return value;
 }
 
 
+inline
+static int s_GetDebugLevel()
+{
+    return s_GetDebugLevelVar();
+}
+
+
+inline
+static int s_SetDebugLevel(int new_level)
+{
+    auto& var = s_GetDebugLevelVar();
+    auto old_level = var;
+    var = new_level;
+    return old_level;
+}
+
+
 NCBI_PARAM_DECL(bool, VDB, DISABLE_PAGEMAP_THREAD);
 NCBI_PARAM_DEF(bool, VDB, DISABLE_PAGEMAP_THREAD, false);
 
@@ -146,6 +164,11 @@ DEFINE_SRA_REF_TRAITS(VResolver, );
 #define VDB_READ_RANDOM_FAILS_RECOVER 1000
 
 #if defined(VDB_RANDOM_FAILS)
+static bool s_Fails(unsigned frequency)
+{
+    static CSafeStatic<CRandom> s_Random([]()->CRandom*{CRandom* r=new CRandom();r->Randomize();return r;}, nullptr);
+    return s_Random->GetRandIndex(frequency) == 0;
+}
 # define SIMULATE_ERROR(name)                                           \
     if ( !(FAILS_VAR(name, _RANDOM_FAILS)) ) {                          \
     }                                                                   \
@@ -155,7 +178,7 @@ DEFINE_SRA_REF_TRAITS(VResolver, );
             --recover_counter;                                          \
         }                                                               \
         else {                                                          \
-            if ( (rand() % (FAILS_VAR(name, _RANDOM_FAILS_FREQUENCY))) == 0 ) { \
+            if ( s_Fails(FAILS_VAR(name, _RANDOM_FAILS_FREQUENCY)) ) {  \
                 recover_counter = (FAILS_VAR(name, _RANDOM_FAILS_RECOVER)); \
                 ERR_POST("VDB: Simulated " #name " failure: "<<CStackTrace()); \
                 NCBI_THROW(CSraException, eOtherError,                  \
@@ -509,6 +532,18 @@ CVDBMgr::CVDBMgr(void)
 }
 
 
+int CVDBMgr::GetDebugLevel()
+{
+    return s_GetDebugLevel();
+}
+
+
+int CVDBMgr::SetDebugLevel(int new_level)
+{
+    return s_SetDebugLevel(new_level);
+}
+
+
 string CVDBMgr::FindAccPath(const string& acc) const
 {
     if ( !m_Resolver ) {
@@ -523,9 +558,12 @@ string CVDBMgr::FindDereferencedAccPath(const string& acc_or_path) const
     string path;
     if ( CVPath::IsPlainAccession(acc_or_path) ) {
         // resolve VDB accessions
+        if ( s_GetDebugLevel() >= CVDBMgr::eDebugResolve ) {
+            LOG_POST_X(6, "CVDBMgr: resolving VDB accession: " << acc_or_path);
+        }
         path = FindAccPath(acc_or_path);
-        if ( s_GetDebugLevel() >= 2 ) {
-            LOG_POST_X(4, "CVDBMgr: resolved accession: " << acc_or_path << " -> " << path);
+        if ( s_GetDebugLevel() >= CVDBMgr::eDebugResolve ) {
+            LOG_POST_X(4, "CVDBMgr: resolved VDB accession: " << acc_or_path << " -> " << path);
         }
     }
     else {
@@ -534,13 +572,16 @@ string CVDBMgr::FindDereferencedAccPath(const string& acc_or_path) const
     }
 
     // resolve symbolic links for correct timestamp and longer-living reference
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugResolve ) {
+        LOG_POST_X(7, "CVDBMgr: checking file symbolic link: " << acc_or_path << " -> " << path);
+    }
     CDirEntry de(path);
     if ( de.Exists() ) {
         de.DereferencePath();
         if ( de.GetPath() != path ) {
             path = de.GetPath();
-            if ( s_GetDebugLevel() >= 2 ) {
-                LOG_POST_X(5, "CVDBMgr: resolved file link: " << acc_or_path << " -> " << path);
+            if ( s_GetDebugLevel() >= CVDBMgr::eDebugResolve ) {
+                LOG_POST_X(5, "CVDBMgr: resolved file symbolic link: " << acc_or_path << " -> " << path);
             }
         }
     }
@@ -567,7 +608,7 @@ CTime CVDBMgr::GetTimestamp(const string& path) const
     }
     _ASSERT(!timestamp.IsEmpty());
     _ASSERT(timestamp.IsUniversalTime());
-    if ( s_GetDebugLevel() >= 2 ) {
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugResolve ) {
         LOG_POST_X(3, "CVDBMgr: timestamp of " << path << ": " << CTime(timestamp).ToLocalTime());
     }
     return timestamp;
@@ -898,7 +939,7 @@ static void s_InitLocalKNS(KNSManager* kns_mgr)
 }
 
 
-static void s_VDBInit()
+void CVDBMgr::s_VDBInit()
 {
     CFastMutexGuard guard(sx_SDKMutex);
     static bool initialized = false;
@@ -917,7 +958,7 @@ static void s_VDBInit()
             KLogLevelSet(ask_level);
             KLogHandlerSet(VDBLogWriter, 0);
             KLogLibHandlerSet(VDBLogWriter, 0);
-            if ( s_GetDebugLevel() >= 2 ) {
+            if ( s_GetDebugLevel() >= CVDBMgr::eDebugInitialize ) {
                 const char* msg = "info: VDB initialized";
                 size_t written;
                 VDBLogWriter(0, msg, strlen(msg), &written);
@@ -1058,6 +1099,9 @@ void CVDBMgr::CommitConfig() const
 CVDB::CVDB(const CVDBMgr& mgr, const string& acc_or_path)
     : m_Name(acc_or_path)
 {
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugOpen ) {
+        LOG_POST_X(8, "CVDBMgr: opening VDB: " << acc_or_path);
+    }
     DECLARE_SDK_GUARD();
     CFinalRequestContextUpdater ctx_updater;
     string path = CVPath::ConvertAccOrSysPathToPOSIX(acc_or_path);
@@ -1095,6 +1139,9 @@ CVDB::CVDB(const CVDBMgr& mgr, const string& acc_or_path)
             NCBI_THROW2(CSraException, eOtherError, msg, rc);
         }
     }
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugOpen ) {
+        LOG_POST_X(9, "CVDBMgr: opened VDB: " << acc_or_path << " (" << path << ")");
+    }
     if ( 0 ) {
         const KDatabase* kdb = 0;
         if ( rc_t rc = VDatabaseOpenKDatabaseRead(*this, &kdb) ) {
@@ -1441,7 +1488,7 @@ void CVDBCursor::ReadElements(TVDBRowId row, const CVDBColumn& column,
                        '['<<row<<"]["<<start<<".."<<(start+count-1)<<
                        "] only "<<read_count<<" elements are read");
     }
-    if ( s_GetDebugLevel() >= 9 ) {
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugData ) {
         LOG_POST(Info<<"VDB "<<*this<<column<<'['<<row<<"]: @"<<start<<" #"<<count);
     }
 }
@@ -1543,6 +1590,14 @@ void CVDBColumn::Init(const CVDBCursor& cursor,
 {
     DECLARE_SDK_GUARD();
     CFinalRequestContextUpdater ctx_updater;
+    if ( !name ) {
+        if ( missing == eMissing_Throw ) {
+            NCBI_THROW_FMT(CSraException, eInvalidArg,
+                           "NULL column name is not allowed"<<cursor<<*this);
+        }
+        m_Index = kInvalidIndex;
+        return;
+    }
     m_Name = name;
     //SIMULATE_SCHEMA_ERROR();
     if ( rc_t rc = VCursorAddColumn(cursor, &m_Index, name) ) {
@@ -1650,7 +1705,7 @@ void CVDBValue::x_Get(const CVDBCursor& cursor, const CVDBColumn& column)
                         <<cursor<<column<<": "<<bit_offset,
                         RC(rcApp, rcColumn, rcDecoding, rcOffset, rcUnsupported));
     }
-    if ( s_GetDebugLevel() >= 9 ) {
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugData ) {
         CNcbiOstrstream s;
         if ( bit_length == 8 ) {
             s << '"' << NStr::PrintableString(CTempString((const char*)m_Data, m_ElemCount)) << '"';
@@ -1714,7 +1769,7 @@ void CVDBValue::x_Get(const CVDBCursor& cursor,
                         cursor<<column<<'['<<row<<"]: "<<bit_offset,
                         RC(rcApp, rcColumn, rcDecoding, rcOffset, rcUnsupported));
     }
-    if ( s_GetDebugLevel() >= 9 ) {
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugData ) {
         CNcbiOstrstream s;
         if ( bit_length == 8 ) {
             s << '"' << NStr::PrintableString(CTempString((const char*)m_Data, m_ElemCount)) << '"';
@@ -1830,7 +1885,7 @@ void CVDBValueFor4Bits::x_Get(const CVDBCursor& cursor,
     m_RawData = static_cast<const char*>(data);
     m_ElemOffset = bit_offset >> 2;
     m_ElemCount = elem_count;
-    if ( s_GetDebugLevel() >= 9 ) {
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugData ) {
         CNcbiOstrstream s;
         if ( bit_length == 4 ) {
             for ( uint32_t i = 0; i < elem_count; ++i ) {
@@ -1928,7 +1983,7 @@ void CVDBValueFor2Bits::x_Get(const CVDBCursor& cursor,
     m_RawData = static_cast<const char*>(data);
     m_ElemOffset = bit_offset >> 1;
     m_ElemCount = elem_count;
-    if ( s_GetDebugLevel() >= 9 ) {
+    if ( s_GetDebugLevel() >= CVDBMgr::eDebugData ) {
         CNcbiOstrstream s;
         if ( bit_length == 2 ) {
             for ( uint32_t i = 0; i < elem_count; ++i ) {
