@@ -1,4 +1,4 @@
-/*  $Id: bytesrc.cpp 578752 2019-01-24 19:25:20Z lavr $
+/*  $Id: bytesrc.cpp 684400 2024-06-21 17:51:55Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -102,7 +102,7 @@ CRef<CSubSourceCollector>
 CByteSourceReader::SubSource(size_t /*prevent*/,
                              CRef<CSubSourceCollector> parent)
 {
-    return CRef<CSubSourceCollector>(new CMemorySourceCollector(parent));
+    return CRef<CSubSourceCollector>(new CMemorySourceCollector(parent, IsMultiPart()));
 }
 
 
@@ -256,7 +256,10 @@ CFStreamByteSource::CFStreamByteSource(const string& fileName, bool binary)
                                            IFStreamFlags(binary)))
 {
     if ( !*m_Stream ) {
-        NCBI_THROW(CUtilException,eNoInput,"file not found: " + fileName);
+        CNcbiError::SetFromErrno();
+        NCBI_THROW(CUtilException,eNoInput, string("No input data: ")
+            + NCBI_ERRNO_STR_WRAPPER(NCBI_ERRNO_CODE_WRAPPER())
+            + string(": ") + fileName);
     }
 }
 
@@ -433,8 +436,10 @@ CFileByteSourceReader::CFileByteSourceReader(const CFileByteSource* source)
     if ( !m_FStream ) {
 //        THROW1_TRACE(runtime_error, "file not found: " +
 //                     source->GetFileName());
-        NCBI_THROW(CUtilException,eNoInput,
-                   "file not found: " +source->GetFileName());
+        CNcbiError::SetFromErrno();
+        NCBI_THROW(CUtilException,eNoInput,  string("No input data: ")
+            + NCBI_ERRNO_STR_WRAPPER(NCBI_ERRNO_CODE_WRAPPER())
+            + string(": ") + source->GetFileName());
     }
     m_Stream = &m_FStream;
 }
@@ -530,20 +535,27 @@ CRef<CByteSource> CFileSourceCollector::GetSource(void)
 /////////////////////////////////////////////////////////////////////////////
 
 CMemoryChunk::CMemoryChunk(const char* data, size_t dataSize,
-                           CRef<CMemoryChunk> prevChunk)
-    : m_Data(new char[dataSize]),
-      m_DataSize(dataSize)
+                           CRef<CMemoryChunk> prevChunk, ECopyData copy /*= eCopyData*/)
+    : m_DataSize(dataSize),
+      m_CopyData(copy)
 {
-    memcpy(m_Data, data, dataSize);
+    if (m_CopyData == eNoCopyData) {
+        m_Data = data;
+    } else {
+        char *buffer = new char[dataSize];
+        memcpy(buffer, data, dataSize);
+        m_Data = buffer;
+    }
     if ( prevChunk ) {
         prevChunk->m_NextChunk = this;
     }
 }
 
-
 CMemoryChunk::~CMemoryChunk(void)
 {
-    delete[] m_Data;
+    if (m_CopyData != eNoCopyData) {
+        delete[] m_Data;
+    }
     CRef<CMemoryChunk> next = m_NextChunk;
     m_NextChunk.Reset();
     while ( next && next->ReferencedOnlyOnce() ) {
@@ -623,14 +635,44 @@ bool CMemoryByteSourceReader::EndOfData(void) const
     return !(m_CurrentChunk->GetNextChunk());
 }
 
+bool CMemoryByteSourceReader::Pushback(const char* data, size_t size)
+{
+    if (m_CurrentChunkOffset >= size) {
+        m_CurrentChunkOffset -= size;
+        return true;
+    }
+    return CByteSourceReader::Pushback(data, size);
+}
+
+size_t CMemoryByteSourceReader::GetNextPart(char** buffer, size_t /*copy_count*/)
+{
+    while ( m_CurrentChunk ) {
+        size_t avail = GetCurrentChunkAvailable();
+        if ( avail == 0 ) {
+            // End of current chunk
+            CConstRef<CMemoryChunk> rest = m_CurrentChunk->GetNextChunk();
+            m_CurrentChunk = rest;
+            m_CurrentChunkOffset = 0;
+        }
+        else {
+            size_t c = avail;
+            *buffer = const_cast<char*>(m_CurrentChunk->GetData(m_CurrentChunkOffset));
+            m_CurrentChunkOffset += c;
+            return c;
+        }
+    }
+    return 0;
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 // CMemorySourceCollector
 /////////////////////////////////////////////////////////////////////////////
 
 CMemorySourceCollector::CMemorySourceCollector(CRef<CSubSourceCollector>
-                                               parent)
-    : CSubSourceCollector(parent)
+                                               parent, bool no_copy)
+    : CSubSourceCollector(parent),
+      m_CopyData(no_copy ? CMemoryChunk::eNoCopyData : CMemoryChunk::eCopyData)
 {
 }
 
@@ -644,7 +686,7 @@ void CMemorySourceCollector::AddChunk(const char* buffer,
                                       size_t bufferLength)
 {
     CSubSourceCollector::AddChunk(buffer, bufferLength);
-    m_LastChunk = new CMemoryChunk(buffer, bufferLength, m_LastChunk);
+    m_LastChunk = new CMemoryChunk(buffer, bufferLength, m_LastChunk, m_CopyData);
     if ( !m_FirstChunk ) {
         m_FirstChunk = m_LastChunk;
     }
