@@ -1,4 +1,4 @@
-/*  $Id: bzip2.cpp 538813 2017-06-14 12:15:14Z ivanov $
+/*  $Id: bzip2.cpp 676281 2023-12-13 16:37:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -25,21 +25,31 @@
  *
  * Authors:  Vladimir Ivanov
  *
- * File Description:  BZip2 Compression API
+ * File Description:  BZip2 Compression API wrapper
  *
  */
 
 #include <ncbi_pch.hpp>
 #include <corelib/ncbi_limits.h>
-#include <util/compress/bzip2.hpp>
 #include <util/error_codes.hpp>
+
+
+/// Error codes for ERR_COMPRESS and OMPRESS_HANDLE_EXCEPTIONS are really
+/// a subcodes and current maximum  value is defined in 'include/util/error_codes.hpp':
+///     NCBI_DEFINE_ERRCODE_X(Util_Compress, 210, max);
+/// For new values use 'max'+1 and update it there.
+///
+#define NCBI_USE_ERRCODE_X   Util_Compress
+
+#if defined(HAVE_LIBBZ2)
+
+#include <util/compress/bzip2.hpp>
 #include <bzlib.h>
 
 
-#define NCBI_USE_ERRCODE_X   Util_Compress
-
 BEGIN_NCBI_SCOPE
 
+
 // Macro to check flags
 #define F_ISSET(mask) ((GetFlags() & (mask)) == (mask))
 
@@ -52,8 +62,13 @@ BEGIN_NCBI_SCOPE
 
 // Maximum size of the chuck compressed by bzip2,
 // but there is no limit to the number of chunks in the buffer.
+// (bzip2 operates with 'unsigned int' values internally)
 const unsigned int kMaxChunkSize = kMax_UInt;
 
+// Default bzip2 verbosity.
+// Can be changed in range [0..4] for debug purposes only.
+const int kVerbosity = 0;
+
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -62,11 +77,13 @@ const unsigned int kMaxChunkSize = kMax_UInt;
 //
 
 
-CBZip2Compression::CBZip2Compression(ELevel level, int verbosity,
-                                     int work_factor, int small_decompress)
-    : CCompression(level), m_Verbosity(verbosity), m_WorkFactor(work_factor),
-      m_SmallDecompress(small_decompress)
+CBZip2Compression::CBZip2Compression(ELevel level)
+    : CCompression(level)
 {
+    // Set advanced compression parameters
+    SetWorkFactor( GetWorkFactorDefault() );
+    SetSmallDecompress( GetSmallDecompressDefault() );
+
     // Initialize the compressor stream structure
     m_Stream = new bz_stream;
     if ( m_Stream ) {
@@ -100,6 +117,18 @@ CCompression::ELevel CBZip2Compression::GetLevel(void) const
 }
 
 
+bool CBZip2Compression::HaveSupport(ESupportFeature feature)
+{
+    switch (feature) {
+    case eFeature_NoCompression:
+    case eFeature_Dictionary:
+    case eFeature_EstimateCompressionBufferSize:
+        return false;
+    }
+    return false;
+}
+
+
 bool CBZip2Compression::CompressBuffer(
                         const void* src_buf, size_t  src_len,
                         void*       dst_buf, size_t  dst_size,
@@ -169,7 +198,6 @@ bool CBZip2Compression::DecompressBuffer(
     // Check parameters
     if ( !src_len ) {
         if ( F_ISSET(fAllowEmptyData) ) {
-            SetError(BZ_OK);
             return true;
         }
         src_buf = NULL;
@@ -234,20 +262,36 @@ bool CBZip2Compression::DecompressBuffer(
 }
 
 
+CCompression::SRecommendedBufferSizes 
+CBZip2Compression::GetRecommendedBufferSizes(size_t round_up)
+{
+    SRecommendedBufferSizes sizes;
+    sizes.compression_in    = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    sizes.compression_out   = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    sizes.decompression_in  = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    sizes.decompression_out = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    return sizes;
+}
+
+
 bool CBZip2Compression::CompressFile(const string& src_file,
                                      const string& dst_file,
-                                     size_t        buf_size)
+                                     size_t        file_io_bufsize,
+                                     size_t        compression_in_bufsize,
+                                     size_t        compression_out_bufsize)
 {
-    CBZip2CompressionFile cf(GetLevel(), m_Verbosity, m_WorkFactor, m_SmallDecompress);
+    CBZip2CompressionFile cf(GetLevel());
     cf.SetFlags(cf.GetFlags() | GetFlags());
+    cf.SetWorkFactor(GetWorkFactor());
 
     // Open output file
-    if ( !cf.Open(dst_file, CCompressionFile::eMode_Write) ) {
+    if ( !cf.Open(dst_file, CCompressionFile::eMode_Write,
+                  compression_in_bufsize, compression_out_bufsize) ) {
         SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         return false;
     } 
     // Make compression
-    if ( !CCompression::x_CompressFile(src_file, cf, buf_size) ) {
+    if ( !CCompression::x_CompressFile(src_file, cf, file_io_bufsize) ) {
         if ( cf.GetErrorCode() ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         }
@@ -263,20 +307,24 @@ bool CBZip2Compression::CompressFile(const string& src_file,
 
 bool CBZip2Compression::DecompressFile(const string& src_file,
                                        const string& dst_file,
-                                       size_t        buf_size)
+                                       size_t        file_io_bufsize,
+                                       size_t        decompression_in_bufsize,
+                                       size_t        decompression_out_bufsize)
 {
-    CBZip2CompressionFile cf(GetLevel(), m_Verbosity, m_WorkFactor, m_SmallDecompress);
+    CBZip2CompressionFile cf(GetLevel());
     cf.SetFlags(cf.GetFlags() | GetFlags());
+    cf.SetSmallDecompress(GetSmallDecompress());
 
     // Open output file
-    if ( !cf.Open(src_file, CCompressionFile::eMode_Read) ) {
+    if ( !cf.Open(dst_file, CCompressionFile::eMode_Write,
+                  decompression_in_bufsize, decompression_out_bufsize) ) {
         if ( cf.GetErrorCode() ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         }
         return false;
     } 
     // Make decompression
-    if ( CCompression::x_DecompressFile(cf, dst_file, buf_size) ) {
+    if ( CCompression::x_DecompressFile(cf, dst_file, file_io_bufsize) ) {
         SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         cf.Close();
         return false;
@@ -288,6 +336,13 @@ bool CBZip2Compression::DecompressFile(const string& src_file,
 }
 
 
+bool CBZip2Compression::SetDictionary(CCompressionDictionary&, ENcbiOwnership)
+{
+    SetError(BZ_CONFIG_ERROR, "No dictionary support");
+    return false;
+}
+
+
 // Please, see a ftp://sources.redhat.com/pub/bzip2/docs/manual_3.html#SEC17
 // for detailed error descriptions.
 const char* CBZip2Compression::GetBZip2ErrorDescription(int errcode)
@@ -327,6 +382,18 @@ string CBZip2Compression::FormatErrorMessage(string where,
 }
 
 
+//----------------------------------------------------------------------------
+// 
+// Advanced parameters
+//
+
+int CBZip2Compression::GetWorkFactorDefault(void)  { return   0; };
+int CBZip2Compression::GetWorkFactorMin(void)      { return   0; };
+int CBZip2Compression::GetWorkFactorMax(void)      { return 250; };
+
+bool CBZip2Compression::GetSmallDecompressDefault(void) { return false; };
+
+
 
 //////////////////////////////////////////////////////////////////////////////
 //
@@ -335,12 +402,15 @@ string CBZip2Compression::FormatErrorMessage(string where,
 
 
 CBZip2CompressionFile::CBZip2CompressionFile(
-    const string& file_name, EMode mode,
-    ELevel level, int verbosity, int work_factor, int small_decompress)
-    : CBZip2Compression(level, verbosity, work_factor, small_decompress), 
-      m_FileStream(0)
+        const string& file_name,
+        EMode mode,
+        ELevel level,
+        size_t compression_in_bufsize,
+        size_t compression_out_bufsize
+    )
+    : CBZip2Compression(level), m_FileStream(0), m_EOF(true), m_HaveData(false)
 {
-    if ( !Open(file_name, mode) ) {
+    if ( !Open(file_name, mode, compression_in_bufsize, compression_out_bufsize) ) {
         const string smode = (mode == eMode_Read) ? "reading" : "writing";
         NCBI_THROW(CCompressionException, eCompressionFile, 
                    "[CBZip2CompressionFile]  Cannot open file '" + file_name +
@@ -350,9 +420,8 @@ CBZip2CompressionFile::CBZip2CompressionFile(
 }
 
 
-CBZip2CompressionFile::CBZip2CompressionFile(
-    ELevel level, int verbosity, int work_factor, int small_decompress)
-    : CBZip2Compression(level, verbosity, work_factor, small_decompress), 
+CBZip2CompressionFile::CBZip2CompressionFile(ELevel level)
+    : CBZip2Compression(level), 
       m_FileStream(0), m_EOF(true), m_HaveData(false)
 {
     return;
@@ -369,18 +438,23 @@ CBZip2CompressionFile::~CBZip2CompressionFile(void)
 }
 
 
-bool CBZip2CompressionFile::Open(const string& file_name, EMode mode)
+bool CBZip2CompressionFile::Open(const string& file_name, EMode mode,
+                                 size_t /*compression_in_bufsize*/,
+                                 size_t /*compression_out_bufsize*/)
 {
+    /* We use bzip2 implementation to work with compression files
+       so in/out internal compression buffer sizes are not used.
+    */
     int errcode;
 
     if ( mode == eMode_Read ) {
         m_FileStream = fopen(file_name.c_str(), "rb");
-        m_File = BZ2_bzReadOpen(&errcode, m_FileStream, m_SmallDecompress, m_Verbosity, 0, 0);
+        m_File = BZ2_bzReadOpen(&errcode, m_FileStream, GetSmallDecompress() ? 1 : 0, kVerbosity, 0, 0);
         m_DecompressMode = eMode_Unknown;
         m_EOF = false;
     } else {
         m_FileStream = fopen(file_name.c_str(), "wb");
-        m_File = BZ2_bzWriteOpen(&errcode, m_FileStream, GetLevel(), m_Verbosity, m_WorkFactor);
+        m_File = BZ2_bzWriteOpen(&errcode, m_FileStream, GetLevel(), kVerbosity, GetWorkFactor());
     }
     m_Mode = mode;
 
@@ -390,7 +464,6 @@ bool CBZip2CompressionFile::Open(const string& file_name, EMode mode)
         ERR_COMPRESS(20, FormatErrorMessage("CBZip2CompressionFile::Open", false));
         return false;
     };
-    SetError(BZ_OK);
     return true;
 } 
 
@@ -491,10 +564,8 @@ bool CBZip2CompressionFile::Close(void)
 // CBZip2Compressor
 //
 
-
-CBZip2Compressor::CBZip2Compressor(
-                  ELevel level, int verbosity, int work_factor, TBZip2Flags flags)
-    : CBZip2Compression(level, verbosity, work_factor)
+CBZip2Compressor::CBZip2Compressor(ELevel level, TBZip2Flags flags)
+    : CBZip2Compression(level)
 {
     SetFlags(flags);
 }
@@ -521,7 +592,7 @@ CCompressionProcessor::EStatus CBZip2Compressor::Init(void)
     // Initialize the compressor stream structure
     memset(STREAM, 0, sizeof(bz_stream));
     // Create a compressor stream
-    int errcode = BZ2_bzCompressInit(STREAM, GetLevel(), m_Verbosity, m_WorkFactor);
+    int errcode = BZ2_bzCompressInit(STREAM, GetLevel(), kVerbosity, GetWorkFactor());
     SetError(errcode, GetBZip2ErrorDescription(errcode));
 
     if ( errcode == BZ_OK ) {
@@ -607,16 +678,18 @@ CCompressionProcessor::EStatus CBZip2Compressor::Finish(
                       /* out */      size_t* out_avail)
 {
     *out_avail = 0;
+
+    // Default behavior on empty data -- don't write header/footer
+    if ( !GetProcessedSize()  &&  !F_ISSET(fAllowEmptyData) ) {
+        // This will set a badbit on a stream
+        return eStatus_Error;
+    }
     if ( !out_size ) {
         return eStatus_Overflow;
     }
-    // Default behavior on empty data -- don't write header/footer
-    if ( !GetProcessedSize()  &&  !F_ISSET(fAllowEmptyData) ) {
-        return eStatus_EndOfData;
-    }
-    // To simplify this method, limit output size,
-    // the upper level code will process all necessary
-    // data in the loop if necessary. 
+
+    // To simplify this method, limit output size, the upper level code 
+    // will process all necessary data in the loop if necessary. 
     LIMIT_SIZE_PARAM_UINT(out_size);
 
     STREAM->next_in   = 0;
@@ -663,9 +736,8 @@ CCompressionProcessor::EStatus CBZip2Compressor::End(int abandon)
 //
 
 
-CBZip2Decompressor::CBZip2Decompressor(int verbosity, int small_decompress,
-                                       TBZip2Flags flags)
-    : CBZip2Compression(eLevel_Default, verbosity, 0, small_decompress)
+CBZip2Decompressor::CBZip2Decompressor(TBZip2Flags flags)
+    : CBZip2Compression(eLevel_Default)
 {
     SetFlags(flags);
 }
@@ -684,7 +756,7 @@ CCompressionProcessor::EStatus CBZip2Decompressor::Init(void)
     // Initialize the decompressor stream structure
     memset(STREAM, 0, sizeof(bz_stream));
     // Create a compressor stream
-    int errcode = BZ2_bzDecompressInit(STREAM, m_Verbosity, m_SmallDecompress);
+    int errcode = BZ2_bzDecompressInit(STREAM, kVerbosity, GetSmallDecompress() ? 1 : 0);
     SetError(errcode, GetBZip2ErrorDescription(errcode));
 
     if ( errcode == BZ_OK ) {
@@ -769,8 +841,7 @@ CCompressionProcessor::EStatus CBZip2Decompressor::Process(
 }
 
 
-CCompressionProcessor::EStatus CBZip2Decompressor::Flush(
-                      char*, size_t, size_t*)
+CCompressionProcessor::EStatus CBZip2Decompressor::Flush(char*, size_t, size_t*)
 {
     switch (m_DecompressMode) {
         case eMode_Unknown:
@@ -784,8 +855,7 @@ CCompressionProcessor::EStatus CBZip2Decompressor::Flush(
 }
 
 
-CCompressionProcessor::EStatus CBZip2Decompressor::Finish(
-                      char*, size_t, size_t*)
+CCompressionProcessor::EStatus CBZip2Decompressor::Finish(char*, size_t, size_t*)
 {
     switch (m_DecompressMode) {
         case eMode_Unknown:
@@ -798,7 +868,7 @@ CCompressionProcessor::EStatus CBZip2Decompressor::Finish(
         default:
             ;
     }
-    return eStatus_Success;
+    return eStatus_EndOfData;
 }
 
 
@@ -817,3 +887,5 @@ CCompressionProcessor::EStatus CBZip2Decompressor::End(int abandon)
 
 
 END_NCBI_SCOPE
+
+#endif  /* HAVE_LIBBZ2 */
