@@ -1,4 +1,4 @@
-/*  $Id: lzo.cpp 658835 2022-11-09 13:48:43Z ivanov $
+/*  $Id: lzo.cpp 676281 2023-12-13 16:37:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,6 @@
 #include <ncbi_pch.hpp>
 #include <corelib/ncbi_limits.h>
 #include <corelib/ncbifile.hpp>
-#include <util/compress/lzo.hpp>
 #include <util/error_codes.hpp>
 
 
@@ -45,6 +44,7 @@
 
 #if defined(HAVE_LIBLZO)
 
+#include <util/compress/lzo.hpp>
 #include <lzo/lzo1x.h>
 
 
@@ -144,20 +144,6 @@ CLZOCompression::CLZOCompression(ELevel level)
     return;
 }
 
-// @deprecated
-CLZOCompression::CLZOCompression(ELevel level, size_t blocksize)
-    : CCompression(level)
-{
-    if (blocksize > kMax_UInt) {
-        ERR_COMPRESS(41, FormatErrorMessage("CLZOCompression:: block size is too big"));
-        return;
-    }
-    m_BlockSize = blocksize;
-    m_Param.reset(new SCompressionParam);
-    m_Param->workmem = 0;
-    return;
-}
-
 
 CLZOCompression::~CLZOCompression(void)
 {
@@ -879,12 +865,17 @@ size_t CLZOCompression::GetBlockSizeMax(void)     { return numeric_limits<lzo_ui
 //
 
 
-CLZOCompressionFile::CLZOCompressionFile(const string& file_name, EMode mode, ELevel level)
-    : CLZOCompression(level),
-      m_Mode(eMode_Read), m_File(0), m_Stream(0)
+CLZOCompressionFile::CLZOCompressionFile(
+        const string& file_name,
+        EMode mode,
+        ELevel level,
+        size_t compression_in_bufsize,
+        size_t compression_out_bufsize
+    )
+    : CLZOCompression(level), m_Mode(eMode_Read), m_File(0), m_Stream(0)
 {
     // Open file
-    if ( !Open(file_name, mode) ) {
+    if ( !Open(file_name, mode, compression_in_bufsize, compression_out_bufsize) ) {
         const string smode = (mode == eMode_Read) ? "reading" : "writing";
         NCBI_THROW(CCompressionException, eCompressionFile, 
                    "[CLZOCompressionFile]  Cannot open file '" + file_name +
@@ -900,33 +891,6 @@ CLZOCompressionFile::CLZOCompressionFile(ELevel level)
     return;
 }
 
-// @deprecated
-CLZOCompressionFile::CLZOCompressionFile(
-    const string& file_name, EMode mode, ELevel level, size_t blocksize)
-    : CLZOCompression(level),
-      m_Mode(eMode_Read), m_File(0), m_Stream(0)
-{
-    SetBlockSize(blocksize);
-    if ( !Open(file_name, mode) ) {
-        const string smode = (mode == eMode_Read) ? "reading" : "writing";
-        NCBI_THROW(CCompressionException, eCompressionFile, 
-                   "[CLZOCompressionFile]  Cannot open file '" + file_name +
-                   "' for " + smode + ".");
-    }
-    return;
-}
-
-// @deprecated
-CLZOCompressionFile::CLZOCompressionFile(
-    ELevel level, size_t blocksize)
-    : CLZOCompression(level),
-      m_Mode(eMode_Read), m_File(0), m_Stream(0)
-{
-    // Set advanced compression parameters
-    SetBlockSize(blocksize);
-    return;
-}
-
 
 CLZOCompressionFile::~CLZOCompressionFile(void)
 {
@@ -1151,15 +1115,6 @@ CLZOCompressor::CLZOCompressor(ELevel level, TLZOFlags flags)
     SetFlags(flags | fStreamFormat);
 }
 
-// @deprecated
-CLZOCompressor::CLZOCompressor(ELevel level, size_t blocksize, TLZOFlags flags)
-    : CLZOCompression(level), m_NeedWriteHeader(true)
-{
-    SetFlags(flags | fStreamFormat);
-    SetBlockSize(blocksize);
-}
-
-
 CLZOCompressor::~CLZOCompressor()
 {
     if ( IsBusy() ) {
@@ -1394,17 +1349,6 @@ CLZODecompressor::CLZODecompressor(TLZOFlags flags)
 }
 
 
-// @deprecated
-CLZODecompressor::CLZODecompressor(size_t blocksize, TLZOFlags flags)
-    : CLZOCompression(eLevel_Default),
-      m_BlockLen(0), m_HeaderLen(kMaxHeaderSize), m_HeaderFlags(0)
-
-{
-    SetFlags(flags | fStreamFormat);
-    SetBlockSize(blocksize);
-}
-
-
 CLZODecompressor::~CLZODecompressor()
 {
 }
@@ -1690,40 +1634,6 @@ bool CLZODecompressor::DecompressCache(void)
 }
 
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// CLZOStreamCompressor /  CLZOStreamDecompressor -- deprecated constructors
-//
-
-// @deprecated
-CLZOStreamCompressor::CLZOStreamCompressor(
-    CLZOCompression::ELevel    level,
-    streamsize                 in_bufsize,
-    streamsize                 out_bufsize,
-    size_t                     blocksize,
-    CLZOCompression::TLZOFlags flags
-    )
-    : CCompressionStreamProcessor(
-            new CLZOCompressor(level, flags), eDelete, in_bufsize, out_bufsize)
-{
-    GetCompressor()->SetBlockSize(blocksize);
-}
-
-// @deprecated
-CLZOStreamDecompressor::CLZOStreamDecompressor(
-    streamsize                 in_bufsize,
-    streamsize                 out_bufsize,
-    size_t                     blocksize,
-    CLZOCompression::TLZOFlags flags
-    )
-    : CCompressionStreamProcessor(
-            new CLZODecompressor(flags), eDelete, in_bufsize, out_bufsize)
-{
-    GetDecompressor()->SetBlockSize(blocksize);
-}
-
-
-
 
 END_NCBI_SCOPE
 
