@@ -1,4 +1,4 @@
-/*  $Id: zstd.cpp 658835 2022-11-09 13:48:43Z ivanov $
+/*  $Id: zstd.cpp 676281 2023-12-13 16:37:07Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,6 @@
 #include <ncbi_pch.hpp>
 #include <corelib/ncbi_limits.h>
 #include <corelib/ncbifile.hpp>
-#include <util/compress/zstd.hpp>
 #include <util/error_codes.hpp>
 
 
@@ -45,6 +44,7 @@
 
 #if defined(HAVE_LIBZSTD)
 
+#include <util/compress/zstd.hpp>
 #include <zstd.h>
 #include <zstd_errors.h>
 
@@ -547,10 +547,12 @@ bool CZstdCompression::SetDecompressionParameters(void)
 //
 
 CZstdCompressionFile::CZstdCompressionFile(
-    const string& file_name, EMode mode, ELevel level,
-    size_t compression_in_bufsize, size_t compression_out_bufsize)
-        : CZstdCompression(level),
-            m_Mode(eMode_Read), m_File(0), m_Stream(0)
+        const string& file_name,
+        EMode mode, ELevel level,
+        size_t compression_in_bufsize,
+        size_t compression_out_bufsize
+    )
+    : CZstdCompression(level), m_Mode(eMode_Read), m_File(0), m_Stream(0)
 {
     if ( !Open(file_name, mode, compression_in_bufsize, compression_out_bufsize) ) {
         const string smode = (mode == eMode_Read) ? "reading" : "writing";
