@@ -1,4 +1,4 @@
-/*  $Id: strbuffer.cpp 657791 2022-10-17 18:01:55Z gouriano $
+/*  $Id: strbuffer.cpp 693975 2025-02-24 13:58:35Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -464,7 +464,8 @@ char CIStreamBuffer::FillBufferNoEOF(const char* pos)
 
 bool CIStreamBuffer::TryToFillBuffer(void)
 {
-    return FillBuffer(m_CurrentPos, true) < m_DataEndPos;
+    const char* pos = FillBuffer(m_CurrentPos, true);
+    return pos < m_DataEndPos;
 }
 
 
